#include "aplicaciowin.h"
#include "qimagedialog.h"
#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qaccel.h>
#include <qwhatsthis.h>
#include <qinputdialog.h>
#include <qaction.h>
#include "generadorterreny.h"
#include "fitxerterreny.h"

const char* fileOpenText = "Obre el model.";

/**************************************************************************************************/
/*   AplicacioWin::AplicacioWin() : QMainWindow( 0, "Finestra principal", WDestructiveClose )     */
/*Constructor de la classe principal AplicacioWin, que permet crear les finestra 2D de l'aplicació*/
/**************************************************************************************************/

AplicacioWin::AplicacioWin()
    : QMainWindow( 0, "Finestra principal", WDestructiveClose )
{

    QAction *nouTerrenyAction = new QAction( QPixmap::fromMimeSource("filenew.png" ), "&Nou Terreny", CTRL+Key_N, this, "crear_nou" );
    connect( nouTerrenyAction, SIGNAL( activated() ), this, SLOT( nouTerreny() ) );

    QAction *terrenyAleatoriAction = new QAction( "Nou Terreny &Aleatori", CTRL+Key_A, this, "crear_nou_aleatori" );
    connect( terrenyAleatoriAction, SIGNAL( activated() ), this, SLOT( nouTerrenyAleatori() ) );

    QAction *desaTerrenyAction = new QAction( QPixmap::fromMimeSource("filesave.png" ), "&Desa Terreny", CTRL+Key_D, this, "desa_model" );
    connect( desaTerrenyAction, SIGNAL( activated() ), this, SLOT( desaTerrenyCom() ) );

    QAction *carregaTerrenyAction = new QAction( QPixmap::fromMimeSource("fileopen.png" ), "&Carrega Terreny", CTRL+Key_C, this, "carrega_model" );
    connect( carregaTerrenyAction, SIGNAL( activated() ), this, SLOT( carregaTerreny() ) );

    QAction *sortirAction = new QAction( "&Surt", CTRL+Key_S, this, "sortir" );
    connect( sortirAction, SIGNAL( activated() ), this, SLOT( closeWindow() ) );

    QAction *mostrarAction = new QAction( "&Mostra Terreny", CTRL+Key_M, this, "mostrar" );
    connect( mostrarAction, SIGNAL( activated() ), this, SLOT( mostraTerreny() ) );

    QAction *configAction = new QAction( "C&onfigura...", CTRL+Key_O, this, "configurar" );
    connect( configAction, SIGNAL( activated() ), this, SLOT( configura() ) );
    
    QAction *camiMinimAction = new QAction( "&Trobar cami mes curt", CTRL+Key_T, this, "trobarcamimescurt" );
    connect( camiMinimAction, SIGNAL( activated() ), this, SLOT( calculaCamiMinim() ) );
    
    QAction *jocProvesAction = new QAction( "T&estejar Funcions", CTRL+Key_E, this, "testejarfuncions" );
    connect( jocProvesAction, SIGNAL( activated() ), this, SLOT( realitzarJocProves() ) );
    
    corbesAction = new QAction( QPixmap::fromMimeSource("corbes.png" ), "Genera Cor&bes de Nivell", CTRL+Key_B, this, "corbes" );
    connect( corbesAction, SIGNAL( toggled( bool ) ), this, SLOT( visualitzaCorbes( bool ) ) );
    corbesAction->setToggleAction( TRUE );
    
    texturaAction = new QAction( QPixmap::fromMimeSource("colors.png" ), "Diagrames de &Voronoi", CTRL+Key_V, this, "textura" );
    connect( texturaAction, SIGNAL( activated() ), this, SLOT( tipusVisualitzacio( ) ) );
    
    veureTexturaAction = new QAction( QPixmap::fromMimeSource("colors.png" ), "&Dibuixar Diagrames de Voronoi", CTRL+Key_D, this, "textura" );
    connect( veureTexturaAction, SIGNAL( toggled( bool ) ), this, SLOT( visualitzaTextura( bool ) ) );
    veureTexturaAction->setToggleAction( TRUE );
    
    QAction *saveImaAction = new QAction( "E&xporta a Imatge", CTRL+Key_I, this, "salvar_imatge" );
    connect( saveImaAction, SIGNAL( activated() ), this, SLOT( desaImatge() ) );

/**************** MenuBar ******************************/

    QPopupMenu *fileMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Arxiu", fileMenu );

    QPopupMenu *nous = new QPopupMenu( this );

    fileMenu->insertItem( "Nou", nous );

    nouTerrenyAction->addTo( nous );
    terrenyAleatoriAction->addTo( nous );

    carregaTerrenyAction->addTo( fileMenu );
    desaTerrenyAction->addTo( fileMenu );
    saveImaAction->addTo( fileMenu );
    sortirAction->addTo( fileMenu );

    QPopupMenu* editaMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Edita", editaMenu );

    QPopupMenu* visualMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Visualitza", visualMenu );
    mostrarAction->addTo( visualMenu );
    
    QPopupMenu* configMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Preferencies", configMenu );
    configAction->addTo( configMenu );

    QPopupMenu* calculsMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Calculs", calculsMenu );
    
    menuBar()->insertSeparator();

    QPopupMenu* help = new QPopupMenu( this );
    menuBar()->insertItem( "A&juda", help );

    help->insertItem( "Q&uant a", this, SLOT(about()), Key_F1);
    help->insertItem( "Quant a &Qt", this, SLOT(aboutQt()));
    help->insertSeparator();
    help->insertItem( "Que es a&ixo", this, SLOT(whatsThis()), SHIFT+Key_F1);
    help->insertSeparator();
    jocProvesAction->addTo(help); 

/*******************************************************/

    terreny = Terreny::getInstancia();
    editor = new EditorTerreny( this );
    visor = new VisorWin();
    visor->setCaption("Visor de terrenys");

/**************** ToolBar ******************************/

    fileTools = new QToolBar( this, "operacions amb arxius" );
    addToolBar( fileTools, tr( "Operacions amb models" ), DockTop, TRUE );

    QToolButton *nouTool = new QToolButton( QPixmap::fromMimeSource("filenew.png" ), "Nou", QString::null,
                                              this, SLOT(nouTerreny()), fileTools, "nou_tool" );

    nouTool->setPopup( nous );

    carregaTerrenyAction->addTo( fileTools );
    desaTerrenyAction->addTo( fileTools );

    fileTools->addSeparator();

    QActionGroup *grp = new QActionGroup( this );
    connect( grp, SIGNAL( selected( QAction * ) ), this, SLOT( canviaMode( QAction * ) ) ); 

    renderAction = new QAction( QPixmap::fromMimeSource("render.png" ), "Mode &Ediccio", CTRL+Key_E, grp, "ediccio" );    
    renderAction->setToggleAction( TRUE );

    selectAction = new QAction( QPixmap::fromMimeSource("select.png" ), "Mode Se&leccio", CTRL+Key_L, grp, "seleccio" );  
    selectAction->setToggleAction( TRUE );    
        
    renderAction->setOn( TRUE );

    puntVisio = new QAction( QPixmap::fromMimeSource("punt_visio.png" ), "Inserir &Punt (seu)", CTRL+Key_P, grp, "punt" );    
    puntVisio->setToggleAction( TRUE );

    segmentVisio = new QAction( QPixmap::fromMimeSource("segment_visio.png"), "Inserir &Segment", CTRL+Key_S, grp, "segment" );
    segmentVisio->setToggleAction( TRUE );    

    poligonalVisio = new QAction( QPixmap::fromMimeSource("poligonal_visio.png"), "Inserir P&oligonal", CTRL+Key_O, grp, "poligonal" ); 
    poligonalVisio->setToggleAction( TRUE );    
    
    poligonVisio = new QAction( QPixmap::fromMimeSource("poligon_visio.png"), "Inserir Poli&gon", CTRL+Key_G, grp, "poligon" ); 
    poligonVisio->setToggleAction( TRUE );    

    grp->addTo( fileTools );
    grp->addTo( editaMenu );
    
    fileTools->addSeparator();

    QActionGroup *grp2 = new QActionGroup( this );
    connect( grp2, SIGNAL( selected( QAction * ) ), this, SLOT( canviaCalculs( QAction * ) ) ); 

    visibilitatAction = new QAction( "&Visibilitat", CTRL+Key_V, grp2, "visibilitat" );   
    visibilitatAction->setToggleAction( TRUE );
    	
    distanciesAction = new QAction( "&Distancies", CTRL+Key_D, grp2, "distancies" );  
    distanciesAction->setToggleAction( TRUE ); 
    distanciesAction->setOn(TRUE);
       
    grp2->addTo( calculsMenu );
    camiMinimAction->addTo(calculsMenu); 
        
    visioAction = new QAction( QPixmap::fromMimeSource("icon_ull.png" ), "Elements &Visibles", CTRL+Key_V, this, "visio_visibles" );
    visioAction->setToggleAction( TRUE );
    connect( visioAction, SIGNAL( toggled( bool ) ), editor, SLOT( mostraElementsVisio( bool ) ) );
    
    visioAction->addTo( fileTools );
    visioAction->addTo( visualMenu );
    
    fileTools->addSeparator();
    
    vertexsAction = new QAction( QPixmap::fromMimeSource("vertexs.png" ), "Visuali&tza vertexs", CTRL+Key_T, this, "veureVertexs" );
    vertexsAction->setToggleAction( TRUE );    
    connect( vertexsAction, SIGNAL( toggled( bool ) ), editor, SLOT( mostraVertexs( bool ) ) );

    vertexsAction->addTo( fileTools );
    vertexsAction->setOn( TRUE );

    triangAction = new QAction( QPixmap::fromMimeSource("trg.png"), "Visualitza t&riangulacio", CTRL+Key_R, this, "veureTriang");
    triangAction->setToggleAction( TRUE );    
    connect( triangAction, SIGNAL( toggled( bool ) ), editor, SLOT( mostraTriangulacio( bool ) ) );    

    triangAction->addTo( fileTools );
    triangAction->setOn( TRUE );

    fileTools->addSeparator();

    corbesAction->addTo( fileTools );
    corbesAction->addTo( visualMenu );
    
    fileTools->addSeparator();
    
    veureTexturaAction->addTo( fileTools );
    texturaAction->addTo( visualMenu );
    
    fileTools->addSeparator();
    
    calcularDistanciesAction = new QAction( QPixmap::fromMimeSource("metro.png" ), "&Propagar terreny", CTRL+Key_D, this, "calcular" );
    connect( calcularDistanciesAction, SIGNAL( toggled( bool ) ), this, SLOT( calculaDistancies()) );
    calcularDistanciesAction->setToggleAction( TRUE );
    
    calcularDistanciesAction->addTo( fileTools );
    calcularDistanciesAction->addTo( calculsMenu );
    
    fileTools->addSeparator();
    
    inserirPuntCamiMinimAction = new QAction( QPixmap::fromMimeSource("punto.png" ), "&Inserir Punt Cami Minim", CTRL+Key_I, this, "inserir" );
    connect( inserirPuntCamiMinimAction, SIGNAL( toggled( bool ) ), this, SLOT( inserirPuntCamiMinim()) );
    inserirPuntCamiMinimAction->setToggleAction( TRUE );
    
    inserirPuntCamiMinimAction->addTo( fileTools );
    inserirPuntCamiMinimAction->addTo( editaMenu );
    

/************a*******************************************/

    setCaption ( "Nou terreny - Modelat de terranys" );

    setCentralWidget( editor );

    setGeometry ( 0, 0, 500, 500 );

    connect( editor, SIGNAL( terrenyModificat() ), visor->getVisor(), SLOT( actualitza() ) );
    connect( editor, SIGNAL( rataStatus( double, double ) ), this, SLOT( actualitzaCoords( double, double ) ) );
    connect( editor, SIGNAL( mostrarInfVisibilitatAmbRata( int, bool,double, double ) ), this, SLOT( mostraInformacioVisibilitatICoords( int,bool,double,double ) ) );

    visio=Visio::instance();
//     regions_hiperboliques = RegionsHiperboliques::instance();
    
//     regions_hiperboliques->show();
    editor->show();
    visor->show();    

    statusBar()->message( "Aplicacio preparada", 2000 );
    
    prefs.lmx = visor->getVisor()->getLmx();
    prefs.lmy = visor->getVisor()->getLmy();
//     prefs.lmx = regions_hiperboliques->getVisualitzadorRegionsHiperboliques()->getLmx();
//     prefs.lmy = regions_hiperboliques->getVisualitzadorRegionsHiperboliques()->getLmy();
}

/***********************************************************************************/
/*   AplicacioWin::~AplicacioWin() { }                                             */
/*Destructor de la classe AplicacioWin, que permet destruir objectes d'aquest tipus*/
/***********************************************************************************/

AplicacioWin::~AplicacioWin() 
{ 
	if(terreny) delete terreny;
// 	if(regions_hiperboliques) delete regions_hiperboliques;
	if(editor) delete editor;
// 	if(visio) delete visio;
}

/****************************************************************/
/*   void AplicacioWin::closeWindow()                           */
/*Metode que permet finalitzar l'aplicació tancant les finestres*/
/****************************************************************/

void AplicacioWin::closeWindow()
{
     close();
}

/***************************************************************/
/*   void AplicacioWin::about()                                */
/*Metode que assigna el missatge d'informació sobre l'aplicació*/
/***************************************************************/

void AplicacioWin::about()
{
    QMessageBox::about( this, "Aplicacio",
                        "Editor 2D: Finestra d'interaccio\n" 
                        "Generador, modelador i editor de terrenys\n"
			"Resolucio de problemes de proximitat en terenys");
}

/**********************************************************/
/*   void AplicacioWin::aboutQt()                         */
/*Metode que assigna el missatge d'informació sobre les Qt*/
/**********************************************************/

void AplicacioWin::aboutQt()
{
    QMessageBox::aboutQt( this, "Aplicacio" );
}

/***************************************************************************/
/*   void AplicacioWin::closeEvent( QCloseEvent *e )                       */
/*Metode que s'executa en el moment en que es desitja sortir de l'aplicació*/
/***************************************************************************/

void AplicacioWin::closeEvent( QCloseEvent *e )
{
       e->accept();
}

/*******************************************************/
/*   void AplicacioWin::mostraTerreny()                */
/*Metode que simplement fa que es pinti el terreny 2.5D*/
/*******************************************************/

void AplicacioWin::mostraTerreny()
{
   visor->show();
}

/************************************************************/
/*   void TerAplicacioWin::carregaTerreny()                 */
/*Metode que permet carregar un terreny guardat en un fitxer*/
/************************************************************/

void AplicacioWin::carregaTerreny()
{
    QString format;
    FitxerTerreny ft;
    QString fn = QFileDialog::getOpenFileName(
                    QDir::homeDirPath(),
                    "Fitxer de text (*.t2d);;Xarxa triangular irregular (*.tin)",
                    this,
                    "obre_fitxer",
                    "Carrega terreny"
                 );

    if ( !fn.isEmpty() ) 
    {       
       terreny->clear();
       int estat = ft.carrega( fn, terreny );
       if ( estat == CORRECTE )
       {
          double *limits = terreny->maximsIMinims();
          prefs.xmin = floor( limits[0] );
          prefs.ymin = floor( limits[1] ); 
          prefs.zmin = floor( limits[2] );
          prefs.xmax = ceil( limits[3] );  
          prefs.ymax = ceil( limits[4] );
          prefs.zmax = ceil( limits[5] );
          renderAction->setOn( TRUE ); 
          vertexsAction->setOn( TRUE );
          triangAction->setOn( TRUE );
          corbesAction->setOn( FALSE );
          editor->modificaMida( limits );
	  
	  veureTexturaAction->setOn(FALSE);
	  visio->clear();
	  
          (visor->getVisor())->modificaMida( prefs.xmin, prefs.ymin, prefs.zmin, 
                                             prefs.xmax, prefs.ymax, prefs.zmax );
//           editor->clear();
   
          fitxer = fn;
  	  setCaption( fitxer + " - Modelat de terrenys" );	  
       }
       else if( estat == ERROR_OBRIR )
       {
          QMessageBox::warning( this, "Modelat de terrenys", "No s'ha pogut obrir el fitxer.", QMessageBox::Ok,
                                QMessageBox::NoButton, QMessageBox::NoButton );
       }
       else
       {
          QMessageBox::warning( this, "Modelat de terrenys", "El format del fitxer es incorrecte.", QMessageBox::Ok,
                                QMessageBox::NoButton, QMessageBox::NoButton );
       }
    }  
    else 
    {
	   statusBar()->message( "Carrega abortada", 2000 );
    }
}

/***************************************************/
/*   void AplicacioWin::desaTerreny()              */
/*Metode que permet guardar un terreny en un fitxer*/
/***************************************************/

void AplicacioWin::desaTerreny()
{
    FitxerTerreny ft;
    if ( fitxer.isEmpty() ) 
    {
        desaTerrenyCom();
        return;
    }

    int resultat = ft.desa( fitxer, terreny );

    if ( resultat == CORRECTE )
    {
      setCaption( fitxer + " - Modelat de terrenys" );
      statusBar()->message( QString( "Terreny %1 desat." ).arg( fitxer ), 2000 );
      return;
    }
    else if( resultat == ERROR_OBRIR )
    {
       QMessageBox::warning( this, "Modelat de terrenys", "No s'ha pogut obrir el fitxer.", QMessageBox::Ok,
                             QMessageBox::NoButton, QMessageBox::NoButton );
    }
    else
    {
       QMessageBox::warning( this, "Modelat de terrenys", "El format del fitxer es incorrecte.", QMessageBox::Ok,
                             QMessageBox::NoButton, QMessageBox::NoButton );
    }
    statusBar()->message( QString( "Terreny no desat." ), 2000 );
}

/********************************************************************/
/*   void AplicacioWin::desaTerrenyCom()                            */
/*Metode que s'executa en el cas que el nom del fitxer estigui buit */
/********************************************************************/

void AplicacioWin::desaTerrenyCom()
{
    QString format;
    QString fn = QFileDialog::getSaveFileName( 
                    QDir::homeDirPath(),
                    "Xarxa triangular irregular (*.tin);;Fitxer de text (*.t2d)",
                    this,
                    "Desa_terreny",
                    "Desa terreny com... ",
                    &format 
                 );

    if ( !fn.isEmpty() ) 
    {
        if( format.contains("(*.tin)") ) format = ".tin"; 
        else format = ".t2d";     

        fitxer = fn + format;

        desaTerreny();
    } 
    else 
    {
	statusBar()->message( "Terreny no desat.", 2000 );
    }
}

/*****************************************************************************/
/*   void AplicacioWin::nouTerreny()                                         */
/*Metode que dona l'opció de guardar el model antic abans d'iniciar un de nou*/
/*****************************************************************************/

void AplicacioWin::nouTerreny()
{
   terreny->clear();
   renderAction->setOn( TRUE ); 
   vertexsAction->setOn( TRUE );
   triangAction->setOn( TRUE );
   corbesAction->setOn( FALSE );
   editor->clear(); 
   (visor->getVisor())->initVista(); 
   visio->clear();
   fitxer = "";
   setCaption ( "Nou terreny - Modelat de terranys" );
}

/******************************************************************/
/*   void AplicacioWin::nouTerrenyAleatori()                      */
/*Metode que permet crear un nou terreny amb valors de Z aleatoris*/
/******************************************************************/

void AplicacioWin::nouTerrenyAleatori()
{
    bool ok;
    int iteracions;
    float *dades;   
    GeneradorTerreny gt;

    iteracions = QInputDialog::getInteger("Genera Terreny Aleatori", "Numero d'iteracions:", 100, 3, 10000, 1, &ok, this );
    if( ok ) 
    { 
       terreny->clear();
//        qDebug("%f,%f,%f,%f", prefs.xmax, prefs.xmin, prefs.ymax, prefs.ymin);
       gt.generaParticleDeposition( iteracions, (prefs.xmax - prefs.xmin), (prefs.ymax - prefs.ymin) ); 
       dades = gt.getDades();
       for( int i = 0; i < round(prefs.xmax - prefs.xmin + 1); i++ )
       {
         for( int j = 0; j < round(prefs.ymax - prefs.ymin + 1); j++ )
         {
//             terreny->insereix( Punt3d( i + prefs.xmin, j + prefs.ymin, dades[i + (int)round(prefs.ymax - prefs.ymin) * j] ) );
               terreny->inserirPunt(Punt3d( i + prefs.xmin, j + prefs.ymin, dades[i + (int)round(prefs.ymax - prefs.ymin) * j]*2.));
         }
       }
//        limits[0] = prefs.xmin;  
//        limits[1] = prefs.ymin; 
//        limits[2] = prefs.zmin; 
//        limits[3] = prefs.xmax; 
//        limits[4] = prefs.ymax; 
//        limits[5] = prefs.zmax;
//        editor->modificaMida( limits );
       dades = NULL;       
       renderAction->setOn( TRUE ); 
       vertexsAction->setOn( TRUE );
       triangAction->setOn( TRUE );
       corbesAction->setOn( FALSE );
       visor->mostraTerreny();
//        editor->clear();   
       fitxer = "";
       setCaption ( "Nou terreny - Modelat de terranys" );
    }
}

/********************************************************************************************/
/*   void AplicacioWin::configura()                                                         */
/*Metode que permet configurar l'aplicació, fent com una mena de zoom sobre la zona indicada*/
/********************************************************************************************/

void AplicacioWin::configura()
{
    ConfigDialeg *confd = new ConfigDialeg( this );
    confd->setDades( prefs );
    if ( confd->exec() == QDialog::Accepted )
    {
       prefs = confd->dades();
       double limits[6];
       QColor colors[4];    

       limits[0] = prefs.xmin;  
       limits[1] = prefs.ymin; 
       limits[2] = prefs.zmin; 
       limits[3] = prefs.xmax; 
       limits[4] = prefs.ymax; 
       limits[5] = prefs.zmax;

       colors[0] = prefs.cfons2d;
       colors[1] = prefs.cpunts2d;
       colors[2] = prefs.clinies2d;
       colors[3] = prefs.cseleccio;

       terreny->limits[0] = prefs.xmin;
       terreny->limits[1] = prefs.ymin;
       terreny->limits[2] = prefs.zmin;
       terreny->limits[3] = prefs.xmax;
       terreny->limits[4] = prefs.ymax;
       terreny->limits[5] = prefs.zmax;
       
       visor->getVisor()->setLmx(prefs.lmx);
       visor->getVisor()->setLmy(prefs.lmy);
       
//        regions_hiperboliques->getVisualitzadorRegionsHiperboliques()->setLmx(prefs.lmx);
//        regions_hiperboliques->getVisualitzadorRegionsHiperboliques()->setLmy(prefs.lmy);
       
       editor->modificaMida( limits );
       editor->modificaColors( colors );
       editor->modificaMidaGraella(prefs.files,prefs.columnes);

       (visor->getVisor())->modificaMida( prefs.xmin, prefs.ymin, prefs.zmin, 
                                          prefs.xmax, prefs.ymax, prefs.zmax );       
    }
}

/******************************************************************************************************/
/*   void AplicacioWin::keyPressEvent( QKeyEvent *ke )                                                */
/*Metode que detecta quan es prem una tecla i en el cas que sigui la ESC elimina l'element seleccionat*/
/******************************************************************************************************/

void AplicacioWin::keyPressEvent( QKeyEvent *ke )
{
      if ( ke->key() == Qt::Key_Delete ) editor->eliminaElement();
}

/**************************************************************************************/
/*   void AplicacioWin::actualitzaCoords( double x, double y )                        */
/*Metode que permet veure en cada moment la posicio on ens trobem amb el reatoli X i Y*/
/**************************************************************************************/

void AplicacioWin::actualitzaCoords( double x, double y )
{
   statusBar()->message( QString("X: %1, Y: %2").arg(x).arg(y) );
}

void AplicacioWin::mostraInformacioVisibilitatICoords( int n, bool counting,double x, double y  )
{
   if(counting) statusBar()->message( QString("X: %1, Y: %2            Visible des de: %3").arg(x).arg(y).arg(n) );
   else
   	if(n==0) statusBar()->message( QString("X: %1, Y: %2            No visible").arg(x).arg(y) );
	else statusBar()->message( QString("X: %1, Y: %2            Visible").arg(x).arg(y) );
}


/**********************************************************************/
/*   void AplicacioWin::visualitzaCorbes( bool b )                    */
/*Metode que s'executa en el cas que es vulguin visualitzar les corbes*/
/**********************************************************************/

void AplicacioWin::visualitzaCorbes( bool b )
{
   if( b )
   {
      visor->obteNivellsIMostra();
   }
   else
   {
      visor->mostraTerreny();
   }
   editor->mostraCorbes( b );
}

/*****************************************************************/
/*   void AplicacioWin::desaImatge()                             */
/*Metode que permet capturar una imatge i guardar-la com a fitxer*/
/*****************************************************************/

void AplicacioWin::desaImatge()
{
    QImageDialog* imadiag  = new QImageDialog( this );

    if ( imadiag->exec() == QDialog::Accepted )
    {
       if ( imadiag->getNom() != QString::null )
       {
          editor->grabFrameBuffer().save( imadiag->getNom(), (const char *)(imadiag->getFormat()), 100 );
       }
    }
}

/***************************************************************************************************/
/*   void AplicacioWin::canviaMode( QAction *type )                                                */
/*Metode que permet canviar de mode, és a dir, passar de dibuix a selecció i al revès              */
/*Metode que tambe permet escollir un dels objectes de visio per representar-lo sobre el terreny 2D*/
/***************************************************************************************************/

void AplicacioWin::canviaMode( QAction *type )
{
	if (type == selectAction)
	{
		editor->setAccio(GL_SELECT);
	}
	else 
	{
		if (type == renderAction)
		{
			editor->setAccio(GL_RENDER);
		}  
		else
		{
			visioAction->setOn( TRUE );
			if (type == puntVisio)
			{
				editor->setVisio(PUNT_VISIO);
			}
			else
			{
				if (type == segmentVisio)
				{
					editor->setVisio(SEGMENT_VISIO);
				}  
				else
				{
					if (type == poligonVisio)
					{
						editor->setVisio(POLIGON_VISIO);
					}
					else
					{
						if (type == poligonalVisio)
						{
							editor->setVisio(POLIGONAL_VISIO);
						}
					}
				}
			}
		}
	}
}

/***************************************************************************************************/
/*   void AplicacioWin::canviaCalculs( QAction *type )                                                */
/*Metode que permet canviar el tipus de càlculs, de mode, és a dir, passar de realitzar càlculs de visibilitat a càlculs de distàncies i al revès. Els botons es comportaran de forma diferent           */
/***************************************************************************************************/

void AplicacioWin::canviaCalculs( QAction *type )
{
	if (type == visibilitatAction)
	{
		//Quan s'escull visibilitat es poden afegir punts, segments, ... igual que abans
		cout << "He escollit Visibilitat" << endl;
		editor->configuracioVisio(); //Ens fica a true el mode_visibilitat
	}
	else 
	{
		if (type == distanciesAction)
		{
			/*Quan s'escull distàncies i s'introdueixin punts, segments, ... es posarà també la z del terreny)*/
			cout << "\nHe escollit Distancies" << endl;
			editor->configuracioDistancies(); //Ens fica a true el mode_distancies
		}  
	}
}	
		

/******************************************************************************************************/
/*   void AplicacioWin::tipusVisualitzacio()                                                          */
/*Metode que permet escollir el tipus de visualització de les parts visibles dd els diferents segments*/
/******************************************************************************************************/

void AplicacioWin::tipusVisualitzacio(  )
{
	bool er=false;
	QErrorMessage *errors;
	
	/*Primer de tot cal comprovar si s'ha realitzat el càlcul de visibilitat*/
	if( visio->calculada_textura())
	{
		/*Creem una nova subfinestra */
		QDialog * finestra = new QDialog( this, "Tipus de Visualitzacio" );
		
		QVBox * vertical=new QVBox(finestra);
		
		vertical->setFont(QFont("", 8, 1, false));
		
		QVButtonGroup * tipus=new QVButtonGroup(vertical); /*La primera opcio del bloc de controls consistira en inserir diferents tipus d'obstacles dins de l'escena amb les seves propietats*/
		
		tipus->setTitle(QString("Tipus de visualitzacio"));
		
		tipus->setFrameStyle(QFrame::Box | QFrame::Raised);
		tipus->setLineWidth(2);
		tipus->setFrameShadow(QFrame::Sunken);
		tipus->setMargin(50);
		
		unio = new QRadioButton("Unio", tipus);
		connect( unio, SIGNAL(clicked()), editor, SLOT(activaUnio()));
		unio->setChecked(true);
		connect( editor, SIGNAL(desactiva1()), this, SLOT(boto1Desactivat()));
		interseccio = new QRadioButton("Interseccio", tipus);
		connect( interseccio, SIGNAL(clicked()), editor, SLOT(activaInterseccio()));
		connect( editor, SIGNAL(desactiva1()), this, SLOT(boto1Desactivat()));
		counting = new QRadioButton("Counting", tipus);
		connect( counting, SIGNAL(clicked()), editor, SLOT(activaCounting()));
		connect( editor, SIGNAL(desactiva1()), this, SLOT(boto1Desactivat()));
		
		QHBox * horitzontal_comb = new QHBox(tipus);
		combinat = new QRadioButton("Combinat", horitzontal_comb);
		connect( combinat, SIGNAL(clicked()), editor, SLOT(activaCombinat()));
		connect( editor, SIGNAL(desactiva2()), this, SLOT(boto2Desactivat()));
		
		QHBox * horitzontal_comb2 = new QHBox(horitzontal_comb);
		QSpinBox *spin1=new QSpinBox(horitzontal_comb2);
		spin1->setMinValue(1);
		spin1->setMaxValue(visio->numeroObjectes());
// 		spin1->setPrefix("1er  "); 
		connect( spin1, SIGNAL(valueChanged(int)), editor, SLOT(primerSegmentVisio(int)));
		
		QSpinBox *spin2=new QSpinBox(horitzontal_comb2);
		spin2->setMinValue(1);
		spin2->setMaxValue(visio->numeroObjectes());
// 		spin2->setPrefix("2on  "); 
		connect( spin2, SIGNAL(valueChanged(int)), editor, SLOT(segonSegmentVisio(int)));
		
		QSpinBox *spin3=new QSpinBox(horitzontal_comb2);
		spin3->setMinValue(0);
		spin3->setMaxValue(visio->numeroObjectes());
// 		spin3->setPrefix("[3er]  "); 
		connect( spin3, SIGNAL(valueChanged(int)), editor, SLOT(tercerSegmentVisio(int)));
		
		QHBox * horitzontal = new QHBox(tipus);
		segment = new QRadioButton("Objecte", horitzontal);
		connect( segment, SIGNAL(clicked()), editor, SLOT(activaSegment()));
		connect( editor, SIGNAL(desactiva3()), this, SLOT(boto3Desactivat()));
		
		QSpinBox *spin=new QSpinBox(horitzontal);
		spin->setMinValue(0);
		spin->setMaxValue(visio->numeroObjectes());
		spin->setPrefix("Index  ");
		connect( spin, SIGNAL(valueChanged(int)), editor, SLOT(segmentEscollit(int)));
		
		horitzontal = new QHBox(vertical);
		QPushButton * acceptar=new QPushButton("Acceptar", horitzontal);
	 	connect( acceptar, SIGNAL( clicked() ), finestra, SLOT( accept() ) );
		QPushButton * cancelar=new QPushButton("Cancelar", horitzontal);
	 	connect( cancelar, SIGNAL( clicked() ), finestra, SLOT( reject() ) );
		tipus->setMaximumSize(300, 175);
		tipus->setMinimumSize(300, 175);
	
		vertical->setMaximumSize(300, 225);
		vertical->setMinimumSize(300, 225);
		
		finestra->setMaximumSize(300, 225);
		finestra->setMinimumSize(300, 225);
		finestra->show();
	}
	else
	{
		/*Informem a l'usuari de que primer cal que realitzi el càlcul de visibilitat a traves d'un missatge*/
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Calcul de visibilitat no realitzat!" ));
		errors->message( "Per poder escollir una opcio de visualitzacio cal haver realitzat el calcul de visibilitat anteriorment."); 
		er = true;
	}
	
	if(!er)
	{
		editor->veureTextura( true );
		veureTexturaAction->setOn( TRUE );
// 		cout<<"TEXTURA"<<endl;
	}
}

/*********************************************************/
/*   void AplicacioWin::visualitzaTextura( bool b )      */
/*Metode que permet visualitzar o no la textura escollida*/
/*********************************************************/

void AplicacioWin::visualitzaTextura( bool b )
{
	if (editor->getModeVisibilitat()==TRUE)
	{
		bool er = false;
		QErrorMessage *errors;
		if ( b )
		{
			if( !visio->calculada_textura())
			{
				/*Informem a l'usuari de que primer cal que realitzi el càlcul de visibilitat a traves d'un missatge*/
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Calcul de visibilitat no realitzat!" ));
				errors->message( "Per poder escollir una opcio de visualitzacio cal haver realitzat el calcul de visibilitat anteriorment."); 
				veureTexturaAction->setOn( FALSE );
				er = true;
			}
		}
	if(!er)
		editor->veureTextura( b );

	}
	else //Estem en mode distàncies
	{	
	editor->dibuixarDiagramesVoronoi();	
	}
}



/*********************************************************/
/*   void AplicacioWin::calculaDistancies      */
/*Metode que permet veure el resultat de calcular les distancies*/
/*********************************************************/

void AplicacioWin::calculaDistancies()
{
	if (editor->getModeVisibilitat()==TRUE)
	{
		QErrorMessage *errors;
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Opcio no operativa en mode visualitzacio. Escull el mode distancies."); 
	}
	else //Estem en mode Distancies
	{
		editor->calculsDistancies();
	}
}


/*********************************************************/
/*   void AplicacioWin::inserirPuntCamiMinim()           */
/*Metode que ens permetrà inserir un Punt al Terreny per tal de calcular-li el camí mínim*/
/*********************************************************/

void AplicacioWin::inserirPuntCamiMinim()
{
	if (editor->getModeVisibilitat()==TRUE)
	{
		QErrorMessage *errors;
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Opcio no operativa en mode visualitzacio. Escull el mode distancies."); 
	}
	else //Estem en mode Distancies
	{
		editor->setVisio(PUNT_VISIO);
		editor->configuracioPuntDistancies();
		cout << "\nInserirem un punt de camí mínim" << endl;
	}
}


/*******************************************************/
/*   void AplicacioWin::calculaCamiMinim()             */
/*Metode que ens crida al mètode d'editor de calcular el camí mínim*/
/*******************************************************/

void AplicacioWin::calculaCamiMinim()
{
  	if (editor->getModeVisibilitat()==TRUE)
	{
		QErrorMessage *errors;
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Opcio no operativa en mode visualitzacio. Escull el mode distancies."); 
	}
	else
	{
		editor->trobaCami();
	}
}

/*******************************************************/
/*   void AplicacioWin::realtizarJocProves()             */
/*Metode que ens crida al mètode d'editor de realtizar el joc de proves per tal de testejar diferents funcions*/
/*******************************************************/

void AplicacioWin::realitzarJocProves()
{
  	if (editor->getModeVisibilitat()==TRUE)
	{
		QErrorMessage *errors;
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Opcio no operativa en mode visualitzacio. Escull el mode distancies."); 
	}
	else
	{
		editor->jocProves();
	}
}


/***********************************************************/
/*   void AplicacioWin::boto1Desactivat()                  */
/*Metode que permet desactivar el conjunt de radio buttons */
/***********************************************************/

void AplicacioWin::boto1Desactivat()
{
	unio->setChecked(false);
	interseccio->setChecked(false);
	counting->setChecked(false);
}

/***********************************************************/
/*   void AplicacioWin::boto2Desactivat()                  */
/*Metode que permet desactivar el radio button del segment */
/***********************************************************/

void AplicacioWin::boto2Desactivat()
{
	combinat->setChecked(false);
}

/***********************************************************/
/*   void AplicacioWin::boto2Desactivat()                  */
/*Metode que permet desactivar el radio button del segment */
/***********************************************************/

void AplicacioWin::boto3Desactivat()
{
	segment->setChecked(false);
}


