#ifndef CONFIGDIALEG_H
#define CONFIGDIALEG_H

#include <qvariant.h>
#include <qdialog.h>
#include "geometriabasica.h"

class QVBoxLayout;
class QHBoxLayout;
class QGridLayout;
class QSpacerItem;
class QTabWidget;
class QWidget;
class QGroupBox;
class QLabel;
class QLineEdit;
class QButtonGroup;
class QPushButton;

struct ConfigInfo {
    double xmin, ymin, zmin;
    double xmax, ymax, zmax;
    double lmx, lmy;
    QColor cfons2d;
    QColor cpunts2d;
    QColor clinies2d;  
    QColor cseleccio;
    int files,columnes;

    ConfigInfo()
    {
       xmin = ymin = zmin = 0.0;
       xmax = ymax = zmax = 20.0;
       lmx = lmy = 0.0;
       files = columnes = 500;

       cfons2d = Qt::black;
       cpunts2d = Qt::green;
       clinies2d = Qt::blue;
       cseleccio = Qt::red;
    }

    ConfigInfo( const ConfigInfo &p )
    {
       xmin = p.xmin;
       ymin = p.ymin;
       zmin = p.zmin;
       xmax = p.xmax;
       ymax = p.ymax;
       zmax = p.zmax;
       
       lmx = p.lmx;
       lmy = p.lmy;
       
       files=p.files;
       columnes=p.columnes;

       cfons2d = p.cfons2d;
       cpunts2d = p.cpunts2d;
       clinies2d = p.clinies2d;
       cseleccio = p.cseleccio;
    }

    ConfigInfo &operator=( const ConfigInfo &p )
    {
       xmin = p.xmin;
       ymin = p.ymin;
       zmin = p.zmin;    
       xmax = p.xmax;
       ymax = p.ymax;
       zmax = p.zmax;

       lmx = p.lmx;
       lmy = p.lmy;
       
       files=p.files;
       columnes=p.columnes;
       cfons2d = p.cfons2d;
       cpunts2d = p.cpunts2d;
       clinies2d = p.clinies2d;
       cseleccio = p.cseleccio;
  
       return (*this);
    }
};

/************************************************************************************************************************/
/*                                      class ConfigDialeg : public QDialog                                             */
/*Classe que descriu una finestra de dialeg que permet modificar la configuracio de la finestra 2D (coordenades i color)*/
/************************************************************************************************************************/

class ConfigDialeg : public QDialog
{
    Q_OBJECT

public:
    ConfigDialeg( QWidget* parent = 0, const char* name = 0, bool modal = FALSE, WFlags fl = 0 );
    ~ConfigDialeg();

    QTabWidget* tabWidget2;
    QWidget* tab;
    QGroupBox* groupBox1;
    QLabel* textLabel2;
    QLabel* textLabel3;
    QLabel* textLabel4;
    QLineEdit* xminEdit;
    QLineEdit* yminEdit;
    QLineEdit* zminEdit;
    QLabel* textLabel1_2;
    QLabel* textLabel2_2;
    QLabel* textLabel3_2;
    QLineEdit* xmaxEdit;
    QLineEdit* ymaxEdit;
    QLineEdit* zmaxEdit;
    
    QWidget* tab_2;
    QButtonGroup* grupColors;
    QLabel* textLabel5_2;
    QLabel* textLabel6;
    QLabel* textLabel9;
    QLabel* textLabel1;
    QPushButton* botoColorFons2d;
    QPushButton* botoColorLinies2d;
    QPushButton* botoColorPunts2d;
    QPushButton* botoColorSelect;
    QPushButton* botoHelp;
    QPushButton* botoOk;
    QPushButton* botoCancel;

    QWidget* tab_3;
    QGroupBox* groupBox2;
    QLabel* textLabel32;
    QLabel* textLabel33;
    QLineEdit* lmxEdit;
    QLineEdit* lmyEdit;
    
    QWidget* tab_4;
    QGroupBox* groupBox4;
    QLabel* textLabel4_1;
    QLabel* textLabel4_2;
    QLineEdit* filaEdit;
    QLineEdit* colEdit;
    
    ConfigInfo dades();
    void setDades( ConfigInfo & );    

protected:
    QVBoxLayout* ConfigDialegLayout;
    QVBoxLayout* tabLayout;
    QSpacerItem* spacer3;
    QHBoxLayout* groupBox1Layout;
    QHBoxLayout* groupBox2Layout;
    QHBoxLayout* groupBox4Layout;
    QSpacerItem* spacer12;
    QSpacerItem* spacer13;
    QVBoxLayout* layout20;
    QVBoxLayout* layout30;
    QVBoxLayout* layout40;
    QVBoxLayout* layout16;
    QVBoxLayout* layout18;
    QVBoxLayout* layout17;
     QVBoxLayout* layout4_1;
    QVBoxLayout* layout4_2;
    QVBoxLayout* tabLayout_2;
    QVBoxLayout* tabLayout_3;
     QVBoxLayout* tabLayout_4;
    QSpacerItem* spacer16;
    QSpacerItem* spacer4_1;
    QHBoxLayout* grupColorsLayout;
    QSpacerItem* spacer14;
    QVBoxLayout* layout18_2;
    QVBoxLayout* layout17_2;
    QHBoxLayout* Layout1;
    QSpacerItem* Horizontal_Spacing2;

protected slots:
    virtual void languageChange();

private slots:
    void triaColors(int);
    void comprovarInfo();

private:
    ConfigInfo conf;
};

#endif // CONFIGDIALEG_H
