/************
 * Fitxer: corbesnivell.cpp
 * PTFC: Disseny d'una eina de modelat de terrenys i c�cul de corbes de nivell
 * Autor: Sergio Jaraiz Ojea
 * @ Gener 2006 
 * 
 */  

#include "corbesnivell.h"
#include <CGAL/Object.h>
#include <CGAL/intersections.h>
#include <iostream>

#define NIVELL_MAXIM   100000.0

CorbesNivell::CorbesNivell() {}

CorbesNivell::~CorbesNivell() {}

void es( pCara a )
{
  qDebug("CARA: (%f, %f, %f), (%f, %f, %f), (%f, %f, %f).", 
          a->vertex(0)->point().x(), a->vertex(0)->point().y(), a->vertex(0)->point().z(),
          a->vertex(1)->point().x(), a->vertex(1)->point().y(), a->vertex(1)->point().z(),
          a->vertex(2)->point().x(), a->vertex(2)->point().y(), a->vertex(2)->point().z()
        );
}

void es( Triangle3 a )
{
  qDebug("TRIANGLE: (%f, %f, %f), (%f, %f, %f), (%f, %f, %f).", a[0].x(), a[0].y(), a[0].z(),
                                                                a[1].x(), a[1].y(), a[1].z(),
                                                                a[2].x(), a[2].y(), a[2].z()
        );
}

void es( Segment3D s )
{
  qDebug("SEGMENT: (%f, %f, %f), (%f, %f, %f).", s.source().x(), s.source().y(), s.source().z(),
                                                 s.target().x(), s.target().y(), s.target().z()
        );
}

bool diferents( pCara a, pCara b )
{
  return(  !iguals( a->vertex(0)->point().x() ,b->vertex(0)->point().x() ) ||
           !iguals( a->vertex(1)->point().y() ,b->vertex(1)->point().y() ) ||
           !iguals( a->vertex(2)->point().z() ,b->vertex(2)->point().z() )
        );
}

struct inicialitza
{
   void operator()( Cara &c )
   {
      c.nivell = 0;      
   }
};

void CorbesNivell::generaCorbes( const Nivells &nv, Delaunay &trig )
{
   //qDebug("generaCorbes");
   unsigned int i;
   pCara seguent, primer;
   bool adjacent_valid = TRUE;
   corbes2d.clear();
   corbes3d.clear();           
  
   //inicialitzar
   for( IteradorCares fit = trig.finite_faces_begin(); fit != trig.finite_faces_end(); ++fit )
   {
        fit->info().nivell = 0;
        for_each( fit->info().cares.begin(), fit->info().cares.end(), inicialitza() );
   }

//qDebug("Inicialitzat.");   
   for( i = 1; i < nv.size(); i++ )
   {
     cb.punts.clear();
     cb.color = nv[i].color;
     for( IteradorCares fit = trig.finite_faces_begin(); fit != trig.finite_faces_end(); ++fit )
     {                   
       if( mesPetit( fit->info().nivell, nv[i].valor ) )
       { 
         if( entreNivells( trig.triangle(fit), nv[i-1].valor, nv[i].valor ) ) 
         {           
//qDebug("Entrenivells Fora");           
           pintaSencer( fit->info().cares, nv[i].color );
	   fit->info().nivell = NIVELL_MAXIM;
         }
	 else if( sotaNivell ( trig.triangle(fit), nv[i].valor ) )
         {
//qDebug("Sotanivell Fora");
            retallaIPinta( (pCara)fit, nv, i );
            fit->info().nivell = NIVELL_MAXIM;
         }                                
	 else if( intercepta( trig.triangle(fit), nv[i].valor ) )
         {  
	    //qDebug("Intercepta");
            primer = (pCara)fit;	     
            seguent = primer;
            do
            {
               //qDebug("Dins Do");               
               retallaIPinta( seguent, nv, i );               
	       seguent->info().nivell = nv[i].valor;	       
	       adjacent_valid = seguentAdjacent( seguent, trig, nv[i].valor );	       
            } while( ( seguent != primer ) && adjacent_valid );
            //qDebug("Fora Do");               
	    //cb.punts.push_back( inicial );
         }                   
       }
     }
     //qDebug("PUNTS AQUI= %d",cb.punts.size());
     corbes2d.push_back( cb );
   }
}

bool CorbesNivell::seguentAdjacent( pCara &seguent, Delaunay &tri, double nivell )
{
//qDebug("SeguentAdjacent ");
    if( !tri.is_infinite(seguent->neighbor(0)) &&
        intercepta( tri.triangle(seguent->neighbor(0)), nivell ) && 
	( mesPetit( seguent->neighbor(0)->info().nivell, nivell) ) )
    {	
//qDebug("Adjacent 0");
	   seguent = seguent->neighbor(0);
	   return TRUE;
    }
    else if( !tri.is_infinite(seguent->neighbor(1)) &&
             intercepta( tri.triangle(seguent->neighbor(1)), nivell ) && 
	     ( mesPetit( seguent->neighbor(1)->info().nivell, nivell) ) )
    {	   	
//qDebug("Adjacent 1");	   
           seguent = seguent->neighbor(1);
	   return TRUE;
    }    
    else if( !tri.is_infinite(seguent->neighbor(2)) &&
             intercepta( tri.triangle(seguent->neighbor(2)), nivell ) && 
	     (mesPetit( seguent->neighbor(2)->info().nivell, nivell) ) )
    {	 
//qDebug("Adjacent 2");	   
	   seguent = seguent->neighbor(2);           
	   return TRUE;
    }
    else return FALSE;
}

CGAL::Object CorbesNivell::interseccio( Triangle3 t, double value )
{
    CGAL::Object res1, res2, res3;   

    Segment3D seg1(t[0], t[1]);
    Segment3D seg2(t[1], t[2]);
    Segment3D seg3(t[0], t[2]);   
    
    Plane pla(Punt3d(0.0, 0.0, value), Vector(0.0, 0.0, 1.0)); 

    Punt3d ipoint1, ipoint2, ipoint3;
    Segment3D isegment1, isegment2, isegment3;

    res1 = CGAL::intersection(pla, seg1);
    res2 = CGAL::intersection(pla, seg2);
    res3 = CGAL::intersection(pla, seg3);

    //El triangle intercepta completament
    if( CGAL::assign(isegment1, res1) && CGAL::assign(isegment2, res2) && CGAL::assign(isegment3, res3) )
    {
       return CGAL::make_object(t); 
    }
    else if( CGAL::assign(ipoint1, res1) && CGAL::assign(ipoint2, res2) && CGAL::assign(ipoint3, res3) )  //Triangle3 inclinat
    { 
        if (ipoint1 == ipoint2) //Intersecciona amb un dels vertexs
        {
           Segment3D s = Segment3D(ipoint1, ipoint3);
           return CGAL::make_object(s);           
        }
        else //if (ipoint1 == ipoint3) || (ipoint2 == ipoint3)
        {
           Segment3D s = Segment3D(ipoint1, ipoint2);
           return CGAL::make_object(s);                     
        }
    }    
    else if( CGAL::assign(isegment1, res1) )
    {
      //std::cout << "EL OTRO1 " << isegment << std::endl;      
      Segment3D s = Segment3D( t[1], t[0] );            
      return CGAL::make_object(s);
    }
    else if( CGAL::assign(isegment1, res2) )
    {
      //std::cout << "EL OTRO2 " << isegment << std::endl;  
      Segment3D s = Segment3D( t[1], t[2] );
      return CGAL::make_object(s);
    }
    else if( CGAL::assign(isegment1, res3) )
    {
      //std::cout << "EL OTRO3 " << isegment << std::endl;  
      Segment3D s = Segment3D( t[0], t[2] );
      return CGAL::make_object(s);
    }  
    else if (CGAL::assign(ipoint1, res1) && CGAL::assign(ipoint2, res2))  //interseccio segment o vertex
    {
        if (ipoint1 == ipoint2) //interseccio es un vertex
        {
           //Punt3d p = ipoint1;
           //return CGAL::make_object(p);         
           return res1;
        }
        else //interseccio es un segment
        {
           Segment3D s = Segment3D(ipoint1, ipoint2);
           return CGAL::make_object(s);
        }	
    }
    else if (CGAL::assign(ipoint2, res2) && CGAL::assign(ipoint3, res3))  
    { 
        if (ipoint2 == ipoint3) 
        {
           //Punt3d p = ipoint2;
           //return CGAL::make_object(p);         
           return res2;
        }
        else 
        {
           Segment3D s = Segment3D(ipoint2, ipoint3);
           return CGAL::make_object(s);
        }	
    }
    else if (CGAL::assign(ipoint1, res1) && CGAL::assign(ipoint3, res3))
    {
        if (ipoint1 == ipoint3) 
        {
           //Punt3d p = ipoint1;
           //return CGAL::make_object(p);         
           return res1;
        }
        else 
        {
           Segment3D s = Segment3D(ipoint1, ipoint3);
           return CGAL::make_object(s);
        }	
    }
    else //No intersecta
    {
         return CGAL::Object();
    }  
}

bool CorbesNivell::sotaNivell( Triangle3 t, double nivell )
{
   return (
           ( mesPetit(t[0].z(), nivell) || iguals(t[0].z(), nivell)) && 
           ( mesPetit(t[1].z(), nivell) || iguals(t[1].z(), nivell)) && 
           ( mesPetit(t[2].z(), nivell) || iguals(t[2].z(), nivell))
          );
}

bool CorbesNivell::sobreNivell( Triangle3 t, double nivell )
{   
   return (
           ( mesGran(t[0].z(), nivell) || iguals(t[0].z(), nivell)) && 
           ( mesGran(t[1].z(), nivell) || iguals(t[1].z(), nivell)) && 
           ( mesGran(t[2].z(), nivell) || iguals(t[2].z(), nivell))
          );
}

bool CorbesNivell::entreNivells( Triangle3 t, double ninf, double nsup )
{
   return ( sobreNivell( t, ninf ) && sotaNivell( t, nsup ) );
}

bool CorbesNivell::intercepta( Triangle3 t, double nivell )
{   
   return( do_intersect( Plane(Punt3d(0.0, 0.0, nivell), Vector(0.0, 0.0, 1.0)), t ) );
}

void CorbesNivell::retallaIPinta( pCara cara, const Nivells &nv, int i )
{
   LlistaCares::iterator cit, inici, fi, primer, seguent;
   bool adjacent_valid = FALSE;

   inici = cara->info().cares.begin();
   fi = cara->info().cares.end();
//qDebug("NIVELL %f ---------------------------------", nv[i].valor);
   for( cit = inici; cit != fi; ++cit )
   {                   
     if( mesPetit( cit->nivell, nv[i].valor ) )
     {         
       if( entreNivells( cit->t, nv[i-1].valor, nv[i].valor ) ) 
       {         
//qDebug("Entrenivells");
         corbes3d.push_back( Cara( cit->t[0], cit->t[1], cit->t[2], nv[i].color) );
	 cit->nivell = NIVELL_MAXIM;         
       }
       else if( sotaNivell ( cit->t, nv[i].valor ) )
       {
//qDebug("Sotanivell");
          retallaPerSobre( cit->t, nv, i-1 );
          cit->nivell = NIVELL_MAXIM;
       }                        
       else if( intercepta( cit->t, nv[i].valor ) )
       {  
	  //qDebug("Intercepta");
          primer = cit;	     
          seguent = primer;
          do
          { 	       
             //qDebug("Dins Do, retallaIPinta");               
             retallaPerSota( seguent->t, nv, i );             
	     seguent->nivell = nv[i].valor;	       
	     adjacent_valid = buscaAdjacent( seguent, cara->info().cares, nv[i].valor );
          } while( ( seguent != primer ) && adjacent_valid );
          //qDebug("Fora Do, retallaIPinta");               
	  //cb.punts.push_back( inicial );
       } // Si la cara esta entre nivells pintar totes les cares interiors del mateix color                     
     }
   }
//qDebug("---------------------------------");   
}

void CorbesNivell::retallaPerSota( Triangle3 t, const Nivells &nv, int i )
{
   Punt3d p;
   Segment3D s;
   double nivell = nv[i].valor;
   double z0, z1, z2;
   CGAL::Object res = interseccio( t, nivell );
   LlistaPunts::iterator m;
   
   
   z0 = t[0].z();
   z1 = t[1].z();
   z2 = t[2].z();
   
   if( CGAL::assign( s, res ) ) //intercepta un segment
   { 
      if( mesGran(z0, nivell) && mesGran(z1, nivell) )
      {
//qDebug("Sota - 1");
         retallaPerSobre( Triangle3( s.source(), t[2], s.target() ), nv, i-1 );	 
	 cb.punts.push_back( s.source() );
	 cb.punts.push_back( s.target() );
      }
      else if( mesGran(z1, nivell) && mesGran(z2, nivell) )
      {
//qDebug("Sota - 2");
         retallaPerSobre( Triangle3( s.target(), t[0], s.source() ), nv, i-1 );	   
	 cb.punts.push_back( s.source() );
	 cb.punts.push_back( s.target() );
      }
      else if( mesGran(z0, nivell) && mesGran(z2, nivell) )
      {
//qDebug("Sota - 3");
         retallaPerSobre( Triangle3( s.source(), t[1], s.target() ), nv, i-1 );
	 cb.punts.push_back( s.source() );
	 cb.punts.push_back( s.target() );
      }
      else if( mesGran(z0, nivell) )
      {
         if( iguals( z1, nivell) && mesPetit( z2, nivell ) )
	 {
//qDebug("Sota - 4");
	   retallaPerSobre( Triangle3( s.source(), t[2], s.target() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
	 else if( iguals( z2, nivell ) && mesPetit( z1, nivell ) )
	 {
//qDebug("Sota - 5");
	   retallaPerSobre( Triangle3( s.source(), t[1], s.target() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
  	   cb.punts.push_back( s.target() );
	 }
	 else if( mesPetit(z0, nivell) || mesPetit(z1, nivell)  ||  mesPetit(z2, nivell) )
	 {
//qDebug("Sota - 6");
            retallaPerSobre( Triangle3( s.source(), t[1], t[2] ), nv, i-1 );
	    retallaPerSobre( Triangle3( s.source(), t[2], s.target() ), nv, i-1 );
	    cb.punts.push_back( s.source() );
  	    cb.punts.push_back( s.target() );
	 }
      }
      else if( mesGran(z1, nivell) )
      {
         if( iguals( z0, nivell) && mesPetit( z2, nivell ) )
	 {
//qDebug("Sota - 7");
	   retallaPerSobre( Triangle3( s.target(), t[2], s.source() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
	 else if( iguals( z2, nivell ) && mesPetit( z0, nivell ) )
	 {
//qDebug("Sota - 8");
	   retallaPerSobre( Triangle3( s.target(), t[0], s.source() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
	 else if( mesPetit(z0, nivell) || mesPetit(z1, nivell)  ||  mesPetit(z2, nivell) )
	 {
//qDebug("Sota - 9");      
           retallaPerSobre( Triangle3( s.target(), t[2], t[0] ), nv, i-1 );
	   retallaPerSobre( Triangle3( s.target(), t[0], s.source() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
      }
      else if( mesGran(z2, nivell) )
      {
         if( iguals( z0, nivell) && mesPetit( z1, nivell ) )
	 {
//qDebug("Sota - 10");
	   retallaPerSobre( Triangle3( s.source(), t[1], s.target() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
	 else if( iguals( z1, nivell ) && mesPetit( z0, nivell ) )
	 {
//qDebug("Sota - 11");
	   retallaPerSobre( Triangle3( s.target(), t[0], s.source() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
	 else if( mesPetit(z0, nivell) || mesPetit(z1, nivell)  ||  mesPetit(z2, nivell) )
	 {
//qDebug("Sota - 12");
           retallaPerSobre( Triangle3( s.target(), t[0], t[1] ), nv, i-1 );
  	   retallaPerSobre( Triangle3( s.target(), t[1], s.source() ), nv, i-1 );
	   cb.punts.push_back( s.source() );
	   cb.punts.push_back( s.target() );
	 }
      }
      else  //Una aresta de la cara intercepta completament
      {
         if( mesPetit(z0, nivell) || mesPetit(z1, nivell)  ||  mesPetit(z2, nivell) )        
         {
//qDebug("Sota - 13");
            retallaPerSobre( Triangle3( t[0], t[1], t[2] ), nv, i-1 );
	    cb.punts.push_back( s.source() );
	    cb.punts.push_back( s.target() );
         }     
         else 
         {
//qDebug("Tercer error");
	    cb.punts.push_back( s.source() );
	    cb.punts.push_back( s.target() );
         }
      }
   }
   else if( CGAL::assign( p, res ) )
   {
      if( mesPetit(z0, nivell) || mesPetit(z1, nivell)  ||  mesPetit(z2, nivell) )        
      {
//qDebug("Sota - 14");
         retallaPerSobre( Triangle3( t[0], t[1], t[2] ), nv, i-1 );
      }   
      //else qDebug("Quart error"); 
   }
   else 
   {
//qDebug("Sota - 15");
     retallaPerSobre( Triangle3( t[0], t[1], t[2] ), nv, i-1 );
   }
}

void CorbesNivell::retallaPerSobre( Triangle3 t, const Nivells &nv, int i )
{
   Punt3d p;
   Segment3D s;
   CGAL::Object res = interseccio( t, nv[i].valor );
   double z0, z1, z2;
   
   z0 = t[0].z();
   z1 = t[1].z();
   z2 = t[2].z();
   
   if( CGAL::assign( s, res ) )
   { 
      if( mesPetit(z0, nv[i].valor) && mesPetit(z1, nv[i].valor) )
      {
//qDebug("Sobre - 1");
         corbes3d.push_back( Cara( t[2], s.target(), s.source(), nv[i+1].color ) );	 
      }
      else if( mesPetit(z1, nv[i].valor) && mesPetit(z2, nv[i].valor) )
      {
//qDebug("Sobre - 2");
         corbes3d.push_back( Cara( t[0], s.source(), s.target(), nv[i+1].color ) );
      }
      else if( mesPetit(z0, nv[i].valor) && mesPetit(z2, nv[i].valor) )
      {
//qDebug("Sobre - 3");
         corbes3d.push_back( Cara( t[1], s.target(),s.source(), nv[i+1].color ) );
      }
      else if( mesPetit(z0, nv[i].valor) )
      {
         if( iguals( z1, nv[i].valor) && mesGran( z2, nv[i].valor ) )
	 {
//qDebug("Sobre - 4");	  
	   corbes3d.push_back( Cara( t[2], s.target(), s.source(), nv[i+1].color ) );
	 }
	 else if( iguals( z2, nv[i].valor) && mesGran( z1, nv[i].valor ) )
	 {
//qDebug("Sobre - 5");
	   corbes3d.push_back( Cara( t[1], s.target(), s.source(), nv[i+1].color ) );	 
	 }
         else if( mesGran(z0, nv[i].valor) || mesGran(z1, nv[i].valor)  ||  mesGran(z2, nv[i].valor) )
	 {
//qDebug("Sobre - 6");    
           corbes3d.push_back( Cara( t[2], s.target(), s.source(), nv[i+1].color ) );
	   corbes3d.push_back( Cara( t[2], s.source(), t[1], nv[i+1].color ) );
	 }
      }
      else if( mesPetit(z1, nv[i].valor) )
      {
         if( iguals( z0, nv[i].valor) && mesGran( z2, nv[i].valor ))
	 {
//qDebug("Sobre - 7");
	   corbes3d.push_back( Cara( t[2], s.source(), s.target(), nv[i+1].color ) );
	 }
         else if( iguals( z2, nv[i].valor) && mesGran( z0, nv[i].valor ) )
	 {
//qDebug("Sobre - 8");
	   corbes3d.push_back( Cara( t[0], s.source(), s.target(), nv[i+1].color ) );	 
	 }
	 else if( mesGran(z0, nv[i].valor) || mesGran(z1, nv[i].valor)  ||  mesGran(z2, nv[i].valor) )
	 {
//qDebug("Sobre - 9");             
	   corbes3d.push_back( Cara( t[0], s.source(), s.target(), nv[i+1].color ) );
	   corbes3d.push_back( Cara( t[0], s.target(), t[2], nv[i+1].color ) );
	 }
      }
      else if( mesPetit(z2, nv[i].valor) )
      {
         if( iguals( z0, nv[i].valor) && mesGran( z1, nv[i].valor ) )
	 {
//qDebug("Sobre - 10");
	   corbes3d.push_back( Cara( t[1], s.target(), s.source(), nv[i+1].color ) );
	 }
	 else if( iguals( z1, nv[i].valor) && mesGran( z0, nv[i].valor ) )
	 {
//qDebug("Sobre - 11");
	   corbes3d.push_back( Cara( t[0], s.source(), s.target(), nv[i+1].color ) );
	 }
         else if( mesGran(z0, nv[i].valor) || mesGran(z1, nv[i].valor)  ||  mesGran(z2, nv[i].valor) )
	 {
//qDebug("Sobre - 12");    
           corbes3d.push_back( Cara( t[1], s.source(), s.target(), nv[i+1].color ) );
	   corbes3d.push_back( Cara( t[1], s.target(), t[0], nv[i+1].color ) );
	 }
      }
      else  //Una aresta de la cara intercepta completament
      {
         if( mesGran(z0, nv[i].valor) || mesGran(z1, nv[i].valor)  ||  mesGran(z2, nv[i].valor) )
         {
//qDebug("Sobre - 13");
            corbes3d.push_back( Cara( t[0], t[1], t[2], nv[i+1].color ) );
         }     
         //else qDebug("Primer error");
      }
   }
   else if( CGAL::assign( p, res ) )
   {
      if( mesGran(z0, nv[i].valor) || mesGran(z1, nv[i].valor)  ||  mesGran(z2, nv[i].valor) )
      {
//qDebug("Sobre - 14");
         corbes3d.push_back( Cara( t[0], t[1], t[2], nv[i+1].color ) );
      }    
      //else qDebug("Segon error");
   }   
   else  
   {
//qDebug("Sobre - 15");
     corbes3d.push_back( Cara( t[0], t[1], t[2], nv[i+1].color ) );
   }  
}

void CorbesNivell::pintaSencer( LlistaCares &llc, QColor color )
{

   LlistaCares::iterator lcit;
   //qDebug("COLOR=(%d,%d,%d)", color.red(), color.green(), color.blue());
   for( lcit = llc.begin(); lcit != llc.end(); ++lcit )
   {
      corbes3d.push_back( Cara( (*lcit)[0], (*lcit)[1], (*lcit)[2], color) );
      //es(lcit->t);
   }
/*   
   for( LlistaCares::iterator itt = corbes3d.begin(); itt != corbes3d.end(); ++itt )
   {
      es(itt->t);
      color = itt->color;
      qDebug("COLOR=(%d,%d,%d)", color.red(), color.green(), color.blue());
   }
*/
}

bool CorbesNivell::sonAdjacents( Cara &a, Cara &b)
{
   if( a != b )
   {
      return( ( a.teVertex(b[0]) && a.teVertex(b[1]) ) || 
              ( a.teVertex(b[1]) && a.teVertex(b[2]) ) ||
	      ( a.teVertex(b[2]) && a.teVertex(b[0]) ) );     
   }
   else return FALSE;
}

bool CorbesNivell::buscaAdjacent( LlistaCares::iterator &ct, LlistaCares &lc, double nivell )
{
   bool trobat;

   trobat = FALSE;
   LlistaCares::iterator it = lc.begin();
   while( (it != lc.end()) && !trobat )
   {
      if(sonAdjacents( *it, *ct ) && mesPetit( it->nivell, nivell ) && intercepta( it->t, nivell ) )
      {
         ct = it;
         trobat = TRUE;
      }

      ++it;
   }
   if( trobat ) return TRUE;
   else return FALSE;
}
