/************
 * Fitxer: corbesnivell.h
 * PTFC: Disseny d'una eina de modelat de terrenys i c�cul de corbes de nivell
 * Autor: Sergio Jaraiz Ojea
 * @ Gener 2006 
 * 
 */  

#ifndef CORBES_DE_NIVELL_H
#define CORBES_DE_NIVELL_H

#include <qcolor.h>
#include <vector>
#include "geometriabasica.h"

struct Nivell 
{
  double  valor;
  QColor color;

  Nivell()
  {
      valor = 0.0; 
      color = Qt::green;
  }

  Nivell( double value, QColor c )
  {
      valor = value;
      color = c;
  }
  
  Nivell( const Nivell& aux )
  {
       color = aux.color;
       valor = aux.valor;
  }

  Nivell& operator=( const Nivell& aux )
  {       
       color = aux.color;
       valor = aux.valor;

       return (*this);
  }

  friend bool menor( const Nivell &a, const Nivell &b )
  {
     return mesPetit( a.valor, b.valor );
  }
};

typedef std::vector<Nivell> Nivells;

class CorbesNivell
{
public:
   CorbesNivell();
   ~CorbesNivell();

   void generaCorbes( const Nivells &, Delaunay& );
   LlistaCorbes corbes2d;      
   LlistaCares corbes3d;
   
private:

   bool sotaNivell( Triangle3, double );
   bool sobreNivell( Triangle3, double );
   bool entreNivells( Triangle3, double , double );
   bool intercepta( Triangle3, double );

   bool sonAdjacents( Cara &, Cara & );
   bool seguentAdjacent( pCara &, Delaunay &, double );
   bool buscaAdjacent( LlistaCares::iterator &, LlistaCares &, double );

   void retallaIPinta( pCara, const Nivells &, int );
   void retallaPerSota( Triangle3, const Nivells &, int );
   void retallaPerSobre( Triangle3, const Nivells &, int );
   void pintaSencer( LlistaCares&, QColor );
   CGAL::Object interseccio( Triangle3, double );  
   void guardaCorba2d( Segment3D, int );
   
   Punt3d inicial;
   Corba cb;
   bool inici_nivell;
};

#endif
