#ifndef CLASSE_EDITORTERRENY_H
#define CLASSE_EDITORTERRENY_H

#include <qgl.h>
#include <math.h>
#include <qsize.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include "zdialeg.h"
#include <qaction.h>
#include <vector>
#include <list.h>
#include <algorithm>
#include "terreny.h"
#include "event.h"
#include "finestra.h"
#include "pseudosource.h"
#include "geometriabasica.h"
#include <qerrormessage.h>
#include <pair.h>

#include "Cg/cg.h"
#include "Cg/cgGL.h"

#include "visio.h"

struct InfoMon_ET
{
	double xmin, ymin, zmin;
	double xmax, ymax, zmax;
	Punt vertex_xmin, vertex_xmax, vertex_ymin, vertex_ymax ;
	bool _modificat;
};

struct InfoMostrar_ET
{
	bool dibuixa;
	bool veure_corbes;
	bool veure_triangulacio;
	bool veure_vertexs;
	bool veure_parts_visibles;
	bool veure_elements_visio;
	bool comprovar_visibilitat;
	
	bool veure_diagrama_voronoi;
	
	bool veure_dibuix;
	bool propagacio;
	bool repintar_finestres;
	bool pintar_cami;
			
	GLuint seleccionat; //index d'un vertex o punt seleccionat amb la rata.
	
	/*int index1;
	int index2;
	int index3;*/
};

struct InfoOpenGL_ET
{
	QColor color_fons;
	QColor color_punts;
	QColor color_linies;
	QColor color_seleccio;
	QColor color_skinny;
};

struct InfoMostrarVisibilitat_ET
{
    int tipus_visualitzacio;
    int tipus_anterior;
    int segment_escollit;
    int primer_segment_visio;
    int segon_segment_visio;
    int tercer_segment_visio;
};

struct InfoGraphicHardware_ET
{
	CGparameter text;
	GLuint texID;
	CGcontext context;
	CGprogram fragmentProgram;
	CGprofile fragmentProfile;
	bool haveLoadedPrograms;
};

struct InfoTexturaVisio_ET
{
	double xminT, yminT, zminT;
	double xmaxT, ymaxT, zmaxT;
	
	int amplada;
	int alcada;
	
	CGparameter numero_segments; // numero de segmetns dels quals la tetura guarda informacio.
	CGparameter numeros_esperats; 
	GLubyte *textura;
};

struct InfoVisibilitat_ET
{
	int visio;
	
	//CREACIO ELEMENTS VISIO: punts, segments i poligonals.
	bool punt_segment;
	bool dibuixant_poligon;
	bool punts_pla_poligon;
	Punt inici_segment, fi_segment;
	Punt punt_actual;
	vector<Punt> punts_poligon;
	Pla * pla_poligon;

	// parametres utilitzats per comprvar el resultat
	Segment raig;
	Triangle triangle;
	vector<pCara> triangles_inter;
	Punt punt_comprovar;
	int tip;
	
	InfoTexturaVisio_ET text;   // informacio referent a la textura on es guarda la informacio de la visibilitat
	InfoGraphicHardware_ET gh;   // informacio necessaria per ulitzar GH per pintar la informacio guardada a la textura.
	InfoMostrarVisibilitat_ET mostrar;  // quina informacio de la guardada a la textura volem mostrar cada vegada.
};

struct InfoCalculs_ET //Informació relativa a calculs de distàncies
{

	bool mode_visibilitat;
	bool mode_distancies;
	bool calcul_distancies;
	Event * eventactual;
	bool punt_distancies;
	vector<pCara> caresSource;	

};

// typedef std::vector<pVertex>  VectorVertexs;

/******************************************************************/
/*              class EditorTerreny : public QGLWidget            */
/*   Classe que permet representar i gestionar el terreny en 2D   */
/******************************************************************/

class EditorTerreny : public QGLWidget
{
    Q_OBJECT
public:
    EditorTerreny( QWidget *parent=0, const char *name=0 );
    ~EditorTerreny();

    bool carregaTerreny( const QString& fn );
    bool desaTerreny( const QString& fn );
    void mostraVisor();
    void clear();
    void eliminaElement();
    void generaTerreny( int );
    void modificaMida( double[6] );   
    void modificaColors( QColor[4] );  
    void setAccio( GLenum );
    void setVisio( int );
    bool modificat();
    void setModificat();
    void modificaMidaGraella( int w, int h );
    void veureTextura(bool);
    
    void LoadCgPrograms();
    void ChooseProfiles();
    void handleCgError();
    
    void configuracioVisio();
    void configuracioDistancies();
    void configuracioPuntDistancies();
    bool getModeVisibilitat();
    bool getModeDistancies();
    
    void calculsDistancies();
    void calculsCaminsMinims();
    
    void triemMon(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax);
    void dibuixemQ(); 
    void dibuixemFinestres();
    void dibuixarDiagramesVoronoi();
    void trobaCami();

    void inicialitzaVertexs();
    void comprovaVertexs();
    void mirarIndexs(double &index1, double &index2, double &index3, double &index_1);
    void esborrarFinestresRepetides(double &borrades);
    list<Event*> inicialitzacio();
    void pintarInicialitzacio(list<Event*> E);
    void repintarFinestres();
    //list<Event*> propagacio(list<Event*> E);
    void propagacio(list<Event*> E);
    void propagacioNOMESPSEUDOSOURCE(list<Event*> E);
    void propagacioNOMES1FINESTRA(list<Event*> E);
    //list<Event*> deteccioEventsReals(list<Event*> E, list<Event*> PossiblesEvents);
    void deteccioEventsReals(list<Event*> &E, list<Event*> PossiblesEvents);
    void actualitzar(list<Event*> &Bona, Finestra * f, Finestra * fn);
    vector<Punt> camiMesCurt(Punt p, pCara cara, pCara &cara_source);
    //void trobarDistanciaMinima(Punt p, pCara cara, Finestra &f, double &x);
    void trobarDistanciaMinima2(Punt p, pCara cara, Finestra &f, double &x, vector<Punt> &vp);
    void saltarVeina(Finestra f, double x, Finestra &fnova, double &xnova, Punt &puntnou, vector<Punt> &vp);
    void tractarVertex(int j, pCara cara, Finestra &fnova, double &xnova, Punt &puntnou);
    void jocProves();
       
private:
	
	Terreny *mesh;
        Visio * objectes_visio;
	Visio * objectes_distancies;
	Visio * objectes_cami_minim;
//     VectorVertexs vertexs;
  
	InfoMon_ET info_mon;     // referent al terreny
	InfoMostrar_ET info_mostrar;   // que volem veure per pantalla
	InfoOpenGL_ET info_GL;     // colors en què pintem utilitzant openGL
	InfoVisibilitat_ET info_vis;  // informaico necessaria quan estem tractant visilbitat (crear objectes, visulatizar infomracio ulitzant GH)
	InfoCalculs_ET info_calculs; 
    
	bool pressed;
	GLenum accio;
	QPopupMenu *auxmenu;  
	QErrorMessage *errors;
	
	int posicio;  
    
    //TERRENY
    void marcaSeleccionat();
    void initIndexos();  
    void dibuixaPunts( GLenum, const QColor &color = Qt::yellow );
    void dibuixaTriangulacio2D( const QColor & );
    
    //BEZIER
    void cercle( double centre_x, double centre_y, double radi, unsigned int segments );
    void dibuixaCorbes();
    
    //VISIBILITAT
    void dibuixarObjectesVisio( );
    void dibuixarElementVisio( );
    // crear elements visio (punts, segments, poligons, poligonals)
    void tractarFiSegment( Objecte * punter );
    void tractarIniciSegment();
    bool comprovarPoligonMenys2Vertexs(const double &x, const double &y, const double &z);
    bool comprovarPoligonMes2Vertexs(QMouseEvent *e, Punt mon);
    void tractarPuntVisio(Objecte * punter);
    void tractarPuntDistancies(Objecte * punter);
    void tractarElementVisio(QMouseEvent *e, const double &x, const double &y, const double &z);
    void tractarElementDistancies(QMouseEvent *e, const double &x, const double &y, const double &z);
    //Guardar i mostrar informacio visibilitat
    void construirTextura();
    void inicialitzaCG();  // inicialitzar per fer servir hardwaregraphic
    void assignarParametres(); 
    void mostarResultatsVisibilitat();
    void comprovarResultatVisibilitat();
    
    //DISTANCIES
    void assignarColorDV(int j);
    void activarFragment();
    void mostrarDiagramaVoronoi();
    void propagacioCaraAproximada(pCara cara);
    
    
private slots:    
    void modificaZ();
    void borraElement();
    //BEZIER
    void mostraInfo();
    //VISIBILITAT
    void activaUnio();
    void activaInterseccio();
    void activaCounting();
    void activaSegment();
    void activaCombinat();
    void segmentEscollit(int segment);
    void primerSegmentVisio( int segment);
    void segonSegmentVisio( int segment);
    void tercerSegmentVisio( int segment);

public slots:    
    void mostraTriangulacio( bool );
    void mostraVertexs( bool );
    void mostraCorbes( bool ); //BEZIER
    void mostraElementsVisio( bool ); //VISIBILITAT

signals:
    void terrenyModificat();
    void rataStatus( double , double );
    void mostrarInfVisibilitatAmbRata(int,bool,double,double);
    void desactiva1();
    void desactiva2();
    void desactiva3();

protected:
    void initializeGL();
    void resizeGL( int w, int h );
    void paintGL();

    void mouseReleaseEvent( QMouseEvent * );
    void mousePressEvent( QMouseEvent * );
    void mouseMoveEvent( QMouseEvent * );

    void processHits( GLint, GLuint[] );
};

#endif   //CLASSE_EDITORTERRENY_H
