#ifndef EVENT_H
#define EVENT_H

#include <math.h>
#include <vector>
#include <qcolor.h>
#include "geometriabasica.h"
#include "segment.h"
#include <list.h>


/********************************************************************************************/
/*                           class Event                                                   */
/*   Classe abstracta que defineix els mètodes comuns per les finestres i els pseudo-sources*/
/********************************************************************************************/


class Event
{
	protected:
		
		int tipus_event;
		double pes;
				
	public:
		Event(); /*Constructor per defecte que inicialitza el tipus d'event a indefinit*/
		virtual ~Event();
		Event(int t); /*Constructor amb parametres que permet indicar el tipus d'event*/
				
		Event(const Event &ev); //Constructor de copia
		void swap(Event &evx, Event &evy); //Swap		
						
		int tipusEvent() const;
		double getpes() const;
	
		void assignarTipusEvent(int t);
		void setpes(double pes);
		
		// Redefinició d'operadors
		bool operator<(Event const &e )const; //Ordena segons el valor de l'atribut pes
		bool operator<(Event* const &e )const; //Ordena segons el valor de l'atribut pes
		static bool ordena(Event* const &e, Event* const &ev); //Ordena segons el valor de l'atribut pes de 2 punters
		
		//Mètodes virtuals
		//Aquests mètodes només es poden cridar des de punters a events (des d'un event que no sigui punter no)	
		virtual void escriure()const{}
		virtual double getb0()const{return 0;}
 		virtual double getb1()const{return 0;}
 		virtual double getd0()const{return 0;}
		virtual double getd1()const{return 0;}
		virtual double getsigma()const{return 0;} //Es a Finestra i a PseudoSource
		virtual pCara getcara()const{return NULL;} //Es a Finestra i a PseudoSource
		virtual int getj()const{return 0;}
		virtual int getindex()const{return 0;} //Es a Finestra i a PseudoSource
		virtual void getParametres(pCara &cara_, double &b0_, double &b1_, double &d0_,double &d1_, double &sigma_, int &index_, int &jfinestra_)const{}
		virtual pVertex getvertex() const{return NULL;}
		
		virtual void setb0(double b0_){}
		virtual void setb1(double b1_){}
		virtual void setd0(double d0_){}
		virtual void setd1(double d1_){}
		virtual void setsigma(double sigma_){} //Es a Finestra i a PseudoSource
		virtual void setcara(pCara cara_){} //Es a Finestra i a PseudoSource
		virtual void setj(int j_){}
		virtual void setindex(int index_){} //Es a Finestra i a PseudoSource
		//void setpes(double pes);
		virtual void setvertex(pVertex vertex_){}
		virtual void modifica_finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_){}
		virtual void modifica_pseudosource(double sigma_, int index_, pVertex vertex_, pCara cara_){}
				
		virtual bool menor(Finestra * fin){return 0;} //Diu si una Finestra és menor que una altra segons l'atribut b0.
		virtual Punt2d trobarSource(bool veins=false){return Punt2d(0,0);} //Donada la finestra trobem el source s
		virtual bool interseca(Finestra * fin, double &x, double &d, double &errors, double e, bool veina){return 0;}
		virtual bool no_elimina(Finestra * f, double x, double d, double errors, bool veins=false, double e=0){return 0;}
		virtual bool no_eliminada(){return 0;}
		virtual void actualitzarFinestres(Finestra * f, double x, double d, double errors, bool veins=false, double e=0){}
		virtual double distancia(double x){return min(-1000.,x);}
		virtual void setb0d0(double x, double e=0, double veins=false){}
		virtual void setb1d1(double x, double e=0, double veins=false){} 
		virtual double distanciaPuntProper(Punt p, double &x) {return 0;}
		virtual Punt3d trobarPunt3d(double x) {return Punt3d(0,0,0);}
		
		virtual void pintarFinestres(Delaunay *dt) {}
		virtual void pintar(Segment s, double e) {}
		virtual void pintarPseudoSource() {}
		virtual void pintarEvent() {}
		
		virtual bool getPinta() {return 0;}
		virtual void setPinta(bool pintar) {}
						
		virtual list<Event*> propagar(Delaunay *dt) {}
		virtual bool vertexHiperbolic(const pCara &cara, const pVertex &vertex, Delaunay *dt) {return 0;}
		//llista<Event* > propagar(Terreny const & mesh);
		virtual bool operator==(Event e);
		
};

#endif
