#include "finestra.h"
#include "pseudosource.h"


/**********************************************************************************************/
/*   Finestra::Finestra()                                                                     */
/*Constructor per defecte de la classe Finestra que inicialitza els atributs a 0              */
/**********************************************************************************************/

Finestra::Finestra() 
{
	b0 = 0.0;
	b1 = 0.0;
	d0 = 0.0;
	d1 = 0.0;
	sigma = 0.0;
	cara = NULL;
	j = 0;
	index = 0;
	pes=sigma;
	tipus_event = EVENT_FINESTRA;
	pinta=FALSE;
}

/*****************************************************************************/
/*   Finestra::Finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_) */
/*Constructor amb parametres que construeix la finestra amb els valors indicats                                 */
/*****************************************************************************/

Finestra::Finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_) 
{
	this->b0=b0_;
	this->b1=b1_;
	this->d0=d0_;
	this->d1=d1_;
	this->sigma=sigma_;
	this->cara=cara_;
	this->j=j_;
	this->index=index_;	
	this->pes=sigma_;
	pes=min(d0,d1)+sigma;	
	tipus_event = EVENT_FINESTRA;
	pinta=FALSE;
}

/***************************************************/
/*   Finestra::Finestra(const Finestra &fin)    */
/*Constructor de copia                             */  
/***************************************************/

Finestra::Finestra(const Finestra &fin)
{
	fin.getParametres(cara,b0,b1,d0,d1,sigma,index,j);
	pes=min(d0,d1)+sigma;	
	tipus_event = EVENT_FINESTRA;	
	this->setPinta(FALSE);
	//*this=fin;
}

/*****************************************************************/
/*   Finestra::swap(const Finestra &finx, const Finestra &finy) */
/*Metode swap                                                   */  
/****************************************************************/

void Finestra::swap(Finestra &finx,Finestra &finy)
{
Finestra tmp;
tmp=finx;
finx=finy;
finy=tmp;
}

/************************************/
/*   Finestra::~Finestra()          */
/*Destructor de la classe Finestra  */
/************************************/

Finestra::~Finestra() {}

/*****************************************************/
/*   double Finestra::getb0()                    */
/*Metode que retorna l'atribut b0 de la finestra */
/*****************************************************/

double Finestra::getb0()const
{
	return (b0);
}

/*****************************************************/
/*   double Finestra::getb1()                    */
/*Metode que retorna l'atribut b1 de la finestra */
/*****************************************************/

double Finestra::getb1()const
{
	return (b1);
}

/*****************************************************/
/*   double Finestra::getd0()                    */
/*Metode que retorna l'atribut d0 de la finestra */
/*****************************************************/

double Finestra::getd0()const
{
	return (d0);
}

/*****************************************************/
/*   double Finestra::getd1()                    */
/*Metode que retorna l'atribut d1 de la finestra */
/*****************************************************/

double Finestra::getd1()const
{
	return (d1);
}

/*****************************************************/
/*   double Finestra::getsigma()                    */
/*Metode que retorna l'atribut sigma de la finestra */
/*****************************************************/

double Finestra::getsigma()const
{
	return (sigma);
}

/*****************************************************/
/*   pCara Finestra::getcara()                       */
/*Metode que retorna l'atribut cara de la finestra   */
/*****************************************************/

pCara Finestra::getcara()const
{
	return (cara);
}

/*****************************************************/
/*   int Finestra::getj()                            */
/*Metode que retorna l'atribut j de la finestra      */
/*****************************************************/

int Finestra::getj()const
{
	return (j);
}


/*******************************************************/
/*   int Finestra::getindex()                       */
/*Metode que retorna l'atribut index de la finestra */
/*******************************************************/

int Finestra::getindex()const
{
	return index;
}

/*******************************************************/
/*   void Finestra::getParametres(pCara &cara_, double &b0_, double &b1_, double & d0_,double &d1_, double &sigma_, int &index_, int &jfinestra_)const                       */
/*Metode que retorna els paràmetres de Finestra */
/*******************************************************/

void Finestra::getParametres(pCara &cara_, double &b0_, double &b1_, double & d0_,double &d1_, double &sigma_, int &index_, int &jfinestra_)const
{
	cara_=cara;
	b0_=b0;
	b1_=b1;
	d0_=d0;
	d1_=d1;
	sigma_=sigma;
	index_=index;
	jfinestra_=j;
}

/*******************************************************/
/*   double Finestra::getpes()                         */
/*Metode que retorna l'atribut pes de la finestra      */
/*******************************************************/
/*
double Finestra::getpes()const
{
	return pes;
}*/

/***************************************************************/
/*   void   Finestra::setb0(double bo_)                         */
/*Metode que permet modificar el valor de l'atribut b0         */
/***************************************************************/

void Finestra::setb0(double b0_)
{
	this->b0=b0_;
}

/***************************************************************/
/*   void Finestra::setb1(double b1_)                           */
/*Metode que permet modificar el valor de l'atribut b1         */
/***************************************************************/

void Finestra::setb1(double b1_)
{
	this->b1=b1_;
}

/***************************************************************/
/*   void Finestra::setd0(double d0_)                      */
/*Metode que permet modificar el valor de l'atribut d0    */
/***************************************************************/

void Finestra::setd0(double d0_)
{
	this->d0=d0_;
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
}

/***************************************************************/
/*   void Finestra::setd1(double d1_)                           */
/*Metode que permet modificar el valor de l'atribut d1         */
/***************************************************************/

void Finestra::setd1(double d1_)
{
	this->d1=d1_;
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
}

/***************************************************************/
/*   void Finestra::setsigma(double sigma_)                     */
/*Metode que permet modificar el valor de l'atribut sigma      */
/***************************************************************/

void Finestra::setsigma(double sigma_)
{
	this->sigma=sigma_;
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
}

/**********************************************************/
/*   void Finestra::setcara(pCara cara_)                  */
/*Metode que permet modificar el valor de l'atribut cara */
/*********************************************************/

void Finestra::setcara(pCara cara_)
{
	this->cara=cara_;
}

/********************************************************/
/*   void Finestra::setj(int j_)                        */
/*Metode que permet modificar el valor de l'atribut j  */
/********************************************************/

void Finestra::setj(int j_)
{
	this->j=j_;
}

/***************************************************************/
/*   void Finestra::setindex(int index_)                        */
/*Metode que permet modificar el valor de l'atribut index      */
/***************************************************************/

void Finestra::setindex(int index_)
{
	this->index=index_;
}

/***************************************************************/
/*   void Finestra::setpes(double pes)                       */
/*Metode que permet modificar el valor de l'atribut pes      */
/***************************************************************/
/*
void Finestra::setpes(double pes)
{
	this->pes=pes;
}*/


/**********************************************************************/
/*   void Finestra::modifica_finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_);*/
/*Metode que permet modificar tots els abributs de finestra de cop    */
/**********************************************************************/

void Finestra::modifica_finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_)
{
	this->b0=b0_;
	this->b1=b1_;
	this->d0=d0_;
	this->d1=d1_;
	this->sigma=sigma_;
	this->cara=cara_;
	this->j=j_;
	this->index=index_;
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
	this->pinta=false;
}

/******************************************************************************** */
/*   bool Finestra::menor(Finestra * fin)                                          */
/*Metode que ens diu si una Finestra és menor que una altra segons l'atribut b0  */
/*********************************************************************************/

bool Finestra::menor(Finestra * fin)
{
	bool menor=false;
	if ( this->getb0() < fin->getb0() )
	{
	menor=true;
	}
	
	return (menor);
}

/******************************************************************************** */
/*   Punt2d Finestra::trobarSource(double e,bool veins)                                 */
/*Metode que ens troba el source s donada la finestra amb paràmetres             */
/*********************************************************************************/

Punt2d Finestra::trobarSource(double e, bool veins)
{
double a,b,c,d;
double Z;
d=0;

if (!veins)
{
	a=(d0*d0-d1*d1+b1*b1-b0*b0)/(2*(b1-b0));
	b=d0*d0;
	c=(b0-a)*(b0-a);
	Z=b-c;

	if (Z>0+1e-4 || Z>0-1e-4)
	{	
		d=sqrt(fabs(Z));
	}	
	else
	{
		cout << "Error: Arrel negativa en el trobarSource no veins!! " << endl;
	}

return Punt2d(a,d);
}
else //Veins
{
	a=(d1*d1-d0*d0+(e-b0)*(e-b0)-(e-b1)*(e-b1))/(2*(b1-b0));
	b=d1*d1;
	c=(e-b1-a)*(e-b1-a);
	Z=b-c;
	
	if (Z>0+1e-4 || Z>0-1e-4)
	{	
		d=sqrt(fabs(Z));
	}	
	else
	{
		cout << "Error: Arrel negativa en el trobarSource veins!! " << endl;
	}

return Punt2d(a,(-1)*(d));
}

}


/******************************************************************************** */
/*   bool Finestra::interseca(Finestra * fin, double &x, double &d, double &errors, double e, bool veina)                       */
/*Metode que retorna si dues finestres intersequen o no. També retorna el punt on intersequen i la distància del punt equidistant a una source */
/*********************************************************************************/

bool Finestra::interseca(Finestra * fin, double &x, double &d, double &errors, double e, bool veina)
{
bool interseca;
interseca=false;
double S,h,a,b,c,x1,x2;
Punt2d s,r;
double s1,s2,r1,r2,b0_,b1_;

if (veina)
{ 
	b0_=e-fin->getb1(); 
	b1_=e-fin->getb0();
}
else
{	
	b0_=fin->getb0();
	b1_=fin->getb1();
} 

double b0maxim=max(b0,b0_) + 1e-4;
double b1minim=min(b1,b1_) - 1e-4;

if (((fin->getb1()-fin->getb0())<1e-4) || ((this->getb1()-this->getb0())<1e-4))
{
	interseca=false;
}
else
{
	if ((b0maxim<b1minim) && (*this!=*fin))
	{
		s=this->trobarSource();
		r=fin->trobarSource(e,veina);
		s1=s.x();
		s2=s.y();
		r1=r.x();
		r2=r.y();
		
		cout << "Source Finestra f1 = " << s1 << " , " << s2 << endl;
		cout << "Source Finestra f2 = " << r1 << " , " << r2 << endl;
		
		//Manera Article Ax2+Bx+C=0 
		double A,B,C;
		double alfa,beta,gamma;
		alfa=r1-s1;
		beta=fin->getsigma()-this->getsigma();
		gamma=fabs((s1*s1)+(s2*s2))-fabs((r1*r1)+(r2*r2))-(beta*beta);
		A=(alfa*alfa)-(beta*beta);
		B=(gamma*alfa)+(2*r1*(beta*beta));
		C=0.25*(gamma*gamma)-(fabs((r1*r1)+(r2*r2))*(beta*beta));
	
		if (fabs(A)<1e-4) 
		{ 
			if (fabs(B)<1e-2)
			{
				cout << "A==0 i B==0 " << endl;
				x=-1;
				/*cout << "A=(alfa*alfa)-(beta*beta)" << endl ;
				cout << "alfa*alfa = " << alfa*alfa << endl;
				cout << "beta*beta = " << beta*beta << endl;*/
			}
			else
			{
				//Mirar que x>=0-TOL i x<=e+TOL. Si passa això, retornar x=-c/b. Altrament retornar x=-1
				x=-C/B; 
				cout << "A==0" << endl;
				cout << "B= " << B << ", C= " << C << endl;
				/*cout << "A=(alfa*alfa)-(beta*beta)" << endl ;
				cout << "alfa*alfa = " << alfa*alfa << endl;
				cout << "beta*beta = " << beta*beta << endl;*/
				if ((x>=0-1e-4) && (x<=e+1e-4))
				{		
					d=sqrt(((s1-x)*(s1-x))+(s2*s2));
					interseca=true;
				}
				else
				{
					x=-1;
					interseca=true;
				}
			}	
		}
		else //A!=0
		{
			//x1=-B+sqrt(B^2-4AC)))/(2A)
			//x2=-B-sqrt(B^2-4AC)))/(2A)
			double bb=B*B;
			double ac=4*A*C;
			double Z=B*B-(4*A*C);
		
			if (Z>0+1e-4 || Z>0-1e-4)
			{
				x1=(-B+sqrt(fabs(Z)))/(2*A);
				x2=(-B-sqrt(fabs(Z)))/(2*A);
				//cout << "VALORS INTERSECA x1 = " << x1 << ", x2 = " << x2 << ", A = " << A << ", B = " << B << ", C = " << C << ", b0maxim = " << b0maxim << ", b1minim = " << b1minim << ", Z = " << Z << ", BB = " << bb << ", 4AC = " << ac << endl;	
				
				if ((x1>b0maxim-2.*TOLERANCE) && (x1<b1minim+2.*TOLERANCE))
				{
					x=x1;
					/*cout << "----------------- Agafo x=-B+sqrt(B^2-4AC)))/(2A) " << " (A,B,C) = ( " << A << " , " << B << " , " << C << " ) " << endl;*/
					interseca=true;
					d=sqrt(((s1-x)*(s1-x))+(s2*s2));
				}		
				else if ((x2>b0maxim-2.*TOLERANCE) && (x2<b1minim+2.*TOLERANCE))	
				{ 
					x=x2;
					/*cout << "----------------- Agafo x=-B-sqrt(B^2-4AC)))/(2A) " << " (A,B,C) = ( " << A << " , " << B << " , " << C << " ) " << endl;*/
					interseca=true; 
					d=sqrt(((s1-x)*(s1-x))+(s2*s2));
				}
				else
				{
					x=-1;
					//cout << "ENTREM A L'ELSE ********************************************************** " << endl;
					interseca=true;
				}	
			}
			else
			{
				cout << "ERROR : DISCRIMINANT NEGATIU!!!!!!!!!!!!" << endl;
				x=-1;
				interseca=true;
			}
		}
		
	}
	
	else  { interseca=false; }
}

return interseca;
}


/******************************************************************************** */
/*   bool Finestra::no_elimina(Finestra * fn, double x, double d, double errors, bool veins=false, double e=0)      */
/*Metode que retorna cert si dues finestres no s'eliminen */
/*********************************************************************************/
bool Finestra::no_elimina(Finestra * fn, double x, double d, double errors, bool veins, double e)
{
//cout << "metode no elimina. ara cridarem al actualitzarFinestres " << endl;
char tecla;
cout << "Finestra f1 abans actualitzar : ( " << this->getb0() << " , " << this->getb1() << " , " << this->getd0() << " , " << this->getd1() << " , " << this->getsigma() << " ) " << endl;
cout << "Finestra f2 abans actualitzar : ( " << fn->getb0() << " , " << fn->getb1() << " , " << fn->getd0() << " , " << fn->getd1() << " , " << fn->getsigma() <<  " ) " << endl;
//cout << "Distancia de f1 a x = " << this->distancia(x) << endl;
//cout << "Distancia de f2 a x = " << fn->distancia(x,e,veins) << endl;
actualitzarFinestres(fn,x,d,errors,veins,e); 
cout << "Finestra f1 despres actualitzar : ( " << this->getb0() << " , " << this->getb1() << " , " << this->getd0() << " , " << this->getd1() << " , " << this->getsigma() << " ) " << endl;
cout << "Finestra f2 despres actualitzar : ( " << fn->getb0() << " , " << fn->getb1() << " , " << fn->getd0() << " , " << fn->getd1() << " , " << fn->getsigma() << " ) " << endl;

//cout << "Nova Distancia de f1 a x = " << this->distancia(x) << endl;
//cout << "Nova Distancia de f2 a x = " << fn->distancia(x,e,veins) << endl;
return fn->no_eliminada();
}


/******************************************************************************** */
/*   bool Finestra::no_eliminada()   */
/*Metode que retorna cert si una finestra no és eliminada */
/*********************************************************************************/

bool Finestra::no_eliminada()
{
return (fabs(b0-b1)>1e-5);
}


/******************************************************************************** */
/*   bool Finestra::actualitzarFinestres(Finestra * f, double x, double d, double errors, bool veins, double e=0)   */
/*Metode que actualitza les dues finestres (la this i la f) */
/*********************************************************************************/

void Finestra::actualitzarFinestres(Finestra * f, double x, double d, double errors, bool veins, double e)
{
double valor,distanciaf1x,distanciaf2x;
errors=0;
	if(x>-1*1e-7)
	{
		distanciaf1x=this->distancia(x);
		distanciaf2x=f->distancia(x,e,veins);
		cout << "Distancia de f1 a x = " << distanciaf1x << endl;
		cout << "Distancia de f2 a x = " << distanciaf2x << endl;
		
		if(!veins)
		{
			if(f->getb0()-1e-4<=this->getb0() && (f->getb1()-1e-4<=this->getb1()))
			{ 
				cout << "NO VEINS: IF " << endl;
				f->setb1d1(x,e,veins);
				this->b0=x;
				this->setd0(d);
			}
			else if (f->getb0()+1e-4>=this->getb0() && (f->getb1()+1e-4>this->getb1()))
			{
				cout << "NO VEINS: ELSE IF " << endl;
				f->setb0d0(x,e,veins);
				this->b1=x;
				this->setd1(d);
			}
			else //Casos que no hauria d'entrar
			{
				cout << "NO VEINS: UI UI UI NO HI HAURIA D'ENTRAR" << endl; 
				f->setb1d1(x,e,veins);
				this->b0=x;
				this->setd0(d);
				errors=errors+1;
			}
			
		}
		else //Cas que son veins
		{	
			if(e-f->getb1()-1e-4<=this->getb0() && (e-f->getb0()-1e-4<=this->getb1()))
			{
				cout << "VEINS: IF " << endl;
				f->setb0d0(x,e,veins);
				this->b0=x;
				this->setd0(d);
			}
			else if (e-f->getb1()+1e-4>=this->getb0() && (e-f->getb0()+1e-4>=this->getb1()))
			{			
				cout << "VEINS: ELSE IF " << endl;
				f->setb1d1(x,e,veins);
				this->b1=x;
				this->setd1(d);
			}	
			else //Casos que no hi hauria d'entrar
			{
				cout << "VEINS: UI UI UI NO HI HAURIA D'ENTRAR " << endl;
				f->setb0d0(x,e,veins);
				this->b0=x;
				this->setd0(d);
				errors=errors+1;
			}
		}
	}
	else // no hi ha punt equidistant en la interseccio dels intervals [b0,b1],[fin.b0,fin.b1]
	{
		if(!veins)
		{
			cout << "NO VEINS i x és -1" << endl;
			if(b0<f->getb0() && b1>f->getb1())
			{
				cout << "Cas 1" << endl;
				f->setb1d1(f->getb0(),e,veins);
				cout << "actualitzo el b1 de la f2" << endl;
				errors=errors+1;
					if(distancia(f->getb0())>f->getd0()) cout<<"UI ui ui!!!!!!!!!!!!!!!!!!"<<endl;
			}
			else if(f->getb0()<b0 && f->getb1()>b1)
			{
				cout << "Cas 2" << endl;
				this->setb0d0(this->getb1(),e,veins);
				cout << "actualitzo el b0 de la f1" << endl;
				errors=errors+1;
					if(f->distancia(b0)>d0) cout<<"UI ui ui (2) !!!!!!!!!!!!!!!"<<endl;
			}
			else if (this->getb0()-1e-4<=f->getb0() && this->getb1()-1e-4<=f->getb1())
			{
				cout << "Cas 3" << endl;
				distanciaf1x=this->distancia(this->getb1());
				distanciaf2x=f->distancia(this->getb1());
				cout << "Distancia de f1 a x = " << distanciaf1x << endl;
				cout << "Distancia de f2 a x = " << distanciaf2x << endl;
				
				if (distanciaf1x<=distanciaf2x)
				{
					f->setb0d0(this->getb1(),e,veins);
					cout << "Distancia f1 mes petita que distancia f2" << endl;
				}
				else
				{
					this->setb1d1(f->getb0(),e,veins);
					cout << "Distancia f2 mes petita que distancia f1" << endl;
				}
			}
			else if (f->getb0()-1e-4<=this->getb0() && f->getb1()-1e-4<=this->getb1())
			{
				cout << "Cas 4" << endl;
				distanciaf1x=this->distancia(f->getb1());
				distanciaf2x=f->distancia(f->getb1());
				cout << "Distancia de f1 a x = " << distanciaf1x << endl;
				cout << "Distancia de f2 a x = " << distanciaf2x << endl;
			
				if (distanciaf1x<=distanciaf2x)
				{
					f->setb1d1(this->getb0(),e,veins);
					cout << "Distancia f1 mes petita que distancia f2" << endl;
				}
				else
				{
					this->setb0d0(f->getb1(),e,veins);
					cout << "Distancia f2 mes petita que distancia f1" << endl;
				}
			}
		}
		else
		{
			cout << "VEINS i x és -1" << endl;
			if(b0<e-f->getb1() && b1>e-f->getb0())
			{
				cout << "Cas 1" << endl;
				f->setb1d1(f->getb0(),e,false);
				cout << "actualitzo el b1 de la f2" << endl;
					if(distancia(e-f->getb1())>f->getd1()) cout << "UI ui ui (v) !!!!!!!!!!!!!!!!!!" << endl;
			}
			else if(e-f->getb1()<b0 && e-f->getb0()>b1)
			{
				cout << "Cas 2" << endl;
				this->setb0d0(this->getb1(),e,false);
				cout << "actualitzo el b0 de la f1" << endl;
					if(f->distancia(e-b0)>d0) cout << "UI ui ui (2v) !!!!!!!!!!!!!!!" << endl;
			}
			else if (this->getb0()-1e-4<=e-f->getb1() && this->getb1()-1e-4<=e-f->getb0())
			{
				cout << "Cas 3" << endl;
				distanciaf1x=this->distancia(this->getb1());
				distanciaf2x=f->distancia(e-this->getb1());
				cout << "Distancia de f1 a x = " << distanciaf1x << endl;
				cout << "Distancia de f2 a x = " << distanciaf2x << endl;
			
				if (distanciaf1x<=distanciaf2x)
				{
					f->setb1d1(this->getb1(),e,veins);
					cout << "Distancia f1 mes petita que distancia f2" << endl;
				}
				else
				{
					this->setb1d1(f->getb1(),e,veins);
					cout << "Distancia f2 mes petita que distancia f1" << endl;
				}
			}
			else if (e-f->getb1()-1e-4<=this->getb0() && e-f->getb0()-1e-4<=this->getb1())
			{
				cout << "Cas 4" << endl;
				distanciaf1x=this->distancia(e-f->getb0());
				distanciaf2x=f->distancia(f->getb0());
				cout << "Distancia de f1 a x = " << distanciaf1x << endl;
				cout << "Distancia de f2 a x = " << distanciaf2x << endl;
			
				if (distanciaf1x<=distanciaf2x)
				{
					f->setb0d0(this->getb0(),e,veins);
					cout << "Distancia f1 mes petita que distancia f2" << endl;
				}
				else
				{
					this->setb0d0(f->getb0(),e,veins);
					cout << "Distancia f2 mes petita que distancia f1" << endl;
				}
			}
		}
	}
}


double Finestra::distancia(double x, double e, bool veins)
{
	Punt2d s;
	if (!veins)
	{
		s=trobarSource();
	}
	else
	{
		s=trobarSource(e,veins);
	}
	//cout << "METODE DISTANCIA SOURCE TROBAT = " << s.x() << " , " << s.y() << endl;
	return sqrt((s.x()-x)*(s.x()-x)+s.y()*s.y())+this->getsigma();
}


void Finestra::setb0d0(double x, double e, bool veins)
{
	if (!veins)
	{
		this->d0=this->distancia(x)-this->getsigma();
		this->b0=x;
	}
	else
	{
		this->d0=this->distancia(e-x)-this->getsigma();
		this->b0=e-x;
	}
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
}


void Finestra::setb1d1(double x, double e, bool veins)
{
	if (!veins)
	{
		this->d1=this->distancia(x)-this->getsigma();
		this->b1=x;
	}
	else
	{
		this->d1=this->distancia(e-x)-this->getsigma();
		this->b1=e-x;
	}
	this->pes=min(this->getd0(),this->getd1())+this->getsigma();
}


double Finestra::distanciaPuntProper(Punt p, double &x)
{
Punt pproper;
double valor;
double distancia;

Punt p1(this->getb0(),0.0,0.0);
Punt p2(this->getb1(),0.0,0.0);

Segment sf(p1,p2);
cout << "Mida segment = " << sf.midaSegment() << endl;
pproper=sf.calculaProperRecta2D(p);
distancia=pproper.distancia2D(p);
Punt3d paux(pproper.x(),pproper.y(),0.0);
x=(sf.coord_p_on_e(paux)*sf.midaSegment())+this->getb0();
valor=this->distancia(x);
return (valor+distancia);
}


Punt3d Finestra::trobarPunt3d(double x)
{
Punt3d p;
double e;

Segment s(cara->vertex((j+1)%3)->point(),cara->vertex((j+2)%3)->point());
e=s.midaSegment();
p=s.e_t(x/e);

return p;
}


void Finestra::pintarFinestres(Delaunay *dt)
{
pCara cara_veina;
int jv;
double e;
list<Finestra>::iterator fj1,fvj1,fj2,fvj2;
Finestra f;

//cout << "\nEntro a pintarFinestres" << endl;

cara_veina=cara->neighbor(j);
jv=cara_veina->index(cara);
//cout << "jveina = " << jv << endl;

if(!dt->is_infinite(cara_veina))
{
	fj1=cara_veina->info().LlistaFinestres[(jv+1)%3].begin();
	fvj1=cara_veina->info().LlistaFinestres[(jv+1)%3].end();
	fj2=cara_veina->info().LlistaFinestres[(jv+2)%3].begin();
	fvj2=cara_veina->info().LlistaFinestres[(jv+2)%3].end();
	
	while (fj1!=fvj1)
	{
		//cout << "Entro a j1" << endl;
		Segment s(cara_veina->vertex((jv)%3)->point(),cara_veina->vertex((jv+2)%3)->point());
		e=s.midaSegment();
		f=*fj1;
		//f=cara_veina->info().LlistaFinestres[(jv+1)%3].front();
		cout << "\n" ;
		f.escriure();
		f.pintar(s,e);
		fj1++;
		//cout << "Surto de j1" << endl;
	}
	
	while (fj2!=fvj2)
	{
		//cout << "Entro a j2" << endl;
		Segment s(cara_veina->vertex((jv)%3)->point(),cara_veina->vertex((jv+1)%3)->point());
		e=s.midaSegment();
		//f=cara_veina->info().LlistaFinestres[(jv+2)%3].front();
		f=*fj2;
		f.escriure();
		f.pintar(s,e);
		fj2++;
		//cout << "Surto de j2" << endl;
	}
}

cout << "Surto de pintarFinestres\n" << endl;
}


void Finestra::pintar(const Segment &s, double e)
{
//cout << "b0 = " << this->getb0() << " , b1 = " << this->getb1() << " , e = " << e << endl;
Punt3d pb0,pb1;
pb0=s.e_t(this->getb0()/e);
pb1=s.e_t(this->getb1()/e);

//Pintem les noves Finestres de color vermell
//glColor3f(1.0,0.0,0.0);
/*if(this->getPinta()==TRUE)
{
	glColor3f(1.0,1.0,0.33);
}*/
cout << "Finestra Nova = [" << pb0.x() << ", " << pb0.y() << "] a [" << pb1.x() << ", " << pb1.y() << "]" << endl; 
glBegin(GL_LINES);
	glVertex3f(pb0.x(),pb0.y(),0.0);
	glVertex3f(pb1.x(),pb1.y(),0.0);
glEnd();
cout << "He dibuixat la finestra nova de color vermell" << endl;
}


void Finestra::pintarEvent()
{
this->escriure();

double e;

Segment s(cara->vertex((j+1)%3)->point(),cara->vertex((j+2)%3)->point());
e=s.midaSegment();
this->pintar(s,e);
cout << "He dibuixat l'event finestra" << endl;
}


void Finestra::pintarEvent3d()
{
double e;

Segment s(cara->vertex((j+1)%3)->point(),cara->vertex((j+2)%3)->point());
e=s.midaSegment();
this->pintar3d(s,e);
cout << "He dibuixat l'event finestra" << endl;
}


void Finestra::pintar3d(const Segment &s, double e)
{
//cout << "b0 = " << this->getb0() << " , b1 = " << this->getb1() << " , e = " << e << endl;
Punt3d pb0,pb1;
pb0=s.e_t(this->getb0()/e);
pb1=s.e_t(this->getb1()/e);

//Pintem les noves Finestres de color vermell
glColor3f(1.0,0.0,0.0);
/*if(this->getPinta()==TRUE)
{
	glColor3f(1.0,1.0,0.33);
}*/
cout << "Finestra Nova = [" << pb0.x() << ", " << pb0.y() << "] a [" << pb1.x() << ", " << pb1.y() << "]" << endl; 
glBegin(GL_LINES);
	glVertex3f(pb0.x(),pb0.y(),pb0.z());
	glVertex3f(pb1.x(),pb1.y(),pb1.z());
glEnd();
cout << "He dibuixat la finestra nova de color vermell" << endl;
}


void Finestra::setPinta(bool pintar)
{
this->pinta=pintar;
}


bool Finestra::getPinta()
{
return pinta;
}


//Redefinició d'operadors

/*bool Finestra::operator<(Finestra const &fin)const
{
	bool menor=false;
	if ( this->getb0() < fin.getb0() )
	{
	menor=true;
	}
	
	return (menor);
}*/

bool Finestra::operator==(Finestra fin)const
{
	bool iguals=false;
	/*if(fabs(b0-fin.getb0())<=TOLERANCE && fabs(b1-fin.getb1())<=TOLERANCE && fabs(d0-fin.getd0())<=TOLERANCE && fabs(d1-fin.getd1())<=TOLERANCE && fabs(sigma-fin.getsigma())<=TOLERANCE && fabs(pes-fin.getpes())<=TOLERANCE && this->cara==fin.getcara() && this->j==fin.getj() && this->index==fin.getindex() && tipus_event==fin.tipusEvent())
	{
	iguals=true;
	}*/
	if(fabs(b0-fin.getb0())<=1e-3 && fabs(b1-fin.getb1())<=1e-3 && fabs(d0-fin.getd0())<=1e-3 && fabs(d1-fin.getd1())<=1e-3 && fabs(sigma-fin.getsigma())<=1e-3 && fabs(pes-fin.getpes())<=1e-3 && this->cara==fin.getcara() && this->j==fin.getj() && this->index==fin.getindex() && tipus_event==fin.tipusEvent())
	{
	iguals=true;
	}
	
	return (iguals);
}

bool Finestra::operator!=(Finestra fin)const
{
	bool diferents=true;
	if(*this==fin)
	{
		diferents=false;
	}
	return(diferents);
}

void Finestra::escriure()const
{
//cout<<"F "<<pes<<" (b0, b1, d0, d1, sigma, - , j, index)=("<<b0<<", "<<b1<<", "<<d0<<", "<<d1<<", "<<sigma<<", "<<j<<", "<<index<<")"<<endl;
cout << "F "<< pes << "  (" << b0 << ", " << b1 << ", " << d0 << ", " << d1 << ", " << sigma << ", " << j << ", " << index << ")" << endl;
}

//Mètode que ens propaga una Finestra
list<Event*> Finestra::propagar(Delaunay *dt)
{
	list<Event*> E;
	pCara cara, cara_veina;
	double b0,b1,d0,d1,sigma,l;
	int index, jfinestra, indexpseudo;
	Punt2d s,t; //s serà la font (source), t la imatge del vertex vei al pla.
 	Punt3d aux;
	bool veins=false;
		
	//Obtenim els paràmetres de la Finestra f
	this->getParametres(cara,b0,b1,d0,d1,sigma,index,jfinestra);
	this->escriure();
	//Trobem la font s 
	s=this->trobarSource(veins); //Calculem la font s (source)
	cout << "Source x = " << s.x() << " y = " << s.y() << endl;
	
	//Fer l'unfolding de la pCara de la finestra, trobar la cara adjacent i llavors fer l'unfolding del vèrtex oposat
	cara_veina=cara->neighbor(jfinestra);
			
	if (this->getb0()<0+1e-7)
	{
		if((cara->vertex(jfinestra)->info().distancia==-1) ||   (cara->vertex(jfinestra)->info().distancia>this->getd1()+this->getsigma()))
		{
			cara->vertex(jfinestra)->info().distancia=this->getd1()+this->getsigma();
			cara->vertex(jfinestra)->info().index=this->getindex();
		}
	}
	
	if (!dt->is_infinite(cara_veina))
	{
		//Per tenir l, 1agafar els 2 punts i calcular la longitud
		Punt3d pvei(cara_veina->vertex(cara_veina->index(cara))->point());
		Punt p1(cara->vertex((jfinestra+1)%3)->point());
		Punt p2(cara->vertex((jfinestra+2)%3)->point());
		l=p1.distancia(p2.getPunt());
		
		aux=cara->info().T[jfinestra](cara_veina->vertex(cara_veina->index(cara))->point());
		t=Punt2d(aux.x(),aux.y());
 		cout<<"T"; Punt(aux).escriure();
		
		double dist1, dist2;
		dist1=p1.distancia(pvei); 
		dist2=p2.distancia(pvei);
		//double y,x; 
		//x=((dist1*dist1)-(dist2*dist2)+(l*l))/(2*l); 
		//y=-1*sqrt((dist1*dist1)-(x*x));t=Punt2d(x,y);cout <<" T =("<<t.x()<<", "<< t.y()<<")";
		
		//Interseccions de finestres
		double P[4];
		double U[4];
		double V[4];
		double e[2];
		
		P[0]=0; P[1]=l; P[2]=b0; P[3]=b1; //Inicialització vector P
		U[0]=t.x()/dist1; U[1]=(t.x()-l)/dist2; U[2]=s.x()-b0; U[3]=s.x()-b1; //Inicialització vector U
		V[0]=t.y()/dist1; V[1]=t.y()/dist2; V[2]=s.y(); V[3]=s.y(); //Inicialització vector V
		e[0]=dist1; e[1]=dist2; //Inicialització vector e
		
		cout<<"p (" << P[0] << "," << P[1] << "," << P[2] << "," << P[3] << ")  u=(" << U[0] << "," << U[1] << "," << U[2] << "," << U[3] << ")  v=(" << V[0] << "," << V[1] << "," << V[2] << "," << V[3] << ")  e=(" << e[0] << "," << e[1] << ")" << endl;
		
		int c1,c2,i,j;
		double ss,tt,t1,t2;
		i=0; j=2; t1=-1; t2=-1;
			
		while ((i<2) && (t2<0))
		{
			if(fabs(V[i])<TOLERANCE) 
			{
				ss=0;
				if(fabs(U[i])<TOLERANCE) cout<<"vi=0, ui=0 ........ UPS!!";
				else{ cout<<"vi=0";  tt=(P[j]-P[i])/U[i]; }
			}
			else 
			{
				double den=U[i]*V[j]/V[i]-U[j];
				if(fabs(den)<TOLERANCE) 
				{
					if(fabs((P[j]-P[i]))<TOLERANCE)
					{
						cout << " Numerador i Denominador = 0 ....... " << endl;
						if (i==0) 
						{
						tt=0;
						}
						else 
						{
						tt=dist2;
						}
					}
					else
					{
					cout<<" Numerador !=0 i dennominador = 0  ........ UPS!!" << endl;
					}
				}
				else{ ss=(P[j]-P[i])/den; tt=ss*V[j]/V[i];}
			}
			cout << " i = " << i << " j = " << j << " ss = " << ss << " tt = " << tt /*<<" e[0] = " << e[0] << " e[1] = " << e[1] */<< endl;
			
			if ((tt>-1*TOLERANCE) && (tt<=e[i]+TOLERANCE))
			{
				if (t1==-1){t1=tt;c1=i;}
				else{t2=tt;c2=i;}
				j++;
			}
			else i++;
		}
		
		cout << " c1 = " << c1 << " c2 = " << c2 << " j = " << j << endl;
		
		Punt3d s3(s.x(),s.y(),0.);
		Punt2d porigen(0,0);
		//Finestra * f2;

		cout << "PSEUDOVERTEX : ( " << s.x() << " , " << s.y() << " ) " << endl;
		char tecla;
		//Creem les finestres
		if ((c1==0) && (c2==0) && (c1==c2))
		{
			Punt pini(porigen+(t1/dist1)*(t-porigen)), pfi(porigen+(t2/dist1)*(t-porigen));
			Finestra * f2=new Finestra (t1,t2,pini.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+2)%3)),index);
			cout << "n ";
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+2)%3))].push_front(*f2); 
			f2->escriure();
			E.push_back(f2);
		}
		else if ((c1==1) && (c2==1) && (c1==c2))
		{
			Punt2d pa(l,0);
			Punt pini(pa+(t1/dist2)*(t-pa)), pfi(pa+(t2/dist2)*(t-pa));
			Finestra * f2=new Finestra(dist2-t1,dist2- t2,pini.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+1)%3)),index);
			cout << "n ";
			f2->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+1)%3))].push_front(*f2);
			E.push_back(f2);
		}
		else if(c1!=0 || c2!=1) cout<<"CAS NO CONTEMPLAT"<<endl;
		else // c1=0 i c2=1
		{
			Punt2d pa(l,0);
			Punt pini(porigen+(t1/dist1)*(t-porigen)), pmig(t), pfi(pa+(t2/dist2)*(t-pa));
			
			// una finestra com la del c1=c2=0 q va des de pini a pvei
			Finestra * f2=new Finestra (t1,dist1,pini.distancia(s3),pmig.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+2)%3)),index);
			cout << "n ";
			f2->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+2)%3))].push_front(*f2);
			E.push_back(f2);
			
			int index_vei=cara_veina->index(cara);
			if((cara_veina->vertex(index_vei)->info().distancia==-1) ||  (cara_veina->vertex(index_vei)->info().distancia>f2->getd1()+f2->getsigma()))
			{
				cara_veina->vertex(index_vei)->info().distancia=f2->getd1()+f2->getsigma();
				cara_veina->vertex(index_vei)->info().index=f2->getindex();
				if(vertexHiperbolic(cara_veina,cara_veina->vertex(index_vei),dt))
				{
					E.push_back(new PseudoSource(f2->getd1()+sigma, index, cara_veina->vertex(index_vei), cara_veina));
				}	
			}
			
			Finestra * f3=new Finestra (0.,dist2- t2,pmig.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+1)%3)),index);
			cout << "n ";
			f3->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+1)%3))].push_front(*f3);
			E.push_back(f3);
		}
	}
	else
	{ cout << "Cara Veina Infinita No Propaguem" << endl; }
	
	E.sort(Event::ordena); //Ordenem la llista d'events segons el pes.
	//E.sort(); //Ordenem la llista d'events a retornar de menys a més pes
	return E;
}


bool Finestra::vertexHiperbolic(const pCara &cara, const pVertex &vertex, Delaunay * dt)
{
double angle=0;

CirculadorCares cir=(dt->incident_faces(vertex,cara));
				
		do {
			if(dt->is_infinite(cir))
			{
				cout << "Es un vertex frontera (false)" << endl;
				return false;
			}
			else
			{
				int m=0;
				while (cir->vertex(m)!=vertex)
				{ m++ ; }
								
				Vector3D y((Punt(cir->vertex(m)->point())),(Punt(cir->vertex((m+1)%3)->point())));
				Vector3D x((Punt(cir->vertex(m)->point())),(Punt(cir->vertex((m+2)%3)->point())));
				angle=angle + acos((y*x)/(sqrt((y*y)*(x*x))));
			
				if (angle>=2*PI) { return true; }
				
			}
			cir++;
		
		} while (cir!=cara);
	
return false;
}

