#ifndef FINESTRA_H
#define FINESTRA_H

#include <iostream.h>
#include "punt.h"
#include "segment.h"
#include "objecte.h"
#include "event.h"
#include "geometriabasica.h"
#include <qcolor.h>
#include <math.h>
#include <vector>
#include "linia.h"
#include <list.h>

class PseudoSource;

/***********************************************************************************************/
/*                                        class Finestra                                       */
/*   Classe Finestra que conté tots els mètodes necessaris per poder treballar amb finestres   */
/***********************************************************************************************/

class Finestra : public Event
{
	private:
		/*Atributs de classe que corresponen a la tupla {b0,b1,d0,d1,sigma,tau,index,pes] amb què haurem de treballar*/
		
		double b0; //Representa l'extrem de la banda esquerra
		double b1; //Representa l'extrem de la banda dreta Van de [0,||e||]
		double d0; //Distància al pseudo-source des de b0
		double d1; //Distància al pseudo-source des de b1
		double sigma; //Distància al pseudo-source original
		pCara cara; //Punter a la cara (una pCara)
		int j; //Index vertex oposat
		int index; //Index de la posicio del vector on hi ha el punt (VectorVisio de la classe Visio)
		
		bool pinta;
		//double pes; //Pes que tindrà la finestra, servirà per ordenar-les a la cua de prioritat (list). Pes = sigma + d (altura)
				
	public:
		
		Finestra(); //Constructor per defecte que inicialitza els atributs a 0
		~Finestra(); //Destructor
		Finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_);
		
		Finestra(const Finestra &fin); //Constructor de copia
		void swap(Finestra &finx, Finestra &finy); //Swap	
	
		/*Metodes que retornen els atributs de finestra*/
		double getb0() const;
 		double getb1() const;
 		double getd0() const;
		double getd1() const;
		double getsigma() const;
		pCara getcara() const;
		int getj() const;
		int getindex() const;
		void  getParametres(pCara &cara_, double &b0_, double &b1_, double & d0_,double &d1_, double &sigma_, int &index_, int &jfinestra_)const;
		//double getpes() const;

		/*Metodes que permeten modificar el valor dels atributs de forma individual o bé tots de cop*/
		void setb0(double b0_);
		void setb1(double b1_);
		void setd0(double d0_);
		void setd1(double d1_);
		void setsigma(double sigma_);
		void setcara(pCara cara_);
		void setj(int j_);
		void setindex(int index_);
		//void setpes(double pes);
		void modifica_finestra(double b0_, double b1_, double d0_, double d1_, double sigma_, pCara cara_, int j_, int index_);
		
		bool menor(Finestra * fin);  //Ens diu si una Finestra és menor que una altra segons l'atribut b0.
		Punt2d trobarSource(double e=0, bool veins=false); //Donada la finestra trobem el source s
		bool interseca(Finestra * fin, double &x, double &d, double &errors, double e, bool veina);
		bool no_elimina(Finestra * f, double x, double d, double errors, bool veins=false, double e=0);
		bool no_eliminada();
		void actualitzarFinestres(Finestra * f, double x, double d, double errors, bool veins=false, double e=0);
		void setb0d0(double x, double e=0, bool veins=false);
		void setb1d1(double x, double e=0, bool veins=false);
		double distanciaPuntProper(Punt p, double &x);
		Punt3d trobarPunt3d(double x);
		
		void pintarFinestres(Delaunay *dt);
		void pintar(const Segment &s, double e);
		void pintarEvent();
		void pintarEvent3d();
		void pintar3d(const Segment&s, double e);
		
		bool getPinta();
		void setPinta(bool pintar);
						
  		// Redefinició d'operadors
		bool operator==(Finestra fin)const;
		//bool operator<(Finestra const &fin)const;
		bool operator!=(Finestra fin)const;
		
		void escriure()const;
		double distancia(double x, double e=0, bool veins=false);
		
		list<Event*> propagar(Delaunay *dt);
		bool vertexHiperbolic(const pCara &cara, const pVertex &vertex, Delaunay *dt);

} ;

#endif
