#include "fitxerterreny.h"

/*********************************************************************************************/
/*   FitxerTerreny::FitxerTerreny()                                                          */
/*Constructor de la classe FitxerTerreny que tan sols inicialitza el valor de les coordenades*/
/*********************************************************************************************/

FitxerTerreny::FitxerTerreny()
{
   inicialitza();
}

/************************************************************************************/
/*   FitxerTerreny::~FitxerTerreny() {}                                             */
/*Destructor de la classe FitxerTerreny, que permet destruir objectes d'aquest tipus*/
/************************************************************************************/

FitxerTerreny::~FitxerTerreny() {}

/*************************************************************/
/*   void FitxerTerreny::inicialitza()                       */
/*Metode que inicialitza les coordenades al valor per defecte*/
/*************************************************************/

void FitxerTerreny::inicialitza()
{
  (info.punts).clear();
  info.xmin = 10000.0;
  info.ymin = 10000.0;
  info.zmin = 10000.0;
  info.xmax = 0.0;
  info.ymax = 0.0;
  info.zmax = 0.0;
}

/*****************************************************************/
/*   int FitxerTerreny::carrega( const QString &nom, Terreny *t )*/
/*Metode que permet carregar un fitxer amb el nom indicat        */
/*****************************************************************/

int FitxerTerreny::carrega( const QString &nom, Terreny *t )
{
   int estat;
   inicialitza();
   if( nom.endsWith(".tin") )
   {
      estat = carregaTin( nom );
      if( estat != CORRECTE ) return estat;            
   }
   else if( nom.endsWith(".t2d") )
   {
      estat = carregaT2d( nom );      
      if( estat != CORRECTE ) return estat;            
   }
   else return ERROR_FORMAT;

   guardarDades( t );
   return CORRECTE;
}

/*************************************************************************/
/*   int FitxerTerreny::desa( const QString &nom, Terreny *t )           */
/*Metode que permet guardar el terreny sobre un fitxer amb el nom indicat*/
/*************************************************************************/

int FitxerTerreny::desa( const QString &nom, Terreny *t )
{
   int estat;
   if( nom.endsWith(".tin") )
   {
      estat = desaTin( nom, t );
      return estat;
   }
   else if( nom.endsWith(".t2d") )
   {
      estat = desaT2d( nom, t );
      return estat;
   }
   else return ERROR_OBRIR;
}

/*********************************************************/
/*   int FitxerTerreny::carregaTin( const QString &nom ) */
/*Metode que permet carregar un fitxer amb extensio tin  */
/*********************************************************/

int FitxerTerreny::carregaTin( const QString &nom )
{   
   QString s;
   double px, py, pz;
   int nv = 0;

   f.setName( nom );   
   QTextStream ts( &f );

   if ( !f.open( IO_ReadOnly ) )        
	return ERROR_OBRIR;
  
   s = ts.readLine().simplifyWhiteSpace();   
   if( s.isNull() || !(s == "TIN") ) return ERROR_FORMAT;

   s = ts.readLine().simplifyWhiteSpace();
   if( s.isNull() || !(s == "BEGT") ) return ERROR_FORMAT;                
       
   s = ts.readLine().simplifyWhiteSpace();  //Llegir TNAME
   if( s.isNull() ) return ERROR_FORMAT;         

   if( !s.startsWith( "VERT" ) && s.startsWith( "TNAME" ) )  s = ts.readLine().simplifyWhiteSpace();
   if( !s.startsWith( "VERT" ) && s.startsWith( "TCOL" ))  s = ts.readLine().simplifyWhiteSpace();

   if( s.startsWith( "VERT" ) ) 
   {
       nv = extreureNumero( s );
       if( nv == -1 ) return ERROR_FORMAT;
       for(int i = 0; (i < nv) && (!f.atEnd()); i++)
       {
           s = ts.readLine().simplifyWhiteSpace();
           if( s.isNull() ) return ERROR_FORMAT;              

           treureFlag( s );
           if ( !extreureCoordenades( s, px, py, pz ) )
           {
             (info.punts).push_back( Punt3d( px, py, pz ) );

	     if ( px > info.xmax ) info.xmax = px;
             if ( px < info.xmin ) info.xmin = px;
             if ( py > info.ymax ) info.ymax = py;
             if ( py < info.ymin ) info.ymin = py;
             if ( pz > info.zmax ) info.zmax = pz;
             if ( pz < info.zmin ) info.zmin = pz;
           }
           else
           {
             f.close();
             return ERROR_FORMAT;
           }
       }
       if( f.atEnd() ) return ERROR_FORMAT;
   }
   else return ERROR_FORMAT;

   s = ts.readLine().simplifyWhiteSpace();
   if( s.isNull() ) return ERROR_FORMAT;   

   if( s.startsWith( "TRI" ) )
   {
      int n_tri = extreureNumero( s );
      if( n_tri == -1 ) return ERROR_FORMAT;
      for( int i = 0; i < n_tri && (!f.atEnd()); i++ )
      {
         s = ts.readLine().simplifyWhiteSpace();
      }
      if( f.atEnd() ) return ERROR_FORMAT;

      s = ts.readLine().simplifyWhiteSpace();  //Llegir TNAME
      if( s.isNull() ) return ERROR_FORMAT;         
   }

   if( !(s == "ENDT") ) return ERROR_FORMAT;       

   return CORRECTE;
}

/******************************************************************/
/*   int FitxerTerreny::desaTin( const QString &nom, Terreny *t ) */
/*Metode que permet guadar un fitxer amb extensio tin             */
/******************************************************************/

int FitxerTerreny::desaTin( const QString &nom, Terreny *t )
{
   f.setName( nom );   

   if ( !f.open( IO_WriteOnly ) )        
	return ERROR_OBRIR;

   QTextStream ts(&f);
   ts << "TIN" << "\n";
   ts << "BEGT" << "\n";
   ts << "TNAME " << nom << "\n";
   ts << "TCOL " << "170" << " " << "85" << " " << "0" << "\n";
   ts << "VERT " << t->numero_de_vertexs() << "\n"; 

   IteradorPunts fit = (t->triangulacio())->points_begin();
   for( ; fit != (t->triangulacio())->points_end(); ++fit)
   {
      ts << (*fit).x() << " " << (*fit).y() << " " << (*fit).z() << "\n";
   }	
   ts << "ENDT";
   f.close();

   return CORRECTE;
}

/*********************************************************/
/*   int FitxerTerreny::carregaT2d( const QString &nom ) */
/*Metode que permet carregar un fitxer amb extensio t2d  */
/*********************************************************/

int FitxerTerreny::carregaT2d( const QString &nom )
{
   QString s;
   double px, py, pz;

   f.setName( nom );   
   QTextStream ts( &f );

   if ( !f.open( IO_ReadOnly ) )        
	return ERROR_OBRIR;
	
   while ( !f.atEnd() )
   {
      s = ts.readLine().simplifyWhiteSpace();	   

      if ( !extreureCoordenades( s, px, py, pz ) )
      {
        (info.punts).push_back( Punt3d( px, py, pz ) );

	if ( px > info.xmax ) info.xmax = px;
        if ( px < info.xmin ) info.xmin = px;
        if ( py > info.ymax ) info.ymax = py;
        if ( py < info.ymin ) info.ymin = py;
        if ( pz > info.zmax ) info.zmax = pz;
        if ( pz < info.zmin ) info.zmin = pz;
      }
      else
      {
         f.close();
         return ERROR_FORMAT;
      }
   }
   f.close();

   return CORRECTE;
}

/******************************************************************/
/*   int FitxerTerreny::desaT2d( const QString &nom, Terreny *t ) */
/*Metode que permet guadar un fitxer amb extensio t2d             */
/******************************************************************/

int FitxerTerreny::desaT2d( const QString &nom, Terreny *t )
{
   f.setName( nom );   

   if ( !f.open( IO_WriteOnly ) )        
	return ERROR_OBRIR;
   
   QTextStream ts(&f);
   IteradorPunts fit = (t->triangulacio())->points_begin();
   for( ; fit != (t->triangulacio())->points_end(); ++fit)
   {
	 ts << (*fit).x() << " " << (*fit).y() << " " << (*fit).z() << "\n";
   }
   f.close();

   return CORRECTE;	   
}

/********************************************************************************************/
/*   int FitxerTerreny::extreureCoordenades( QString s, double &px, double &py, double &pz )*/
/*Metode que permet extreure les coordenades x, y i z d'una cadena de 3 valors doubles      */
/********************************************************************************************/

int FitxerTerreny::extreureCoordenades( QString s, double &px, double &py, double &pz )
{
    int idx1, idx2;  
    bool ok;

    int n = s.contains(" ");
    if( n != 2) return -1;

    idx1 = s.find(' ', 0);
    idx2 = s.find(' ', idx1 + 1);

    px = py = pz = 0.0;

    px = s.left(idx1).toDouble(&ok);
    if ( ok == FALSE || px < 0.0 ) return -1;	   	
	
    py = s.mid(idx1 + 1, idx2 - idx1 - 1).toDouble(&ok);
    if ( ok == FALSE || py < 0.0 )  return -1;	   

    pz = s.right(s.length() - idx2 - 1 ).toDouble(&ok);
    if ( ok == FALSE || pz < 0.0 ) return -1;	   

    return 0;
}

/******************************************************************/
/*   void FitxerTerreny::guardarDades( Terreny *t )               */
/*Metode que permet guadar les dades obtingudes sobre el terreny  */
/******************************************************************/

void FitxerTerreny::guardarDades( Terreny *t )
{
     for( int i = 0; i < (info.punts).size(); i++ ) //              t->insereix( info.punts[i] );        
     {
          t->inserirPunt( info.punts[i] );        
     }
     
     t->limits[0] = floor(info.xmin);
     t->limits[1] = floor(info.ymin);
     t->limits[2] = floor(info.zmin);
     t->limits[3] = ceil(info.xmax);
     t->limits[4] = ceil(info.ymax);
     t->limits[5] = ceil(info.zmax);
}

/**************************************************/
/*   void FitxerTerreny::treureFlag( QString &m ) */
/*Metode que permet treure un Flag                */
/**************************************************/

void FitxerTerreny::treureFlag( QString &m )
{
   int n = m.contains(" ");
   if( n == 3 )
   {
      m.truncate( m.length() - 2 );
   }
}

/************************************************************/
/*   int FitxerTerreny::extreureNumero( const QString &m )  */
/*Metode que permet obtenir el primer numero de la cadena m */
/************************************************************/

int FitxerTerreny::extreureNumero( const QString &m )
{
   int n; 
   int idx;
   int valor;
   bool ok;

   n = m.contains(" ");

   if( n == 1 )
   {
      idx = m.find(' ', 0);
      valor = m.right(m.length() - idx - 1 ).toInt(&ok);
      if ( ok == FALSE || valor < 0.0 ) return -1;
      else return valor; 	   
   }
   else return -1;
}
