#include "generadorterreny.h"
#include <cstdlib>
#include <ctime>
#include <qapplication.h>
/*
#define filterSize 25*sizeof(float)

static float identitat[25] = {
		0.0,0.0,0.0,0.0,0.0,
		0.0,0.0,0.0,0.0,0.0,
		0.0,0.0,1.0,0.0,0.0,
		0.0,0.0,0.0,0.0,0.0,
		0.0,0.0,0.0,0.0,0.0};

static float filtre[25] = {
		0.0,	0.0,	0.0,	0.0,	0.0,
		0.0,	0.0,	0.0,	0.0,	0.0,
		0.0,	0.0,	1.0,	0.0,	0.0,
		0.0,	0.0,	0.0,	0.0,	0.0,
		0.0,	0.0,	0.0,	0.0,	0.0};
*/

/************************************************************************************/
/*   GeneradorTerreny::GeneradorTerreny()                                           */
/*Constructor de la classe que permet inicialitzar les variables emprades pel calcul*/
/************************************************************************************/

GeneradorTerreny::GeneradorTerreny()
{
   altures = NULL;
   particle_mode = 0;
   iteracions = 0;
   x_max = y_max = 0;
   max_desp = 1.0; 
   min_desp = 0.1;
   srand( time(NULL) );
}

/********************************************************************************/
/*   GeneradorTerreny::~GeneradorTerreny()                                      */
/*Destructor de la classe que elimina les referencies del punter a float altures*/
/********************************************************************************/

GeneradorTerreny::~GeneradorTerreny()
{ 
   delete[] altures;
   altures = NULL;
}

/****************************************/
/*   float *GeneradorTerreny::getDades()*/
/*Metode que retorna el punter altures  */
/****************************************/

float *GeneradorTerreny::getDades()
{
   return altures;
}

/************************************************************************************************/
/*   void GeneradorTerreny::generaParticleDeposition( int iteracions, double xmax, double ymax )*/
/*Metode que utilitza un determinat metode pel calcul de les altures de forma aleatoria         */
/************************************************************************************************/

void GeneradorTerreny::generaParticleDeposition( int iteracions, double xmax, double ymax )
{
   x_max = (int)xmax + 1;
   y_max = (int)ymax + 1;
   //altures.resize( x_max * y_max, 0.0 );

   if( altures != NULL )
   {
	delete[] altures;
        altures = NULL;
   }   
   altures = new float[x_max*y_max];

   for(int i = 0; i < x_max*y_max; i++ )
   {
      altures[i] = 0;
   }

   iteracionsParticleDeposition( iteracions );
   suavitzar( 0.4 );
}

/*************************************************************/
/*   void GeneradorTerreny::suavitzar( float k )             */
/*Metode que suavitza les altures a partir d'un cert valor k */
/*************************************************************/

void GeneradorTerreny::suavitzar( float k ) 
{
   int i, j;

   for( i = 0; i < y_max; i++ )
   {
      for( j = 1; j < x_max; j++ )
      {
	altures[i * x_max + j] = altures[i * x_max + j] * (1-k) + altures[i * x_max + (j-1)] * k;
      }
   }
	
   for( i = 1; i < y_max; i++ )
   {
      for( j=0; j < x_max; j++ )
      {
	altures[i * x_max + j] = altures[i * x_max + j] * (1-k) + altures[(i-1) * x_max + j] * k;
      }
   }	
	

   for( i = 0; i < y_max; i++ )
   {
      for( j = x_max-1; j>-1; j-- )
      {
	 altures[i * x_max + j] = altures[i * x_max + j] * (1-k) + altures[i * x_max + (j+1)] * k;
      }
   }

   for( i = y_max-2; i < -1; i-- )
   {
      for( j = 0; j < x_max; j++ )
      {
	 altures[i * x_max + j] = altures[i * x_max + j] * (1-k) + altures[(i+1) * x_max + j] * k;
      }
   }
}

/*************************************************************/
/*   void GeneradorTerreny::setParticleMode( int mode )      */
/*Metode a partir del qual assignem el metode d'aleatorietat */
/*************************************************************/

void GeneradorTerreny::setParticleMode( int mode ) 
{
   particle_mode = mode;
}

/**************************************************/
/*   void GeneradorTerreny::deposit(int x, int z) */
/*Metode recursiu amb el qual es calculen altures */
/**************************************************/

void GeneradorTerreny::deposit(int x, int z) 
{
	int j,k,kk=0,jj=0,flag;

	flag = 0;
	for (k=-1;k<2;k++)
		for(j=-1;j<2;j++)
			if (k!=0 && j!=0 && x+k>-1 && x+k<x_max && z+j>-1 && z+j<y_max) 
				if (altures[(x+k) * y_max + (z+j)] < altures[x * y_max + z]) {
					flag = 1;
					kk = k;
					jj = j;
				}

	if (!flag)
		altures[x * y_max + z] += max_desp;
	else
		deposit(x+kk,z+jj);
}

/*
deposit( int x, int y ) 
{
qDebug("2");
   int j, k, kk = 0, jj = 0;
   bool flag;

   flag = false;
   iteracions++;
   qDebug("Iteracions = %d", iteracions);
   for( k = -1; k < 2; k++ )
   { 
      for( j = -1; j < 2; j++ )
      {
	 if( (k != 0) && (j != 0) && (x+k > -1) && (x+k < x_max) && (y+j > -1) && (y+j < y_max) ) 
         {
            qDebug("valors = %f - %f", altures[(x+k) * y_max + (y+j)], altures[x * y_max + y]);
	    if( altures[(x+k) * y_max + (y+j)] < altures[x * y_max + y] ) 
            {
		flag = true;
		kk = k;
		jj = j;
	    }
         }

	 if( !flag ) 
             altures[x * y_max + y] += max_desp;
	 else
	     deposit( x+kk, y+jj );
      }
   }
}
*/

/*********************************************************************/
/*   int GeneradorTerreny::iteracionsParticleDeposition( int numIt ) */
/*Metode que va realitzant les iteracions segons el numero escollit  */
/*********************************************************************/

int GeneradorTerreny::iteracionsParticleDeposition( int numIt ) 
{
	int x,y,i,dir;

        //if( altures.empty() ) return -1;
        if( altures == NULL ) return -1;

	x = rand() % x_max;
	y = rand() % y_max;

	for (i=0; i < numIt; i++) {

//		iterationsDone++;
		dir = rand() % 4;

		if (dir == 2) {
			x++;
			if (x >= x_max)
				x = 0;
		}
		else if (dir == 3){
			x--;
			if (x == -1)
				x = x_max-1;
		}
		
		else if (dir == 1) {
			y++;
			if (y >= y_max)
				y = 0;
		}
		else if (dir == 0){
			y--;
			if (y == -1)
				y = y_max - 1;
		}

		if (particle_mode == 0)
			deposit(x,y);
		else
			altures[x * y_max + y] += max_desp;
	}
	return 0;
}
/*
qDebug("3");
   int x, y, i, direccio;

   //if( altures.empty() ) return -1;

   x = rand() % x_max;
   y = rand() % y_max;

   for( i = 0; i < numIt; i++ ) 
   {
      direccio = rand() % 4;
      switch( direccio )
      {
         case 0:
	     y--;
	     if( y < 0 ) y = y_max - 1;
         break;
         case 1:
	     y++;
	     if( y >= y_max ) y = 0;
         break;
         case 2:
	     x++;
	     if( x >= x_max ) x = 0;
         break;
         case 3:
	     x--;
	     if( x < 0 ) x = x_max - 1;
         break;
         default:;
      }

      if ( particle_mode == 0 )
	   deposit( x, y );
      else
	   altures[x * y_max + y] += max_desp;
   }
   return 0;
}
*/
