#include "segment.h"
#include "regio.h"
#include "hiperbola.h"

/**************************************/
/*   Hiperbola::Hiperbola()           */
/*Constructor per defecte de la classe*/
/**************************************/

Hiperbola::Hiperbola()
{}

/******************************************************************************************************************************/
/*   Hiperbola::Hiperbola(const double &ca,const double &cb,const double &ch,const double &ck,const Punt2d &p,const Punt2d& q)*/
/*Constructor amb parametres que inicialitza cadascun dels atributs que permeten definir una hiperbola                        */
/******************************************************************************************************************************/

Hiperbola::Hiperbola(const double &ca,const double &cb,const double &ch,const double &ck,const Punt2d &p,const Punt2d &q)
{
	ctu_=ca;
	ct_=cb;
	cu_=ch;
	cc_=ck;
	source_=p;
	target_=q;
	if(in_range(-cu_/ctu_,source_.x(),target_.x(), false)){assimp_t_=1;}else assimp_t_=0;
	if(in_range(-ct_/ctu_,source_.y(),target_.y(), false))assimp_u_=1;else assimp_u_=0;
}

/***************************************************************/
/*   Hiperbola::Hiperbola(const Segment3D &s)                   */
/*Constructor que rep per parametres el segment que la defineix*/
/***************************************************************/



Hiperbola::Hiperbola(const Segment3D &s, Segment ss_meu, Segment sv_meu)
{

// 	cout<<" h "<<endl;
	Segment seg(s);
	vector<double> v(seg.skew_segment(ss_meu,source_,target_));
	
	ctu_=v[0];
	ct_=v[1];
	cu_=v[2];
	cc_=v[3];
// 	if(source_.x()>target_.x()||(source_.x()==target_.x()&&source_.y()>target_.y()))
// 	{
// 		Punt2d aux=source_;
// 		source_=target_;
// 		target_=source_;
// 	}
	
// 	source_=Punt2d(1,0);
// 	target_=Punt2d(0,1);
// 	source_=seg.getOrigen().skew_point( ss_meu.getSegment_cgal(), sv_meu.getSegment_cgal());
// 	target_=seg.getDesti().skew_point( ss_meu.getSegment_cgal(), sv_meu.getSegment_cgal());
	
}



/****************************************************************************************/
/*   Hiperbola::Hiperbola(double a, double b, double c, double d)                       */
/*Constructor de hiperbola a partir dels parametres de l'equacio, sense source ni target*/  
/****************************************************************************************/

Hiperbola::Hiperbola(double a, double b, double c, double d)
{
	ctu_=a;
	ct_=b;
	cu_=c;
	cc_=d;
}

/********************************************************************************************/
/*   Hiperbola::Hiperbola(const Punt &p, const Segment &s, const Segment &v)                */
/*Constructor que crea una hiperbola i nomes funciona per punts q no estiguin sobre s ni v, */
/*ni al'eix x=0 ni a la recta q uneix el altres 2 extrems de v i s                          */
/********************************************************************************************/

Hiperbola::Hiperbola(const Punt &p, const Segment &s, const Segment &v)
{
	double l=1.;
	Vector3D u=v.getVector();
	u.transforma_unitari();
	
	double lambda;
	Segment d(s.getDesti(),v.getDesti());
	Segment e(s.getOrigen(),v.getOrigen());
	if(s.pertanySegment(p,lambda))
	{ ctu_=0; ct_=0; cu_=1;cc_=-lambda;}
	else if(v.pertanySegment(p,lambda))
	{ ctu_=0; ct_=1; cu_=0;cc_=-lambda; } 
	else if(d.pertanySegment(p,lambda))
	{ ctu_=0; ct_=0; cu_=0; cc_=s.midaSegment(); }
	else if(e.pertanySegment(p,lambda))
	{ ctu_=0; ct_=0; cu_=0; cc_=0;} 
	else  if(fabs(p.y())>TOLERANCE)
	{
		ctu_=u.y()*l/p.y();
		ct_=-u.y()*p.x()/p.y()+u.x();
		cu_=l*(-1+v.getOrigen().y()/p.y());
		cc_=-v.getOrigen().y()*p.x()/p.y()+v.getOrigen().x();
	}
	else if(fabs(u.y())>TOLERANCE)
	{
		ctu_=0; ct_=0; cu_=1;cc_=v.getOrigen().y()/u.y();
	}
	else 
		cout<<"ERROR construccio hiperbola"<<endl;
}

/***************************************************/
/*   Hiperbola::Hiperbola(const Hiperbola &hyp)    */
/*Cosntructor de copia                             */  
/***************************************************/

Hiperbola::Hiperbola(const Hiperbola &hyp)
{
	*this=hyp;
}
/*************************************/
/*   Hiperbola::~Hiperbola()         */
/*Destructor de la classe hiperbola  */
/*************************************/

Hiperbola::~Hiperbola() {}

/******************************************************************************************************************/
/*   const int & Hiperbola::as_t() const                                                                          */
/*Metode que retorna cert si la hiperbola te assimptota en t                                              */
/******************************************************************************************************************/


void Hiperbola::escriure()const
{
cout<<ctu_<<"tu + "<<ct_<<" t + "<<cu_<<" u + "<<cc_<<"= 0    s=("<<source_.x()<<", "<<source_.y()<<"), ("<<target_.x()<<", "<<target_.y()<<")."<<endl;
}

bool  Hiperbola::as_t() const
{
	return in_t_range(-cu()/ctu()); 
}

/******************************************************************************************************************/
/*   const int & Hiperbola::as_u() const                                                                          */
/*Metode que retorna cert si la hiperbola te assimptota en u                                          */
/******************************************************************************************************************/

bool Hiperbola::as_u() const
{
	return in_u_range(-ct()/ctu()); 
}

/*****************************************************/
/*   const double & Hiperbola::ctu() const           */
/*Metode que retorna la constant ctu de la hiperbola */  
/*****************************************************/

const double & Hiperbola::ctu() const
{
	return ctu_;
}

/*****************************************************/
/*   const double & Hiperbola::ct() const            */
/*Metode que retorna la constant ct  de la hiperbola */  
/*****************************************************/

const double & Hiperbola::ct() const
{
	return ct_;
}

/*****************************************************/
/*   const double & Hiperbola::cu() const            */
/*Metode que retorna la constant cu de la hiperbola  */  
/*****************************************************/

const double & Hiperbola::cu() const
{
	return cu_;
}

/*****************************************************/
/*   const double & Hiperbola::cc() const            */
/*Metode que retorna la constant cc de la hiperbola  */  
/*****************************************************/ 

const double & Hiperbola::cc() const
{
	return cc_;
}

/*****************************************************/
/*   const Punt2d & Hiperbola::source() const        */
/*Metode que retorna el punt d'inici de la hiperbola */  
/*****************************************************/

const Punt2d & Hiperbola::source()const
{
	return source_;
}

/*****************************************************/
/*   const Punt2d & Hiperbola::target() const        */
/*Metode que retorna el punt de fi de la hiperbola   */  
/*****************************************************/

const Punt2d & Hiperbola::target() const
{
	return target_;
}

/***************************************************************/
/*   void Hiperbola::canviar_source_u(const double &u)         */
/* Metode que permet modificar el valor d'inici de la hiperbola*/
/***************************************************************/

void Hiperbola::canviar_source_u(const double &u)
{
	Punt2d p(source_.x(),u);
	source_=p;
}

/***************************************************************/
/*   void Hiperbola::canviar_target_u(const double &u)         */
/* Metode que permet modificar el valor de fi de la hiperbola  */
/***************************************************************/

void Hiperbola::canviar_target_u(const double &u)
{
	Punt2d p(target_.x(),u);
	target_=p;
}


void Hiperbola::canviar_source(const Punt2d &p)
{
	source_=p;
}

void Hiperbola::canviar_target(const Punt2d &p)
{
	target_=p;
}


/*******************************************************************************************/
/*   bool Hiperbola::is_ft()const                                                          */
/* Els segments d'hiperbola que tenim es poden parametritzar tant en funcio de t com de u. */
/* farem is_ft()=true si menys error parametritzant per t si t_max-t_min>u_max-u_min       */
/*******************************************************************************************/

bool Hiperbola::is_ft()const
{

	return this->t_max()-this->t_min()>=this->u_max()-this->u_min();
}

/**********************************************************************/
/*   bool Hiperbola::is_fu()const                                     */
/* Metode que retorna cert si es funcio de t u=f(t) en tot l'interval */
/**********************************************************************/

bool Hiperbola::is_fu()const
{
	return !this->is_ft();
}

bool Hiperbola::in_open_range(double primer, double segon, double tercer, bool restringit)const
{
	if(segon<tercer)
		if(primer>=segon && primer<=tercer)
			if(restringit)
				if(primer>=0 && primer<=1)
					return true;
				else
					return false;
			else
				return true;
		else
			return false;
	else
		if(primer>=tercer && primer<=segon)
			if(restringit)
				if(primer>=0 && primer<=1)
					return true;
				else
					return false;
			else
				return true;
		else
			return false;
}

bool Hiperbola::in_range(double primer, double segon, double tercer, bool restringit)const
{
	if(segon<tercer)
		if(primer>segon && primer<tercer)
			if(restringit)
				if(primer>0 && primer<1)
					return true;
				else
					return false;
			else
				return true;
		else
			return false;
	else
		if(primer>tercer && primer<segon)
			if(restringit)
				if(primer>0 && primer<1)
					return true;
				else
					return false;
			else
				return true;
		else
			return false;
}

/*************************************************************************************/
/*   bool Hiperbola::is_ft_r()const                                                  */
/* Metode que retorna cert si es funcio de u t=f(u) en l'interval restringit a [0,1] */
/*************************************************************************************/

bool Hiperbola::is_ft_r()const
{
	return !in_range(-cu_/ctu_,source_.x(),target_.x(), true);
}

/*************************************************************************************/
/*   bool Hiperbola::is_ft_r()const                                                  */
/* Metode que retorna cert si es funcio de t u=f(t) en l'interval restringit a [0,1] */
/*************************************************************************************/

bool Hiperbola::is_fu_r()const
{

	return !in_range(-ct_/ctu_,source_.y(),target_.y(), true);
}

/*********************************************************************************************/
/*   double Hiperbola::u(const double &t)const                                               */
/* Metode que retorna la coordenada u del  punt de la hiperbola a partir de la coordenada t  */
/*********************************************************************************************/

double Hiperbola::u(const double &t)const
{
	double den=ctu_*t +cu_;
	if(fabs(den)>TOLERANCE) return -(cc_+ct_*t)/den;
	else{ cout<<"no es pot calcular y("<<t<<")."<<endl; return 0;}
	
}

/*********************************************************************************************/
/*   double Hiperbola::t(const double &u)const                                               */
/* Metode que retorna la coordenada t del  punt de la hiperbola a partir de la coordenada u  */
/*********************************************************************************************/

double Hiperbola::t(const double & u)const
{
	double den=ctu_*u +ct_;
	if(fabs(den)>TOLERANCE) return -(cc_+cu_*u)/den;
	else{ cout<<"no es pot calcular x("<<u<<")."<<endl; return 0;}
}

/**************************************************************************/
/*   Punt2d Hiperbola::p_u(const double &t)const                          */
/* Metode que troba el punt de la hiperbola a partir de la coordenada t   */
/**************************************************************************/

Punt2d Hiperbola::p_u(const double &t)const
{
	double u;
	double den=ctu_*t +cu_;
	if(fabs(den)>TOLERANCE) u=-(cc_+ct_*t)/den;
	else cout<<"no es pot calcular y("<<t<<")."<<endl;
	Punt2d p(t,u);

	return p;
}

/**************************************************************************/
/*   Punt2d Hiperbola::p_t(const double &u)const                          */
/* Metode que troba el punt de la hiperbola a partir de la coordenada u   */
/**************************************************************************/

Punt2d Hiperbola::p_t(const double & u)const
{
	double t;
	double den=ctu_*u +ct_;
	if(fabs(den)>TOLERANCE) t=-(cc_+cu_*u)/den;
	else cout<<"no es pot calcular x("<<u<<")."<<endl;
	Punt2d p(t,u);

	return p;
}

/***********************************************************************************************************/
/*   bool Hiperbola::u_funcio_t_en(const double & t)const                                                  */
/* Metode que indica si la hiperbola té una unica imatge pel valor de t (puc posar u en funció de t en t=t)*/
/***********************************************************************************************************/

bool Hiperbola::u_funcio_t_en(const double & t)const
{
	if(fabs(ctu_*t+cu_)<TOLERANCE)
		return 0;
	else
		return 1;
}

/***********************************************************************************************************/
/*   bool Hiperbola::t_funcio_u_en(const double & u)const                                                  */
/* Metode que indica si la hiperbola té una unica imatge pel valor de u (puc posar t en funció de u en u=u)*/
/***********************************************************************************************************/

bool Hiperbola::t_funcio_u_en(const double & u)const
{
	if(fabs(ctu_*u +ct_)<TOLERANCE)
		return 0;
	else
		return 1;
}

/****************************************************************/
/*   Hiperbola &Hiperbola::operator= (const Hiperbola &hyp)     */
/*Redefinicio de l'operador d'assignacio per l'objecte hiperbola*/  
/****************************************************************/

/*Hiperbola &Hiperbola::operator= (const Hiperbola &hyp)
{
	if(this != &hyp)
	{
 		ctu_=hyp.ctu();
		ct_=hyp.ct();
		cu_=hyp.cu();
		cc_=hyp.cc();
		source_=hyp.source();
		target_=hyp.target();
		assimp_t_=hyp.as_t();
		assimp_u_=hyp.as_u();
	}
	return  *this;
}*/

/************************************************************/
/*   bool Hiperbola::operator==(const Hiperbola &hyp)const  */
/*Redefinicio de l'operador igualtat per l'objecte hiperbola*/  
/************************************************************/

bool Hiperbola::operator==(const Hiperbola &hyp)const
{


	if(fabs(ctu_-hyp.ctu())<TOLERANCE&&fabs(ct_-hyp.ct())<TOLERANCE&&fabs(cu_-hyp.cu())<TOLERANCE&&fabs(cc_-hyp.cc())<TOLERANCE)
		return true;

	double k;	
	if(ctu_!=0.) k=hyp.ctu()/ctu_;
	else if(ct_!=0.) k=hyp.ct()/ct_;
	else if(cu_!=0.) k=hyp.cu()/cu_;
	else if(cc_!=0.) k=hyp.cc()/cc_;
	else  
		return false; // pq siguin iguals tots coeefs de hyp han de ser 0 i complirien l'if del return true primer

	if(fabs(ctu_-k*hyp.ctu())<TOLERANCE&&fabs(ct_-k*hyp.ct())<TOLERANCE&&fabs(cu_-k*hyp.cu())<TOLERANCE&&fabs(cc_-k*hyp.cc())<TOLERANCE)
		return true;
	
	return false;

	//si quan surt k=0, son diferents, hyp te una component 0 i this no en te cap. Ptt s'ha de complir k!=0 i coef's iguals

	return (k!=0 && fabs(k*ctu_-hyp.ctu())<TOLERANCE && fabs( k*ct_-hyp.ct())<TOLERANCE && fabs(k*cu_-hyp.cu())<TOLERANCE && fabs(k*cc_-hyp.cc())<TOLERANCE);
}
/************************************************************/
/*   bool Hiperbola::operator!=(const Hiperbola &hyp)const  */
/*Redefinicio de l'operador diferencia per l'objecte hiperbola*/  
/************************************************************/

bool Hiperbola::operator!=(const Hiperbola &hyp)const
{
	return !(*this==hyp);
}

/*****************************************************************/
/*   bostream & operator<<( ostream &os, const Hiperbola &hyp )  */
/*Redefinicio de l'operador << per l'objecte hiperbola           */  
/*****************************************************************/

ostream & operator<<( ostream &os, const Hiperbola &hyp )
{
	os<<hyp.ctu()<<" "<<hyp.ct()<<"  "<<hyp.cu()<<"  "<<hyp.cc()<<"       "<<hyp.source().x()<<" "<<hyp.source().y()<<"     "<<hyp.target().x()<<" "<<hyp.target().y()<<endl;;

	return os;
}

/****************************************/
/*   double Hiperbola::t_min()const     */
/*Metode que retorna el valor minim de t*/  
/****************************************/

double Hiperbola::t_min()const
{
	if(source_.x()<target_.x()) return source_.x();
	else return target_.x();
	
}

/****************************************/
/*   double Hiperbola::t_max()const     */
/*Metode que retorna el valor maxim de t*/  
/****************************************/

double Hiperbola::t_max() const
{
	if(source_.x()<target_.x()) return target_.x();
	else return source_.x();
}

/****************************************/
/*   double Hiperbola::u_min()const     */
/*Metode que retorna el valor minim de u*/  
/****************************************/

double Hiperbola::u_min()const
{
	return min(source_.y(),target_.y());
}

/****************************************/
/*   double Hiperbola::u_max()const     */
/*Metode que retorna el valor maxim de u*/  
/****************************************/

double Hiperbola::u_max()const
{
	return max(source_.y(),target_.y());
}

/***************************************************************************/
/*   bool Hiperbola::possible(const Hiperbola &h,const Hiperbola &hh)      */
/*Metode que indica si es o no possible l'existencia de les dues hiperboles*/  
/***************************************************************************/

bool Hiperbola::possible(const Hiperbola &h,const Hiperbola &hh)const
{
	return max(h.t_min(),hh.t_min())<min(h.t_max(),hh.t_max())&& max(h.t_min(),hh.t_min())<1 && min(h.t_max(),hh.t_max())>0;
}

/**********************************************************************************************************/
/*   bool is_point_(const double &ctu,const double &ct,const double &cu,const double &cc,const Punt2d &p) */
/*Metode que indica si un punt forma part o no de la hiperbola a partir del valor dels parametres         */  
/**********************************************************************************************************/

bool Hiperbola::is_point_(const double &ctu,const double &ct,const double &cu,const double &cc,const Punt2d &p)const
{

	double d=fabs(ctu*p.x()*p.y()+ct*p.x()+cu*p.y()+cc);
	if( d>TOLERANCE ){ cout<<" a is point de l ahiperbola, surt una diferencia de d: "<<d<<endl; return false;}
	else return true;
}

/***************************************************************/
/*   bool Hiperbola::is_point(const Punt2d &p)const            */
/*Metode que indica si un punt forma part o no de la hiperbola */  
/***************************************************************/

bool Hiperbola::is_point(const Punt2d &p)const
{
	double d=fabs(ctu_*p.x()*p.y()+ct_*p.x()+cu_*p.y()+cc_);
	if( d>TOLERANCE) cout<<" d punt hyp "<<d;
	return fabs(ctu_*p.x()*p.y()+ct_*p.x()+cu_*p.y()+cc_)<(TOLERANCE);

}

/****************************************************************/
/*   bool Hiperbola::in_u_restricted_range(const double &d)const*/
/*Metode que indica si es troba dins de u en l'interval [0,1]   */  
/****************************************************************/

bool Hiperbola::in_u_restricted_range(const double &d)const
{
	return (d>min(source_.y(),target_.y())+1e-6&&d<max(source_.y(),target_.y())-1e-6);
	return in_range(d,source_.y(),target_.y(),true);
}

/****************************************************************/
/*   bool Hiperbola::in_t_restricted_range(const double &d)const*/
/*Metode que indica si es troba dins de t en l'interval [0,1]    */  
/****************************************************************/

bool Hiperbola::in_t_restricted_range(const double &d)const
{
	return (d>min(source_.x(),target_.x())+1e-6&&d<max(source_.x(),target_.x())-1e-6);
	return (d>min(source_.x(),target_.x())+TOLERANCE&&d<max(source_.x(),target_.x())-TOLERANCE);
	return in_range(d,source_.x(),target_.x(),true);
}

/****************************************************************/
/*   bool Hiperbola::in_t_range(const double &d)const           */
/*Metode que indica si es troba dins de t contant en els extrems*/  
/****************************************************************/

bool Hiperbola::in_u_range(const double &d)const
{
	return (d>min(source_.y(),target_.y())-TOLERANCE&&d<max(source_.y(),target_.y())+TOLERANCE);
	return in_range(d,source_.y(),target_.y(),false);
}

/****************************************************************/
/*   bool Hiperbola::in_t_range(const double &d)const           */
/*Metode que indica si es troba dins de t contant en els extrems*/  
/****************************************************************/

bool Hiperbola::in_t_range(const double &d) const
{
	return (d>min(source_.x(),target_.x())-TOLERANCE&&d<max(source_.x(),target_.x())+TOLERANCE);
	return in_range(d,source_.x(),target_.x(),false);
}

/****************************************************************/
/*   bool Hiperbola::in_u_range(const double &d)const           */
/*Metode que indica si es troba dins de u excepte en els extrems*/  
/****************************************************************/

bool Hiperbola::in_u_open_range(const double &d)const
{
	return in_open_range(d,source_.y(),target_.y(),false);

}

/****************************************************************/
/*   bool Hiperbola::in_t_range(const double &d)const           */
/*Metode que indica si es troba dins de t excepte en els extrems*/  
/****************************************************************/

bool Hiperbola::in_t_open_range(const double &d) const
{
	return in_open_range(d,source_.x(),target_.x(),false);
}

/**********************************************/
/*   vector<double> Hiperbola::t_range() const*/
/*Metode que retorna el rang de t             */  
/**********************************************/

vector<double> Hiperbola::t_range() const
{
	vector<double> r;
	r.push_back(min(source_.x(),target_.x()));
	r.push_back(max(source_.x(),target_.x()));
	return r;
}

/**********************************************/
/*   vector<double> Hiperbola::u_range() const*/
/*Metode que retorna el rang de u             */  
/**********************************************/

vector<double> Hiperbola::u_range() const
{
	vector<double> r;
	r.push_back(min(source_.y(),target_.y()));
	r.push_back(max(source_.y(),target_.y()));
	return r;
}

/**************************************************************************************/
/*   vector<Punt2d> Hiperbola::int_ailla_u(const Hiperbola &hyp)*/
/*Metode retorna els punts d'interseccio suposant que es pot aillar la y de l'actual */  
/**************************************************************************************/
vector<Punt2d> Hiperbola::int_ailla_u(const Hiperbola &hyp_, const Hiperbola &hyp)const
{
	vector<Punt2d> lp;
	double a=-hyp.ctu()*hyp_.ct()+hyp.ct()*hyp_.ctu();
	double b=-hyp.ctu()*hyp_.cc()+hyp.ct()*hyp_.cu()-hyp.cu()*hyp_.ct()+hyp.cc()*hyp_.ctu();
	double c=-hyp.cu()*hyp_.cc()+hyp.cc()*hyp_.cu();
	if(b*b-4.*a*c>=0)
	{
		double t=(-b+sqrt(b*b-4.*a*c))/(2.*a);
		for(int i=0;i<2;i++)
		{
			if(hyp_.in_t_range(t)&&hyp.in_t_range(t))
			{
				double u=-(hyp_.ct()*t+hyp_.cc())/(hyp_.ctu()*t+hyp_.cu());
				if(hyp_.in_u_range(u)&&hyp.in_u_range(u))
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!hyp_.is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_u), hem trobat un punt que no és d'una"<<endl;
					lp.push_back(p);
				}
			}
			t=(-b-sqrt(b*b-4.*a*c))/(2.*a);
		}
	}
	
	return lp;
}
/*
vector<Punt2d> Hiperbola::int_ailla_u(const Hiperbola &hyp, bool rangGran)const
{
	vector<Punt2d> lp;
	double a=-hyp.ctu()*hyp_.ct()+hyp.ct()*hyp_.ctu();
	double b=-hyp.ctu()*hyp_.cc()+hyp.ct()*hyp_.cu()-hyp.cu()*hyp_.ct()+hyp.cc()*hyp_.ctu();
	double c=-hyp.cu()*hyp_.cc()+hyp.cc()*hyp_.cu();
	if(b*b-4.*a*c>=0)
	{
		double t=(-b+sqrt(b*b-4.*a*c))/(2.*a);
		for(int i=0;i<2;i++)
		{
			if(hyp_.in_t_range(t)&&hyp.in_t_range(t))
			{
				double u=-(hyp_.ct()*t+hyp_.cc())/(hyp_.ctu()*t+hyp_.cu());
				if(hyp_.in_u_range(u)&&hyp.in_u_range(u))
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!hyp_.is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_u), hem trobat un punt que no és d'una"<<endl;
					lp.push_back(p);
				}
			}
			t=(-b-sqrt(b*b-4.*a*c))/(2.*a);
		}
	}
	
	return lp;*/
	/*vector<Punt2d> lp;
	double a=-hyp.ctu()*ct()+hyp.ct()*ctu();
	double b=-hyp.ctu()*cc()+hyp.ct()*cu()-hyp.cu()*ct()+hyp.cc()*ctu();
	double c=-hyp.cu()*cc()+hyp.cc()*cu();
	if(a>0)
		if(b*b-4.*a*c>=0)
		{
			double t=(-b+sqrt(b*b-4.*a*c))/(2.*a);
			for(int i=0;i<2;i++)
			{
				bool tDinsRang;
				if(rangGran)  tDinsRang=in_t_range(t)||hyp.in_t_range(t);
				else  tDinsRang=in_t_range(t)&&hyp.in_t_range(t);
				if(tDinsRang)
				{
					double u=-(ct()*t+cc())/(ctu()*t+cu());
					bool uDinsRang;
					if(rangGran)  uDinsRang=in_u_range(u)||hyp.in_u_range(u);
					else  uDinsRang=in_u_range(u)&&hyp.in_u_range(u);
					if(uDinsRang)
					{
						Punt2d p(t,u);
						if(!hyp.is_point(p)||!is_point(p)) cout<<" Problemes interseccio hiperboles (int_ailla_u), (punt que no és d'una)"<<endl;
						lp.push_back(p);
					}
				}
				t=(-b-sqrt(b*b-4.*a*c))/(2.*a);
			}
		}
	else
		if(b!=0)
		{
			double t=-c/b;
			bool tDinsRang;
			if(rangGran)  tDinsRang=in_t_range(t)||hyp.in_t_range(t);
			else  tDinsRang=in_t_range(t)&&hyp.in_t_range(t);
			if(tDinsRang)
			{
				double u=-(ct()*t+cc())/(ctu()*t+cu());
				bool uDinsRang;
				if(rangGran)  uDinsRang=in_u_range(u)||hyp.in_u_range(u);
				else  uDinsRang=in_u_range(u)&&hyp.in_u_range(u);
				if(uDinsRang)
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!is_point(p)) cout<<" Problemes interseccio hiperboles (int_ailla_u), (punt que no és d'una)"<<endl;
					lp.push_back(p);
				}
			}
		}
	return lp; */
//}

/**************************************************************************************/
/*   vector<Punt2d> Hiperbola::int_ailla_t(const Hiperbola &hyp)*/
/*Metode retorna els punts d'interseccio suposant que es pot aillar la x del'actual                       */  
/**************************************************************************************/

vector<Punt2d> Hiperbola::int_ailla_t(const Hiperbola &hyp_, const Hiperbola &hyp) const
{
	vector<Punt2d> lp;
	
	double a=-hyp.ctu()*hyp_.cu()+hyp.cu()*hyp_.ctu();
	double b=-hyp.ctu()*hyp_.cc()-hyp.ct()*hyp_.cu()+hyp.cu()*hyp_.ct()+hyp.cc()*hyp_.ctu();
	double c=-hyp.ct()*hyp_.cc()+hyp.cc()*hyp_.ct();
	double D=b*b-4*a*c;
	if(D>=0)
	{
		double u=(-b+sqrt(D))/(2.*a);
		for(int i=0;i<2;i++)
		{
			if(hyp_.in_u_range(u)&&hyp.in_u_range(u))
			{
				double t=-(hyp_.cu()*u+hyp_.cc())/(hyp_.ctu()*u+hyp_.ct());
				if(hyp_.in_t_range(t)&&hyp.in_t_range(t))
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!hyp_.is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_t), hem trobat un punt que no és d'una"<<endl;
					lp.push_back(p);
				}
			}
			u=(-b-sqrt(D))/(2.*a);
		}
	}
	return lp;

}

/*
vector<Punt2d> Hiperbola::int_ailla_t(const Hiperbola &hyp, bool rangGran)const
{
vector<Punt2d> lp;
	
	double a=-hyp.ctu()*hyp_.cu()+hyp.cu()*hyp_.ctu();
	double b=-hyp.ctu()*hyp_.cc()-hyp.ct()*hyp_.cu()+hyp.cu()*hyp_.ct()+hyp.cc()*hyp_.ctu();
	double c=-hyp.ct()*hyp_.cc()+hyp.cc()*hyp_.ct();
	double D=b*b-4*a*c;
	if(D>=0)
	{
		double u=(-b+sqrt(D))/(2.*a);
		for(int i=0;i<2;i++)
		{
			if(hyp_.in_u_range(u)&&hyp.in_u_range(u))
			{
				double t=-(hyp_.cu()*u+hyp_.cc())/(hyp_.ctu()*u+hyp_.ct());
				if(hyp_.in_t_range(t)&&hyp.in_t_range(t))
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!hyp_.is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_t), hem trobat un punt que no és d'una"<<endl;
					lp.push_back(p);
				}
			}
			u=(-b-sqrt(D))/(2.*a);
		}
	}
	return lp;*/
	/*
	vector<Punt2d> lp;
	
	double a=-hyp.ctu()*cu()+hyp.cu()*ctu();
	double b=-hyp.ctu()*cc()-hyp.ct()*cu()+hyp.cu()*ct()+hyp.cc()*ctu();
	double c=-hyp.ct()*cc()+hyp.cc()*ct();
	double D=b*b-4*a*c;
	if(a!=0)
		if(D>=0)
		{
			double u=(-b+sqrt(D))/(2.*a);
			for(int i=0;i<2;i++)
			{
				bool uDinsRang;
				if(rangGran)  uDinsRang=in_u_range(u)||hyp.in_t_range(u);
				else  uDinsRang=in_u_range(u)&&hyp.in_u_range(u);
				if(uDinsRang)
				{
					double t=-(cu()*u+cc())/(ctu()*u+ct());
					bool tDinsRang;
					if(rangGran)  tDinsRang=in_t_range(t)||hyp.in_t_range(t);
					else  tDinsRang=in_t_range(t)&&hyp.in_t_range(t);
					if(tDinsRang)
					{
						Punt2d p(t,u);
						if(!hyp.is_point(p)||!is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_t), hem trobat un punt que no és d'una"<<endl;
						lp.push_back(p);
					}
				}
				u=(-b-sqrt(D))/(2.*a);
			}
		}
	else
		if(b!=0)
		{
			double u=-c/b;
			bool uDinsRang;
			if(rangGran)  uDinsRang=in_u_range(u)||hyp.in_t_range(u);
			else  uDinsRang=in_u_range(u)&&hyp.in_u_range(u);
			if(uDinsRang)
			{
				double t=-(cu()*u+cc())/(ctu()*u+ct());
				bool tDinsRang;
				if(rangGran)  tDinsRang=in_t_range(t)||hyp.in_t_range(t);
				else  tDinsRang=in_t_range(t)&&hyp.in_t_range(t);
				if(tDinsRang)
				{
					Punt2d p(t,u);
					if(!hyp.is_point(p)||!is_point(p)) cout<<" Problemes interseccio d'hiperboles (int_ailla_t), hem trobat un punt que no és d'una"<<endl;
					lp.push_back(p);
				}
			}
		}
	return lp;

}
*/
/*****************************************************************************************/
/*   bool Hiperbola::interseccio(const Hiperbola &hyp,vector<Punt2d> &p,bool extrems=false)const*/
/*   Metode que retorna cert si les 2 hiperboles interseccionen, a p hi retorna els pts d'interseccio.    */
/*  si extrems= false els extrems de les hiperboles no es compten com a pts interseccio, si es true sí */  
/******************************************************************************************/

bool Hiperbola::interseccio(const Hiperbola &hyp,vector<Punt2d> &p,bool dinsQuadratUnitat,bool extrems,bool rangGran)const
{
// 	if(extrems) p=intersection(hyp,rangGran);
// 	else p=intersection_no_end_points(hyp,rangGran);
	p.clear();
	if(extrems)
		p=intersection(hyp);
	else 
		p=interseccioNoExtrems(hyp);
//  	else p=intersection_no_end_points(hyp);

	if ( p.size()==0 ) return false;
	else
		if(!dinsQuadratUnitat) return true;
		else
		{
			bool intersecciona=false;
			unsigned int i=0;
			while (! intersecciona && i<p.size())
			{
			    intersecciona= p[i].x()>=0.&& p[i].x()<=1.&&p[i].y()>=0.&& p[i].y()<=1.;
			    i++;
			 }
			return intersecciona;
		}
	
}

vector<Punt2d> Hiperbola::interseccioNoExtrems(const Hiperbola &h)const
{
	vector<Punt2d> p;
	if(interseccioAillarTRR(h,p)){}
	else if(interseccioAillarURR(h,p)){}
	else if(h.interseccioAillarTRR(*this,p)){}
	else if (h.interseccioAillarURR(*this,p)){}
	else {cout<<" Cap ";}
	
	return p;
	
	//comprovem el resultat:
	bool interseca=false;
	double rx=target_.x()-source_.x();
// 	double yx=target_.y()-source_.y();
	double rx1=h.target().x()-h.source().x();
// 	double yx1=h.target().y()-h.source().y();
	if(fabs(rx)>TOLERANCE && fabs(rx1)>TOLERANCE)
	{
		double inc=rx/100.;
		int i0=0;
		double x=source_.x()+i0*inc;
		while( (!in_t_range(x) || ! h.in_t_range(x) || fabs(h.avaluarPunt(Punt2d(x,avaluarX(x))))<0.0001 )&i0<100)
		{
			i0++;
			x=source_.x()+inc*(double)i0;
		}
		double xs=h.avaluarPunt(Punt2d(x,avaluarX(x)));
		double xi;
		for(int i=i0;i<100&&!interseca&&in_t_restricted_range(x)&&h.in_t_restricted_range(x);i++)
		{
			x=source_.x()+inc*(double)i;
			xi=h.avaluarPunt(Punt2d(x,avaluarX(x)));
			interseca=fabs(xi)>TOLERANCE && xi*xs<0;
		}
		
		if((!interseca&&p.size()>0)||(interseca && p.size()==0))
		{
			cout<<"PROBLEMES A INTERSECCIO: xs"<<xs<<" xi "<<xi<<" interseca "<<interseca <<" p.size() "<<p.size()<<endl;
			h.escriure();
			escriure();
		}
	}
	else 
		cout<<"NO POGUT COMPROVAR"<<endl;
	
	return p;
}

bool Hiperbola::interseccioAillarTRR(const Hiperbola &h, vector<Punt2d> &p)const
{
	
	double a=-ctu_*h.ct()+ct_*h.ctu();
	double b=-ctu_*h.cc()+ct_*h.cu()-cu_*h.ct()+cc_*h.ctu();
	double c=-cu_*h.cc()+cc_*h.cu();
	
	double t,u;
	if(fabs(a)>1e-17)
	{
		double D=b*b-4*a*c;
		Punt2d pi;
		if(D>=0)
		{
			t=(-b+sqrt(D))/(2.*a);
			if( in_t_restricted_range(t) && h.in_t_restricted_range(t))
// 			if( in_t_range(t) && h.in_t_range(t))
			{
				double den=h.ctu()*t+h.cu();
				if(fabs(den)>1e-17)
				{
					u=-(h.ct()*t+h.cc())/den;
					pi=Punt2d(t,u);
					if(fabs(avaluarPunt(pi))>1e-6||fabs(h.avaluarPunt(pi))>1e-6)
						cout<<"ERROR TROBAT UN PT NO DE LES HIP"<<endl;
					if(in_u_restricted_range(u)&& h.in_u_restricted_range(u))
// 					if( in_u_range(u) && h.in_u_range(u))
						p.push_back(Punt2d(t,u));
				}
				else       //cout<<" (no es pot aillar u) den=0  (t+)"<<endl;
					return false;
			}
			
			t=(-b-sqrt(D))/(2.*a);
			if( in_t_restricted_range(t) && h.in_t_restricted_range(t))
// 			if( in_t_range(t) && h.in_t_range(t))
			{
				double den=h.ctu()*t+h.cu();
				if(fabs(den)>TOLERANCE)
				{
					u=-(h.ct()*t+h.cc())/den;
					if(in_u_restricted_range(u)&& h.in_u_restricted_range(u))
// 					if( in_u_range(u) && h.in_u_range(u))
						p.push_back(Punt2d(t,u));
				}
				else //cout<<" (no es pot aillar u) den=0  (t-)"<<endl;
					return false;
			}
		}
		else
		{
			cout<<"Disc <0"<<endl;
			return false;
		}
	}
	else if(fabs(b)>1e-17) //eq linal
	{
		t=c/b;
		if( in_t_restricted_range(t) && h.in_t_restricted_range(t))
// 		if( in_t_range(t) && h.in_t_range(t))
		{
			double den=h.ctu()*t+h.cu();
			if(fabs(den)>TOLERANCE)
			{
				u=-(h.ct()*t+h.cc())/den;
// 				if( in_u_range(u) && h.inD_u_range(u))
				if(in_u_restricted_range(u)&& h.in_u_restricted_range(u))
					p.push_back(Punt2d(t,u));
			}
			else //cout<<" (no es pot aillar u) den=0  (t lineal)"<<endl;
				return false; 
		}
	}
	else if(fabs(c)>1e-17)
		cout<<"Eq INCOMPATIBLE"<<endl;
		
	for(unsigned int i=0;i<p.size();i++)
		if(fabs(avaluarPunt(p[i]))>1e-6 || fabs(h.avaluarPunt(p[i]))>1e-6 )
			cout<<"PROBLEMES pt no d'hips  1111111111111111111"<<fabs(avaluarPunt(p[i]))<<" "<<fabs(h.avaluarPunt(p[i]))<<endl;
	
	return true;	
	
}


bool Hiperbola::interseccioAillarURR(const Hiperbola &h, vector<Punt2d> &p)const
{

	vector<Punt2d> q;
	Hiperbola h1(ctu_,cu_,ct_,cc_,Punt2d(source_.y(),source_.x()),Punt2d(target_.y(),target_.x()));
	Hiperbola h2(h.ctu(),h.cu(),h.ct(),h.cc(), Punt2d(h.source().y(),h.source().x()),Punt2d(h.target().y(),h.target().x()));
	bool b=h1.interseccioAillarTRR(h2,q);
	for(unsigned int i=0;i<q.size();i++)
		p.push_back(Punt2d(q[i].y(),q[i].x()));
		
	return b;
}


/*****************************************************************/
	/*   vector<Punt2d> Hiperbola::intersection(const Hiperbola &hyp)*/
/*Metode que calcula els punts d'interseccio entre 2 hiperboles  */  
/*****************************************************************/

vector<Punt2d> Hiperbola::intersection(const Hiperbola &hyp) const
{
	vector<Punt2d> lp;
	
	if(((fabs(hyp.cu())+fabs(hyp.ctu()))>TOLERANCE)){  lp=int_ailla_u(hyp, *this);   }
	else if(((fabs(this->cu())+fabs(this->cu()))>TOLERANCE)){  lp=int_ailla_u(*this,hyp);   }
	else if(((fabs(hyp.ct())+fabs(hyp.ctu()))>TOLERANCE)){ lp=int_ailla_t(hyp,*this);   }
	else if(((fabs(this->ct())+fabs(this->ctu()))>TOLERANCE)){  lp=int_ailla_t(*this,hyp);   }
	else cout<<"error, hiperbola tota igual a 0 ";
	
	return lp;
}/*
vector<Punt2d> Hiperbola::intersection(const Hiperbola &hyp,bool dinsRangGran)const
{
	vector<Punt2d> lp;
	
	if(((fabs(hyp.cu())+fabs(hyp.ctu()))>TOLERANCE)){ lp=hyp.int_ailla_u(*this,dinsRangGran);}{{  lp=int_ailla_u(hyp, *this); 
	else if(((fabs(this->cu())+fabs(this->cu()))>TOLERANCE)){  lp=int_ailla_u(hyp,dinsRangGran);   }
	else if(((fabs(hyp.ct())+fabs(hyp.ctu()))>TOLERANCE)){ lp=hyp.int_ailla_t(*this,dinsRangGran);   }
	else if(((fabs(this->ct())+fabs(this->ctu()))>TOLERANCE)){  lp=int_ailla_t(hyp,dinsRangGran);   }
	else cout<<"error, hiperbola tota igual a 0 ";
	
	return lp;
} */

/*************************************************************************************/
/*   vector<Punt2d> Hiperbola::intersection_no_end_points(const Hiperbola &hyp)      */
/*Metode que calcula els punts d'interseccio entre 2 hiperboles que no siguin extrems*/  
/*************************************************************************************/
vector<Punt2d> Hiperbola::intersection_no_end_points(const Hiperbola &hyp) const
{
	vector<Punt2d> lp,lp_f;

	if(((fabs(hyp.ctu())+fabs(hyp.cu()))>TOLERANCE)){  lp=int_ailla_u(hyp, *this);   }
	else if(((fabs(this->cu())+fabs(this->cu()))>TOLERANCE)){  lp=int_ailla_u(*this,hyp);   }
	else if(((fabs(hyp.ct())+fabs(hyp.ctu()))>TOLERANCE)){ lp=int_ailla_t(hyp,*this);   }
	else if(((fabs(this->ct())+fabs(this->ctu()))>TOLERANCE)){  lp=int_ailla_t(*this,hyp);   }
	else cout<<"error, hiperbola tota igual a 0 o una es un punt";
	
	for(unsigned int j=0;j<lp.size();j++)
	{
		
		if(!((Punt(lp[j])==Punt(this->source()) || Punt(lp[j])==Punt(this->target())) && (Punt(lp[j])==Punt(hyp.source()) || Punt(lp[j])==Punt(hyp.target())) ))  { lp_f.push_back(lp[j]); }
	}
	return lp_f;
}
/*
vector<Punt2d> Hiperbola::intersection_no_end_points(const Hiperbola &hyp,bool dinsRangGran)const
{
	vector<Punt2d> lp,lp_f;
		
	if(((fabs(hyp.ctu())+fabs(hyp.cu()))>TOLERANCE)){  lp=hyp.int_ailla_u(*this,dinsRangGran);   }
	else if(((fabs(this->cu())+fabs(this->cu()))>TOLERANCE)){  lp=int_ailla_u(hyp,dinsRangGran);   }
	else if(((fabs(hyp.ct())+fabs(hyp.ctu()))>TOLERANCE)){ lp=hyp.int_ailla_t(*this),dinsRangGran;   }
	else if(((fabs(this->ct())+fabs(this->ctu()))>TOLERANCE)){  lp=int_ailla_t(hyp,dinsRangGran);   }
	else cout<<"error, hiperbola tota igual a 0 o una es un punt";
	
	for(unsigned int j=0;j<lp.size();j++)
	{
		if(!((Punt(lp[j])==Punt(source()) || Punt(lp[j])==Punt(target())) && (Punt(lp[j])==Punt(hyp.source()) || Punt(lp[j])==Punt(hyp.target())) ))  { lp_f.push_back(lp[j]); }
	}
	return lp_f;
}*/

/********************************************************/
/*   double Hiperbola::avaluarX(double valor_x)         */
/*Metode que avalua la x retornant el valor corresponent*/  
/********************************************************/

double Hiperbola::avaluarX(double valor_x)const
{
	return((-cc_-ct_*valor_x)/(ctu_*valor_x+cu_));
}


double Hiperbola::avaluarY(double valor_y)const
{
	return((-cc_-cu_*valor_y)/(ctu_*valor_y+ct_));
}


/*****************************************************/
/*   double Hiperbola::rangX()                       */
/*Metode que retorna el rang de les x de la hiperbola*/  
/*****************************************************/

double Hiperbola::rangX()const
{
	return(fabs(source_.x()-target_.x()));
}

double Hiperbola::rangY() const
{
	return(fabs(source_.y()-target_.y()));
}

/*************************************************************************************************************************/
/*   double Hiperbola::getXAssimptota()                                                                                  */
/*si retornem un valor diferent de NO_HI_HA_ASSIMPTOTA, aquest valor és la X de l'assimptota. Sempre que l'assimptota no */
/*estigui en el rang (x_min,x_max) considerarem que la hipèrbola no té assímptota                                        */  
/*************************************************************************************************************************/

double Hiperbola::getXAssimptota()const
{
	double x_min,x_max;
	if(source_.x()<target_.x())
	{
		x_min=source_.x();
		x_max=target_.x();
	}
	else
	{
		x_min=target_.x();
		x_max=source_.x();
	}
	
	double x_assim=-cu_/ctu_;
	
	if(x_assim>=x_min && x_assim<=x_max) return(x_assim);
	else return(10000);
}

/***************************************************************/
/*   void Hiperbola::pintarRegio(bool dalt)                    */
/*Metode que permet pintar la regio corresponent a la hiperbola*/  
/***************************************************************/

void Hiperbola::pintarRegio(bool dalt,double lx, double ly)const
{
	double y_projeccio;
	vector<Punt> discretitzacio;
	int num_trossets = 250;
	double x_min,x_max;
	if(source_.x()<target_.x())
	{
		x_min=source_.x();
		x_max=target_.x();
	}
	else
	{
		x_min=target_.x();
		x_max=source_.x();
	}
	double augment=fabs(x_max-x_min)/num_trossets;
	if(source_.x()<target_.x())
		discretitzacio.push_back(source_);
	else
		discretitzacio.push_back(target_);
	
	for(int i=1;i<num_trossets;i++)
	{
		double x=x_min+i*augment;
		double y=avaluarX(x);
		discretitzacio.push_back(Punt(x,y,0.0));
	}
	if(source_.x()>target_.x())
		discretitzacio.push_back(source_);
	else
		discretitzacio.push_back(target_);
	
	if(dalt) y_projeccio=100000;//pintarem cap a dalt la regió
	else y_projeccio=-100000;

	for(unsigned int i=1;i<discretitzacio.size();i++)
	{
		glBegin(GL_POLYGON);
			glVertex2f(discretitzacio[i-1].x()*lx,discretitzacio[i-1].y()*ly);
			glVertex2f(discretitzacio[i-1].x()*lx,y_projeccio*ly);
			glVertex2f(discretitzacio[i].x()*lx,y_projeccio*ly);
			glVertex2f(discretitzacio[i].x()*lx,discretitzacio[i].y()*ly);
		glEnd();
	}
}

vector<Punt> Hiperbola::pintar()const
{
	vector<Punt> discretitzacio;
	int num_trossets =18;
	double x_min,x_max;
	if(source_.x()<target_.x())
	{
		x_min=source_.x();
		x_max=target_.x();
	}
	else
	{
		x_min=target_.x();
		x_max=source_.x();
	}
	double augment=fabs(x_max-x_min)/num_trossets;
	if(source_.x()<target_.x())
		discretitzacio.push_back(source_);
	else
		discretitzacio.push_back(target_);
	
	for(int i=1;i<num_trossets;i++)
	{
		double x=x_min+i*augment;
		double y=avaluarX(x);
		discretitzacio.push_back(Punt(x,y,0.0));
	}
	if(source_.x()>target_.x())
		discretitzacio.push_back(source_);
	else
		discretitzacio.push_back(target_);
	return discretitzacio;
	
	glBegin(GL_LINE_STRIP);
		for(unsigned int i=0;i<discretitzacio.size();i++)
		{
			glVertex2f(discretitzacio[i].x(),discretitzacio[i].y());
		}
	glEnd();
}

void Hiperbola::pintarNoDiscretitzar(double lx, double ly)const
{
	if(rectaVertical())
	{
		cout<<"L";
		glBegin(GL_LINES);
			glVertex2f(source_.x()*lx,source_.y()*ly);
			glVertex2f(target_.x()*lx,target_.y()*ly);
		glEnd();
	}
	else
	{
		cout<<"H";
		double trossos=100.;
		double x_min=source().x();
		double h=(source_.x()+target_.x())/trossos;
		if(source().x()>target().x()) x_min=target().x();
		glBegin(GL_LINE_STRIP);
			for(unsigned int i=0;i<trossos;i++)
			{
				double x=x_min+h*(double)trossos;
				glVertex2f(x*lx,avaluarX(x)*ly);
			}
		glEnd();
	}
}

bool Hiperbola::rectaVertical()const
{
	return (fabs(ctu_)<TOLERANCE&&fabs(cu_)<TOLERANCE)||(fabs(cu_)<TOLERANCE&&fabs(cc_)<TOLERANCE);
}

bool Hiperbola::rectaHoritzontal()const
{
	return (fabs(ctu_)<TOLERANCE&&fabs(ct_)<TOLERANCE)||(fabs(ct_)<TOLERANCE&&fabs(cc_)<TOLERANCE);
}

bool Hiperbola::esPunt()const
{
	return ctu_==0 && ct_==0 && cu_==0;
}

/***************************************************************/
/*   void Hiperbola:: avaluarPunt( Punt2d p)const               */
/*Metode que avalua un punt en l'equacio de la hiperbola i retorna el valor obtingut*/  
/***************************************************************/
double Hiperbola::avaluarPunt( Punt2d p)const
{
	double d=ctu()*p.x()*p.y()+ct()*p.x()+cu()*p.y()+cc();
	return d;
}

/***************************************************************/
/*        void Hiperbola::canviarSigne()         */
/*Metode que canvia el signe dels coeficients de la hiperbola */  
/***************************************************************/
void Hiperbola::canviarSigne()
{
	ctu_=-ctu_;
	ct_=-ct_;
	cu_=-cu_;
	cc_=-cc_;
}


/***************************************************************/
/*       Regio Hiperbola::regio(const Hiperbola &h, bool comprovarSigne) const         */
/*Metode que crea una regio a partir de dues hiperboles amb un extrem comu, (acutal i h)  */  
/* Es busca un pt en comu i l'altre punt on es posara una recta vertical i s'envia a reguio(hip,pt,pt,comprobSigne) */
/* *this es la de rang maxim en x                                                                                       */
/* Comprova que el signe de dins la regio sigui <= si comprovarSigne=true (valor x defecte)*/
/***************************************************************/
Regio Hiperbola::regio(const Hiperbola &h, bool comprovarSigne) const
{
	Punt2d comu,altre;
 	if(fabs(avaluarPunt(h.source()))<TOLERANCE || (Punt)h.source()==(Punt)source_ || (Punt)h.source()==(Punt)target_)   // h.source() vertex comu
	{
		comu=h.source();
		altre=h.target(); // this es la de rang maxim
	}
	else
	{
		comu=h.target();
		altre=h.source(); // this es la de rang maxim
	}
	return regio(h,comu,altre,comprovarSigne);
}


/*****************************************************************************************************/
/*       Regio Hiperbola::regio( const Hiperbola &h,  const Punt2d &p , bool cap, bool comprovarSigne) const         */
/*Metode que crea una regio a partir de dues hiperboles amb un extrem comu i q intersequen al punt p (acutal i h)  */  
/* (les dues hip determinen una regio peix, cap= true si es preten general la regio del cap false si es vol la cua. */
/* Es busca un pt en comu i l'altre punt depenent del valor de cap                                */
/* Comprova que el signe de dins la regio sigui <= si comprovarSigne=true (valor x defecte)*/
/************************************************************************************************/
Regio Hiperbola::regio( const Hiperbola &h,  const Punt2d &p , bool cap, bool comprovarSigne) const
{
// cout<<"Regio Hiperbola::regio(h,p cap) const"<< "cap= "<<cap<<endl;
	if(cap)
	{
		if(degenerada()||h.degenerada()) cout<<"REGIO CAP AMB 1 hip DEGENERADA !!!!!!!!!!!!! NO POT SER "<<endl;
		Punt2d comu;
		if(/*fabs(avaluarPunt(h.source()))<TOLERANCE||*/ (Punt)h.source()==(Punt)source_ || (Punt)h.source()==(Punt)target_)
			comu=h.source();
		else comu=h.target();
		vector<Hiperbola> hip;
		hip.push_back(Hiperbola(ctu(),ct(),cu(),cc(),comu,p));
		hip.push_back(Hiperbola(h.ctu(),h.ct(),h.cu(),h.cc(),comu,p));
		
		if(comprovarSigne)
		{
			Punt2d punt_mig;
			double x_mig=(comu.x()+p.x())/2.;
			if(h.u_funcio_t_en(x_mig))
			{
				Punt2d aux=p_u(x_mig);
				Punt2d auxh= h.p_u(x_mig);
				 punt_mig=Punt2d (x_mig,(aux.y()+auxh.y())/2.);
			}
			else
			{
				double y_mig=(comu.y()+p.y())/2.;
				if(!h.t_funcio_u_en(y_mig))
					cout<<"error cap"<<endl;
				Punt2d aux=p_u(y_mig);
				Punt2d auxh=h.p_u(y_mig);
				punt_mig=Punt2d ((aux.x()+auxh.x())/2.,y_mig);
				
			}
				
			for(unsigned int i=0;i<2;i++)
				if(hip[i].avaluarPunt(punt_mig)>0) hip[i].canviarSigne();
		}
		
		return Regio(hip[0],hip[1]);	
	}
	else
	{    
		Punt2d altre;
		if((fabs(h.source().x()-source_.x())<TOLERANCE&&fabs(h.source().y()-source_.y())<TOLERANCE)||(fabs(h.source().x()-target_.x())<TOLERANCE&&fabs(h.source().y()-target_.y())<TOLERANCE)) altre=h.target();
		else altre=h.source(); 
		
		return regio(h,p,altre,comprovarSigne);
	}
}

/*****************************************************************************************************/
/*       Regio Hiperbola:: regio(const Hiperbola &h,const Punt2d & comu,const Punt2d &altre,  bool comprovarSigne) const         */
/*Metode que crea una regio a partir de dues hiperboles q comparteixen el punt comu i acaba de determinar-la  */  
/*amb una recta vertical a l'altre punt, canviara doncs els extrems de les hiperboles pq quedi una regio ben definida.. */
/* Comprova que el signe de dins la regio sigui <= si comprovarSigne=true (valor x defecte)*/
/************************************************************************************************/
Regio Hiperbola:: regio(const Hiperbola &h,const Punt2d & comu,const Punt2d &altre,  bool comprovarSigne) const
{
	Punt2d punt_mig;
	vector<Hiperbola> hip;
	
	double x_mig=(comu.x()+altre.x())/2.;
			
	if(u_funcio_t_en(altre.x())&&h.u_funcio_t_en(altre.x()))
	{
		hip.push_back(Hiperbola(ctu(),ct(),cu(),cc(),comu,Punt2d(altre.x(),avaluarX(altre.x()))));
		hip.push_back(Hiperbola(h.ctu(),h.ct(),h.cu(),h.cc(),comu, Punt2d(altre.x(),h.avaluarX(altre.x()))));
		hip.push_back(Hiperbola(0.,1.,0.,-altre.x(),Punt2d(altre.x(),avaluarX(altre.x())),Punt2d(altre.x(),h.avaluarX(altre.x()))));
	}
	else 
	{
		if(u_funcio_t_en(altre.x())&&fabs(h.avaluarPunt(altre))<TOLERANCE)
		{ 
			hip.push_back(Hiperbola(ctu(),ct(),cu(),cc(),comu,Punt2d(altre.x(),avaluarX(altre.x()))));
			hip.push_back(Hiperbola(h.ctu(),h.ct(),h.cu(),h.cc(),comu, altre));
			hip.push_back(Hiperbola(0.,1.,0.,-altre.x(),Punt2d(altre.x(),avaluarX(altre.x())),altre));
		}
		else if(h.u_funcio_t_en(altre.x())&&fabs(avaluarPunt(altre))<TOLERANCE)
		{ 
			hip.push_back(Hiperbola(ctu(),ct(),cu(),cc(),comu,altre));
			hip.push_back(Hiperbola(h.ctu(),h.ct(),h.cu(),h.cc(),comu, Punt2d(altre.x(),h.avaluarX(altre.x()))));
			hip.push_back(Hiperbola(0.,1.,0.,-altre.x(),altre,Punt2d(altre.x(),h.avaluarX(altre.x()))));
		}
		else
		{
			cout<<"Hiperbola::regio(h,comu,altre,cS) no podem trobar la x!!!!!!!!!!!!!1"<<endl;
			
			hip.push_back(Hiperbola(ctu(),ct(),cu(),cc(),comu,altre));
			hip.push_back(Hiperbola(h.ctu(),h.ct(),h.cu(),h.cc(),comu, Punt2d(altre.x(),h.avaluarX(altre.x()))));
			hip.push_back(Hiperbola(0.,1.,0.,-altre.x(),altre,Punt2d(altre.x(),h.avaluarX(altre.x()))));
			hip[0].escriure();
			hip[1].escriure();
			hip[2].escriure();
		}
	}
	
	if(u_funcio_t_en(x_mig)&&h.u_funcio_t_en(x_mig))
	{
		Punt2d aux=p_u(x_mig);
		Punt2d auxh=h.p_u(x_mig);
		punt_mig=Punt2d((aux.x()+auxh.x())/2.,(aux.y()+auxh.y())/2.);
	}
	else 
	{
		double y_mig=(comu.y()+altre.y())/2.;
		Punt2d aux=p_t(y_mig);
		Punt2d auxh=h.p_t(y_mig);
		punt_mig=Punt2d((aux.x()+auxh.x())/2.,(aux.y()+auxh.y())/2.);
	}
	
	if(comprovarSigne)
		for(unsigned int i=0;i<3;i++)
			if(hip[i].avaluarPunt(punt_mig)>0) 
				hip[i].canviarSigne();
	
	return Regio(hip[0],hip[1],hip[2]);
}

/*****************************************************************************************************/
/*         Regio Hiperbola::regio(const Hiperbola &h1, const Hiperbola &h2, bool comprovarSigne)const      */
/*Metode que crea una regio a partir de tres hiperboles segur q n'hi ha una de degenerada (punt o recta )  */  
/* la hiperbola (this) NO es DEGENERADA!!!!!!!!!!!!                                                                   */
/* Comprova que el signe de dins la regio sigui <= si comprovarSigne=true (valor x defecte)*/
/************************************************************************************************/

// la hiperbola (this) es no degenerada, h1 i h2 son degenerades
Regio Hiperbola::regio(const Hiperbola &h1, const Hiperbola &h2, bool comprovarSigne)const
{

	char c1, c2;
	bool deg1=h1.degenerada(c1);
	bool deg2=h2.degenerada(c2);	

	vector<Hiperbola> hip;
	hip.clear();

// 		cout<<"deg1; "<<deg1<<" tipus: "<<c1<<" deg2: "<<deg2<<" tipus "<<c2<<endl;
// 		cout<<"pt mig: ("<<punt_mig.x()<<", "<<punt_mig.y()<<")"<<endl;
	
	if((c1=='v'&&c2=='v')||(c1=='h'&&c2=='h')) // regio: 2 r // + hip, unim els 2 extrems rectes q no son de l'hip x una recta. Partirem la regio en 2
	{
		Punt2d p0,p1;
		if(fabs(avaluarPunt(h1.source())>TOLERANCE)) p0=h1.source();
		else p0=h1.target();
		if(fabs(avaluarPunt(h2.source())>TOLERANCE)) p1=h2.source();
		else p1=h2.source();
		Punt2d v(p0.x()-p1.x(),p0.y()-p1.y());  //creem la recta q uneix els 2 extrems
		if(fabs(v.x())>TOLERANCE)  v=Punt2d(1,v.y()/v.x());
		else if(fabs(v.y())>TOLERANCE)  v=Punt2d(v.x()/v.y(),1);
		Hiperbola ha(0,-v.y(),v.x(),v.y()*p0.x()-v.x()*p0.y(),p0,p1);
// 		ha.escriure();
// 		cout<<"p0 ("<<p0.x()<<","<<p0.y()<<")"<<endl;
// 		cout<<"p1 ("<<p1.x()<<","<<p1.y()<<")"<<endl;
// 		cout<<"v ("<<v.x()<<","<<v.y()<<")"<<endl;
		Punt2d p;
		char ca;
		if(ha.degenerada(ca))
			if(ca=='h' && in_u_range(ha.cc()/ha.cu()))
			{
				p=p_t(ha.cc()/ha.cu());
				hip.push_back(h1);               // 1a recta
				hip.push_back(Hiperbola(ha.ctu(),ha.ct(),ha.cu(),ha.cc(),p0,p));  //recta auxiliar (un segment)
				if(fabs(h1.avaluarPunt(source_))) hip.push_back(Hiperbola(ctu_,ct_,cu_,cc_,source_,p));	// mitja hiperbola
				else hip.push_back(Hiperbola(ctu_,ct_,cu_,cc_,target_,p));
				
				hip.push_back(h2);         // 2a recta
				hip.push_back(Hiperbola(ha.ctu(),ha.ct(),ha.cu(),ha.cc(),p1,p));           // recta auxiliar (l'altre segment)
				if(fabs(h2.avaluarPunt(source_))) hip.push_back(Hiperbola(ctu_,ct_,cu_,cc_,source_,p));      // altra mitat d'hiperbola
				else hip.push_back(Hiperbola(ctu_,ct_,cu_,cc_,target_,p));
			}
			else
				cout<<" ARA NO TOCAROA !!!!!!"<<endl;
		else      // cal partir-ho amb triangles: 3 regions, posem recta // a l'auxiliar a l'altre extrem, partirm rectangle en 2 i la cua fa 3.
		{
			cout<<"cal acabar-ho LLLLLLLLLLLLLLLL"<<endl;
			hip.push_back(h1);               // 1a recta
			
			hip.push_back(h2);         // 2a recta
			
			hip.push_back(Hiperbola(ha.ctu(),ha.ct(),ha.cu(),ha.cc(),p0,p1));    // recta auxiliar
		}
	}
	else if((c1=='v'&&c2=='h')||(c1=='h'&&c2=='v'))
	{
// 		escriure();
// 		h1.escriure();
// 		h2.escriure();
		hip.push_back(h1);
		hip.push_back(h2);
		hip.push_back(*this);
		if(comprovarSigne)
		{
			double x_mig,y_mig;
			if(rangX()>rangY())
			{
				x_mig=(source_.x()+target_.x())/2.;
				double y0=avaluarX(x_mig);
				double y1;
				if(c1=='h') y1=h1.source().y();
				else y1=h2.source().y();
				y_mig=(y0+y1)/2.;
			}
			else
			{
				y_mig=(source_.y()+target_.y())/2.;
				double x0=avaluarY(y_mig);
				double x1;
				if(c1=='v') x1=h1.source().x();
				else x1=h2.source().x();
				x_mig=(x0+x1)/2.;
			}
			Punt2d punt_mig(x_mig,y_mig);	
			for(unsigned int k=0;k<3;k++)
				if(hip[k].avaluarPunt(punt_mig)>0) hip[k].canviarSigne();
		}
		
	}
	else
	{
// 		cout<<"else"<<endl;
		if(!deg1|| c1!='p') hip.push_back(h1);    
		if(!deg2 || c2!='p') hip.push_back(h2);
		hip.push_back(*this);    // hipebola segura (te rang en x)
	}
	
// 	cout<<"noves hiperboles"<<endl;
	if(comprovarSigne &&( hip.size()==3 ||  hip.size()==6)&& !((c1=='v'&&c2=='h')||(c1=='h'&&c2=='v')))
		for(unsigned int i=0;i<hip.size();i=i+3)
		{
// 			double x_min=min(min(hip[i].t_min(),hip[i+1].t_min()),hip[i+2].t_min());
// 			double x_max=max(max(hip[i].t_max(),hip[i+1].t_max()),hip[i+2].t_max());
// 			double x_mig=(x_min+x_max)/2.;
			double x_mig=(hip[i+2].source().x()+hip[i+2].target().x())/2.;
			Punt2d aux=p_u(x_mig);
			Punt2d auxh;
			if(hip[i].u_funcio_t_en(x_mig)) auxh=Punt2d(hip[i].p_u(x_mig));
			else auxh=Punt2d(hip[i+1].p_u(x_mig));
			
			Punt2d punt_mig((aux.x()+auxh.x())/2.,(aux.y()+auxh.y())/2.);
			for(unsigned int k=0;k<3;k++)
			{
// 				hip[i+k].escriure();
				if(hip[i+k].avaluarPunt(punt_mig)>0) hip[i+k].canviarSigne();
			}
		}	
	else if(!((c1=='v'&&c2=='h')||(c1=='h'&&c2=='v')))
		cout<<" NO 3 NI 6 hiperboles: "<<hip.size()<<" què ha passat? "<<endl;
		
	return Regio(hip);
}


/*****************************************************************************************************/
/*        int Hiperbola::hiperbolaRangMaxim(const Hiperbola &h1,const Hiperbola &h2)cons       */
/*Metode que donades tres hiperboles (this, h1, h2) retorna l'index de la hiperbola de rang màxim   */  
/*                  0 si es this------------------1 si es h1 ------------------- 2 si es h2                                              */
/************************************************************************************************/
int Hiperbola::hiperbolaRangMaxim(const Hiperbola &h1,const Hiperbola &h2)const
{
	double r;
	int i=0;
	r=rangX();
	if(h1.rangX()>r)
	{
		 i=1;
		 r=h1.rangX();
	}
	if(h2.rangX()>r)
	{
		i=2;
		r=h2.rangX();
	}
	return i;
/*
	if(t_min()<=(h1.t_min()+TOLERANCE) && t_min()<=(h2.t_min()+TOLERANCE)  && t_max()>=(h1.t_max()-TOLERANCE) && t_max()>=(h2.t_max()-TOLERANCE )) return 0;
	
	else if(h1.t_min()<=t_min()+TOLERANCE  && h1.t_min()<=(h2.t_min()+TOLERANCE)  && h1.t_max()>=(t_max()-TOLERANCE) && h1.t_max()>=(h2.t_max()-TOLERANCE)) return 1;
	
	else if(h2.t_min()<=t_min()+TOLERANCE  && h2.t_min()<=(h1.t_min()+TOLERANCE ) && h2.t_max()>=(t_max()-TOLERANCE )&& h2.t_max()>=(h1.t_max()-TOLERANCE)) return 2;*/
}

int Hiperbola::hiperbolaRangYMaxim(const Hiperbola &h1,const Hiperbola &h2)const
{
	double r;
	int i=0;
	r=rangY();
	if(h1.rangY()>r)
	{
		 i=1;
		 r=h1.rangY();
	}
	if(h2.rangY()>r)
	{
		i=2;
		r=h2.rangY();
	}
	return i;
}


/***************************************************/
/*      bool Hiperbola::degenerada() const        */
/*Metode que retorna cert si la hiperbola es una recta vertical, horitzontal o un punt   */  
/*********************************************************************************/
bool Hiperbola::degenerada() const
{
	char c;
	return degenerada(c);
}


/***************************************************/
/*      bool Hiperbola::degenerada(char &c) const        */
/*Metode que retorna cert si la hiperbola es una recta vertical, horitzontal o un punt   */  
/*   el caracter c es de retorn on hi ha una 'b' (bona,no degenerada),'v' (recta vertical),'h' (recta horitzontal),'p'  (un punt). */
/*********************************************************************************/
bool Hiperbola::degenerada(char &c) const
{
	bool r_vertical=(ctu()==0&&fabs(ct())==1&&cu()==0);
	bool r_horitzontal=false;
	bool punt=false;
	c='b'; // en principi es bona	
	if(r_vertical)
		c='v';
	else
	{
		r_horitzontal=(ctu()==0&&ct()==0&&fabs(cu())==1);
		if(r_horitzontal)
			c='h';
		else
		{
			punt=(ctu()==0&&ct()==0&&cu()==0);
			if(punt) c='p';
		}
	}
		
	return r_vertical || r_horitzontal || punt;
}

void Hiperbola::setExtrems(double x_max, double y_max)
{
// 	cout<<"SetExtrems"<<endl;
	if(rectaVertical())
	{
		source_=Punt2d(-1.*cc_/ct_,0);
		target_=Punt2d(-1.*cc_/ct_,y_max);
		
	}
	else if(rectaHoritzontal())
	{
		source_=Punt2d(0,-1.*cc_/cu_);
		target_=Punt2d(x_max,-1.*cc_/cu_);
	}
	else if(esPunt())
	{
		if(cc_==0)
			source_=Punt2d(0,0);
		else
			source_=Punt2d(x_max,y_max);
		target_=source_;
	}
	else
	{
		// sempre podem aillar la u, es a dir sempre podem trobar la Y, busquem 
		vector<Punt2d> p;
		double y0=u(0);
		double y1=u(x_max);
		
		double x0=t(0);
		double x1=t(y_max);
			
		if(y0>-TOLERANCE && y0<y_max+TOLERANCE) 
			p.push_back(Punt2d(0,y0));
		if(y1>-TOLERANCE&&y1<y_max+TOLERANCE)
			p.push_back(Punt2d(x_max,y1));
		if(x0>-TOLERANCE&&x0<x_max+TOLERANCE)
			p.push_back(Punt2d(x0,0));
		if(x1>-TOLERANCE&&x1<x_max+TOLERANCE)
			p.push_back(Punt2d(x1,y_max));
		
		// mirem si hi ha punts repetits.
		if(p.size()>2)
		{
			vector<Punt2d> aux=p;
			p.clear();
			for(unsigned int i=0;i<aux.size();i++)
			{
				bool repetit=false;
				for(unsigned int j=i+1;j<aux.size()&&!repetit;j++)
					if(aux[i]==aux[j]) repetit=true;
				if(!repetit) p.push_back(aux[i]);
			}
		}
			
		if(p.size()==2)
		{
			if(p[0].x()<p[1].x())
			{
				source_=p[0];
				target_=p[1];
			}
			else
			{
				source_=p[1];
				target_=p[0];
			}
		}
 		else
			cout<<p.size()<<" punts. ERROR (Hiperbola::setExtrems):  x_max="<<x_max<<", y_max="<<y_max<<".  (x0,x1)= "<<x0<<", "<<x1<<"  (y0,y1) = ("<< y0<<", "<<y1<<")"<<endl;
// 		cout<<"x_max="<<x_max<<", y_max="<<y_max<<".  (x0,x1)= "<<x0<<", "<<x1<<"  (y0,y1) = ("<< y0<<", "<<y1<<")"<<endl;
	}
// 	cout<<"FI SetExtrems"<<endl;
}

vector<Punt2d> Hiperbola::mostrejar(const double &inc_,const Punt2d&inici, const Punt2d&fi, const bool &reves) const
{
// 	cout<<"mh"<<endl;
	vector<Punt2d> p;
	
	
	if(degenerada()) // una recta o un punt amb 2 punts (els extrems) n'hi ha prou. 
	{
		cout<<"RECTA!!!!!!!!!!!!"<<endl;
		double vx=fi.x()-inici.x();
		double vy=fi.y()-inici.y();
		double inc_r=0.05;
		if(!reves)
			for(double  i=0;i<=1;i=i+inc_r)
				p.push_back(Punt2d(inici.x()+i*vx,inici.y()+i*vy));
		else
			for(double  i=0;i<=1;i=i+inc_r)
				p.push_back(Punt2d(fi.x()-i*vx,fi.y()-i*vy));

	}
	else
	{
		double av;
		if(fabs(ctu_)>1e-5) av=ct_/ctu_;
// 		if(av<0 && av<1) cout<<" av: "<<av;
// 		cout<<"\n"; escriure();
		double x0,x1;
		double epsilon=1e-4;
		double d_max=0.1;
		double inc=0.1;
		double y0,y1;
		bool girat=false;
		
		if(false)
		{
			if(reves){x0=fi.x(); y0=fi.y();x1=inici.y();y1=inici.y();inc=-inc_;}
			else{x0=inici.x(); y0=inici.y(); x1=fi.x(); y1=fi.y();}
		}
		else
		{
			if(inici.x()<=fi.x()){x0=inici.x();x1=fi.x(), y0=inici.y();y1=fi.y();}
			else{x0=fi.x();x1=inici.x(), y0=fi.y();y1=inici.y();girat=true;} 
// 			cout<<"p0,p1: ( "<<x0<<", "<<y0<<") - ("<< x1<<", "<<y1<<") ";
		}
		if(true) // DISCRETITZAR TOTA LA HIPERBOLA INCREMENT X CONSTANT, 79.2273 segons (T PETIT) FUNCIONA
		{
			vector<Punt> pts=pintar();
			unsigned int n=pts.size();
			for(unsigned int l = 0; l<pts.size();l++)
				if(reves) p.push_back(Punt2d(pts[n-l-1].x(),pts[n-l-1].y()));
				else p.push_back(Punt2d(pts[l].x(),pts[l].y()));
			return p;
		}
			
		if(false)  //DISCRETITZAR TOTA LA HIPERBOLA, DES DEL SOURCE FINS AL TARGET controlant la longitud.. MOLT LENT!!! (No funciona)
		{
			
			if(source_.x()<=target_.x()){x0=source_.x(); y0=source_.y();x1=target_.x();y1=target_.y();}
			else{x1=source_.x(); y1=source_.y();x0=target_.x();y0=target_.y();}
			
			double x=x0,y=y0;
			double xa=x,ya=y;
			p.push_back(Punt2d(x0,y0));
// 			epsilon=0.5;
// 			d_max=0.5;
			int n=0;
			while(x<x1&&n<100000)
			{
				n++;
				x=xa+inc;
				y=avaluarX(x);
				if((x-xa)*(x-xa)+(y-ya)*(y-ya)<d_max*5)
					inc=inc*2.;
				else
				{
					inc=inc/2.;
					x=xa+inc;
					y=avaluarX(x);
					int m=0;
					while( (x-xa)*(x-xa)+(y-ya)*(y-ya)>d_max*5 && inc>epsilon&&m<1000)
					{
						m++;
						inc=inc/2.;
						x=xa+inc;
						y=avaluarX(x);
					}
					if(m==1000) cout<<"m";
				}
				p.push_back(Punt2d(x,y));
				xa=x; ya=y;
			}
			if(n==100000) cout<<"n";
			p.push_back(Punt2d(x1,y1));
			
			if(reves)
			{
				vector<Punt2d> pt;
				for(unsigned int  j=p.size();j>0;j--) pt.push_back(p[j-1]);
				return pt;
			}
			return p;
		}
		
		if(false)
		{
			if(x0<0)
			{
				if(y0<0)
				{
					p.push_back(Punt2d(0,0));
					if(x1>0) 
					{
						double y=avaluarX(0);
						if(y>=0 && y<=1){ p.push_back(Punt2d(0,y)); x0=0;y0=y;}
						else if( y>1){ p.push_back(Punt2d(0,1)); x0=0; y0=1;}
						else // y<0
						{
							double x=avaluarY(0);
							if(x>0&&x<1){p.push_back(Punt2d(x,0)); x0=x;y0=0;}
							if(x>1){ p.push_back(Punt2d(1,0)); x0=1; y0=0;}
						}
					}
				}
				else if(y0>1)
				{
					p.push_back(Punt2d(0,1));
					if(x1>0)
					{
						double y=avaluarX(0);
						if(y>=0 && y<=1){ p.push_back(Punt2d(0,y)); x0=0;y0=y;}
						else if( y<0){ p.push_back(Punt2d(0,0)); x0=0; y0=0;}
						else // y>1
						{
							double x=avaluarY(1);
							if(x>0&&x<1){p.push_back(Punt2d(x,1)); x0=x;y0=1;}
							if(x>1){ p.push_back(Punt2d(1,1)); x0=1; y0=1;}
						}
					}
				}
				else
				{
					p.push_back(Punt2d(0,y0));
					if(x1>0)
					{
						double y=avaluarX(0);
						if(y>=0 && y<=1){ p.push_back(Punt2d(0,y)); x0=0;y0=y;}
						else if( y<0){ p.push_back(Punt2d(0,0)); x0=0; y0=0;}
						else if(y>1) {p.push_back(Punt2d(0,1)); x0=0;y0=y;}
					}
				}
			}
			else if(x0>1)
			{
				if(y0<0) p.push_back(Punt2d(1,0));
				else if (y0>1) p.push_back(Punt2d(1,1));
				else p.push_back(Punt2d(1,y0));
			}
			else // x0 esta be. 
			{
				if(y0>=0&&y0<=1) p.push_back(Punt2d(x0,y0));
				else
				{
					if(y0<0)
					{
						p.push_back(Punt2d(x0,0));
						if(y1>0)
						{
							double x=avaluarY(0);
							if(x>x0 && x<x1)
							{ 
								if(x>1){ p.push_back(Punt2d(1,0));}
								else{ p.push_back(Punt2d(x,0));}
								x0=x; y0=0;
							}
						}
					}
					else if(y0>1) p.push_back(Punt2d(x0,1));
					double x=avaluarY(0);
				 }
				
			}
			
		}
		
		if(true)  // DISCRETITZEM NOMES LA PART DE LA HIPERBOLA QUE ESTA DINS EL QUADRAT [0,1] CONSIDERANT ELS EXTREMS SI CAL. 
		{
			if(x0<0)
			{
	 			if(y0>0&&y0<1) p.push_back(Punt2d(0,y0));
				else if(y0>=1) p.push_back(Punt2d(0,1));
				else p.push_back(Punt2d(0,0));
				
				if(x1>0)
				{
					x0=0; y0=avaluarX(x0);
					if(y0<0)
					{
						p.push_back(Punt2d(0,0));
						if(y1>0)
						{
							 y0=0; x0=avaluarY(y0);
							 if(x0<x1) p.push_back(Punt2d(x0,y0));
// 							 else if (x0>x1) p.push_back(Punt2d(x1,0));
// 						/*if(in_t_restricted_range(x0))*/ p.push_back(Punt2d(x0,y0));//if(x0>1) p.push_back(Punt2d(1,0));
						}
					}
					else if(y0>1)
					{
						p.push_back(Punt2d(0,1));
						if(y1<1) 
						{
							y0=1;x0=avaluarY(y0);
						/*if(in_t_restricted_range(x0))*/ p.push_back(Punt2d(x0,y0));//if(x0>1) p.push_back(Punt2d(1,1));
						}
					}
					else /*if(x0>0 && x0<1)*/ p.push_back(Punt2d(x0,y0));
				}
			}
			else if(x0>1)
			{
				if(false)
				{
					/*if(y0<1&&y0>0) p.push_back(Punt2d(1,y0));
					else */if(y0>1) p.push_back(Punt2d(1,1));
					else p.push_back(Punt2d(1,0));
					
					x0=1; y0=avaluarX(x0);
					if(y0<0)
					{
						p.push_back(Punt2d(1,0));
		// 				y0=0;x0=avaluarY(y0);
		// 				if(x0<0) p.push_back(Punt2d(0,0));
		// 				else if(x0>1) p.push_back(Punt2d(1,0));
		// 				else p.push_back(Punt2d(x0,y0));
					}
					else if(y0>1)
					{
						p.push_back(Punt2d(1,1));
		// 				y0=1;x0=avaluarY(y0);
		// 				if(x0<0) p.push_back(Punt2d(0,1));
		// 				else if(x0>1) p.push_back(Punt2d(1,1));
		// 				else p.push_back(Punt2d(x0,y0));
					}
					else p.push_back(Punt2d(1,y0));
				}
			}
			else if(y0<0)
			{
				p.push_back(Punt2d(x0,0));
				if(y1>0)
				{
					y0=0; double x=avaluarY(y0);
					/*if(x<0){ p.push_back(Punt2d(x0,0)); p.push_back(Punt2d(0,0));} // res pq  l'arc que considerem la x creix.
					else*/ if(x>1 ){ p.push_back(Punt2d(x0,0)); p.push_back(Punt2d(1,0));}
					else if( x>0){ p.push_back(Punt2d(x0,y0)); x0=x;}
	// 				else p.push_back(Punt2d(x0,y0));
	// 			       if(in_t_restricted_range(x0))/* if(x0>0+1e-5&&x0<1-1e-5)*/ p.push_back(Punt2d(x0,y0));
				}
// 				else
// 					p.push_back(Punt2d(x0,0));
			}
			else if(y0>1)
			{
				p.push_back(Punt2d(x0,1));
				if(y1<1)
				{
// 					p.push_back(Punt2d(x0,1));
					y0=1;double x=avaluarY(y0);
					if(x0<0){ p.push_back(Punt2d(0,1)); x0=x;}
// 					else if(x0>1) p.push_back(Punt2d(1,1));
					else if(x<1){ p.push_back(Punt2d(x0,y0)); x0=x; }
// 					if(in_t_restricted_range(x0)) p.push_back(Punt2d(x0,y0));
				}
// 				else p.push_back(Punt2d(x0,1));
			}
			else p.push_back(Punt2d(x0,y0));
			
	// 		if(x0>0&&x0<1&&y0<0&&y1<1)  p.push_back(Punt2d(x0,y0));
			
			double x=x0,y=y0;
			double xa=x,ya=y;
			bool continuar=true;
			while(continuar)
			{
				x=xa+inc;
				y=avaluarX(x);
				if((x-xa)*(x-xa)+(y-ya)*(y-ya)<d_max)
					inc=inc*2.;
				else
				{
					inc=inc/2.;
					x=xa+inc;
					y=avaluarX(x);
					while( (x-xa)*(x-xa)+(y-ya)*(y-ya)>d_max && inc>epsilon)
					{
						inc=inc/2.;
						x=xa+inc;
						y=avaluarX(x);
					}
				}
				if(x>0&&x<1&&y>0&&y<1&&x<x1&&y<max(y0,y1))
				{
					p.push_back(Punt2d(x,y));
					xa=x; ya=y;
	// 				if(x0<x1) continuar=(bool)((x<x1));
	// 				else continuar=(bool) ((x>x1));
				}
				else if(fabs(x-av)<1e-5)
				{
					if(y<1) p.push_back(Punt2d(x,1));
					else if (y<0) p.push_back(Punt2d(x,0));
					while(x<x1&&x<1&&(y>1||y<0))
					{
						x=x+0.1;
						y=avaluarX(x);
						if(y>1) p.push_back(Punt2d(x,1));
						else if (y<0) p.push_back(Punt2d(x,0));
						else p.push_back(Punt2d(x,y));
					}
					xa=x; ya=y;
				}
				else continuar=false;
	
			}
			
			if(x1<0)
			{
				if(y1<0)p.push_back(Punt2d(0,0));
				else if (y1>1) p.push_back(Punt2d(0,1));
				else p.push_back(Punt2d(0,y1));
				
	// 			x1=0;y1=avaluarX(0);
	// 			if(y1<0) p.push_back(Punt2d(0,0));
	// 			else if(y1>1) p.push_back(Punt2d(0,1));
	// 			else p.push_back(Punt2d(x1,y1));
			}
			else if(x1>1)
			{
				if(x0<1) p.push_back(Punt2d(1,avaluarX(1)));
				
				if(y1<0)p.push_back(Punt2d(1,0));
				else if (y1>1) p.push_back(Punt2d(1,1));
				else p.push_back(Punt2d(1,y1));
				
	// 			x1=1;y1=avaluarX(1);
	// 			if(y1<0) p.push_back(Punt2d(1,0));
	// 			else if(y1>1) p.push_back(Punt2d(1,1));
	// 			else p.push_back(Punt2d(x1,y1));
			}
			else
			{
				if(y1<0) p.push_back(Punt2d(x1,0));
				else if(y1>1) p.push_back(Punt2d(x1,1));
				else p.push_back(Punt2d(x1,y1));
			}
			
// 			for(unsigned int k=0;k<p.size();k++)
// 				if(p[k].x()>1) cout<<" ("<<p[k].x()<<", "<<p[k].y()<<") ";
		}
			
		
		if(reves!=girat)
		{
			vector<Punt2d> pa;
			for(unsigned int k=p.size();k>0;k--)
				pa.push_back(p[k-1 ]);
			return pa;
		}
		
	}
	
	return p;
	if(false)
	{
		// eq: x²+y²=a²  x=a cosh t, y = a sinh t;   --------> uv=a²
		// u=x+y, v=x-y, u=(x+y)/2, v=(x-y)/2
		
		
		cout<<"\n"<<endl;
		escriure();
// 		cout<<"i: ("<<inici.x()<<", "<<inici.y()<<")"<<endl;
		double b=cu_/ctu_;
		double c=ct_/ctu_;
		double d=cc_/ctu_;
		double a=b*c-d;
		cout<<"b "<<b<<" c "<<c<<" d "<<d<<" a2 "<<a<<" a "<<sqrt(fabs(a));
		double u=1,v=1;
		if(a<0) u=-1;
		a=sqrt(fabs(a));
// 		double x0=u*(inici.x()+b);
// 		double x1=u*(fi.x()+b);
// 		double y0=inici.y()+c;
// 		double y1=fi.y()+c;
// 		double  u0= (x0+y0)/2.;
// 		double v0=(x0-y0)/2.;
// 		double  u1= (x1+y1)/2.;
// 		double v1=(x1-y1)/2.;
		double t0,t1,tm;
// 		if(u0<0) u=-1*u;
// 		if(v0<0) v=-1;
		double u0=(u*(inici.x()+b)+(inici.y()+c))/(2.*a);
		double u1=(u*(fi.x()+b)+(fi.y()+c))/(2.*a);
		cout<<" (u0, u1)= ("<<u0<<", "<<u1<<") ";
		if(u0<0 && u1<0) v=-1;
		else if(u0<0 ||u1<0) cout<<" arc d'hiperbola te dues branques, u<0 i u>0"<<endl;
		
		if(fabs(u*u0-u*u1)<1e-5)
		{
			cout<<" u ";
// 			cout<<" p0' "<<u*(p0+b)/a<<" p1' "<<u*(p1+b)/a<<endl;
			t0=acosh(v*u0);
			t1=acosh(v*u1);
		}
		else
		{
// 			cout<<" p0 "<<v*((p0+c)/a)<<" p1 "<<v*((p1+c)/a);
			t0=asinh((u*(inici.x()+b)-(inici.y()+c))/(2.*a));
			t1=asinh((u*(fi.x()+b)-(fi.y()+c))/(2.*a));
		}
		cout<<" t: ("<<t0<<", "<<t1<<")";
		double inc=fabs(t0-t1)/20.;
		int iter=20;
		if(inc<TOLERANCE){ inc=inc_; iter=ceil(fabs((t0-t1)/inc));}
		double ut,vt,t;
		
		if(!reves)
			if(t0>t1) inc=-inc;
		else
		{
			t=t0; t0=t1;t1=t;
			if(t0<t1) inc=-inc;
		}
		t=t0;
		for(int i=0;i<iter;i++)
		{
			t=t+inc;
			ut=v*cosh(t);
			vt=sinh(t);
			cout<<"( "<<u*a*(ut+vt)-b<<", "<<a*(ut-vt)-c<<") ";
			p.push_back(Punt2d((ut+vt)-b,(ut-vt)-c));
		}
			
		if(reves)p.push_back(inici);
		else p.push_back(fi);
		
		
	/*	
		cout<<"\n"<<endl;
		escriure();
// 		cout<<"i: ("<<inici.x()<<", "<<inici.y()<<")"<<endl;
		double b=cu_/ctu_;
		double c=ct_/ctu_;
		double d=cc_/ctu_;
		double a=b*c-d;
		cout<<"b "<<b<<" c "<<c<<" d "<<d<<" a2 "<<a<<" a "<<sqrt(fabs(a));
		double u=1,v=1;
		if(a<0) u=-1;
		a=sqrt(fabs(a));
// 		double x0=max(0,min( inici.x(),fi.x()));
// 		double x1=min(1,max(inici.x(),fi.x()));
		if(u*(inici.x()+c)<0) u=-1*u;
		if(inici.y()+b<0) v=-1;
// 		double x0=min( inici.x(),fi.x());
// 		double x1=max(inici.x(),fi.x());
		double p0=inici.y();
		double p1=fi.y();
// 		double x0=max(0,min( inici.x(),1));
// 		double x1=max(0,min( fi.x(),1));
		double t0,t1,tm;
		
		if(fabs(p0-p1)<1e-5)
		{
			cout<<" x ";
// 			x0=max(0,min( inici.y(),1));
// 			x1=max(0,min( fi.y(),1));
			p0= inici.x();
			p1=fi.x();
			cout<<" p0' "<<u*(p0+b)/a<<" p1' "<<u*(p1+b)/a<<endl;
			t0=acosh(fabs((p0+b)/a));
			t1=acosh(fabs((p1+b)/a));
		}
		else
		{
			cout<<" p0 "<<v*((p0+c)/a)<<" p1 "<<v*((p1+c)/a);
			t0=asinh(fabs((p0+c)/a));
			t1=asinh(fabs((p1+c)/a));
		}
		cout<<"("<<t0<<", "<<t1<<")";
// 		double inc=max(inc_,fabs(t0-t1)/20.);
		double inc=fabs(t0-t1)/20.;
		if(inc<TOLERANCE) inc=inc_;
		if(!reves)
			if(t0<t1)
				for(double t=t0;t<t1;t=t+inc)
					p.push_back(Punt2d(u*a*cosh(t)-c,v*a*sinh(t)-b));
			else //t0>t1
				for(double t=t0;t>t1;t=t-inc)
					p.push_back(Punt2d(u*a*cosh(t)-c,v*a*sinh(t)-b));
		else
			if(t0<t1)
				for(double t=t1;t>t0;t=t-inc)
					p.push_back(Punt2d(u*a*cosh(t)-c,v*a*sinh(t)-b));
			else //t0>t1
				for(double t=t1;t<t0;t=t+inc)
					p.push_back(Punt2d(u*a*cosh(t)-c,v*a*sinh(t)-b));
					
		if(reves)p.push_back(inici);
		else p.push_back(fi);*/
	}
	
// 	for(unsigned int i=0;i<p.size();i++)
// 		cout<<"("<<p[i].x()<<", "<<p[i].y()<<") ";
	return p;
}

/*
 Regio Hiperbola::regio(const Hiperbola  &h,const  Punt2d &p,const Punt2d &psigne,bool comprovarSigne) const
// Regio Hiperbola::regio(Hiperbola h,Punt2d p, Punt2d psigne,bool comprovarSigne) const
{
	Hiperbola aux_hip1,aux_hip2;
	if(fabs(h.avaluarPunt(source())<TOLERANCE))  // source_ es de h, pertant el pt comu es el source
	{
		aux_hip1=Hiperbola(h.ctu(), h.ct(), h.cu(), h.cc(), source(), p);
		aux_hip2=Hiperbola(ctu(),ct(),cu(),cc(),source(),p);
	}
	else 
	{
		aux_hip1=Hiperbola(h.ctu(), h.ct(), h.cu(), h.cc(), target(), p);
		aux_hip2=Hiperbola(ctu(),ct(),cu(),cc(),target(),p);
	}
	if(comprovarSigne)
	{  
		if(!(aux_hip1.in_t_range(psigne.x())&&aux_hip1.in_u_range(psigne.y())))
		{
			aux_hip1.canviarSigne();
			aux_hip2.canviarSigne();
		}
	}
	
	return Regio(aux_hip1, aux_hip2);
}
	

 Regio Hiperbola::regio(const Hiperbola &h1,const Hiperbola &h2,const Punt2d &p, const Punt2d  &psigne, bool comprovarSigne) const
// Regio Hiperbola::regio(Hiperbola h1, Hiperbola h2, Punt2d p, Punt2d  psigne) const
{
	Hiperbola aux_hip,aux_hip1,aux_hip2(h2);
	if(fabs(avaluarPunt(h2.source())<TOLERANCE))  // h2.source() es de l'hiperbola actual, per tant h2.target() es de l'altre
	{
		aux_hip=Hiperbola(ctu(),ct(),cu(),cc(), h2.source(), p);
		aux_hip1=Hiperbola(h1.ctu(),h1.ct(),h1.cu(),h1.cc(),h2.target(),p);
	}
	else // h2.target() es de l'hiperbola actual, per tant h2.source() es de l'altre
	{
		aux_hip=Hiperbola(ctu(),ct(),cu(),cc(), h2.target(), p);
		aux_hip1=Hiperbola(h1.ctu(),h1.ct(),h1.cu(),h1.cc(),h2.source(),p);
	}
	
	if(comprovarSigne)
		if(!( (aux_hip.in_t_range(psigne.x()) ||aux_hip1.in_t_range(psigne.x()) ) && (aux_hip.in_u_range(psigne.y())||aux_hip1.in_u_range(psigne.y()))))
		{
			aux_hip.canviarSigne();
			aux_hip1.canviarSigne();
			aux_hip2.canviarSigne();
		}
	
	return Regio(aux_hip, aux_hip1, aux_hip2);
}*/
