#ifndef _HIPERBOLA
#define _HIPERBOLA

#include <iostream.h>
#include <qgl.h>
#include "punt.h"
#include <vector.h>
#include "vector3D.h"
#include "geometriabasica.h"

class Segment;
class Regio;

class Hiperbola
{
	private:
		double ctu_,ct_,cu_,cc_;
		Punt2d source_,target_;
		int assimp_t_,assimp_u_;
	public:
		Hiperbola();
		~Hiperbola();
		Hiperbola(const double &ca,const double &cb,const double &ch,const double &ck,const Punt2d &p,const Punt2d &q);
		//construccio copia
		Hiperbola(const Hiperbola &hyp);
		Hiperbola(const Segment3D &s, Segment ss, Segment sv);
		Hiperbola(double a, double b, double c, double d);
		Hiperbola(const Punt &p, const Segment &s, const Segment &v);
		//Hiperbola &operator= (const Hiperbola &hyp);

		double avaluarX(double valor_x)const;
		double avaluarY(double valor_x)const;
		double avaluarPunt(Punt2d p)const;
		double rangX()const;
		double rangY()const;
		double getXAssimptota()const;
		void pintarRegio(bool dalt,double lx, double ly)const;
// 		void pintar()const;
		vector<Punt> pintar()const;
		void pintarNoDiscretitzar(double lx, double ly)const;
		bool rectaVertical()const;
		bool rectaHoritzontal()const;
		void escriure()const;
		
		bool as_t() const;
		bool as_u() const;

		const double &ctu() const;
		const double &ct() const;
		const double &cu() const;
		const double &cc() const;
		const  Punt2d & source() const;
		const  Punt2d & target() const;

		bool is_fu()const; // cert si es funcio de u t=f(u) en tot l'interval
		bool is_ft()const;  // cert si es funcio de t u=f(t) en tot l'interval
		bool is_fu_r()const;  // cert si es funcio de u t=f(u) en l'interval restringit a [0,1]
		bool is_ft_r()const;  // cert si es funcio de t u=f(t) en l'interval restringit a [0,1]

		Punt2d p_u(const double & t)const;    // trobar el punt de la hiperbola a partir de la coordenada t 
		Punt2d p_t(const double & u)const;  // trobar el punt de la hiperbola a partir de la coordenada u
		double u(const double & t)const;   // retorna la coordenada u del  punt de la hiperbola a partir de la coordenada t 
		double t(const double & u)const;   // retorna la coordenada t del  punt de la hiperbola a partir de la coordenada u

		double t_min()const;
		double t_max()const;

		double u_min()const;
		double u_max()const;

		bool u_funcio_t_en(const double & t)const; //  la hiperbola té una unica imatge pel valor de t (puc posar u en funció de t en t=t)
		bool t_funcio_u_en(const double & u)const;  //  la hiperbola té una unica imatge pel valor de u (puc posar t en funció de u en u=u)
		
		void canviar_source_u(const double &u);
		void canviar_target_u(const double &u);
		void canviar_source(const Punt2d &p);
		void canviar_target(const Punt2d &p);
		
		bool operator==(const Hiperbola &hyp)const;
		bool operator!=(const Hiperbola &hyp)const;

		bool interseccioAillarTRR(const Hiperbola &h, vector<Punt2d> &p)const;
		bool interseccioAillarURR(const Hiperbola &h, vector<Punt2d> &p)const;
		vector<Punt2d> intersection_no_end_points(const Hiperbola &hyp) const;
		vector<Punt2d> interseccioNoExtrems(const Hiperbola &hyp)const;
		vector<Punt2d> intersection(const Hiperbola &hyp) const;
		vector<Punt2d> int_ailla_t(const Hiperbola &hyp_, const Hiperbola &hyp) const;
		vector<Punt2d> int_ailla_u(const Hiperbola &hyp_, const Hiperbola &hyp)const;
		
// 		vector<Punt2d> intersection(const Hiperbola &hyp,bool rangGran=false)const;
// 		vector<Punt2d> intersection_no_end_points(const Hiperbola &hyp,bool rangGran=false)const;
		bool interseccio(const Hiperbola &hyp,vector<Punt2d> &p,bool dinsQuadratUnitat=true, bool extrems=false,bool rangGran=false)const;

		bool in_u_range(const double &d)const;
		bool in_t_range(const double &d)const;
		bool in_u_open_range(const double &d)const;
		bool in_t_open_range(const double &d)const;

		bool in_u_restricted_range(const double &d)const;
		bool in_t_restricted_range(const double &d)const;
		
		bool in_range(double primer, double segon, double tercer, bool restringit)const;
		bool in_open_range(double primer, double segon, double tercer, bool restringit)const;

		vector<double> t_range()const;
		vector<double> u_range()const;
		
		bool possible(const Hiperbola &h,const Hiperbola &hh)const;
		bool is_point_(const double &ctu,const double &ct,const double &cu,const double &cc,const Punt2d &p)const;
		bool is_point(const Punt2d &p)const;
		
		bool esPunt()const;
		
// 		vector<Punt2d> int_ailla_t(const Hiperbola &hyp,bool rangGran=true)const;
// 		vector<Punt2d> int_ailla_u( const Hiperbola &hyp,bool rangGran=true)const;
		
		Punt2d skew_point(const Punt3d &p, Segment3D ss, Segment3D sv)const;
		vector<double> skew_segment(const Segment3D &s, Segment3D ss)const;
		
		void canviarSigne();
		
		Regio regio(const Hiperbola &h, const Punt2d &comu, const Punt2d &altre, bool comprovarSigne=true) const;
		Regio regio(const Hiperbola &h, const Punt2d &p, bool cap, bool comprovarSigne=true) const;
		Regio regio(const Hiperbola &h, bool comprovarSigne=true) const;
		Regio regio(const Hiperbola &h1, const Hiperbola &h2, bool comprovarSigne=true)const;
		
		int hiperbolaRangMaxim(const Hiperbola &h1,const Hiperbola &h2)const;
		int hiperbolaRangYMaxim(const Hiperbola &h1,const Hiperbola &h2)const;
		
		bool degenerada() const;
		bool degenerada(char &c) const;
		
		void setExtrems(double x_max, double y_max);
		
		vector<Punt2d> mostrejar(const double &inc,const Punt2d&inici, const Punt2d&fi, const bool & reves)const;
		
// 		Regio regio(const Hiperbola & h, const Punt2d &p, const Punt2d &psigne, bool comprovarSigne=true)const;
// 		Regio regio(const Hiperbola & h1, const Hiperbola & h2, const Punt2d &p, const Punt2d  &psigne, bool comprovarSigne=true)const;
		/*Regio regio( Hiperbola h, Punt2d p) const;
		Regio regio(Hiperbola  h, Punt2d p, Punt2d  psigne)const;
		Regio regio(Hiperbola  h1, Hiperbola h2,  Punt2d p, Punt2d psigne)const;
		*/
};

ostream & operator<<( ostream &os, const Hiperbola &hyp );

#endif

