#include "linia.h"
#include "punt.h"

/**************************************/
/*   Linia::Linia()                   */
/*Constructor per defecte de la classe*/
/**************************************/

Linia::Linia()
{}

/*****************************************/
/*   Linia::Linia(Line)                  */
/*Constructor amb parametres de la classe*/
/*****************************************/

Linia::Linia(Line l)
{
	linia_cgal = l;
}

/*********************************/
/*   Linia::~Linia()             */
/*Destructor de la classe Linia  */
/*********************************/

Linia::~Linia() {}

/************************************/
/*   Line2D Linia::on_xy()          */
/*Metode que retorna la linia amb 2d*/
/************************************/

Line2D Linia::on_xy() const
{
	
	Punt2d p(linia_cgal.point(1).x(),linia_cgal.point(1).y());
	Vector2D v(linia_cgal.to_vector().x(),linia_cgal.to_vector().y());
	Line2D l2(p,v);
	return l2;
}

/******************************/
/*   Line Linia::getLinia()   */
/*Metode que retorna la linia */
/******************************/

const Line & Linia::getLinia()const 
{
	return linia_cgal;
}

/**********************************************************************************************/
/*   Punt3d Linia::on_l(const Punt3d2 &p)                                                       */
/*El punt p esta sobre la projeccio de l sobre pla xy, retorna la coordenada z pq punt sobre l*/
/**********************************************************************************************/

Punt3d Linia::on_l(const Punt2d &p) const
{
	Punt3d p1(p.x(),p.y(),0.);
	Punt3d pt;
	
	if(linia_cgal.has_on ( p1)) return p1;
	else
	{
		CGAL::Object result;
		result=CGAL::intersection(linia_cgal.perpendicular_plane(p1) ,linia_cgal);
		if (CGAL::assign(pt, result)) return pt;
		else{ qDebug("Error on_l"); return p1; }
	}
}
/**************************************************************************************************************************/
/*   double Linia::punt_sobre(const Punt &p)                                                                        */
/*Metode que retorna cert si el punt p esta a sobre la lina */
/**************************************************************************************************************************/

bool Linia::punt_sobre(const Punt &p) const
{
	return punt_sobre(p.getPunt());
}

/**************************************************************************************************************************/
/*   double Linia::punt_sobre(const Punt3d &p)                                                                        */
/*Metode que retorna cert si el punt p esta a sobre la lina */
/**************************************************************************************************************************/

bool Linia::punt_sobre(const Punt3d &p) const
{

	Punt3d source=linia_cgal.point(0);
	
	if(linia_cgal.is_degenerate ())
		return p==source;
	else
	{
		double lambda;
		int i0=0;
		bool colineal = true;
		Vector u(source,linia_cgal.point(1));
		while(i0<3 && fabs(u.cartesian(i0))<TOLERANCE )
			i0++;
		
		lambda=(p.cartesian(i0)-source.cartesian(i0))/(u.cartesian(i0));
		for(int j=0;j<2;j++)
			if(fabs(p.cartesian(j)-(source.cartesian(j)+lambda*u.cartesian(j)))>TOLERANCE)
				colineal=false;
		return colineal;
	}
}
