#include "punt.h"
#include "pla.h"
#include "segment.h"
#include "triangle.h"


/***************************************************************************************/
/*   Pla::Pla()                                                                        */
/*Constructor que inicialitza cadascuna de les variables globals de l'objecte pla creat*/
/***************************************************************************************/

Pla::Pla()
{
	vector1.modifica_coordenades(0.0, 0.0, 0.0);
	vector2.modifica_coordenades(0.0, 0.0, 0.0);
}

/***********************************************************************************/
/*   Pla::Pla(Vector3D, Vector3D)                                                  */
/*Constructor que inicialitza el pla a partir dels 2 vectors passats per parametres*/
/***********************************************************************************/

Pla::Pla(Vector3D v1, Vector3D v2)
{
	vector1 = v1;
	vector2 = v2;
	Vector3D prod = v2.producte_vectorial(v1);
	
	pla_cgal = Plane(Punt3d(v1.x(),v1.y(), v1.z()), Vector(prod.x(), prod.y(), prod.z()));
}
/*****************************/
/*   Pla::~Pla()             */
/*Destructor de la classe Pla*/
/*****************************/

Pla::~Pla() {}

/***********************************************************/
/*   Pla::Pla(Plane)                                       */
/*Constructor que inicialitza el pla a partir d'un pla CGAL*/
/***********************************************************/

Pla::Pla(Plane p)
{	
	pla_cgal = p;
	vector1 = Vector3D(p.base1().x(), p.base1().y(), p.base1().z());
	vector2 = Vector3D(p.base2().x(), p.base2().y(), p.base2().z());
}

/********************************************************/
/*   Pla::Pla(Punt, Punt, Punt)                         */
/*Constructor que inicialitza el pla a partir de 3 punts*/
/********************************************************/

Pla::Pla(Punt a, Punt b, Punt c)
{
	vector1.modifica_coordenades( a.x()-b.x(), a.y()-b.y(), a.z()-b.z());
	vector2.modifica_coordenades( c.x()-b.x(), c.y()-b.y(), c.z()-b.z());
	
	pla_cgal = Plane(Punt3d(a.x(),a.y(), a.z()), Punt3d(b.x(), b.y(), b.z()), Punt3d(c.x(), c.y(), c.z()));
}

Pla::Pla(double a,double b,double c,double d)
{
	pla_cgal=Plane(a,b,c,d);
}

/********************************/
/*   Plane Pla::getPla()        */
/*Mètode que retorna el pla cgal*/
/********************************/

const Plane & Pla::getPla()const
{
	return pla_cgal;
}

/*******************************************************************/
/*   Vector3D Pla::getVector1()                                    */
/*Mètode que permet obtenir un dels 2 vectors que defineixen el pla*/
/*******************************************************************/

const Vector3D & Pla::getVector1()const
{
	return vector1;
}

/**************************************************************/
/*   Vector3D Pla::getVector2()                               */
/*Mètode que permet obtenir l'altre vector que defineix el pla*/
/**************************************************************/

const Vector3D & Pla::getVector2()const
{
	return vector2;
}

/******************************************************************/
/*   void Pla::setVector1()                                       */
/*Mètode que permet modificar el primer vector que defineix el pla*/
/******************************************************************/

void Pla::setVector1(Vector3D v1)
{
	vector1 = v1;
}

/******************************************************************/
/*   void Pla::setVector2()                                       */
/*Mètode que permet modificar el segon vector que defineix el pla */
/******************************************************************/

void Pla::setVector2(Vector3D v2)
{
	vector2 = v2;
}

/****************************************************************************************************/
/*   void Pla::puntPertanyPla(Punt)                                                                 */
/*Mètode que permet determinar si el punt passat per paràmetres es troba contingut en el pla definit*/
/****************************************************************************************************/

bool Pla::puntPertanyPla(Punt p)const
{
	return fabs(getPla().a()*p.x()+getPla().b()*p.y()+getPla().c()*p.z()+getPla().d())<1e-5;	
	
	
}

/********************************************************************************/
/*   void Pla::es_vertical()                                                    */
/*Metode que permet calcular la Z del punt actual a partir del pla que el conte */
/********************************************************************************/

bool Pla::es_vertical(Punt centre)const
{
	Vector3D prod_vectorial, p1;
	
	p1.modifica_coordenades(centre.x(), centre.y(), centre.z());
	prod_vectorial = vector1.producte_vectorial(vector2);
	
// 	qDebug("Producte vectorial: Z:%f", prod_vectorial.z());
	if(prod_vectorial.z() <TOLERANCE && prod_vectorial.z() > -TOLERANCE)
	{
		return true;
	}
	return false;
}


void Pla::escriure()const
{
	cout<<" Pla "<<getPla().a()<<"x + "<<getPla().b()<<"y + "<<getPla().c()<<"z + "<<getPla().d()<<"=0"<<endl;
}
/*******************************************************************************************/
/*   void Pla::calcula_Z(Punt, Punt)                                                       */
/*Metode que permet calcular la Z del punt actual a partir d'un punt del pla i del punt 2d */
/*******************************************************************************************/

double Pla::calcula_Z(Punt punt2d, Punt centre)const
{
	Vector3D prod_vectorial, p1;
	double d, coord_z;
	
	p1.modifica_coordenades(centre.x(), centre.y(), centre.z());
	prod_vectorial = vector1.producte_vectorial(vector2);
	
	d = -(prod_vectorial.producteEscalar(p1));
	
	if(prod_vectorial.z() >TOLERANCE || prod_vectorial.z() < -TOLERANCE)
	{
		coord_z = -(prod_vectorial.x()*punt2d.x() + prod_vectorial.y()*punt2d.y() + d) / prod_vectorial.z();
	}
	
	return coord_z;
}

Punt Pla::puntSobrePla(double x, double y, Punt centre)const
{
	Punt punt2d(x, y, 0.);
	punt2d.modifica_z(calcula_Z(punt2d, centre));
	return punt2d;
}

/*******************************************************************************************/
/*   Punt3d Pla::on_plane(const Punt3d &p)                                                   */
/*Metode que retorna el punt amb x, y igual a les de p, pero amb z perque estigui sobre Pla*/
/*******************************************************************************************/

Punt3d Pla::on_plane_3d(const Punt3d &p) const
{
	Punt3d pt;
	if(pla_cgal.c()==0) cout<<"error, coeficient de z en el pla=0, no es pot trobar 1! punt donades x,y"<<endl;
	else
	{
		Punt3d aux(p.x(),p.y(),(-pla_cgal.d()-p.x()*pla_cgal.a()-p.y()*pla_cgal.b())/pla_cgal.c());
		pt=aux;
	}
	return pt;
}

/*******************************************************************************************/
/*   Punt3d Plaon_plane(const Punt2d &p)                                                    */
/*Metode que retorna el punt amb x, y igual a les de p, pero amb z perque estigui sobre Pla*/
/*******************************************************************************************/

Punt3d Pla::on_plane_2d(const Punt2d &p) const
{
	Punt3d pt(p.x(),p.y(),0);
	return on_plane_3d(pt);
}

/**************************************************/
/*   double Pla:distancia(const Punt3d &p)         */
/*Metode que retorna la distancia d'un punt al pla*/
/**************************************************/

double Pla::distancia(const Punt3d &p) const
{
	return (pla_cgal.a()*p.x()+pla_cgal.b()*p.y()+pla_cgal.c()*p.z()+pla_cgal.d())/sqrt(pla_cgal.a()*pla_cgal.a()+pla_cgal.b()*pla_cgal.b()+pla_cgal.c()*pla_cgal.c());
}

/***********************************************************************/
/*   bool Pla::different_side(const Punt3d &p, const Punt3d &q)          */
/*Metode que informa si els punts es troben a diferents costats del pla*/
/***********************************************************************/

bool Pla::different_side(const Punt3d &p, const Punt3d &q) const
{
	return !mateixCostat(p,q);
	return pla_cgal.has_on(p)||pla_cgal.has_on(q) || (pla_cgal.has_on_positive_side(p)&&pla_cgal.has_on_negative_side(q)) || (pla_cgal.has_on_positive_side(q)&&pla_cgal.has_on_negative_side(p));
}


/***********************************************************************/
/* bool Pla::mateixCostat(const Punt3d &p,const Punt3d&q)const  */
/*Metode que informa si els punts es troben al mateix costat del pla*/
/***********************************************************************/

bool Pla::mateixCostat(const Punt3d &p,const Punt3d&q,bool sobre)const
{
	double dp=pla_cgal.a()*p.x()+pla_cgal.b()*p.y()+pla_cgal.c()*p.z()+pla_cgal.d();
	double dq=pla_cgal.a()*q.x()+pla_cgal.b()*q.y()+pla_cgal.c()*q.z()+pla_cgal.d();
	bool mc;
	if(sobre) mc=(dp*dq>=0 || fabs(dq)<TOLERANCE*100. || fabs(dp)<TOLERANCE*100.); //|| fabs(dp*dq)<TOLERANCE;
	else
		mc=(dp*dq>TOLERANCE/100. && fabs(dp)>TOLERANCE/100. && fabs(dq)>TOLERANCE/100.);
	
	return mc;
	//return pla_cgal.has_on_positive_side(p)==pla_cgal.has_on_positive_side(q);
}

/***********************************************************************/
/*                      bool Pla::interseccio(const Segment &s, bool extrems)const                                  */
/*               bool Pla::interseccio(const Segment &s, Punt3d &q,bool extrems)const                          */
/*       bool Pla::interseccio(const Segment &s, Punt3d &q, bool &seg,bool extrems)const               */
/*bool Pla::interseccio(const Segment &s_, Punt3d &q, bool &seg,double &h,bool extrems)const   */
/* Metode que retorna cert si el segment s interseca amb el pla.                                                                                                             */
/*    extrems= true  es tenen en compte els extrems del segment (si segment s interseca anb el pla a un extrem de s retorna true) */
/*    Punt3d q es un parametre de sortida, retorna el punt itnerseccio del pla anb s                                                                                  */
/*    bool seg es un boleà q prendra el valor de cert si s esta contingut al pla (si la interseccio es un segmet)                                     */
/***********************************************************************/
bool Pla::interseccio(const Segment &s, bool extrems)const
{
// 	 Punt3d q;
// 	double h;
// 	bool seg;
// 	return interseccio(s,q,seg,h,extrems);
	
// 	cout<<"s"<<endl;
	double aval_origen=avaluarPunt(s.getOrigen());
	double aval_desti=avaluarPunt(s.getDesti());
	if(extrems)
		return (fabs(aval_origen)<TOLERANCE||fabs(aval_desti)<TOLERANCE|| aval_origen*aval_desti<0);
	
	return ( !fabs(aval_origen)<TOLERANCE && !fabs(aval_desti)<TOLERANCE && aval_origen*aval_desti<0);
}

bool Pla::interseccio(const Segment &s, Punt3d &q,bool extrems)const
{
	double h;
	bool seg;
	return interseccio(s,q,seg,h,extrems);
}

bool Pla::interseccio(const Segment &s, Punt3d &q, bool &seg,bool extrems)const
{
	double h;
	return interseccio(s,q,seg,h,extrems);
}
bool Pla::interseccio(const Segment &s, double &h, bool extrems)const
{
	Punt3d q;
	bool seg;
	return interseccio(s,q,seg,h,extrems);
}
/*    q i seg son parametres de retorn seg=true si la interseccio es un segment, a q hi ha el/un punt d'interseccio.  */
/*    si extrems= false si el pt interseccio es un extrem de s_ retorna fals, pero esta posat a q. */

bool Pla::interseccio(const Segment &s_, Punt3d &q, bool &seg,double &h,bool extrems)const
{
	seg=false;
	Vector u(s_.getSegment_cgal().to_vector());
	Punt3d p=s_.getSegment_cgal().source();
	Vector w(getPla().point(),p);
	if(w*w<TOLERANCE){ p=s_.getSegment_cgal().target(); w=Vector(getPla().point(),p); }
	Vector n=getPla().orthogonal_vector();
	
	double D=n*u;
	double N=-n*w;
	if(fabs(D)<TOLERANCE)
		if(N==0){ seg=true; q=s_.source(); return true;}
		else return false;
	
	h=N/D;
	
	if(extrems && ( h<-TOLERANCE*100 || h>1+TOLERANCE*100)) return false; // if(extrems) si h==0 o h==1 ha de dir q true	
	if(!extrems &&(h<TOLERANCE*100 || h>1-TOLERANCE*100)) return false;  // if (!extrems) si la interseccio es a un extrem false;
	//if(!extrems &&(fabs(h)<TOLERANCE || fabs(h-1)<TOLERANCE)) return false;  // if (!extrems) si la interseccio es a un extrem false;
	
	q=Punt3d(p.x()+h*u.x(),p.y()+h*u.y(),p.z()+h*u.z());
	return true;
}


/*******************************************************************************************************************/
/*                    bool Pla::interseccio(const Linia &l)const                                                                                 */
/*                   bool Pla::interseccio(const Linia &l, Punt3d &q)const                                                                  */ 
/*                   bool Pla::interseccio(const Linia &l, double &h)const                                                               */
/*                    bool Pla::interseccio (const Linia &l, Punt3d &q, bool &linia)const                                             */                             
/*                    bool Pla::interseccio (const Linia &l_, Punt3d &q,double &h, bool &linia)const                          */                          
/* Metode que retorna cert si el segment s interseca amb el pla.                                                                                                             */
/*    extrems= true  es tenen en compte els extrems del segment (si segment s interseca anb el pla a un extrem de s retorna true) */
/*    Punt3d q es un parametre de sortida, retorna el punt itnerseccio del pla anb s                                                                                  */
/*    bool seg es un boleà q prendra el valor de cert si s esta contingut al pla (si la interseccio es un segmet)                                     */
/***********************************************************************/
bool Pla::interseccio(const Linia &l)const
{
 	Punt3d q;
	return interseccio(l,q);
}

bool Pla::interseccio(const Linia &l, Punt3d &q)const
{
	double h;
	bool linia;
	return interseccio(l,q,h,linia);
}
bool Pla::interseccio(const Linia &l, double &h)const
{
	Punt3d q;
	bool linia;
	return interseccio(l,q,h,linia);
}
bool Pla::interseccio (const Linia &l, Punt3d &q, bool &linia)const
{
	double h;
	return interseccio(l,q,h,linia);
}

bool Pla::interseccio (const Linia &l_, Punt3d &q,double &h, bool &linia)const
{
	Linia l(l_.getLinia());
	linia=false;
	if(fabs(l.getLinia().to_vector()*getPla().orthogonal_vector())<TOLERANCE) // linia paralela al pla o sobre el pla
	{
		h=0;
		if(puntPertanyPla(Punt(l.getLinia().point(0))))
		{ 
			linia=true; 
			q=l.getLinia().point(0); 
			if(!puntPertanyPla(q)) 
				cout<<"ERROR LINIAAAA Interseccio pla!!!!!!!!111"<<endl;
// 			cout<<"I    ";
			return true;
		}
// 		cout<<"NI   ";
		return false;
	}
	else
	{
		Punt3d p=l.getLinia().point(0);
		Vector u(p,l.getLinia().point(1));
		
		Vector w(getPla().point(),p);
		h=-(getPla().orthogonal_vector()*w)/(getPla().orthogonal_vector ()*u);
		Vector vq=u*h;
		q=Punt3d(p.x()+vq.x(),p.y()+vq.y(),p.z()+vq.z()); 
		if(!puntPertanyPla(q))
		{
			cout<<getPla().a()*q.x()+getPla().b()*q.y()+getPla().c()*q.z()+getPla().d();
		 	cout<<"ERROR LINIAAAA Interseccio pla!!!!!!!!222222222222222222222222222222"<<endl;
		}
// 		cout<<"I    ";
		return true;
	}
}


/*******************************************************************************************************************/
/*                                          bool Pla::interseccio (const Triangle &t, Segment &s, bool &triangle,bool extrems)const                        */
/*           Metode que comprova si el triangle t interseca el pla. 
                   Segment s es de retorn:  conte el segment interseccio, en cas de  */
/*                bool triangle  es de retorn: es true si el triangle esta contingut al pla   */
/*                extrems = false => si el triangle interseca amb el pla en un vertex, retorna fals                                                      */
/*********************************************************************************************************************/

bool Pla::interseccio (const Triangle &t, Segment &s, bool &triangle,bool extrems)const
{
	Pla PlaTriangle(t.getTriangle_cgal().supporting_plane());
	Vector n1=PlaTriangle.getPla().orthogonal_vector();
	
	Punt3d p=pla_cgal.point();
	Vector n2=pla_cgal.orthogonal_vector();
	
	Vector n12=CGAL::cross_product(n1,n2);
	triangle=false;
	if(n12*n12<TOLERANCE)  // paral.lels
		if( puntPertanyPla(Punt(t.getTriangle_cgal().vertex(0)))){ triangle=true;   return false; }
		else return false;
	
	int i=0;
	
	while(i<3&&(mateixCostat(t.getTriangle_cgal().vertex(i),t.getTriangle_cgal().vertex(i+1),!extrems) || mateixCostat(t.getTriangle_cgal().vertex(i),t.getTriangle_cgal().vertex(i+2),!extrems)))
		 i++;
		if(i==3)
	{
		triangle=false;
		int j=0;
		while(j<3 &&!puntPertanyPla(Punt(t.getTriangle_cgal().vertex(j)))) j++;
		if(j==3)
			return false;
		
		Segment s1(Punt(t.getTriangle_cgal().vertex(j+1)),Punt(t.getTriangle_cgal().vertex(j+2)));
		Punt3d q;
		bool B;
		if(interseccio(s1,q,B,extrems))
		{
			s=Segment(Punt(t.getTriangle_cgal().vertex(j)),Punt(q));
			return true;
		}
		if(extrems) 
		{
			s=Segment(Punt(t.getTriangle_cgal().vertex(j)),Punt(t.getTriangle_cgal().vertex(j)));
			return true;
		}
		return false;
	}
		
	if(i==3){  cout<<"PROBLEMA"<<endl; return false;}
	Punt3d pi1,pi2;
	bool segment;
	Segment s1(t.getTriangle_cgal().vertex(i),t.getTriangle_cgal().vertex(i+1));
	if(interseccio(s1,pi1,segment,extrems) )
	{
		if(segment){ s=s1; 	return true; }
		Segment s2(t.getTriangle_cgal().vertex(i),t.getTriangle_cgal().vertex(i+2));
		if( interseccio(s2,pi2,segment,extrems) ) 
		{
			if(segment) triangle=true;
			s=Segment(pi1,pi2);	
			return true;
		}
		else
	 	{
			cout<<"i) extrems="<<extrems<< "ERROR: diu 2 costats != i no interseccio"<<endl;
			return false;
		}
	}
	double dp=pla_cgal.a()*s1.getOrigen().x()+pla_cgal.b()*s1.getOrigen().y()+pla_cgal.c()*s1.getOrigen().z()+pla_cgal.d();
	double dq=pla_cgal.a()*s1.getDesti().x()+pla_cgal.b()*s1.getDesti().y()+pla_cgal.c()*s1.getDesti().z()+pla_cgal.d();
	if(fabs(dp)<1e-10&&fabs(dq)<1e-10) return true;
	cout<<"ii)  extrems="<<extrems<<"ERROR: ..... diu 2 costats != i no interseccio"<<dp<<dq<<endl;
	return false;
}


/*
bool Pla::interseccio (const Pla pla,  Linia  &l, bool &iguals)const
{
	Vector n=CGAL::cross_product(getPla().orthogonal_vector(), pla.getPla().orthogonal_vector());
	
	if(fabs(n.x())<TOLERANCE,fabs(n.y())<TOLERANCE,fabs(n.z())<TOLERANCE)
	{
		iguals= getPla()==pla.getPla();
		return iguals;
	}
	else
	{
		
		double max=fabs(n.x());
		int i=0;
		if(fabs(n.y())>max){ max=fabs(n.y()); i=1;}
		if(fabs(n.z())>max){ max=fabs(n.z()); i=2; }
		double x0,y0,z0;
		if(i==0)
		{
			x0=0;
			y0=(getPla().c()*pla.getPla().d()-getPla.d()*pla.getPla().c())/(getPla().b()*pla.getPla().c()-getPla.b()*pla.getPla().b());
			z0=(getPla().d()*pla.getPla().b()-getPla.b()*pla.getPla().d())/(getPla().b()*pla.getPla().c()-getPla.b()*pla.getPla().b());
		}
		else if (i==1)
		{
			x0=(getPla().c()*pla.getPla().d()-getPla.d()*pla.getPla().c())/(getPla().a()*pla.getPla().c()-getPla.c()*pla.getPla().a());
			y0=0;
			z0=(getPla().d()*pla.getPla().a()-getPla.a()*pla.getPla().d())/(getPla().a()*pla.getPla().c()-getPla.c()*pla.getPla().a());
		}
		else
		{
			x0=(getPla().b()*pla.getPla().d()-getPla.d()*pla.getPla().b())/(getPla().a()*pla.getPla().b()-getPla.b()*pla.getPla().a());
			y0=(getPla().d()*pla.getPla().a()-getPla.a()*pla.getPla().d())/(getPla().a()*pla.getPla().b()-getPla.b()*pla.getPla().a());
			z0=0;
		}
		Punt3d p(x0,y0,z0);
		l=Linia(Line(p,n));
		return true;
	}
}
*/

/*****************************************************/
/*    Vector3D Pla::vectorNormal() const      */ 
/* Metode que retorna el vector normal al pla                  */
/*************************************************/

Vector3D Pla::vectorNormal() const
{
	Vector n=getPla().orthogonal_vector ();
	return Vector3D(n.x(),n.y(),n.z());
}

void Pla::canviarSigne()
{
	pla_cgal = Plane(-pla_cgal.a(), -pla_cgal.b(), -pla_cgal.c(), -pla_cgal.d());
}

double Pla::avaluarPunt(Punt p)const
{
	return (pla_cgal.a()*p.x() + pla_cgal.b()*p.y() + pla_cgal.c()*p.z() + pla_cgal.d());
}

