#ifndef PLA_H
#define PLA_H

#include <math.h>
#include <vector>
#include <qcolor.h>
#include "vector3D.h"
#include "linia.h"
#include "segment.h"
//#include "triangle.h"

#include "geometriabasica.h"

class punt;
class Triangle;
/************************************************************************************************/
/*                                        class Pla                                             */
/*   Classe filla de la classe abstracta boundary que implementa els mètodes específics d'un pla*/
/************************************************************************************************/

class Pla 
{
	private:
		Vector3D vector1;
		Vector3D vector2;
		
		Plane pla_cgal;
	public:
		Pla(); /*Constructor per defecte que inicialitza els atributs de l'esfera*/
		~Pla();
		Pla(Vector3D, Vector3D); /*Constructor amb parametres que permet indicar els 2 vectors que defineixen el pla*/
		Pla(Punt, Punt, Punt); /*Constructor amb parametres que permet crear el pla a partir de 3 punts*/
		Pla(Plane);
		Pla(double,double,double,double);
		
		const Plane & getPla()const;
		
		const Vector3D &  getVector1()const;
		const Vector3D & getVector2()const;
		Vector3D vectorNormal() const;
		
		void setVector1(Vector3D);
		void setVector2(Vector3D);
		
		bool puntPertanyPla(Punt)const;
		bool es_vertical(Punt)const;
		double calcula_Z(Punt punt2d, Punt centre)const;
		Punt puntSobrePla(double x, double y, Punt centre)const;
		
		Punt3d on_plane_3d(const Punt3d &p)const;
		Punt3d on_plane_2d(const Punt2d &p)const;
		double distancia(const Punt3d &p)const; 
		bool different_side(const Punt3d &p, const Punt3d &q)const;
		bool mateixCostat(const Punt3d &p,const Punt3d&q,bool sobre=true)const;
		
		bool interseccio(const Segment &s, Punt3d &q, bool &seg,double &h,bool extrems=true)const;
		bool interseccio(const Segment &s, Punt3d &q, bool &seg,bool extrems=true)const;
		bool interseccio(const Segment &s, Punt3d &q, Punt3d puntPla,bool &seg,bool extrems=true)const;
		bool interseccio(const Segment &s, Punt3d &q,bool extrems=true)const;
		bool interseccio(const Segment &s, bool extrems)const;
		bool interseccio(const Segment &s, double &h, bool extrems=true)const;
			
		bool interseccio (const Linia &l, Punt3d &q,double &h, bool &linia)const;
		bool interseccio (const Linia &l, Punt3d &q, bool &linia)const;
		bool interseccio (const Linia &l, Punt3d &q)const;
		bool interseccio(const Linia &l, double &h)const;
		bool interseccio(const Linia &l)const;
	
		bool interseccio(const Triangle &t, Segment &s, bool & tr,bool extrems=true)const;	
		//bool interseccio (const Triangle &t, Segment &s, bool &triangle)const;
		bool interseccio (const Punt3d &p1, const Punt3d &p2, const Punt3d &p3,Segment &s, bool &triangle, bool extrems=true )const;
		//bool interseccio (const Triangle & t, Segment &s, bool &triangle)const;
		
		bool interseccio(const Pla &pla, Linia &l, bool & iguals)const;	
		
		void canviarSigne();
		
		double avaluarPunt(Punt)const;
		void escriure()const;
};

#endif
