#include "poligon.h"
#include "poligon_convex.h"

/****************************************************/
/*   Poligon::Poligon()                             */
/*Constructor per defecte que ho inicialitza tot a 0*/
/****************************************************/

Poligon::Poligon() 
{
	costats=0;
	tipus_objecte = POLIGON_VISIO;
}

/******************************************************************************************/
/*   Poligon::Poligon(vector<Punt> p, int cos)                                            */
/*Constructor amb parametres que permet configurar cadascuna de les propietats del poligon*/
/******************************************************************************************/

Poligon::Poligon(vector<Punt> p, int cos)  
{
	punts=p;
	costats=cos;
	tipus_objecte=POLIGON_VISIO;
}

/**********************************************************/
/*   Poligon_Convex::Poligon_Convex(pCara triangle) */
/*Constructor a partir d'una cara de la triangulacio      */
/**********************************************************/

Poligon::Poligon(pCara triangle)
{
	punts.push_back(Punt(triangle->vertex(0)->point().x(), triangle->vertex(0)->point().y(), triangle->vertex(0)->point().z()));
	punts.push_back(Punt(triangle->vertex(1)->point().x(), triangle->vertex(1)->point().y(), triangle->vertex(1)->point().z()));
	punts.push_back(Punt(triangle->vertex(2)->point().x(), triangle->vertex(2)->point().y(), triangle->vertex(2)->point().z()));
	
	costats=3;
	tipus_poligon=POLIGON_CONVEX;
}

/**********************************************************/
/*   Poligon::Poligon_Convex(Triangle3 triangle)          */
/*Constructor a partir d'una cara de la triangulacio      */
/**********************************************************/

Poligon::Poligon(Triangle3 triangle)
{
	punts.push_back(Punt(triangle.vertex(0).x(), triangle.vertex(0).y(), triangle.vertex(0).z()));
	punts.push_back(Punt(triangle.vertex(1).x(), triangle.vertex(1).y(), triangle.vertex(1).z()));
	punts.push_back(Punt(triangle.vertex(2).x(), triangle.vertex(2).y(), triangle.vertex(2).z()));
	
	costats=3;
	tipus_poligon=POLIGON_CONVEX;
}

/*********************************/
/*   Poligon::~Poligon()         */
/*Destructor de la classe Poligon*/
/*********************************/

Poligon::~Poligon() {}

/******************************************/
/*   int Poligon::retorna_punts() */
/*Metode que retorna els punts del poligon*/
/******************************************/

vector<Punt> Poligon::retorna_punts()const
{
	return punts;
}

/***************************************************/
/*   int Poligon::retorna_costats()      */
/*Metode que retorna el número de punts del poligon*/
/***************************************************/

int Poligon::retorna_costats()const
{
	return costats;
}

/*****************************************************************************/
/*   Punt Poligon::obtenirVertex(int posicio)                                */
/*Metode que retorna el vertex que ocupa la posicio indicada                 */
/*****************************************************************************/

Punt Poligon::obtenirVertex(int posicio)const
{
	return punts[posicio-1];
}

/********************************************************************************************************/
/*   int Poligon::orientacio()                                                                */
/*Metode que retorna la orientacio del poligon segons l'ordre en el qual l'usuari ha introduit els punts*/
/********************************************************************************************************/

int Poligon::orientacio() const
{
// 	double coord1, coord2, coord3;
	double determinant=0;
	Punt p;
	int minim=0, anterior=0;
	Vector3D vector1, vector2;
	int orienta;

	/*Primer cal trobar un vertex convex. Podrem agafar directament un vertex de coordenada minima o maxima, 
	tant de x com de y perque sempre sera convex*/
	for(unsigned int i=1;i<punts.size();i++)
	{
		if(punts[i].x()<punts[minim].x())
		{
			minim=i;
		}
	}

	if(minim==0)
	{
		anterior=punts.size()-1;
	}
	else
	{
		anterior=minim-1;
	}

	vector1.modifica_coordenades((punts[minim].x()-punts[anterior].x()), (punts[minim].y()-punts[anterior].y()), 0.0);
	vector2.modifica_coordenades((punts[(minim+1)%punts.size()].x()-punts[anterior].x()), (punts[(minim+1)%punts.size()].y()-punts[anterior].y()), 0.0);

	determinant=(vector1.x()*vector2.y())-(vector1.y()*vector2.x());
	/*Segons el signe del determinant la orientacio sera positiva(1) o negativa(0)*/
	if(determinant>0)
	{
		orienta=1;
	}
	else
	{
		orienta=0;
	}
	return(orienta);
}

/**********************************************************************************************/
/*   bool Poligon::vertexConvex(Punt A, Punt B, Punt C, int orientacio)             */
/*A partir de tres punts i de la orientacio del poligon determinem si el punt B es o no convex*/
/**********************************************************************************************/

bool Poligon::vertexConvex(Punt A, Punt B, Punt C, int orientacio) 
{
	Vector3D vector1, vector2;
	bool convex=false;
	double determinant=0;

	vector1.modifica_coordenades((B.x()-A.x()), (B.y()-A.y()), 0.0);
	vector2.modifica_coordenades((C.x()-A.x()), (C.y()-A.y()), 0.0);

	determinant=(vector1.x()*vector2.y())-(vector1.y()*vector2.x());

	if(((determinant>0)&&(orientacio==1)) ||((determinant<0)&&(orientacio==0)))
	{
		convex=true;
	}
	return(convex);
}

/****************************************************************************************/
/*   bool Poligon::esquerra(Punt pA, Punt pB, Punt pC)                        */
/*Metode que informa si el punt pC es troba a l'esquerra del segment definit per pA i pB*/
/****************************************************************************************/

bool Poligon::esquerra(const Punt &pA,const Punt &pB, const Punt & pC)const
{
	Vector3D vectAB, vectAC;
	bool esquerra=true;

	vectAB.modifica_coordenades((pB.x()-pA.x()),(pB.y()-pA.y()),0.0);
	vectAC.modifica_coordenades((pC.x()-pA.x()),(pC.y()-pA.y()),0.0);

	if(((vectAB.x()*vectAC.y())-(vectAC.x()*vectAB.y()))<=0)
	{
		esquerra=false;
	}

	return(esquerra);
}

/***********************************************************************************************************************/
/*   bool Poligon::interiorTriangle(Punt p, Punt triangle[3])                                                */
/*A partir d'una terna de punts que defineixen un triangle determinem si el punt "p" es o no interior a aquest triangle*/
/***********************************************************************************************************************/

bool Poligon::interiorTriangle(const Punt &p,  Punt triangle[3]) const
{
	if(p==triangle[0] || p==triangle[1] || p==triangle[2]) /*En el cas que aquest punt "p" sigui algun dels extrems del 
	triangle no el considerarem com a interior*/
	{
		return(false);
	}
	else
	{
		return((esquerra(triangle[0], triangle[1], p) && esquerra(triangle[1], triangle[2], p) && esquerra(triangle[2], triangle[0], p))||
		(!esquerra(triangle[0], triangle[1], p) && !esquerra(triangle[1], triangle[2], p) && !esquerra(triangle[2], triangle[0], p)));
	}
}

/**********************************************************************/
/*   bool Poligon::pertany(Punt p, vector<Punt> llista)     */
/*Metode que indica si el punt "p" es troba dins de la llista indicada*/
/**********************************************************************/

bool Poligon::pertany(const Punt &p, const vector<Punt> &llista) const
{
	bool trobat=false;
	unsigned int i=0;

	while (i<llista.size() && !trobat)
	{
		if(llista[i]==p)
		{
			trobat=true;
		}
		i++;
	}
	return(trobat);
}

/*****************************************************************************************************/
/*   bool Poligon::orella(vector<Punt> llistaConcaus, vector<Punt> llista, int posicio)    */
/*Un triangle es orella quan no conte cap altre punt del poligon en el seu interiorEn aquest cas     */
/*caldra comprovar si existeix un vertex concau a dins del triangle que te com a centre el vertex de */
/*la posicio indicada. No sera orella en els casos seguents: 1.-Si el punt de la posicio de la llista*/
/*es concau,2.- Si algun punt de la llista de concaus cau dins del triangle indicat                  */
/*****************************************************************************************************/

bool Poligon::orella(const vector<Punt> &llistaConcaus, const vector<Punt> & llista, int posicio) const
{
	bool orella=true, trobat=false;
	Punt triangle[3];
	unsigned int i=0;

	if(llistaConcaus.size()!=0)
	{
		if(pertany(llista[posicio], llistaConcaus))
		{
			orella=false;
		}
		else
		{
			while (i<llistaConcaus.size() && !trobat)
			{
				triangle[0]=llista[posicio-1];
				triangle[1]=llista[posicio];
				triangle[2]=llista[posicio+1];
				if(interiorTriangle(llistaConcaus[i], triangle))
				{
					trobat=true;
				}
				i++;
			}
			if(trobat)
			{
				orella=false;
			}
		}
	}
	return(orella);
}

/*******************************************************************************/
/*   vector< Triangle >  Poligon::triangulacio()                 */
/*Aquest metode retorna per parametres els triangles en els quals s'ha dividit */
/*el poligon despres de realitzar el proces de triangulacio                    */
/*******************************************************************************/

void Poligon::triangulacio() 
{
	int orienta; 
	vector<Punt> llistaConcaus, llista;
	
	int n=0, x=0;
	bool trobat=false;
	unsigned int i=0,j=0;
	Punt actual, predecessor;
	
	/*Primerament cal calcular la orientacio del poligon*/
	orienta=orientacio();

	for( i=0; i<punts.size(); i++)
	{
		if(!vertexConvex(punts[i], punts[(i+1)%punts.size()], punts[(i+2)%punts.size()], orienta)) /*En cada iteracio analitzarem al vertex i+1.
		Si aquest no es convex l'introduirem dins de la llista de concaus*/
		{
			llistaConcaus.push_back(punts[(i+1)%punts.size()]);
		}
	}

	i=0;
	llista=punts;
	n=llista.size();
	x=i+2; /*Ens situem sobre el tercer element de la llista de punts que formen el triangle*/

// 	cout<<"Numero de punts: "<<n<<" x: "<<x<<endl;
	while(n>3)
	{
		if(orella(llistaConcaus, llista, x-1))
		{
			/*S'afegeix a la llista el triangle format pels vertexs de les posicions x-2, x-1 i x*/
// 			aux_triangles.push_back(llista[x-2]);
// 			aux_triangles.push_back(llista[x-1]);
// 			aux_triangles.push_back(llista[x]);
			triangles_triangulacio.push_back(Triangle(llista[x-2], llista[x-1], llista[x]));
// 			cout<<"ORELLA "<<x<<endl;
// 			aux_triangles.clear();
			
			n=n-1;
			/*Proces que ens permet eliminar de la llista de punts el que ja hem processat, es a dir, x-1*/
			actual=llista[x];
			predecessor=llista[x-2];
			for( j=x-1;j<llista.size()-1;j++)
			{
				llista[j]=llista[j+1];
			}
			llista.pop_back();
			if(pertany(actual, llistaConcaus ) && vertexConvex(llista[x-2], llista[x-1], llista[x], orienta)) /*En el cas que abans, el punt 
			de la posicio x-1 fos concua cal controlar si actualment ho es o no*/
			{
				/*En el cas que ara no ho sigui cal que l'elimini de la llista de concaus*/
				j=0;
				while((j<llistaConcaus.size())&&(!trobat))
				{
					if(llistaConcaus[j]==actual)
					{
						trobat=true;
						for(unsigned int k=j;k<llistaConcaus.size()-1;k++)
						{
							llistaConcaus[k]=llistaConcaus[k+1];
						}
						llistaConcaus.pop_back();
					}
					j++;
				}
				trobat=false;
			}
			/*A continuacio s'inspecciona el seguent, es a dir, el predecessor, x*/
			if((x>2) && pertany(predecessor, llistaConcaus) && vertexConvex(llista[x-3], llista[x-2], llista[x-1], orienta))
			{
				j=0;
				while((j<llistaConcaus.size())&&(!trobat))
				{
					if(llistaConcaus[j]==predecessor)
					{
						trobat=true;
						for(unsigned int k=j;k<llistaConcaus.size()-1;k++)
						{
							llistaConcaus[k]=llistaConcaus[k+1];
						}
						llistaConcaus.pop_back();
					}
					j++;
				}
				trobat=false;
			}
			if(x>2)
			{
				x--;
			}
		}
		else
		{
			x = (x+1)%n;
		}
	}
	/*Afegeixo l'ultim triangle a la llista de triangles que retornarem*/
// 	aux_triangles.push_back(llista[x-2]);
// 	aux_triangles.push_back(llista[x-1]);
// 	aux_triangles.push_back(llista[x]);
// 	cout<<"FINAL "<<x<<endl;
	triangles_triangulacio.push_back(Triangle(llista[x-2], llista[x-1], llista[x]));
}

/*************************************************************************************/
/*   bool Poligon::interseccioPoligonTriangle3D(Triangle cara)                       */
/*Metode que permet detectar interseccio entre el poligon i el triangle indicat en 3D*/
/*************************************************************************************/

bool Poligon::interseccioPoligonTriangle3D(const Triangle &cara)const
{
	bool interseccio=false;
	unsigned int i=0;
	
// 	cout<<"CARA: (("<<cara.getTriangle_cgal().vertex(0).x()<<", "<<cara.getTriangle_cgal().vertex(0).y()<<", "<<cara.getTriangle_cgal().vertex(0).z()<<") , ("<<cara.getTriangle_cgal().vertex(1).x()<<", "<<cara.getTriangle_cgal().vertex(1).y()<<", "<<cara.getTriangle_cgal().vertex(1).z()<<") , ("<<cara.getTriangle_cgal().vertex(2).x()<<", "<<cara.getTriangle_cgal().vertex(2).y()<<", "<<cara.getTriangle_cgal().vertex(2).z()<<") )"<<endl;
// 	
	while(i<triangles_triangulacio.size() && !interseccio)
	{
// 		cout<<"interseccioPoligonTriangle3D: "<<triangles_triangulacio.size()<<endl;
// 		cout<<"TRIANGLE: (("<<triangles_triangulacio[i].getTriangle_cgal().vertex(0).x()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(0).y()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(0).z()<<") , ("<<triangles_triangulacio[i].getTriangle_cgal().vertex(1).x()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(1).y()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(1).z()<<") , ("<<triangles_triangulacio[i].getTriangle_cgal().vertex(2).x()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(2).y()<<", "<<triangles_triangulacio[i].getTriangle_cgal().vertex(2).z()<<") )"<<endl;
		interseccio = triangles_triangulacio[i].interseccio(cara);
		i++;
	}
	cout<<"interseccioPoligonTriangle3D: "<<interseccio<<endl;
	
	return interseccio;
}

/***********************************************************************************************/
/*   bool Poligon::intersecciona2D(Segment ss)                                                   */
/*Metode que ens informa sobre l'existencia d'interseccio entre el poligon i el segment indicat*/
/***********************************************************************************************/

bool Poligon::intersecciona2D(const Segment &s, bool extrems) const
{
// 	if (S.P0 == S.P1) {        // the segment S is a single point
// 		// test for inclusion of S.P0 in the polygon
// 		return cn_PnPoly( s.getOrigen(), V, n ); 
// 	}
// 	
	float  tE = 0;             // the maximum entering segment parameter
	float  tL = 1;             // the minimum leaving segment parameter
	float  t, N, D;            // intersect parameter t = N / D
	Vector3D dS;   // the segment direction vector
	Vector3D e;                  // edge vector
	
/*	cout<<"Aresta E Origen: "<<s.getOrigen().x()<<" - "<<s.getOrigen().y()<<" - "<<s.getOrigen().z()<<endl;
	cout<<"Aresta E Desti: "<<s.getDesti().x()<<" - "<<s.getDesti().y()<<" - "<<s.getDesti().z()<<endl;*/
	
	dS.converteix(s.getDesti(), s.getOrigen());
	for (int i=0; i < costats-1; i++)  // process polygon edge V[i]V[i+1]
	{
		/*cout<<"Iterador "<<i<<" Numero de punts "<<punts.size()<<endl;
		*/e.converteix(punts[i+1], punts[i]);
		N = perp(e, s.getOrigen()-punts[i]);// = -dot(ne, S.P0-V[i])
		D = -perp(e, dS);      // = dot(ne, dS)
		if (fabs(D) < TOLERANCE)
		{ // S is nearly parallel to this edge
			if (N < 0)             // P0 is outside this edge, so
				return false;      // S is outside the polygon
		}
		else
		{
			t = N / D;
			if (D < 0)
			{           // segment S is entering across this edge
				if (t > tE) 
				{      // new max tE
					tE = t;
					if (tE > tL)   // S enters after leaving polygon
						return false;
				}
			}
			else 
			{                 // segment S is leaving across this edge
				if (t < tL) 
				{      // new min tL
					tL = t;
					if (tL < tE)   // S leaves before entering polygon
						return false;
				}
			}
		}
	}
	
	// tE <= tL implies that there is a valid intersection subsegment
// 	IS->P0 = s.getOrigen() + tE * dS;   // = P(tE) = point where S enters polygon
// 	IS->P1 = s.getOrigen() + tL * dS;   // = P(tL) = point where S leaves polygon
	
	if(!extrems && (tE==0 && tL==0) || (tE==1 && tL==1 ))
		return false;
	else
		return true;
}

/****************************************************************************/
/*   bool Poligon::interseccionaPoligonSegment2D(Segment s, bool extrems)   */
/*Metode que permet detectar interseccio entre el poligon i un segment en 2D*/
/****************************************************************************/

bool Poligon::interseccionaPoligonSegment2D(const Segment &s, bool extrems)const
{
	bool interseccio=false;
	unsigned int i=0;
	
	while(i<triangles_triangulacio.size() && !interseccio)
	{
		Triangle3 tr = triangles_triangulacio[i].getTriangle_cgal();
		Poligon_Convex p(tr);
// 		cout<<"interseccionaPoligonSegment2D: (("<<tr.vertex(0).x()<<", "<<tr.vertex(0).y()<<", "<<tr.vertex(0).z()<<") , ("<<tr.vertex(1).x()<<", "<<tr.vertex(1).y()<<", "<<tr.vertex(1).z()<<") , ("<<tr.vertex(2).x()<<", "<<tr.vertex(2).y()<<", "<<tr.vertex(2).z()<<") )"<<endl;
// 		cout<<"Segment E Origen: "<<s.getOrigen().x()<<" - "<<s.getOrigen().y()<<" - "<<s.getOrigen().z()<<endl;
// 		cout<<"Segment E Desti: "<<s.getDesti().x()<<" - "<<s.getDesti().y()<<" - "<<s.getDesti().z()<<endl;
		interseccio = p.intersecciona2D(s, extrems);
		i++;
	}
	return interseccio;
}

/******************************************************************************/
/*  bool Poligon::entrada(pCara triangle, Punt punt)                          */
/*Metode que informa si el segment es entrada o no del poligon segons el punt.*/
/*Ser l'entrada significa que es tracta de l'aresta més propera a un punt     */
/******************************************************************************/

bool Poligon::entrada(const Segment &aresta,const Punt &punt, const Poligon & poligon_visio, bool cas_infinit)const
{
	if(cas_infinit)
		return false;
	
	bool entrada=true, trobat=false, continua=true;
	Segment aux;
	int i=0,j=0;
	Vector3D v1, v2, v3;
	double dist, dist_altre, angle_actual, angle_altre, angle_actual_180, angle_altre_180;
	Punt proper, proper_altre, aux_lluny;
	
	if(!poligon_visio.interseccionaPoligonSegment2D(aresta, false))
		return false;
	dist = aresta.distancia2D(punt);
	
	while(i<3 && !trobat)
	{
		if(punts[i] == aresta.getOrigen())
		{
			trobat = true;
			j=i+1;
			while(j<(i+3) && continua)
			{
				aux.setOrigen(punts[j%costats]);
				aux.setDesti(punts[(j+1)%costats]);
				if(poligon_visio.interseccionaPoligonSegment2D(aux, false))
				{
					dist_altre = aux.distancia2D(punt);
//					if(dist_altre<dist && fabs(dist_altre-dist)>=0.0000001)
					if(dist_altre<dist && fabs(dist_altre-dist)>=TOLERANCE)
						return false;
//					if(fabs(dist_altre-dist)<0.0000001)
					if(fabs(dist_altre-dist)<TOLERANCE)
					{
						proper = aresta.calculaProperRecta2D(punt);
						proper_altre = aux.calculaProperRecta2D(punt);
						if(proper == proper_altre)
						{
							if(proper == aresta.getDesti()) v1.converteix(aresta.getOrigen(),proper);
							else v1.converteix(aresta.getDesti(),proper);
							if(proper == aux.getDesti()) v2.converteix(aux.getOrigen(),proper);
							else v2.converteix(aux.getDesti(),proper);
							v3.converteix(punt,proper);
							
							angle_actual = v3.angleEntreVectors(v1,0);
							angle_altre = v3.angleEntreVectors(v2,0);
							angle_actual_180 = angle_actual;
							angle_altre_180 = angle_altre;
							if(angle_actual>180) angle_actual_180=360-angle_actual;
							if(angle_altre>180) angle_altre_180=360-angle_altre;
	// 						if( angle_actual_180>angle_altre_180 && fabs(angle_actual_180-angle_altre_180)>=0.000001)
							if( angle_actual_180>angle_altre_180 && fabs(angle_actual_180-angle_altre_180)>=TOLERANCE)
								return false;
	//							if(fabs(angle_actual_180-angle_altre_180)<0.000001)
							if(fabs(angle_actual_180-angle_altre_180)<TOLERANCE&&angle_actual > angle_altre)
								return false;
						}
						else
						{
							v1.converteix(proper,punt);
							v2.converteix(proper_altre, punt);
							v3.modifica_coordenades(1.0, 0.0, 0.0);
							
							if(v3.angleEntreVectors(v1,0) > v3.angleEntreVectors(v2,0))
								return false;
						}
					}
				}
				j++;
			}
		}
		i++;
	}
	return true;
// 	return entrada;
}

/*********************************************************************************/
/*   Segment Poligon::seguent(Segment aresta, int &posicio)                      */
/*Mètode que retorna el segment que segueix a l'actual segons el segment  indicat*/
/*********************************************************************************/

Segment Poligon::seguent(const Segment &aresta, int &posicio)const
{
	int i=0;
	bool trobat=false;
	Segment seg;
	
	while(i<3 && !trobat)
	{
// 		cout<<"PUNT: "<<punts[i].x()<<" - "<<punts[i].y()<<" ; "<<aresta.getOrigen().x()<<" - "<<aresta.getOrigen().y()<<endl;
		if(punts[i] == aresta.getOrigen())
		{
// 			cout<<"IGUALS origen"<<endl;
			if(punts[(i+1)%costats] == aresta.getDesti())
			{
// 				cout<<"IGUALS desti"<<endl;
				seg.setOrigen(punts[(i+1)%costats]);
				seg.setDesti(punts[(i+2)%costats]);
				posicio = i;
				trobat = true;
			}
		}
		i++;
	}
	return seg;
}

Triangle Poligon::getTriangle()const
{
	return triangles_triangulacio[0];
}

void Poligon::escriure()const
{

	for(unsigned int i=0;i<punts.size();i++)
		cout<<"("<<punts[i].x()<<", "<<punts[i].y()<<" "<<punts[i].z()<<") ";
	cout<<endl;
}