#include "poligon_convex.h"

/****************************************************/
/*   Poligon_Convex::Poligon_Convex()               */
/*Constructor per defecte que ho inicialitza tot a 0*/
/****************************************************/

Poligon_Convex::Poligon_Convex() 
{
	costats=0;
	tipus_poligon = POLIGON_CONVEX;
}

/******************************************/
/*   Poligon_Convex::~Poligon_Convex()    */
/*Destructor de la classe Poligon_Convex  */
/******************************************/

Poligon_Convex::~Poligon_Convex() {}

/******************************************************************************************/
/*   Poligon_Convex::Poligon_Convex(vector<Punt> p, int cos)                              */
/*Constructor amb parametres que permet configurar cadascuna de les propietats del poligon*/
/******************************************************************************************/

Poligon_Convex::Poligon_Convex(vector<Punt> p, int cos)  
{
	punts=p;
	costats=cos;
	tipus_poligon=POLIGON_CONVEX;
	baricentre=calcula_baricentre();
}

/******************************************************************************************/
/*   Poligon_Convex::Poligon_Convex(vector<Punt> p)                                       */
/*Constructor amb parametres que permet configurar cadascuna de les propietats del poligon*/
/******************************************************************************************/

Poligon_Convex::Poligon_Convex(vector<Punt> p)  
{
	punts = calcularConvexHull(p);
	costats = punts.size();
// 	cout<<"Numero de punts poligon: "<<punts.size()<<endl;
// 	for(int i=0;i<costats;i++)
// 		cout<<"Poligon: "<<punts[i].x()<<" - "<<punts[i].y()<<" - "<<punts[i].z()<<" - "<<endl;
	tipus_poligon=POLIGON_CONVEX;
	baricentre=calcula_baricentre();
}

/**********************************************************/
/*   Poligon_Convex::Poligon_Convex(pCara triangle) */
/*Constructor a partir d'una cara de la triangulacio      */
/**********************************************************/

Poligon_Convex::Poligon_Convex(pCara triangle)
{
	punts.push_back(Punt(triangle->vertex(0)->point().x(), triangle->vertex(0)->point().y(), triangle->vertex(0)->point().z()));
	punts.push_back(Punt(triangle->vertex(1)->point().x(), triangle->vertex(1)->point().y(), triangle->vertex(1)->point().z()));
	punts.push_back(Punt(triangle->vertex(2)->point().x(), triangle->vertex(2)->point().y(), triangle->vertex(2)->point().z()));
	
	costats=3;
	tipus_poligon=POLIGON_CONVEX;
	baricentre=calcula_baricentre();
}

/**********************************************************/
/*   Poligon_Convex::Poligon_Convex(pCara triangle) */
/*Constructor a partir d'una cara de la triangulacio      */
/**********************************************************/

Poligon_Convex::Poligon_Convex(Triangle3 triangle)
{
	punts.push_back(Punt(triangle.vertex(0).x(), triangle.vertex(0).y(), triangle.vertex(0).z()));
	punts.push_back(Punt(triangle.vertex(1).x(), triangle.vertex(1).y(), triangle.vertex(1).z()));
	punts.push_back(Punt(triangle.vertex(2).x(), triangle.vertex(2).y(), triangle.vertex(2).z()));
	
	costats=3;
	tipus_poligon=POLIGON_CONVEX;
	baricentre=calcula_baricentre();
}

vector<Punt> Poligon_Convex::getPunts()const
{
	return punts;
}


/***********************************************************************************/
/*   vector<Punt> Poligon_Convex::calcularConvexHull(vector<Punt> p)               */
/*Metode que permet calcular el Convex Hull equivalent al conjunt de punts indicats*/
/***********************************************************************************/

vector<Punt> Poligon_Convex::calcularConvexHull(vector<Punt> p)
{
	vector<Punt> H;
	int    bot=0, top=(-1);  // indices for bottom and top of the stack
	int    i, j, m;                // array scan index
	
	Punt temp;
	int p_size=(int)p.size();
	for (i=1; i<p_size; i++)
	{
		for( j=0 ; j<p_size - 1; j++)
		{
			if (p[j].x() > p[j+1].x())
			{
				temp = p[j];
				p[j] = p[j+1];
				p[j+1] = temp;
			}
			else
			{
				if(p[j].x()==p[j+1].x())
				{
					if(p[j].y() > p[j+1].y())
					{
						temp = p[j];
						p[j] = p[j+1];
						p[j+1] = temp;
					}
					else
					{
						if(p[j].y()==p[j+1].y())
						{
							for(m=j;m<p_size-1;m++)
								p[m]=p[m+1];
							p.pop_back();
						}
					}
				}
			}
		}
	}
	
	// Get the indices of points with min x-coord and min|max y-coord
	int minmin = 0, minmax;
	int n=p_size;
	float xmin = p[0].x();
	
	for (i=1; i<n; i++)
		if (p[i].x() != xmin) break;
	minmax = i-1;
	if (minmax == n-1)        // degenerate case: all x-coords == xmin
	{
		H.push_back( p[minmin] );
		top++;
		if (p[minmax].y() != p[minmin].y())  // a nontrivial segment
		{
			H.push_back( p[minmax] );
			top++;
		}
		H.push_back( p[minmin] );           // add polygon endpoint
		top++;
		return H;
	}
	// Get the indices of points with max x-coord and min|max y-coord
	int maxmin, maxmax = n-1;
	float xmax = p[n-1].x();
	for (i=n-2; i>=0; i--)
		if (p[i].x() != xmax) break;
	maxmin = i+1;
	
	// Compute the lower hull on the stack H
	H.push_back( p[minmin] );      // push minmin point onto stack
	top++;
	i = minmax;
	while (++i <= maxmin)
	{
		// the lower line joins P[minmin] with P[maxmin]
		if (isLeft( p[minmin], p[maxmin], p[i]) >= 0 && i < maxmin)
			continue;          // ignore P[i] above or on the lower line
			
		while (top > 0)        // there are at least 2 points on the stack
		{
			// test if P[i] is left of the line at the stack top
			if (isLeft( H[top-1], H[top], p[i]) > 0)
				break;         // P[i] is a new hull vertex
			else
			{
				H.pop_back();         // pop top point off stack
				top--;
			}
		}
		H.push_back( p[i] );       // push P[i] onto stack
		top++;
	}
	
	// Next, compute the upper hull on the stack H above the bottom hull
	if (maxmax != maxmin)      // if distinct xmax points
	{
		H.push_back( p[maxmax] );  // push maxmax point onto stack
		top++;
	}
	bot = top;                 // the bottom point of the upper hull stack
	i = maxmin;
	while (--i >= minmax)
	{
		// the upper line joins P[maxmax] with P[minmax]
		if (isLeft( p[maxmax], p[minmax], p[i]) >= 0 && i > minmax)
			continue;          // ignore P[i] below or on the upper line
		while (top > bot)    // at least 2 points on the upper stack
		{
			// test if P[i] is left of the line at the stack top
			if (isLeft( H[top-1], H[top], p[i]) > 0)
				break;         // P[i] is a new hull vertex
			else
			{
				H.pop_back();         // pop top point off stack
				top--;
			}
		}
		H.push_back( p[i] );       // push P[i] onto stack
		top++;
	}
	if (minmax != minmin)
	{
		H.push_back(p[minmin]);  // push joining endpoint onto stack
		top++;
	}
	H.pop_back();
	return H;
}

/******************************************************/
/*   Punt Poligon_Convex::calcula_baricentre()        */
/*Metode que permet calcular el baricentre del poligon*/
/******************************************************/

Punt Poligon_Convex::calcula_baricentre()const
{
	double suma_x=0, suma_y=0, suma_z=0;
	Punt bar;
	
	for(unsigned int i=0; i<punts.size();i++)
	{
		suma_x = suma_x + punts[i].x();
		suma_y = suma_y + punts[i].y();
		suma_z = suma_z + punts[i].z();
	}
	bar.modifica_coordenades((suma_x/(double)punts.size()), (suma_y/(double)punts.size()), (suma_z/(double)punts.size()));
	return(bar);
}

/*********************************************************************/
/*   bool Poligon_Convex::punt_contingut2D(Punt punt)          */
/*Metode que indica si un segment es troba contingut dins del poligon*/
/*********************************************************************/

bool Poligon_Convex::punt_contingut2D(const Punt &punt,bool aLaFrontera)const
{

	int sobre=0;
	
	Vector3D bari(baricentre.x(),baricentre.y(),baricentre.z());
	Vector3D p(punt.x(),punt.y(),punt.z());
	for(unsigned int i=0;i<punts.size();i++)
	{
		Vector3D vs(punts[i].y()-punts[(i+1)%punts.size()].y(),-(punts[i].x()-punts[(i+1)%punts.size()].x()),0);
		double d=-(vs.x()*punts[i].x()+vs.y()*punts[i].y());
		if(fabs(vs.producteEscalar(p)+d)<TOLERANCE) sobre++;
		else if((vs.producteEscalar(bari)+d)*(vs.producteEscalar(p)+d)<0) return false;
	}
// 	if(!aLaFrontera&&sobre>0) //volem que si un punt esta a la frontera del poligon la resposta sigui fals. 
// 		return false;  //El punt esta sobre alguna recta i al costat q toca de les altres=>esta a sobre la frontera
	
	return true;	
}

/******************************************************************************/
/*  bool Poligon_Convex::entrada(Face_handle triangle, Punt punt)             */
/*Metode que informa si el segment es entrada o no del poligon segons el punt.*/
/*Ser l'entrada significa que es tracta de l'aresta més propera a un punt     */
/******************************************************************************/

bool Poligon_Convex::entrada(const Segment &aresta, const Punt &punt, const Poligon_Convex &quadrilater, bool cas_infinit)const
{
	
	if(cas_infinit)
		return false;

// 	cout<<" ePC "<<endl;/ / 	quadrilater.escriure();// 	cout<<" ENTRADA (?) no cas infinit ";
	bool entrada=true, continua=true;
	Segment aux;
	int i=0,j=0;
	Vector3D v1, v2, v3;
	double dist, dist_altre, angle_actual, angle_altre, angle_actual_180, angle_altre_180;
	Punt proper, proper_altre, aux_lluny;
	
	if(!quadrilater.intersecciona2D(aresta, false)) // si intersecciona només anb un extrem retorna fals
		return false;
	
	dist = aresta.distancia2D(punt);
// 	qDebug("SI INTERSECCIONA distancia: %f", dist);
	
	while(i<3 && punts[i]!=aresta.getOrigen()) i++;
	// punts[i]==aresta.getOrigen(),
	
	if(i==3){cout<<"ERROR EXTREM ARESTA NO TROBAT (entrada pC)"<<endl; return true;}
	
	j=i+1;
	while(j<(i+3) && continua)
	{
		aux=Segment(punts[j%costats],punts[(j+1)%costats]);
		if(quadrilater.intersecciona2D(aux, false))
		{
			dist_altre = aux.distancia2D(punt);
			if(dist_altre<dist && fabs(dist_altre-dist)>=TOLERANCE) //0.0000001)
				return false;
		
			if(fabs(dist_altre-dist)<TOLERANCE) //0.0000001)
			{
				proper = aresta.calculaProperRecta2D(punt);
				proper_altre = aux.calculaProperRecta2D(punt);
				if(proper == proper_altre)
				{
					if(proper == aresta.getDesti()) v1.converteix(aresta.getOrigen(),proper);
					else v1.converteix(aresta.getDesti(),proper);
					if(proper == aux.getDesti()) v2.converteix(aux.getOrigen(),proper);
					else v2.converteix(aux.getDesti(),proper);
					v3.converteix(punt,proper);
					angle_actual = v3.angleEntreVectors(v1,0);
					angle_altre = v3.angleEntreVectors(v2,0);
					angle_actual_180 = angle_actual;
					angle_altre_180 = angle_altre;
					if(angle_actual>180) angle_actual_180=360-angle_actual;
					if(angle_altre>180) angle_altre_180=360-angle_altre;
			
					if( angle_actual_180>angle_altre_180 && fabs(angle_actual_180-angle_altre_180)>=TOLERANCE) //0.000001)
						return false;
				
					if(fabs(angle_actual_180-angle_altre_180)<TOLERANCE/*0.000001*/&& angle_actual > angle_altre)
						return false;
				}
				else
				{
					v1.converteix(proper,punt);
					v2.converteix(proper_altre, punt);
					v3.modifica_coordenades(1.0, 0.0, 0.0);
					
					if(v3.angleEntreVectors(v1,0) > v3.angleEntreVectors(v2,0))
						return false;
				}
			}
		}
		j++;
	}
	
	return entrada;
}

/***********************************************************************************************/
/*   bool Poligon_Convex::intersecciona2D(Segment ss)                                            */
/*Metode que ens informa sobre l'existencia d'interseccio entre el poligon i el segment indicat*/
/***********************************************************************************************/

bool Poligon_Convex::intersecciona2D(const Segment &ss, bool extrems)const
{
	
	if(punt_contingut2D(ss.getOrigen(),extrems) || punt_contingut2D(ss.getDesti(),extrems)||punt_contingut2D(Punt(ss.punt_mig_3d()),true))
 		return  true;
 
	for(int i=0;i<costats;i++)
		if(Segment(punts[i],punts[(i+1)%costats]).interseccioSegments2D(ss,extrems)) return true;
		
	return false;
}


/* ***********************************************************************************************************/
/*  bool Poligon_Convex::aresta_oposada_es_entrada(const pCara &tr, int vertex_oposat)const                                                          */
/* retorna cert si l'aresta del triangle tr oposada al vertex_oposat es l'entrada del triangle tr tenint en compte el poligon_convex actual */
/*  o sigui les arestes de tr que no intersequen al poligon_Convex es considera que tenen pes infinit                                                       */
/* ************************************************************************************************************/

bool Poligon_Convex::aresta_oposada_es_entrada(const pCara &tr, int vertex_oposat)const
{
	
	if(vertex_oposat<0 || vertex_oposat>2) return false;
	
	Segment s(Punt(tr->vertex((vertex_oposat+1)%3)->point()),Punt(tr->vertex((vertex_oposat+2)%3)->point()));
	
	if(!intersecciona2D(s,false)) return false;

	for(int i=vertex_oposat+1;i<vertex_oposat+3;i++)
	{
		int j=0;
		while(fabs(tr->info().entrada[i%3][j]-tr->info().entrada[vertex_oposat][j])<TOLERANCE&j<4) j++;
		if(j<4 && tr->info().entrada[i%3][j]<tr->info().entrada[vertex_oposat][j]) 
		{
			s=Segment(Punt(tr->vertex((i+1)%3)->point()),Punt(tr->vertex((i+2)%3)->point()));
			if(intersecciona2D(s,false)) return false;
		}
	}
	return true;
}


