#include "pseudosource.h"
#include "finestra.h"


/**********************************************************************************************/
/*   PseudoSource::PseudoSource()                                                             */
/*Constructor per defecte de la classe PseudoSource que inicialitza els atributs a 0          */
/**********************************************************************************************/

PseudoSource::PseudoSource() 
{
	sigma = 0.0;
	index = 0;
	vertex=NULL;
	cara=NULL;
	pes=sigma;	
	tipus_event = EVENT_PSEUDOSOURCE;
}

/**************************************************************************************************/
/*   PseudoSource::PseudoSource(double sigma_, int index_, pVertex vertex_, pCara cara_           */
/*Constructor amb parametres que construeix el PseudoSource amb els valors indicats               */
/**************************************************************************************************/

PseudoSource::PseudoSource(double sigma_, int index_, pVertex vertex_, pCara cara_) 
{
	this->sigma=sigma_;
	this->index=index_;
	this->vertex=vertex_;
	this->cara=cara_;		
	this->pes=sigma_; //Pes = sigma	
	tipus_event = EVENT_PSEUDOSOURCE;
}


/******************************************/
/*   PseudoSource::~PseudoSource()       */
/*Destructor de la classe PseudoSource   */
/*****************************************/

PseudoSource::~PseudoSource() {}


/*****************************************************/
/*   double PseudoSource::getsigma()                 */
/*Metode que retorna l'atribut sigma del pseudosource*/
/*****************************************************/

double PseudoSource::getsigma()const
{
	return (sigma);
}


/*******************************************************/
/*   int PseudoSource::getindex()                      */
/*Metode que retorna l'atribut index del pseudosource  */
/*******************************************************/

int PseudoSource::getindex()const
{
	return index;
}

/********************************************************/
/*   pVertex PseudoSource::getvertex()                  */
/*Metode que retorna l'atribut vertex del pseudosource  */
/*******************************************************/

pVertex PseudoSource::getvertex()const
{
	return vertex;
}

/*******************************************************/
/*   pCara PseudoSource::getcara()                    */
/*Metode que retorna l'atribut cara del pseudosource  */
/*******************************************************/

pCara PseudoSource::getcara()const
{
	return cara;
}

/*******************************************************/
/*   double PseudoSource::getpes()                     */
/*Metode que retorna l'atribut pes del pseudosource    */
/*******************************************************/
/*
double PseudoSource::getpes()const
{
	return pes;
}*/


/***************************************************************/
/*   void PseudoSource::setsigma(double sigma_)                */
/*Metode que permet modificar el valor de l'atribut sigma      */
/***************************************************************/

void PseudoSource::setsigma(double sigma_)
{
	this->sigma=sigma_;
}


/***************************************************************/
/*   void PseudoSource::setindex(int index_)                   */
/*Metode que permet modificar el valor de l'atribut index      */
/***************************************************************/

void PseudoSource::setindex(int index_)
{
	this->index=index_;
}

/***************************************************************/
/*   void PseudoSource::setvertex(pVertex vertex_)             */
/*Metode que permet modificar el valor de l'atribut vertex    */
/*********************************************************** ***/

void PseudoSource::setvertex(pVertex vertex_)
{
	this->vertex=vertex_;
}

/***************************************************************/
/*   void PseudoSource::setcara(pCara cara_)                  */
/*Metode que permet modificar el valor de l'atribut cara      */
/***************************************************************/

void PseudoSource::setcara(pCara cara_)
{
	this->cara=cara_;
}

/***************************************************************/
/*   void PseudoSource::setpes(double pes)                     */
/*Metode que permet modificar el valor de l'atribut pes        */
/***************************************************************/
/*
void PseudoSource::setpes(double pes)
{
	this->pes=pes;
}*/


/****************************************************************************************************************/
/*   void PseudoSource::modifica_pseudosource(double sigma_, int index_, pVertex vertex_, pCara cara_);        */
/*Metode que permet modificar tots els abributs de finestra de cop                                             */
/***************************************************************************************************************/

void PseudoSource::modifica_pseudosource(double sigma_, int index_, pVertex vertex_, pCara cara_)
{
	this->sigma=sigma_;
	this->index=index_;
	this->vertex=vertex_;
	this->cara=cara_;
	this->pes=sigma_;
}


void PseudoSource::pintarFinestres(Delaunay *dt)
{
list<Finestra>::iterator fj,fvj;
Finestra f;
int l;
double e;

pintarEvent(); //Pintem el PseudoSource

CirculadorCares fit=(dt->incident_faces(vertex,cara));
fit++;
do{
	if(!dt->is_infinite(fit)) //Només fem quan no és una cara infinita 
	{		
		//Recorrer les finestres que hi ha guardades als 3 costats calculant s i e i les pintem cridant a pintar(s,e) 
		for(l=0; l<3; l++)
		{
			fj=fit->info().LlistaFinestres[l].begin();
			fvj=fit->info().LlistaFinestres[l].end();
						
			while (fj!=fvj)
			{
				Segment s(fit->vertex((l+1)%3)->point(),fit->vertex((l+2)%3)->point());
				e=s.midaSegment();
				f=*fj;
				//f=fit->info().LlistaFinestres[l].front();
				f.pintar(s,e);
				fj++;
			}
		}
	}
fit++;	

} while (fit!=cara);

//cout << "Surto de pintarFinestres PseudoSource" << endl;

}


void PseudoSource::pintarEvent()
{
//cout << "Entro a pintarEvent() PseudoSource" << endl;
Punt3d p=vertex->point();

//Dibuixar el PseudoSource
//glColor3f(1.0,0.2,0.3);
cout << "PseudoSource Nou = [" << p.x() << ", " << p.y() << ", " << p.z() << "]" << endl;
glBegin(GL_POINTS);
	glVertex3f(p.x(), p.y(),p.z());
glEnd();
cout << "He dibuixat el PseudoSource de color rosa" << endl;
}


//Redefinició d'operadors

bool PseudoSource::operator==(PseudoSource ps)const
{
	bool iguals=false;
	if(fabs(sigma-ps.getsigma())<=TOLERANCE && this->index==ps.getindex() && this->vertex==ps.getvertex() && this->cara ==ps.getcara() && fabs(pes-ps.getpes())<=TOLERANCE && tipus_event==ps.tipusEvent())
	{
	iguals=true;
	}
	return (iguals);
}

bool PseudoSource::operator!=(PseudoSource ps)const
{
	bool diferents=true;
	if(*this==ps)
	{
		diferents=false;
	}
	return(diferents);
}

void PseudoSource::escriure()const
{
	cout << "PS  " << pes << " (sigma, index, pv) = (" << sigma << ", " << index << ", (" << vertex->point().x() << ", " << vertex->point().y() << ", " << vertex->point().z()<< ") )" << endl;
}


//Mètode que ens propaga un PseudoSource
list<Event*> PseudoSource::propagar(Delaunay *dt)
{
list<Event*> E;
pCara carapseudo, tri, cara;
pVertex vertexpseudo;
double sigmapseudo,t;
int indexpseudo, l, index, j;
double b0,b1,d0,d1,sigma,pes;
Punt3d puntpseudo, puntvertex1, puntvertex2;
list<Finestra>::iterator fa,fl;

//Obtenim els paràmetres del PseudoSource ps
carapseudo=this->getcara(); 
vertexpseudo=this->getvertex();
puntpseudo=vertexpseudo->point();
sigmapseudo=this->getsigma();
indexpseudo=this->getindex();

//Per cada cara menys la pCara carapseudo, per a cada vèrtex fer 3 finestres (una per cada costat).
CirculadorCares fit=(dt->incident_faces(vertexpseudo,carapseudo));
fit++;
do{
	if(!dt->is_infinite(carapseudo)) //Només fem quan no és una cara infinita
	{		
		for(l=0; l<3; l++) // Creem una finestra per cada vertex amb les distancies que toquin. Cada un serà un Punt3d. 
		{
			puntvertex1=fit->vertex((l+1)%3)->point();
			puntvertex2=fit->vertex((l+2)%3)->point();
														
			d0=sqrt((puntpseudo.x()-puntvertex1.x())*(puntpseudo.x()-puntvertex1.x())+(puntpseudo.y()-puntvertex1.y())*(puntpseudo.y()-puntvertex1.y())+(puntpseudo.z()-puntvertex1.z())*(puntpseudo.z()-puntvertex1.z()));
			d1=sqrt((puntpseudo.x()-puntvertex2.x())*(puntpseudo.x()-puntvertex2.x())+(puntpseudo.y()-puntvertex2.y())*(puntpseudo.y()-puntvertex2.y())+(puntpseudo.z()-puntvertex2.z())*(puntpseudo.z()-puntvertex2.z()));
			b0=0;
			b1=sqrt((puntvertex1.x()-puntvertex2.x())*(puntvertex1.x()-puntvertex2.x())+(puntvertex1.y()-puntvertex2.y())*(puntvertex1.y()-puntvertex2.y())+(puntvertex1.z()-puntvertex2.z())*(puntvertex1.z()-puntvertex2.z()));
			sigma=this->getsigma();
			cara=fit; 
			index=this->getindex(); //Index de la posicio del vector on hi ha el punt
			j=l; //Serà el vertex oposat
			
			Finestra * f = new Finestra(b0,b1,d0,d1,sigma,cara,j,index);
			//f=new Finestra(ps_v.distancia(puntvertex1),ps_v.distancia(puntvertex2),0,Punt(puntvertex1).distancia(puntvertex2),ps.getsigma(),fit,l,ps.getindex());
			f->escriure();
			
			fa=fit->info().LlistaFinestres[j].begin();
			fl=fit->info().LlistaFinestres[j].end();
		
			bool iguals=false;
			while (fa!=fl)
			{
				Finestra fff=*fa;
					if((*(Finestra *)f)==fff)				
					{
						iguals=true;
					}
				fa++;
			}	
			
			if (iguals)
			{ }
			else
			{
				//fit->info().LlistaFinestres[j].push_front(*f); //Insereix la finestra f a l'inici de la Llista Finestres						
				//Inserim la finestra a la llista d'events (serà del tipus EVENT_FINESTRA(0))
				E.push_back(f); //Insereix events tipus Finestra al final de la llista d'Events
				}	
			//d0 es la distancia del vertex l+1 al pseudosource
			if((fit->vertex((l+1)%3)->info().distancia==-1) || (fit->vertex((l+1)%3)->info().distancia>d0+sigma))
			{ 
				fit->vertex((l+1)%3)->info().distancia=d0+sigma;
			 	fit->vertex((l+1)%3)->info().index=index;	
			}				
		}
	}
fit++;	

} while (fit!=carapseudo);

E.sort(Event::ordena); //Ordenem la llista d'events a retornar de menys a més pes
return E;
}

