#include "pla.h"
#include "punt.h"
#include "segment.h"

/*****************************************************************************************/
/*   Punt::Punt()                                                                        */
/*Constructor per defecte de la classe Punt que inicialitza les coordenades del punt a 0 */
/*****************************************************************************************/

Punt::Punt() 
{
	coord_x = 0.0 ;
	coord_y = 0.0 ;
	coord_z = 0.0 ;
	
	punt_cgal = Punt3d(0.0, 0.0, 0.0);
	tipus_objecte = PUNT_VISIO;
}

/***************************************************************************/
/*   Punt::Punt(double x, double y, double z)                              */
/*Constructor amb parametres que construeix el punt amb els valors indicats*/
/***************************************************************************/

Punt::Punt(double x, double y, double z) 
{
	coord_x = x ;
	coord_y = y ;
	coord_z = z ;
	
	punt_cgal = Punt3d(x, y, z);
	
	tipus_objecte = PUNT_VISIO;
}

/*******************************************************/
/*   Punt::Punt(Punt3d * p)                             */
/*Constructor que rep com a parametres un punt del CGAL*/
/*******************************************************/

Punt::Punt(Punt3d p)
{
	coord_x = p.x() ;
	coord_y = p.y() ;
	coord_z = p.z() ;
	
	punt_cgal = p;
	
	tipus_objecte = PUNT_VISIO;
}

/*******************************************************/
/*   Punt::Punt(Punt2d * p)                             */
/*Constructor que rep com a parametres un punt del CGAL*/
/*******************************************************/

Punt::Punt(Punt2d p)
{
	coord_x = p.x() ;
	coord_y = p.y() ;
	coord_z = 0.0 ;
	
	punt_cgal = Punt3d(p.x(), p.y(), 0.0);
	
	tipus_objecte = PUNT_VISIO;
}

/************************************************/
/*   Punt3d * Punt::getPunt()                    */
/*Metode que retorna el punt del cgal equivalent*/
/************************************************/

Punt3d Punt::getPunt() const
{
	return punt_cgal;
}

/******************************/
/*   Punt::~Punt()            */
/*Destructor de la classe Punt*/
/******************************/

Punt::~Punt() {}

/*************************************************/
/*   void Punt::getPunt(Punt3d *)                 */
/*Metode que modifica el punt del cgal equivalent*/
/*************************************************/

void Punt::setPunt(Punt3d p)
{
	coord_x = p.x() ;
	coord_y = p.y() ;
	coord_z = p.z() ;
	
	punt_cgal = p;
}

/*******************************************/
/*   Punt2d Punt::on_xy()                  */
/*Metode que retorna el punt en 2D del CGAL*/
/*******************************************/

Punt2d Punt::on_xy() const
{
	Punt2d p(punt_cgal.x(),punt_cgal.y());
	return p;
}

/********************************************************************************/
/*   Punt3d Punt::on_xy0()                                                       */
/*Metode que retorna el punt 3D del CGAL equivalent pero amb la coordenada z a 0*/
/********************************************************************************/

Punt3d Punt::on_xy0() const
{
	Punt3d p(punt_cgal.x(),punt_cgal.y(),0);
	return p;
}

/***********************************************************************************/
/*   Punt3d punt_mig(const Punt3d3 &q)                                               */
/*Metode que retorna el punt mig entre 2 punts, l'actual i el passat per parametres*/
/***********************************************************************************/

Punt3d Punt::punt_mig(const Punt3d &q) const
{
	Punt3d pm(0.5*(punt_cgal.x()+q.x()),0.5*(punt_cgal.y()+q.y()),0.5*(punt_cgal.z()+q.z()));
	return pm;
}

/***************************************************/
/*   double Punt::x()                              */
/*Metode que retorna la primera coordenada del punt*/
/***************************************************/

double Punt::x()const
{
	return (coord_x );
}

/***************************************************/
/*   double Punt::y()                              */
/*Metode que retorna la segona coordenada del punt */
/***************************************************/

double Punt::y()const
{
	return (coord_y );
}

/***************************************************/
/*   double Punt::z()                              */
/*Metode que retorna la tercera coordenada del punt*/
/***************************************************/

double Punt::z()const
{
	return (coord_z );
}

/***************************************************************/
/*   double Punt::modifica_x()                                 */
/*Metode que permet modificar el valor de la primera coordenada*/
/***************************************************************/

void Punt::modifica_x(double x)
{
	punt_cgal = Punt3d(x, punt_cgal.y(),punt_cgal.z());
	coord_x=x;
}

/***************************************************************/
/*   double Punt::modifica_y()                                 */
/*Metode que permet modificar el valor de la segona coordenada */
/***************************************************************/

void Punt::modifica_y(double y)
{
	punt_cgal = Punt3d(punt_cgal.x(), y,punt_cgal.z());
	coord_y=y;
}

/***************************************************************/
/*   double Punt::modifica_z()                                 */
/*Metode que permet modificar el valor de la tercera coordenada*/
/***************************************************************/

void Punt::modifica_z(double z)
{
	punt_cgal = Punt3d(punt_cgal.x(), punt_cgal.y(),z);
		
	coord_z=z;
}

/**********************************************************************/
/*   void Punt::modifica_coordenades(double x, double y, double z)    */
/*Metode que permet modificar les tres coordenades del vector de cop  */
/**********************************************************************/

void Punt::modifica_coordenades(double x, double y, double z)
{
	coord_x=x;
	coord_y=y;
	coord_z=z;
	punt_cgal = Punt3d(x,y,z);
}


/***************************************************************************************************************/
/*   bool Punt::buscar_z(Delaunay *dt, Punt &punt)             */
/*Mètode que ens calcula la z d'un punt del terreny de Delaunay*/
/***************************************************************************************************************/

bool Punt::buscar_z(Delaunay *dt)
{
bool correcte=false;
pCara cara;
Pla pla;
Linia linia;
Localitzacio tipus;
int vertex;

cara = dt->locate(Punt3d(coord_x,coord_y,coord_z),tipus,vertex);

switch(tipus)
	{
	case Delaunay::VERTEX: 
	//punt=Punt(coord_x,coord_y,cara->vertex(vertex)->point().z());
	coord_z=cara->vertex(vertex)->point().z();
	correcte=true;
	break; 

	case Delaunay::EDGE:
	//vigilar amb vertex+1 i +2 (posar modul 3) //punt2d pot ser que s'hagi de crear a fora mes amunt
	linia=Linia(Line(cara->vertex(vertex+1)->point(),cara->vertex(vertex+2)->point()));
	//punt=Punt(linia.on_l(Punt2d(coord_x,coord_y)));
	coord_z=linia.on_l(Punt2d(coord_x,coord_y)).z();
	correcte=true;
	break;

	case Delaunay::FACE:
	pla=Plane(cara->vertex(0)->point(), cara->vertex(1)->point(), cara->vertex(2)->point());
	//punt=pla.puntSobrePla(coord_x, coord_y, Punt(cara->vertex(0)->point()) );
	coord_z=pla.puntSobrePla(coord_x, coord_y, Punt(cara->vertex(0)->point()) ).z();
	correcte=true;
	break; 

	case Delaunay::OUTSIDE_CONVEX_HULL: //Significa que el punt es troba fora de la triangulació
	break;
	
	case Delaunay::OUTSIDE_AFFINE_HULL:
	break; 

	}
	
return correcte;

}


/***************************************************************************************************************/
/*   bool Punt::puntCorrecte(Delaunay)                                                                         */
/*Mètode que indica si el punt seleccionat de visio es o no acceptable, és a dir es troba per sobre del terreny*/
/***************************************************************************************************************/

bool Punt::puntCorrecte(Delaunay *dt)
{
	bool correcte = false;
	pCara cara;
	Plane * pla;
	Localitzacio tipus;
	Orientacio orientacio;
	int vertex;
	
	/*A partir de la triangulació determinem la cara sobre la qual es trobaria el punt introduit sense tenir la coordenada z. A partir d'aquesta informació podrem determinar si el punt es troba per sobre o no creant un pla que contingui la cara i observant si el punt es troba a la part positiva*/
	
	cara = dt->locate(Punt3d(coord_x, coord_y, coord_z), tipus, vertex);
	
	switch(tipus)
	{
		case Delaunay::EDGE: /*En el cas que es tracti d'una aresta el mètode de càlcul serà un altre CANVIAR!!!*/
			pla = new Plane(cara->vertex(0)->point(), cara->vertex(1)->point(), cara->vertex(2)->point()); 
			orientacio = pla->oriented_side(Punt3d(coord_x, coord_y, coord_z));
			
			if(orientacio == CGAL::ON_POSITIVE_SIDE)
			{
				correcte = true;
			}
			break;
			delete pla;
		case Delaunay::VERTEX:
			if(coord_z > cara->vertex(vertex)->point().z())
			{
				correcte = true;
			}
			break;
		case Delaunay::FACE:
			if(!dt->is_infinite(cara))
			{
				pla = new Plane(cara->vertex(0)->point(), cara->vertex(1)->point(), cara->vertex(2)->point()); 
				orientacio = pla->oriented_side(Punt3d(coord_x, coord_y, coord_z));
			
				if(orientacio == CGAL::ON_POSITIVE_SIDE)
				{
					correcte = true;
				}
				delete pla;
			}
			break;
		case Delaunay::OUTSIDE_CONVEX_HULL: /*Significa que el punt es troba fora de la triangulació*/
			break;
		case Delaunay::OUTSIDE_AFFINE_HULL:
			break;
	}	
	return correcte;
}

/*********************************************************/
/*   Punt2d Punt::skew_point(Segment3D ss, Segment3D sv) */
/*Metode que calcula el punt d'skew projection           */  
/*********************************************************/

Punt2d Punt::skew_point(Segment3D ss, Segment3D sv)const
{
    double t=-500.;
    double u=-500.;
    Segment ss_nostre(ss);
    Segment sv_nostre(sv);

	bool sobre_lv=Linia(sv_nostre.getSegment_cgal().supporting_line()).punt_sobre(punt_cgal);
	bool sobre_ls=Linia(ss_nostre.getSegment_cgal().supporting_line()).punt_sobre(punt_cgal);    

	if(!(sobre_ls||sobre_lv))
// 	if(!(ss_nostre.punt_sobre(punt_cgal) || sv_nostre.punt_sobre(punt_cgal)))
	{
		Pla pla_v(Plane((sv).supporting_line(),punt_cgal));
		Pla pla_s(Plane((ss).supporting_line(),punt_cgal));
		
		Punt3d ps,pv;
		bool linias,liniav;
		bool intersecav=pla_v.interseccio(Linia(ss.supporting_line()),ps,u,linias);
		bool intersecas=pla_s.interseccio(Linia(sv.supporting_line()),pv,t,liniav);
		if(!intersecav || !intersecas) cout<<"PROBLEMES SKEW_POINT"<<endl;
	}
// 	else if (ss_nostre.punt_sobre(punt_cgal))
	else if(sobre_ls)
	{
		u=coord_p_on_e(ss);
// 		qDebug("SKP d'un punt sobre el segment ss.");
	}
	else 
	{
		t=coord_p_on_e(sv); 
// 		qDebug("SKP d'un punt sobre el segment sv.");
	}
	Punt2d co(t,u);
	return co;
	/*
 double t=-500.;
    double u=-500.;
    Segment ss_nostre(ss);
    Segment sv_nostre(sv);
    
	if(!(ss_nostre.punt_sobre(punt_cgal) || sv_nostre.punt_sobre(punt_cgal)))
	{
		Pla pla_v(Plane((sv).supporting_line(),punt_cgal));
		Pla pla_s(Plane((ss).supporting_line(),punt_cgal));
		
		Punt3d ps,pv;
		bool linias,liniav;
		bool intersecav=pla_v.interseccio(Linia(ss.supporting_line()),ps,u,linias);
		bool intersecas=pla_s.interseccio(Linia(sv.supporting_line()),pv,t,liniav);
		if(!intersecav || !intersecas) cout<<"PROBLEMES SKEW_POINT"<<endl;
	}
	else if (ss_nostre.punt_sobre(punt_cgal))
	{
		u=coord_p_on_e(ss);
		qDebug("SKP d'un punt sobre el segment ss.");
	}
	else 
	{
		t=coord_p_on_e(sv); 
		qDebug("SKP d'un punt sobre el segment sv.");
	}
	Punt2d co(t,u);
	return co;*/
}

// retorna la coordenada de p en el segment s parametereitzat de s(1) a s(2) per 0,1, si p no es de s retorna -1
double Punt::coord_p_on_e(const  Segment3D &e)const
{
	double lambda;
	int i0=0;
	bool colineal=true;

	while(i0<3 && fabs(e.to_vector().cartesian(i0))<TOLERANCE ) // e.to_vector().(i0)!=0
		i0++;
	
	if(i0==3) i0=2;
	
	lambda=(punt_cgal.cartesian(i0)-e.source().cartesian(i0))/(e.to_vector().cartesian(i0));

	for(int j=0;j<2;j++)
	{
		double lambda1=fabs(punt_cgal.cartesian(j)-(e.source().cartesian(j)+lambda*e.to_vector().cartesian(j)));
                if(lambda1>TOLERANCE) {qDebug(" coord_p_on_e, (classe Punt), dif lambdes= %f",lambda1);colineal=false;}
	}
	
	return lambda;

}

/************************************************/
/*   double Punt::distancia(const Punt3d &q)     */
/*Metode que calcula la distancia entre 2 punts */  
/************************************************/

double Punt::distancia(const Punt3d &q)const
{
	Vector3D v(punt_cgal,q);
	return sqrt(v*v);
	return v*v;
}

double Punt::distancia2D(const Punt &p)const
{
	return sqrt((coord_x-p.x())*(coord_x-p.x()) + (coord_y-p.y())*(coord_y-p.y()) );
}

double Punt::angle2D(const Punt &p, const Punt &q)const // angle (actual,p,q) entre 0 i 180 en graus
{
	Punt pt(coord_x-p.x(),coord_y-p.y(),0);
	Punt pq(p.x()-q.x(),p.y()-q.y(),0);
 	double lpt=sqrt(pt.x()*pt.x()+pt.y()*pt.y());
	double lpq=sqrt(pq.x()*pq.x()+pq.y()*pq.y());
	double dot=pt.x()*pq.x()+pt.y()*pq.y();
	double z_perp=pt.x()*pq.y()-pt.y()*pq.x();
	
	if(fabs(lpt)<TOLERANCE || fabs(lpq)<TOLERANCE)
	{
		cout<<"ERROR VECTOR BUSCAR ANGLE format 3 pts i 2 IGUALS"<<endl;
		char kdflj; 
		cin>>kdflj;
	}
	if(z_perp==0 && dot<0) return 180;
	return acos(dot/(lpt*lpq))*180/PI; 
	
		
		
	Vector3D tp(p,*this);
	Vector3D tq(*this,q);
	double angle=tp.angleEntreVectors (tq,false);
	if(angle>180) return angle-180;
	return angle;
}

double Punt::angle2Dccw(const Punt &p, const Punt &q)const // angle (actual,p,q) en sentit ccw (contrari agulles rellotge)
{
	Punt pt(coord_x-p.x(),coord_y-p.y(),0);
	Punt pq(p.x()-q.x(),p.y()-q.y(),0);
 	double lpt=sqrt(pt.x()*pt.x()+pt.y()*pt.y());
	double lpq=sqrt(pq.x()*pq.x()+pq.y()*pq.y());
	double dot=pt.x()*pq.x()+pt.y()*pq.y();
	double z_perp=pt.x()*pq.y()-pt.y()*pq.x();
	
	if(fabs(lpt)<TOLERANCE || fabs(lpq)<TOLERANCE)
	{
		cout<<"ERROR VECTOR BUSCAR ANGLEccw format 3 pts i 2 IGUALS"<<endl;
		char kdflj; 
		cin>>kdflj;
	}
	if(z_perp==0 && dot<0) return 180;
	if(z_perp<0) return 360-acos(dot/(lpt*lpq))*180/PI; 
	return acos(dot/(lpt*lpq))*180/PI; 
}


/*Redefinicio d'operadors*/

Punt Punt::operator+(Punt const &altre)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x + altre.coord_x , coord_y + altre.coord_y, coord_z + altre.coord_z);
	return pr;
}

Punt Punt::operator+(double n)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x + n , coord_y + n, coord_z + n);
	return pr;
}

Punt Punt::operator*(Punt const &altre)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x * altre.coord_x , coord_y * altre.coord_y, coord_z * altre.coord_z);
	return pr;
}

Punt Punt::operator*(double n)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x * n , coord_y * n, coord_z * n);
	return pr;
}

Punt Punt::operator/(Punt const &altre)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x / altre.coord_x , coord_y / altre.coord_y, coord_z / altre.coord_z);
	return pr;
}

Punt Punt::operator/(double n)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x / n , coord_y / n, coord_z / n);
	return pr;
}

Punt Punt::operator-(Punt const &altre)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x - altre.coord_x , coord_y - altre.coord_y, coord_z - altre.coord_z);
	return pr;
}

Punt Punt::operator-(double n)const
{
	Punt pr;
	
	pr.modifica_coordenades(coord_x - n , coord_y - n, coord_z - n);
	return pr;
}

/*Redefinicio dels operadors d'igualtat i diferencia per poder realitzar comparacions, entre objectes d'aquesta 
mateixa classe*/

bool Punt::operator==(Punt p)const
{
	bool iguals=false;
// 	if(fabs(coord_x-p.x())<0.00001 &&  fabs(coord_y-p.y())<0.00001 && fabs(coord_z-p.z())<0.00001)
	if(fabs(coord_x-p.x())<TOLERANCE &&  fabs(coord_y-p.y())<TOLERANCE && fabs(coord_z-p.z())<TOLERANCE)
	{
		iguals=true;
	}
	return(iguals);
}

bool Punt::operator!=(Punt p)const
{
	bool diferents=true;
	if(*this==p)
	{
		diferents=false;
	}
	return(diferents);
}
bool Punt::operator<(Punt const &p)const // ordre lexicografic (sense tolerance)
{
	if(coord_x<p.x()) return true;
	if(coord_x==p.x()&&coord_y<p.y()) return true;
	if(coord_x==p.x()&&coord_y==p.y()&&coord_z<p.z()) return true;
	return false;
}
bool  Punt::operator>(Punt const &p)const // ordre lexicografic (sense tolerance)
{
	return !(*this<p)&&!(*this==p);
}

bool Punt::mes_petit(Punt const &p)const // ordre lexicografic (amb tolerance)
{
	if(coord_x<p.x()&&fabs(coord_x-p.x())>TOLERANCE) return true;
	if(fabs(coord_x-p.x())<TOLERANCE&&coord_y<p.y()&&fabs(coord_y-p.y())>TOLERANCE) return true;
	if(fabs(coord_x-p.x())>TOLERANCE&&fabs(coord_y-p.y())<TOLERANCE&&coord_z<p.z()&&fabs(coord_z-p.z())>TOLERANCE) return true;
	return false;
}

bool  Punt::mes_gran(Punt const &p)const // ordre lexicografic (amb tolerance)
{
	return !mes_petit(p)&&!(*this==p);
}

void Punt::escriure()const
{
	cout<<"("<<coord_x<<", "<<coord_y<<", "<<coord_z<<")"<<endl;
}

vector<Punt> Punt::ordenats_x(const vector<Punt> &p)const
{
	vector<Punt> vp;
	int j=0;
	vp.push_back(Punt(coord_x,coord_y,coord_z));
	for(int i=0;i<p.size();i++)
	{
		vp.push_back(p[i]);
		for(j=0;j<i+1 && vp[j]<vp[i+1]; j++){}
		if(j!=i+1) 
		{
			for(int k=i+1;k>j;k--) vp[k]=vp[k-1];
			vp[j]=p[i];
		}
	}
	return vp;
}
vector<Punt> Punt::ordenats_y(const vector<Punt> &p)const
{
	vector<Punt> py;
	for(unsigned int i=0;i<p.size();i++)
		py.push_back(Punt(p[i].y(),p[i].z(),p[i].x()));
	
	py=Punt(coord_y,coord_z,coord_x).ordenats_x(py);
	
	for(unsigned int i=0;i<py.size();i++)
		py[i]=Punt(py[i].z(),py[i].x(),py[i].y());
	
	return py;
}


vector<Punt> Punt::ordenats_z(const vector<Punt> &p)const
{
	vector<Punt> pz;
	for(unsigned int i=0;i<p.size();i++)
		pz.push_back(Punt(p[i].z(),p[i].x(),p[i].y()));
	pz=Punt(coord_z,coord_x,coord_y).ordenats_x(pz);
	for(unsigned int i=0;i<pz.size();i++)
		pz[i]=Punt(pz[i].y(),pz[i].z(),pz[i].x());
	return pz;
}

