#ifndef PUNT_H
#define PUNT_H

#include <math.h>
#include <vector>
#include "objecte.h"
#include "linia.h"
#include "geometriabasica.h"

class Pla;
class Segment;
/******************************************************************/
/*                         class Punt                             */
/*   Classe Punt3d que permet el tractament de punts en l'aplicació*/
/******************************************************************/

class Punt : public Objecte
{
	private:
		double coord_x ;
		double coord_y ;
		double coord_z ;
		
		Punt3d punt_cgal;
	public:
		Punt () ; /*Constructor per defecte que inicialitza les coordenades del vector a 0*/
		~Punt();
		Punt (double x, double y, double z) ; /*Constructor amb parametres*/
		Punt (Punt3d);
		Punt (Punt2d);

		/*Metodes que retornen les coordenades del vector*/
		double x() const;
		double y() const;
		double z()const ;

		/*Metodes per la gestio de punts cgal*/
		Punt3d getPunt()const;
		void setPunt(Punt3d);
		
		Punt2d on_xy()const;
		Punt3d on_xy0()const;
		Punt3d punt_mig(const Punt3d &q)const;
		Punt2d skew_point(Segment3D ss, Segment3D sv)const;
		
		/*Metodes que permeten modificar el valor de les coordenades de forma individual, o be, totes de cop*/
		void modifica_x(double x) ;
		void modifica_y(double y) ;
		void modifica_z(double z) ;
		void modifica_coordenades(double x, double y, double z);
		double coord_p_on_e(const  Segment3D &e)const;

		bool puntCorrecte(Delaunay *);
		bool buscar_z(Delaunay *dt);
		
		double distancia(const Punt3d &q)const;
		double distancia2D(const Punt &p)const;
		double angle2D(const Punt &p, const Punt &q)const; // angle (actual,p,q) entre 0 i 180 en graus
		double angle2Dccw(const Punt &p, const Punt &q)const; // angle (actual,p,q) en sentit ccw (contrari agulles rellotge)
		
		Punt operator+(Punt const &altre)const;
		Punt operator+(double n)const;
		Punt operator*(Punt const &altre)const;
		Punt operator*(double n)const;
		Punt operator/(Punt const &altre)const;
		Punt operator/(double n)const;
		Punt operator-(Punt const &altre)const;
		Punt operator-(double n)const;
		
		void escriure()const;
			
		/*Redefinicio dels operadors d'igualtat i diferencia per poder realitzar comparacions, entre objectes d'aquesta 
		mateixa classe*/
		bool operator==(Punt p)const;
		bool operator!=(Punt p)const;
		bool operator<(Punt const &p)const; // ordre lexicografic (sense tolerance)
		bool  operator>(Punt const &p)const; // ordre lexicografic (sense tolerance)
		bool mes_petit(Punt const &p)const; // ordre lexicografic (amb tolerance)
		bool  mes_gran(Punt const &p)const; // ordre lexicografic (amb tolerance)
		
		vector<Punt> ordenats_x(const vector<Punt> &p)const;
		vector<Punt> ordenats_y(const vector<Punt> &p)const;
		vector<Punt> ordenats_z(const vector<Punt> &p)const;
};

#endif
