#include "regio.h"

/**************************************/
/*   Regio::Regio()                   */
/*Constructor per defecte de la classe*/
/**************************************/

Regio::Regio()
{
	hiperboles.clear();
}

/*****************************************************************/
/*   Regio::Regio(Hiperbola, Hiperbola, Hiperbola)               */
/*Constructor que permet crear regions formades per 3 hiperboles */
/*****************************************************************/

Regio::Regio(const Hiperbola &a,const  Hiperbola &b, const Hiperbola &c)
{
	hiperboles.clear();
	hiperboles.push_back(a);
	hiperboles.push_back(b);
	hiperboles.push_back(c);
}


/*****************************************************************/
/*   Regio::Regio(Hiperbola, Hiperbola)               */
/*Constructor que permet crear regions formades per 2 hiperboles */
/*****************************************************************/

Regio::Regio(const Hiperbola &a, const Hiperbola & b)
{
	hiperboles.clear();
	hiperboles.push_back(a);
	hiperboles.push_back(b);
}

/*****************************************************************/
/*   Regio::Regio(Hiperbola)                                                              */
/*Constructor que permet crear regions formades per 1 hiperbola */
/* volem q nomes sigui una hiperbola, pertant la delimitem x dues rectes */
/*una a cada extrem i dues hiperboles iguals pero de diferent signe*/
/*****************************************************************/

Regio::Regio(const Hiperbola &a)
{
	hiperboles.clear();
	hiperboles.push_back(a);
	hiperboles.push_back(Hiperbola(-a.ctu(),-a.ct(),-a.cu(),-a.cc(),a.source(),a.target()));
}


/*****************************************************************/
/*   Regio::Regio(Hiperbola, Hiperbola, Hiperbola)               */
/*Constructor que permet crear regions formades per un vector d' hiperboles */
/*****************************************************************/

Regio::Regio(const vector<Hiperbola> &h)
{
	hiperboles = h;
}

/*********************************/
/*   Regio::~Regio()             */
/*Destructor de la classe Regio  */
/*********************************/

Regio::~Regio() {}

/***************************************************************/
/*   vector<Hiperbola> Regio::getRegio()                       */
/*Metode que retorna la regio formada pel vector de hiperboles */
/***************************************************************/

vector<Hiperbola> Regio::getRegio()const
{
	return hiperboles;
}

/***************************************************************/
/*   vector<Hiperbola> Regio::getRegio()                       */
/*Metode que modifica la regio formada pel vector de hiperboles*/
/***************************************************************/

void Regio::setRegio(vector<Hiperbola> h)
{
	hiperboles = h;
}

/***********************************************/
/*   Hiperbola Regio::getHiperbola(int posicio)*/
/*Metode que retorna la hiperbola desitjada    */
/***********************************************/

Hiperbola Regio::getHiperbola(int posicio)const
{
	return hiperboles[posicio-1];
}

/*********************************************************/
/*   int Regio::numero_hiperboles()                      */
/*Metode que retorna el nombre de hiperboles de la regio */
/*********************************************************/

int Regio::numero_hiperboles()const
{
	return hiperboles.size();
}

/****************************************************/
/*   int Regio::hiperbolaMaximRangX()               */
/*Metode que retorna el maxim rang de les hiperboles*/
/****************************************************/

int Regio::hiperbolaMaximRangX()const
{
	int i=0;
	double rang=0;
	for(unsigned int j=0;j<hiperboles.size();j++)
	{
		double r_h=hiperboles[j].rangX();
		if(r_h>rang) 
		{
			rang=r_h;
			i=j;
		}
	}
	return(i);
}

int Regio::hiperbolaMaximRangY()const
{
	int i=0;
	double rang=0;
	for(unsigned int j=0;j<hiperboles.size();j++)
	{
		double r_h=hiperboles[j].rangY();
		if(r_h>rang) 
		{
			rang=r_h;
			i=j;
		}
	}
	return(i);
}

int Regio::hiperbolaMaximRang()const
{
	int i=0;
	double rang=0;
	for(unsigned int j=0;j<hiperboles.size();j++)
	{
		double r_h=max(hiperboles[j].rangY(),hiperboles[j].rangX());
		if(r_h>rang) 
		{
			rang=r_h;
			i=j;
		}
	}
	return(i);
}

/***************************************************************/
/*   Punt Regio::getPuntNoPertanyHiperbola(int index_hiperbola)*/
/*Metode que retorna el punt que no pertant a la hiperbola     */
/***************************************************************/

Punt2d Regio::getPuntNoPertanyHiperbola(int index_hiperbola)const
{
	if(hiperboles.size()==2)
	{
		double x;
		if(hiperboles[(index_hiperbola+1)%2].source().x()<hiperboles[(index_hiperbola+1)%2].target().x())
		{
			x=hiperboles[(index_hiperbola+1)%2].source().x()+hiperboles[(index_hiperbola+1)%2].rangX()/2;
		}
		else
		{
			x=hiperboles[(index_hiperbola+1)%2].target().x()+hiperboles[(index_hiperbola+1)%2].rangX()/2;
		}
		Punt2d punt(x, hiperboles[(index_hiperbola+1)%2].avaluarX(x));
		return(punt);
	}
	else
	{
		Punt2d no1 = hiperboles[index_hiperbola].source();
		Punt2d no2 = hiperboles[index_hiperbola].target();
	
		Hiperbola h=hiperboles[(index_hiperbola+1)%hiperboles.size()];
	
		if(h.source()!=no1 && h.source()!=no2) return(h.source());
	
		if(h.target()!=no1 && h.target()!=no2) return(h.target());
	}
}

/***************************************************************/
/*   void Regio::pintarRegio()                                 */
/*Metode que permet pintar les hiperboles que formen una regio */
/***************************************************************/

void Regio::pintarRegio(double lx, double ly)const
{
	if(hiperboles.size()==3)
	{
		bool recta= false;
		int index_recta;
		for(unsigned int j=0;j<3;j++)
		{
			if(hiperboles[j].ctu()==0 && hiperboles[j].ct()==1 && hiperboles[j].cu()==0)
			{
				recta=true;
				index_recta=j;
			}
		}
		if(recta)
		{
			int i0=(index_recta+1)%3, i1=(index_recta+2)%3;
		 	Punt p(hiperboles[i0].source().x(), hiperboles[i0].source().y(), 0.0);
			Punt mig=Punt3d(p.punt_mig(Punt3d(hiperboles[i0].target().x(), hiperboles[i0].target().y(), 0.0)));
			double y_0=hiperboles[i0].avaluarX(mig.x());
			double y_1=hiperboles[i1].avaluarX(mig.x());
//			hiperboles[i0].pintar();
	//		hiperboles[i1].pintar();
			if(y_0>y_1)
			{
				hiperboles[i0].pintarRegio(false,lx,ly);
				hiperboles[i1].pintarRegio(true,lx,ly);
			}
			else
			{
				hiperboles[i0].pintarRegio(true,lx,ly);
				hiperboles[i1].pintarRegio(false,lx,ly);
			}
		}
		else
		{
			int index=hiperbolaMaximRangX();
			Punt2d p = getPuntNoPertanyHiperbola(index);
			double valor_y=hiperboles[index].avaluarX(p.x());
			int ant=index-1;
			if(ant==-1) ant=hiperboles.size()-1;
			int post=(index+1)%hiperboles.size();
			double ass=hiperboles[index].getXAssimptota();
			bool ordre=true;
			if(ass!=10000)
			{
				if((ass<0 && p.x()<ass) || (ass>0 && ass<p.x())) //hem de canviar l'ordre
				{
					ordre=false;
				}
				else ordre=true;
			}
//			hiperboles[index].pintar();
			//hiperboles[ant].pintar();
			//hiperboles[post].pintar();
			if(valor_y<p.y())
			{
				hiperboles[index].pintarRegio(ordre,lx,ly);
				hiperboles[ant].pintarRegio(!ordre,lx,ly);
				hiperboles[post].pintarRegio(!ordre,lx,ly);
			}
			else
			{
				hiperboles[index].pintarRegio(!ordre,lx,ly);
				hiperboles[ant].pintarRegio(ordre,lx,ly);
				hiperboles[post].pintarRegio(ordre,lx,ly);
			}
		}
	}
	else if(hiperboles.size()==2)
	{
		Punt p(hiperboles[0].source().x(), hiperboles[0].source().y(), 0.0);
		Punt mig=Punt3d(p.punt_mig(Punt3d(hiperboles[0].target().x(), hiperboles[0].target().y(), 0.0)));
		double y_0=hiperboles[0].avaluarX(mig.x());
		double y_1=hiperboles[1].avaluarX(mig.x());
//		hiperboles[0].pintar();
//		hiperboles[1].pintar();
		if(y_0>y_1)
		{
			hiperboles[0].pintarRegio(false,lx,ly);
			hiperboles[1].pintarRegio(true,lx,ly);
		}
		else
		{
			hiperboles[0].pintarRegio(true,lx,ly);
			hiperboles[1].pintarRegio(false,lx,ly);
		}
	}
}

bool Regio::prepararRegio()
{
	bool dues_parts=false;
	
	if(hiperboles.size()==2)
	{
// 		double d0,d1;
		if(hiperboles[0].esPunt()||hiperboles[1].esPunt())
		{
			int i=0;
			if(hiperboles[1].esPunt()) i=1;
			if(hiperboles[(i+1)%2].avaluarPunt(hiperboles[i].source())>0) hiperboles[(i+1)%2].canviarSigne();
		}
		else
		{
			double d0,d1;
			d0=hiperboles[1].avaluarPunt(hiperboles[0].source());
			d1=hiperboles[1].avaluarPunt(hiperboles[0].target());
			double da=hiperboles[0].avaluarPunt(hiperboles[1].source());
			if(d0*d1>=0) // no  partir-la
			{
				if(d0>0) hiperboles[1].canviarSigne();
				if(fabs(da)<TOLERANCE)  da=hiperboles[0].avaluarPunt(hiperboles[1].target());
				if(da>0) hiperboles[0].canviarSigne();
			}
			else
 			{
				dues_parts=true;
				if(fabs(da)<TOLERANCE)
				{
					da=hiperboles[0].avaluarPunt(hiperboles[1].target());
					if(da*d1<0) hiperboles[1].canviarSigne();
				}
				else
					if(da*d0<0) hiperboles[1].canviarSigne();
 			}
		}
	}
	else
		cout<<"regio amb 1! hiperbola, no pot ser!!!!!!!"<<endl;
		
	return dues_parts;
}

void Regio::afegirHiperbola(const Hiperbola &h)
{
	hiperboles.push_back(h);
}

void Regio::afitarRegio(vector<bool> &rectangle,vector<vector<Punt2d> > &punts_rectangles,bool &aconseguit)const
{
	aconseguit=true;
	punts_rectangles.clear();
// 	rectangle=vector<bool>((int)hiperboles.size(),false);
	rectangle=vector<bool>(2,false);
	//TROBAR PUNTS PER FER UN BOUNDING BOX MÉS ACURAT
	vector<Punt2d> rect;
	for(unsigned int i=0;i<2;i++)
	{
		rect.clear();
		if(hiperboles[i].rangX()==0 || hiperboles[i].rangY()==0) 
			punts_rectangles.push_back(rect);
		else
		{
			double a1=hiperboles[i].ctu(), b1=hiperboles[i].ct(), c1=hiperboles[i].cu(), d1=hiperboles[i].cc();
			Punt2d p1=hiperboles[i].source();
			Punt2d p2=hiperboles[i].target();
			
			Punt2d u(p2.x()-p1.x(),p2.y()-p1.y());
			
			double pendent=u.y()/u.x();
			Punt2d aux((p1.x()+p2.x())/2.,(p1.y()+p2.y())/2.);
			if( (a1*aux.x()*aux.y()+b1*aux.x()+c1*aux.y()+d1)<=0)
			{
// 				cout<<"Hiperbola CONVEXA"<<endl;
				rectangle[i]=true;
				double arrel=sqrt(a1*d1*pendent-pendent*c1*b1);
				double x1=(-2.*pendent*c1+2.*arrel)/(2.*pendent*a1);
				double y1=-(b1+arrel)/a1;
				double x2=(-2.*pendent*c1-2.*arrel)/(2.*pendent*a1);
				double y2=-(b1-arrel)/a1;
				Punt2d punt_bo;
				if( ((x1<=p2.x() && x1>=p1.x()) || (x1>=p2.x() && x1<=p1.x())) && ((y1<=p2.y() && y1>=p1.y()) ||(y1>=p2.y() && y1<=p1.y())) ) 
					punt_bo=Punt2d(x1,y1);
				else if(((x2<=p2.x()&& x2>=p1.x())||(x2>=p2.x()&&x2<=p1.x())) && ((y2<=p2.y() && y2>=p1.y())||(y2>=p2.y()&& y2<=p1.y())))
					punt_bo=Punt2d(x2,y2);
				else{ rectangle[i]=false; aconseguit=false; /*cout<<"MERDAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA"<<endl;*/}
				
				double C=p1.x()*p2.y()-p2.x()*p1.y();
				double modul=sqrt(u.x()*u.x()+u.y()*u.y());
				double distancia=(-u.y()*punt_bo.x()+u.x()*punt_bo.y()+C)/(modul);
				
		//		cout<<"DISTANCIA: "<<distancia<<endl;
				Punt2d u_t((u.y()/modul)*distancia,(-u.x()/modul)*distancia);
				
				Punt2d rect_2,rect_3;
// 				rect.clear();
				rect.push_back(p1);
				rect.push_back(Punt2d(p1.x()+u_t.x(),p1.y()+u_t.y()));
				rect.push_back(Punt2d(p2.x()+u_t.x(),p2.y()+u_t.y()));
				if(a1*rect[1].x()*rect[1].y()+b1*rect[1].x()+c1*rect[1].y()+d1<0)
				{ rect[1]=Punt2d(p1.x()-u_t.x(),p1.y()-u_t.y()); rect[2]=Punt2d(p2.x()-u_t.x(),p2.y()-u_t.y());}
				rect.push_back(p2);
				punts_rectangles.push_back(rect);
			}
			else
				punts_rectangles.push_back(rect);
		}
	}
}


void Regio::pintarRegioAfitada(const vector<bool> & rectangle,const vector<vector<Punt2d> > &punts_rectangle)const
{
		if(hiperboles.size()==3)
		{
			Punt2d p_h1=hiperboles[0].source(),p_h2=hiperboles[1].target(),p_h3;
			if(hiperboles[2].source()!=p_h1 &&hiperboles[2].target()!=p_h2) p_h3=hiperboles[2].source();
			else p_h3=hiperboles[2].target();
// 			cout<<" t ";
		//	glColor3f(1,0,0);
			glBegin(GL_TRIANGLES);
				glVertex3f(p_h1.x(),p_h1.y(),0);
				glVertex3f(p_h2.x(),p_h2.y(),0);
				glVertex3f(p_h3.x(),p_h3.y(),0);
			glEnd();
// 			pixels_pas=pixels_pas+(int)( ((p_h1.distancia(p_h2)*p_h1.distancia(p_h3))/2.0)/area_pixel );
		}
// 		cout<<" "<<rectangle.size()<<" "<< punts_rectangle.size();
		for(unsigned int k=0;k<rectangle.size();k++)
			if(rectangle[k])
			{
// 				cout<<" r "<<punts_rectangle[k].size()<<endl;
				glBegin(GL_QUADS);
					glVertex3f(punts_rectangle[k][0].x(),punts_rectangle[k][0].y(),0);
					glVertex3f(punts_rectangle[k][1].x(),punts_rectangle[k][1].y(),0);
					glVertex3f(punts_rectangle[k][2].x(),punts_rectangle[k][2].y(),0);
					glVertex3f(punts_rectangle[k][3].x(),punts_rectangle[k][3].y(),0);
				glEnd();
			}
}

vector<vector<Punt2d> > Regio::mostrejar(const double &inc)const
{
	vector<vector<Punt2d> > p_hip;
	Punt2d vertex;

	bool reves0 = true;
	bool reves1 = true;
	
	if(Punt(hiperboles[0].source())==Punt(hiperboles[1].source()) ||Punt( hiperboles[0].source())==Punt(hiperboles[1].target()))
	{
		reves0=false;
		if(Punt(hiperboles[0].source())==Punt(hiperboles[1].source())) reves1=false;
	}
	else if(Punt(hiperboles[0].target())==Punt(hiperboles[1].source())) reves1=false;
	
	p_hip.push_back(hiperboles[0].mostrejar(inc,hiperboles[0].source(),hiperboles[0].target(),reves0));
	p_hip.push_back(hiperboles[1].mostrejar(inc,hiperboles[1].source(),hiperboles[1].target(),reves1));
	
	// en ppi la hiperboles[3] si es que existeix es una recta vertical q uneix les 2 hiperboles de veritat.
	char c;
	if(hiperboles.size()==3) 
		if(hiperboles[3].degenerada(c)) if(c!='v') cout<<"NO VERTICAL"<<endl;
	
	return p_hip;
	
	int n=hiperboles.size();
	int rM=0;
	if(n==3) rM=hiperbolaMaximRang();
	int id[2 ];
	for(int i=1;i<n;i++)
	   if(Punt(hiperboles[rM].source())==Punt(hiperboles[(rM+i)%n].source())) id[1]=i;
	   else if(Punt(hiperboles[rM].source())==Punt(hiperboles[(rM+i)%n].target())) id[1]=-i;
	if(n==3)
		for(int i=1;i<n;i++)   
			if(Punt(hiperboles[rM].target())==Punt(hiperboles[(rM+i)%n].target())) id[2]=i;
			else if(Punt(hiperboles[rM].target())==Punt(hiperboles[(rM+i)%n].source())) id[2]=-i;
	
	for(int i=1;i<n;i++)
	{
		p_hip.push_back(hiperboles[(rM+abs(id[i]))%n].mostrejar(inc,hiperboles[(rM+abs(id[i]))%n].source(),hiperboles[(rM+abs(id[i]))%n].target(),id[i]>0));
		if(i==1) vertex=p_hip[0][p_hip[0].size()-1];
	}
		
	p_hip.push_back(hiperboles[rM].mostrejar(inc,hiperboles[rM].source(),vertex,false));
	if(n==3) p_hip.push_back(hiperboles[rM].mostrejar(inc,vertex,hiperboles[rM].target(),false));
	
	
	return p_hip;
}



