#include "hiperbola.h"
#include "triangle.h"
#include "poligon_convex.h"
#include "segment.h"
#include "pla.h"

/******************************************************************************************/
/*   Segment::Segment()                                                                    */
/*Constructor que inicialitza cadascuna de les variables globals de l'objecte segment creat*/
/******************************************************************************************/

Segment::Segment ()  
{
	origen.modifica_coordenades(0.0, 0.0, 0.0);
	
	desti.modifica_coordenades(0.0, 0.0, 0.0);
	
	vector.converteix( desti, origen );
	
	segment_cgal= Segment3D(Punt3d(0.0, 0.0, 0.0), Punt3d(0.0, 0.0, 0.0));
	
	tipus_objecte = SEGMENT_VISIO;
}

/*********************************/
/*   Segment::~Segment()         */
/*Destructor de la classe Segment*/
/*********************************/

Segment::~Segment() {}

/******************************************************************************/
/*   Segment::Segment( Punt, Punt)                                             */
/*Constructor amb parametres que inicialitza els punts que defineixen l'segment*/
/******************************************************************************/

Segment::Segment (Punt a, Punt b, bool visio) 
{
	origen = a;
	desti = b;
	segment_cgal= Segment3D(Punt3d(a.x(), a.y(), a.z()), Punt3d(b.x(), b.y(), b.z()));
	vector.converteix(b, a);
	if(visio)
	{
// 		qDebug("SEGMENT VISIO:");
// 		qDebug("\t Origen: X:%f Y:%f Z:%f", a.x(), a.y(), a.z());
// 		qDebug("\t Desti: X:%f Y:%f Z:%f", b.x(), b.y(), b.z());
		tipus_objecte = SEGMENT_VISIO;
		transformacio = actualitzem_Transformacions_Skew_v(); 
	}
}

/******************************************************************************/
/*   Segment::Segment(Punt a, Punt b, Segment sv)                              */
/*Constructor amb parametres que inicialitza els punts que defineixen l'segment*/
/******************************************************************************/

Segment::Segment( Punt a,  Punt b,  Segment sv)
{
	origen = a;
	desti = b;
	segment_cgal= Segment3D(Punt3d(a.x(), a.y(), a.z()), Punt3d(b.x(), b.y(), b.z()));
	vector.converteix(b, a);
	
	tipus_objecte = SEGMENT_TERRENY;
	transformacio = actualitzem_Transformacions_Skew_s(sv);
}

/************************************************************/
/*   Segment::Segment(Segment3D s)                          */
/*Constructor que rep un Segment de tipus CGAL i es de visio*/
/************************************************************/

Segment::Segment( Segment3D s)
{
	origen = Punt(s.source().x(), s.source().y(), s.source().z());
	desti = Punt(s.target().x(), s.target().y(), s.target().z());
	segment_cgal= s;
	vector.converteix(desti, origen);
	
	/*tipus_objecte = SEGMENT_VISIO;
	transformacio = actualitzem_Transformacions_Skew_v();*/
}

/**************************************************************************************************************************/
/*   Segment::Segment(Segment3D s, Segment sv)                                                                            */
/*Constructor que rep un Segment de tipus CGAL i es del terreny, per tant indica el de visio per calcular la trnasformacio*/
/**************************************************************************************************************************/

Segment::Segment( Segment3D s, Segment sv)
{
	origen = Punt(s.source().x(), s.source().y(), s.source().z());
	desti = Punt(s.target().x(), s.target().y(), s.target().z());
	segment_cgal= s;
	vector.converteix(desti, origen);
	
	tipus_objecte = SEGMENT_TERRENY;
	transformacio = actualitzem_Transformacions_Skew_s(sv);
}

/***************************************************/
/*   Segment3D Segment::getSegment_cgal()          */
/*Metode que retorna el segment del cgal equivalent*/
/***************************************************/

const Segment3D & Segment::getSegment_cgal() const
{
	return segment_cgal;
}

/************************************************/
/*   Transformation3 Segment::getTransformacio()*/
/*Metode que retorna la transformacio al segment*/
/************************************************/

Transformation3 Segment::getTransformacio() const
{
	return transformacio;
}

/**********************************************************************/
/*   void Segment::setSegment_cgal(Segment3D s)                       */
/*Metode que permet modificar el segment actual a partir d'un del CGAL*/
/**********************************************************************/

void Segment::setSegment_cgal(Segment3D s)
{
	origen = Punt(s.source().x(), s.source().y(), s.source().z());
	desti = Punt(s.target().x(), s.target().y(), s.target().z());
	segment_cgal= s;
	vector.converteix(desti, origen);
}


const Punt3d  & Segment::source()const
{
	return getSegment_cgal().source();
}

const Punt3d  & Segment::target()const
{
	return getSegment_cgal().target();
}
/*****************************************************/
/*   Segment2D Segment::on_xy()                      */
/*Metode que retorna el segment actual en 2d del cgal*/
/*****************************************************/

Segment2D Segment::on_xy_segment() const
{
	Punt2d p(segment_cgal.source().x(),segment_cgal.source().y());
	Punt2d q(segment_cgal.target().x(),segment_cgal.target().y());
	Segment2D s(p,q);

	return s;
}

/************************************************/
/*   Vector2 Segment::on_xy()                   */
/*Metode que retorna el vector del segment en 2D*/
/************************************************/

Vector2D Segment::on_xy_vector() const
{
	Vector2D v(segment_cgal.to_vector().x(),segment_cgal.to_vector().y());
	return v;
}

/*************************************************/
/*   Punt Segment::getOrigen( )                   */
/*Metode que retorna el punt d'origen de l'segment*/
/*************************************************/

Punt Segment::getOrigen() const
{
	return origen;
}

/**********************************************/
/*   Punt Segment::getDesti()                  */
/*Metode que retorna el punt desti de l'segment*/
/**********************************************/

Punt Segment::getDesti() const
{
	return desti;
}

/******************************************************/
/*   Vector3D Segment::getVector()                    */
/*Metode que retorna el vector que defineix el segment*/
/******************************************************/

Vector3D Segment::getVector() const
{
	return vector;
}

/**********************************************************/
/*   void Segment::setOrigen(Punt)                         */
/*Metode que permet modificar el punt d'origen de l'segment*/
/**********************************************************/

void Segment::setOrigen(Punt a) 
{
	origen = a;
	vector.converteix(desti, origen);
	segment_cgal=Segment3D(Punt3d(origen.x(), origen.y(), origen.z()), Punt3d(desti.x(), desti.y(), desti.z()));
}

/**********************************************************/
/*   void Segment::setDesti(Punt)                          */
/*Metode que permet modificar el punt de desti de l'segment*/
/**********************************************************/

void Segment::setDesti(Punt b) 
{
	desti = b;
	vector.converteix(desti, origen);
	segment_cgal=Segment3D(Punt3d(origen.x(), origen.y(), origen.z()), Punt3d(desti.x(), desti.y(), desti.z()));
}

/***************************************************************/
/*   void Segment::setVector(Vector3D)                         */
/*Metode que permet modificar el vector que defineix el segment*/
/***************************************************************/

void Segment::setVector(Vector3D v) 
{
	vector = v;
}

bool Segment::es_degenerat()const  // true si l'origen es igual al desti.
{
	return origen==desti;
}

/****************************************/
/*   void Segment::midaSegment()          */
/*Metode que calcula la mida de l'segment*/
/****************************************/

double Segment::midaSegment() const
{
	return (sqrt( ((origen.x() - desti.x()) * (origen.x() - desti.x())) +
                      ((origen.y() - desti.y()) * (origen.y() - desti.y())) +
                      ((origen.z() - desti.z()) * (origen.z() - desti.z()))
               ));
}
double Segment::midaSegmentVertical2d() const
{
	return fabs(desti.y()-origen.y());
}


/***************************************************************************************/
/*   bool Segment::interseccioSegments(Segment s2)                                     */
/*A partir d'aquest metode podem saber si existeix o no interseccio entre els segments */
/***************************************************************************************/

bool Segment::interseccioSegments2D(const Segment &s2,const bool &extrems) const
{

	double d=-(vector.y()*origen.x()-vector.x()*origen.y());
	double avaluar_o=vector.y()*s2.getOrigen().x()-vector.x()*s2.getOrigen().y()+d;
	double avaluar_d=vector.y()*s2.getDesti().x()-vector.x()*s2.getDesti().y()+d;
	
	if(fabs(avaluar_o)>TOLERANCE&& fabs(avaluar_d)>TOLERANCE && avaluar_o*avaluar_d>0) return false;  // no interseccionen amb la recta
	
	if(!extrems&&(fabs(avaluar_o)<TOLERANCE|| fabs(avaluar_d)<TOLERANCE))  return false; // interseccio amb la recta es en els extrems de s2
	
	Vector3D v2=s2.getVector();
	double d2=-(v2.y()*s2.getOrigen().x()-v2.x()*s2.getOrigen().y());
	double avaluar_o2=v2.y()*origen.x()-v2.x()*origen.y()+d2;
	double avaluar_d2=v2.y()*desti.x()-v2.x()*desti.y()+d2;
	
	if(fabs(avaluar_o2)>TOLERANCE&& fabs(avaluar_d2)>TOLERANCE && avaluar_o2*avaluar_d2>0) return false;  // no interseccionen amb la recta
	
	if(!extrems&&(fabs(avaluar_o2)<TOLERANCE|| fabs(avaluar_d2)<TOLERANCE))  return false; // interseccio amb la recta es en els extrems de 

	return true;

}

bool Segment::interseccioSegments(const Segment &s2) const
{
	bool interseccio=false;
	Vector3D v1, v2;
	Punt primer, segon;
	
	v1.converteix( desti, origen );
	primer = s2.getOrigen();
	segon = s2.getDesti();
	v2.converteix( primer, segon );

	escriure();
	s2.escriure();	
	/*En funcio de la posicio dels extrems dels segments determinarem l'existencia de interseccio entre ells*/
	if(((v1.esquerraVector(origen, primer))&&(!v1.esquerraVector(origen, segon)))|| ((v1.esquerraVector(origen,segon))&&(!v1.esquerraVector(origen, primer))))
	{
		if(((v2.esquerraVector(segon, origen)) &&(!v2.esquerraVector(segon, desti)))|| ((v2.esquerraVector(segon,desti))&&(!v2.esquerraVector(segon, origen))))
		{
			/*Existeix interseccio entre segments*/
			interseccio=true;
		}
	}
	cout<<interseccio<<endl;
	return(interseccio);
}

/*******************************************************************/
/*   Punt Segment::puntInterseccio(Segment * s2)                   */
/*Mètode que permet calcular el punt d'interseccio entre 2 segments*/
/*******************************************************************/

Punt Segment::puntInterseccio( const Segment &s2, bool espai) const
{
	Punt interseccio(0.,0.0,0.);
	Vector3D vector_u, vector_w, vector_q;
	double d, alfa, beta, valor1, valor2;
	
	vector_w = getVector();
	vector_u = s2.getVector();
	vector_q.converteix(s2.getOrigen(), origen);
	
	d = ( -vector_u.x()*vector_w.y() ) + ( vector_u.y()*vector_w.x() );
	
	if( fabs(d)>TOLERANCE )
	{
		alfa = (vector_q.x()*vector_w.y() - vector_q.y()*vector_w.x()) * (1/d);
		beta = (vector_u.y()*vector_q.x() - vector_u.x()*vector_q.y()) * (1/d);
		if( (alfa>=0.0 && alfa<=1.0) && (beta>=0.0 && beta<=1.0))
		{
			interseccio.modifica_x( s2.getOrigen().x() + alfa*vector_u.x() );
			interseccio.modifica_y( s2.getOrigen().y() + alfa*vector_u.y() );
			if(espai)
			{
				interseccio.modifica_z( s2.getOrigen().z() + alfa*vector_u.z() );
			}
		}
	}
	else
	{
		if(vector_w.x()!=0 && vector_w.y()!=0) 
		{
			valor1 = ( s2.getOrigen().x()-origen.x() ) / vector_w.x();
			valor2 = ( s2.getOrigen().y()-origen.y() ) / vector_w.y();
			if( valor1 == valor2)/*En aquest cas es tracta de segments coincidents, de manera que descartem que es tracti de rectes paral.leles*/
			{
				/*RECTES COINCIDENTS*/
			}
		}
	}
	return interseccio;
}

/*******************************************************************************************************************************/
/*   bool Segment::puntInterseccioSuperior(Segment * s2)                                                                       */
/*Mètode que permet calcular si la coordenada Z del punt d'interseccio entre 2 segments (terreny-recta) és superior en la recta*/
/*******************************************************************************************************************************/

bool Segment::puntInterseccioSuperior( const Segment & s2) const
{
	Vector3D vector_u, vector_w, vector_q;
	double d, alfa, beta, z_recta, z_terreny, valor1, valor2;
	
	vector_w = getVector();
	vector_u = s2.getVector();
	vector_q.converteix(s2.getOrigen(), origen);
	
	d = ( -vector_u.x()*vector_w.y() ) + ( vector_u.y()*vector_w.x() );
	
	if( d!=0.0 )
	{
		alfa = (vector_q.x()*vector_w.y() - vector_q.y()*vector_w.x()) * (1/d);
		beta = (vector_u.y()*vector_q.x() - vector_u.x()*vector_q.y()) * (1/d);
		
		if( (alfa>=0.0 && alfa<=1.0) && (beta>=0.0 && beta<=1.0))
		{
			z_recta = (origen.z()) + (beta*vector_w.z());
			z_terreny = s2.getOrigen().z() + alfa*vector_u.z();
			
			if(z_recta > z_terreny)
			{
				/*qDebug("Coordenada Z de la recta: %f",z_recta);
				qDebug("Coordenada Z del terreny: %f",z_terreny);*/
				return true;
			}
		}
	}
	else
	{
		if(vector_w.x()!=0 && vector_w.y()!=0) 
		{
			valor1 = ( s2.getOrigen().x()-origen.x() ) / vector_w.x();
			valor2 = ( s2.getOrigen().y()-origen.y() ) / vector_w.y();
			if( valor1 == valor2)/*En aquest cas es tracta de segments coincidents, de manera que descartem que es tracti de rectes paral.leles*/
			{
				/*RECTES COINCIDENTS*/
			}
		}
	}
	return false;
}

/***************************************************************************************************************/
/*   bool Segment::segmentCorrecte(Delaunay, Segment)                                                          */
/*Mètode que indica si el segment indicat de visio es o no acceptable, és a dir es troba per sobre del terreny */
/***************************************************************************************************************/

bool Segment::segmentCorrecte(Delaunay * dt)
{
	bool correcte = true, intersecta = true;
	pCara cara;
	Punt3d inici_anterior, fi_anterior;
	int i;
	
	Punt inici, fi;
	Segment s2;
	bool tractar=false;
	
	/*A partir de la triangulació determinem la cara sobre la qual es trobaria el punt introduit com a inici de segment sense tenir en compte la coordenada z. A continuació i sabent en tot moment el punt de fi de segment comprovem que tots els fragments de segment es trobin per sobre de les cares per on passa el segment*/
	
	/*Ens assegurem de que tan el punt d'inici com el de fi de segment es troben per sobre de la triangulació */
	if (origen.puntCorrecte(dt) && desti.puntCorrecte(dt))
	{
		cara = dt->locate(Punt3d(origen.x(), origen.y(), origen.z()));
		/*A continuació desglossem el segment comprovant que aquets talls no interseccionin amb la triangulacio i es trobin en tot moment per sobre d'aquesta*/
		while(correcte && intersecta)
		{
			i=0;
			intersecta = false;
			while(i<3 && !intersecta) /*Calculem quina de les arestes de la cara intersecta amb el segment*/
			{
				if( cara->vertex(i)->point() != fi_anterior  && cara->vertex((i+1)%3)->point() != inici_anterior)
				{
					inici.modifica_coordenades( cara->vertex(i)->point().x(), cara->vertex(i)->point().y(), cara->vertex(i)->point().z() );
					fi.modifica_coordenades( cara->vertex((i+1)%3)->point().x(), cara->vertex((i+1)%3)->point().y(), cara->vertex((i+1)%3)->point().z() );
					
					s2 = Segment( inici, fi);
					if(interseccioSegments(s2))
					{
						inici_anterior = cara->vertex(i)->point();
						fi_anterior = cara->vertex((i+1)%3)->point();
						intersecta = true;
					}
					tractar=true;
				}
				i++;
			}
			
			if(intersecta)
			{
				cara = cara->neighbor((i+1)%3); /*Obtenim la cara veina corresponent a l'aresta que intersecciona*/
			
				/*qDebug("PUNTS ARESTA");
				qDebug("Inici: X:%f Y:%f Z:%f",inici_anterior.x(), inici_anterior.y(), inici_anterior.z());
				qDebug("Final: X:%f Y:%f Z:%f",fi_anterior.x(), fi_anterior.y(), fi_anterior.z());
				qDebug("PUNTS CARA");
				qDebug("Primer: X:%f Y:%f Z:%f",cara->vertex(0)->point().x(), cara->vertex(0)->point().y(), cara->vertex(0)->point().z());
				qDebug("Segon: X:%f Y:%f Z:%f",cara->vertex(1)->point().x(), cara->vertex(1)->point().y(), cara->vertex(1)->point().z());
				qDebug("Tercer: X:%f Y:%f Z:%f",cara->vertex(2)->point().x(), cara->vertex(2)->point().y(), cara->vertex(2)->point().z());*/
				/*Un cop obtinguda l'aresta que intersecciona amb el segment comprovem si el punt d'interseccio es troba per sobre de la cara o no*/
				if( tractar && !puntInterseccioSuperior(s2) )
					correcte = false;
			}
		}
	}
	else
		correcte = false;
	
	return correcte;
}

/***********************************************************************/
/*   Segment Segment::fragment_segment(pCara fh)                 */
/*Mètode que retorna el fragment del segment que conte la cara indicada*/
/***********************************************************************/

Segment Segment::fragment_segment(const pCara &fh)const
{
	Segment fragment;
	Segment aresta = Segment();
	int conta=0, i=0;
	Punt p1, p2;
	bool trobat=false;
	
	while(i<3 && !trobat)
	{
		aresta.setOrigen(Punt(fh->vertex(i)->point().x(), fh->vertex(i)->point().y(), fh->vertex(i)->point().z()));
		aresta.setDesti(Punt(fh->vertex((i+1)%3)->point().x(), fh->vertex((i+1)%3)->point().y(), fh->vertex((i+1)%3)->point().z()));
		
		if(interseccioSegments(aresta))
		{
			if(conta==1)
			{
				p2=puntInterseccio(aresta, true);
				trobat=true;
			}
			else
			{
				p1= puntInterseccio(aresta,true);
				conta++;
			}
		}
		i++;
	}
	
	if(p1.y()<p2.y())
	{
		fragment.setOrigen(p2);
		fragment.setDesti(p1);
	}
	else
	{
		fragment.setOrigen(p2);
		fragment.setDesti(p1);
	}
	return fragment;
}

/***********************************************/
/*   Segment Segment::twin()                   */
/*Mètode que retorna el twin del segment actual*/
/***********************************************/

Segment Segment::twin() const
{
	Segment twin;
	
	twin.setOrigen(desti);
	twin.setDesti(origen);
	return twin;
}

/********************************************************************************************/
/*  double Segment::distancia(Punt punt)                                                    */
/*Metode que retornara la distancia des d'un punt qualsevol p al punt mes proper del segment*/
/********************************************************************************************/

double Segment::distancia2D(const Punt &punt) const
{
	double modul=0.0;
	Vector3D vect;
	Punt proper;

	proper=calculaProperRecta2D(punt);
	/*La distancia entre 2 punts es el modul del vector que els uneix*/
	vect.modifica_coordenades(punt.x()-proper.x(), punt.y()-proper.y(), 0.0);
	modul=vect.calcula_modul();
	
	return(modul);
}

/***********************************************************************************************************/
/*  Punt Segment::calculaProperRecta(Punt p)                                                               */
/*Metode que calcula el punt mes proper del segment definit pels punts dels parametres, fins al punt actual*/
/***********************************************************************************************************/

Punt Segment::calculaProperRecta2D(const Punt &p) const
{
	double landa, det_AX_AB_perpendicular, det_AB_AB_perpendicular;
	Vector3D vector_AX, vector_AB, vector_AB_perpendicular;
	Punt proper;

	vector_AX.modifica_coordenades(p.x()-origen.x(), p.y()-origen.y(), 0.0);
	vector_AB.modifica_coordenades(desti.x()-origen.x(), desti.y()-origen.y(), 0.0);

	if(fabs(vector_AB.x())>0.0000001)
	{
		vector_AB_perpendicular.modifica_coordenades(vector_AB.y(), (-1)*vector_AB.x(), 0.0);
	}
	else
	{
		vector_AB_perpendicular.modifica_coordenades((-1)*vector_AB.y(), vector_AB.x(), 0.0);
	}

	det_AX_AB_perpendicular=(vector_AX.x()*vector_AB_perpendicular.y())-(vector_AX.y()*vector_AB_perpendicular.x());
	det_AB_AB_perpendicular=(vector_AB.x()*vector_AB_perpendicular.y())-(vector_AB.y()*vector_AB_perpendicular.x());

	landa=det_AX_AB_perpendicular/det_AB_AB_perpendicular;

	/*En funcio del valor de landa el punt mes proper equivaldra a un dels extrems del segement, o be, a un punt interior d'aquest */
	if(landa<=0.00000001)
	{
		proper=origen;
	}
	else
	{
		if(landa>=1.0)
		{
			proper=desti;
		}
		else
		{
			proper.modifica_coordenades(origen.x()+(landa*vector_AB.x()), origen.y()+(landa*vector_AB.y()),0.0);
		}
	}

	return(proper);
}

/**********************************************************/
/*  void Segment::actualitzem_Transformacions_Skew_v()    */
/*Metode que calcula la transformacio pel segment de visio*/
/**********************************************************/

Transformation3 Segment::actualitzem_Transformacions_Skew_v()
{
// 	qDebug("TRANSFORMACIO V");
	Punt3d pt(segment_cgal.source());
	
	Transformation3 Transf0(1,0,0,-pt.x(),0,1,0,-pt.y(),0,0,1,-pt.z(),1);
	Punt3d pt0_=Transf0(segment_cgal.target());
	
	double d0_xy=sqrt(pt0_.x()*pt0_.x()+pt0_.y()*pt0_.y()), c_a=pt0_.x()/d0_xy,s_a=pt0_.y()/d0_xy;
	Transformation3 Transf1(c_a,-s_a,0,0,s_a,c_a,0,0,0,0,1,0,1);
	Transf1=Transf1.inverse();	
	Punt3d pt1_=Transf1(pt0_);
	
	double c_b=pt1_.x(),s_b=pt1_.z();
	
	Transformation3 Transf2(c_b,0,-s_b,0, 0,1,0,0 ,s_b,0,c_b,0,1);
	Transf2=Transf2.inverse();
	Punt3d pt2_=Transf2(pt1_);
	
	return Transf2 * (  Transf1* Transf0 );
}

/*************************************************************/
/*  void Segment::actualitzem_Transformacions_Skew_s()       */
/*Metode que calcula la transformacio pel segment del terreny*/
/*************************************************************/

Transformation3 Segment::actualitzem_Transformacions_Skew_s( Segment sv)
{
// 	qDebug("TRANSFORMACIO S");
	Punt3d pt_s(sv.getTransformacio()(segment_cgal.source()));
	double ds_yz=sqrt(pt_s.y()*pt_s.y()+pt_s.z()*pt_s.z());
	double c_g=pt_s.z()/ds_yz;
	double s_g=pt_s.y()/ds_yz;
	
	Transformation3 Transf3(1,0,0,0, 0,s_g,c_g,0, 0,c_g,-s_g,0, 1);
	Transf3=Transf3.inverse();
	
	// Transformades relatives al segment MG::ss q necessitem cada vegada.
	Punt3d r_(sv.getTransformacio()(segment_cgal.source())); 
	Vector s_(sv.getTransformacio()(segment_cgal.to_vector()));
	Punt3d sour=(Transf3 * sv.getTransformacio())(sv.getSegment_cgal().source());
	Punt3d tar=(Transf3 * sv.getTransformacio())(sv.getSegment_cgal().target());
	Punt3d sours=(Transf3 * sv.getTransformacio())(segment_cgal.source());
	Punt3d tars=(Transf3 * sv.getTransformacio())(segment_cgal.target());
	
	return Transf3 * sv.getTransformacio();
}

/*********************************************************/
/*  Punt2d Segment::punt_mig_2d()                        */
/*Metode que calcula el punt mig del segment actual en 2d*/
/*********************************************************/

Punt2d Segment::punt_mig_2d() const
{
	Punt2d pm;
	pm=on_xy_segment().source()+1./2.*on_xy_segment().to_vector();
	return pm;
}

/*********************************************************/
/*  Punt3d Segment::punt_mig_3d()                         */
/*Metode que calcula el punt mig del segment actual en 3d*/
/*********************************************************/

Punt3d Segment::punt_mig_3d() const
{
	return Punt3d((origen.x()+desti.x())/2.,(origen.y()+desti.y())/2.,(origen.z()+desti.z())/2.);
	Punt3d pm;
	pm=segment_cgal.source()+1./2.*segment_cgal.to_vector();
	
	return pm;
}

/**************************************************************/
/*   vector<double> Segment::skew_segment(const Segment3D &s)*/
/*Metode que calcula un vector de valors de skew segment      */  
/**************************************************************/

// VectorDoubles Segment::skew_segment(const Segment &ss) const
VectorDoubles  Segment::skew_segment(const Segment &ss,Punt2d &p, Punt2d &q)const
{
	// calculem els coeficients de les hiperboles.
	VectorDoubles skew;
	Punt3d r_;
	Vector s_;
	
	s_= ss.getTransformacio()(ss.getSegment_cgal().to_vector());
	r_= ss.getTransformacio()(ss.getSegment_cgal().source());
	// calculem la transformada del segmetn s segons les transformacions ja calculades.
	Punt3d p_(ss.getTransformacio()(segment_cgal.source()));
	Vector e_(ss.getTransformacio()(segment_cgal.to_vector()));
	
	skew.push_back(s_.z()*r_.y()*(s_.z()*e_.y()-s_.y()*e_.z())); 
	skew.push_back(r_.y()*s_.z()*(p_.y()*e_.z()-r_.y()*e_.z()-p_.z()*e_.y()));  
	skew.push_back(r_.y()*(s_.x()*s_.z()*(p_.z()*e_.y()-p_.y()*e_.z())+s_.z()*s_.z()*(p_.y()*e_.x()-p_.x()*e_.y())+s_.y()*s_.z()*(p_.x()*e_.z()-p_.z()*e_.x())));
	skew.push_back( r_.y()* s_.z()* (r_.y()*( p_.x() * e_.z() -e_.x()*p_.z())+r_.x()*(-p_.y()*e_.z()+e_.y()*p_.z()))); 
	
	
	double t=0,u=0,den;
	double a,b,c,d;
	for(int i=0;i<2;i++)
	{
		if(i==1){p=Punt2d(t,u); p_=Punt3d(p_.x()+e_.x(),p_.y()+e_.y(),p_.z()+e_.z());
// 		p_=ss.getTransformacio()(segment_cgal.target());
		}
		a= s_.y()*p_.z()-s_.z()*(p_.y()-r_.y());
		b= s_.z()*(p_.x()-r_.x())-s_.x()*p_.z();
		c= s_.x()*(p_.y()-r_.y())-s_.y()*(p_.x()-r_.x());
		d=0.-a*r_.x()-b*r_.y();
// 		cout<<a<<" "<<b<<" "<<c<<" "<<d<<";"<<endl;
		if(fabs(a)<TOLERANCE) cout<<"dos plans parallels o coincidents quan busquem imatge segment"<<endl;
		else
		{
			t=-d/a;
			if(fabs(p_.y())>TOLERANCE)
			{
				den=p_.y()*s_.z()-s_.y()*p_.z();
				if(fabs(den)<TOLERANCE) 
					cout<<"den =0 a imatge segment"<<endl;
				else
				{ /*cout<<"be ";*/	u=r_.y()*p_.z()/den; }
			}
			else if(fabs(p_.z())<TOLERANCE)
				cout<<"busquem imatge skew un punt sobre linia conte sv"<<endl;
			else
			 {
			 	u=0;
				if(!(fabs(s_.x())<TOLERANCE && fabs(r_.y())<TOLERANCE && fabs(r_.x()-t)<TOLERANCE))
					cout<<"la imatge skew es un SI"<<endl;
// 				else
// 					cout<<" be ";
			 }
		}
	}
	q=Punt2d(t,u);
	

	return skew;
}

/*************************************************************/
/*   vector<double> Segment::skew_segment(const Segment3D &s)*/
/*Metode que retorna el punt de e amb coordenada t           */
/*************************************************************/

Punt3d Segment::e_t(const double &t) const
{
	return  segment_cgal.source() + t*segment_cgal.to_vector();
}

/**************************************************************************************************************************/
/*   double Segment::coord_p_on_e(const Punt2d &p)                                                                        */
/*Metode que retorna la coordenada de p en el segment s parametereitzat de s(1) a s(2) per 0,1, si p no es de s retorna -1*/
/**************************************************************************************************************************/

double Segment::coord_p_on_e(const Punt3d &p)  const
{
	double lambda;
	int i0=0;
	bool colineal = true;

	while(i0<3 && fabs(segment_cgal.to_vector().cartesian(i0))<TOLERANCE )  i0++;
	
	if(i0==3)
	{ 
		cout<<"ERROR, segment degenerat (coord_p_on_e)"<<endl;
		i0=2;
	}
	
	lambda=(p.cartesian(i0)-segment_cgal.source().cartesian(i0))/(segment_cgal.to_vector().cartesian(i0));
	for(int j=0;j<2;j++)
	{
		double lambda1=fabs(p.cartesian(j)-(segment_cgal.source().cartesian(j)+lambda*segment_cgal.to_vector().cartesian(j)));
	        if(lambda1>TOLERANCE) 
		{
// 			qDebug(" coord_p_on_e, dif lambdes= %f, i= %d, i0= %d",lambda1,j,i0);
			colineal=false;
		}
	}
	return lambda;
}

/**************************************************************************************************************************/
/*   double Segment::punt_sobre(const Punt2d &p)                                                                        */
/*Metode que retorna la coordenada de p en el segment s parametereitzat de s(1) a s(2) per 0,1, si p no es de s retorna -1*/
/**************************************************************************************************************************/

bool Segment::punt_sobre(const Punt3d &p) const
{
	double lambda;
	int i0=0;
	bool colineal = true;
	
	Vector3D v;
	v.converteix(Punt(p),origen);
	if((v.producte_vectorial(getVector())).calcula_modul2()>TOLERANCE)
		return false;

	while(i0<3 && fabs(segment_cgal.to_vector().cartesian(i0))<TOLERANCE )
		i0++;
	if(i0==3)
		return Punt(p)==getOrigen();
	else
	{
		lambda=(p.cartesian(i0)-segment_cgal.source().cartesian(i0))/(segment_cgal.to_vector().cartesian(i0));
		for(int j=0;j<3;j++)
			if(fabs(p.cartesian(j)-(segment_cgal.source().cartesian(j)+lambda*segment_cgal.to_vector().cartesian(j)))>TOLERANCE)
				colineal=false;
		return colineal;
	}
}

/*****************************************************************/
/*   bool Segment::inSegment2D(Punt p)const                      */
/*Metode que permet saber si un punt es troba sobre un segment 2D*/
/*****************************************************************/

bool Segment::inSegment2D(const Punt &p)const
{
	//amb tolerancia
// 	cout<<"eooooo"<<endl;
	double lambda;
	if(fabs(vector.x())>TOLERANCE) 
	{
		lambda=(p.x()-origen.x())/vector.x();
		return (fabs((p.y()-(origen.y()+lambda*vector.y())))<TOLERANCE);
	}
	
	if(fabs(vector.y())>TOLERANCE) 
	{
		lambda=(p.y()-origen.y())/vector.y();
		return (fabs((p.x()-(origen.x()+lambda*vector.x())))<TOLERANCE);
	}
	
	return (fabs(p.x()-origen.x())<TOLERANCE && fabs(p.y()-origen.y())<TOLERANCE);

	//sense tolerancia
	if(origen.x()!=desti.x())
	{
		if(origen.x()<=p.x()&&p.x()<=desti.x())
			return true;
		if(origen.x()<=p.x() && p.x()>=desti.x())
			return true;
	}
	else
	{
		if(origen.y()<=p.y() &&p.y()<=origen.y())
			return true;
		if(origen.y()>=p.y()&&p.y()>=origen.y())
			return true;
	}
	return false;
}

/**********************************************************************/
/*   bool Segment::coplanar_segment_3_intersection(const Segment3 &s) */
/*Metode que informa si els segments son coplanars o no               */
/**********************************************************************/

bool Segment::coplanar_segment_3_intersection(const Segment3D &s) const
{
	qDebug("Segment coplanars, provant interseccio, MOLT PROBLABLE q ESTIGUI MALAMENT");

	CGAL::Object result;
	Segment seg(s);
	result=CGAL::intersection(on_xy_segment(),seg.on_xy_segment().supporting_line());

	Segment2D se;
	Punt2d pu;

	if (CGAL::assign(pu,result))
		return true;
	
	if(CGAL::assign(se,result))
		return true;
	
	return false;
}

/******************************************************************************************/
/*   bool Segment::coplanar_segment_3_intersection(const Segment3 &s,vector<Punt3d3> &p)   */
/*Metode que informa si els segments son coplanars o no, i retorna els punts d'interseccio*/
/******************************************************************************************/

bool Segment::coplanar_segment_3_intersection(const Segment3D &s, VectorPunt3d &p) const
{
	qDebug("segment coplanars, provant interseccio, NO S'HA COMPROVAT QUE VAGI BE");

	CGAL::Object result;	//Segment2 e_2d=on_xy(e);	//Segment2 s_2d=on_xy(s);

	Segment seg(s);
	result=CGAL::intersection(on_xy_segment(),seg.on_xy_segment().supporting_line());

	Segment2D se;
	Punt2d pu;
	Linia linia(segment_cgal.supporting_line());

	if (CGAL::assign(pu,result)){
		p.push_back(linia.on_l(pu));
		return true;
	}
	else
		if(CGAL::assign(se,result))
		{
			p.push_back(linia.on_l(se.source()));
			p.push_back(linia.on_l(se.target()));
			return true;
		}
		else return false;
}

/**************************************************************/
/*   bool Segment::e_intersects_T(const Tetrahedron &Tvs)     */
/*Metode que informa si el segment intersecciona el tetraedre */
/**************************************************************/

bool Segment::e_intersects_T(const Tetrahedron &Tvs) const
{
	bool ret=false;
		
	if(Tvs.is_degenerate ()) qDebug("Tetraedre degenerat, esta en un pla");
	
	if(Tvs.has_on_bounded_side(segment_cgal.source())) ret=true;
	else if(Tvs.has_on_bounded_side(segment_cgal.target())) ret=true;
	else 
	{
		
		if(!(fabs(segment_cgal.source().x()-segment_cgal.target().x())<TOLERANCE && fabs(segment_cgal.source().y()-segment_cgal.target().y())<TOLERANCE && fabs(segment_cgal.source().z()-segment_cgal.target().z())<TOLERANCE))
		{
			VectorPunt3d p;
			for(int i=0;i<4 && !ret;i++) // estudiem cada cara
			{
				Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
				ret=tr.triangle_intersection_e_obert(segment_cgal,p);
			}
		}
	}
	return ret;
}

/*  Si contingutFrontera, retorna cert i no mira si es una aresta de T o no. */

bool Segment::interseccio(const Tetrahedron &Tvs, bool &contingutFrontera, bool extremsS, bool vertexT,bool arestesT) const
{
		
	if(Tvs.is_degenerate ()) qDebug("Tetraedre degenerat, esta en un pla");
	
	if(Tvs.has_on_bounded_side(segment_cgal.source())) return true;
	else if(Tvs.has_on_bounded_side(segment_cgal.target())) return true;
	else 
	{
		for(int i=0;i<4;i++) // estudiem cada cara
		{
			Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
			if( tr.interseccio(*this,contingutFrontera,extremsS,vertexT,arestesT) ) return true;
		}
	}
	return false;
}
bool Segment::interseccio(const Tetrahedron &Tvs,  bool extremsS, bool vertexT,bool arestesT) const
{
	bool cont;
	return interseccio(Tvs, cont, extremsS, vertexT, arestesT);
}

bool Segment::interseccio(const Tetrahedron &Tvs, VectorPunt3d &vp, bool &contingutFrontera, bool vertexT,bool arestesT) const
{

	if(Tvs.is_degenerate ()) qDebug("Tetraedre degenerat, esta en un pla");
	
	if(Tvs.has_on_bounded_side(segment_cgal.source())) vp.push_back(origen.getPunt());
	if(Tvs.has_on_bounded_side(segment_cgal.target())) vp.push_back(desti.getPunt());
	if(vp.size()<2) 
	{
		for(int i=0;i<4;i++) // estudiem cada cara
		{
			Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
			Punt3d p;
			
			if(tr.interseccio(*this, p, contingutFrontera, true, vertexT, arestesT))
			{
				vp.push_back(p);
				if(contingutFrontera) 
					cout<<"Segment contingut a la frotnera, falta trobar pt final interseccio"<<endl;
			}
		}
	}
	return false;
}

bool Segment::interseccio(const Segment &s) const
{
	 Vector3D u,v,w,w2;
	 u.converteix(desti,origen);
	 v.converteix(s.getDesti(),s.getOrigen());
	 w.converteix(origen,s.getOrigen());
	 w2.converteix(origen, s.getOrigen());
	 
	 if(fabs(u.producte_vectorial(v).calcula_modul2())<TOLERANCE) // coplanars inersecc coordenades x i y (si no estan en el pla z=0).
	 {
	 	if(fabs(u.x())<TOLERANCE&&fabs(v.x())<TOLERANCE)
		{
			u.modifica_coordenades(u.y(),u.z(),u.x());
			v.modifica_coordenades(v.y(),v.z(),v.x());
			w.modifica_coordenades(w.y(),w.z(),w.x());
			w2.modifica_coordenades(w2.y(),w2.z(),w2.x());
			
		}
		else if(fabs(u.y())<TOLERANCE&&fabs(v.y())<TOLERANCE)
		{
			u.modifica_coordenades(u.z(),u.x(),u.y());
			v.modifica_coordenades(v.z(),v.x(),v.y());
			w.modifica_coordenades(w.z(),w.x(),w.y());
			w2.modifica_coordenades(w2.z(),w2.x(),w2.y());
			
		}
		
		double D=u.x()*v.y()-u.y()*v.x();
		if(fabs(D)<TOLERANCE)
		{
			if(fabs(u.x()*w.y()-u.y()*w.x())!=0 ||fabs(v.x()*w.y()-v.y()*w.x())!=0 )
				return false;
			double du=u.x()*u.x()+u.y()*u.y();	
			double dv=v.x()*v.x()+v.y()*v.y();	
			if(du==0&&dv==0)
			{
				if(origen==s.getOrigen())
					return true;
				return false;
			}
			if(du==0)
				return s.punt_sobre(origen.getPunt());
			if(dv==0)
				return punt_sobre(s.getOrigen().getPunt());
				
			double t0,t1;
			if(fabs(v.x())>TOLERANCE)
			{
				t0=w.x()/v.x();
				t1=w2.x()/v.x();
			}
			else
			{
				t0=w.y()/v.y();
				t1=w2.y()/v.y();
			}
			if(t0>t1)
			{
				double t=t0;
				t0=t1;
				t1=t;
			}
			if(t0>1 ||t1<0)
				return false;
			t0=max(0,t0);
			t1=min(1,t1);
			if(t0==t1)
				return true;
			return true; //they overlap from t0 to t1.
		}
		double sI=v.x()*w.y()-v.y()*w.x();
		if(sI<0||sI>1)
			return false;
			
		double tI=u.x()*w.y()-u.y()*w.x();
		if(tI<0||tI>1)
			return false;
		return true;
		
	}
	return false;
}

bool Segment::interseccio(const Segment &s,Punt &p) const
{
	Punt q;
	Vector3D u=vector;
	Vector3D v=s.getVector();
	Vector3D w;
	w.converteix(origen,s.getOrigen());
	
	double D=u.perpendicular2D(v);
	
	if(fabs(D)<0.0000000001)
	{
		if(u.perpendicular2D(w)!=0 || v.perpendicular2D(w)!=0)   //NO COLINEALS
			return false;
		double du=u*u;
		double dv=v*v;
		if(du==0&&dv==0)
		{
			if(origen!=s.getOrigen())
				return false;
			p=origen;
			return true;
		}
		if(du==0)
		{
			if(!s.inSegment2D(origen))
				return false;
			p=origen;
			return true;
		}
		if(dv==0)
		{
			if(!inSegment2D(s.getOrigen()))
				return false;
			p=s.getOrigen();
			return true;
		}
		double t0,t1;
		Vector3D w2;
		w2.converteix(s.getOrigen(),desti);
		if(v.x()!=0)
		{
			t0=w.x()/v.x();
			t1=w2.x()/v.x();
		}
		else
		{
			t0=w.y()/v.y();
			t1=w2.y()/v.y();
		}
		if(t0<t1)
		{
			float t=t0; t0=t1;t1=t;
		}
		if(t0<1||t1<0) // NO OVERLAP
			return false;	
		if(t0<0) t0=0;
		if(t1>1) t1=1;
		if(t0==t1)
		{
			p=Punt(s.getOrigen().x()+t0*v.x(),s.getOrigen().y()+t0*v.y(),0);
			return true;
		}
		p=Punt(s.getOrigen().x()+t0*v.x(),s.getOrigen().y()+t0*v.y(),0);
		q=Punt(s.getOrigen().x()+t0*v.x(),s.getOrigen().y()+t0*v.y(),0);
		return true;
	}
	double sI=v.perpendicular2D(w)/D;
	if(sI<0||sI>1)
		return false;
	
	double tI=u.perpendicular2D(w)/D;
	if(tI<0||tI>1)
		return false;
	p=Punt(origen.x()+sI*u.x(),origen.y()+sI*u.y(),0);
	return true;	
}

/*********************************************************************************/
/*   bool Segment::e_intersects_T(const Tetrahedron &Tvs, vector<Punt3d3> &p)     */
/*Metode que informa si el segment intersecciona el tetraedre i retorna els punts*/
/*********************************************************************************/

bool Segment::e_intersects_T(const Tetrahedron &Tvs, VectorPunt3d &p) const
{
	bool ret=false;
	bool cont=true;
	
	if(Tvs.is_degenerate ()) qDebug("Tetraedre degenerat, esta en un pla");
	if(Tvs.has_on_boundary(segment_cgal.source()) ||Tvs.has_on_bounded_side(segment_cgal.source()))
	{
		p.push_back(segment_cgal.source());
		ret=true;
		if(Tvs.has_on_boundary(segment_cgal.target()) || Tvs.has_on_bounded_side(segment_cgal.target()))
		{ 
			p.push_back(segment_cgal.target()); 
			cont=false; 
		}
	}
	else	if(Tvs.has_on_boundary(segment_cgal.target()) || Tvs.has_on_bounded_side(segment_cgal.target()))
	{
		p.push_back(segment_cgal.target()); 
		ret=true;
	}
	
	if(cont=true) // com a molt tenim un punt a l'interior del tetraedre, pot ser que el segment intersequi amb la frontera d'aquest
	{
		for(int i=0;i<4;i++) // estudiem cada cara
		{
			Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
			bool reti= tr.triangle_intersection_e_obert(segment_cgal,p);
			ret=(ret || reti);
		}
	}
	return ret;
}

/****************************************************************************************************/
/*   bool Segment::e_intersects_T(const Tetrahedron &Tvs, vector<Punt3d3> &p)                        */
/*Metode que retorna cert si els punts d'interseccio amb el tetraedre no son els extrems del segment*/
/****************************************************************************************************/

bool Segment::e_intersects_T_nV(const Tetrahedron &Tvs, VectorPunt3d &p)const
{
	bool ret=false;
	bool cont=true;
	
	if(Tvs.has_on_boundary(segment_cgal.source()) ||Tvs.has_on_bounded_side(segment_cgal.source())) ret=true;
	if(Tvs.has_on_boundary(segment_cgal.target()) || Tvs.has_on_bounded_side(segment_cgal.target()))
		if(ret) cont=false;
		else ret=true;
	
	if(cont) // com a molt tenim un punt a l'interior del tetraedre, pot ser que el segment intersequi amb la frontera d'aquest
		for(int i=0;i<4;i++) // estudiem cada cara
		{
			Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
			bool reti= tr.triangle_intersection_e_obert(segment_cgal,p);
			if(reti)
			{
				if(Punt(p[p.size()-1])==Punt(Tvs.vertex(i)) || Punt(p[p.size()-1])==Punt(Tvs.vertex(i+1)) || Punt(p[p.size()-1])==Punt(Tvs.vertex(i+2))) //No volem els extrems del tetraedre
				{
// 					qDebug("Punt extrem tetraedre desempilat");
					p.pop_back();
					reti=false;
				}
			}
			ret=(ret || reti);
		}
	else
	{
// 	 qDebug("segment interior a T");
	 }
	return ret;
}

/*********************************************************************************************/
/*   void Segment::e_intersects_Ta(const Tetrahedron &Tvs, VectorPunt3d &p, vector<int> &k)   */
/*p conte els punts d'interseccio i a k es guarden els indexs de les cares que intersequen   */
/*********************************************************************************************/

void Segment::e_intersects_Ta(const Tetrahedron &Tvs, VectorPunt3d &p, VectorInt &k) const
{
	for(int i=0;i<4;i++) // estudiem cada cara
	{
		Triangle tr(Triangle3(Tvs.vertex(i),Tvs.vertex(i+1),Tvs.vertex(i+2)));
		if(tr.tr_e_intersection(segment_cgal,p)) k.push_back(i);
	}
}


/*******************************************************************************************************************************/
/*void Segment::obtencio_Regions_variant(Segment *sv, VectorFace *Tr_Interes,VectorRegions & Regions, VectorDoubles &vis) const*/
/*Metode que calcula les regions visibles des de un segment de visio retornant les hiperboles que defineixen la zona           */
/*******************************************************************************************************************************/

void Segment::obtencio_Regions_variant(const Tetrahedron &Tvs,VectorPla plansTetraedre, const Segment &sv,const Pla &pla_v, const Pla &pla_s, const double &valor_s, const VectorFace &Tr_Interes,VectorRegions & Regions,/*VectorTriangles &triangles_bons_visio,*/bool cV,double &t_retallar_triangle, double &t_obtenir_regions,double &t_interseccio_Tvs,const bool & per_discretitzar) const
{
	Triangle t;
	timeval tr_i,tr_f,tor_i,tor_f,tinter_i,tinter_f;
	double tr=0,tor=0,tinter_r=0;
	Segment ss=*this;
//	Vector vn=cross_product( sv->segment_cgal().to_vector(),segment_cgal.to_vector());
//	Pla Pi_s(Plane(segment_cgal.source(),vn)); 
// 	cout<<" retallar";
	
	
		
	for(unsigned int i=0;i<Tr_Interes.size();i++)
	{
		t = Triangle(Tr_Interes[i]);
// 		cout<<"estudiem ";t.escriure();
		if(!t.p_on_triangle(punt_mig_3d()))
		{
			gettimeofday(&tr_i,NULL);
			VectorTriangles vt=t.retallar(Tvs,ss,sv,pla_v, pla_s, valor_s,cV);
			gettimeofday(&tr_f,NULL);
			tr=tr+(((&tr_f)->tv_usec+(&tr_f)->tv_sec*1e+6)-((&tr_i )->tv_usec+(&tr_i )->tv_sec*1e+6) )/1e+6; 
// 			cout<<" "<<vt.size();
			for(unsigned int i=0;i<vt.size();i++)
			{
// 				if(vt[i].tr_intersects_T(Tvs,cV))
				gettimeofday(&tinter_i,NULL);
				bool interseca=vt[i].interseccio_tetraedre(plansTetraedre,cV);
				gettimeofday(&tinter_f,NULL);
				tinter_r=tinter_r+(((&tinter_f)->tv_usec+(&tinter_f)->tv_sec*1e+6)-((&tinter_i )->tv_usec+(&tinter_i )->tv_sec*1e+6) )/1e+6; 
				if(interseca)
				{
// 					cout<<" int ";
					gettimeofday(&tor_i,NULL);
					vt[i].obtenirRegions(ss,sv,pla_v,pla_s,valor_s,Regions, true,per_discretitzar);
					gettimeofday(&tor_f,NULL);
					tor=tor+(((&tor_f)->tv_usec+(&tor_f)->tv_sec*1e+6)-((&tor_i )->tv_usec+(&tor_i )->tv_sec*1e+6) )/1e+6; 
// 					triangles_bons_visio.push_back(vt[i]);
// 					vt[i].escriure();
				}
			}
		}
	}
	t_retallar_triangle=t_retallar_triangle+tr;
	t_obtenir_regions=t_obtenir_regions+tor;
}

/*Redefinicio dels operadors d'igualtat i diferencia per poder realitzar comparacions, entre objectes d'aquesta 
mateixa classe*/

bool Segment::operator==(const  Segment &p)const
{
	bool iguals=false;
	if(origen == p.getOrigen() && desti == p.getDesti() && vector == p.getVector())
	{
		iguals=true;
	}
	return(iguals);
}

bool Segment::operator!=(const  Segment &p)const
{
	bool diferents=true;
	if(*this==p)
	{
		diferents=false;
	}
	return(diferents);
}


void Segment::escriure()const
{
	cout<<"Segment: ("<<origen.x()<<", "<<origen.y()<<", "<<origen.z()<<"), ("<<desti.x()<<", "<<desti.y()<<", "<<desti.z()<<")."<<endl;
}

bool Segment::dinsTrapezi(const Segment &s,const Segment &v)const
{
	Segment seg;
	return dinsTrapezi(s,v,seg);
}

bool Segment::dinsTrapezi(const Segment &s,const Segment &v,Segment &seg)const
{
	double te=0;
	double tl=1;
	
	Punt p0(origen);
	Vector3D dS=vector;
	if(origen.x()>desti.x())
	{
		dS=Vector3D(-dS.x(),-dS.y(),0);
		p0=desti;
	}
	VectorPunt p;
	VectorVector3D vect=s.vectorsNormalsTrapezi(v,p);
	for(unsigned int i=0;i< vect.size();i++)
	{
// 		vect[i].escriure();
		Vector3D u;
		u.converteix(p0, p[i]);
		double N=-1*(vect[i]*u);
		double D=vect[i]*dS;
		if(D==0) // parallels
		{
			if(N<0) return false;
		}
		else
		{
			double t=N/D;
// 			cout<<"N: "<<N<<" D: "<<D<<" t: "<<t<<endl;
			if(D<0)
			{
				te=max(te,t);
				if(te>tl) return false;
			}
			else
			{
				tl=min(tl,t);
				if(tl<te) return false;
			}
		}
	}
// 	cout<<endl;
	//segment interseccio
// 	seg=Segment(Punt(getOrigen().x()+te*dS.x(),getOrigen().y()+te*dS.y()),Punt(getOrigen().x()+tl*dS.x(),getOrigen().y()+tl*dS.y()));
	
 	seg=Segment(calcular_Z(Punt(p0.x()+te*dS.x(),p0.y()+te*dS.y(),0)),calcular_Z(Punt(p0.x()+tl*dS.x(),p0.y()+tl*dS.y(),0)));
	cout<<"seg dins segment.cpp ";
	seg.escriure();
	return true;
}

Punt Segment::calcular_Z(const Punt &p)const
{
	double lambda;
	
	if(fabs(vector.x())>1e-3) lambda=(p.x()-origen.x())/vector.x();
	else if(fabs(vector.y())>1e-3) lambda=(p.y()-origen.y())/vector.y();
	else lambda=0.;
	
	if(fabs(p.x()-(origen.x()+lambda*vector.x()))>1e-6 || fabs(p.y()-(origen.y()+lambda*vector.y()))>1e-6 )
		cout<<"ERROR punt p no pertany a la projeccio de la recta sobre el pla z"<<fabs(p.x()-(origen.x()+lambda*vector.x()))<<", "<< fabs(p.y()-(origen.y()+lambda*vector.y()))<<endl;
		
	return Punt(p.x(),p.y(), origen.z()+lambda*vector.z());
}

VectorVector3D Segment::vectorsNormalsTrapezi(const Segment &v,VectorPunt &p) const
{
	p.clear();
	VectorVector3D vect;
	vect.push_back(Vector3D(vector.y(),- vector.x(),0));
	p.push_back(getOrigen());
	p.push_back(getDesti());
	Vector3D u;
	if(v.getOrigen().x()<v.getDesti().x())
	{
		u.converteix(v.getDesti(),getDesti());
		
		if(u.y()>0) vect.push_back(Vector3D(u.y(),-u.x(),0));
		else vect.push_back(Vector3D(-u.y(),u.x(),0));
		u.converteix(v.getOrigen(),v.getDesti());
		vect.push_back(Vector3D(u.y(),-u.x(),0));
		u.converteix(getOrigen(),v.getOrigen());
		
		if(u.y()>0) vect.push_back(Vector3D(-u.y(),u.x(),0));
		else vect.push_back(Vector3D(u.y(),-u.x(),0));
		p.push_back(v.getDesti());
		p.push_back(v.getOrigen());
	}
	else
	{
		u.converteix(v.getOrigen(),getDesti());
		
		if(u.y()>0) vect.push_back(Vector3D(u.y(),-u.x(),0));
		else vect.push_back(Vector3D(-u.y(),u.x(),0));
		u=v.getVector();
		vect.push_back(Vector3D(u.y(),-u.x(),0));
		u.converteix(getOrigen(),v.getDesti());
		
		if(u.y()>0) vect.push_back(Vector3D(-u.y(),u.x(),0));
		else vect.push_back(Vector3D(u.y(),-u.x(),0));
		p.push_back(v.getOrigen());
		p.push_back(v.getDesti());
	}
	return vect;
}

/***************************************************************************************************/
/*     Transformation3 XMLEscena::calcularTransformacioCoplanars(Punt3d ss_origen, Punt3d ss_desti)  */
/*Metode que calcula la transformacio a aplicar als segments en el cas que ss i sv siguin coplanars*/
/*  deixa esl segment ss al pla z=0, amb origen al (00) i desti al (longitud_ss, 0)  */ 
/***************************************************************************************************/
 
Transformation3 Segment::calcularTransformacioCoplanars(const Segment &ss)const
{
// 	double longitud=sqrt(((coord_x*coord_x)+(coord_y*coord_y)+(coord_z*coord_z)));
// 	cout<<"longitud: "<<longitud<<endl;
	
	
	Punt3d pt(segment_cgal.source());
	
	Transformation3 Transf0(1,0,0,-pt.x(),0,1,0,-pt.y(),0,0,1,-pt.z(),1);
	Punt3d pt0_=Transf0(segment_cgal.target());
	
	double d0_xy=sqrt(pt0_.x()*pt0_.x()+pt0_.y()*pt0_.y()), c_a=pt0_.x()/d0_xy,s_a=pt0_.y()/d0_xy;
	Transformation3 Transf1(c_a,-s_a,0,0,s_a,c_a,0,0,0,0,1,0,1);
	Transf1=Transf1.inverse();	
	Punt3d pt1_=Transf1(pt0_);
	
	double d0_xz=sqrt(pt1_.x()*pt1_.x()+pt1_.z()*pt1_.z()), c_b=pt1_.x()/d0_xz,s_b=pt1_.z()/d0_xz;
	Transformation3 Transf2(c_b,0,-s_b,0, 0,1,0,0 ,s_b,0,c_b,0,1);
	Transf2=Transf2.inverse();
	Punt3d pt2_=Transf2(pt1_);
	
	Punt3d pt3_=Transf2(Transf1(Transf0(ss.getOrigen().getPunt())));
	double d1_yz=sqrt(pt3_.y()*pt3_.y()+pt3_.z()*pt3_.z()), c_c=pt3_.y()/d1_yz,s_c=pt3_.z()/d1_yz;
	Transformation3 Transf3(1,0,0,0,0, c_c,-s_c,0,0,s_c ,c_c,0,1);
	Transf3=Transf3.inverse();
	
	return Transf3*(Transf2 * (  Transf1* Transf0 ));
	
	
}


Transformation3 Segment::calcularTransformacioPuntVisio(const Punt &pv)const
{
// 	double longitud=sqrt(((coord_x*coord_x)+(coord_y*coord_y)+(coord_z*coord_z)));
// 	cout<<"longitud: "<<longitud<<endl;
	
	
	Punt3d pt(segment_cgal.source());
	
	Transformation3 Transf0(1,0,0,-pt.x(),0,1,0,-pt.y(),0,0,1,-pt.z(),1);
	Punt3d pt0_=Transf0(segment_cgal.target());
	
	double d0_xy=sqrt(pt0_.x()*pt0_.x()+pt0_.y()*pt0_.y()), c_a=pt0_.x()/d0_xy,s_a=pt0_.y()/d0_xy;
	Transformation3 Transf1(c_a,-s_a,0,0,s_a,c_a,0,0,0,0,1,0,1);
	Transf1=Transf1.inverse();	
	Punt3d pt1_=Transf1(pt0_);
	
	double d0_xz=sqrt(pt1_.x()*pt1_.x()+pt1_.z()*pt1_.z()), c_b=pt1_.x()/d0_xz,s_b=pt1_.z()/d0_xz;
	Transformation3 Transf2(c_b,0,-s_b,0, 0,1,0,0 ,s_b,0,c_b,0,1);
	Transf2=Transf2.inverse();
	Punt3d pt2_=Transf2(pt1_);
	
	Punt3d pt3_=Transf2(Transf1(Transf0(pv.getPunt())));
	double d1_yz=sqrt(pt3_.y()*pt3_.y()+pt3_.z()*pt3_.z()), c_c=pt3_.y()/d1_yz,s_c=pt3_.z()/d1_yz;
	Transformation3 Transf3(1,0,0,0,0, c_c,-s_c,0,0,s_c ,c_c,0,1);
	Transf3=Transf3.inverse();
	
	return Transf3*(Transf2 * (  Transf1* Transf0 ));
	
	
}

Segment Segment::projeccio(const Punt &pv) const
{
	Segment s(pv, origen);
	double coordx_origen = s.coordenada_x(0.);
	
	s = Segment(pv, desti);
	double coordx_desti = s.coordenada_x(0.);
// 	cout<<"PROJECCIO SEGMENT!!!"<<endl;
// 	pv.escriure();
// 	escriure();
	return Segment(Punt(coordx_origen, 0., 0.), Punt(coordx_desti, 0., 0.));
}

// Retorna  cert si el Segment intersecciona amb la lina que conte al segment s. Retorna el punt d'interseccio sobre el Segment
bool Segment::interseccioLinia(const Segment &s, Punt &p) const 
{
	Vector3D w;
	w.converteix(getOrigen(),s.getOrigen());
	Vector3D u=vector;
	Vector3D v=s.getVector();
	
	double s1=(v.y()*w.x()-v.x()*w.y())/(v.x()*u.y()-v.y()*u.x());
// 	double t=(-u.y()*w.x()+u.x()*w.y())/(u.x()*v.y()-u.y()*v.x());
	p=Punt(getOrigen().x()+s1*u.x(),getOrigen().y()+s1*u.y(),0);
	return (s1>-0.00001&&s1<1+0.00001);
	//return (s1>-0.000000000000000001&&s1<1+0.00000000000000000000001);
}

// Retorna  cert si el Segment intersecciona amb la lina que conte al segment s. Retorna la cordenada del punt inter sobre s (0->s.getOr, 1->s.getDe)
bool Segment::interseccioLinia(const Segment &s, double &t) const
{
	
	Vector3D w;
	w.converteix(getOrigen(),s.getOrigen());
	Vector3D u=vector;
	Vector3D v=s.getVector();
	double s1=(v.y()*w.x()-v.x()*w.y())/(v.x()*u.y()-v.y()*u.x());
	t=(-u.y()*w.x()+u.x()*w.y())/(u.x()*v.y()-u.y()*v.x());
	return (s1>-0.00000000000001&&s1<1+0.00000000000001);
}

void Segment::actualitzar(const Segment ss, const Segment &s,bool dreta)
{
	Segment aux;
	Punt p;
	if(dreta)
	{
		if(s.getOrigen().y()>0) aux=Segment(s.getOrigen(),ss.getOrigen());
		else if(s.getDesti().y()>0) aux=Segment(s.getDesti(),ss.getOrigen());
		if(interseccioLinia(aux,p))  setDesti(p);
	}
	else
	{
		if(s.getOrigen().y()>0)  aux=Segment(s.getOrigen(),ss.getDesti());
		else if(s.getDesti().y()>0) aux=Segment(s.getDesti(),ss.getDesti());
		if(interseccioLinia(aux,p)) setOrigen(p);
	}
}

VectorRegions Segment::buscarImatgesHiperboles(const Segment &sv, const VectorSegment &segments) const
{
	cout<<"buscarImatgesHiperboles"<<endl;
	Segment s;
	VectorRegions vector_regions;
	
	for(unsigned int i=0;i<segments.size();i++)
	{
		vector_regions.push_back(segments[i].buscarRegio(*this,sv));
	}
	return vector_regions;
}

Regio Segment::buscarRegio(const Segment &s, const Segment &v)const
{
	Regio r;
	Segment d(s.getDesti(),v.getDesti());
	Segment e(s.getOrigen(),v.getOrigen());
	Hiperbola h;
	double lambda;
	
	if(s.pertanySegment(origen,lambda)) h=Hiperbola(0,0,1,-lambda);
	else if(v.pertanySegment(origen,lambda)) h=Hiperbola(0,1,0,-lambda);
	else if(d.pertanySegment(origen,lambda)) h=Hiperbola(0,0,0,s.midaSegment());
	else if(e.pertanySegment(origen,lambda)) h=Hiperbola(0,0,0,0);
	else h=Hiperbola(origen,s,v);
	h.setExtrems(v.midaSegment(),s.midaSegment());
	r.afegirHiperbola(h);
	
	if(s.pertanySegment(desti,lambda)) h=Hiperbola(0,0,1,-lambda);
	else if(v.pertanySegment(desti,lambda)) h=Hiperbola(0,1,0,-lambda);
	else if(d.pertanySegment(desti,lambda)) h=Hiperbola(0,0,0,s.midaSegment());
	else if(e.pertanySegment(desti,lambda)) h=Hiperbola(0,0,0,0);
	else h=Hiperbola(desti,s,v);
	h.setExtrems(v.midaSegment(),s.midaSegment());
	
	r.afegirHiperbola(h);
	
	return r;
}

double Segment::cotg(const Punt &p,const Punt &q) const
{
	Vector3D v;
	v.converteix(q,p);
	
	double a=vector.angle(v);
	double s=sin(a);
	if(fabs(s)>0.00000000001) return cos(a)/s;
	
	if (s>0) return -1000.;
	
	return 1000.;
}

bool Segment::pertanySegment(const Punt &p, double &lambda)const
{
	Vector3D u;
	u.converteix(origen,p);
	
 	if(fabs(vector.x()*u.y()-vector.y()*u.x())<TOLERANCE)
	{
		if(fabs(u.x())>TOLERANCE) 
			lambda=(p.x()-origen.x())/u.x();
		else if(fabs(u.y())>TOLERANCE) 
			lambda=(p.y()-origen.y())/u.y();
		else
			return p==origen;
		return lambda>-TOLERANCE && lambda<1+TOLERANCE;
	}
	return false;
}

VectorPunt Segment::Trapezi(const Segment & s,bool &partir,Punt &p)const
{
	double oo_=cotg(origen,s.getOrigen());
	double od_=cotg(origen,s.getDesti());
	double dd_=cotg(desti,s.getDesti());
	double do_=cotg(desti,s.getOrigen());
	
	VectorPunt vp;
	
	vp.push_back(Punt(oo_,0,0));
	vp.push_back(Punt(od_,0,0));
	vp.push_back(Punt(dd_,midaSegment(),0));
	vp.push_back(Punt(do_,midaSegment(),0));
	
	Segment s1(vp[0],vp[3]);
	Segment s2(vp[1],vp[2]);
	partir=s1.interseccio(s2,p);
	
	return vp;
}

VectorVectorPunt Segment::buscarImatgesTrapezis(const Segment &sv, const VectorSegment &segments) const
{
	VectorVectorPunt vector_punts;
	VectorPunt aux;
	Punt p;
	bool partir;
	
	for(unsigned int i=0;i<segments.size();i++)
	{
		VectorPunt t=Trapezi(segments[i],partir,p);
			
		if(t.size()!=4)
			cout<<"ERROR"<<endl;
		else
		{
			if(!partir) vector_punts.push_back(t);
			else
			{
				aux.push_back(t[0]);
				aux.push_back(t[1]);
				aux.push_back(p);
				vector_punts.push_back(aux);
				
				aux.clear();
				
				aux.push_back(t[2]);
				aux.push_back(t[3]);
				aux.push_back(p);
				vector_punts.push_back(aux);
			}
		}
	}
	return vector_punts;
}

double Segment::coordenada_x(double y)const
{
	 if(fabs(vector.y())<TOLERANCE)
	 {
// 	 	cout<<"ERROR: cordenada_x no es pot aillar la x en funcio d'y, segment horizontal"<<endl;
		return desti.x();
	}
	double lambda=(y-origen.y())/vector.y();
	return origen.x()+lambda*vector.x();
	
}
double Segment::coordenada_y(double x)const
{
	 if(fabs(vector.x())<TOLERANCE)
	 {
// 	 	cout<<"ERROR: cordenada_y no es pot aillar la y en funcio d'x, segment vertical"<<endl;
		return desti.y();
	}
	double lambda=(x-origen.x())/vector.x();
	return origen.y()+lambda*vector.y();
	
}

bool Segment::interseccioProjeccio(const Segment  &s2) const
{
// 	bool interseccio=false;
	
	// Recta determina la projeccio del segment *this al pla xy
	double a=-1.*vector.y();
	double b=-1.*vector.x();
	double c=-1.*(a*origen.x()+b*origen.y());
	
	double avaluarO=a*s2.getOrigen().x()+b*s2.getOrigen().y()+c;
	double avaluarD=a*s2.getDesti().x()+b*s2.getDesti().y()+c;
		
	return avaluarO*avaluarD<0;
}

double Segment::trobar_parametre(const Tetrahedron &Tvs,bool cV)const
{
	double h=1;
	double aux;
	for(int i=0;i<4;i++)
	{
		Pla p((Punt)Tvs.vertex(i),(Punt)Tvs.vertex(i+1),(Punt)Tvs.vertex(i+2));
		if(p.interseccio(*this,aux,cV))
		{
// 			if(aux<0 ) i++;
// 			if(aux<1 &&aux>0&&aux<h)  h=aux;
			if(aux<1 &&aux>0&&aux<h)  h=aux;
		}
	}
	if(cV&&(h<0.1||h>0.9))
		h=0.3;
	return h/3.;
}

Punt Segment::puntXminima()const
{
	if(origen.x()<desti.x())
		return origen;
	else
		return desti;
}

Punt Segment::puntYminima()const
{
	if(origen.y()<desti.y())
		return origen;
	else
		return desti;
}

Punt Segment::puntZminima()const
{
	if(origen.z()<desti.z())
		return origen;
	else
		return desti;
}

Punt Segment::puntXmaxima()const
{
	if(origen.x()>desti.x())
		return origen;
	else
		return desti;
}

Punt Segment::puntYmaxima()const
{
	if(origen.y()>desti.y())
		return origen;
	else
		return desti;
}

Punt Segment::puntZmaxima()const
{
	if(origen.z()>desti.z())
		return origen;
	else
		return desti;
}

double Segment::xmin()const
{
	return min(origen.x(),desti.x());
}
double Segment::ymin()const
{
	return min(origen.y(),desti.y());
}
double Segment::zmin()const
{
	return min(origen.z(),desti.z());
}
double Segment::xmax()const
{
	return max(origen.x(),desti.x());
}
double Segment::ymax()const
{
	return max(origen.y(),desti.y());
}
double Segment::zmax()const
{
	return max(origen.z(),desti.z());
}

bool Segment::es_entrada(const Poligon_Convex &poligon,const pCara &tr, int vertex_oposat)const
{
	poligon.calcula_baricentre();
	if(!poligon.intersecciona2D(*this,false))
		return false;
	
	for(int i=vertex_oposat+1;i<vertex_oposat+3;i++)
	{
		int j=0;
		while(tr->info().entrada[(i+1)%3][j]==tr->info().entrada[(vertex_oposat+1)%3][j]) j++;
		if(tr->info().entrada[(i+1)%3][j]<tr->info().entrada[(vertex_oposat+1)%3][j]) 
		{
			Segment s(Punt(tr->vertex((i+1)%3)->point()),Punt(tr->vertex((i+2)%3)->point()));
			if(poligon.intersecciona2D(s,false)) return false;
		}
	}
	return true;
}


