#ifndef SEGMENT_H
#define SEGMENT_H

#include <iostream.h>
#include <vector.h>
#include <vector>

#include <sys/time.h>
#include <unistd.h>

#include "punt.h"
#include "geometriabasica.h"
#include "vector3D.h"
#include "objecte.h"
#include "linia.h"
#include "regio.h"
#include "triangle.h"
#include <math.h>
#include "poligon.h"

typedef std::vector<double>  VectorDoubles;
typedef std::vector<Punt3d>  VectorPunt3d;
typedef std::vector<Punt>  VectorPunt;
typedef std::vector<vector<Punt> >  VectorVectorPunt;
typedef std::vector<Vector3D>  VectorVector3D;
typedef std::vector<int>  VectorInt;
// typedef std::vector<Triangle>  VectorTriangles;
typedef std::vector<Triangle3>  VectorTriangles3D;
typedef std::vector<Regio>  VectorRegions;
typedef std::vector<pCara>  VectorFace;
typedef std::vector<Segment>  VectorSegment;
typedef std::vector<Pla>  VectorPla;

class Hiperbola;
class Triangle;
class Poligon_Convex;
class Pla;
typedef std::vector<Triangle>  VectorTriangles;
/**************************************************************************************************/
/*                                             class Segment                                      */
/*   Classe Segment que conté tots aquells mètodes que permeten tractar un objecte d'aquest tipus */
/**************************************************************************************************/

class Segment : public Objecte
{
	private:
		Punt origen;
		Punt desti;
		Vector3D vector;
		Segment3D segment_cgal;
		Transformation3 transformacio;
	public:
		Segment () ; /*Constructor per defecte que inicialitza els punts que defineixen l'aresta*/
		~Segment();
		Segment ( Punt ,  Punt ,bool visio = false) ; /*Constructor amb parametres*/
		Segment( Punt a, Punt b,  Segment sv); /*Constructor per segmwents sobre terreny*/
		Segment( Segment3D s);
		Segment( Segment3D s, Segment sv);
		
		/*Metodes per la gestio del segment CGAL*/
		const Segment3D & getSegment_cgal() const;
		void setSegment_cgal(Segment3D s);
		Segment2D on_xy_segment() const;
		Vector2D on_xy_vector() const;
		
		Punt2d punt_mig_2d()const;
		Punt3d punt_mig_3d()const;
		
		bool es_degenerat()const; // true si l'origen es igual al desti.
		
		double coordenada_x(double y)const;
		double coordenada_y(double x)const;
		
		/*Metodes que retornen els punts d'origen i desti de l'aresta*/
		Punt getOrigen() const;
		Punt getDesti() const;
		Vector3D getVector() const;
		Transformation3 getTransformacio() const;
		const Punt3d  &source()const;
		const Punt3d  & target()const;
		
		/*Metodes que permeten modificar els punts d'origen i de desti de l'aresta*/
		void setOrigen(Punt) ;
		void setDesti(Punt) ;
		void setVector(Vector3D) ;
		
		/*Mètode que retorna la mida de l'aresta*/
		double midaSegment() const; 
		double midaSegmentVertical2d() const; 
		
		bool interseccioSegments2D(const Segment &s2,const bool &extrems) const;
		bool interseccioSegments(const Segment &s2) const;
		
		bool segmentCorrecte(Delaunay *);
		Punt puntInterseccio(const Segment &s2, bool) const;
		bool puntInterseccioSuperior(const Segment &s2)const;
		Segment fragment_segment(const pCara &fh)const;
		Segment twin()const;
		
		double distancia2D(const Punt &punt)const ;
		Punt calculaProperRecta2D(const Punt &p)const ; 
		
		Transformation3 actualitzem_Transformacions_Skew_v();
		Transformation3 actualitzem_Transformacions_Skew_s( Segment sv);
		
		VectorDoubles skew_segment(const Segment &,Punt2d &p, Punt2d &q)const;
		Punt3d e_t(const double &t)const;
		double coord_p_on_e(const Punt3d &p)const;
		
		bool punt_sobre(const Punt3d &p)const;
		bool coplanar_segment_3_intersection(const Segment3D &s)const;
		bool coplanar_segment_3_intersection(const Segment3D &s, VectorPunt3d &p)const;
		
		bool interseccio(const Tetrahedron &Tvs,bool &contingutFrontera, bool extremsS=true, bool vertexT=true,bool arestesT=true) const;
		bool interseccio(const Tetrahedron &Tvs,bool extremsS=true, bool vertexT=true,bool arestesT=true) const;
		bool interseccio(const Tetrahedron &Tvs, VectorPunt3d &vp, bool &contingutFrontera, bool vertexT=true,bool arestesT=true) const;
		
		// trobem la interseccio de la projeccio 2 segments (s i *this) al pla xy, el punt interseseccio te z=0;
		bool interseccio(const Segment &s) const; 
		bool interseccio(const Segment &s,Punt &p) const;
		
		bool e_intersects_T(const Tetrahedron &Tvs)const;
		bool e_intersects_T(const Tetrahedron &Tvs, VectorPunt3d &p)const;
		bool e_intersects_T_nV(const Tetrahedron &Tvs, VectorPunt3d &p)const;
		void e_intersects_Ta(const Tetrahedron &Tvs, VectorPunt3d &p, VectorInt &k)const;

		void obtencio_Regions_variant(const Tetrahedron &Tvs,VectorPla plansTetraedre, const Segment &sv,const Pla &pla_v, const Pla &pla_s, const double &valor_s, const VectorFace &Tr_Interes,VectorRegions & Regions,/*VectorTriangles &triangles_bons_visio,*/bool cV,double &t_retallar_triangle, double &t_obtenir_regions,double &t_interseccio_Tvs,const bool &per_discretitzar) const;
// 		void obtencio_Regions_variant(const Tetrahedron &Tvs,const VectorPla plansTetraedre, const Segment &sv, const VectorFace &Tr_Interes,VectorRegions & Regions,/*VectorTriangles &triangles_bons_visio,*/bool cV,double &t_retallar_triangle, double &t_obtenir_regions, double &t_interseccio_Tvs) const;
		
		VectorVector3D vectorsNormalsTrapezi(const Segment &v,VectorPunt &p) const;
		bool dinsTrapezi(const Segment &s,const Segment &v,Segment &seg)const;
		bool dinsTrapezi(const Segment &s,const Segment &v)const;
		
		// cert si el segment *this intersecciona amb la recta que conte s.
		bool interseccioLinia(const Segment &s, double &t) const;
		bool interseccioLinia(const Segment &s, Punt &p) const; // retorna la projeccio del punt interseccio al pla z=0;
		Transformation3 calcularTransformacioCoplanars(const Segment &ss)const;
		Transformation3 calcularTransformacioPuntVisio(const Punt &pv)const;
		
		Segment projeccio(const Punt &pv) const;
		
		void actualitzar(const Segment ss, const Segment &s,bool dreta);
		VectorRegions buscarImatgesHiperboles(const Segment &sv, const VectorSegment &segments) const;
		VectorPunt Trapezi(const Segment & s,bool &partir,Punt &p)const;
		VectorVectorPunt buscarImatgesTrapezis(const Segment &sv, const VectorSegment &segments) const;
		
		bool pertanySegment(const Punt &p, double &lambda)const;
		Regio buscarRegio(const Segment &s, const Segment &v)const;
		
		bool inSegment2D(const Punt &p)const;
		double cotg(const Punt &p,const Punt &q) const;
		
		// cert si la projeccio al pla_xy del segment s2 i la del segment *this interseccionen
		bool interseccioProjeccio(const Segment  &s2) const;
		
		Punt calcular_Z(const Punt &p)const;
		
		bool operator==(const Segment &p)const;
		bool operator!=(const Segment &p)const;	
			
		void escriure()const;
		double trobar_parametre(const Tetrahedron &Tvs,bool cV)const;
		
		Punt puntXminima()const;
		Punt puntYminima()const;
		Punt puntZminima()const;
		Punt puntXmaxima()const;
		Punt puntYmaxima()const;
		Punt puntZmaxima()const;
		
		double xmin()const;
		double ymin()const;
		double zmin()const;
		double xmax()const;
		double ymax()const;
		double zmax()const;
		
		bool es_entrada(const Poligon_Convex &poligon, const pCara &tr,int vertex_oposat)const;
			
// 		vector<Punt2d> ajustarQuadrat01_2d(const Punt2d &p3)const;
		
} ;

#endif
