#include "superficiebezier.h"

/*******************************************************/
/*   SuperficieBezier::SuperficieBezier() : m_value(5) */
/*Constructor de la classe SuperficieBezier per defecte*/
/*******************************************************/

SuperficieBezier::SuperficieBezier() : m_value(5)
{	
}

/******************************************************************************/
/*   SuperficieBezier::~SuperficieBezier()                                    */
/*Destructor de la classe que elimina les referencies, en el cas que n'hi hagi*/
/******************************************************************************/

SuperficieBezier::~SuperficieBezier() 
{
}

/*******************************************************************************************************/
/*   void SuperficieBezier::generaSuperficie( Delaunay &_dt, int m )                                   */
/*Metode que omple l'estructura de cadascuna de les cares generant la corresponent superficie de Bezier*/
/*******************************************************************************************************/

void SuperficieBezier::generaSuperficie( Delaunay &_dt, int m )
{
	//qDebug("generaSuperficie");
	pVertex vh300, vh030, vh003;   
	Punt p0, p1, p2;
	
	m_value = m;
	v_vertex.clear();   
	//qDebug("Resolucio %d", m);
	if( m == 1 || m == 2 )
	{
		IteradorCares fit = _dt.finite_faces_begin();
		for( ; fit != _dt.finite_faces_end(); ++fit)
		{
			fit->info().cares.clear();
			fit->info().cares.push_back( Cara( fit->vertex(0)->point(),                                                                                 fit->vertex(1)->point(),                                                                                 fit->vertex(2)->point() ) );  
			//qDebug("MMM__ %d", fit->info().cares.size());
		}     
	}
	else if( m > 2 )
	{
		IteradorCares fit = _dt.finite_faces_begin();
		for( ; fit != _dt.finite_faces_end(); ++fit)
		{
			/*A partir de l'iterador fit que fa referència a la cara actual, accedim els 3 vèrtexs que la formen a través de: (*fit).vertex(num). Els tres vèrtexs que formen la cara són guardats en les variables vh003, vh030 i vh300.*/
			vh003 = (*fit).vertex(0);
			vh030 = (*fit).vertex(1);
			vh300 = (*fit).vertex(2);
			
			B003 = vh003->point() - CGAL::ORIGIN;
			B030 = vh030->point() - CGAL::ORIGIN; 
			B300 = vh300->point() - CGAL::ORIGIN;
			
			N003 = generaNormalVertex( vh003, _dt );
			N030 = generaNormalVertex( vh030, _dt );
			N300 = generaNormalVertex( vh300, _dt );
			
			fit->info().v_normals[0] = N003;
			fit->info().v_normals[1] = N030;
			fit->info().v_normals[2] = N300;
			
			generaPuntsControl();
			
			fit->info().control[0] = B012;
			fit->info().control[1] = B021;
			fit->info().control[2] = B120;
			fit->info().control[3] = B210;
			fit->info().control[4] = B102;
			fit->info().control[5] = B201;
			fit->info().control[6] = B111;
			
			generaVertexs();
			generaCares( (fit->info()).cares );
		}
	}
}

/*********************************************************************************/
/*   void SuperficieBezier::generaPuntsControl()                                 */
/*Metode que genera els punts de control corresponents amb les formules de Bezier*/
/*********************************************************************************/

void SuperficieBezier::generaPuntsControl()
{
      //qDebug("genera punts");
      B210 = (2.0/3.0)*B300 + (1.0/3.0)*B030 + ((1.0/3.0)*((N300*(B300-B030))*N300));
      B201 = (2.0/3.0)*B300 + (1.0/3.0)*B003 + ((1.0/3.0)*((N300*(B300-B003))*N300));
      B120 = (2.0/3.0)*B030 + (1.0/3.0)*B300 + ((1.0/3.0)*((N030*(B030-B300))*N030));
      B021 = (2.0/3.0)*B030 + (1.0/3.0)*B003 + ((1.0/3.0)*((N030*(B030-B003))*N030));
      B102 = (2.0/3.0)*B003 + (1.0/3.0)*B300 + ((1.0/3.0)*((N003*(B003-B300))*N003));
      B012 = (2.0/3.0)*B003 + (1.0/3.0)*B030 + ((1.0/3.0)*((N003*(B003-B030))*N003));
      B111 = ( (1.0/4.0)*( B210 + B201 + B120 + B021 + B102 + B012 ) ) - ( (1.0/6.0)*( B300 + B030 + B003 ) );
}

/*****************************************************************************************/
/*   void SuperficieBezier::generaVertexs()                                              */
/*Metode que genera els vertexs nous per generar la superficie amb la resolucio desitjada*/
/*****************************************************************************************/

void SuperficieBezier::generaVertexs()
{
   //qDebug("genera vertexs");
   double u = 0.0, v = 0.0, w = 0.0;
   v_vertex.clear();

   for (int i = 0; i < m_value; i++)
   {
      u = (double)i / (double)(m_value - 1);
      for (int j = 0; j < m_value-i; j++)
      {
         v = (double)j / (double)(m_value - 1);
	 w = 1.0 - u - v;
	 v_vertex.push_back( Bezier(u, v, w) );
      }
   }
}

/****************************************************************************************/
/*   void SuperficieBezier::generaCares( LlistaCares &lc )                              */
/*Metode que genera les noves cares per generar la superficie amb la resolucio desitjada*/
/****************************************************************************************/

void SuperficieBezier::generaCares( LlistaCares &lc )
{	
   //qDebug("genera triangles");
   int k1 = 0, k2 = 0, k3 = 0, k4 = 0;   

   lc.clear();
   for (int i = 1; i < m_value; i++)
   {
      for (int j = 1; j < m_value-i+1; j++)
      {
         k1 = (((2*m_value+2-i)*(i-1))/2)+j-1;
         k2 = (((2*m_value+1-i)*i)/2)+j-1;
         k3 = (((2*m_value+2-i)*(i-1))/2)+j;
         k4 = (((2*m_value+1-i)*i)/2)+j;                  
         //lc.push_back( Cara( v_vertex[k1], v_vertex[k2], v_vertex[k3] ) );
         lc.push_back( Cara( v_vertex[k1], v_vertex[k3], v_vertex[k2] ) );	 
         if (j < m_value-i) 
         {
            //lc.push_back( Cara( v_vertex[k2], v_vertex[k4], v_vertex[k3] ) );
	    lc.push_back( Cara( v_vertex[k4], v_vertex[k2], v_vertex[k3] ) );
         }
      }
   }        
   //qDebug("ANTES DE SALIR DE GENERATRIANGLES");
}

/*******************************************************************************************/
/*   Punt3d SuperficieBezier::Bezier(double u, double v, double w)                         */
/*Metode que omple els paràmetres "B" segons tres valors de double, retornant el nou vertex*/
/*******************************************************************************************/

Punt3d SuperficieBezier::Bezier(double u, double v, double w)
{
   //qDebug("bezier");
	B200 = u * B300 + v * B210 + w * B201;	
	B110 = u * B210 + v * B120 + w * B111;	
	B020 = u * B120 + v * B030 + w * B021;	
        B101 = u * B201 + v * B111 + w * B102;
	B011 = u * B111 + v * B021 + w * B012;	
	B002 = u * B102 + v * B012 + w * B003;	
	B100 = u * B200 + v * B110 + w * B101;	
	B010 = u * B110 + v * B020 + w * B011;	
	B001 = u * B101 + v * B011 + w * B002;
	B000 = u * B100 + v * B010 + w * B001;	
	
	return (CGAL::ORIGIN + B000);
}

/*****************************************************************************************/
/*   Vector SuperficieBezier::generaNormalVertex( pVertex vh, const Delaunay &dt)      */
/*Metode que retorna un vector3d equivalent a la normal del vertex indicat per parametres*/
/*****************************************************************************************/

Vector SuperficieBezier::generaNormalVertex( pVertex vh, const Delaunay &dt)
{  
  // qDebug("genera normal");
    double area = 0.0, suma_arees = 0.0, longitud = 0.0;
    Vector arexnorm = Vector(0.0, 0.0, 0.0);
    Vector normal;
    Vector vab, vac, v;

    CirculadorCares fc = dt.incident_faces( vh ), done(fc);
    if ( fc != 0 )
    {
         do {
           if ( !dt.is_infinite( fc ) )
           {
                 Punt3d p0 = fc->vertex(0)->point(); 
                 Punt3d p1 = fc->vertex(1)->point();
                 Punt3d p2 = fc->vertex(2)->point();

                 vab = p1 - p0; // vector AB 
                 vac = p2 - p0; // vector AC 
                 v = CGAL::cross_product( vab, vac );   //  AB ^ AC
                 longitud = sqrt( v * v );  //  || AB ^ AC ||    
                 area = longitud / 2.0; //  || AB ^ AC || / 2
                 normal =  v / longitud; //Normalitzat                  
	         arexnorm = arexnorm + ( area * normal );  // A1N1 + ... + AnNn
                 //qDebug("(%f,%f,%f)",normal.x(),normal.y(),normal.z());
	         suma_arees += area;  // A1 + ... + An
	   }
         } while ( ++fc != done );
   }

   Vector fg = arexnorm / suma_arees;
   double nor = sqrt( fg * fg );

   return ( fg / nor );
}

