#ifndef SUPERFICIE_DE_BEZIER_H
#define SUPERFICIE_DE_BEZIER_H

#include <cmath>
#include <cassert>
#include <qmessagebox.h>
#include <vector>
#include "geometriabasica.h"
#include "punt.h"

typedef std::vector<Punt3d> Vertexs;

/****************************************************************************/
/*                         class SuperficieBezier                           */
/*   Classe que tracta el terreny permetent generar una superficie de Bezier*/
/****************************************************************************/

class SuperficieBezier 
{

public:
    SuperficieBezier();
    ~SuperficieBezier();
	
    void generaSuperficie( Delaunay &_dt, int m = 5 );

private:
    // Punts de control
    Vector B300, B030, B003;
    Vector B210, B201, B120, B021, B102, B012, B111;
    Vector B200, B110, B020, B101, B011, B002, B100, B010, B001, B000;

    // Vectors normals
    Vector N300, N030, N003;

    Vertexs v_vertex;
    int m_value;   // Nombre de divisions

    void generaPuntsControl();
    void generaVertexs();
    void generaCares( LlistaCares &lc );

    Vector generaNormalVertex( pVertex vh, const Delaunay &_dt );
    Punt3d Bezier( double u, double v, double w );
};


#endif
