#include "terreny.h"
#include <qfile.h>
#include "qmessagebox.h"

/******************************************************************************************************************/
/*   Terreny::Terreny()                                                                                           */
/*Constructor que segueix el patró que proporciona el Singleton fent que tan sols es crei un objecte tipus Terreny*/
/******************************************************************************************************************/

Terreny* Terreny::instancia = 0;

Terreny::Terreny()
{
	limits[0] =0;  
       limits[1] = 0; 
       limits[2] = 0.; 
       limits[3] = 20.; 
       limits[4] = 20.; 
       limits[5] = 20.;
       tipus = INDEFINIT;
	dt.clear();     
	vertexs.clear();
	_resolucio = 1;
	_modificat = FALSE;
}

/*********************************************************************************************/
/*   Terreny::Terreny(Delaunay d)                                                            */
/*Constructor que inicialitza cadascuna de les variables, a més d'assignar el Delaunay passat*/
/*********************************************************************************************/

Terreny::Terreny(Delaunay d, VectorVertexs v) 
{
	dt=d;
	vertexs=v;
}

/*********************************************************/
/*   Terreny * Terreny:: getInstancia()                  */
/*Mètode que permet seguir la implementacio del Singleton*/
/*********************************************************/

Terreny* Terreny::getInstancia()
{
	if ( instancia == 0 ) 
	{
		instancia = new Terreny();        
	}
	return instancia;    
}

/*****************************************************/
/*   Terreny::~Terreny()                             */
/*Destructor de la classe que elimina les referencies*/
/*****************************************************/

Terreny::~Terreny()
{
// 	delete instancia;
}

/************************************************/
/*   void Terreny::setModificat( bool b )       */
/*Indiquem si el terreny ha estat o no modificat*/
/************************************************/

void Terreny::setModificat( bool b )
{
	_modificat = b;
}

/******************************************************/
/*   bool Terreny::modificat()                        */
/*Metode que retorna el valor de la variable modificat*/
/******************************************************/

bool Terreny::modificat()
{
	return _modificat;
}

/******************************************************/
/*   int Terreny::resolucio()                         */
/*Metode que retorna el valor de la variable resolucio*/
/******************************************************/

int Terreny::resolucio()
{
	return _resolucio;
}

/**************************************************/
/*   void Terreny::setResolucio( int novares )    */
/*Metode que assigna valor a la variable resolucio*/
/**************************************************/

void Terreny::setResolucio( int novares )
{
	_resolucio = novares;
}

/*****************************************************/
/*   pVertex Terreny::insereix( const Punt3d &p )    */
/*Metode que permet inserir un punt a la triangulacio*/
/*****************************************************/

pVertex Terreny::insereix( const Punt3d &p )
{
	pVertex aux = dt.insert( p );
	
	return( aux );
}

/********************************************************************/
/*   void Terreny::inserirPunt(Punt3d)                               */
/*Mètode que permet inserir un punt dins de l'estructura de Delaunay*/
/********************************************************************/

void Terreny::inserirPunt(const Punt3d &p )
{
	pVertex vh;
	
	vh = dt.insert(p);
	
	vertexs.push_back( vh );
}

/*********************************************************/
/*   void Terreny::inserirPuntMogut(Punt3d p, int posicio)*/
/*Mètode que permet inserir un punt que s'esta moguent   */
/*********************************************************/

void Terreny::inserirPuntMogut(Punt3d p, int posicio)
{
	pVertex vh;
	
	vh = dt.insert(p);
	vertexs[posicio-1]=vh;
}

/**************************************************************/
/*   void Terreny::modificarPunt(int posicio, double z)       */
/*Mètode que permet modificar la coordenada Z d'un punt       */
/**************************************************************/

void Terreny::modificarPunt(int posicio, double z)
{
	vertexs[posicio-1]->set_point( Punt3d( vertexs[posicio-1]->point().x(), vertexs[posicio-1]->point().y(), z ) );
}

/******************************************************/
/*   void Terreny::elimina( pVertex vh )              */
/*Metode que elimina el punt indicat segons el pVertex*/
/******************************************************/

void Terreny::elimina( pVertex vh )
{
	dt.remove( vh );
}

/*********************************************************************/
/*   void Terreny::eliminarPunt(int)                                 */
/*Mètode que permet eliminar un punt dins de l'estructura de Delaunay*/
/*********************************************************************/

void Terreny::eliminarPunt(int posicio)
{
	dt.remove( obtenirPunt(posicio) );
}

/**********************************************************/
/*   pVertex Terreny::obtenirPunt(int posicio)      */
/*Mètode que retorna el punt que ocupa la posicio indicada*/
/**********************************************************/

pVertex Terreny::obtenirPunt(int posicio)
{
	return vertexs[posicio - 1];
}

/**************************************************************/
/*   void Terreny::eliminarVertexs(int)                       */
/*Mètode que permet eliminar el vèrtexs de la posicio indicada*/
/**************************************************************/

void Terreny::eliminarVertexs(int posicio)
{
	VectorVertexs::iterator it;
	for(it = vertexs.begin(); it != vertexs.end(); ++it)
	{
		if ((*it) == vertexs[posicio - 1]) 
		{
			vertexs.erase( it ); /*Instrucció que permet eliminar el vèrtexs escollit*/
			break;
		}
	}
}

/******************************************************************/
/*   void Terreny::clear()                                        */
/*Mètode que permet esborrar el contingut de l'estructura Delaunay*/
/******************************************************************/

void Terreny::clear()
{
	dt.clear();
	vertexs.clear();
	_modificat = FALSE;
}

/*************************************************************/
/*   int Terreny::numero_de_vertexs()                        */
/*Mètode que retorna el numero de vertexs que formen la malla*/
/*************************************************************/

int Terreny::numero_de_vertexs()
{
	return dt.number_of_vertices(); //return vertexs.size();
}

/***********************************************************/
/*   int Terreny::numero_de_cares()                        */
/*Mètode que retorna el numero de cares que formen la malla*/
/***********************************************************/

int Terreny::numero_de_cares()
{
	return dt.number_of_faces();
}

/*
int Terreny::numero_de_arestes()
{
  return dt.number_of_edges();
}
*/

/***********************************************************************/
/*   void Terreny::generaBezier( int resolucio )                       */
/*Mètode que permet generar la superficie de bezier segons la resolucio*/
/***********************************************************************/

void Terreny::generaBezier( int resolucio )
{
	sb.generaSuperficie( dt, resolucio );
}

/****************************************************/
/*   void Terreny::generaCorbes( const Nivells &nv )*/
/*Mètode que genera les corbes de nivell desitjades */
/****************************************************/

void Terreny::generaCorbes( const Nivells &nv )
{ 
	cn.generaCorbes( nv, dt );
}

/**************************************************************/
/*   void Terreny::setTriangulacio( const Delaunay &nova_dt ) */
/*Mètode que permet assignar una nova triangulacio de Delaunay*/
/**************************************************************/

void Terreny::setTriangulacio( const Delaunay &nova_dt )
{
	dt = nova_dt;
}

/************************************************/
/*   void Terreny::setDelaunay(Delaunay d)      */
/*Mètode que permet modificar el Delaunay actual*/
/************************************************/

void Terreny::setTriangulacio( const Delaunay &d, const VectorVertexs &v)
{
	dt = d;
	vertexs = v;
}

/************************************************/
/*   Delaunay *Terreny::triangulacio()          */
/*Mètode que retorna la triangulacio de Delaunay*/
/************************************************/

Delaunay *Terreny::triangulacio()
{
   return &dt;
}

/*****************************************************************************/
/*   void Terreny::corbes3D( LlistaCares &lca )                              */
/*Mètode que retorna per parametres la llista de cares per les corbes en 2.5D*/
/*****************************************************************************/

void Terreny::corbes3D( LlistaCares &lca )
{
   lca = cn.corbes3d;
}

/***************************************************************************/
/*   void Terreny::corbes2D( LlistaCorbes &lco )                           */
/*Mètode que retorna per parametres la llista de cares per les corbes en 2D*/
/***************************************************************************/

void Terreny::corbes2D( LlistaCorbes &lco )
{
   lco = cn.corbes2d;
}

/***************************************************/
/*   Vector normal( Triangle &t )                */
/*Mètode que calcula el vector normal a un triangle*/
/***************************************************/

Vector normal( Triangle3 &t )
{
  Punt3d p0 = t[0]; 
  Punt3d p1 = t[1];
  Punt3d p2 = t[2];

  Vector vab = p1 - p0; // vector AB 
  Vector vac = p2 - p0; // vector AC 
  Vector v = CGAL::cross_product( vab, vac );   //  AB ^ AC
  double longitud = sqrt( v * v );  //  || AB ^ AC ||    

  return( v / longitud ); //Normalitzat                  
}

/**********************************************************************/
/*   int Terreny::obtenirResolucioMax( double abase )                 */
/*Mètode que retorna el valor de resolucio maxima a partir d'un double*/
/**********************************************************************/

int Terreny::obtenirResolucioMax( double abase )
{
    Vector vab, vac, v;
    Punt3d p0, p1, p2;
    double longitud, area;
    double amin = 10000.0;
    int comptador;

    comptador = 100;
    IteradorCares fit = dt.finite_faces_begin();
    for( ; fit != dt.finite_faces_end(); ++fit)
    {
       p0 = fit->vertex(0)->point(); 
       p1 = fit->vertex(1)->point(); 
       p2 = fit->vertex(2)->point(); 
       vab = p1 - p0; // vector AB 
       vac = p2 - p0; // vector AC 
       v = CGAL::cross_product( vab, vac );   //  AB ^ AC
       longitud = sqrt( v * v );  //  || AB ^ AC ||    
       area = longitud / 2.0; //  || AB ^ AC || / 2
       if( area < amin ) amin = area;       
       comptador--;
       if( comptador == 0 ) break;
    }
    return( (int)( sqrt( amin / (0.0001 * abase ) ) + 1.0 ) );
}

/**************************************************/
/*   double *Terreny::maximsIMinims()             */
/*Mètode que retorna els limits de la triangulacio*/
/**************************************************/

double *Terreny::maximsIMinims()
{
	cout<<"limits: ["<<limits[0]<<", "<<limits[3]<<"]x["<<limits[1]<<", "<<limits[4]<<"]"<<endl;
	return limits;
}

/***********************************************************/
/*   VectorVertexs Terreny::getVertexs()                   */
/*Mètode que retorna els vertexs que formen la triangulació*/
/***********************************************************/

VectorVertexs Terreny::getVertexs()
{
	return vertexs;
}

/************************************************************************/
/*   Punt Terreny::vertex_xmin( int &posicio)                           */
/*Mètode que retorna el vertex amb x minima dins de tota la triangulacio*/
/************************************************************************/

Punt Terreny::vertex_xmin( int &posicio)const
{
	unsigned int i;
	double min = ample_graella;
	Punt p;
	
	for(i=0; i<vertexs.size(); i++)
	{
		if(vertexs[i]->point().x() < min)
		{
			min = vertexs[i]->point().x();
			posicio = i+1;
		}
	}
	p.modifica_coordenades(min, vertexs[posicio-1]->point().y(), vertexs[posicio-1]->point().z());
	
	return p;
}

/********************************************************************************************/
/*   bool Terreny::es_cara_infinita(pCara cara)                                             */
/*Mètode que retorna un boolea que indica si la cara passada per parametres és o no infinita*/
/********************************************************************************************/

bool Terreny::es_cara_infinita(pCara cara)const
{
	return dt.is_infinite(cara);
}

/*********************************************************************************************/
/*   Segment Terreny::seguentInfinita(Segment s, pCara &f) const                             */
/*Mètode que permet saber si la cara adjacent al segment de la cara indicada es o no infinita*/
/*********************************************************************************************/

Segment Terreny::seguentInfinita(const Segment&s, pCara &f) const
{
	int i=0;
	while(Punt(f->vertex(i)->point())!=s.getDesti()&&i<3) i++;
	
	if(i==3)
	{
		cout<<" ERROR SeguentInfinita"<<endl;
		return s;
	}
	
	CirculadorCares fc=dt.incident_faces(f->vertex(i),f);
	fc++;
	while((dt.is_infinite(fc) || !(dt.is_infinite(fc->vertex(0))|| dt.is_infinite(fc->vertex(1)) || dt.is_infinite(fc->vertex(2))))&&fc!=f)
		fc++;
	
	f=fc;
	
	 // trobem l'altre extrem de l'aresta, ha de deixar la cara infinita a l'esquerre, per tant la finita a la dreta normal seria  i, i+1, aquí i, i+2
	 int k=0;
	 while(Punt(fc->vertex(k)->point())!=s.getDesti() && k<3) k++;
	 
	 return Segment(s.getDesti(),fc->vertex((k+2)%3)->point());
}

void Terreny::seguentInfinita(int &v, pCara &f) const
{
	// l'aresta oposada al vertex es (f->vertex(v+1),f->vertex(v+2)) pero al ser per la cara infinita es al reves, per tant el desti de l'aresta actual te index v+2
	int index_desti=(v+1)%3;   // index del desti de l'areta que tenim en el triangle f.
	
	CirculadorCares fc=dt.incident_faces(f->vertex(index_desti),f);
	fc++;
	while((dt.is_infinite(fc) || !(dt.is_infinite(fc->vertex(0))|| dt.is_infinite(fc->vertex(1)) || dt.is_infinite(fc->vertex(2))))&&fc!=f)
		fc++;  // fc contindra la cara finita que conte la seguent aresta infinita
	
	// trobem el vertex oposat a la seguent aresta infinita: trobem index del vertex desti i li sumem 1 perquè es el cas infinit..
	int i=0;
	while(i<3&&f->vertex(index_desti)->point()!=fc->vertex(i)->point()) i++; //quan acabi i conte l'index del vertex 
// 	while(i<3&&Punt(f->vertex(index_desti)->point())!=Punt(fc->vertex(i)->point())) i++; //quan acabi i conte l'index del vertex 
	
	f=fc;
	v=(i+1)%3;
	if(i>=3) cout<<"ERROR SeguentInfiniga void"<<endl;
	 
}
/**************************************************************************************************************/
/*   vector<pCara> Terreny::recorrer_triangulacio(Segment ss, Segment sv, pCara triangle, Punt punt)          */
/*Mètode que retorna el vector de cares que interseccionen el quadrilater format pels segments ss i sv        */
/**************************************************************************************************************/
/*
vector<pCara> Terreny::recorrer_triangulacio(const Segment &ss, const Segment &sv, const pCara & triangle_,const Punt &punt, Tetrahedron &tetraedre)const
{
	vector<pCara> cares;
	Segment aresta_e, aresta_inicial, s1, s2;
	vector<Punt> punts;
	Poligon_Convex quadrilater, cara, cara_veina;
	Punt aux1, aux2;
	int posicio;
	pCara trianglev;
	pCara triangle=triangle_;
	bool cV=Triangle(triangle).conteVertex(ss);
	
	punts.push_back(ss.getOrigen());
	punts.push_back(ss.getDesti());
	s1=Segment(ss.getOrigen(),sv.getOrigen());
	s2 = Segment(ss.getDesti(), sv.getDesti());
	
	if(s1.interseccioProjeccio(s2))
	{
		punts.push_back(sv.getOrigen());
		punts.push_back(sv.getDesti());
	}
	else
	{
		punts.push_back(sv.getDesti());
		punts.push_back(sv.getOrigen());
	}
	
	cares.push_back(triangle);
	tetraedre = Tetrahedron(punts[0].getPunt(), punts[1].getPunt(), punts[2].getPunt(), punts[3].getPunt());
	
	quadrilater = Poligon_Convex(punts);
	
	aresta_inicial=Segment(Punt(triangle->vertex(0)->point()),Punt(triangle->vertex(1)->point()));
	aresta_e = aresta_inicial;
	posicio=2; //el vei de l'aresta_inicial es el neighbor(posicio);
	
	cara = Poligon_Convex( triangle );
	
	int iteracio=0;
	do
	{
		iteracio++;
		if(cara.entrada(aresta_e, punt, quadrilater,posicio<0))  //final un boolea diu si mirem costat infinit,
		{
			if( Triangle(triangle).tr_intersects_T(tetraedre,cV)){ cares.push_back(triangle);}
// 			cares.push_back(triangle);
			  //passem a al cara veina
			if(!es_cara_infinita(triangle->neighbor(posicio))) //cara adjacent no infinita 
			{
				triangle = triangle->neighbor(posicio);
				cara=Poligon_Convex(triangle);
				aresta_e = cara.seguent(aresta_e.twin(), posicio);  //agafem la nova aresta i la nova posicio x passar al neighbor del triangle..
			}
			else //en cas de que la nova cara sigui finita, hem de recorrer arestes per fora, cal anar en compte
			{
				aresta_e=seguentInfinita(aresta_e.twin(),triangle); 	// retorn la proxima aresta tb infinita, triangle: punter a la cara finita te aresta_e.
				posicio=-1;  // x saber q es tracta d'un edge infinit.
				cara= Poligon_Convex(triangle); //conte cara finita q es adjacent a l'infinita i comparteix aresta_e
			}
		}
		else //No entrada if(rev(e)=entry(face_of(rev(e))) .. //nomes ho pot ser si face_of(rev(e)) no infinita.
		{
			if(posicio!=-1)
				trianglev = triangle->neighbor(posicio);
			else
				trianglev = triangle;
			
			bool twin_entrada_cara_veina=false;
			if(!es_cara_infinita(trianglev))
			{
				if(cara_veina.entrada(aresta_e.twin(), punt, quadrilater,false)) //twin entrada cara veina //rev(e)=entry(face_of(rev(e)))
				{
					triangle = trianglev;
					aresta_e = cara_veina.seguent(aresta_e.twin(), posicio);
					cara=cara_veina;
					twin_entrada_cara_veina=true;
				}
			}
			
			if(!twin_entrada_cara_veina)
			{
				if(posicio!=-1) //cas finit
				{
					aresta_e = cara.seguent(aresta_e, posicio);
				}
				else //cas infinit
				{
					aresta_e=seguentInfinita(aresta_e,triangle);  // retorn la segu aresta infinita;  punter a la cara finita conte aresta seguent.
					posicio=-1; // x saber q es tracta d'un edge infinit.
					cara= Poligon_Convex(triangle);
				}
			}
		}
	}while(aresta_e != aresta_inicial);
	
	return cares;
}
*/

/*****************************************************************************************/
/*   bool Terreny::poligon_visio_intersecciona_terreny(Poligon poligon_visio)            */
/*Mètode que retorna true en el cas que el poligon de visio interseccioni amb el terreny */
/*****************************************************************************************/

bool Terreny::poligon_visio_intersecciona_terreny(Poligon &poligon_visio)const
{
	vector<pCara> cares;
	Segment aresta_e, aresta_inicial, s1, s2;
	Poligon cara, cara_veina;
	Punt aux1, aux2, punt;
	int posicio;
	int vertex;
	Localitzacio tipus;
	pCara trianglev, triangle;
	
	poligon_visio.triangulacio();

	triangle = dt.locate(poligon_visio.retorna_punts()[0].getPunt(), tipus, vertex);
	aresta_inicial = Segment(Punt(triangle->vertex(0)->point()),Punt(triangle->vertex(1)->point()));
	aresta_e = aresta_inicial;
	posicio=2; //el vei de l'aresta_inicial es el neighbor(posicio);
	cara = Poligon( triangle );
	punt = Punt(((triangle->vertex(0)->point().x() + triangle->vertex(1)->point().x() + triangle->vertex(2)->point().x())/3.),((triangle->vertex(0)->point().y() + triangle->vertex(1)->point().y() + triangle->vertex(2)->point().y())/3.), ((triangle->vertex(0)->point().z() + triangle->vertex(1)->point().z() + triangle->vertex(2)->point().z())/3.));
// 	cout<<"Abans del bucle !!!"<<endl;
	int iteracio=0;
	do
	{
		iteracio++;
		
		if(cara.entrada(aresta_e, punt, poligon_visio, posicio<0))  //final un boolea diu si mirem costat infinit,
		{
			if( poligon_visio.interseccioPoligonTriangle3D(Triangle(triangle)) )
				 return true;
			if(!es_cara_infinita(triangle->neighbor(posicio))) //cara adjacent no infinita 
			{
				triangle = triangle->neighbor(posicio);
				cara=Poligon(triangle);
				aresta_e = cara.seguent(aresta_e.twin(), posicio);  //agafem la nova aresta i la nova posicio x passar al neighbor del triangle..
			}
			else //en cas de que la nova cara sigui finita, hem de recorrer arestes per fora, cal anar en compte
			{
				aresta_e=seguentInfinita(aresta_e.twin(),triangle); 	// retorn la proxima aresta tb infinita, triangle: punter a la cara finita te aresta_e.
				
				posicio=-1;  // x saber q es tracta d'un edge infinit.
				cara=Poligon(triangle); //conte cara finita q es adjacent a l'infinita i comparteix aresta_e
			}
		}
		else //No entrada if(rev(e)=entry(face_of(rev(e))) .. //nomes ho pot ser si face_of(rev(e)) no infinita.
		{
			if(posicio!=-1)
				trianglev = triangle->neighbor(posicio);
			else
				trianglev = triangle;
			
			bool twin_entrada_cara_veina=false;
			if(!es_cara_infinita(trianglev))
			{
				cara_veina = Poligon(trianglev);
				if(cara_veina.entrada(aresta_e.twin(), punt, poligon_visio,false)) //twin entrada cara veina //rev(e)=entry(face_of(rev(e)))
				{
					triangle = trianglev;
					aresta_e = cara_veina.seguent(aresta_e.twin(), posicio);
					cara=cara_veina;
					twin_entrada_cara_veina=true;
				}
			}
			
			if(!twin_entrada_cara_veina)
			{
				if(posicio!=-1) //cas finit
					aresta_e = cara.seguent(aresta_e, posicio);
				else //cas infinit
				{
					aresta_e=seguentInfinita(aresta_e,triangle);  // retorn la segu aresta infinita;  punter a la cara finita conte aresta seguent.
					posicio=-1; // x saber q es tracta d'un edge infinit.
					cara= Poligon(triangle);
				}
			}
		}
	}while(aresta_e != aresta_inicial);
	
	return false;
}


/*****************************************************************************************************************/
/*   bool Terreny::triangles_intersequen_poliedre(pCara triangle, Segment ss)                                                                                           */
	/*  Mètode que retorna els triangles que intersequen el paralelapipede definit pel triangle i el segment                                                         */
/*   tr:  triangle estem estudiant                                                                                                                                                             */
/*   triangle_inici : triangle conte el punt hem utiltizat x calcular les entrades, (el punt mig de sv o pv),                                                      */
/*   Poligon_Convex: poligon (amb punts a l'espai) recorrem tots els triangles de la triangulacio q intersequen la seva projeccio al pla xy      */
/*  vector<Pla> plans:   conte tots els plans que ens determinen una regio a l'espai, dels que intersequin la projeccio del poligon nomes retornem els que intersequen */
/*    aquesta regio.                                                                                                                                                                              */
/****************************************************************************************************************/
vector < pCara > Terreny::triangles_intersequen_poliedre(const pCara &tr,const pCara &triangle_inici,const Poligon_Convex &poligon,const vector< Pla > &plans)const 
{
	vector<pCara> cares_interseccionen;

	// busquem aresta e_start, ha de ser del triangle_inici i ha d'intersecar la projeccio del poligon al pla xy.
	int vertex_inicial=0;
	bool intersecciona=false;
	while(vertex_inicial<3&&!intersecciona)
	{
		Segment s(Punt(triangle_inici->vertex((vertex_inicial+1)%3)->point()),Punt(triangle_inici->vertex((vertex_inicial+2)%3)->point()));
		intersecciona=poligon.intersecciona2D(s,false);
		if(!intersecciona) vertex_inicial++;
	}
	
//  	for( ;vertex_inicial<3 && !poligon.intersecciona2D(s,false);vertex_inicial++)
// 		s = Segment(Punt(triangle_inici->vertex((vertex_inicial+1)%3)->point()),Punt(triangle_inici->vertex((vertex_inicial+2)%3)->point()));

	if(vertex_inicial==3){cout<<"no trobem aresta inicial (cin) "<<endl; char dkkd; cin>>dkkd; return cares_interseccionen;}

	pCara triangle = triangle_inici; 
	int vertex=vertex_inicial;
	unsigned int iteracio=0;
	bool c_finita=true;

	bool primera_cara_infinita=true;
	do
	{
		iteracio++;
		pCara trianglev=triangle->neighbor(vertex);
		if(!c_finita) trianglev=triangle;
				
		if(c_finita&&poligon.aresta_oposada_es_entrada(triangle,vertex))  //aresta de triangle oposada a posicio  es l'entrada del triangle considerant el poligon.
		{
			if( triangle!=tr&&triangle!=triangle_inici&&Triangle(triangle).interseccioParalelapipede(plans) ) 
				cares_interseccionen.push_back(triangle);
			
			
			c_finita=!es_cara_infinita(trianglev); //passem a al cara veina
			if(c_finita) //cara adjacent no infinita 
			{
				vertex=trianglev->index(triangle);
				triangle=trianglev;
			}
		}
		else  if(!es_cara_infinita(trianglev))
		{
			if(c_finita&&poligon.aresta_oposada_es_entrada(trianglev, trianglev->index(triangle)))
			{
				if(c_finita) vertex=trianglev->index(triangle);
				triangle = trianglev;
			}
			else if(poligon.aresta_oposada_es_entrada(trianglev, vertex))
				c_finita=true;
		}
		else if(es_cara_infinita(trianglev)&&primera_cara_infinita)//considerem q es l'entada de la cara infintia
		{
			primera_cara_infinita=false;
			c_finita=false;
		}
		if(c_finita) vertex=(vertex+1)%3;   //seguent
		else  seguentInfinita(vertex,triangle);  // retorn la segu aresta infinita;  punter a la cara finita conte aresta seguent.
		
	}while(!(triangle==triangle_inici&&vertex==vertex_inicial&&c_finita)&& iteracio<10*dt.number_of_faces());
	
	if(iteracio==10*dt.number_of_faces())
	{
		cout<<"passat limit"<<endl;
		Triangle triangle_actual(tr);
		cares_interseccionen.clear();
		IteradorCares fit;
		Punt3d c=triangle_actual.circumcenter();
		for( fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
		{
			if( Triangle(fit).interseccioParalelapipede(plans) )
				if(!Triangle(fit).dinsTriangle(c)) cares_interseccionen.push_back(fit);
		}
	}
	
	return cares_interseccionen;
}


vector<pCara> Terreny::triangles_intersequen_paralelapipede(const pCara &tr,const pCara &triangle_sv,const Segment &sv)const 
{
	
	/*Primer cal crear el poligon de visio format pel segment ss i el triangle*/
	Triangle triangle_actual(tr);
	
	vector<Punt> punts_poligon;
	vector<Pla> plans=triangle_actual.plansDelimitadors(sv);
// vector<Pla> plans;
	punts_poligon.push_back(sv.getOrigen());
	punts_poligon.push_back(sv.getDesti());
	punts_poligon.push_back(Punt(tr->vertex(0)->point()));
	punts_poligon.push_back(Punt(tr->vertex(1)->point()));
	punts_poligon.push_back(Punt(tr->vertex(2)->point()));
	
	Poligon_Convex poligon(punts_poligon);

	return triangles_intersequen_poliedre(tr,triangle_sv,poligon,plans);
	
	vector<pCara>  cares;
	Punt3d c=triangle_actual.circumcenter();
	for(IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
		if( Triangle(fit).interseccioParalelapipede(plans) && !Triangle(fit).dinsTriangle(c)) 
				cares.push_back(fit);
	return cares;

}

vector<pCara> Terreny::provant_interseccio(const pCara &tr,const pCara &triangle_pv,const Triangle &triangle)const
{
	cout<<"provant_interseccio"<<endl;
	vector<Punt> punts_poligon;
	punts_poligon.push_back(Punt(triangle.getTriangle_cgal().vertex(0)));
	punts_poligon.push_back(Punt(triangle.getTriangle_cgal().vertex(1)));
	punts_poligon.push_back(Punt(triangle.getTriangle_cgal().vertex(2)));
	punts_poligon.push_back(Punt(tr->vertex(0)->point()));
	punts_poligon.push_back(Punt(tr->vertex(1)->point()));
	punts_poligon.push_back(Punt(tr->vertex(2)->point()));
		
	Poligon_Convex poligon(punts_poligon);
	vector<Pla> plans;
	cout<<"POLIGON: ";
	poligon.escriure();
	
	return triangles_intersequen_poliedre(tr,triangle_pv,poligon,plans);
	
	vector<pCara>  cares;
	for(IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
	{
		bool intersecciona=false;
		for(int i =0;i<3;i++)
			intersecciona=intersecciona||poligon.intersecciona2D(Segment(Punt(fit->vertex(i)->point()),Punt(fit->vertex((i+1)%3)->point())),false);
		if(intersecciona) cares.push_back(fit);
	}
	return cares;
}


vector<pCara> Terreny::provant_interseccio(const pCara &tr,const pCara &triangle_pv,const Punt &p)const
{
	cout<<"provant_interseccio"<<endl;
	vector<Punt> punts_poligon;
	punts_poligon.push_back(p);
	punts_poligon.push_back(Punt(tr->vertex(0)->point()));
	punts_poligon.push_back(Punt(tr->vertex(1)->point()));
	punts_poligon.push_back(Punt(tr->vertex(2)->point()));
		
	Poligon_Convex poligon(punts_poligon);
	vector<Pla> plans;
	cout<<"POLIGON: ";
	poligon.escriure();
	Triangle tpv(triangle_pv);
	Triangle triang(tr);
	bool intersecciona=false;
	for(int i =0;i<3;i++)
		intersecciona=intersecciona||poligon.intersecciona2D(Segment(Punt(triangle_pv->vertex(i)->point()),Punt(triangle_pv->vertex((i+1)%3)->point())),false);
		
	return triangles_intersequen_poliedre(tr,triangle_pv,poligon,plans);
	
	vector<pCara>  cares;
	for(IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
	{
		bool intersecciona=false;
		for(int i =0;i<3;i++)
			intersecciona=intersecciona||poligon.intersecciona2D(Segment(Punt(fit->vertex(i)->point()),Punt(fit->vertex((i+1)%3)->point())),false);
		if(intersecciona) cares.push_back(fit);
	}
	return cares;
}

/******************************************************************************************************/
/*   bool Terreny::triangles_intersequen_paralelapipede(pCara triangle, Segment ss)                   */
/*Mètode que retorna els triangles que intersequen el paralelapipede definit pel triangle i el segment*/
/******************************************************************************************************/

vector<pCara> Terreny::triangles_intersequen_piramide(const pCara &tr,const pCara &triangle_pv,const Punt &pv)const 
{
	
	Triangle triangle_actual(tr);
	vector<Pla> plans=triangle_actual.plansDelimitadors(pv);

	vector<Punt> punts_poligon;
	punts_poligon.push_back(pv);
	punts_poligon.push_back(Punt(tr->vertex(0)->point()));
	punts_poligon.push_back(Punt(tr->vertex(1)->point()));
	punts_poligon.push_back(Punt(tr->vertex(2)->point()));
		
	Poligon_Convex poligon(punts_poligon);
	
	return triangles_intersequen_poliedre(tr,triangle_pv,poligon,plans);

	vector<pCara>  cares_interseccionen;
	Punt3d c=triangle_actual.circumcenter();
	for(IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
	{
		Triangle aux(fit);
// 		if( aux.interseccioPiramide(plans)&& !Triangle(fit).dinsTriangle(c)) 
		if( aux.interseccioParalelapipede(plans)&& !Triangle(fit).dinsTriangle(c)) 
			cares_interseccionen.push_back(fit);
	}
	return cares_interseccionen;

}


/************************************************************************************/
/*   vector<Segment> Terreny::segments_triangle( pCara cara)                        */
/*Mètode que retorna el vector de segments que conte el triangle de la cara indicada*/
/************************************************************************************/

vector<Segment> Terreny::segments_triangle(const pCara &cara) const
{	
	Punt p0, p1, p2, xminima, xmaxima, xmitja, punt1, punt2;
	Segment seg, recta, aresta, aresta1, aresta2;
	vector<Segment> vector_segments;
	double rang_x;
	Pla triangle;
	int pixel;
	double coord_z;
	double x_minV2d, y_minV2d, x_maxV2d, y_maxV2d;
	vector_segments.clear();
	x_minV2d = limits[0] ;
	y_minV2d = limits[1] ; 
	x_maxV2d = limits[3] ;  
	y_maxV2d = limits[4] ;
	
	rang_x = x_maxV2d-x_minV2d;
	
//  	cout<<"LIMITS: "<<x_minV2d<<" - "<<x_maxV2d<<" - "<<y_minV2d<<" - "<<y_maxV2d<< " T: "<<tria.xmax()<<"-"<<tria.xmax()<<tria.ymin()<<"-"<<tria.ymax()<<endl;
	if(!es_cara_infinita(cara))
	{
		Triangle tria(cara);
		if(x_maxV2d>tria.xmin()&&x_minV2d<tria.xmax()&& y_maxV2d>tria.ymin()&&y_minV2d<tria.ymax())
		{
			pVertex vh0 = cara->vertex(0);
			pVertex vh1 = cara->vertex(1);
			pVertex vh2 = cara->vertex(2);         
		
			p0.modifica_coordenades(vh0->point().x(), vh0->point().y(), vh0->point().z());
			p1.modifica_coordenades(vh1->point().x(), vh1->point().y(), vh1->point().z());
			p2.modifica_coordenades(vh2->point().x(), vh2->point().y(), vh2->point().z());
		
			/*Determinar la xminima i la x maxima del triangle actual*/
			if(p0.x()<=p1.x() && p0.x()<=p2.x())
			{
				xminima = p0;
				if(p1.x()>=p2.x()){xmaxima = p1;xmitja = p2;}
				else{	xmaxima = p2;xmitja = p1;}
			}
			else
			{
				if(p1.x()<=p0.x() && p1.x()<=p2.x())
				{
					xminima = p1;
					if(p0.x()>=p2.x()){xmaxima = p0;xmitja = p2;}
					else{xmaxima = p2;xmitja = p0;}
				}
				else
				{
					xminima = p2;
					if(p1.x()>=p0.x()){xmaxima = p1;xmitja = p0;}
					else{xmaxima = p0;xmitja = p1;}
				}
			}
			
			aresta.setOrigen(xminima);
			aresta.setDesti(xmaxima);
			
			aresta1.setOrigen(xminima);
			aresta1.setDesti(xmitja);
	
			aresta2.setOrigen(xmitja);
			aresta2.setDesti(xmaxima);
			triangle = Pla(p0,p1,p2);
		
			/*Desde la ximinima fins a la xmaxima convertides a pixels anirem guardant-nos els segments*/
			int i0=(int)floor(((max(x_minV2d,xminima.x())-x_minV2d)/rang_x)*ample_graella);
			int i1 =ceil(((min(x_maxV2d,xmaxima.x())-x_minV2d)/rang_x)*ample_graella)+1;
			for(pixel=i0;pixel<i1;pixel++)
			{
				recta = Segment(Punt((x_minV2d+((double)pixel/(double)ample_graella)*rang_x), 0.0, 0.0), Punt((x_minV2d+((double)pixel/(double)ample_graella)*rang_x), tria.ymax()+1./*20.0*/, 0.0) );
				punt1 = aresta.puntInterseccio(recta, false);
				punt1.modifica_y(min(max(punt1.y(),y_minV2d),y_maxV2d));
				coord_z = triangle.calcula_Z(punt1,p1);
				punt1.modifica_z(coord_z);
				
				if((x_minV2d+((double)pixel/(double)ample_graella)*rang_x)< xmitja.x())
					punt2 = aresta1.puntInterseccio(recta,false);
				else
					punt2 = aresta2.puntInterseccio(recta,false);
					
				punt2.modifica_y(min(max(punt2.y(),y_minV2d),y_maxV2d));
				coord_z = triangle.calcula_Z(punt2,p1);
				punt2.modifica_z(coord_z);
				
				if(xmaxima.y()<xmitja.y())
				{
					seg.setOrigen(punt1);
					seg.setDesti(punt2);
				}
				else
				{
					seg.setDesti(punt1);
					seg.setOrigen(punt2);
				}
			
				vector_segments.push_back(seg);
			}
// 			cout<<endl;
		}
	}
	
// 	char kdklj;
// 	cin>>kdklj;
	return vector_segments;
}


vector<vector<Segment> > Terreny::obtenir_segments() const
{	
	Punt p0, p1, p2, xminima, xmaxima, xmitja, punt1, punt2;
	Segment seg, recta, aresta, aresta1, aresta2;
	vector<vector<Segment> > tots_segments;
	vector<Segment> vector_segments;
	
	double rang_x;
	Pla triangle;
	int pixel;
	double coord_z;
	double x_minV2d, y_minV2d, x_maxV2d, y_maxV2d;
	
	
	x_minV2d = limits[0] ;
	y_minV2d = limits[1] ; 
	x_maxV2d = limits[3] ;  
	y_maxV2d = limits[4] ;
	
	rang_x = x_maxV2d-x_minV2d;
	
//  	cout<<"LIMITS: "<<x_minV2d<<" - "<<x_maxV2d<<" - "<<y_minV2d<<" - "<<y_maxV2d<< " T: "<<tria.xmax()<<"-"<<tria.xmax()<<tria.ymin()<<"-"<<tria.ymax()<<endl;
	for(IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
	{
		pCara cara=fit;
		vector_segments.clear();
		if(!es_cara_infinita(cara))
		{
			Triangle tria(cara);
			if(x_maxV2d>tria.xmin()&&x_minV2d<tria.xmax()&& y_maxV2d>tria.ymin()&&y_minV2d<tria.ymax())
			{
				pVertex vh0 = cara->vertex(0);
				pVertex vh1 = cara->vertex(1);
				pVertex vh2 = cara->vertex(2);         
			
				p0.modifica_coordenades(vh0->point().x(), vh0->point().y(), vh0->point().z());
				p1.modifica_coordenades(vh1->point().x(), vh1->point().y(), vh1->point().z());
				p2.modifica_coordenades(vh2->point().x(), vh2->point().y(), vh2->point().z());
			
				/*Determinar la xminima i la x maxima del triangle actual*/
				if(p0.x()<=p1.x() && p0.x()<=p2.x())
				{
					xminima = p0;
					if(p1.x()>=p2.x())
					{
						xmaxima = p1;
						xmitja = p2;
					}
					else
					{
						xmaxima = p2;
						xmitja = p1;
					}
				}
				else
				{
					if(p1.x()<=p0.x() && p1.x()<=p2.x())
					{
						xminima = p1;
						if(p0.x()>=p2.x())
						{
							xmaxima = p0;
							xmitja = p2;
						}
						else
						{
							xmaxima = p2;
							xmitja = p0;
						}
					}
					else
					{
						xminima = p2;
						if(p1.x()>=p0.x())
						{
							xmaxima = p1;
							xmitja = p0;
						}
						else
						{
							xmaxima = p0;
							xmitja = p1;
						}
					}
				}
				
				aresta.setOrigen(xminima);
				aresta.setDesti(xmaxima);
				
				aresta1.setOrigen(xminima);
				aresta1.setDesti(xmitja);
		
				aresta2.setOrigen(xmitja);
				aresta2.setDesti(xmaxima);
				triangle = Pla(p0,p1,p2);
			
				/*Desde la ximinima fins a la xmaxima convertides a pixels anirem guardant-nos els segments*/
				int i0=(int)floor(((max(x_minV2d,xminima.x())-x_minV2d)/rang_x)*ample_graella);
				int i1 =ceil(((min(x_maxV2d,xmaxima.x())-x_minV2d)/rang_x)*ample_graella)-1;
				for(pixel=i0;pixel<=i1;pixel++)
				{
					recta = Segment(Punt((x_minV2d+((double)pixel/(double)ample_graella)*rang_x), 0.0, 0.0), Punt((x_minV2d+((double)pixel/(double)ample_graella)*rang_x), tria.ymax()+1./*20.0*/, 0.0) );
					punt1 = aresta.puntInterseccio(recta, false);
					punt1.modifica_y(min(max(punt1.y(),y_minV2d),y_maxV2d));
					coord_z = triangle.calcula_Z(punt1,p1);
					punt1.modifica_z(coord_z);
					
					if((x_minV2d+((double)pixel/(double)ample_graella)*rang_x)< xmitja.x())
						punt2 = aresta1.puntInterseccio(recta,false);
					else
						punt2 = aresta2.puntInterseccio(recta,false);
					punt2.modifica_y(min(max(punt2.y(),y_minV2d),y_maxV2d));
					coord_z = triangle.calcula_Z(punt2,p1);
					punt2.modifica_z(coord_z);
					
					if(xmaxima.y()<xmitja.y())
					{
						seg.setOrigen(punt1);
						seg.setDesti(punt2);
					}
					else
					{
						seg.setDesti(punt1);
						seg.setOrigen(punt2);
					}
				
					vector_segments.push_back(seg);
				}
	// 			cout<<endl;
			}
		}
		tots_segments.push_back(vector_segments);
	}

	return tots_segments;
}



/********************************************************************************************************************/
/*   void Terreny::setamplada_graella(int a)                                                                                */
/*Mètode que permet assignar l'amplada_graella de la finestra 2d per poder calcular els segments de pixels de cada triangle */
/********************************************************************************************************************/

void Terreny::setAmpladaGraella(int a)
{
	ample_graella = a;
}

/*******************************************************************************************************************/
/*   int Terreny::getamplada_graella()                                                                                     */
/*Mètode que permet obtenir l'amplada_graella de la finestra 2d per poder calcular els segments de pixels de cada triangle */
/*******************************************************************************************************************/

int Terreny::getAmpladaGraella()
{
	return ample_graella;
}

/******************************************************************************************************************************/
/*   vector<pCara> Terreny::triangles_intersequen(Segment ss, Segment *sv, pCara triangle, Punt punt, Tetrahedron &tetraedre) */
/*Mètode que retorna el vector de cares que interseccionen el quadrilater format pels segments ss i sv                        */
/******************************************************************************************************************************/

vector<pCara> Terreny::triangles_intersequen(const  Segment &ss, const  Segment &sv,const pCara &triangle, Tetrahedron &tetraedre) const
{
	/*Triangle * tr;
	vector<pCara> cares;
	Segment aresta_e, aresta_inicial, vect1, *vect2;
	vector<Punt> punts;
	vector<Punt3d> points;
// 	Poligon_Convex *quadrilater, *poligon, *poligon_vei;
	Punt aux1, aux2;
	int i=0;
// 	bool primer = true, entrada = false;
	
	punts.push_back(ss.getOrigen());
	punts.push_back(ss.getDesti());
	vect1.setOrigen(ss.getOrigen());
	vect1.setDesti(sv.getOrigen());
	vect2 = new Segment(ss.getDesti(), sv.getDesti());
	if(vect1.interseccioSegments(vect2))
	{
		punts.push_back(sv.getOrigen());
		punts.push_back(sv.getDesti());
	}
	else
	{
		punts.push_back(sv.getDesti());
		punts.push_back(sv.getOrigen());
	}
	for(i=0;i<4;i++)
	{
		points.push_back(Punt3d(punts[i].x(), punts[i].y(), punts[i].z()));
	}
	cares.push_back(triangle);
	tetraedre = Tetrahedron(points[0], points[1], points[2], points[3] );
	IteradorCares fit;
	Triangle triang(triangle);
	for( fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
   	{
		tr = new Triangle(fit); 
		if( .tr_intersects_T(tetraedre) )
			cares.push_back(fit);
		delete tr;
	}
	delete vect2;
		
	return cares;*/
	
	// empilem les cares que intersequen el quadrilater format per ss i sv.
	Segment s1, s2;
	vector<Punt> punts;
	Poligon_Convex quadrilater;
	vector<pCara> cares;
	punts.push_back(ss.getOrigen());
	punts.push_back(ss.getDesti());
	s1=Segment(ss.getOrigen(),sv.getOrigen());
	s2 = Segment(ss.getDesti(), sv.getDesti());
	if(s1.interseccioProjeccio(s2))
	{
		punts.push_back(sv.getOrigen());
		punts.push_back(sv.getDesti());
	}
	else
	{
		punts.push_back(sv.getDesti());
		punts.push_back(sv.getOrigen());
	}
	Vector3D vect0(ss.getVector().y(),-ss.getVector().x(),0.);
	Vector3D vect1(punts[2].x()-punts[1].x(),punts[2].y()-punts[1].y(),0.);
	if(vect0.producteEscalar(vect1)>0) // vect1 va de fora a dins, o sigui q esta del reves;
	{
		punts[1]=punts[3];
		punts[3]=ss.getDesti();
	}
	quadrilater = Poligon_Convex(punts);
	//punts[3] esta a l'esquerre de ss si el determinant es negatiu
	IteradorCares fit;
	Triangle tr;
	
	bool interseca=false;
	int i=0;
	for( fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
	{
		tr = Triangle(fit); 
		i=0;
		while(i<3 && !interseca)
		{
			Segment s(tr.getTriangle_cgal().vertex(i),tr.getTriangle_cgal().vertex((i+1)%3));
			
 			if( quadrilater.intersecciona2D(s,false))
			{
				cares.push_back(fit);
				interseca = true;
			}
			i++;
		}
		interseca=false;
	}
	
	return cares;
}

/********************************************************************************************************/
/*   vector<Segment> Terreny::segments_intersequen(Segment ss, Segment *sv)                             */
/*Mètode que retorna els segments que intersequen el trapezi format per ss i sv i estan sobre el terreny*/
/********************************************************************************************************/

vector<Segment> Terreny::segments_intersequen(Segment &ss, Segment &sv,pCara p) const
{
	if(ss.getOrigen().x()>ss.getDesti().x()) ss=ss.twin();	
	if(sv.getOrigen().x()>sv.getDesti().x()) sv=sv.twin();


	// busquem el pla del trapezi amb els segments reals
	vector<Segment> vector_segments;
	Pla pla_trapezi(ss.getOrigen() , sv.getOrigen(), sv.getDesti()) ;
	if(pla_trapezi.getPla().is_degenerate())
		pla_trapezi=Pla(ss.getDesti() , sv.getOrigen(), sv.getDesti()) ;
	
	Triangle tr;
	IteradorCares fit;
	Segment segment_ss, segment_contingut, aresta;
	bool triangle_contingut;
	
	// busquem la transformacio q passarà els segments a les posicions q volem.
	Transformation3 transformacio;
	transformacio = ss.calcularTransformacioCoplanars(sv);
		
	/*Crear variables auxiliars per ss i sv aplicant la transformacio*/
	ss.setOrigen(transformacio(ss.getOrigen().getPunt()));
	ss.setDesti(transformacio(ss.getDesti().getPunt()));
	sv.setDesti(transformacio(sv.getDesti().getPunt()));
	sv.setOrigen(transformacio(sv.getOrigen().getPunt()));
		
	if(sv.getOrigen().x()>sv.getDesti().x()) sv=Segment(sv.getDesti(),sv.getOrigen());

	/*Per cadascun dels triangles del terreny comprovem si interseca amb el pla format per ss i sv*/
	for( fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
   	{
		tr = Triangle(fit); 
		/*En el cas que si que intersequi calculem el segment contingut en el pla per on interseca*/
		if(pCara(fit)==p){}
// 			cout<<"trianlge conte ss, no considerat"<<endl;
		else if(pla_trapezi.interseccio(tr, segment_ss, triangle_contingut))
		{
			// intersequem cara i pla trapezi (real), transformem el segment i estudiem la part q hi ha dins el trapezi amb seg ja rotat.. 
// 			cout<<"p/t";
			segment_ss.setOrigen(transformacio(segment_ss.getOrigen().getPunt()));
			segment_ss.setDesti(transformacio(segment_ss.getDesti().getPunt()));
// 			cout<<"tr/pla "; segment_ss.escriure();
			if(triangle_contingut)
			{
				for(int i=0;i<3;i++)
				{
					aresta = Segment(fit->vertex(i)->point(), fit->vertex((i+1)%3)->point());
					if(aresta.dinsTrapezi(ss, sv, segment_contingut))
						vector_segments.push_back(segment_contingut);
				}
			}
			else
			{
				if(segment_ss.dinsTrapezi(ss, sv, segment_contingut))
					vector_segments.push_back(segment_contingut);
			}
		}
	}
	return vector_segments;
}

/* retorna cert si el segment s interseca la mesh. */
bool Terreny::linia_interseca(const Segment &s)const
{
	IteradorCares fit;
	
	/*Per cadascun dels triangles del terreny comprovem si interseca amb el pla format per ss i sv*/
	for( fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++ )
   	{
		Triangle tr(fit); 
		if(tr.interseccio(s,false)) return true;
	}
	return false;
}


/****************************************************/
/*   vector<pCara> Terreny::getCares()              */
/*Mètode que retorna les cares que formen el terreny*/
/****************************************************/

vector<pCara> Terreny::getCares()const
{
	vector<pCara> cares;
	IteradorCares fit;
	
	fit = dt.finite_faces_begin();
	
	for( ; fit != dt.finite_faces_end(); ++fit)
	{
		if(!es_cara_infinita(fit))
			cares.push_back(fit);

// 		for(int i=0;i<3;i++)
// 			cout<<i<<" ("<<fit->info().entrada[i][0]<<", "<<fit->info().entrada[i][1]<<", "<<fit->info().entrada[i][2]<<", "<<fit->info().entrada[i][3]<<"); ";
// 		cout<<endl;
	}
	return cares;
}

void Terreny::setSegmentIntersecciona(Segment s)
{
	segment_intersecciona = s;
}

void Terreny::setTriangleIntersecciona(Triangle t)
{
	triangle_intersecciona = t;
}
void Terreny::setCaresIntersecciona(vector<pCara> c)
{
	cares_interseccio = c;
}

int Terreny::tipusInterseccio()
{
	return tipus;
}

void  Terreny::settipusInterseccio(int t)
{
	tipus = t;
}

bool Terreny::comprovarInterseccio()
{
	if(tipus != INDEFINIT)
		return true;
	else
		return false;
}

Segment Terreny::getSegmentIntersecciona()
{
	return segment_intersecciona;
}

Triangle Terreny::getTriangleIntersecciona()
{
	return triangle_intersecciona;
}

vector<pCara> Terreny::getCaresIntersecciona()
{
	return cares_interseccio;
}

void Terreny::esborrarEntrades()
{
	for( IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++)
		fit->info().entradaCalculada=false;
}

void Terreny::calculaEntrades(const Punt &pv,pCara &triangle_pv)
{
	int vertex;
	Localitzacio tipus;
	triangle_pv = dt.locate(pv.getPunt(), tipus, vertex);
	buscadorDEntrades(pv);
	
}

void Terreny::calculaEntrades(const Segment &sv,pCara &triangle_pv)
{
	int vertex;
	Localitzacio tipus;
	triangle_pv = dt.locate(sv.punt_mig_3d(), tipus, vertex);
	buscadorDEntrades(Punt(sv.punt_mig_3d()));

}

void Terreny::buscadorDEntrades(const Punt &p)
{
	esborrarEntrades();
	//l'entrada guardada a fit->info().entrada[i] es l'entrada de l'aresta oposada al vertex fit->vertex(i)
	Punt p_aux(p.x()+1,p.y(),0);
	for( IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++)
	{
		for(int i=0;i<3;i++)
		{
			pCara cara_veina=fit->neighbor(i);
			if(!cara_veina->info().entradaCalculada||dt.is_infinite(cara_veina))
			{
				Segment s(Punt(fit->vertex((i+1)%3)->point()),Punt(fit->vertex((i+2)%3)->point()));
				Punt x=s.calculaProperRecta2D(p);
				Punt u=s.getOrigen(), v=s.getDesti();
				if(p.distancia2D(u)>p.distancia2D(v)){ u=s.getDesti(); v=s.getOrigen();}
				fit->info().entrada[i][0]=s.distancia2D(p);
				fit->info().entrada[i][1]=p_aux.angle2D(p,x);
				fit->info().entrada[i][2]=u.angle2D(p,v);
				fit->info().entrada[i][3]=u.angle2Dccw(p,v);
			}
			else
			{
				int index=cara_veina->index(fit);
				fit->info().entrada[i][0]=cara_veina->info().entrada[index][0];
				fit->info().entrada[i][1]=cara_veina->info().entrada[index][1];
				fit->info().entrada[i][2]=cara_veina->info().entrada[index][2];
				fit->info().entrada[i][3]=cara_veina->info().entrada[index][3];
			}
		}
		fit->info().entradaCalculada=true;
	}
}


void Terreny::calculGirsDesplegament()
{

for (IteradorCares fit = dt.finite_faces_begin(); fit != dt.finite_faces_end(); fit++)
    {
        Pla pla_cara(Plane(fit->vertex(0)->point(),fit->vertex(1)->point(),fit->vertex(2)->point()));
        //   pla_cara.escriure();
        for(int i=0;i<3;i++)
        {
            pCara cara_veina=fit->neighbor(i);
            if(!es_cara_infinita(cara_veina))
            {
                Punt p(fit->vertex((i+1)%3)->point());
                Vector3D u(p,Punt(fit->vertex((i+2)%3)->point()));
                Vector3D v(p,Punt(fit->vertex(i)->point()));
                Vector3D n=u.producte_vectorial(v);
                v=n.producte_vectorial(u);
               
                Vector3D vaux(p,Punt(cara_veina->vertex(cara_veina->index(fit))->point()));
                Vector3D naux=u.producte_vectorial(vaux);
               
                double angle=n.angle(naux,true); //false
               
                u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
                Transformation3 N(u.x(),v.x(),n.x(),p.x(),u.y(),v.y(),n.y(),p.y(),u.z(),v.z(),n.z(),p.z(),1);
                Transformation3 Ra(1,0,0,0,-cos(angle),-sin(angle),0,-sin(angle),cos(angle),1);
                Transformation3 Na=N.inverse();
                   
                //Punt p_imatge(N(Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point()))));
		Punt p_imatge(Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point())));
		
		//Si totes dos han qudat més grosses quedar-nso amb la z més petita. Guardar la z de p_imatge i comparar-la amb l'altra.
		double valorzeta=p_imatge.z();
			
		//if (!pla_cara.puntPertanyPla(p_imatge))
		if(fabs(p_imatge.z())>1e-6)
		{
			Ra=Transformation3(1,0,0,0,-cos(-1*angle),-sin(-1*angle),0,-sin(-1*angle),cos(-1*angle),1);
                                
			//if(!pla_cara.puntPertanyPla(Punt(N(Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point()))))))
			if(fabs(Punt(Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point()))).z())>1e-6)//comprovacio
//			        cout<< pla_cara.avaluarPunt(Punt(N(Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point())))));
				cout << " z " << Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point())).z();
			//comprovacio
                	else cout << "i2";//comprovacio
		}
                else cout << "i"; //comprovacio
                
		//cout << " p_imatge.y() = " << p_imatge.y() << endl;  
		double y=Ra(Na(cara_veina->vertex(cara_veina->index(fit))->point())).y();
		if(y>0) cout << " " << y;
		
		//fit->info().T[i]=(N * ( Ra * (Na)));
		fit->info().T[i]=(Ra*Na);
		// LA TRANSFORMACIO ES Na*Ra
		/*Punt(fit->info().T[i](Punt3d (1,0,0))).escriure();
		Punt(fit->info().T[i](Punt3d (0,1,0))).escriure();
		Punt(fit->info().T[i](Punt3d (0,0,1))).escriure();*/
            }
        }
        cout << "; ";
    }
    cout << endl;
    
}

//Retorna cert si un vèrtex és hiperbòlic (>2*PI)
bool Terreny::vertexHiperbolic(const pCara &cara, const pVertex &vertex)
{
double angle=0;

CirculadorCares cir=(dt.incident_faces(vertex,cara));
				
		do {
			if(es_cara_infinita(cir))
			{
				cout << "Es un vertex frontera (false)" << endl;
				return true;
				//return false;
			}
			else
			{
				int m=0;
				while (cir->vertex(m)!=vertex)
				{ m++ ; }
								
				Vector3D y((Punt(cir->vertex(m)->point())),(Punt(cir->vertex((m+1)%3)->point())));
				Vector3D x((Punt(cir->vertex(m)->point())),(Punt(cir->vertex((m+2)%3)->point())));
				angle=angle + acos((y*x)/(sqrt((y*y)*(x*x))));
			
				if (angle>=2*PI) { return true; }
				
			}
			cir++;
		
		} while (cir!=cara);
	
return false;

}


void Terreny::inicialitzarVertexs()
//Inicialitza les distàncies de tots els vèrtexs de la triangulació a -1 
{
for (IteradorVertexs vit = dt.finite_vertices_begin(); vit != dt.finite_vertices_end(); vit++)
{ 
	vit->info().distancia=-1;
	vit->info().index=-1; 

} 
}


//Comprovem que al final de tot no hi hagi cap vèrtex amb distància = -1
void Terreny::comprovarVertexs()
{
for (IteradorVertexs vit = dt.finite_vertices_begin(); vit != dt.finite_vertices_end(); vit++)
{ 
	if (vit->info().distancia==-1)  // vit es una mena de pVertex.
	{
		char a;
		cout << "Vertex amb distància -1 = ( " << vit->point().x() << " , " << vit->point().y() << " , " << vit->point().z() << ")" << endl;
		cout << "Hem trobat un vèrtex amb distància = -1. Prem tecla per continuar. " << endl;
		cin >> a;
	}
	if (vit->info().index==-1)  // vit es una mena de pVertex.
	{
		char a;
		cout << "Vertex amb index -1 = ( " << vit->point().x() << " , " << vit->point().y() << " , " << vit->point().z() << ")" << endl;
		cout << "Hem trobat un vèrtex amb index = -1. Prem tecla per continuar. " << endl;
		cin >> a;
	}		
}
}

bool Terreny::infoVertexNoCorrecte()
{
	int distancia=0;
	int index=0;
	for (IteradorVertexs vit = dt.finite_vertices_begin(); vit != dt.finite_vertices_end(); vit++)
	{ 
		if(vit->info().distancia<0) distancia++;
		if(vit->info().index<0) index++;
	 } 
	 
	 //cout<<"distancies negatives: "<<distancia<<endl;
	 //cout<<" index negatius: "<<index<<endl;
	 return distancia>0 && index>0;
}


//Mètode que ens propaga un PseudoSource
list<Event*> Terreny::propagarPseudoSource(PseudoSource ps)
{
list<Event*> E;
pCara carapseudo, tri, cara;
pVertex vertexpseudo;
double sigmapseudo,t;
int indexpseudo, l, index, j;
double b0,b1,d0,d1,sigma,pes;
Punt3d puntpseudo, puntvertex1, puntvertex2;

//Obtenim els paràmetres del PseudoSource ps
carapseudo=ps.getcara(); 
vertexpseudo=ps.getvertex();
puntpseudo=vertexpseudo->point();
sigmapseudo=ps.getsigma();
indexpseudo=ps.getindex();

//Per cada cara menys la pCara carapseudo, per a cada vèrtex fer 3 finestres (una per cada costat).
CirculadorCares fit=(dt.incident_faces(vertexpseudo,carapseudo));
fit++;
do{
	if(!es_cara_infinita(carapseudo)) //Només fem quan no és una cara infinita
	{		
		for(l=0; l<2; l++) // Creem una finestra per cada vertex amb les distancies que toquin. Cada un serà un Punt3d. 
		{
			puntvertex1=fit->vertex((l+1)%3)->point();
			puntvertex2=fit->vertex((l+2)%3)->point();
														
			d0=sqrt((puntpseudo.x()-puntvertex1.x())*(puntpseudo.x()-puntvertex1.x())+(puntpseudo.y()-puntvertex1.y())*(puntpseudo.y()-puntvertex1.y())+(puntpseudo.z()-puntvertex1.z())*(puntpseudo.z()-puntvertex1.z()));
			d1=sqrt((puntpseudo.x()-puntvertex2.x())*(puntpseudo.x()-puntvertex2.x())+(puntpseudo.y()-puntvertex2.y())*(puntpseudo.y()-puntvertex2.y())+(puntpseudo.z()-puntvertex2.z())*(puntpseudo.z()-puntvertex2.z()));
			b0=0;
			b1=sqrt((puntvertex1.x()-puntvertex2.x())*(puntvertex1.x()-puntvertex2.x())+(puntvertex1.y()-puntvertex2.y())*(puntvertex1.y()-puntvertex2.y())+(puntvertex1.z()-puntvertex2.z())*(puntvertex1.z()-puntvertex2.z()));
			sigma=ps.getsigma();
			cara=fit; 
			index=ps.getindex(); //Index de la posicio del vector on hi ha el punt
			j=l; //Serà el vertex oposat
			
			Finestra * f = new Finestra(b0,b1,d0,d1,sigma,cara,j,index);
			//f=new Finestra(ps_v.distancia(puntvertex1),ps_v.distancia(puntvertex2),0,Punt(puntvertex1).distancia(puntvertex2),ps.getsigma(),fit,l,ps.getindex());
			f->escriure();
			
			fit->info().LlistaFinestres[j].push_front(*f); //Insereix la finestra f a l'inici de la Llista Finestres
					
			//Inserim la finestra a la llista d'events (serà del tipus EVENT_FINESTRA(0))
			E.push_back(f); //Insereix events tipus Finestra al final de la llista d'Events
			//cout << "Inserim a la llista d'Events la finestra amb j = " << j << " i amb pes = " << f->getpes() << endl;
				
			//d0 es la distancia del vertex l+1 al pseudosource
			if((fit->vertex((l+1)%3)->info().distancia==-1) || (fit->vertex((l+1)%3)->info().distancia>d0))
			{ fit->vertex((l+1)%3)->info().distancia=d0;
			  fit->vertex((l+1)%3)->info().index=index;	
			}				
		}
	}
fit++;	

} while (fit!=carapseudo);

E.sort(Event::ordena); //Ordenem la llista d'events a retornar de menys a més pes
return E;
}


//Mètode que ens propaga una Finestra
list<Event*> Terreny::propagarFinestra(Finestra f)
{
	list<Event*> E;
	pCara cara, cara_veina;
	double b0,b1,d0,d1,sigma,l;
	int index, jfinestra, indexpseudo;
	Punt2d s,t; //s serà la font (source), t la imatge del vertex vei al pla.
 	Punt3d aux;
	bool veins=false;
	
	//Obtenim els paràmetres de la Finestra f
	f.getParametres(cara,b0,b1,d0,d1,sigma,index,jfinestra);
	f.escriure();
	//Trobem la font s 
	s=f.trobarSource(veins); //Calculem la font s (source)
	cout << "Source x = " << s.x() << " y = " << s.y() << endl;
	
	//Fer l'unfolding de la pCara de la finestra, trobar la cara adjacent i llavors fer l'unfolding del vèrtex oposat
	cara_veina=cara->neighbor(jfinestra);
			
	if (!es_cara_infinita(cara_veina))
	{
		//Per tenir l, 1agafar els 2 punts i calcular la longitud
		Punt3d pvei(cara_veina->vertex(cara_veina->index(cara))->point());
		Punt p1(cara->vertex((jfinestra+1)%3)->point());
		Punt p2(cara->vertex((jfinestra+2)%3)->point());
		l=p1.distancia(p2.getPunt());
		
		aux=cara->info().T[jfinestra](cara_veina->vertex(cara_veina->index(cara))->point());
		t=Punt2d(aux.x(),aux.y());
 		cout<<"T"; Punt(aux).escriure();
		
		double dist1, dist2;
		dist1=p1.distancia(pvei); 
		dist2=p2.distancia(pvei);
		//double y,x; 
		//x=((dist1*dist1)-(dist2*dist2)+(l*l))/(2*l); 
		//y=-1*sqrt((dist1*dist1)-(x*x));t=Punt2d(x,y);cout <<" T =("<<t.x()<<", "<< t.y()<<")";
		
		//Interseccions de finestres
		double P[4];
		double U[4];
		double V[4];
		double e[2];
		
		P[0]=0; P[1]=l; P[2]=b0; P[3]=b1; //Inicialització vector P
		U[0]=t.x()/dist1; U[1]=(t.x()-l)/dist2; U[2]=s.x()-b0; U[3]=s.x()-b1; //Inicialització vector U
		V[0]=t.y()/dist1; V[1]=t.y()/dist2; V[2]=s.y(); V[3]=s.y(); //Inicialització vector V
		e[0]=dist1; e[1]=dist2; //Inicialització vector e
		
		cout<<"p (" << P[0] << "," << P[1] << "," << P[2] << "," << P[3] << ")  u=(" << U[0] << "," << U[1] << "," << U[2] << "," << U[3] << ")  v=(" << V[0] << "," << V[1] << "," << V[2] << "," << V[3] << ")  e=(" << e[0] << "," << e[1] << ")" << endl;
		
		int c1,c2,i,j;
		double ss,tt,t1,t2;
		i=0; j=2; t1=-1; t2=-1;
			
		while ((i<2) && (t2<0))
		{
			if(fabs(V[i])<TOLERANCE) 
			{
				ss=0;
				if(fabs(U[i])<TOLERANCE) cout<<"vi=0, ui=0 ........ UPS!!";
				else{ cout<<"vi=0";  tt=(P[j]-P[i])/U[i]; }
			}
			else 
			{
				double den=U[i]*V[j]/V[i]-U[j];
				if(fabs(den)<TOLERANCE) 
				{
					if(fabs((P[j]-P[i]))<TOLERANCE)
					{
						cout << " Numerador i Denominador = 0 ....... " << endl;
						if (i==0) 
						{
						tt=0;
						}
						else 
						{
						tt=dist2;
						}
					}
					else
					{
					cout<<" Numerador !=0 i dennominador = 0  ........ UPS!!" << endl;
					}
				}
				else{ ss=(P[j]-P[i])/den; tt=ss*V[j]/V[i];}
			}
			cout << " i = " << i << " j = " << j << " ss = " << ss << " tt = " << tt /*<<" e[0] = " << e[0] << " e[1] = " << e[1] */ << endl;
			
			if ((tt>-1*TOLERANCE) && (tt<=e[i]+TOLERANCE))
			{
				if (t1==-1){t1=tt;c1=i;}
				else{t2=tt;c2=i;}
				j++;
			}
			else i++;
		}
		
		cout << " c1 = " << c1 << " c2 = " << c2 << " j = " << j << endl;
		
		Punt3d s3(s.x(),s.y(),0.);
		Punt2d porigen(0,0);
		//Finestra * f2;

		cout << "PSEUDOVERTEX : ( " << s.x() << " , " << s.y() << " ) " << endl;
		char tecla;
		//Creem les finestres
		if ((c1==0) && (c2==0) && (c1==c2))
		{
			Punt pini(porigen+(t1/dist1)*(t-porigen)), pfi(porigen+(t2/dist1)*(t-porigen));
			Finestra * f2=new Finestra (t1,t2,pini.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+2)%3)),index);
			cout << "n ";
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+2)%3))].push_front(*f2); 
			f2->escriure();
			E.push_back(f2);
		}
		else if ((c1==1) && (c2==1) && (c1==c2))
		{
			Punt2d pa(l,0);
			Punt pini(pa+(t1/dist2)*(t-pa)), pfi(pa+(t2/dist2)*(t-pa));
			Finestra * f2=new Finestra(dist2-t1,dist2- t2,pini.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+1)%3)),index);
			cout << "n ";
			f2->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+1)%3))].push_front(*f2);
			E.push_back(f2);
		}
		else if(c1!=0 || c2!=1) cout<<"CAS NO CONTEMPLAT"<<endl;
		else // c1=0 i c2=1
		{
			Punt2d pa(l,0);
			Punt pini(porigen+(t1/dist1)*(t-porigen)), pmig(t), pfi(pa+(t2/dist2)*(t-pa));
			
			// una finestra com la del c1=c2=0 q va des de pini a pvei
			Finestra * f2=new Finestra (t1,dist1,pini.distancia(s3),pmig.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+2)%3)),index);
			cout << "n ";
			f2->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+2)%3))].push_front(*f2);
			E.push_back(f2);
			
			int index_vei=cara_veina->index(cara);
			if((cara_veina->vertex(index_vei)->info().distancia==-1) ||  (cara_veina->vertex(index_vei)->info().distancia>f2->getd1()))
			{
				cara_veina->vertex(index_vei)->info().distancia=f2->getd1();
				cara_veina->vertex(index_vei)->info().index=f2->getindex();
				if(vertexHiperbolic(cara_veina,cara_veina->vertex(index_vei)))
					E.push_back( new PseudoSource(f2->getd1()+sigma, i, cara_veina->vertex(index_vei), cara_veina));
			}
			
			Finestra * f3=new Finestra (0.,dist2- t2,pmig.distancia(s3),pfi.distancia(s3),sigma,cara_veina,cara_veina->index(cara->vertex((jfinestra+1)%3)),index);
			cout << "n ";
			f3->escriure();
			//cara_veina->info().LlistaFinestres[cara_veina->index(cara->vertex((jfinestra+1)%3))].push_front(*f3);
			E.push_back(f3);
		}
	}
	else
	{ cout << "Cara Veina Infinita No Propaguem" << endl; }
	
	E.sort(Event::ordena); //Ordenem la llista d'events segons el pes.
	//E.sort(); //Ordenem la llista d'events a retornar de menys a més pes
	return E;
}


