#ifndef  CLASSE_TERRENY_H
#define  CLASSE_TERRENY_H

#include "geometriabasica.h"
#include <qgl.h>
#include <vector.h>
#include <vector3D.h>
#include <list.h>
#include "superficiebezier.h"
#include "corbesnivell.h"
#include "segment.h"
#include "aresta.h"
#include "regio.h"
#include "pla.h"
#include "finestra.h"
#include "event.h"
#include "pseudosource.h"
#include "poligon_convex.h"

typedef std::vector<pVertex>  VectorVertexs;

/*************************************************************/
/*                         class Mesh                        */
/*   Classe que permet guardar les dades referents a la malla*/
/*************************************************************/

class Terreny
{
	public:
		Terreny(Delaunay, VectorVertexs ); /*Constructor amb parametres que permet configurar la malla amb la triangularització de Delaunay*/
		
		static Terreny* getInstancia();
		~Terreny();
		
		pVertex insereix( const Punt3d & );
		void elimina( pVertex vh );
		void clear();
		
		void inserirPunt(const Punt3d & );
		void inserirPuntMogut(Punt3d, int);
		void modificarPunt(int, double);
		void eliminarPunt(int);
		pVertex obtenirPunt(int);
		
		/*Mètode que permet modificar el Delaunay*/
		void setTriangulacio( const Delaunay & );
		void setTriangulacio( const Delaunay &, const VectorVertexs &);
		
		/*Metode que retorna la malla*/
		Delaunay *triangulacio();
		
		void generaTerreny();
		
		void generaBezier( int );
		void generaCorbes( const Nivells & );
		
		int numero_de_vertexs();
		int numero_de_cares();
		//int numero_de_arestes();
		
		int carrega( const QString& fn );
		int desa( const QString& fn );
		
		void setAmpladaGraella(int a);
		int getAmpladaGraella();
		
		void corbes3D( LlistaCares& );
		void corbes2D( LlistaCorbes& );
		
		bool modificat();
		void setModificat( bool );
		
		int resolucio();
		void setResolucio( int );
		int obtenirResolucioMax( double );
		
		double *maximsIMinims();
		double limits[6];
		
		void eliminarVertexs(int);
		VectorVertexs getVertexs();
		
		Punt vertex_xmin( int &posicio)const;
		bool es_cara_infinita(pCara cara)const;
		Segment seguentInfinita(const Segment&s, pCara &f) const;
		void seguentInfinita( int & v, pCara &f) const;
		vector<pCara> recorrer_triangulacio(const Segment &ss,const  Segment &sv,const   pCara &triangle,const  Punt & punt, Tetrahedron &tetraedre)const;
		vector<Segment> segments_triangle(const pCara &cara) const;
		vector<vector<Segment> > obtenir_segments() const;
		
		vector<pCara> triangles_intersequen(const  Segment &ss, const  Segment &sv,const pCara &triangle, Tetrahedron &tetraedre) const;
		vector<Segment> segments_intersequen(  Segment &ss, Segment &sv,pCara p) const;
		bool linia_interseca(const Segment &sv)const;
		vector<pCara> getCares()const;
		
		bool poligon_visio_intersecciona_terreny(Poligon &poligon_visio)const;
		
		vector<pCara> provant_interseccio(const pCara &tr,const pCara &triangle_pv,const Triangle &triangle)const;
		vector<pCara> provant_interseccio(const pCara &tr,const pCara &triangle_pv,const Punt &p)const;
		vector<pCara>  triangles_intersequen_paralelapipede(const pCara &tr,const pCara &triangle_sv, const Segment &ss)const;
		vector<pCara> triangles_intersequen_piramide(const pCara &tr,const pCara &triangle_pv, const Punt &pv)const;
		vector<pCara> triangles_intersequen_poliedre(const pCara &tr,const pCara &triangle_inici,const Poligon_Convex &poligon,const vector<Pla> &plans)const; 
		
		void setSegmentIntersecciona(Segment);
		void setTriangleIntersecciona(Triangle);
		void setCaresIntersecciona(vector<pCara>);
		
		Segment getSegmentIntersecciona();
		Triangle getTriangleIntersecciona();
		vector<pCara> getCaresIntersecciona();
		
		bool comprovarInterseccio();
		int tipusInterseccio();
		void settipusInterseccio(int t);
		
		void esborrarEntrades();
		void calculaEntrades(const Punt &pv,pCara &triangle_pv);
		void calculaEntrades(const Segment &sv,pCara &triangle_pv);
		void buscadorDEntrades(const Punt &pv);
		
		void calculGirsDesplegament();
		void inicialitzarVertexs();
		void comprovarVertexs();
		bool infoVertexNoCorrecte();
		bool vertexHiperbolic(const pCara &cara, const pVertex &vertex);
		list<Event*> propagarPseudoSource(PseudoSource ps); 
		list<Event*> propagarFinestra(Finestra f);
				
	protected:
		Terreny();
		
	private:
		Delaunay dt;
		static Terreny* instancia;
		bool _modificat;
		int _resolucio;
		SuperficieBezier sb;  
		CorbesNivell cn;
		
		VectorVertexs vertexs;
		
		int ample_graella;
		
		// comprovacio resultat visibilitat
		Segment segment_intersecciona;
		Triangle triangle_intersecciona;
		vector<pCara> cares_interseccio;
		int tipus;
};

#endif   //CLASSE_TERRENY_H
