#include "triangle.h"
#include "poligon_convex.h"
#include "segment.h"


/**************************************/
/*   Triangle::Triangle()             */
/*Constructor per defecte de la classe*/
/**************************************/

Triangle::Triangle()
{}

/***************************************/
/*   Triangle::Triangle(pCara fh)*/
/*Constructor per defecte de la classe */
/***************************************/

Triangle::Triangle(pCara fh)
{
	triangle = Triangle3(fh->vertex(0)->point(), fh->vertex(1)->point(), fh->vertex(2)->point());
}

/************************************/
/*   Triangle::Triangle(Triangle tr)*/
/*Constructor amb parametres        */
/************************************/

Triangle::Triangle(Triangle3 tr)
{
	triangle = tr;
}

/**************************************************/
/*   Triangle::Triangle(Punt p1, Punt p2, Punt p3)*/
/*Constructor de Triangle a partir de 3 punts     */
/**************************************************/

Triangle::Triangle(Punt p1, Punt p2, Punt p3)
{
	triangle = Triangle3(p1.getPunt(), p2.getPunt(), p3.getPunt());
}

/************************************/
/*   Triangle::~Triangle()          */
/*Destructor de la classe Triangle  */
/************************************/

Triangle::~Triangle() {}

/**************************************/
/*   Triangle2 Triangle::on_xy()      */
/*Metode que retorna el triangle en 2d*/
/**************************************/

void Triangle::escriure() const
{
	cout<<"("<<triangle.vertex(0).x()<<", "<<triangle.vertex(0).y()<<", "<<triangle.vertex(0).z()<<"), ( "<<triangle.vertex(1).x()<<", "<<triangle.vertex(1).y()<<", "<<triangle.vertex(1).z()<<"), ( "<<triangle.vertex(2).x()<<", "<<triangle.vertex(2).y()<<", "<<triangle.vertex(2).z()<<")"<<endl;
}

double Triangle::xmin() const
{
	
         return min(min(triangle.vertex(0).x(),triangle.vertex(1).x()),triangle.vertex(2).x());

}
double Triangle::xmax() const
{
	return max(max(triangle.vertex(0).x(),triangle.vertex(1).x()),triangle.vertex(2).x());
}

double Triangle::ymin() const
{
	return min(min(triangle.vertex(0).y(),triangle.vertex(1).y()),triangle.vertex(2).y());
}

double Triangle::ymax() const
{
	return max(max(triangle.vertex(0).y(),triangle.vertex(1).y()),triangle.vertex(2).y());
}

double Triangle::zmin() const
{
	return min(min(triangle.vertex(0).z(),triangle.vertex(1).z()),triangle.vertex(2).z());
}

double Triangle::zmax() const
{
	return max(max(triangle.vertex(0).z(),triangle.vertex(1).z()),triangle.vertex(2).z());
}

Triangle2 Triangle::on_xy()const
{
	Punt p(triangle.vertex(0));
	Punt q(triangle.vertex(1));
	Punt r(triangle.vertex(2));
	Triangle2 tr2(p.on_xy(),q.on_xy(),r.on_xy());
	return tr2;
}

/*************************************************/
/*   Punt3d Triangle::circumcenter()              */
/*Metode que calcula el circumcentre del triangle*/
/*************************************************/

Punt3d Triangle::circumcenter()const
{
	Punt3d c((triangle.vertex(1).x()+triangle.vertex(2).x()+triangle.vertex(3).x())/3.,(triangle.vertex(1).y()+triangle.vertex(2).y()+triangle.vertex(3).y())/3.,(triangle.vertex(1).z()+triangle.vertex(2).z()+triangle.vertex(3).z())/3.);
	return c;
}

/***************************************************************/
/*   bool triangle::p_on_triangle(const Punt3d &p)              */
/* Metode que indica si el punt es troba sobre el triangle o no*/
/***************************************************************/

bool Triangle::p_on_triangle(const Punt3d &p)const
{
	Punt p2(circumcenter());
	Punt p3(p);
	Punt * aux, * aux2;
	Punt2d pm(p2.on_xy());
	Punt2d pp(p3.on_xy());

	bool inside=true, sobre=false;
	
// 	qDebug("Punt a dins del triangle: X:%f Y:%f Z:%f", p.x(), p.y(), p.z());
	
	for(int i=0;i<3 && !sobre && inside;i++)
	{
		aux = new Punt(triangle.vertex(i));
		aux2 = new Punt (triangle.vertex(i+1));
		Line2D l(aux->on_xy(),aux2->on_xy());
		if(l.has_on_positive_side(pm) != l.has_on_positive_side(pp))
		{//qDebug("Punt fora del triangle!!!!!!!!!");
		inside=false;i=3;}
		if(fabs(l.a()*pp.x()+l.b()*pp.y()+l.c())<TOLERANCE){ //qDebug("Punt a sobre la linia!!!");
		sobre=true;}
		delete aux;
		delete aux2;
		
	}
	
	return (inside || sobre);
}

/*******************************************************************************************/
/*   bool tr_e_intersection(const Segment3D &e,vector<Punt3d> &p)                           */
/* Metode que retorna per parametres els punts d'interseccio entre el segment i el triangle*/
/*******************************************************************************************/

bool Triangle::tr_e_intersection(const Segment3D &e,VectorPunt3d &p) const
{	
	CGAL::Object result;
	Punt3d punt;
	Line line;
	result=CGAL::intersection(triangle.supporting_plane(),e.supporting_line());
	bool ret=false;
	Segment seg(e);

	if (CGAL::assign(punt,result))
	{
	      double d =seg.coord_p_on_e(punt); //,col); bool col;
	     if(p_on_triangle(punt) && d<=1 && d >=0){ p.push_back(punt); ret=true; }
	}
	else{
		if (CGAL::assign(line,result)) // e esta contingut al pla determinat pel triangle
		{
			qDebug("tr_e_intersection: intersequem un segment e que esta contingut al pla del triangle tr COMPROVAR SI FUNCIONA!!!");
			int cont=0;
			bool source=false, target=false;
			if(p_on_triangle(e.source())){ p.push_back(e.source()); source=true;}
			if (p_on_triangle(e.target())){ p.push_back(e.target()); target=true;}
			if(!(source&&target))  // cal trobar els dos punts de tall de l'edge amb la frontera del triangle si és que interseca amb el triangle. 
			{
				for(int j=0;j<3;j++)
				{
					Segment3D s(triangle.vertex(j),triangle.vertex(j+1)); // un costat del triangle
					VectorPunt3d intersecpoint;
					if(seg.coplanar_segment_3_intersection(s,intersecpoint))
					{
						cont++;
						if(intersecpoint.size()==1){ ret=true; p.push_back(intersecpoint.front());}
						else if(intersecpoint.size()==2){ ret=true; p.push_back(intersecpoint[0]); p.push_back(intersecpoint[1]);}
					}
				}
			}
			else qDebug("ERROR?: edge a la frontera del tetraedre: ");//<< e.source()<<" "<<e.target()<<endl;
		}
	}
	return ret;
}

/***********************************************************************************/
/*   bool Triangle::tr_e_intersection(const Segment3D &e)                          */
/* Metode que calcula si existeix o no interseccio entre el triangle i el segment e*/
/***********************************************************************************/

bool Triangle::tr_e_intersection(const Segment3D &e) const
{
	CGAL::Object result;
	Punt3d punt;
	Line line;
	result=CGAL::intersection(triangle.supporting_plane(),e.supporting_line());
	bool ret=false;
	Segment seg(e);

	if (CGAL::assign(punt,result))
	{
	     double d =seg.coord_p_on_e(punt);
	     /*qDebug("Es un punt!!!");
	     qDebug("Dins del triangle %i", p_on_triangle(punt));*/
	     if(p_on_triangle(punt) && d<=1 && d >=0)
	     {
	     	ret=true; 
	     }
	}
	else{
		if (CGAL::assign(line,result)) // e esta contingut al pla determinat pel triangle
		{
			/*cout<<"tr_e_intersection: intersequem un segment e que esta contingut al pla del triangle tr  COMPROVAR SI FUNCIONA (bool)"<<endl;*/
						
			if(p_on_triangle(e.source()))  ret=true;
			else if (p_on_triangle(e.target())) ret=true;
			else  // si e contingut a tr, ja contemplat, sino, segur que talla un dels seus costats, fem interseccions de cada costat de tr amb e
			{
				for(int j=0;j<3 && !ret;j++)
				{
					Segment3D s(triangle.vertex(j),triangle.vertex(j+1)); // un costat del triangle
					cout<<"interseccio segments coplanars"<<endl;
					ret=seg.coplanar_segment_3_intersection(s);	
				}
			}
		}
	}
	return ret;
}

/*********************************************************************************************************************/
/*   bool Triangle::triangle_intersection_e_obert(const Segment3D &e, VectorPunt3d &p)                                */
/* Metode que calcula si existeix o no interseccio entre el triangle i el segment e sense tenir en compte els extrems*/
/*********************************************************************************************************************/

bool Triangle::triangle_intersection_e_obert(const Segment3D &e, VectorPunt3d &p) const
{
	CGAL::Object result;
	result=CGAL::intersection(triangle.supporting_plane(),e.supporting_line());
	bool ret=false;

	Punt3d punt;
	Line line;
	Segment seg(e);
	
	/*qDebug("Triangle: Primer: X:%f Y:%f Z:%f",triangle.vertex(0).x(),triangle.vertex(0).y(),triangle.vertex(0).z());
	qDebug(" Segon: X:%f Y:%f Z:%f",triangle.vertex(1).x(),triangle.vertex(1).y(),triangle.vertex(1).z());
	qDebug("Tercer: X:%f Y:%f Z:%f",triangle.vertex(2).x(),triangle.vertex(2).y(),triangle.vertex(2).z());*/
			
	if (CGAL::assign(punt,result))
	{
	     double d = seg.coord_p_on_e(punt); //,col); bool col;
	     if(p_on_triangle(punt) && d<1 && d >0){ p.push_back(punt); ret=true; }
	     
	}
	else if (CGAL::assign(line,result)) // e esta contingut al pla determinat pel triangle
	{
		qDebug("Hi ha un segment en un pla que conté un costat del tetraedre");
		int cont=0;
		
		//cal trobar punts de tall del segment e que no siguin els extrems
		for(int j=0;j<3;j++)
		{
			Segment3D s(triangle.vertex(j),triangle.vertex(j+1)); // un costat del triangle
			VectorPunt3d intersecpoint;
			if(seg.coplanar_segment_3_intersection(s,intersecpoint))
			{
				cont++;
				if(intersecpoint.size()==1&&e.source()!=intersecpoint[0]&&e.target()!=intersecpoint[0]){ ret=true; p.push_back(intersecpoint.front());}
				else if(intersecpoint.size()==2)
				{
				 	ret=true;
					if(e.source()!=intersecpoint[0]&&e.target()!=intersecpoint[0]) p.push_back(intersecpoint[0]);
					if(e.source()!=intersecpoint[1]&&e.target()!=intersecpoint[1]) p.push_back(intersecpoint[1]); 
				}
				//else{ ret=false; cont++;}
			}
		}
	}
	else qDebug("ERROR?: edge a la frontera del tetraedre: ");//<< e.source()<<" "<<e.target()<<endl;

	return ret;

}

/*******************************************************************************************************************/
/*   bool Triangle::tr_l_intersection(const Line3 &l,vector<Punt3d3> &p)                                            */
/* Metode que calcula si existeix o no interseccio entre el triangle i la linia l retornant els punts d'interseccio*/
/*******************************************************************************************************************/
/*
bool Triangle::tr_l_intersection(const Line &l,VectorPunt3d &p) const
{
	CGAL::Object result;
	Punt3d punt;
	Line line;
	result=CGAL::intersection(triangle.supporting_plane(),l);
	bool ret=false;
	Linia linia(l);

	if (CGAL::assign(punt,result))
	{
		if(p_on_triangle(punt)){ p.push_back(punt); ret=true; }
	}
	else if (CGAL::assign(line,result)) // l esta contingut al pla determinat pel triangle
	{
		Linia linia(l);
		Line2D l2(linia.on_xy());
		for(int j=0;j<3;j++)
		{
			Segment2D s((Punt(triangle.vertex(j))).on_xy(),(Punt(triangle.vertex(j+1))).on_xy()); // un costat del triangle
			CGAL::Object result1=CGAL::intersection(s,l2);
			Punt2d p2;
			Segment2D s_aux;
			if(CGAL::assign(p2,result1)) p.push_back(linia.on_l(p2));
			else if(CGAL::assign(s_aux,result1))
			{
				p.push_back(linia.on_l(s_aux.source()));
				p.push_back(linia.on_l(s_aux.source()));
				j=3;
			}
		}
	}
	if(p.size()>0) return true;
	else return false;
}
*/
/***************************************************************************************************************/
/*   bool Triangle::interseccio_triangles(const Triangle3 &tr_b,vector<Punt3d3> &pt_inter)                      */
/*Metode que calcula si existeix interseccio entre dos triangles i retorna els punts                           */
/***************************************************************************************************************/

bool Triangle::tri_tri_intersection(const Triangle3 &tr_b, VectorPunt3d &pt_inter)const
{
	
	VectorPunt3d p_int;
	Triangle tr2(tr_b);
	unsigned int pt_size=pt_inter.size();
	
	if(tri_plane_intersection(tr_b.supporting_plane(),p_int)) // els dos plans no són coplanars, els 2 costats que surten de i tallen al pla
	{
		if(p_int.size()==2) // 2 punts intersecció tr_a amb pla conté tr_b
		{
			Segment3D seg_a(p_int[0], p_int[1]); //segment tr_a interseccio el pla que conte tr_b.
			Segment2D seg2_a;
			Punt2d pt2_a;
			//triangles no paralels a l'eix OZ calculem la interseccio projectant sobre el pla z=0
			CGAL::Object result;
			Segment seg(seg_a);
			result=CGAL::intersection(seg.on_xy_segment(),tr2.on_xy());
			if(CGAL::assign(seg2_a,result))
			{
				Linia linia(seg_a.supporting_line());
				pt_inter.push_back(linia.on_l(seg2_a.source()));
				pt_inter.push_back(linia.on_l(seg2_a.target()));
			}
			else if(CGAL::assign(pt2_a,result))
			{
				Linia linia(seg_a.supporting_line());
				pt_inter.push_back(linia.on_l(pt2_a));
			}
		}
		else if(p_int.size()==1)
		{
			if(tr2.p_on_triangle(p_int[0])){ pt_inter.push_back(p_int[0]);}
		}
		else if(p_int.size()==3) //Coplanars, calcular la interseccio de la projeccio a z=0.
		{
			qDebug("interseccio triangles 2d del CGAL diu que no acaba de funcionar, arreglar-ho");
			CGAL::Object result;
			Punt2d rp;
			Segment2D rs;
			Triangle2 rt;
			Pla pl(triangle.supporting_plane());
			result=CGAL::intersection(on_xy(),tr2.on_xy());
			if(CGAL::assign(rp,result)){ pt_inter.push_back(pl.on_plane_2d(rp));}
			else if(CGAL::assign(rs,result))
			{
				pt_inter.push_back(pl.on_plane_2d(rs.source()));
				pt_inter.push_back(pl.on_plane_2d(rs.target()));
			}
			else if(CGAL::assign(rt,result))
			{
				for(int j=0;j<3;j++) pt_inter.push_back(pl.on_plane_2d(rt.vertex(j)));
			}
		}
		else qDebug("ERROR (tri_tri_intersection() amb retorn) %d",p_int.size());
	}
	return  pt_size<pt_inter.size();
}

/***************************************************************************************************************/
/*   bool Triangle::interseccio_triangles(const Triangle3 &tr_b)                                               */
/*Metode que calcula si existeix interseccio entre dos triangles i no retorna els punts                        */
/***************************************************************************************************************/

bool Triangle::tri_tri_intersection(const Triangle3 &tr_b) const
{
	bool interseca=false;
	VectorPunt3d p_int;
	Triangle tr2(tr_b);
	
	p_int.clear();
	if(tri_plane_intersection(tr_b.supporting_plane(),p_int)) // els dos plans no són coplanars, els 2 costats que surten de i tallen al pla
	{
		if(p_int.size()==2) // 2 punts intersecció tr_a amb pla conté tr_b
		{
			Segment3D seg_a(p_int[0], p_int[1]); //segment tr_a interseccio el pla que conte tr_b.
			Segment2D seg2_a;
			Segment seg(seg_a);
			Punt2d pt2_a;
			//triangles no paralels a l'eix OZ calculem la interseccio projectant sobre el pla z=0
			CGAL::Object result;
			result=CGAL::intersection(seg.on_xy_segment(),tr2.on_xy());
			if(CGAL::assign(seg2_a,result)/*||CGAL::assign(pt2_a,result)*/) interseca=true;
		}
		/*else if(p_int.size()==1)
		{
			interseca=tr2.p_on_triangle(p_int[0]);
		}*/
		else if(p_int.size()==3) //Coplanars, calcular la interseccio de la projeccio a z=0.
		{
			/*qDebug("interseccio triangles 2d del CGAL diu que no acaba de funcionar, arreglar-ho");
			CGAL::Object result;
			Punt2d rp;
			Segment2D rs;
			Triangle2 rt;
			result=CGAL::intersection(on_xy(),tr2.on_xy());
			if(CGAL::assign(rp,result)|| CGAL::assign(rs,result) || CGAL::assign(rt,result))
			{
				interseca=true;
			}*/
			
			Poligon_Convex poligon(triangle);
			int i=0;
			while(i<3 && !interseca)
			{
				Segment3D seg_a(tr_b.vertex(i), tr_b.vertex(i+1)); 
				Segment seg(seg_a);
				if(poligon.intersecciona2D(seg,false))
				{
					interseca=true;
				}
				i++;
			}
		}
		else if (p_int.size()!=1)
		qDebug("ERROR (tri_tri_intersection()) ");
	}
	return  interseca;
}

/******************************************************************************************/
/*   bool Triangle::tr_intersection_T_edges(const Tetrahedron &Tvs, vector<Punt3d3> &p_trT)*/
/*Metode que calcula si existeix interseccio amb les arestes del tetraedre i el triangle  */
/******************************************************************************************/

/* bool Triangle::tr_intersection_T_edges(const Tetrahedron &Tvs, VectorPunt3d &p_trT) const
{
	unsigned int size=p_trT.size();
	for(int i=0;i<4;i++)
	{
		for(int j=i+1;j<4;j++)
		{
			Segment3D s(Tvs.vertex(i),Tvs.vertex(j));
			tr_e_intersection(s,p_trT);
		}
	}
	return p_trT.size()>size;
}*/

/**************************************************************/
/*   bool Triangle::tr_intersects_T(const Tetrahedron &Tvs)   */
/*Metode que calcula si existeix interseccio entre el triangle*/
/**************************************************************/
bool Triangle::interseccio_tetraedre(vector<Pla> pla,bool cV) const  // CERT SI MOLT PROBABLE QUE INTERSEQUI, NO ES 100% SEGUR.
{
// punt dins si pla[i].avaluar(punt)<0  si es =0 esta a sobre=> ha de retornar true nomes si cV es true. Amb tolerancia <0=><-TOLERANCE /==0 => <TOLERANCE
	unsigned int i=0;
	int  j=0;
	double llindar=-1e-5; //<0
	if(cV)  llindar=1e-5; //==0
		
	while(i<pla.size())
	{
		while(i<pla.size() && pla[i].avaluarPunt(Punt(triangle.vertex(0)))<llindar)
			i++;
		if(i==pla.size())
			return true;
		j=1;
		
		while(j<3 && pla[i].avaluarPunt(Punt(triangle.vertex(j)))>-llindar)
			j++;
		if(j==3)
			return false;
		i++;
	}
	return true;

}

bool Triangle::tr_intersects_T(const Tetrahedron &Tvs,bool cV) const
{
// 	cout<<"tr_intersects_T"<<endl;
	bool interseca=false;
	
	for(int i=0;i<3&&!interseca;i++)
		if(Tvs.has_on_bounded_side(triangle.vertex(i)))
			interseca=(cV||!Tvs.has_on_boundary(triangle.vertex(i)));
		
	int l;
	for(int k=0;k<4 && ! interseca ;k++) // hi entrerà si tots els vèrtex de tr son exteriors a Tvs, i mentres no hi hagi cap pt interseccio.
	{
		Triangle  tr_a(Triangle3(Tvs.vertex(k),Tvs.vertex(k+1),Tvs.vertex(k+2)));
		interseca=interseccio(tr_a,true,cV);
		l=k;
	}

	return interseca;
}

/************************************************************************************************************/
/*   bool Triangle::interseccio_tetraedre(const Tetrahedron &Tvs, vector<Punt3d3> &p)                        */
/*Metode que calcula si existeix interseccio entre un triangle i un tetraedre                               */
/* retrona cert si tr interseca Tvs, a p hi ha els punts d'intersecció del triangle amb el tetraedre        */
/* PROBLEMA: FALTEN PUNTS D'INTERSECCIO, ELS FORMATS PER LES ARESTES DEL TETRAEDRE si tr te vertex interiors*/
/* a Tvs, SON REPES si tr tots vertex exteriors,                                                            */
/************************************************************************************************************/

bool Triangle::tr_intersects_T(const Tetrahedron &Tvs, VectorPunt3d &p) const
{
		cout<<"tr_intersects_T amb punt........... ...... ..... ..... ..... ..... ....."<<endl;
		unsigned int mida_p=p.size();
	
	// separem els vertes de tr segons siguin interiors/no_interiors a Tvs
	VectorInt no_interior,interior;
	for(int i=0;i<3;i++)
	{
		if(Tvs.has_on_boundary(triangle.vertex(i)) || Tvs.has_on_bounded_side(triangle.vertex(i)))
		{
			p.push_back(triangle.vertex(i)); // els interiors ja estan guardats. 
			interior.push_back(i);
		}
		else no_interior.push_back(i);
	}

	// tractem el cas en què hi hagi punts interiors i d'altres d'exteriors.
	if(no_interior.size()>0&&interior.size()>0)
	{
		VectorInt cares_a,cares_b,cares_ab;
		VectorPunt3d p_a,p_ab,p_b;
		Segment3D s1, s12, s2;
		if(interior.size()==1)
		{
			s1 =Segment3D(triangle.vertex(interior[0]),triangle.vertex(no_interior[0]));
			s12 = Segment3D(triangle.vertex(no_interior[0]),triangle.vertex(no_interior[1]));
			s2 = Segment3D(triangle.vertex(interior[0]), triangle.vertex(no_interior[1]));
		}
		else if(interior.size()==2)
		{
			s1 =Segment3D(triangle.vertex(interior[1]),triangle.vertex(no_interior[0]));
			s12 = Segment3D(triangle.vertex(interior[0]),triangle.vertex(interior[1]));
			s2 = Segment3D(triangle.vertex(no_interior[0]), triangle.vertex(interior[0]));
		}
		Segment s_a(s1),s_ab(s12),s_b(s2); // guardem a s_a,s_b les arestes que segur que intersequen, uneixen un vertex dins i un de fora, s_ab pot o no intersecar.
		unsigned int pa=p.size();
		s_a.e_intersects_Ta(Tvs, p, cares_a);
		if(p.size()==pa) qDebug("error lll");
		s_ab.e_intersects_Ta(Tvs, p, cares_ab);
		pa=p.size();
		s_b.e_intersects_Ta(Tvs, p, cares_b);
		if(p.size()==pa) qDebug("error 222");
	}
	else if(no_interior.size()==3)// cas en què tots siguin exteriors
	{
		for(int k=0;k<4;k++)
		{
			Triangle3 tr_a(Tvs.vertex(k),Tvs.vertex(k+1),Tvs.vertex(k+2));
//			bool va=tri_tri_intersection(tr_a,tr,p);  // p té pts repes (si una aresta de Tvs intersea tr, el pt interseccio surt 2 cops 1 x cada cara q la conté).
			tri_tri_intersection(tr_a,p);  // p té pts repes (si una aresta de Tvs intersea tr, el pt interseccio surt 2 cops 1 x cada cara q la conté).
		}
	}
	else if(interior.size()==3)
	{
		qDebug("Triangle interior");
	}
	else qDebug("NO POT SER!!!"); 
		
	
	qDebug(" TOTAL: %d punts interseccio trobats amb T_vs",p.size()-mida_p);
	return p.size()>mida_p;
}

/************************************************************************************************/
/*   bool Triangle::tri_plane_intersection(const Plane &pla,VectorPunt3d &p)                     */
/*Metode que retrona cert si tr_a interseca al pla, a p[i] hi ha els pts interseccio del costat */
/************************************************************************************************/

bool Triangle::tri_plane_intersection(const Plane &pla,VectorPunt3d &p) const//MIRAT
{
	unsigned int p_size=p.size();
	for(int i=0;i<3;i++)
	{
		Segment3D s(triangle.vertex(i),triangle.vertex(i+1));
		CGAL::Object result;
		Punt3d ps;
		Segment3D seg;
		result=CGAL::intersection(pla,s);	
		if(CGAL::assign(ps,result))
			if(ps!=s.source()) p.push_back(ps); 
		else if(CGAL::assign(seg,result))
		{
			/*if(seg.source()!=s.source()){ p.push_back(seg.source());}
			else if(seg.target()!=s.source()){ p.push_back(seg.target());  }*/
			p.push_back(s.target());
		}
	}
	return p.size()>p_size;
}

/**********************************************************************************************************************/
/*   bool Triangle::tri_plane_intersection(const Plane &pla,VectorPunt3d (&p)[3])                                      */
/*Metode que retrona cert si tr_a interseca pla, p[i] tots pts interseccio costat de tr_a amb vertex i,i+1 amb el pla.*/
/**********************************************************************************************************************/

bool Triangle::tri_plane_intersection(const Plane &pla,VectorPunt3d (&p)[3])const
{
	bool interseca=false;
	
	for(int i=0;i<3;i++)
	{
		Segment3D s(triangle.vertex(i),triangle.vertex(i+1));
		CGAL::Object result;
		Punt3d ps;
		Segment3D seg;
		
		result=CGAL::intersection(pla,s);	
		if(CGAL::assign(ps,result))
		{
			p[i].push_back(ps); 
			interseca=true;
		}
		else if(CGAL::assign(seg,result))
		{
			p[i].push_back(seg.source());
			p[i].push_back(seg.target());
			interseca=true;
		}
		
	}
	return interseca;
}

/***********************************************************************************************************************/
/*   bool Triangle::tri_plane_intersection(const Triangle3 &tr_a,const Plane &pla,int &j,VectorPunt3d &p)               */
/*Metode que retrona calcula la interseccio entre el triangle actual i el pla passat per parametres retornant els punts*/
/***********************************************************************************************************************/

bool Triangle::tri_plane_intersection(const Triangle3 &tr_a,const Plane &pla,int &j,VectorPunt3d &p) const
{
	j=10;
	unsigned int p_size=p.size();
	bool abans=false,ara;
	for(int i=0;i<3;i++)
	{
		ara=false;
		Segment3D s(tr_a.vertex(i),tr_a.vertex(i+1));
		CGAL::Object result;
		Punt3d ps;
		Segment3D seg;
		result=CGAL::intersection(pla,s);	
		if(CGAL::assign(ps,result))
			if(ps!=s.source()){ p.push_back(ps);  ara=true;}
		else if(CGAL::assign(seg,result))
		{
			if(seg.source()!=s.source()){ p.push_back(seg.source()); ara=true;}
			else if(seg.target()!=s.source()){ p.push_back(seg.target());   ara=true;}
		}
		if(abans&&ara) j=i;
		abans=ara;
	}
	if(j==10&&(p_size-p.size())>1) j=0;
	else if(j==10&&(p_size-p.size())==1) 
		for(int i=0;i<3;i++)
			if(p[p.size()-1]==tr_a.vertex(i)) {j=i; i=4;}
			
	return j<10;
}

/***********************************************/
/*   Triangle3 Triangle::getTriangle_cgal()    */
/*Metode que retrona el triangle de CGAL actual*/
/***********************************************/

const Triangle3 & Triangle::getTriangle_cgal()const
{
	return triangle;
}

/****************************************************************************************************************************/
/*   bool Triangle::Triangle_Conte_ss(const Segment &ss,const Segment &sv,vector<double> &vis)                               */
/*Estudia el triangle que conte el segment ss, a vis hi retorna el segment de v entre [0,1] q es visible des del triangle tr */
/****************************************************************************************************************************/

bool Triangle::TriangleTapaSegmentVisio(const Segment &sv,VectorDoubles &vis) const
{
	// intersequem el pla q conte tr amb la linia definida x MG::sv, si pt interseccio esta en el sement MG::sv, retornem [coordenada pt,1]
	vis.clear();
	
	bool ret=true;
	Vector3D vs_origen, vs_desti,origens, destins,origendesti, destiorigen;
	Vector n=triangle.supporting_plane().orthogonal_vector ();
	if(triangle.supporting_plane().has_on_negative_side(Punt3d(triangle.vertex(0).x(),triangle.vertex(0).y(),triangle.vertex(0).z()-1.)))
		n=(-1.)*n;
	Vector vinici(sv.getSegment_cgal().source(),circumcenter());
	Vector vfinal(sv.getSegment_cgal().target(),circumcenter());
	
	double nvi=vinici*n;
	double nvf=vfinal*n;
	if(nvi<0&&nvf<0)
	{
		vis.push_back(1);
		vis.push_back(1);
		ret=false;
	}
	else if(nvi<0 || nvf<0)
	{
		double t;
		Punt3d pt;
		Segment3D seg;
		CGAL::Object result;
		result=CGAL::intersection(triangle.supporting_plane(),sv.getSegment_cgal().supporting_line());
		if (CGAL::assign(pt, result))
		{
			 t=sv.coord_p_on_e(pt);
		}
		else
		{
			qDebug(" ERROR en Triangle::Triangle_Conte_ss  (1)");
		}
		if(t>1|| t<0)
			qDebug("ERROR en Triangle::Triangle_Conte_ss (2)");
		
		if(nvi<0)
		{
			vis.push_back(t);
			vis.push_back(1);
		}
		else 
		{
			vis.push_back(0);
			vis.push_back(t);
		}
	}
	else
	{
		vis.push_back(0);
		vis.push_back(1);
	}
	return ret;
 }
 
 bool Triangle::TriangleTapaPuntVisio(const Punt &pv) const
{
	Vector n=triangle.supporting_plane().orthogonal_vector ();
	if(triangle.supporting_plane().has_on_negative_side(Punt3d(triangle.vertex(0).x(),triangle.vertex(0).y(),triangle.vertex(0).z()-1.)))
		n=(-1.)*n;
	Vector v(pv.getPunt(),circumcenter());
	
	return v*n>0;
 }
 
/***************************************************************************************************/
/*   void Triangle::Skew(const Segment &ss, vector<Regio> &Regions, ofstream &SPTriangles_f)                       */
/*   Calcula les hiperboles dels costats d'un triangle (trobarSigne=false per defect)                                             */
/***************************************************************************************************/

VectorHiperboles Triangle::skewProjection(const Segment  &ss,const Segment &sv,const Pla&pla_v, const Pla &pla_s, const double &valor_s, bool trobarSigne)const
{
	Punt2d skp;
	return skewProjection(ss,sv,pla_v,pla_s,valor_s,skp,trobarSigne);
}
/***************************************************************************************************/
/*   void Triangle::Skew(const Segment &ss, vector<Regio> &Regions, ofstream &SPTriangles_f)                       */
/*   Calcula les hiperboles dels costats d'un triangle i retorna a skp l'skew porjection d'un punt de dins la regio */
/*   i=1 si triangle es coplanar amb ss, i=2 si coplanar amb sv.   */
/***************************************************************************************************/

VectorHiperboles Triangle::skewCasCoplanar(int i, const Segment  &ss, const Segment  &sv, const Pla &pla_v,  bool trobarSigne)const //1 sola recta, 1! segment
{
	VectorHiperboles h;
	vector<Segment> seg;
	seg.push_back(ss);
	seg.push_back(sv);
	int i_c=i-1;  // segment coplanar amb el triangle 
	int i_nc=(i_c+1)%seg.size();   // segment no coplanar amb el triangle.
// 	cout<<"skewCasCoplanar "<<i<<endl;
// 	escriure();
	
	Pla pla_t(triangle.supporting_plane());
	Linia l(seg[i_nc].getSegment_cgal().supporting_line());
	Punt3d q;
	bool linia;
	double coord=0,coord_aux=0; //coordenada interseccio pla_t amb l sobre l.
	if(pla_t.interseccio(l,q,coord,linia))
	{
		vector<double> v_d;
		for(int k=0;k<3;k++)
		{
			Linia l_aux(Line(q,Punt3d(triangle.vertex(k).x(),triangle.vertex(k).y(),triangle.vertex(k).z())));
			if(pla_v.interseccio(l_aux, coord_aux))
				v_d.push_back(coord_aux);
		}
		double min=v_d[0];
		double max=v_d[0];
		for(unsigned int k=1;k<v_d.size();k++)
		{
			if(v_d[k]<min) min=v_d[k];
			else if(v_d[k]>max) max=v_d[k];
		}
		if(i==1) h.push_back(Hiperbola(0,1,0,-coord,Punt2d(coord,min), Punt2d(coord,max)));
		else  h.push_back(Hiperbola(0,0,1,-coord,Punt2d(min,coord), Punt2d(max,coord)));
		
		if(seg[i_nc].getOrigen().z()<q.z())
		{
			if(i==1) h.push_back(Hiperbola(0,-1,0,coord,Punt2d(0,min), Punt2d(0,max)));
			else  h.push_back(Hiperbola(0,0,-1,coord,Punt2d(min,0), Punt2d(max,0)));
		}
		else
		{
			if(i==1) h.push_back(Hiperbola(0,-1,0,coord,Punt2d(1,min), Punt2d(1,max)));
			else  h.push_back(Hiperbola(0,0,-1,coord,Punt2d(min,1), Punt2d(max,1)));
		}
	
// 		if(trobarSigne)
// 		{
// 			h.push_back(h[0]);
// 			h[1].canviarSigne();
// 		}
	}
// 	escriure();
// 	for(unsigned int i=0;i<h.size();i++)
// 		h[i].escriure();

	return h;
}


VectorHiperboles Triangle::skewProjection(const Segment  &ss, const Segment  &sv, const Pla& pla_v,const Pla &pla_s,double valor_s, Punt2d  &skp, bool trobarSigne)const
{

	VectorHiperboles h;
	h.clear();

	Pla pla_t(triangle.supporting_plane());
// 	vector<int> h_deg; //hiperboles degenerades

	bool copv=pla_t.puntPertanyPla(sv.getOrigen())&&pla_t.puntPertanyPla(sv.getDesti());
	bool cops=pla_t.puntPertanyPla(ss.getOrigen())&&pla_t.puntPertanyPla(ss.getDesti());
	
// 	if(fabs(t_vs)<TOLERANCE || fabs(t_vv)<TOLERANCE) // triangle coplanar amb el segment ss o amb sv respectivament.
	if(copv||cops)
	{
// 		if(fabs(t_vs)<TOLERANCE) return skewCasCoplanar(1,ss,sv,pla_s,trobarSigne);
// 		else return skewCasCoplanar(2,ss,sv,pla_v,trobarSigne);
// ss.escriure();
// sv.escriure();
// escriure();
// 		cout<<"coplanaritat: "<<cops<<" i  "<<copv<<endl;
		if(cops) return skewCasCoplanar(1,ss,sv,pla_s,trobarSigne);
		else return skewCasCoplanar(2,ss,sv,pla_v,trobarSigne);
		
	}
	else
	{

		Triangle3 tri;
		int i=0;
		while(!ss.punt_sobre(triangle.vertex(i))&&i<3) i++;
		if(i<3)
		{
			Segment seg(Punt(triangle.vertex(i)),Punt(circumcenter()));
			tri=Triangle3(triangle.vertex((i+1)%3),triangle.vertex((i+2)%3),seg.e_t(1e-6));
		}
		else
			tri=triangle;
		
		// vector normal al triangle
		Vector3D nt;
		nt=nt.vector_normal(Punt(tri.vertex(0)),Punt(tri.vertex(1)),Punt(tri.vertex(2))); 
		
		if(trobarSigne)
		{
			Punt p1=puntEntrePlans(ss,sv);
			skp=p1.skew_point(ss.getSegment_cgal(),sv.getSegment_cgal());
		}
		
		vector<int> is,iv;
// 		int n_rectesVerticals=0;
// 		int n_rectesHoritzontals=0;
		for(int i=0;i<3;i++)   // calculem la skew de cada costat del triangle
		{
			Segment s(tri.vertex(i),tri.vertex((i+1)%3));
			
			if(fabs(pla_s.avaluarPunt(s.getOrigen())-valor_s)<TOLERANCE &&fabs(pla_s.avaluarPunt(s.getDesti())-valor_s)<TOLERANCE) //segment s contingut al pla_s -> recta -x+0=0
			{
				cout<<"PUNT PLA s"<<endl;
				Vector3D u;
				u.converteix(Punt(tri.vertex(i)),ss.getOrigen());
				Vector3D sxss=s.getVector().producte_vectorial(ss.getVector());
				
				if(sxss.calcula_modul2()>TOLERANCE) // ss al pla_s però no paral.lel ni coincident amb s.
				{
					is.push_back(i);
					h.push_back(Hiperbola(0.,0.,-1.,0.,Punt2d(0.,0.),Punt2d(0.,1.)));
// 					cout<<"segment al pla // a v q conte s"<<endl;
// 					h_deg.push_back(i);
				}
				else // s  paral.lel o colineal amb ss
				{
					if(u.producte_vectorial(ss.getVector()).calcula_modul2()<TOLERANCE) // colineal amb ss
					{
						double us=ss.coord_p_on_e(s.source());
						double ut=ss.coord_p_on_e(s.target());
						double t;
						if(pla_t.interseccio(Linia(sv.getSegment_cgal().supporting_line()), t))
						{
							h.push_back(Hiperbola(0.,-1.,0.,t,Punt2d(t,us),Punt2d(t,ut)));
// 							h_deg.push_back(i);
						}
						else
							cout<<"NO INterseccio !!  NO CONSIDERAT"<<endl;
						// h.push_back(Hiperbola(0.,0.,-1.,us,Punt2d(0.,us),Punt2d(1.,us)));
						//  h.push_back(Hiperbola(0.,0.,-1.,ut,Punt2d(0.,ut),Punt2d(1.,ut)));
						cout<<"segment colineal amb s"<<endl;
					}
				}
			}
			else if(pla_v.puntPertanyPla(s.getOrigen()) && pla_v.puntPertanyPla(s.getDesti()))  //segment s contingut al pla_v -> recta -y+0=0
			{
				Vector3D u;
				u.converteix(Punt(tri.vertex(i)),sv.getOrigen());
				Vector3D sxsv=s.getVector().producte_vectorial(sv.getVector());
				if(sxsv.calcula_modul2()>TOLERANCE) // sv al pla_v pero no paral.lel ni coincident amb s.
				{
					is.push_back(i);
					h.push_back(Hiperbola(0.,-1.,0.,0.,Punt2d(0.,0.),Punt2d(1.,0.)));
// 					h_deg.push_back(i);
					cout<<"segment al pla // a s q conte v"<<endl;
				}
				else // s paral.lel o colineal amb sv
				{
					if(u.producte_vectorial(sv.getVector()).calcula_modul2()<TOLERANCE) // colineal amb sv
					{
						double ts=sv.coord_p_on_e(s.source());
						double tt=sv.coord_p_on_e(s.target());
						double u;
						if(pla_t.interseccio(Linia(ss.getSegment_cgal().supporting_line()), u))
						{
							h.push_back(Hiperbola(0.,0.,-1.,u,Punt2d(ts,u),Punt2d(tt,u)));
// 							h_deg.push_back(i);
						}
						else
							cout<<"NO INterseccio !!  NO CONSIDERAT"<<endl;
						// h.push_back(Hiperbola(0.,0.,-1.,us,Punt2d(0.,us),Punt2d(1.,us)));
						//  h.push_back(Hiperbola(0.,0.,-1.,ut,Punt2d(0.,ut),Punt2d(1.,ut)));
						cout<<"segment colineal amb sv"<<endl;
					}
				}
			}
			else
			{
				Hiperbola h_aux(s.getSegment_cgal(),ss,sv);
// 				if(h_aux.degenerada()) h_deg.push_back(i);
				if(trobarSigne&&h_aux.avaluarPunt(skp)>0) h_aux.canviarSigne();
					h.push_back(h_aux);
			}
		}
// 		if(h_deg.size()==1)
// 		{
// // 			if(h[(h_deg[0]+1)%3].source().x()==h[h_deg[0]].source().x() || h[(h_deg[0]+1)%3].source().y()==h[h_deg[0]].source().y())
// // 			 	canviar_source(h[h_deg[0]].source());
// // 			else
// // 				 canviar_source(h[h_deg[0]].target());
// 			
// 			h[(h_deg[0]+1)%3].canviar_source(h[h_deg[0]].target());
// 			h[(h_deg[0]+2)%3].canviar_target(h[h_deg[0]].source());
// 		}
// 		
// 		if(is.size()>0 || iv.size()>0)
// 			cout<<"CAL RETALLAR BE!!!!!!!!!!!!!!!! Tenim triangles amb costats sobre els plans de ss o sv i no estan ni a s  ni a v. (Triangle::skewProjection)"<<endl;  // tenim costats del triangles continguts en els plans de ss o sv i no a ss o sv
		
// 		escriure();
// 		for(unsigned int i=0;i<h.size();i++)
// 			h[i].escriure();
			
		return h;
	}
}

void Triangle::obtenirRegionsDiscretitzar(const Segment & ss,const Segment &sv, const Pla &pla_v, const Pla &pla_s, const double &valor_s,VectorRegions&regions)const
{
 	if(!triangle.is_degenerate())
	{
		Punt2d skp;
		vector<Punt2d> p;
// 		unsigned int k0=regions.size();
		VectorHiperboles h=skewProjection(ss,sv,pla_v,pla_s,valor_s,skp,false/*,per_discretitzar*/);
		// source_<target (ordre lexicografic)
		
		if(h.size()<3)
			regions.push_back(h);
		else
		{
			int i=0;
			int n=(int)h.size();
			while(i<n && !h[i].interseccio(h[(i+1)%n],p,false,false,false)) i++;
			if(i==h.size())
				regions.push_back(Regio(h));
			else  // intersequen les hiperboles i, i+1. SEPARAR CAP i CUA i tenen els targets en comu.
			{
				if(Punt(h[i].source())==Punt(h[(i+1)%n].source()) || Punt(h[i].source())==Punt(h[(i+1)%n].target()))
				{
					vector<Hiperbola> h1;
					h1.push_back(Hiperbola(h[i].ctu(),h[i].ct(),h[i].cu(),h[i].cc(),h[i].source(),p[0]));
					h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[i].source(),p[0]));
					regions.push_back(Regio(h1)); //CAP
					h1.clear();
					h1.push_back(h[(i+2)%n]);//CUA
					h1.push_back(Hiperbola(h[i].ctu(),h[i].ct(),h[i].cu(),h[i].cc(),p[0],h[i].target()));
					if(Punt(h[(i+1)%n].source())==Punt(h[i].source()))
						h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[(i+1)%n].target(),p[0]));
					else
						h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[(i+1)%n].source(),p[0]));
					regions.push_back(Regio(h1));
				}
				else
				{
					vector<Hiperbola> h1;
					h1.push_back(Hiperbola(h[i].ctu(),h[i].ct(),h[i].cu(),h[i].cc(),h[i].target(),p[0]));
					h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[i].target(),p[0]));
					regions.push_back(Regio(h1)); //CAP
					h1.clear();
					h1.push_back(h[(i+2)%n]);//CUA
					h1.push_back(Hiperbola(h[i].ctu(),h[i].ct(),h[i].cu(),h[i].cc(),p[0],h[i].source()));
					if(Punt(h[(i+1)%n].source())==Punt(h[i].target()))
						h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[(i+1)%n].target(),p[0]));
					else
						h1.push_back(Hiperbola(h[(i+1)%n].ctu(),h[(i+1)%n].ct(),h[(i+1)%n].cu(),h[(i+1)%n].cc(),h[(i+1)%n].source(),p[0]));
					regions.push_back(Regio(h1));
				}
			}
		}
	}
}


void Triangle::obtenirRegions(const Segment & ss,const Segment &sv, const Pla &pla_v, const Pla &pla_s, const double &valor_s,VectorRegions &regions, bool trobarSigne,const bool & per_discretitzar) const
{
	/*if(per_discretitzar)
		obtenirRegionsDiscretitzar(ss,sv, pla_v, pla_s, valor_s,regions);
	else */if(!triangle.is_degenerate())
	{
// 		escriure();
		Punt2d skp;
// 		unsigned int k0=regions.size();
		VectorHiperboles h=skewProjection(ss,sv,pla_v,pla_s,valor_s,skp,false/*,per_discretitzar*/);
// 		for(unsigned int i=0;i<h.size();i++)
// 			h[i].escriure();
		vector<Punt2d> p;
		unsigned int i=0;
// 	if(false)
		if(h.size()==3)
		{
			int RangMax=h[0].hiperbolaRangMaxim(h[1],h[2]);  // hip de rang maxim
			while(i<3 &&h[i]!=h[(i+1)%3]) i++;	// mirem si hi ha hip repetides
			if(i<3) //h[i]==h[(i+1)%3] 
			{
				if(h[(i+1)%3]!=h[(i+2)%3])
				{
					cout<<"dues hip repetides en una regio"<<endl;
					int j=4;  // j sera la curta igual q la llarga (RangMax segur q es una de les dues iguales)
						
					if(RangMax ==(int) (i+1)%3 ) j=i;
					else if(RangMax== (int) i)  j=(i+1)%3;
					
					 // com q j!=4
					Punt2d pt;
					if(h[j].source()==h[(i+2)%3].source()||h[j].target()==h[(i+2)%3].source()) pt=h[(i+2)%3].source();
					else pt=h[(i+2)%3].target();
					regions.push_back(h[RangMax].regio(h[(i+2)%3],pt,true,trobarSigne));  // creem una regio q equival al cap anb pt interseccio pt.
				}
				else  //totes 3 iguales, nomes hem de pintar la gran.
				{
					cout<<"3 hiperboles repetides en una regio"<<endl;
					regions.push_back(Regio(h[RangMax]));
				}
			}
			else   // No repetides, mirar com son i guardar les regions corresponents.
			{
				int numero_degenerades=(int)h[0].degenerada()+(int)h[1].degenerada()+(int)h[2].degenerada();
				if(numero_degenerades==0)
				{
 					i=0;
 					while(i<3 && !h[i].interseccio(h[(i+1)%3],p,false,false,false)) i++;  //QuadratUnitat, extrems i rangGran=false
					
					if(i==h.size())
					{
// 						cout<<"NI, "; 
						regions.push_back(h[RangMax].regio(h[(RangMax+1)%3],trobarSigne));
						regions.push_back(h[RangMax].regio(h[(RangMax+2)%3],trobarSigne));
					}
					else     // h[i] i h[(i+1)%3] interseccionen en el quadrat cal crear dues regions.
					{
// 						cout<<"I, ";
						if(p.size()>1){ cout<<"Triangle::regions +d'un pt intersccio"<<endl;}
						else if(p.size()!=1){ cout<<"Triangle::regions  intersecciona pero no pts a p"<<endl;}
						else
						{
							Punt2d pt=p[0];
							if((int)i==RangMax)
							{
								regions.push_back(h[i].regio(h[(i+1)%3],pt,true,trobarSigne));  // La regio cap del peix, (i o i+1 es el rangMax)
								regions.push_back(h[i].regio(h[(i+1)%3],pt,false,trobarSigne));  // La regio  cua del peix, (i o i+1 es el rangMax)
								regions.push_back(h[i].regio(h[(RangMax+2)%3],trobarSigne));   // La part de la regio de 3 q falta
							}
							else
							{
								regions.push_back(h[(i+1)%3].regio(h[i],pt,true,trobarSigne));  // La regio cap del peix, (i o i+1 es el rangMax)
								regions.push_back(h[(i+1)%3].regio(h[i],pt,false,trobarSigne));  // La regio  cua del peix, (i o i+1 es el rangMax)
								regions.push_back(h[(i+1)%3].regio(h[(RangMax+1)%3],trobarSigne));   // La part de la regio de 3 q falta
							 }
							p.clear();
						}
					}
				}
				else if (numero_degenerades==1)
				{
					i=0;
 					while(i<3 && !h[i].interseccio(h[(i+1)%3],p,false,false,false)) i++;  //QuadratUnitat, extrems i rangGran=false
					
					if(i==h.size())
					{
// 						cout<<"NI, "; 
						char c;
						if(!h[(RangMax+1)%3].degenerada(c)||c!='v') regions.push_back(h[RangMax].regio(h[(RangMax+1)%3],trobarSigne));
						if(!h[(RangMax+2)%3].degenerada(c)||c!='v') regions.push_back(h[RangMax].regio(h[(RangMax+2)%3],trobarSigne));
					}
					else     // h[i] i h[(i+1)%3] interseccionen en el quadrat cal crear dues regions.
					{
// 						cout<<"I, ";
						if(p.size()>1){ cout<<"Triangle::regions +d'un pt intersccio"<<endl;}
						else if(p.size()!=1){ cout<<"Triangle::regions  intersecciona pero no pts a p"<<endl;}
						else
						{
							Punt2d pt=p[0];
							if((int)i==RangMax)
							{
								regions.push_back(h[i].regio(h[(i+1)%3],pt,true,trobarSigne));  // La regio cap del peix, (i o i+1 es el rangMax)
								regions.push_back(h[i].regio(h[(i+1)%3],pt,false,trobarSigne));  // La regio  cua del peix, (i o i+1 es el rangMax)
								if(!h[(RangMax+2)%3].degenerada()) 
									regions.push_back(h[i].regio(h[(RangMax+2)%3],trobarSigne));   // La part de la regio de 3 q falta
							}
							else
							{
								regions.push_back(h[(i+1)%3].regio(h[i],pt,true,trobarSigne));  // La regio cap del peix, (i o i+1 es el rangMax)
								regions.push_back(h[(i+1)%3].regio(h[i],pt,false,trobarSigne));  // La regio  cua del peix, (i o i+1 es el rangMax)
								if(!h[(RangMax+1)%3].degenerada())
									regions.push_back(h[(i+1)%3].regio(h[(RangMax+1)%3],trobarSigne));   // La part de la regio de 3 q falta
							 }
							p.clear();
						}
					}
// 					for(unsigned int kj=0;kj<h.size();kj++)
// 						h[kj].escriure();
				}
				else/* if((int)h[0].degenerada()+(int)h[1].degenerada()+(int)h[2].degenerada()==2)*/
				{
// 					cout<<"una regio degenerada"<<endl;
					int n_deg=0;
					char c;
					while(h[n_deg].degenerada(c) && n_deg<3) n_deg++;
// 					cout<<"regio no degenerada "<<n_deg;
					if(n_deg<3) 
					{
						Regio r_aux=h[n_deg].regio(h[(n_deg+1)%3],h[(n_deg+2)%3]);
						if(r_aux.numero_hiperboles()==3)
							regions.push_back(r_aux);
						else if(r_aux.numero_hiperboles()==6)
						{
							regions.push_back(Regio(r_aux.getHiperbola(1),r_aux.getHiperbola(2),r_aux.getHiperbola(3)));
							regions.push_back(Regio(r_aux.getHiperbola(4),r_aux.getHiperbola(5),r_aux.getHiperbola(6)));
						}
					}
				}
			}
		}
		else if (h.size()==1)
		{
			h.push_back(h[0]);
			h[1].canviarSigne();
			regions.push_back(Regio(h[0],h[1]));
		}
		else if(h.size()==2)
			regions.push_back(Regio(h[0],h[1]));
		else if (h.size()==4)
			cout<<" 22222222222222222222222222222222222 4 hiperb 22222222222222222222222222222222222222222222222" <<endl;
		else
			cout<<"ERROR------ h.size()!=1..3  (Triangle::obtenirRegions) N hiperboles:"<<h.size()<<endl;
// 		cout<<"Regions  noves"<<endl;
// 		for(unsigned int k=k0;k<regions.size();k++)
// 			for(int l=0; l<regions[k].numero_hiperboles(); l++)
// 				regions[k].getHiperbola(l+1).escriure();
	
	}
	else
	 	qDebug("enviat triangle degenerat a regions");
}

/********************************************************************************************************************/
/*        Punt Triangle::puntEntrePlans(const Segment  &ss,const Segment &sv)const                                                                           */
/*      Metode q busca un punt del triangle p que estigui entre les plans parallels q passen per ss i sv.                                                */
/********************************************************************************************************************/
Punt Triangle::puntEntrePlans(const Segment  &ss,const Segment &sv)const 
{
	Vector vn=cross_product( sv.getSegment_cgal().to_vector (),ss.getSegment_cgal().to_vector());
	Pla Pi_v(Plane(sv.getSegment_cgal().source(),vn));
	Pla Pi_s(Plane(ss.getSegment_cgal().source(),vn)); /*Plans paral.lels que contenen els segments ss i sv */
	
	Punt3d pv,ps;
	int iv=4,is=4;
	bool intersecciov=false,interseccios=false;
	for(int i=0;i<3&&(!intersecciov||!interseccios);i++)
	{
		Segment s(Punt(triangle.vertex(i)),Punt(triangle.vertex(i+1)));
		if(!intersecciov){ intersecciov=Pi_v.interseccio(s,pv); iv=i;}
		if(!interseccios){ interseccios=Pi_s.interseccio(s,ps); is=i;}
	}
	
	if(!intersecciov && !interseccios) 
		return Punt(circumcenter());
		
	if(intersecciov && interseccios)
	{
		Punt pm((ps.x()+pv.x())/2.,(ps.y()+pv.y())/2.,(ps.z()+pv.z())/2.);
		if(is!=iv) return pm;
		return pm+(Punt(circumcenter())-pm)*1e-4;
	}
	
	Punt3d p1=ss.punt_mig_3d();
	Punt3d p2=sv.punt_mig_3d();
	Punt3d circTvs((p1.x()+p2.x())/2.,(p1.y()+p2.y())/2.,(p1.z()+p2.z())/2.); // baricentre del tetrahedre

	if(intersecciov) 
	{
		if(Pi_v.mateixCostat(triangle.vertex(iv),circTvs)) 
			return Punt((triangle.vertex(iv).x()+pv.x())/2.,(triangle.vertex(iv).y()+pv.y())/2.,(triangle.vertex(iv).z()+pv.z())/2.);
		return Punt((triangle.vertex(iv+1).x()+pv.x())/2.,(triangle.vertex(iv+1).y()+pv.y())/2.,(triangle.vertex(iv+1).z()+pv.z())/2.);
	}
	
	if(interseccios)
	{
		if(Pi_s.mateixCostat(triangle.vertex(iv),circTvs)) 
			return Punt((triangle.vertex(iv).x()+ps.x())/2.,(triangle.vertex(iv).y()+ps.y())/2.,(triangle.vertex(iv).z()+ps.z())/2.);
		return Punt((triangle.vertex(iv+1).x()+ps.x())/2.,(triangle.vertex(iv+1).y()+ps.y())/2.,(triangle.vertex(iv+1).z()+ps.z())/2.);
	}
		cout<<"ERROR NO TROBAT!!!!!!!!!!!!!!!!!!!!!!222222222222222222"<<endl;
		return Punt(-100.,-100.,-100.);

}


/********************************************************************************************************************/
/*                                                                                                                                                                                            */
/*            bool Triangle::interseccio(const Triangle  &t,  bool &s,  bool extrems1, bool extrems2) const                    */ 
/*                     bool Triangle::interseccio(const Triangle  &t,  bool extrems1, bool extrems2) const                                    */ 
/*          bool Triangle::interseccio(const Triangle  &t,  bool &s, int &tri_i, bool extrems1, bool extrems2) const        */ 
/*           bool Triangle::interseccio(const Triangle  &t,  bool &s, int &tri_i, bool extrems1, bool extrems2) const        */ 
/*                                                                                                                                                                                            */
/*      Metodes q comproven si el triangle interseca anb el triangle t                                                                                  */
/*                                                                                                                                                                                            */
/*    bool es una variable de sortida on es retorna cert si la interseccio dona un segment                                                   */ 
/*  int &tri_i,  es una variable de sortida on en cas de q siguin coplanars es retornaria informacio, NO PROGRAMAT              */
/*    extrems1= true si es enen en compte els vertex del triangle (actua) per intersecar                                                        */
/*    extrems2= true si es enen en compte els vertex del triangle t per intersecar                                                                   */
/********************************************************************************************************************/

bool Triangle::interseccio(const Triangle  &t,  Segment &s,  bool extrems1, bool extrems2) const 
{
	int tri_i;
	return interseccio(t,  s,tri_i, extrems1, extrems2);
}

bool Triangle::interseccio(const Triangle  &t,  bool extrems1, bool extrems2) const 
{
	Segment s;
	int tri_i;
	return interseccio(t,  s, tri_i,extrems1, extrems2);
}

bool Triangle::interseccio(const Triangle  &t,  Segment &s, int &tri_i, bool extrems1, bool extrems2) const 
{
	Pla p(triangle.supporting_plane());
	Pla pt(t.getTriangle_cgal().supporting_plane());
	
	Segment s1,s2;
	bool estr;
	if(p.interseccio(t,s1,estr,extrems2)) // t interseca amb pla p s1 es el segment interseccio estr=true si t esta contingut a p
	{
// 		cout<<"segment interseccio ";
// 		s1.escriure();
		if(!estr)	// triangles no coplanars
		{
			if(Punt(s1.source())==Punt(s1.target()))
				if(extrems2 && dinsTriangle(s1.source(), estr,extrems1,true)){s=s1; return true; }
				else   return false;
			else
				if(pt.interseccio(*this,s2,estr,extrems1)) // this interseca amb pt, s2 es el segment interseccio i estr=true si esta contiangut a pt
				{      // s1 i s2 han de ser 2 segments de la mateixa linia.
// 					escriure();
// 					cout<<"PLA: "<<pt.getPla().a()<<"x + "<<pt.getPla().b()<<"y + "<<pt.getPla().c()<<"z + "<<pt.getPla().d()<<endl;
// 					s2.escriure();
					if(estr){ cout<<"Triangle coplanars!!!!!!!!!!!! (NO INTERSECCIO FETA)"<<endl; tri_i=2;return false;}
					
					double a=s1.coord_p_on_e(s2.getSegment_cgal().source());
					double b=s1.coord_p_on_e(s2.getSegment_cgal().target());
					double c,d;
					// tenint en compte que el mateix per s1 es [0,1] cal interseccionar [0,1] amb [a,b]
					if(a<b)  
					{	c=min(max(a,0.),1.); d=max(min(b,1.),0.);	}
					else
					{	c=min(max(b,0.),1.);d=max(min(a,1.),0.);	}
					s=Segment(Segment3D(s1.e_t(c),s1.e_t(d)));	
					if(c<d)
						return true;
					else 
						return false;
				}
				else
					return false;
		}
		else
		{
			cout<<"Triangle coplanars!!!!!!!!!!!! (NO INTERSECCIO FETA)"<<endl;
			tri_i=1;
			return false;
		}
	}
	else
		return false;
}
/********************************************************************************************************************/
/*                                                                                                                                                                                            */
/*                bool Triangle::interseccio(const Segment &s, bool extremsS, bool VertexT,bool CostatsT) const                */ 
/*         bool Triangle::interseccio(const Segment &s, bool &alCostat, bool extremsS, bool VertexT,bool CostatsT) const            */ 
/*        bool Triangle::interseccio(const Segment &s, Punt3d &p, bool &seg, bool extremsS, bool VertexT,bool CostatsT) const          */ 
/* bool Triangle::interseccio(const Segment &s, Punt3d &p, bool &seg, bool &alCostat, bool extremsS, bool VertexT,bool CostatsT) const */ 
/*                                                                                                                                                                                            */
/*      Metodes q comproven si el triangle interseca anb el segment s                                                                                  */
/*                                                                                                                                                                                            */
/*      Punt3d &p es  una variable de sortida on es retorna el punt interseccio en cas de q intersequin	                           */ 
/*     bool &seg,nt &tri_i,  es una variable de sortida q pren per valor cert si la interseccio es un segment                    */
/*    bool &alCostat  es una variable de sortida q =true si s interseca amb t a un costat de t                                        */
/*   bool extremsS, bool VertexT,bool CostatsT  true si es tenen en compte els estrems de S, vertex de t i costats de T per intersecar  */
/*                     (tenen per defecte el valor de true)                                                                                                                            */
/********************************************************************************************************************/


bool Triangle::interseccio(const Segment &s, bool extremsS, bool VertexT,bool CostatsT) const 
{
	bool alCostat;
	Punt3d p; 
	bool seg;
	return interseccio(s, p,seg, alCostat, extremsS, VertexT, CostatsT);
}

bool Triangle::interseccio(const Segment &s, bool &alCostat, bool extremsS, bool VertexT,bool CostatsT) const 
{
	Punt3d p; 
	bool seg;
	return interseccio(s, p,seg, alCostat, extremsS, VertexT, CostatsT);
}
bool Triangle::interseccio(const Segment &s, Punt3d &p, bool &seg, bool extremsS, bool VertexT,bool CostatsT) const 
{
	bool alCostat;
	return interseccio(s, p,seg, alCostat, extremsS, VertexT, CostatsT);
}
	
bool Triangle::interseccio(const Segment &s, Punt3d &p, bool &seg, bool &alCostat, bool extremsS, bool VertexT,bool CostatsT) const 
{
	seg=false;
	Vector u(triangle.vertex(0),triangle.vertex(1));
	Vector v(triangle.vertex(0),triangle.vertex(2));
	Vector n=CGAL::cross_product(u,v);
	
	if(fabs(n.x())==0&&fabs(n.y())==0&&fabs(n.z())==0)
		return false;
		
	Vector w0(triangle.vertex(0),s.getSegment_cgal().source());
	double a=-n*w0;
	double b=n*s.getSegment_cgal().to_vector();
	
	if(fabs(b)<TOLERANCE)
		if(a==0) {seg=true; return true;}
		else return false;
	double r=a/b;
	if(r<0 || r>1) return false;
	
	if(!extremsS && (fabs(r)<TOLERANCE || fabs(r-1)<TOLERANCE) ) return false;
		
	p=Punt3d(s.getOrigen().x()+r*s.getSegment_cgal().to_vector().x(),s.getOrigen().y()+r*s.getSegment_cgal().to_vector().y(),s.getOrigen().z()+r*s.getSegment_cgal().to_vector().z());
// 	cout<<"pt interseccio ";Punt(p).escriure();
	return dinsTriangle(p,alCostat,VertexT,CostatsT);
	
}

bool Triangle::intersecciolinia(const Segment &s, double &r, bool VertexT,bool CostatsT) const
{
	 Punt3d p;
	 bool seg;
	 bool alCostat;
	 return intersecciolinia(s,p,r,seg,alCostat,VertexT,CostatsT);
}

bool Triangle::intersecciolinia(const Segment &s, Punt3d &p, double &r, bool &seg, bool &alCostat, bool VertexT,bool CostatsT) const
{
	seg=true;
	Vector u(triangle.vertex(0),triangle.vertex(1));
	Vector v(triangle.vertex(0),triangle.vertex(2));
	Vector n=CGAL::cross_product(u,v);
	
	if(fabs(n.x())==0&&fabs(n.y())==0&&fabs(n.z())==0)
		return false;
		
	Vector w0(triangle.vertex(0),s.getSegment_cgal().source());
	double a=-n*w0;
	double b=n*s.getSegment_cgal().to_vector();
	
	if(fabs(b)<TOLERANCE)
		if(a==0) {seg=true; return true;}
		else return false;
	
	r=a/b;
		
	p=Punt3d(s.getOrigen().x()+r*s.getSegment_cgal().to_vector().x(),s.getOrigen().y()+r*s.getSegment_cgal().to_vector().y(),s.getOrigen().z()+r*s.getSegment_cgal().to_vector().z());
	
	return dinsTriangle(p,alCostat,VertexT,CostatsT);
	
}


/********************************************************************************************************************/
/*                                                                                                                                                                                            */
/*  bool Triangle::dinsTriangle(const Punt3d & p,const Vector & u,const Vector & v,const Punt3d & origen, bool &alCostat, bool VertexT,bool CostatsT)const      */
/*              bool Triangle::dinsTriangle(const Punt3d & p,bool &alCostat, bool VertexT,bool CostatsT)const                           */
/*               bool Triangle::dinsTriangle(const Punt3d & p, bool VertexT,bool CostatsT)const                                                    */
/*                                                                                                                                                                                            */
/*      Metodes q comproven si el punt p esta dins el triangle                                                                                               */
/*              u i v son dos vectors q defineixen el triangle, q surten de l'origen.                                                                       */
/*               alCostat  de sortida = true si p esta a un costat del triangle                                                                               */
/*                VertexT = true si el metode retorna true quan p es un vertex del triangle                                                       */
/*                CostatsT = true si el metode retorna true quan p esta en un costat del triangle                                                       */
/********************************************************************************************************************/


bool Triangle::dinsTriangle(const Punt3d & p,const Vector & u,const Vector & v,const Punt3d & origen, bool &alCostat, bool VertexT,bool CostatsT)const
{
	return dinsTriangle(p,alCostat,VertexT,CostatsT);

	if(!Pla(triangle.supporting_plane()). puntPertanyPla(p)) return false;

	double uu=u*u;
	double uv = u*v;
	double vv=v*v;
	Vector w(origen,p);
	double wu=w*u;
	double wv=w*v;
	double D=uv*uv-uu*vv;
	double s=(uv*wv-vv*wu)/D;
	if(s<0.0 || s>1.0) return false;
	if(!CostatsT && (fabs(s)<TOLERANCE||fabs(s-1)<TOLERANCE)){alCostat=true; return false;}
	double t=(uv*wu-uu*wv)/D;
// 	escriure();
// 	Punt(p).escriure();
// 	cout<<"s "<<s<<" t "<<t;
	if(t<0.0 || t>1.0) return false;
// 	if(!CostatsT && (fabs(s)<TOLERANCE||fabs(s-1)<TOLERANCE)){alCostat=true; return false;}
	if(!CostatsT && (fabs(t)<TOLERANCE||fabs(t-1)<TOLERANCE)){alCostat=true; return false;}
	if(!VertexT && (fabs(s)<TOLERANCE||fabs(s-1)<TOLERANCE)&&(fabs(t)<TOLERANCE||fabs(t-1)<TOLERANCE)) return false;
	return true;
}

bool Triangle::dinsTriangle(const Punt3d & p,bool &alCostat, bool VertexT,bool CostatsT)const
{
	Pla pla(triangle.supporting_plane());
	Vector n=pla.getPla().orthogonal_vector();
	Punt c=Punt(circumcenter());  
	Punt3d ps(c.x()+n.x(),c.y()+n.y(),c.z()+n.z());  // un punt per sobre del triangle
	Punt p_(p);
	int sobre_costat=0;
	alCostat=false;
	
	if(fabs(pla.avaluarPunt(p))>TOLERANCE) return false; // no esta sobre el pla del triangle
	for(int i=0;i<3;i++)
	{
		Pla pi=Pla(Plane(triangle.vertex(i),triangle.vertex(i+1),ps)); // pla per 2 vertexs i un punt q esta per sobre el triangle, q es fix
		double dp=pi.avaluarPunt(p_);
		if(fabs(dp)<TOLERANCE) // punt sobre un costat
		{
			alCostat=true;
			if(!CostatsT) return false;
			else sobre_costat++;
		}
		else
			if(dp*pi.avaluarPunt(c)<0) return false; // punt i circumcentre a dos costats != del pla perpendicular al triangle q passa per 1 costat.
	}
	if(!VertexT&&sobre_costat>1)  // esta sobre dos costat, per tant esta sobre un vertex.
		return false;
	return true;
	
/*
	Vector u(triangle.vertex(0),triangle.vertex(1));
	Vector v(triangle.vertex(0),triangle.vertex(2));
	return dinsTriangle(p,u,v,triangle.vertex(0),alCostat,VertexT,CostatsT);*/
	
}

bool Triangle::dinsTriangle(const Punt3d & p, bool VertexT,bool CostatsT)const
{
	bool alCostat;
	return dinsTriangle(p,alCostat,VertexT,CostatsT);
}


/********************************************************************************************************************/
/*                                                                                                                                                                                            */
/*                 vector<Triangle> Triangle::retallar(const Tetrahedron &T,const Segment &ss,const Segment &sv,bool cV)const              */
/*                                                                                                                                                                                            */
/*      Metode q retalla descomposa la part del triangle contingutda a Tvs amb triangles de manera q no intersequin amb els */                       
 /*                  plans parallels q contenen ss i sv tenint en compte si el segment ss conte vertex (cV) de la cara on pertany                    */
/*                     son tot parametres d'entrada                                                                                                                              */
/********************************************************************************************************************/

//NOU K NO ACABA DE FUNCIONAR
/*
vector<Triangle> Triangle::retallar(const Tetrahedron &Tvs,const Segment &ss,const Segment &sv,const Pla &plav, const Pla &plas, const double &valor_s,bool cV)const
{
	// 	cout<<" RRRREEEETTTTAAALLLLLLLLAAAR : retallar!"<<endl;
	bool aux;
	vector<Triangle> t,t_aux;
	t.clear();
	
	bool interseccionaplav=false,interseccionaplas=false;
	bool trv=false,trs=false;
	int i=0;
	
	while(i<3&&fabs(plav.avaluarPunt(Punt(triangle.vertex(i))))<TOLERANCE) i++;
	if(i==3) trv=true;
	i=0;
	while(i<3&&plav.avaluarPunt(Punt(triangle.vertex(i)))*plav.avaluarPunt(Punt(triangle.vertex(i)))>-TOLERANCE) i++;
	if(i<3) interseccionaplav=true;
	i=0;
	while(i<3&&fabs(plas.avaluarPunt(Punt(triangle.vertex(i)))-valor_s)<TOLERANCE)	i++;
	if(i==3) trs=true;
	i=0;
	while(i<3&&(plas.avaluarPunt(Punt(triangle.vertex(i)))-valor_s)*(plav.avaluarPunt(Punt(triangle.vertex(i)))-valor_s)>-TOLERANCE) i++;
	if(i<3) interseccionaplas=true;
	
	if(interseccionaplas||interseccionaplav||trv||trs)
	{
	// 	cout<<"\n Plas: "<<plas.getPla().a()<<"x +"<<plas.getPla().b()<<"y + "<<plas.getPla().c()<<"z + "<<plas.getPla().d()<<"=0"<<endl;
	// 	cout<<" Plav: "<<plav.getPla().a()<<"x +"<<plav.getPla().b()<<"y + "<<plav.getPla().c()<<"z + "<<plav.getPla().d()<<"=0"<<endl;
		Segment segmentplav,segmentplas;  //contindran els segments interseccio del triangle amb el plav (plas)
		if(trv)
		{
	// 		cout<<" COPLANAR: amb Pla v: "<<trv<<endl;
			if(interseccio(sv.getSegment_cgal(),aux,cV,true,true)) t.push_back(*this);
		}
		else // NO COPLANARS
		{
			Segment snouv;
			vector<Punt3d> pt;
			Punt3d Bari = Punt(ss.punt_mig_3d()).punt_mig(sv.punt_mig_3d());
			bool costatBariv=false;
			int iv=0;
			
			if(interseccionaplav)
			{
				interseccionaplav=plav.interseccio(triangle,segmentplav,trv,cV);
				Triangle t(*this);
				arreglar(t,plav, Bari, iv,segmentplav,&snouv,costatBariv,Tvs,cV);
				if(costatBariv)  t_aux.push_back(Triangle(Triangle3(snouv.source(),snouv.target(),t.getTriangle_cgal().vertex(iv))));
				else 
				{
					t_aux.push_back(Triangle(Triangle3(snouv.source(),snouv.target(),t.getTriangle_cgal().vertex(iv+1))));
					t_aux.push_back(Triangle(Triangle3(t.getTriangle_cgal().vertex(iv+1),t.getTriangle_cgal().vertex(iv+2),snouv.target())));
				}
			}
			else
				t_aux.push_back(*this);
				
			if(interseccionaplas)
			{
				Pla plas_b(Plane(plas.getPla().a(),plas.getPla().b(),plas.getPla().c(),plas.getPla().d()-valor_s));
				for(unsigned int k=0;k<t_aux.size();k++)
				{
					Triangle3 ta=t_aux[k].getTriangle_cgal();
					interseccionaplas=plas_b.interseccio(ta,segmentplas,trs,cV);
					if(trs)
					{
		// 				cout<<" COPLANAR: amb Pla s: "<<trs<<endl;
						if(trv && interseccio(sv.getSegment_cgal())) t.push_back(t_aux[k]);
					}
					else
					{
						Segment snous;
						bool costatBaris=false;
						int is=0;
						// snous te source sobre costat uneix vertex is is+1 i target unint is is+2
		// 				cout <<" pla s "<< interseccionaplas<<" costatBari  s ";
						if(interseccionaplas)
						{
							Triangle t_a;
							t_aux[k].arreglar(t_a,plas_b, Bari, is,segmentplas,&snous,costatBaris,Tvs,cV);
					
/*							ta=t_a.getTriangle_cgal();
		// 					cout<< costatBaris<<" is "<<is;
							if(costatBaris)	t.push_back(Triangle(Triangle3(snous.source(),snous.target(),ta.vertex(is))));
							else 
							{
								t.push_back(Triangle(Triangle3(snous.source(),snous.target(),ta.vertex(is+1))));
								t.push_back(Triangle(Triangle3(ta.vertex(is+1),ta.vertex(is+2),snous.target())));
							}
						}
						else
							t.push_back(t_aux[k]);
		// 				cout<<endl;
					}
				}
			}
			else
				t=t_aux;
	// 		for(unsigned int l=0;l<t.size();l++)
	// 			t[l].escriure();
		}
	}
	else
		t.push_back(*this);
	return t;
}
*/

vector<Triangle> Triangle::retallar(const Tetrahedron &Tvs,const Segment &ss,const Segment &sv,const Pla &plav, const Pla &plas, const double &valor_s,bool cV)const
{
	// 	cout<<" RRRREEEETTTTAAALLLLLLLLAAAR : retallar!"<<endl;
	bool aux;
	vector<Triangle> t,t_aux;
	t.clear();
	
	bool interseccionaplav=false,interseccionaplas=false;
	bool trv=false,trs=false;
	int i=0;
// 
// 	cout<<"\n Plas: "<<plas.getPla().a()<<"x +"<<plas.getPla().b()<<"y + "<<plas.getPla().c()<<"z + "<<plas.getPla().d()<<"=0"<<endl;
// 	cout<<" Plav: "<<plav.getPla().a()<<"x +"<<plav.getPla().b()<<"y + "<<plav.getPla().c()<<"z + "<<plav.getPla().d()<<"=0"<<endl;
	Segment segmentplav,segmentplas;  //contindran els segments interseccio del triangle amb el plav (plas)
	interseccionaplav=plav.interseccio(triangle,segmentplav,trv,cV);
	Pla plas_b(Plane(plas.getPla().a(),plas.getPla().b(),plas.getPla().c(),plas.getPla().d()-valor_s));
// 	interseccionaplas=plas.interseccio(triangle,segmentplas,trs,cV);
	if(trv)
	{
// 		cout<<" COPLANAR: amb Pla v: "<<trv<<endl;
		if(trv && interseccio(sv.getSegment_cgal(),aux,cV,true,true)) t.push_back(*this);
	}
	else // NO COPLANARS
	{
		Segment snouv;
		vector<Punt3d> pt;
		Punt3d Bari = Punt(ss.punt_mig_3d()).punt_mig(sv.punt_mig_3d());
		bool costatBariv=false;
		int iv=0;
		
		if(interseccionaplav)
		{
			Triangle t(*this);
			arreglar(t,plav, Bari, iv,segmentplav,&snouv,costatBariv,Tvs,cV);
			if(costatBariv)  t_aux.push_back(Triangle(Triangle3(snouv.source(),snouv.target(),t.getTriangle_cgal().vertex(iv))));
			else 
			{
				t_aux.push_back(Triangle(Triangle3(snouv.source(),snouv.target(),t.getTriangle_cgal().vertex(iv+1))));
				t_aux.push_back(Triangle(Triangle3(t.getTriangle_cgal().vertex(iv+1),t.getTriangle_cgal().vertex(iv+2),snouv.target())));
			}
		}
		else
			t_aux.push_back(*this);
			
// 		if(interseccionaplas)
			for(unsigned int k=0;k<t_aux.size();k++)
			{
				Triangle3 ta=t_aux[k].getTriangle_cgal();
				interseccionaplas=plas_b.interseccio(ta,segmentplas,trs,cV);
				if(trs)
				{
	// 				cout<<" COPLANAR: amb Pla s: "<<trs<<endl;
					if(trv && interseccio(sv.getSegment_cgal())) t.push_back(t_aux[k]);
				}
				else
				{
					Segment snous;
					bool costatBaris=false;
					int is=0;
					// snous te source sobre costat uneix vertex is is+1 i target unint is is+2
	// 				cout <<" pla s "<< interseccionaplas<<" costatBari  s ";
					if(interseccionaplas)
					{
						Triangle t_a;
						t_aux[k].arreglar(t_a,plas_b, Bari, is,segmentplas,&snous,costatBaris,Tvs,cV);
						ta=t_a.getTriangle_cgal();
	// 					cout<< costatBaris<<" is "<<is;
						if(costatBaris)	t.push_back(Triangle(Triangle3(snous.source(),snous.target(),ta.vertex(is))));
						else 
						{
							t.push_back(Triangle(Triangle3(snous.source(),snous.target(),ta.vertex(is+1))));
							t.push_back(Triangle(Triangle3(ta.vertex(is+1),ta.vertex(is+2),snous.target())));
						}
					}
					else
						t.push_back(t_aux[k]);
	// 				cout<<endl;
				}
			}
// 		else
// 			t=t_aux;
// 		for(unsigned int l=0;l<t.size();l++)
// 			t[l].escriure();
	}
	
	return t;
}

/********************************************************************************************************************/
/*                                                                                                                                                                                            */
/*   void Triangle::arreglar(const Pla &plav, const Punt &Bari, int &iv,const Segment & segmentplav, Segment *snouv,bool &costatBariv,const  Tetrahedron &Tvs)const                           */
/*                                                                                                                                                                                            */
/*     Metode q donat un triangle i que interseca amb plav en el segment segmentplav, retorna utilitzant parametres de retorn:          */
/*                iv:  el vertex del triangle que els dos costats que surten d'ell intersequen al pla plav.                                                             */
/*                costatBari:  un boolea que es cert si el Bari esta al mateix costat del pla que el vertex iv del triangle                                   */
/*                snouv: un segment  que:  - te els extrems a les arestes del triangle q intersequen al pla                                                         */    
/*                                                          - esta al mateix costat del plav que el Bari                                                                                        */
/*                                                          - no interseca al tetraedre Tvs                                                                                                           */  /********************************************************************************************************************/

void Triangle::arreglar(Triangle &t, const Pla &plav, const Punt &Bari, int &iv,const Segment & segmentplav, Segment *snouv,bool &costatBariv,const  Tetrahedron &Tvs,bool cV)const
{
	iv=0;
	while(iv<3 & (plav.mateixCostat(triangle.vertex(iv),triangle.vertex(iv+1))  || plav.mateixCostat(triangle.vertex(iv),triangle.vertex(iv+2)))) iv++;   
	// vertex(iv) te 2 arestes q tallen amb plav.
	t=*this;
	if(iv==3) 
	{
		iv=0;
		while(iv<3 & (!plav.puntPertanyPla(Punt(triangle.vertex(iv))))) iv++;   
		if(iv!=3)
		{
			Segment s(Punt(triangle.vertex(iv)),Punt(triangle.vertex(iv+1)));
			if(!cV)
			{
				t=Triangle(triangle.vertex(iv+1),triangle.vertex(iv+2),s.e_t(1e-6));
				iv=(iv+2)%3;
			}
			else
			{
				t=Triangle(triangle.vertex(iv+1),triangle.vertex(iv+2),s.e_t(-1e-6));
				iv=0;
 				while(iv<3 & (plav.mateixCostat(t.getTriangle_cgal().vertex(iv),t.getTriangle_cgal().vertex(iv+1))  || plav.mateixCostat(t.getTriangle_cgal().vertex(iv),t.getTriangle_cgal().vertex(iv+2)))) 	iv++;   
			}
		}
	}
	if(iv==3)
		cout<<"PROBLEMA  A  ARREGLAR"<<endl;
	else 
	{
		Segment s1,s2;
		costatBariv=plav.mateixCostat(t.getTriangle_cgal().vertex(iv),Bari.getPunt());
// 		cout<<"cB: "<<costatBariv<<" ";
		if(costatBariv )
		{
			s1=Segment (segmentplav.source(),t.getTriangle_cgal().vertex(iv));
			s2=Segment (segmentplav.target(),t.getTriangle_cgal().vertex(iv));
		}
		else
		{
			s1=Segment(segmentplav.source(),t.getTriangle_cgal().vertex(iv+1));
			s2=Segment(segmentplav.target(),t.getTriangle_cgal().vertex(iv+2));
		}
// 		cout<<"v_iv: ("<<t.getTriangle_cgal().vertex(iv).x()<<", "<<t.getTriangle_cgal().vertex(iv).y()<<", "<<t.getTriangle_cgal().vertex(iv).z()<<")"<<endl;
// 		cout<<"s1 ";s1.escriure();
// 		cout<<"s2 ";s2.escriure();
		
		double d1=s1.trobar_parametre(Tvs,cV);
		double d2=s2.trobar_parametre(Tvs,cV);
		
		*snouv=Segment(Segment3D(s1.e_t(d1),s2.e_t(d2)));
		int it=5;
		bool contingutFrontera;
		while(snouv->interseccio(Tvs,contingutFrontera,cV,false,true)&&it>0) // els booleans son (extremsS,vertexT,arestesT)
		{
			d1=d1/2.;
			d2=d2/2.;
			*snouv=Segment(Segment3D(s1.e_t(d1),s2.e_t(d2)));
			it--;
		}
// 		cout<<"T intersca "<<snouv->interseccio(Tvs,contingutFrontera,false,false,true)<<endl;
// 		cout<<"arreglat ";snouv->escriure();
	}

}

bool Triangle::conteVertex(const Segment &s)const
{
	for(unsigned int i=0;i<3;i++)
		if(s.getOrigen()==Punt(triangle.vertex(i))||s.getDesti()==Punt(triangle.vertex(i)))
			return true;
	
	return false;
}

void Triangle::modificacio_segment(Segment &s)const
{
	bool no=true;
	for(unsigned int i=0;i<3&&no;i++)
		if(s.getOrigen()==Punt(triangle.vertex(i))||s.getDesti()==Punt(triangle.vertex(i)))
		{
			no=false;
			Segment s1(Punt(triangle.vertex(i+1)),Punt(triangle.vertex(i+2)));
			Segment s2(Punt(triangle.vertex(i+1)),Punt(triangle.vertex(i)));
			double x=s.getOrigen().x()-1e-3;
			double h1=2,h2=2;
			if(fabs(s1.getVector().x())>TOLERANCE)
			{
				h1=(x-triangle.vertex(i+1).x())/s1.getVector().x();
				if(fabs(s2.getVector().x())>TOLERANCE)
				{
					h2=(x-triangle.vertex(i+1).x())/s2.getVector().x();
					if((h1<0&&h1>1)||(h2<0&&h2>1))
					{
						x=s.getOrigen().x()+1e-3;
						h1=(x-triangle.vertex(i+1).x())/s1.getVector().x();
						h2=(x-triangle.vertex(i+1).x())/s2.getVector().x();
					}
// 					cout<<h1<<" "<<h2<<endl;
// 					cout<<"modificat"<<endl;
					s=Segment(s1.e_t(h1),s2.e_t(h2));
				}
				else
					cout<<"no pot modificar-se pero caldria"<<endl;
			}
			else
				cout<<"no pot modificar-se (caldria)"<<endl;
		}
}


double min(double a, double b)
{
	if(a<b) return a;
	else return b;
}
double max(double a, double b)
{
	if(a<b) return b;
	else return a;
}

bool Triangle::interseccioParalelapipede(const vector<Pla> &pla)const
{
	unsigned int i=0;
	int  j=0;
	while(i<pla.size())
	{
		while(i<pla.size() && pla[i].avaluarPunt(Punt(triangle.vertex(0)))<-1e-9)//TOLERANCE)
			i++;
		if(i==pla.size())
			return true;
		j=1;
		
		while(j<3 && pla[i].avaluarPunt(Punt(triangle.vertex(j)))>-1e-9)//-TOLERANCE)
			j++;
		if(j==3)
			return false;
		i++;
	}
	return true;

}
bool Triangle::interseccioParalelapipede(double xm,double xM,double ym,double yM,double zm,double zM)
{
	 
	if(xmin()>xM || xmax()<xm) return false;
	if(ymin()>yM || ymax()<ym) return false;
	if(zmin()>zM || zmax()<zm) return false;
	return true;
}

// bool Triangle::interseccioPiramide(vector<Pla> pla) const 
// {
// 	int i=0, j=0;
// 	while(i<pla.size())//4)
// 	{
// 		while(i<pla.size()/*4*/ && pla[i].avaluarPunt(Punt(triangle.vertex(0)))<-1e-9)//TOLERANCE)
// 			i++;
// 		if(i==pla.size())//4)
// 			return true;
// 		j=1;
// 		while(j<3 && pla[i].avaluarPunt(Punt(triangle.vertex(j)))>-1e-9)//TOLERANCE)
// 			j++;
// 		if(j==3)
// 			return false;
// 		i++;
// 	}
// 	return true;
// }


vector<Punt> Triangle::vertex_ordenats_x()const
{
	vector<Punt> p;
	int j=0;
	for(int i=0;i<3;i++)
	{
		p.push_back(Punt(triangle.vertex(i+1)));
		for(j=0;j<i&&p[j]<p[i];j++){}
		if(j!=i) 
		{
			for(int k=i;k>j;k--) p[k]=p[k-1];
			p[j]=Punt(triangle.vertex(i+1));
		}
	}
	return p;
}

vector<Punt> Triangle::vertex_ordenats_y()const
{
	
	Triangle tr(Punt(triangle.vertex(0).y(),triangle.vertex(0).z(),triangle.vertex(0).x()),Punt(triangle.vertex(1).y(),triangle.vertex(1).z(),triangle.vertex(1).x()),Punt(triangle.vertex(2).y(),triangle.vertex(2).z(),triangle.vertex(2).x()));
	vector<Punt> p=tr.vertex_ordenats_x();
	for(unsigned int i=0;i<p.size();i++)
		p[i]=Punt(p[i].z(),p[i].x(),p[i].y());
	return p;
}

vector<Punt> Triangle::vertex_ordenats_z()const
{
	
	Triangle tr(Punt(triangle.vertex(0).z(),triangle.vertex(0).x(),triangle.vertex(0).y()),Punt(triangle.vertex(1).z(),triangle.vertex(1).x(),triangle.vertex(1).y()),Punt(triangle.vertex(2).z(),triangle.vertex(2).x(),triangle.vertex(2).y()));
	vector<Punt> p=tr.vertex_ordenats_x();
	for(unsigned int i=0;i<p.size();i++)
		p[i]=Punt(p[i].y(),p[i].z(),p[i].x());
	return p;
}


vector<Pla> Triangle::plansDelimitadors(const Segment &sv)const
{
	vector<Punt3d> p;
	p.push_back(sv.source());
	p.push_back(sv.target());
	
	return plansDelimitadors(p,true);
}

vector<Pla> Triangle::plansDelimitadors(const Punt &pv)const
{
// 	pv.escriure();
	vector<Punt3d> p;
	p.push_back(Punt3d(pv.x(),pv.y(),pv.z()));
// 	return plansDelimitadors(p,false);
	for(int i=0;i<3;i++)
		p.push_back(triangle.vertex(i)); 
		
	double centre_x=0,centre_y=0,centre_z=0;
	for(unsigned int i=0;i<p.size();i++)
	{
		centre_x=centre_x+p[i].x();
		centre_y=centre_y+p[i].y();
		centre_z=centre_z+p[i].z();
	}
	double d=(double)p.size();
	Punt central(centre_x/d,centre_y/d,centre_z/d);
	
	vector<Pla> plans;
	for(unsigned int i=0;i<p.size();i++)
	{
		Pla pla_=Pla(Plane(p[i],p[(i+1)%4],p[(i+2)%4]));
		if(pla_.avaluarPunt(central)<-TOLERANCE)// && pla_.avaluarPunt(p[i1])<-TOLERANCE)
			plans.push_back(pla_);
		else
		{
			pla_.canviarSigne();
			plans.push_back(pla_);
		}
	}
	return plans;
}

vector<Pla> Triangle::plansDelimitadors(vector<Punt3d> p,bool bounding_box)const
{
	vector<Pla> plans;
	
	for(int i=0;i<3;i++)
		p.push_back(triangle.vertex(i)); 
	
	double xmin,xmax,ymin,ymax;
	double zmin,zmax;
	bounding_box=true;
	if(bounding_box)
	{	
		xmin=p[0].x();xmax=p[0].x();
		ymin=p[0].y();ymax=p[0].y();
		zmin=p[0].z();zmax=p[0].z();
		for(unsigned int i=1;i<p.size();i++)
		{
			if(xmin>p[i].x()) xmin=p[i].x();
			if(xmax<p[i].x()) xmax=p[i].x();
			if(ymin>p[i].y()) ymin=p[i].y();
			if(ymax<p[i].y()) ymax=p[i].y();
			if(zmin>p[i].z()) zmin=p[i].z();
			if(zmax<p[i].z()) zmax=p[i].z();
		}
	}
	double centre_x=0,centre_y=0,centre_z=0;
	for(unsigned int i=0;i<p.size();i++)
	{
		centre_x=centre_x+p[i].x();
		centre_y=centre_y+p[i].y();
		centre_z=centre_z+p[i].z();
	}
	double d=(double)p.size();
	Punt3d central(centre_x/d,centre_y/d,centre_z/d);
	
	for(unsigned int i=0;i<p.size();i++)
		for(unsigned int j=i+1;j<p.size();j++)
			for(unsigned int k=j+1;k<p.size();k++)
			{
				Pla pla_=Pla(Plane(p[i],p[j],p[k]));
				int i1;
				int i0=-1;
				for(unsigned int l=0;l<p.size();l++)
				{
					if(l!=i&&l!=j&&l!=k)
					{
						if(i0==-1) i0=l;
						else i1=l;
					}
				}
				if(pla_.avaluarPunt(p[i0])*pla_.avaluarPunt(p[i1])>-TOLERANCE)
				{
					if(pla_.avaluarPunt(central)<-TOLERANCE)// && pla_.avaluarPunt(p[i1])<-TOLERANCE)
						plans.push_back(pla_);
					else
					{
						pla_.canviarSigne();
						plans.push_back(pla_);
					}
				}
			}
			
	if(bounding_box)
	{
		plans.push_back(Pla(-1,0,0,xmin-TOLERANCE));
		plans.push_back(Pla(1,0,0,-xmax-TOLERANCE));
		plans.push_back(Pla(0,-1,0,ymin-TOLERANCE));
		plans.push_back(Pla(0,1,0,-ymax-TOLERANCE));
		plans.push_back(Pla(0,0,-1,zmin-TOLERANCE));
		plans.push_back(Pla(0,0,1,-zmax-TOLERANCE));
	}
	
	return plans;
}
/*
void Triangle::temps()const
{
	cout<<"temps interseccio triangle/segment "<< temps_t_s<<endl;
	cout<<"temps interseccio triangle/triangle "<< temps_t_t<<endl;
	cout<<"temps interseccio triangle/tetraedrer "<< temps_t_t<<endl;
}

Triangle::tempsG(double temps)const
{
	cout<<"temps interseccio triangle/segment "<< temps_t_s<<"     "<< temps_t_s*100./temps<<endl;
	cout<<"temps interseccio triangle/triangle "<< temps_t_t<<"     "<< temps_t_t*100./temps<<endl;
	cout<<"temps interseccio triangle/tetraedre "<< temps_t_T<<"     "<< temps_t_T*100./temps<<endl;
}

*/

