#include "vector3D.h"

/**********************************************************************************************/
/*   Vector3D::Vector3D()                                                                     */
/*Constructor per defecte de la classe Vector3D que inicialitza les coordenades del vector a 0*/
/**********************************************************************************************/

Vector3D::Vector3D() 
{
	coord_x = 0.0 ;
	coord_y = 0.0 ;
	coord_z = 0.0 ;
}

/*****************************************************************************/
/*   Vector3D::Vector3D(double x, double y, double z)                        */
/*Constructor amb parametres que construeix el vector amb els valors indicats*/
/*****************************************************************************/

Vector3D::Vector3D(double x, double y, double z) 
{
	coord_x = x ;
	coord_y = y ;
	coord_z = z ;
}


Vector3D::Vector3D(const Punt &p, const Punt &v)
{
	coord_x=v.x()-p.x();
	coord_y=v.y()-p.y();
	coord_z=v.z()-p.z();
}

/************************************/
/*   Vector3D::~Vector3D()          */
/*Destructor de la classe Vector3D  */
/************************************/

Vector3D::~Vector3D() {}

/*****************************************************/
/*   double Vector3D::retorna_x()                    */
/*Metode que retorna la primera coordenada del vector*/
/*****************************************************/

double Vector3D::x()const
{
	return (coord_x );
}

/*****************************************************/
/*   double Vector3D::retorna_y()                    */
/*Metode que retorna la segona coordenada del vector */
/*****************************************************/

double Vector3D::y()const
{
	return (coord_y );
}

/*****************************************************/
/*   double Vector3D::retorna_z()                    */
/*Metode que retorna la tercera coordenada del vector*/
/*****************************************************/

double Vector3D::z()const
{
	return (coord_z );
}

/***************************************************************/
/*   double Vector3D::modifica_x()                             */
/*Metode que permet modificar el valor de la primera coordenada*/
/***************************************************************/

void Vector3D::modifica_x(double x)
{
	coord_x=x;
}

/***************************************************************/
/*   double Vector3D::modifica_y()                             */
/*Metode que permet modificar el valor de la segona coordenada */
/***************************************************************/

void Vector3D::modifica_y(double y)
{
	coord_y=y;
}

/***************************************************************/
/*   double Vector3D::modifica_z()                             */
/*Metode que permet modificar el valor de la tercera coordenada*/
/***************************************************************/

void Vector3D::modifica_z(double z)
{
	coord_z=z;
}

/**********************************************************************/
/*   void Vector3D::modifica_coordenades(double x, double y, double z)*/
/*Metode que permet modificar les tres coordenades del vector de cop  */
/**********************************************************************/

void Vector3D::modifica_coordenades(double x, double y, double z)
{
	coord_x=x;
	coord_y=y;
	coord_z=z;
}

/**************************************************/
/*   void Vector3D::converteix(Punt p1, Punt p2)  */
/*Metode que permet convertir dos punts a vector  */
/**************************************************/

void Vector3D::converteix(Punt p1, Punt p2)
{
	coord_x=p1.x()-p2.x();
	coord_y=p1.y()-p2.y();
	coord_z=p1.z()-p2.z();
}

/************************************************************/
/*   Vector3D Vector3D::vector_normal(Punt a,Punt b,Punt c) */
/*Metode que calcula el vector normal a partir de 3 punts   */
/************************************************************/

Vector3D Vector3D::vector_normal(Punt a,Punt b,Punt c)
{
	Vector3D v1,v2, prod;
	double modul=0;

	v1.modifica_coordenades(b.x()-a.x(), b.y()-a.y(), b.z()-a.z());
	v2.modifica_coordenades(c.x()-a.x(), c.y()-a.y(), c.z()-a.z());

	prod = v2.producte_vectorial(v1);
	/*Normalitzacio*/
	/*modul=prod.calcula_modul();
	prod.modifica_x(prod.x()/modul);
	prod.modifica_y(prod.y()/modul);
	prod.modifica_z(prod.z()/modul);*/
	
	return (prod);
}

/*******************************************************************************/ 
/*   Vector3D Vector3D::producte_vectorial(Vector3D a )                        */
/*Metode que permet calcular el producte vectorial entre 2 vectors (actual x a)*/
/*******************************************************************************/

Vector3D Vector3D::producte_vectorial(Vector3D a)const
{
	Vector3D c;

	c.modifica_x(y()*a.z()-z()*a.y());
	c.modifica_y(z()*a.x()-x()*a.z());
	c.modifica_z(x()*a.y()-y()*a.x());

	return c;
}

/***************************************************************/
/*   double Vector3D::calcula_modul()                          */
/*Metode que retorna el modul del vector al qual fem referencia*/
/***************************************************************/

double Vector3D::calcula_modul()const
{
	double modul;

	modul=sqrt(((coord_x*coord_x)+(coord_y*coord_y)+(coord_z*coord_z)));
	return(modul);
}

double Vector3D::calcula_modul2()const
{
	return (coord_x*coord_x)+(coord_y*coord_y)+(coord_z*coord_z);
}
/***************************************************************/
/*   double Vector3D::calcula_modul()                          */
/*Transforma el vector actual en el corresponent vector unitari*/
/***************************************************************/

void Vector3D::transforma_unitari() 
{
	double modul;

	modul=calcula_modul();
	coord_x=coord_x/modul;
	coord_y=coord_y/modul;
	coord_z=coord_z/modul;
}


Vector3D Vector3D::normalitzar() 
{
	Vector3D c;
	double modul;
	
	modul=calcula_modul();
	
	//Vector3D c(coord_x/modul,coord_y/modul,coord_z/modul);
	c.coord_x=coord_x/modul;
	c.coord_y=coord_y/modul;
	c.coord_z=coord_z/modul;

	return c;
}


/**********************************************************/
/*   double Vector3D::producteEscalar(Vector3D v2)        */
/*Metode que retorna el producte escalar entre dos vectors*/
/**********************************************************/

double Vector3D::producteEscalar(Vector3D v2)const
{
	return ( coord_x*v2.x() + coord_y*v2.y() + coord_z*v2.z() ) ;
}

double Vector3D::perpendicular2D(Vector3D u)
{
	return coord_x*u.y()-coord_y*u.x();
}

/*******************************************************/
/*   bool Vector3D::iguals(Vector3D v)                 */
/*Metode que ens informa si dos vectors son o no iguals*/
/*******************************************************/

bool Vector3D::iguals(Vector3D v)const
{
	if ( coord_x == v.x() && coord_y == v.y() && coord_z == v.z())
	{
		return true;
	}
	else
	{
		return false;
	}
}

/*******************************************************/
/*   bool Vector3D::multiple(Vector3D v)const                 */
/*Metode que ens informa si dos vectors son multiples un de l'altre o no*/
/*******************************************************/

bool Vector3D::multiple(Vector3D v)const
{
//	double TOLERANCE=0.0000000001;
	double lambda;
	if (v.x()!=0)
	{
		lambda=coord_x/v.x();
		if(fabs(coord_y-lambda*v.y())<TOLERANCE && fabs(coord_z-lambda*v.z())<TOLERANCE) return true;
		else return false;
	}
	else if(v.x()!=0)
	{
		lambda=coord_y/v.y();
		if(fabs(coord_x-lambda*v.x())<TOLERANCE && fabs(coord_z-lambda*v.z())<TOLERANCE) return true;
		else return false;
	}
	else if(v.z()!=0)
	{
		lambda=coord_z/v.z();
		if(fabs(coord_x-lambda*v.x())<TOLERANCE && fabs(coord_y-lambda*v.y())<TOLERANCE) return true;
			else return false;
	}
	else 
	{
// 		cout<<"else"<<endl;
		return  coord_x==0 && coord_y==0 && coord_z==0; 
	}
}

/********************************************************/
/*   double Mesh::determinant (Point a, Point b)        */
/*Mètode que calcula el determinant entre 2 punts donats*/
/********************************************************/
  
double Vector3D::determinant (Vector3D v2)const
{
    double det;
    det = coord_x * v2.y() - v2.x() * coord_y;
    if (det < TOLERANCE && det > -TOLERANCE) det = 0.0;
    return det;
}

/*********************************************************/
/* double Vector3D::angleEntreVectors2D (Vector3D, bool) */
/*Mètode que retorna l'angle que formen 2 vectors        */
/*********************************************************/

double Vector3D::angleEntreVectors (Vector3D v, bool espai)const
{
    double a, dotProductqr, lengthq, lengthr;
    if ( iguals(v) ) return 0.;

    if(espai)
    {
    	dotProductqr = producteEscalar(v);
    	lengthq = calcula_modul();
    	lengthr = v.calcula_modul();
    }
    else
    {
    	dotProductqr = coord_x * v.x() + coord_y * v.y();
    	lengthq = sqrt(coord_x * coord_x + coord_y * coord_y);
    	lengthr = sqrt(v.x() * v.x() + v.y() * v.y());
    }
    
    if (determinant(v) == 0 && dotProductqr < 0) return 180.;

    a = (acos(dotProductqr / (lengthq * lengthr)) * 180.) / PI;

    if (determinant(v) < 0) a = 360. - a;

    return a;
}

double Vector3D::angle(const Vector3D &u, bool fins360)const
{ 
	if (*this==u ) return 0.;

	double det=coord_x*u.y()-coord_y*u.x();
	double uv=(*this)*u;
	if (det == 0 && uv < 0) return M_PI;
 
	double uu=u*u;
	double vv=(*this)*(*this);
	
	double a=acos(u*(*this)/(u.calcula_modul()*calcula_modul()));
	
	if(fins360 && det < 0)  a = 2*M_PI - a;
	
	return a;
}

/************************************************************************/
/* double Vector3D::angleEntreVectors2D (Vector3D, bool)                */
/*Metode que calcula si el punt actual es troba a l'esquerra del vector */
/************************************************************************/

bool Vector3D::esquerraVector(Punt inici, Punt p) const
{
	Vector3D vectAB, vectAC;
	bool esquerra=true;

	vectAC.modifica_coordenades((p.x()-inici.x()),(p.y()-inici.y()),(p.z()-inici.z()));

	/*Estara a l'esquerra si el determinant entre els dos vectors anteriors es mes gran que 0, es positiu*/

	if(((coord_x*vectAC.y())-(vectAC.x()*coord_y))<=0)
	{
		esquerra=false;
	}

	return(esquerra);
}

/************************************************************************/
/* void Vector3D::nega_vector()                                         */
/*Metode que multiplica per (-1) cadascuna de les coordenades del vector*/
/************************************************************************/

void Vector3D::nega_vector()
{
	coord_x = -coord_x;
	coord_y = -coord_y;
	coord_z = -coord_z;
}

bool Vector3D::operator==( Vector3D vect)const
{
	bool iguals=false;
	if(fabs(coord_x - vect.x())<=TOLERANCE && fabs(coord_y- vect.y())<=TOLERANCE && fabs(coord_z- vect.z())<=TOLERANCE)
	{
		iguals=true;
	}
	return(iguals);
}

bool Vector3D::operator!=(Vector3D p)const
{
	bool diferents=true;
	if(*this==p)
	{
		diferents=false;
	}
	return(diferents);
}

Vector3D Vector3D::operator*(double d)const
{ 
	return Vector3D(d*coord_x,d*coord_y,d*coord_z);
}

double Vector3D::operator*(const Vector3D &u)const
{ 
	return coord_x*u.x()+coord_y*u.y()+coord_z*u.z();
}

Vector3D Vector3D::operator/(Vector3D const &altre)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x / altre.coord_x , coord_y / altre.coord_y, coord_z / altre.coord_z);
	return pr;
}

Vector3D Vector3D::operator/(double n)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x / n , coord_y / n, coord_z / n);
	return pr;
}

Vector3D Vector3D::operator-(Vector3D const &altre)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x - altre.coord_x , coord_y - altre.coord_y, coord_z - altre.coord_z);
	return pr;
}

Vector3D Vector3D::operator-(double n)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x - n , coord_y - n, coord_z - n);
	return pr;
}

Vector3D Vector3D::operator+(Vector3D const &altre)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x + altre.coord_x , coord_y + altre.coord_y, coord_z + altre.coord_z);
	return pr;
}

Vector3D Vector3D::operator+(double n)
{
	Vector3D pr;
	
	pr.modifica_coordenades(coord_x + n , coord_y + n, coord_z + n);
	return pr;
}
