#ifndef VECTOR3D_H
#define VECTOR3D_H

#include <iostream.h>
#include "punt.h"
#include "objecte.h"
#include "geometriabasica.h"
#include <qcolor.h>

/***********************************************************************************************/
/*                                        class Vector3D                                       */
/*   Classe Vector3D que conté tots aquells mètodes necessaris per poder treballar amb vectors */
/***********************************************************************************************/

class Vector3D
{
	private:
		/*Atributs de classe que corresponen a les coordenades del vector al qual estem fent referencia. 
		Aquesta classe considera els vectors en 2 o 3 dimensions ja que en el cas que volguem realitzar 
		operacions amb vectors del pla tan sols caldra posar la tercera coordenada a 0*/
		double coord_x ;
		double coord_y ;
		double coord_z ;
	public:
		Vector3D () ; /*Constructor per defecte que inicialitza les coordenades del vector a 0*/
		~Vector3D();
		Vector3D (double x, double y, double z) ; /*Constructor amb parametres*/
		Vector3D(const Punt &p, const Punt &v);

		/*Metodes que retornen les coordenades del vector*/
		double x()const ;
		double y()const ;
		double z() const;

		/*Metodes que permeten modificar el valor de les coordenades de forma individual, o be, totes de cop*/
		void modifica_x(double x) ;
		void modifica_y(double y) ;
		void modifica_z(double z) ;
		void modifica_coordenades(double x, double y, double z);
		void converteix(Punt p1, Punt p2);
		
		double calcula_modul()const; /*Funcio que permet calcular el modul del vector*/
		double calcula_modul2()const; /*Funcio que permet calcular el modul del vector*/
		void transforma_unitari(); /*Metode que transforma el vector actual en el seu propi vector unitari*/
		Vector3D normalitzar();
		double producteEscalar(Vector3D)const;
		double angleEntreVectors (Vector3D , bool )const;
		double angle(const Vector3D &u, bool fins360=false)const;
		bool iguals(Vector3D)const;
		bool multiple(Vector3D v)const;
		double determinant(Vector3D)const;
		Vector3D vector_normal(Punt a,Punt b,Punt c); /*Cacul del pector normal a partir de 3 punts*/
		Vector3D producte_vectorial(Vector3D a)const; /*Cacul del producte vectorial a partir de dos vectors*/
		
		double perpendicular2D(Vector3D u);
		
		bool esquerraVector(Punt inici, Punt p)const;
		void nega_vector();
		
 		bool operator==( Vector3D)const;
		bool operator!=( Vector3D)const;
		Vector3D operator*(double d)const;
		double operator*(const Vector3D &u)const;
		Vector3D operator+(Vector3D const &altre);
		Vector3D operator+(double n);
		Vector3D operator/(Vector3D const &altre);
		Vector3D operator/(double n);
		Vector3D operator-(Vector3D const &altre);
		Vector3D operator-(double n);
} ;

#endif
