#ifndef VISIO_H
#define VISIO_H

#include <math.h>
#include <vector>
#include <qgl.h>
#include <qsize.h>
#include <pair.h>
#include <qmessagebox.h>
#include <qpopupmenu.h>
#include <qvaluevector.h>
#include <qerrormessage.h>
#include "zdialeg.h"
#include <qaction.h>
#include "geometriabasica.h"
#include "vector3D.h"

#include "punt.h"
#include "poligon.h"
#include "poligonal.h"

#include "terreny.h"

typedef std::vector<Objecte *>  VectorVisio;
typedef std::vector<Objecte *>  VectorDistancies;
typedef std::vector<Objecte *>  VectorPuntsCamiMinim;

/***********************************************************************************************************/
/*                         class Visio                                                                     */
/*   Classe que permet guardar cadascun dels objectes de visio que l'usuari hagi introduit sobre el terreny*/
/***********************************************************************************************************/

class Visio
{
	private:
		/*Disposem com a atributs propis el vector d'objectes de visio entre d'altres*/
		VectorVisio objectes;
		VectorDistancies objectes_distancies;
		VectorPuntsCamiMinim objectes_cami_minim;
		vector< vector< pair<Punt, Punt> > > segments_visibles;
		vector< Segment> segments_analitzats;
		
		vector<GLubyte> textura;
		int amplada;
		int alcada;
		double xmin;
		double xmax;
		double ymin;
		double ymax;
		bool calculat;
		bool recalculat;
		
		static Visio * instancia; /*Permetrà fer que la classe Visio sigui un Singleton*/
		
	protected: 
		Visio(); /*Constructor per defecte que inicialitza la classe Visio*/

	public:
		Visio( VectorVisio ); /*Constructor amb parametres que permet configurar el vector d'objectes de visio*/
		~Visio();
		static Visio * instance(); /*Mètode que permet implementar el Singleton*/

		/*Metode que retorna el vector de objectes de visio*/
		VectorVisio getObjecteVisio();
		VectorDistancies getObjecteDistancies();
		VectorPuntsCamiMinim getObjecteCamiMinim();
				
		/*Mètode que permet modificar el vector de objectes de visio*/
		void setDelaunay( VectorVisio);
		void setDelaunayDistancies(VectorDistancies);
		void setDelaunayPuntsCamiMinim(VectorPuntsCamiMinim);
				
		void clear();
		void inserirObjecte(Objecte *);
		void inserirObjecteDistancies(Objecte *);
		void inserirObjecteCamiMinim(Objecte *);
		void eliminarObjecte(int);
		void eliminarObjecteDistancies(int);
		void eliminarObjecteCamiMinim(int);
		Objecte * obtenirObjecte(int);
		Objecte * obtenirObjecteDistancies(int);
		Objecte * obtenirObjecteCamiMinim(int);
		int numeroObjectes();
		int numeroObjectesDistancies();
		int numeroObjectesCamiMinim();
		vector< vector< pair<Punt, Punt> > > getSegmentsVisibles();
		void setSegmentsVisibles(vector< pair<Punt, Punt> > );
		int numeroSegmentsVisibles();
		vector< Segment > getSegmentsAnalitzats();
		void setSegmentsAnalitzats(vector< Segment >);
		int numeroSegmentsAnalitzats();
		void resetSegmentsVisibles();
		void inicialitza_mides(int amplada_, int alcada_, double x_min, double x_max, double y_min, double y_max);
		void inicialitza_buffer_textura();
		void emplena_textura_segment_terreny(vector< pair<Punt, Punt> >, const int& r);
		void emplena_textura_segments_terreny(const Triangle &cara,const vector<int> &conta,float parts_primer_previes,const vector<vector< pair<double, double> > > &segments_visibles,const int &r);
		void emplena_textura(const Triangle &tr,const int &n_sv,double *limits);
		GLubyte * obtenir_Textura();
		bool calculada_textura();
		bool textura_Modificada();
		void get_mides(int &amplada_, int &alcada_, double &x_min, double &x_max, double &y_min, double &y_max);
		void setMidesGraella(int ,int);
		
};

#endif
