#ifndef VISOR_TERRENY_H
#define VISOR_TERRENY_H

#include <qgl.h>
#include <math.h>
#include <qsize.h>
#include <vector.h>
#include <pair.h>
#include <qmessagebox.h>
#include <qpoint.h>

#include <sys/time.h>
#include <unistd.h>

#include "terreny.h"
#include "visio.h"
#include "regio.h"
#include "editorterreny.h"

#include "Cg/cg.h"
#include "Cg/cgGL.h"

#define APROPAR  0
#define ALLUNYAR 1

enum ModeVisual { M_TERRENY, M_ISOLINIES };
enum ModeMoviment { M_ORBITAL, M_DESP };
enum Estats { CALCULA_RES, CALCULA_BEZIER, CALCULA_CORBES, CALCULA_TOT,CALCULA_VISIO };
enum ElementVisio{ SEGMENT_V, PUNT_V};

/************************************************************************************************************************/
/*                                          class VisorTerreny : public QGLWidget                                       */
/*Classe VisorTerreny que permet representar el terreny 2.5D en OpenGL sobre la finestra corresponent (classe VisorWin) */
/************************************************************************************************************************/

struct InfoMon_VT
{
    double xmin, ymin, zmin;
    double xmax, ymax, zmax; 
    
};

struct InfoMostrar_VT
{
	bool filferros;
	bool quadre;
	bool eixos; 
	bool veure_control;
	bool veure_normals;
	bool veure_elements_visio;
	bool comprovar_visibilitat;
	bool finestres;
};

struct InfoOpenGL_VT
{
	double elevacio, rotacio; 
	double zoom;
	Vector centre;
	Vector posicioCam;
	double distancia;
    
	int _moviment;
	
	QColor color_fons;
	QColor color_per_defecte;
	QColor color_quadre;
	QColor color_eix_x;
	QColor color_eix_y;
	QColor color_eix_z;

};


struct InfoBezier_VT
{
	int resolucio;
	Nivells nvs;
};

struct InfoTemps_VT
{
	double total;
	timeval total_i;
	timeval total_f;
	
	double intersec_triangle;
	timeval intersec_triangle_i;
	timeval intersec_triangle_f;
	
	double intersec_segment;
	timeval intersec_segment_i;
	timeval intersec_segment_f;
	
	double regions;
	timeval regions_i;
	timeval regions_f;
	
	double retallar;
	timeval retallar_i;
	timeval retallar_f;
	
	double projeccio;
	timeval projeccio_i;
	timeval projeccio_f;
	
	double guardar;
	timeval guardar_i;
	timeval guardar_f;
	
	double pintar;
	timeval pintar_i;
	timeval pintar_f;
	
	void suma_total(){ 
		total=total+(double)(((&total_f)->tv_usec+(&total_f)->tv_sec*1e+6)-((&total_i)->tv_usec+(&total_i)->tv_sec*1e+6) )/1e+6; 
	}
	
	void suma_intersec_triangle(){
		 intersec_triangle=intersec_triangle+(((&intersec_triangle_f)->tv_usec+(&intersec_triangle_f)->tv_sec*1e+6) - ((&intersec_triangle_i)->tv_usec+(&intersec_triangle_i)->tv_sec*1e+6) )/1e+6; 
	}
	
	void suma_intersec_segment(){ 
		intersec_segment = intersec_segment + (((&intersec_segment_f)->tv_usec+(&intersec_segment_f)->tv_sec*1e+6)-((&intersec_segment_i)->tv_usec+(&intersec_segment_i)->tv_sec*1e+6) )/1e+6; 
	}
	void suma_regions(){ 
		regions=regions+(((&regions_f)->tv_usec+(&regions_f)->tv_sec*1e+6)-((&regions_i)->tv_usec+(&regions_i)->tv_sec*1e+6) )/1e+6; 
	}
	void suma_retallar(){ 
		retallar=retallar+(((&retallar_f)->tv_usec+(&retallar_f)->tv_sec*1e+6)-((&retallar_i)->tv_usec+(&retallar_i)->tv_sec*1e+6) )/1e+6; 
	}
	void suma_projeccio(){
		projeccio =projeccio+(((&projeccio_f)->tv_usec+(&projeccio_f)->tv_sec*1e+6)-((&projeccio_i)->tv_usec+(&projeccio_i)->tv_sec*1e+6) )/1e+6; 
	}
	void suma_guardar(){ 
		guardar=guardar+(((&guardar_f)->tv_usec+(&guardar_f)->tv_sec*1e+6)-((&guardar_i)->tv_usec+(&guardar_i)->tv_sec*1e+6) )/1e+6; 
	}
	void suma_pintar(){ 
		pintar=pintar+(((&pintar_f)->tv_usec+(&pintar_f)->tv_sec*1e+6)-((&pintar_i)->tv_usec+(&pintar_i)->tv_sec*1e+6) )/1e+6; 
	}
	
	/*void suma_total(){ total_t=+_t(double)(total_f - total_i)/CLOCKS_PER_SEC;}
	void suma_intersec_triangle(){ intersec_triangle_t=intersec_triangle_t+(double)(intersec_triangle_f - intersec_triangle_i)/CLOCKS_PER_SEC;}
	void suma_intersec_segment(){ intersec_segment_t=intersec_segment_t+(double)( intersec_segment_f- intersec_segment_i)/CLOCKS_PER_SEC;}
	void suma_retallar(){retallar_t=retallar_t+(double)( retallar_f- retallar_i)/CLOCKS_PER_SEC;}
	void suma_projeccio(){ projeccio_t=projeccio_t+(double)(projeccio_f - projeccio_i)/CLOCKS_PER_SEC;}
	void suma_guardar(){ guardar_t=guardar_t+(double)(guardar_f- lguardar_i)/CLOCKS_PER_SEC;}
	void suma_pintar(){ pintar_t=pintar_t+(double)(pintar_f - pintar_i)/CLOCKS_PER_SEC;}*/
	
	
};

struct InfoCalculsVisibilitat_VT //informacio necessaria pel calcul de visibiliat.
{

	int _element;	// diu si l'element de visio q tractem es punt,segment, polignal,....
	Segment ss;
	Tetrahedron tetraedre;
	vector<Pla> plans_tetraedre;
	vector<Triangle> triangle_bons_visio;
	
	bool veurePintarHiperboles;
	bool amb_triangles;
	bool fitarRegionsAcuradament;
	bool fitarQuadratUnitat;

	double lmx,lmy;
	int bits;
	int maxim_bits;	
	vector<float> vector_lx;
	vector<float> vector_ly;
	vector<double> vector_maxim;
	vector<double> vector_minim;
	vector<bool> vector_coplanars;
	vector<Segment> segments_estudiats;
	float x_negatiu;
	float x_positiu;
	float y_negatiu;
	float y_positiu;
	vector<int> conta;
	int parts_primer_previes;
	int area_llegida;
	int nombre_OV;
	
	vector<vector<Regio> > vector_regions;
	vector<vector<Segment> > vector_segments;
	vector<vector< pair<double, double> > > segments_vis;
	
	void setRegions( vector<double> visible,double ls,double lv,bool coplanars  )
	{
		float lx=min(lv,lmx);
		float ly=min(ls,lmy);
		vector_lx.push_back(lx);
		vector_ly.push_back(ly);
		vector_minim.push_back(visible[0]);
		vector_maxim.push_back(visible[1]);
		vector_coplanars.push_back(coplanars);
	}
	
	void setSegments(double ls,double amplada)
	{
	 	float lx=lmx/amplada;
		float ly=min(ls,lmy);
		vector_lx.push_back(lx);
		vector_ly.push_back(ly);
	}
	
	double canviCoordenadesFila(int aux, double ly_,double alcada)const
	{
		return ly_-(double)(y_positiu-y_negatiu)*(double)aux/alcada;
		double alcada_pixel=(double)(y_positiu-y_negatiu)/alcada;
	// 	if(coplanars&&!hiperboles2D)
	// 		return calculv.y_negatiu+alcada_pixel*aux;
		return (double)ly_-alcada_pixel*(double)aux;
	}
	
};

struct InfoGraphicsHardware_VT //informacio necessaria pel Graphics Hadware
{
	CGcontext context;
	CGprogram fragmentProgram;
	CGprofile fragmentProfile;
	bool haveLoadedPrograms;
	
	CGparameter xM_yM;
	CGparameter a_x2_y2_xy;
	CGparameter a_x_y_c;
	CGparameter b_x2_y2_xy;
	CGparameter b_x_y_c;
	CGparameter c_x2_y2_xy;
	CGparameter c_x_y_c;
	
	CGparameter sv0;
	CGparameter vv;
	CGparameter ss0;
	CGparameter vs;
	CGparameter t0;
	CGparameter n;
	CGparameter u;
	CGparameter v;
	CGparameter uu_vv_uv;

	GLuint fb;
	GLuint color_rb;
	int error;
	

	
	void ChooseProfiles()
	{
		// Make sure that the appropriate profiles are available on the
		// user's system.
		
		if (cgGLIsProfileSupported(CG_PROFILE_FP30))
			fragmentProfile = CG_PROFILE_FP30;
		else 
		{
			if (cgGLIsProfileSupported(CG_PROFILE_FP20))
				fragmentProfile = CG_PROFILE_FP20;
			else 
			{
				fprintf(stderr, "Neither arbfp1 or fp30 or fp20 fragment profiles supported on this system.\n");
				exit(1);
			}
		}
	}
	
	void LoadCgPrograms()
	{
	//    assert(cgIsContext(targeta.context));
		fragmentProgram = cgCreateProgramFromFile(context, CG_SOURCE,  "fragment.cg", fragmentProfile, NULL, NULL);
		if (!cgIsProgramCompiled(fragmentProgram))
		cgCompileProgram(fragmentProgram);
	
		cgGLEnableProfile(fragmentProfile);
		cgGLLoadProgram(fragmentProgram);
	}

	
	void handleCgError() 
	{
		fprintf(stderr, "Cgerror: %s\n", cgGetErrorString(cgGetError()));
		exit(1);
	}

	void carregarFragment()
	{
		// Do one-time setup only once; setup Cg programs and textures and set up OpenGL state.
		ChooseProfiles();
		LoadCgPrograms();
		cgGLEnableProfile(fragmentProfile);
		cgGLBindProgram(fragmentProgram);
			
		xM_yM=cgGetNamedParameter(fragmentProgram, "xM_yM");
		a_x2_y2_xy=cgGetNamedParameter(fragmentProgram, "a_x2_y2_xy");
		a_x_y_c=cgGetNamedParameter(fragmentProgram, "a_x_y_c");
		b_x2_y2_xy=cgGetNamedParameter(fragmentProgram, "b_x2_y2_xy");
		b_x_y_c=cgGetNamedParameter(fragmentProgram, "b_x_y_c");
		c_x2_y2_xy=cgGetNamedParameter(fragmentProgram, "c_x2_y2_xy");
		c_x_y_c=cgGetNamedParameter(fragmentProgram, "c_x_y_c");
		haveLoadedPrograms = true;
	}
	
	void CHECK_FRAMEBUFFER_STATUS()                            
	{                                                           
		GLenum status;                                            
		status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT); 
		switch(status) 
		{                                          
			case GL_FRAMEBUFFER_COMPLETE_EXT:                       
			break;                                                
			case GL_FRAMEBUFFER_UNSUPPORTED_EXT:     
				cout<<"choose different formats"<<endl;               
	// 		 choose different formats                         
			break;                                                
			default:       
				cout<<"programming error; will fail on all hardware"<<endl; 
				break;                                        
	// 		 programming error; will fail on all hardware     
	// 	    assert(0);                                            
		}
	}
	
};



class VisorTerreny : public QGLWidget
{
    Q_OBJECT
public:
    VisorTerreny( QWidget *parent=0, const char *name=0 );
    ~VisorTerreny();

    void setZoom ( int );
    void setMode ( int );
    void setModeDesp ( int );
    void setCalculaVisio(bool);

    void setLmx(float);
    void setLmy(float);
    float getLmx();
    float getLmy();
    
    void dibuixaQuadre();
    void dibuixaEixos();
    void dibuixaPla( double, QColor );
    
    void dibuixaTriangulacio3D();
    void dibuixaSuperficie();
    void dibuixaCorbes3D();
    void dibuixarObjectesVisio();
    void pintarFinestres();
    void prepararFinestresPintar();
    void triemMon(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax);

    void modificaMida( double, double, double, double, double, double );
    void modificaColors( QColor, QColor, QColor, QColor, QColor, QColor );
    void modificaResolucio( int );
    void modificaNivells( const Nivells & );

    void dibuixaPuntsControl( Vector * );
//     void dibuixaNormals( Vector * );

    void generaCorbes();
    int getResolucioMax();
    
    void calcula_visio( bool );

public slots:
    void initVista();

private:
	bool pressed;
	
	QPoint oldpos;
	Terreny *mesh;
	Visio * visio;
	int comptador;	
	
	int _mode;
	int _estat;
	
	InfoMon_VT  info_mon;
	InfoOpenGL_VT info_GL;
	InfoMostrar_VT info_mostrar;
	InfoCalculsVisibilitat_VT info_vis;
	InfoGraphicsHardware_VT info_h;
	InfoBezier_VT info_bezier;
	InfoTemps_VT info_temps;
    
	void generaCalculs();
	void dibuixaVisibilitat();
	
	
	//METODES pel CALCUL de VISIBILITAT
	// segment-segment
	void tractamentCares(const Segment &sv, double x_minV2d, double x_maxV2d, double y_minV2d, double y_maxV2d, bool variant, const int &r,const vector<vector<Segment> > &vector_segments );
	void analitzarTriangle(const pCara &cara ,const pCara &triangle_sv ,const  Segment &sv, bool variant,const vector<double> &part_visible, const int &r, const vector<Segment> &trian) ;
// 	void tractamentSegments(const Segment &sv, bool variant, const vector<pCara> &triangles_intersequen, const pCara & cara, const vector<double> &part_visible,int );
	void tractamentSegments(const Segment &sv, const Pla &pla_v, const Pla &pla_s, bool variant,const vector<pCara> &triangles_intersequen, const pCara &cara, const vector<double> &part_visible,int cs);
	// punt-segment
	void tractamentCares(const Punt &pv, double x_minV2d, double x_maxV2d, double y_minV2d, double y_maxV2d, bool variant, const int &r,const vector<vector<Segment> > &vector_segments );
	void analitzarTriangle( const pCara &cara ,const pCara &triangle_sv, const Punt &pv, bool variant,const int &r, const vector<Segment> &trian);
	void PuntSegment(const Punt &pv, const vector<pCara> &triangles_intersequen);
	//pintar transformada triangles
	void assignarColor(int conta_segments);
	void pintarRegionsHiperboliques();
	void modePintarFrameBuffer();
	void pintarQuadratsInicials(double minim,double maxim,float lx,float ly);
	void pintar3DShaderHiperboles(const vector<Regio> &regions_g,float lx,float ly,float minim,float maxim);
	void pintarSharderHiperbolesCoplanars( vector<Regio> regions_g,float lx,float ly,float minim,float maxim);
	void pintarRegioHiperbolica(Regio r,double x_max, double y_max, double e, double f,float lx, float ly);
	void PintarRegionsAproximadesPerTriangles();
	void pintarSegments();
	void pintarRegioBenAfitada(const Regio &r,const vector<bool> &rectangle,const vector<vector<Punt2d> > &punts_rectangle,double, double) const;
	//obtenir les parts visibles
	void  obtenir_segments_visibles();	
	void projeccioEixY(int conta_segments);
	void emplena_textura(const vector<Segment> &trian,const vector<int> &conta, int parts_primer_previes,const int &r);
	

		    
private slots:
    void veureFils( bool );
    void veureLimits( bool );
    void veureEixos( bool );
    void veurePuntsControl( bool );
    void veureNormals( bool );
    void actualitza();
    void mostraElementsVisio( bool );


protected:
    void initializeGL();
    void resizeGL( int, int );
    void paintGL();

    void mousePressEvent( QMouseEvent * );
    void mouseReleaseEvent( QMouseEvent * );
    void mouseMoveEvent( QMouseEvent * );
    void wheelEvent( QWheelEvent * );
};

#endif   

