#include <qimage.h>
#include <qpixmap.h>
#include <qtoolbar.h>
#include <qtoolbutton.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlabel.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qpushbutton.h>
#include <qaccel.h>
#include <qtextedit.h>
#include <qwhatsthis.h>
#include "visorwin.h"

/******************************************************************************/
/*   VisorWin::VisorWin(): QMainWindow( 0, "Visor 2.5D", WDestructiveClose )  */
/*Constructor de la classe Terreny3D, que permet crear objectes d'aquest tipus*/
/******************************************************************************/

VisorWin::VisorWin()
    : QMainWindow( 0, "Visor 2.5D", WDestructiveClose )
{
    fileTools = new QToolBar( this, "operacions amb models" );
    addToolBar( fileTools, tr( "Operacions amb models" ), DockTop, TRUE );
    
    QActionGroup *grp = new QActionGroup( this );    

    rotaAction = new QAction( QPixmap::fromMimeSource("rota.png" ), "&Rotar Camera", CTRL+Key_R, grp, "rotacions" );
    rotaAction->setToggleAction( TRUE );    
    //rotaAction->addTo( fileTools );    
      
    desplazaAction = new QAction( QPixmap::fromMimeSource("desp.png" ), "&Desplaçar Camera", CTRL+Key_D, grp, "desplacaments" );
    desplazaAction->setToggleAction( TRUE );    
    //desplazaAction->addTo( fileTools );
    grp->addTo( fileTools );
    rotaAction->setOn( TRUE );
     
    fileTools->addSeparator();

    QAction *homeAction = new QAction( QPixmap::fromMimeSource("casa.png" ), "Iniciar vista", Key_Home, this, "iniciaVista" );
    homeAction->addTo( fileTools );

    fileTools->addSeparator();

    QAction *mesZoomAction = new QAction( QPixmap::fromMimeSource("zoomsIcon.png" ), "Apropar Camera", Key_Plus, this, "augmentaZoom" );
    mesZoomAction->addTo( fileTools );

    QAction *menysZoomAction = new QAction( QPixmap::fromMimeSource("zoomrIcon.png" ), "Allunyar Camera", Key_Minus, this, "disminueixZoom" );
    menysZoomAction->addTo( fileTools );

    fileTools->addSeparator();

    QAction *filsAction = new QAction( QPixmap::fromMimeSource("fils.png" ), "Veure &Filferros", CTRL+Key_F, this, "filferros" );    
    filsAction->setToggleAction( TRUE );    

    filsAction->addTo( fileTools );

    QAction *espaiAction = new QAction( QPixmap::fromMimeSource("quadre.png" ), "Veure li&mits de l'espai", CTRL+Key_M, this, "quadre" );    
    espaiAction->setToggleAction( TRUE );    

    espaiAction->setOn( TRUE );
    espaiAction->addTo( fileTools );

    QAction *eixosAction = new QAction( QPixmap::fromMimeSource("eixos.png" ), "Veure &Eixos de coordenades", CTRL+Key_E, this, "eixos" );    
    eixosAction->setToggleAction( TRUE );    

    eixosAction->setOn( TRUE );
    eixosAction->addTo( fileTools );

    fileTools->addSeparator();

    QPopupMenu* arxiuMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Arxiu", arxiuMenu );

    QPopupMenu* configMenu = new QPopupMenu( this );
    menuBar()->insertItem( "&Preferencies", configMenu );

    QAction *configAction = new QAction( "Co&nfigura visor...", CTRL+Key_N, this, "configurar" );
    
    configAction->addTo( configMenu );

    QAction *capturaAction = new QAction( QPixmap::fromMimeSource("captura.png" ), "&Captura Imatge", CTRL+Key_C, this, "capturar" );
    capturaAction->addTo( fileTools );
    capturaAction->addTo( arxiuMenu ); 

    fileTools->addSeparator();
    
    QAction *controlAction = new QAction( QPixmap::fromMimeSource("eixos.png" ), "Veure &punts de control", CTRL+Key_P, this, "punts_control" );    
    controlAction->setToggleAction( TRUE );    

    controlAction->addTo( fileTools );

    QAction *normAction = new QAction( QPixmap::fromMimeSource("eixos.png" ), "Veure n&ormals vertexs", CTRL+Key_O, this, "normals" );    
    normAction->setToggleAction( TRUE );    

    normAction->addTo( fileTools );

    fileTools->addSeparator();
    
    visioAction = new QAction( QPixmap::fromMimeSource("interseccio.png" ), "&Finestres", CTRL+Key_F, this, "visio" );    
    connect( visioAction, SIGNAL( activated() ), this, SLOT( calculVisioDiscretitzant( ) ) ); 
      
    visio_variantAction = new QAction(QPixmap::fromMimeSource("variant.png"), "&Visio variant", CTRL+Key_I, this, "visio_variant");
    connect( visio_variantAction, SIGNAL( activated() ), this, SLOT( calculVisio( ) ) );
    
    visioAction->addTo( fileTools );
    visio_variantAction->addTo( fileTools );
    
    fileTools->addSeparator();
    
    visualitzarAction = new QAction( QPixmap::fromMimeSource("icon_ull.png" ), "Elements &Visibles", CTRL+Key_V, this, "visio_visibles" );
    visualitzarAction->setToggleAction( TRUE );
    
    visualitzarAction->addTo( fileTools );
    
    QToolButton* sortida;
    sortida = new QToolButton( QPixmap::fromMimeSource("sortir.png" ), "Surt del programa", QString::null, this, SLOT(closeWindow()), fileTools, "sortir"); 
  
    QPopupMenu * help = new QPopupMenu( this );
    menuBar()->insertItem( "A&juda", help );

    help->insertItem( "Quant a...", this, SLOT(about()), Key_F1);
    help->insertItem( "Quant a Qt", this, SLOT(aboutQt()));
    help->insertSeparator();
    help->insertItem( "Que es aixo", this, SLOT(whatsThis()), SHIFT+Key_F1);

    vgl = new VisorTerreny( this );

    connect( visualitzarAction, SIGNAL( toggled( bool ) ), vgl, SLOT( mostraElementsVisio( bool ) ) );
    connect( grp, SIGNAL( selected( QAction * ) ), this, SLOT( canviaMode( QAction * ) ) );
    connect( homeAction, SIGNAL( activated() ), vgl, SLOT( initVista() ) );     
    connect( mesZoomAction, SIGNAL( activated() ), this, SLOT( apropar() ) );     
    connect( menysZoomAction, SIGNAL( activated() ), this, SLOT( allunyar() ) );     
    connect( configAction, SIGNAL( activated() ), this, SLOT( configuraVisor() ) );
    connect( capturaAction, SIGNAL( activated() ), this, SLOT( desaImatge() ) );
    connect( filsAction, SIGNAL( toggled( bool ) ), vgl, SLOT( veureFils( bool ) ) );
    connect( espaiAction, SIGNAL( toggled( bool ) ), vgl, SLOT( veureLimits( bool ) ) );
    connect( eixosAction, SIGNAL( toggled( bool ) ), vgl, SLOT( veureEixos( bool ) ) );

    connect( controlAction, SIGNAL( toggled( bool ) ), vgl, SLOT( veurePuntsControl( bool ) ) );
    connect( normAction, SIGNAL( toggled( bool ) ), vgl, SLOT( veureNormals( bool ) ) );

    setCentralWidget( vgl );
    setGeometry ( 0, 0, 500, 500 );
    
    vgl->show();    

    modificat = FALSE;

    statusBar()->message( "Model preparat", 2000 );
}

/********************************************************************************/
/*   VisorWin::~VisorWin() { }                                                  */
/*Destructor de la classe VisorWin, que permet destruir objectes d'aquest tipus */
/********************************************************************************/

VisorWin::~VisorWin() { }

/*************************************************/
/*   void VisorWin::closeWindow()                */
/*Mètode que simplement permet tancar la finestra*/
/*************************************************/

void VisorWin::closeWindow()
{
    hide();
}

/****************************************************************/
/*   void VisorWin::about()                                     */
/*Missatge que es mostra en seleccionar l'opció d'ajuda del menú*/
/****************************************************************/

void VisorWin::about()
{
    QMessageBox::about( this, "Aplicacio",
                     "Visor 3D: Finestra d'interaccio\n" 
                     "Visualitzador de terrenys en 2.5 D\n"
		     "Resolucio de problemes de proximitat en terenys");
}

/**********************************************************************************/
/*   void VisorWin::aboutQt()                                                     */
/*S'indica el nom de la finestra que apareix quan es mostra la informació sobre Qt*/
/**********************************************************************************/

void VisorWin::aboutQt()
{
    QMessageBox::aboutQt( this, "Aplicacio" );
}

/***************************************************/
/*   void VisorWin::closeEvent( QCloseEvent *e )   */
/*Mètode que tanca la finestra de l'aplicació      */
/***************************************************/

void VisorWin::closeEvent( QCloseEvent *e )
{
    e->ignore();
    closeWindow();
}

/**********************************************************************/
/*   void VisorWin::canviaMode( QAction *mode )                       */
/*Mètode que permet indicar el tipus de moviment que es vol utilitzar */
/**********************************************************************/

void VisorWin::canviaMode( QAction *mode )
{
    if( mode == rotaAction )
    {
       vgl->setModeDesp( M_ORBITAL );
    }       
    else if( mode == desplazaAction )
    {
       vgl->setModeDesp( M_DESP );   
    }
}

/***************************************************************/
/*   void VisorWin::allunyar()                                 */
/*Informem al visor que ens trobem en mode de Zoom - (allunyar)*/
/***************************************************************/

void VisorWin::allunyar()
{
    vgl->setZoom(ALLUNYAR);
}

/**************************************************************/
/*   void VisorWin::apropar()                                 */
/*Informem al visor que ens trobem en mode de Zoom + (apropar)*/
/**************************************************************/

void VisorWin::apropar()
{
    vgl->setZoom(APROPAR);
}

/*****************************************************/
/*   void VisorWin::desaImatge()                     */
/*Operació executada en el moment de captura d'imatge*/
/*****************************************************/

void VisorWin::desaImatge()
{
    QImageDialog* imadiag  = new QImageDialog( this );

    if ( imadiag->exec() == QDialog::Accepted )
    {
       if ( imadiag->getNom() != QString::null )
       {
         (vgl->grabFrameBuffer()).save( imadiag->getNom(), (const char *)(imadiag->getFormat()), 100 );
       }
    }
}

/***************************************************************/
/*   void VisorWin::obteNivellsIMostra()                       */
/*Configuració de les corbes de nivell que es mostraran en 2.5D*/
/***************************************************************/

void VisorWin::obteNivellsIMostra()
{
   //qDebug("ObtenirNivellsIMostra");
   ConfigCorbesDialeg *ccd = new ConfigCorbesDialeg( this );
   ccd->setNivells( nivs );
   //ccd->setMaximNivell( value );
   if( ccd->exec() == QDialog::Accepted )
   {
     //qDebug("Estic aqui.");
     nivs = ccd->getNivells();
     vgl->modificaNivells( nivs );
     vgl->setMode( M_ISOLINIES );      
   } 
}

/*****************************************************************/
/*   void VisorWin::actualitza()                                 */
/*Comuniquem la triangulació realitzada en el model 2D al de 2.5D*/
/*****************************************************************/

void VisorWin::actualitza()
{
   vgl->updateGL();
}

/************************************************************************/
/*   VisorTerreny *VisorWin::getVisor()                                 */
/*Metode que retorna el punter al visualitzador 2.5d, es a dir, l'openGL*/
/************************************************************************/

VisorTerreny *VisorWin::getVisor()
{
   return vgl;
}

/**************************************************************************************/
/*   void VisorWin::configuraVisor()                                                  */
/*Opcio a partir de la qual podem configurar els colors amb que es desitja visualitzar*/
/**************************************************************************************/

void VisorWin::configuraVisor()
{
    ConfigVisorDialeg *confd = new ConfigVisorDialeg( this );
   // confd->setResolucioMax( terreny->obtenirResolucioMax() );
    confd->setResolucioMax( 10 ); //vgl->getResolucioMax() ); 
    confd->setDades( prefs );
    if ( confd->exec() == QDialog::Accepted )
    {
       prefs = confd->dades();
     
       vgl->modificaColors( prefs.cfons3d, prefs.cmodel3d, prefs.cquadre3d,
                            prefs.ceix_x, prefs.ceix_y, prefs.ceix_z );
       
       vgl->modificaResolucio( prefs.resolucio );
    }   
}

/*****************************************************/
/*   void VisorWin::mostraTerreny()                  */
/*Metode que indica el tipus de mode de visualització*/
/*****************************************************/

void VisorWin::mostraTerreny()
{
  vgl->setMode( M_TERRENY );
}

/**********************************************************************************************************************/
/*   void VisorWin::calculVisio(bool b )                                                                              */
/*Metode que crida al metode que calcula la visibilitat des dels elements de visio sobre el terreny d'una altre manera*/
/**********************************************************************************************************************/

void VisorWin::calculVisioDiscretitzant( )
{
	//vgl->setCalculaVisio(true);
	vgl->prepararFinestresPintar();
// 	vgl->calcula_visio( true );
}

/**********************************************************************************************************************/
/*   void VisorWin::calculVisioVariant(bool b )                                                                       */
/*Metode que crida al metode que calcula la visibilitat des dels elements de visio sobre el terreny d'una altre manera*/
/**********************************************************************************************************************/

void VisorWin::calculVisio( )
{
	vgl->setCalculaVisio(false);
// 	vgl->calcula_visio( false );
}
