#include "zdialeg.h"

#include <qvariant.h>
#include <qslider.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qvalidator.h>
#include <qmessagebox.h>

/****************************************************************************************/
/*   ZDialeg::ZDialeg( QWidget* parent, const char* name, bool modal, WFlags fl )       */
/*    : QDialog( parent, name, modal, fl )                                              */
/*Constructor de la classe, el qual crea la finestra amb 1 Slider, 1 TextBox i 2 Buttons*/
/****************************************************************************************/

ZDialeg::ZDialeg( QWidget* parent, const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    if ( !name ) setName( "ZDialeg" );

    layout1 = new QHBoxLayout( this, 10); 

    slider1 = new QSlider( this );
    slider1->setOrientation( QSlider::Vertical );
    slider1->setMinValue( 0 );
    slider1->setMaxValue( 20000 );
    slider1->setValue( 20000 );
    layout1->addWidget( slider1 );

    layout2 = new QVBoxLayout( layout1, 50 ); 

    lineEdit1 = new QLineEdit( this, "lineEdit1" );
    lineEdit1->setValidator( new QDoubleValidator( 0.0, 20.0, 3, lineEdit1 ) );
    layout2->addWidget( lineEdit1 );

    layout3 = new QHBoxLayout( layout2, 10 ); 

    buttonOk = new QPushButton( this, "buttonOk" );
    layout3->addWidget( buttonOk );

    buttonCancel = new QPushButton( this, "buttonCancel" );
    layout3->addWidget( buttonCancel );

    languageChange();
    resize( sizeHint() );
    clearWState( WState_Polished );

    _alcada = 0.0;
    blocked = FALSE;

    // signals and slots connections
    connect( buttonOk, SIGNAL( clicked() ), this, SLOT( acceptat() ) );
    connect( buttonCancel, SIGNAL( clicked() ), this, SLOT( reject() ) );
    connect( lineEdit1, SIGNAL( textChanged(const QString&) ), this, SLOT( canviaValor(const QString&) ) );
    connect( slider1, SIGNAL( valueChanged(int) ), this, SLOT( canviaBarra(int) ) );
}

/*********************************************************************************/
/*   ZDialeg::~ZDialeg()                                                         */
/*Destructor de la classe que no necesita esborrar res ja que no hi ha cap punter*/
/*********************************************************************************/

ZDialeg::~ZDialeg() {}

/*****************************************************/
/*   void ZDialeg::languageChange()                  */
/*Metode que permet modificar el llenguatge utilitzat*/
/*****************************************************/

void ZDialeg::languageChange()
{
    setCaption( trUtf8( "\x41\x6c\xc3\xa7\x61\x64\x61" ) );
    lineEdit1->setText( tr( "0.0" ) );
    buttonOk->setText( tr( "Accepta" ) );
    buttonCancel->setText( trUtf8( "\x43\x61\x6e\x63\x65\x6c\xc2\xb7\x6c\x61" ) );
}

/*******************************************************/
/*   void ZDialeg::canviaValor(const QString& t)       */
/*Metode que va variant el valor Slider segons d'alcada*/
/*******************************************************/

void ZDialeg::canviaValor(const QString& t)
{
   double valor;
   int p = 0;
   QString s;
   bool ok;

   if ( blocked ) 
   {
      blocked = FALSE;
      return;
   }
   
   s = t;
   if ( lineEdit1->validator()->validate(s, p) == QValidator::Acceptable ) 
   {
      valor = t.toDouble(&ok);
      if ( ok )
      {
        blocked = TRUE;
        slider1->setValue( slider1->maxValue() - (int)(valor * 1000.0) );
      }
   }
}

/*******************************************************/
/*   void ZDialeg::canviaBarra(int)                    */
/*Metode que va variant el valor d'alcada segons Slider*/
/*******************************************************/

void ZDialeg::canviaBarra(int)
{
   if ( blocked ) 
   {
      blocked = FALSE;
      return;
   }

   blocked = TRUE;
   lineEdit1->setText( QString( "%1" ).arg( (double)(( slider1->maxValue() - slider1->value()) / 1000.0) ) );
}

/**********************************************/
/*   void ZDialeg::acceptat()                 */
/*Metode que comprova l'alçada abans de sortir*/
/**********************************************/

void ZDialeg::acceptat()
{
   double valor;
   int p = 0;
   QString t;
   bool ok;

   t = lineEdit1->text();

   if ( lineEdit1->validator()->validate(t, p) == QValidator::Acceptable ) 
   {
      valor = lineEdit1->text().toDouble(&ok);
      if ( ok )
      {
         _alcada = valor;
      }
      accept();
   }
   else
   {
      QMessageBox::warning( this, "modelat de terrenys", 
                                  QString("Valor incorrecte. \n[0.0 - %1].").arg( (double)(slider1->maxValue() / 1000.0) ),
                                  "Accepta",   QString::null, QString::null, 0 ); 
   }
}

/***************************************/
/*   double ZDialeg::alcada()          */
/*Metode que retorna el valor d'alçada */
/***************************************/

double ZDialeg::alcada()
{
   //qWarning( "ALCADA = %f", _alcada );
   return _alcada;
}

/**************************************************************************/
/*   void ZDialeg::limits( double linf, double lsup )                     */
/*Metode que assigna els valors maxims i minims que pot prendre el Slider */
/**************************************************************************/

void ZDialeg::limits( double linf, double lsup )
{
   slider1->setMinValue( (int)( linf * 1000.0 ) );
   slider1->setMaxValue( (int)( lsup * 1000.0 ) );
   slider1->setValue( (int)( lsup * 1000.0 ) );
   lineEdit1->setValidator( 0 );  /* Elimina el validador anterior */
   lineEdit1->setValidator( new QDoubleValidator( linf, lsup, 3, lineEdit1 ) );   
}

/******************************************************************************/
/*   void ZDialeg::per_defecte( double valor )                                */
/*Metode que assigna el valor d'alcada actual del punt al TextBox i al Slider */
/******************************************************************************/

void ZDialeg::per_defecte( double valor )
{
   lineEdit1->setText( QString( "%1" ).arg( (double)(( slider1->maxValue() - valor ) / 1000.0) ) );
   slider1->setValue( slider1->maxValue() - (int)(valor * 1000.0) );
}
