#include "editorterreny.h"
#include <qfile.h>
#include <qcursor.h>
#include <algorithm>
#include <cstdlib>
#include <ctime>
#include <iostream>
#include "infodialeg.h"

#define BUFSIZE 512

/*****************************************************************************************************/
/*   EditorTerreny::EditorTerreny(QWidget *parent, const char *name ) : QGLWidget( parent, name )    */
/*Constructor que inicialitza cadascuna de les variables globals de l'objecte EditorTerreny creat    */
/*****************************************************************************************************/

EditorTerreny::EditorTerreny(QWidget *parent, const char *name )
     : QGLWidget( parent, name )  
{ 

	info_mon.xmin = info_mon.ymin = info_mon.zmin = 0.0;
        info_mon.xmax = info_mon.ymax = info_mon.zmax = 20.0;
	info_mon._modificat = FALSE;   
	
	info_mostrar.dibuixa = TRUE;
        info_mostrar.seleccionat = 0;
	info_mostrar.veure_elements_visio = FALSE;
	info_mostrar.veure_corbes = FALSE;
        info_mostrar.veure_triangulacio = TRUE;
        info_mostrar.veure_vertexs = TRUE;
	info_mostrar.veure_parts_visibles = FALSE;
	info_mostrar.comprovar_visibilitat=FALSE;
	info_mostrar.veure_dibuix=FALSE;
	info_mostrar.propagacio=TRUE;
	info_mostrar.repintar_finestres=FALSE;
	info_mostrar.veure_diagrama_voronoi=FALSE;
	info_mostrar.pintar_cami=FALSE;
	pressed = FALSE;	

	info_calculs.mode_visibilitat=TRUE;
	info_calculs.mode_distancies=FALSE;
	info_calculs.punt_distancies=FALSE;
	
	/*Colors per defecte del fons, dels punts i de les linies*/
        info_GL.color_fons = Qt::black;
        info_GL.color_punts = Qt::green;
        info_GL.color_linies = Qt::blue;
        info_GL.color_seleccio = Qt::red;
	info_GL.color_skinny.setRgb(150,20,100);

//         vertexs.clear();
       
	/*Inicialització de la triangulació de Delaunay a buit, utilitzant el Singleton creat*/
        mesh = Terreny::getInstancia(); 

	// Visibilitat
	info_vis.visio = INDEFINIT;
	info_vis.punt_segment=false;
	info_vis.punts_pla_poligon = false;
	info_vis.dibuixant_poligon = false;
	info_vis.tip = INDEFINIT;
	
	info_vis.text.xminT = info_vis.text.yminT = info_vis.text.zminT = 0.0;
        info_vis.text.xmaxT = info_vis.text.ymaxT = info_vis.text.zmaxT = 20.0;	
	
	info_vis.gh.context = cgCreateContext();
	info_vis.gh.haveLoadedPrograms = false;
	
	info_vis.mostrar.tipus_anterior = UNIO;
	info_vis.mostrar.tipus_visualitzacio = UNIO;
	info_vis.mostrar.tercer_segment_visio = 0;
	
	info_vis.text.amplada=width();
	info_vis.text.alcada=height();
	
	info_vis.text.amplada=500;
	info_vis.text.alcada=500;
	cout<<"mida textura "<<info_vis.text.amplada<<" x "<<info_vis.text.alcada<<"."<<endl;
	
	objectes_visio=Visio::instance();
	objectes_distancies=Visio::instance();
	objectes_cami_minim=Visio::instance();
	objectes_visio->inicialitza_mides(info_vis.text.amplada, info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
	objectes_distancies->inicialitza_mides(info_vis.text.amplada, info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
	objectes_cami_minim->inicialitza_mides(info_vis.text.amplada, info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
	
        accio = GL_RENDER;

	/*Menú desplegable que es pot observar en el moment en que es selecciona un punt ja creat i es clica sobre ell amb el botó dret. Aquest permet modificar i observar les dades del punt triat*/
        auxmenu = new QPopupMenu( this );
        auxmenu->insertItem( "Modifica Altura",  this, SLOT(modificaZ()) );        
        auxmenu->insertItem( "Elimina element",  this, SLOT(borraElement()) );
        auxmenu->insertItem( "Informacio", this, SLOT(mostraInfo()) );   

	setMouseTracking( TRUE );
	
	mesh->setAmpladaGraella(info_vis.text.amplada);
}

/*********************************************/
/*   EditorTerreny::~EditorTerreny() {}      */
/*Destructor de la classe de EditorTerreny 2D*/
/********************************************/

EditorTerreny::~EditorTerreny() 
{
// 	if(mesh) delete mesh;
// 	for(int i=0;i<objectes_visio->numeroObjectes();i++)
// 		objectes_visio->clear();
	if(objectes_visio) delete objectes_visio;
	if(objectes_distancies) delete objectes_distancies;
	if(objectes_cami_minim) delete objectes_cami_minim;
}

/***************************************************/
/*   void EditorTerreny::initializeGL()            */
/*Mètode que permet inicialitzar la finestra OpenGL*/
/***************************************************/

void EditorTerreny::initializeGL()
{
    qglClearColor( info_GL.color_fons );
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glShadeModel( GL_FLAT );
    glEnable( GL_BLEND );
    glBlendFunc( GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA );    
    glEnable( GL_POINT_SMOOTH );
    glHint( GL_POINT_SMOOTH_HINT, GL_FASTEST );
    glEnable( GL_LINE_SMOOTH );
    glHint( GL_LINE_SMOOTH_HINT, GL_FASTEST );
}

/***************************************************************************************/
/*   void EditorTerreny::inicialitzaCG()                                               */
/*Mètode que permet inicialitzar les variables de CG per poder treballar amb la textura*/
/***************************************************************************************/

void EditorTerreny::inicialitzaCG()
{
	ChooseProfiles();
        LoadCgPrograms();
       
	cgGLEnableProfile(info_vis.gh.fragmentProfile);

	cgGLBindProgram(info_vis.gh.fragmentProgram);
	info_vis.gh.haveLoadedPrograms = true;

	glDisable(GL_DITHER);
	glDisable(GL_BLEND);
	//glEnable(GL_SCISSOR_TEST);
 	glDisable(GL_DEPTH_TEST);
	//glEnable(GL_DEPTH_TEST);
	
	glEnable(GL_COLOR_LOGIC_OP);
	glEnable(GL_LOGIC_OP);
	glLogicOp(GL_OR);

// 	glEnable(GL_TEXTURE_RECTANGLE_ARB);
// 	glGenTextures(1,&info_vis.gh.texID);
// 	glBindTexture(GL_TEXTURE_RECTANGLE_ARB,info_vis.gh.texID);

// 	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
// 	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
// 	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB,GL_TEXTURE_WRAP_S,GL_CLAMP);
// 	glTexParameteri(GL_TEXTURE_RECTANGLE_ARB,GL_TEXTURE_WRAP_T,GL_CLAMP);

	glEnable(GL_TEXTURE_2D);
	glGenTextures(1,&info_vis.gh.texID);
	glBindTexture(GL_TEXTURE_2D,info_vis.gh.texID);

	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S,GL_CLAMP);
	glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T,GL_CLAMP);


}

/********************************************************************************************/
/*   void EditorTerreny::LoadCgPrograms()                                                   */
/*Metode que segons el tipus de visualitzacio escollit obrira un fragment program o un altre*/
/********************************************************************************************/

void EditorTerreny::LoadCgPrograms()
{
	//    assert(cgIsContext(info_vis.gh.context));
	qDebug("Tipus visualitzacio: %d",info_vis.mostrar.tipus_visualitzacio);
	switch(info_vis.mostrar.tipus_visualitzacio)
	{
		case UNIO:
			info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "fragment_textures_unio.cg", info_vis.gh.fragmentProfile, NULL, NULL);
			break;
		case INTERSECCIO:
			info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "fragment_textures_interseccio.cg", info_vis.gh.fragmentProfile, NULL, NULL);
			break;
		case COUNTING:
			info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "fragment_textures_counting.cg", info_vis.gh.fragmentProfile, NULL, NULL);
			break;
		case SEGMENT:
			info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "fragment_textures_segment.cg", info_vis.gh.fragmentProfile, NULL, NULL);
			break;
		case COMBINAT:
			info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "fragment_textures_segment.cg", info_vis.gh.fragmentProfile, NULL, NULL);
			break;
	}
	if (!cgIsProgramCompiled(info_vis.gh.fragmentProgram))
	{
		cgCompileProgram(info_vis.gh.fragmentProgram);
	}
	cgGLEnableProfile(info_vis.gh.fragmentProfile);
	cgGLLoadProgram(info_vis.gh.fragmentProgram);
}

/********************************************************************/
/*   void EditorTerreny::ChooseProfiles()                           */
/*Metode que permet escollir el tipus del fragment Profile utilitzat*/
/********************************************************************/

void EditorTerreny::ChooseProfiles()
{
	// Make sure that the appropriate profiles are available on the
	// user's system.
	
	if (cgGLIsProfileSupported(CG_PROFILE_FP30))
	{
		info_vis.gh.fragmentProfile = CG_PROFILE_FP30;
	}
        else 
	{
		if (cgGLIsProfileSupported(CG_PROFILE_FP20))
			info_vis.gh.fragmentProfile = CG_PROFILE_FP20;
		else {
			fprintf(stderr, "Neither arbfp1 or fp30 or fp20 fragment profiles supported on this system.\n");
			exit(1);
		}
	}
}

/*********************************************************************/
/*   void EditorTerreny::handleCgError()                             */
/*Metode que serveix per indicar el tipus d'error existent amb les CG*/
/*********************************************************************/

void EditorTerreny::handleCgError() 
{
    fprintf(stderr, "Cg error: %s\n", cgGetErrorString(cgGetError()));
    exit(1);
}

/***********************************************************************************/
/*   void EditorTerreny::resizeGL( int w, int h )                                  */
/*Metode executat per defecte cada vegada que es redimensiona la finestra de dibuix*/
/***********************************************************************************/

void EditorTerreny::resizeGL( int w, int h )
{
    glViewport( 0, 0, (GLint)w, (GLint)h );
    
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -1.0, 1.0 );   
    glMatrixMode(GL_MODELVIEW);
    
    objectes_visio->inicialitza_mides(this->width(), this->height(), info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
    objectes_distancies->inicialitza_mides(this->width(), this->height(), info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
    objectes_cami_minim->inicialitza_mides(this->width(), this->height(), info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
    
    glLoadIdentity();
}
// 
/***********************************************************************************/
/*   void EditorTerreny::resizeGraella( int w, int h )                                  */
/*Metode que permet modificar la mida de la graella que cobreix el domini on es guarda la informaico de visibilitat. (textura)*/
/***********************************************************************************/

void EditorTerreny::modificaMidaGraella( int w, int h )
{
	if(info_vis.text.amplada!=w || info_vis.text.alcada!=h)
	{
		info_vis.text.amplada=w;
		info_vis.text.alcada=h;
		mesh->setAmpladaGraella(info_vis.text.amplada);
		objectes_visio->setMidesGraella(info_vis.text.amplada,info_vis.text.alcada);
		objectes_distancies->setMidesGraella(info_vis.text.amplada,info_vis.text.alcada);
		objectes_cami_minim->setMidesGraella(info_vis.text.amplada,info_vis.text.alcada);
	}
}

/*******************************************************************************************************************/
/*   void EditorTerreny::modificaMida( double limits[6] )                                                          */
/*Mètode que permet modificar les mides del terreny indicant els limits de coordenades redimensionant la superfície*/
/*******************************************************************************************************************/

void EditorTerreny::modificaMida( double limits[6] )
{
    makeCurrent();

    info_mon.xmin = limits[0] ;
    info_mon.ymin = limits[1] ; 
    info_mon.zmin = limits[2] ;
    info_mon.xmax = limits[3] ;  
    info_mon.ymax = limits[4] ;
    info_mon.zmax = limits[5] ;
    
//     info_mostrar.veure_parts_visibles = false;
    
    objectes_visio->inicialitza_mides(info_vis.text.amplada,info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
    objectes_distancies->inicialitza_mides(info_vis.text.amplada,info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);
    objectes_cami_minim->inicialitza_mides(info_vis.text.amplada,info_vis.text.alcada, info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    glOrtho( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -1.0, 1.0 );
    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();    
    
    updateGL();
}

/***************************************************************************************************************/
/*   void EditorTerreny::modificaColors( QColor colors[4] )                                                    */
/*Mètode que permet modificar els colors de: fons, punts, linies i seleccio a través d'una taula de 4 posicions*/
/***************************************************************************************************************/

void EditorTerreny::modificaColors( QColor colors[4] )
{
    info_GL.color_fons = colors[0];    
    info_GL.color_punts = colors[1];
    info_GL.color_linies = colors[2];
    info_GL.color_seleccio = colors[3];

    updateGL();
}

/******************************************************************************************************/
/*   void EditorTerreny::paintGL()                                                                    */
/*Mètode que simplement permet visualitzar el terreny en 2D amb cadascuna de les seves particularitats*/
/******************************************************************************************************/

void EditorTerreny::paintGL()
{
	if(info_mostrar.veure_dibuix)
	{ 
		//dibuixemQ();
		dibuixemFinestres();
	}
	else
	{ 
		qglClearColor( info_GL.color_fons );
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
	
		if (info_mostrar.repintar_finestres)
		{
			repintarFinestres();
		}
		else
		{
				
		qglClearColor( info_GL.color_fons );
		glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT ); 
		
		if(!info_mostrar.dibuixa)
		{
			info_mostrar.dibuixa = TRUE;
			return;
		}
		if ( info_mostrar.veure_parts_visibles ) mostarResultatsVisibilitat();
		
		glDisable(GL_LOGIC_OP);
		glDisable(GL_COLOR_LOGIC_OP);
				
		if(info_mostrar.comprovar_visibilitat) comprovarResultatVisibilitat();
		
		if(info_mostrar.veure_diagrama_voronoi) mostrarDiagramaVoronoi();
		
		if(info_mostrar.pintar_cami) calculsCaminsMinims();
		
		if( info_mostrar.veure_triangulacio ) dibuixaTriangulacio2D( info_GL.color_linies );
		
		if( info_mostrar.veure_vertexs ) 
		{
			dibuixaPunts( GL_RENDER, info_GL.color_punts );
			if( info_mostrar.seleccionat > 0  ) marcaSeleccionat();   
		}
		if( info_mostrar.veure_corbes ) dibuixaCorbes();
		
		if( ( info_vis.visio!=INDEFINIT || info_mostrar.veure_elements_visio ) && objectes_visio->numeroObjectes()!=0 ) 
			dibuixarObjectesVisio();
	
		dibuixarElementVisio();
		terrenyModificat();
		}
	}

}


void EditorTerreny::mostarResultatsVisibilitat()
{
	bool modificada=objectes_visio->textura_Modificada();
	if(modificada)
	{
		info_vis.text.xminT=info_mon.xmin;
		info_vis.text.xmaxT=info_mon.xmax;
		info_vis.text.yminT=info_mon.ymin;
		info_vis.text.ymaxT=info_mon.ymax;
		modificada=true;
	}
	info_vis.text.textura = objectes_visio->obtenir_Textura();
// 		GLubyte * prova = new GLubyte[4*this->width()*this->height()];

	if(info_vis.text.textura!=NULL)
	{
		if (!info_vis.gh.haveLoadedPrograms || info_vis.mostrar.tipus_visualitzacio != info_vis.mostrar.tipus_anterior) 
		{
			inicialitzaCG();
			info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
			assignarParametres();
			
			modificada=true;
		}
		else
		{
			cgGLEnableProfile(info_vis.gh.fragmentProfile);
		}
		
		if(modificada) construirTextura();
		
		vector<int> combinat;
		int n=objectes_visio->numeroObjectes();
		switch(info_vis.mostrar.tipus_visualitzacio)
		{
			case SEGMENT:
				cgGLSetParameter1f(info_vis.text.numero_segments, (float)info_vis.mostrar.segment_escollit);
				break;
			case COMBINAT:
				combinat.push_back(info_vis.mostrar.primer_segment_visio);
				combinat.push_back(info_vis.mostrar.segon_segment_visio);
				if(info_vis.mostrar.tercer_segment_visio!=0)
					combinat.push_back(info_vis.mostrar.tercer_segment_visio);
				break;
			case INTERSECCIO:
				if((n-1)/8==0)
				{	
					 cgGLSetParameter4f(info_vis.text.numeros_esperats,((1<<(n%8))-1)/255.,0,0,0);
					 cout<<(1<<(n%8))-1<<", "<<0<<", "<<0<<", "<<0<<endl;
				}
				else if((n-1)/8==1)
				{	
					  cgGLSetParameter4f(info_vis.text.numeros_esperats,1,((1<<(n%8))-1)/255.,0,0);
				 	cout<<255<<", "<<(1<<(n%8))-1<<", "<<0<<", "<<0<<endl;
				}
				else if((n-1)/8==2)
				{	
					  cgGLSetParameter4f(info_vis.text.numeros_esperats,1,1,((1<<(n%8))-1)/255.,0);
				 	cout<<255<<", "<<255<<(1<<(n%8))-1<<", "<<0<<endl;
				}
				else
				{	
					  cgGLSetParameter4f(info_vis.text.numeros_esperats,1,1,1,((1<<(n%8))-1)/255.);
				 	cout<<255<<", "<<255<<", "<<255<<", "<<(1<<(n%8))-1<<endl;
				}
				break;
			default:
				if(info_vis.mostrar.tipus_visualitzacio!=UNIO)
				{
					cgGLSetParameter1f(info_vis.text.numero_segments, (float)n/32.);
				}
				break;
		}
// 		if(!info_mostrar.comprovar_visibilitat) 
// 			mesh->settipusInterseccio(INDEFINIT);

		glEnable(GL_COLOR_LOGIC_OP);
		glEnable(GL_LOGIC_OP);
		glLogicOp(GL_OR);

		if(info_vis.mostrar.tipus_visualitzacio!=COMBINAT)
		{
			glColor3f( 1.0, 0.0, 0.0 );
			glBegin(GL_QUADS);
				glTexCoord2f(0,0);
				glVertex2f( info_vis.text.xminT,info_vis.text.yminT);    
			
//  				glTexCoord2f(0,this->height()); //cas textura rectangular.
				glTexCoord2f(0,1);
				glVertex2f( info_vis.text.xminT, info_vis.text.ymaxT );  
	
// 				glTexCoord2f(this->width(),this->height()); //cas textura rectangular.
				glTexCoord2f(1,1);
				glVertex2f( info_vis.text.xmaxT,info_vis.text.ymaxT );    
			
// 				glTexCoord2f(this->width(),0); //cas textura rectangular.
				glTexCoord2f(1,0);
				glVertex2f( info_vis.text.xmaxT,info_vis.text.yminT);  
			glEnd();
		}
		else
		{
			for(unsigned int k=0;k<combinat.size();k++)
			{
// 				cout<<"COMBINAT: "<<combinat[k]<<endl;
				cgGLSetParameter1f(info_vis.text.numero_segments, (float)combinat[k]);
				switch(k)
				{
					case 0: glColor3f( 1.0, 0.0, 0.0 );
						break;
					case 1: glColor3f( 0.0, 1.0, 0.0 );
						break;
					case 2: glColor3f( 0.0, 0.0, 1.0 );
						break;
				}
// 				cout<<"canviat"<<endl;
				glBegin(GL_QUADS);
					glTexCoord2f(0,0);
					glVertex2f( info_vis.text.xminT,info_vis.text.yminT);    
			
					glTexCoord2f(0,1);
					glVertex2f( info_vis.text.xminT, info_vis.text.ymaxT );  
	
					glTexCoord2f(1,1);
					glVertex2f( info_vis.text.xmaxT,info_vis.text.ymaxT );    
			
					glTexCoord2f(1,0);
					glVertex2f( info_vis.text.xmaxT,info_vis.text.yminT);  
				glEnd();
			}
		}
		
		cgGLDisableProfile(info_vis.gh.fragmentProfile);
		glDisable(GL_TEXTURE_2D);
// 		glDisable(GL_TEXTURE_RECTANGLE_ARB);
	}
	else
	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Punt de visio incorrecte" ));
		errors->message( "El punt introduit es troba fora o per sota de la triangulacio! Cal que es trobi per sobre d'aquesta."); 
	}

}

void EditorTerreny::comprovarResultatVisibilitat()
{
	vector<pCara> cares;
	Punt * p;
	pCara tri;
	int vertex;
	Localitzacio tipus;
	Triangle trpv;
	pCara triangle_pv;
	double x=info_vis.punt_comprovar.x(),y=info_vis.punt_comprovar.y();
	
	switch(objectes_visio->obtenirObjecte(info_vis.mostrar.segment_escollit)->tipusObjecte())
	{
		case PUNT_VISIO:
			/*Crear un segment entre el punt de visio i el nou punt introduit i comprovar si interseca algun triangle de la malla*/
			tri = mesh->triangulacio()->locate(info_vis.punt_comprovar.getPunt(), tipus, vertex);
			p = (Punt *)objectes_visio->obtenirObjecte(info_vis.mostrar.segment_escollit);
			info_vis.raig = Segment((Pla(Triangle(tri).getTriangle_cgal().supporting_plane()).puntSobrePla(x, y, Punt(tri->vertex(0)->point())) ), Punt(p->x(),p->y(),p->z()));
			
			cares = mesh->getCares();
			info_vis.triangles_inter.clear();
			
			for(unsigned int i=0;i<cares.size();i++)
				if(tri != cares[i])
					if(Triangle(cares[i]).interseccio(info_vis.raig,false,false,false))
				info_vis.triangles_inter.push_back(cares[i]);
// 			mesh->calculaEntrades(*p,info_vis.triangle_pv);
// 			info_vis.triangles_inter=mesh->provant_interseccio(tri,triangle_pv,*p);
			mesh->setCaresIntersecciona(info_vis.triangles_inter);
			
// 			if(info_vis.triangles_inter.size()>0)
// 			{
// 				errors = new QErrorMessage( this );
// 				errors->setCaption( tr( "INTERSECCIO PUNT" ));
// 				errors->message( "Punt intersecciona triangulacio"); 
// 			}
			mesh->setSegmentIntersecciona(info_vis.raig);
			mesh->settipusInterseccio(PUNT_VISIO);
			info_vis.tip = PUNT_VISIO;
			break;
		case SEGMENT_VISIO:
			tri = mesh->triangulacio()->locate(info_vis.punt_comprovar.getPunt(), tipus, vertex);
			info_vis.triangle = Triangle((Pla(Triangle(tri).getTriangle_cgal().supporting_plane()).puntSobrePla(x, y, Punt(tri->vertex(0)->point())) ), ((Segment *)objectes_visio->obtenirObjecte(info_vis.mostrar.segment_escollit))->getOrigen(), ((Segment *)objectes_visio->obtenirObjecte(info_vis.mostrar.segment_escollit))->getDesti());
			
			cares = mesh->getCares();
			info_vis.triangles_inter.clear();
			cout<<"TRIANGLES INTERSECCIONEN"<<endl;
			for(unsigned int i=0;i<cares.size();i++)
				if(tri != cares[i])
				{
					if(Triangle(cares[i]).interseccio(info_vis.triangle,false,false))
					{
						info_vis.triangles_inter.push_back(cares[i]);
						cout<<"index: "<<i<<" "; 
						Triangle triaaaa(cares[i]);
						triaaaa.escriure();
					}
				}
				else 
				{
					cout<<"TRIANGLE ACTUAL -- index: "<<i<<endl;
				}			
//			comprovacio de que funciona el tirangles_intersequen_poliedre.
			Segment *sv=(Segment*)objectes_visio->obtenirObjecte(info_vis.mostrar.segment_escollit);
// 			mesh->calculaEntrades(Punt(sv->punt_mig_3d()),triangle_pv);
// 			triangle_pv = mesh->triangulacio()->locate(sv->punt_mig_3d(), tipus, vertex);
// 			info_vis.triangles_inter=mesh->triangles_intersequen_paralelapipede(tri,triangle_pv,*sv);
// 			info_vis.triangles_inter=mesh->provant_interseccio(tri,triangle_pv,info_vis.triangle);
		
			mesh->setCaresIntersecciona(info_vis.triangles_inter);
/*			if(triangles_inter.size()>0)
			{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "INTERSECCIO SEGMENT" ));
				errors->message( "Segment intersecciona triangulacio"); 
			}*/
			mesh->setTriangleIntersecciona(info_vis.triangle);	
			mesh->settipusInterseccio(SEGMENT_VISIO);
			
			info_vis.tip = SEGMENT_VISIO;
			break;
	}

	for(unsigned int i=0;i<info_vis.triangles_inter.size();i++)
	{
		glColor3f( 0.1, 0.1, 0.8 );
		glBegin( GL_TRIANGLES );
			glVertex2f(info_vis.triangles_inter[i]->vertex(0)->point().x(), info_vis.triangles_inter[i]->vertex(0)->point().y());
			glVertex2f(info_vis.triangles_inter[i]->vertex(1)->point().x(), info_vis.triangles_inter[i]->vertex(1)->point().y());
			glVertex2f(info_vis.triangles_inter[i]->vertex(2)->point().x(), info_vis.triangles_inter[i]->vertex(2)->point().y());
		glEnd();
	}
	switch(info_vis.tip)
	{
		case PUNT_VISIO:
			glColor3f( 0.1, 0.8, 0.1 );
			glBegin(GL_LINES);
				glVertex2f( info_vis.raig.getOrigen().x(),info_vis.raig.getOrigen().y());
				glVertex2f( info_vis.raig.getDesti().x(),info_vis.raig.getDesti().y());
			glEnd();
			
			break;
		case SEGMENT_VISIO:
			glColor3f( 0.1, 0.8, 0.1 );
			glBegin( GL_TRIANGLES );
				glVertex2f(info_vis.triangle.getTriangle_cgal().vertex(0).x(), info_vis.triangle.getTriangle_cgal().vertex(0).y());
				glVertex2f(info_vis.triangle.getTriangle_cgal().vertex(1).x(), info_vis.triangle.getTriangle_cgal().vertex(1).y());
				glVertex2f(info_vis.triangle.getTriangle_cgal().vertex(2).x(), info_vis.triangle.getTriangle_cgal().vertex(2).y());
			glEnd();
			break;
	}	
}

/**************************************************************************************************************/
/*   void EditorTerreny::assignarParametres()                                                                 */
/*Mètode que en funció del tipus de visualitzacio escollida passara uns parametres o altres al fragment shader*/
/**************************************************************************************************************/

void EditorTerreny::assignarParametres()
{
	switch(info_vis.mostrar.tipus_visualitzacio)
	{
		case SEGMENT:
			info_vis.text.numero_segments=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numero_segments");
// 			qDebug("PASSEM PARAMETRE SEGMENT ESCOLLIT %f",(float)info_vis.mostrar.segment_escollit);
// 			cgGLSetParameter1f(info_vis.text.numero_segments, (float)info_vis.mostrar.segment_escollit);
			break;
		case COMBINAT:
			info_vis.text.numero_segments=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numero_segments");
// 			info_vis.text.numero_segments=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numero_segments1");
// 			info_vis.text.numero_segments=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numero_segments2");
// 			cgGLSetParameter1f(info_vis.text.numero_segments, (float)(info_vis.mostrar.primer_segment_visio-1));
// 			cgGLSetParameter1f(info_vis.text.numero_segments, (float)(info_vis.mostrar.segon_segment_visio-1));
			break;
		case INTERSECCIO:
			info_vis.text.numeros_esperats=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numeros_esperats");
			break;
		default:
			if(info_vis.mostrar.tipus_visualitzacio!=UNIO)
			{
				info_vis.text.numero_segments=cgGetNamedParameter(info_vis.gh.fragmentProgram, "numero_segments");
				cgGLSetParameter1f(info_vis.text.numero_segments, (float)objectes_visio->numeroObjectes());
			}
			break;
	}
}

/*******************************************************************************************************/
/*   void EditorTerreny::construirTextura()                                                            */
/*Mètode que permet realitzar les crides de CG per crear la textura amb les mides i el vector de floats*/
/*******************************************************************************************************/

void EditorTerreny::construirTextura()
{
 //    cas textura rectangular.	
// 	glTexImage2D(GL_TEXTURE_RECTANGLE_ARB,0,GL_RGBA,this->height(),this->width(),0,GL_RGBA,GL_FLOAT,0);
// 	glTexImage2D(GL_TEXTURE_RECTANGLE_ARB,0,GL_RGBA,this->height(),this->width(),0,GL_RGBA,GL_UNSIGNED_BYTE,0);
// 	int error=glGetError();
// 	if(error!=GL_NO_ERROR) cout<<gluErrorString(error)<<endl;
// 
// 	glTexSubImage2D(GL_TEXTURE_RECTANGLE_ARB,0,0,0,this->height(),this->width(),GL_RGBA,GL_UNSIGNED_BYTE,info_vis.text.textura);
// 	error=glGetError();
// 	if(error!=GL_NO_ERROR) cout<<"Error en el subimage: "<<gluErrorString(error)<<endl;
// 	glBindTexture(GL_TEXTURE_RECTANGLE_ARB,info_vis.gh.texID);

// textura 2d
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,info_vis.text.alcada,info_vis.text.amplada,0,GL_RGBA,GL_FLOAT,0);
	glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,info_vis.text.alcada,info_vis.text.amplada,0,GL_RGBA,GL_UNSIGNED_BYTE,0);
	int error=glGetError();
	if(error!=GL_NO_ERROR) cout<<gluErrorString(error)<<endl;

	glTexSubImage2D(GL_TEXTURE_2D,0,0,0,info_vis.text.alcada,info_vis.text.amplada,GL_RGBA,GL_UNSIGNED_BYTE,info_vis.text.textura);
	error=glGetError();
	if(error!=GL_NO_ERROR) cout<<"Error en el subimage: "<<gluErrorString(error)<<endl;
	glBindTexture(GL_TEXTURE_2D,info_vis.gh.texID);
	
}

/*****************************************************************************************/
/*   void EditorTerreny::dibuixarElementVisio()                                          */
/*Mètode que permet anar pintant el segment o el poligon que s'estigui editant actualment*/
/*****************************************************************************************/

void EditorTerreny::dibuixarElementVisio()
{
	glColor3f( 0.8, 0.0, 0.5);
	if(info_vis.visio == SEGMENT_VISIO && info_vis.punt_segment)
	{
		glBegin( GL_LINES );
			glVertex2f( info_vis.inici_segment.x(), info_vis.inici_segment.y() );      
			glVertex2f( info_vis.punt_actual.x(), info_vis.punt_actual.y() );       
		glEnd();
	}       
	else
	{
		if(info_vis.visio == POLIGON_VISIO  && info_vis.dibuixant_poligon)
		{
			glBegin(GL_LINE_LOOP);
			for(unsigned int i=0; i<info_vis.punts_poligon.size();i++)
			{
				glVertex2f( info_vis.punts_poligon[i].x(), info_vis.punts_poligon[i].y() );     
			}
			glVertex2f( info_vis.punt_actual.x(), info_vis.punt_actual.y() );      
			glEnd();
		}
		else
		{
			if(info_vis.visio == POLIGONAL_VISIO  && info_vis.dibuixant_poligon)
			{
				for(unsigned int i=0; i<info_vis.punts_poligon.size()-1;i++)
				{
					glBegin(GL_LINES);
					glVertex2f( info_vis.punts_poligon[i].x(), info_vis.punts_poligon[i].y() );     
					glVertex2f( info_vis.punts_poligon[i+1].x(), info_vis.punts_poligon[i+1].y() );     
					glEnd();
				}
				glBegin(GL_LINES);
					glVertex2f( info_vis.punts_poligon[info_vis.punts_poligon.size()-1].x(), info_vis.punts_poligon[info_vis.punts_poligon.size()-1].y() );     
					glVertex2f( info_vis.punt_actual.x(), info_vis.punt_actual.y() );      
				glEnd();
			}
		}
	}
}

/**********************************************************************************/
/*   void EditorTerreny::dibuixarObjectesVisio(  )                                */
/*Mètode que permet pintar els diferents objectes de visio introduits per l'usuari*/
/**********************************************************************************/

void EditorTerreny::dibuixarObjectesVisio( )
{
	int i,j,k;
	Objecte * obj;
	Objecte * objdist;
	Punt p;
	
	glLineWidth(2);
	
	if (info_calculs.mode_visibilitat)
	{glColor3f( 0.8, 0.0, 0.5 );}
	if (info_calculs.mode_distancies)
	{glColor3f( 0.0, 1.0, 1.0 );}
	
	for(i=0; i<objectes_visio->numeroObjectes(); i++)
	{
		if((info_vis.mostrar.tipus_visualitzacio==SEGMENT) && info_mostrar.veure_parts_visibles)
			if(i != (info_vis.mostrar.segment_escollit - 1)) glColor3f( 0.8, 0.8, 0.8 );
			else glColor3f( 0.8, 0.0, 0.5 );
			
		if((info_vis.mostrar.tipus_visualitzacio==COMBINAT) && info_mostrar.veure_parts_visibles)
			if((i == (info_vis.mostrar.primer_segment_visio-1)) || (i == (info_vis.mostrar.segon_segment_visio-1)) || (i == (info_vis.mostrar.tercer_segment_visio-1)))
				glColor3f( 0.8, 0.0, 0.5 );
			else  	glColor3f( 0.8, 0.8, 0.8 );
		
		obj = objectes_visio->obtenirObjecte(i+1);
		switch(obj->tipusObjecte())
		{
			case PUNT_VISIO: 
				glBegin( GL_POINTS );
				glVertex2f( ((Punt *)obj)->x(), ((Punt *)obj)->y() ); 
				glEnd();
				break;
			case SEGMENT_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				glBegin( GL_LINES );
					glVertex2f( ((Segment *)obj)->getOrigen().x(), ((Segment *)obj)->getOrigen().y() );      
					glVertex2f( ((Segment *)obj)->getDesti().x(), ((Segment *)obj)->getDesti().y() );       
				glEnd();
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar segments en mode distancies."); 
				}
				break;
			case POLIGON_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				glBegin( GL_LINE_LOOP );
					for(int j=0;j<((Poligon *)obj)->retorna_costats();j++)
					{
						p = ((Poligon *)obj)->obtenirVertex(j+1);
						glVertex2f( p.x(), p.y() );      
					}
				glEnd();
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar poligons en mode distancies."); 
				}
				break;
			case POLIGONAL_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				for(int j=0;j<((Poligonal *)obj)->numero_de_punts()-1;j++)
				{
					glBegin( GL_LINES );
						p = ((Poligonal *)obj)->getPunt(j+1);
						glVertex2f( p.x(), p.y() );  
						p = ((Poligonal *)obj)->getPunt(j+2);
						glVertex2f( p.x(), p.y() );    
					glEnd();
				}
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar poligonals en mode distancies."); 
				}
				break;
		}
	}
	
	for(k=0; k<objectes_distancies->numeroObjectesDistancies(); k++)
	{
		objdist = objectes_distancies->obtenirObjecteDistancies(k+1);
		switch(objdist->tipusObjecte())
		{
			case PUNT_VISIO: 
				glColor3f(1.0,1.0,0.33);
				glBegin( GL_POINTS );
				glVertex2f( ((Punt *)objdist)->x(), ((Punt *)objdist)->y() ); 
				glEnd();
				break;
			case SEGMENT_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				glBegin( GL_LINES );
					glVertex2f( ((Segment *)obj)->getOrigen().x(), ((Segment *)obj)->getOrigen().y() );      
					glVertex2f( ((Segment *)obj)->getDesti().x(), ((Segment *)obj)->getDesti().y() );       
				glEnd();
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar segments en mode distancies."); 
				}
				break;
			case POLIGON_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				glBegin( GL_LINE_LOOP );
					for(int j=0;j<((Poligon *)obj)->retorna_costats();j++)
					{
						p = ((Poligon *)obj)->obtenirVertex(j+1);
						glVertex2f( p.x(), p.y() );      
					}
				glEnd();
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar poligons en mode distancies."); 
				}
				break;
			case POLIGONAL_VISIO: 
				if (info_calculs.mode_visibilitat)
				{
				for(int j=0;j<((Poligonal *)obj)->numero_de_punts()-1;j++)
				{
					glBegin( GL_LINES );
						p = ((Poligonal *)obj)->getPunt(j+1);
						glVertex2f( p.x(), p.y() );  
						p = ((Poligonal *)obj)->getPunt(j+2);
						glVertex2f( p.x(), p.y() );    
					glEnd();
				}
				}
				else
				{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Error de visualitzacio" ));
				errors->message( "No es poden visualitzar poligonals en mode distancies."); 
				}
				break;
		}
	
	}
	
	glLineWidth(0.5);
}

/**************************************************************************************/
/*   void EditorTerreny::marcaSeleccionat()                                           */
/*Mètode que permet visualitzar el punt que ha estat seleccionat amb el color escollit*/
/**************************************************************************************/

void EditorTerreny::marcaSeleccionat()
{
    pVertex vh = mesh->obtenirPunt(info_mostrar.seleccionat);
    
    glBegin( GL_POINTS );
       qglColor( info_GL.color_seleccio );
       glVertex2d( vh->point().x(), vh->point().y() );
    glEnd();
}

/*******************************************************************************************************/
/*   void EditorTerreny::cercle( double centre_x, double centre_y, double radi, unsigned int segments )*/
/*Mètode que permet visualitzar el punt introduit en forma de cercle centrat on s'ha clickat           */
/*******************************************************************************************************/

void EditorTerreny::cercle( double centre_x, double centre_y, double radi, unsigned int segments )
{
      glBegin( GL_LINE_LOOP );
      	qglColor( Qt::red );    
      	for( unsigned int i = 0; i < segments; i++ )
      	{	          
        glVertex2d( radi*sin(((2*PI)/segments)*i) + centre_x, radi*cos(((2*PI)/segments)*i) + centre_y );
      	}
      glEnd();
}

/****************************************************************/
/*   void EditorTerreny::mouseReleaseEvent( QMouseEvent *e)     */
/*Mètode que s'executa en el moment en que el botó es deixa anar*/
/****************************************************************/

void EditorTerreny::mouseReleaseEvent( QMouseEvent *e )
{   
	GLint viewport[4];
	GLdouble modelview[16];
	GLdouble projection[16];
	double x, y, z;
	int xrata, yrata;
	bool continua=false;
	Objecte *obj;
	
	if (accio != GL_SELECT) /*En el cas que no estiguem en mode selecció*/
	{
		/*Modifiquem el valor del punt amb el que obtenim per paràmetres que representa el punt on s'ha deixat anar el mouse*/
		
		xrata = e->x();
		yrata = e->y();
		
		makeCurrent();
		
		glGetIntegerv( GL_VIEWPORT, viewport ); /*Preguntem a OpenGL les mides actuals de la finestra*/
		glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
		glGetDoublev( GL_PROJECTION_MATRIX, projection );
		
		x = y = z = 0.0;
				
		/*A partir dels punts de pantalla fem la corresponent conversió segons els valors mínims i màxims que representen el plà*/
		gluUnProject( (GLdouble)xrata, (GLdouble)(viewport[3] - yrata), -1.0, modelview, projection, viewport, &x, &y, &z );
		
		/*Un cop obtingudes les coordenades x i y del punt s'obre una finestra en el qual es demana introduir el valor corresponent a l'alçada*/
		Punt punt(x,y,.0);		
		
		if(info_vis.mostrar.tipus_visualitzacio == SEGMENT)
		{
			info_mostrar.comprovar_visibilitat=TRUE;
			info_vis.punt_comprovar=Punt(x,y,0);
		}
		else
		{	
			info_mostrar.comprovar_visibilitat=FALSE;
			mesh->settipusInterseccio(INDEFINIT);
			//Aquí és on es mostra el diàleg de l'alçada de la z
			if (info_calculs.mode_visibilitat)
			{
				if(!info_vis.punts_pla_poligon)
				{
				ZDialeg *cz = new ZDialeg( this );
				
				cz->limits( 0.0, info_mon.zmax*2 );
				if( cz->exec() == QDialog::Accepted )
				{
					z = cz->alcada();    
					continua= true;
				}
				delete cz;
				}
			
				if(continua || info_vis.punts_pla_poligon)
				{					
					if(info_vis.visio == INDEFINIT)
					{
						/*A partir dels valors de les coordenades (x,y,z), introduim el nou punt a la triangulació de Delaunay i ens guardem aquest nou punt al vector de vertexs*/
						//Punt p(x,y,0.);
						
						mesh->inserirPunt(Punt3d( x, y, z ));
						
						//Comprovem el valor de les coordenades (x, y) del nou punt per tal de comprovar si aquest es troba als extrems
						if(x < info_mon.vertex_xmin.x()) 
						{
							info_mon.vertex_xmin.modifica_coordenades( x, y, z );
							posicio = mesh->numero_de_vertexs();
						}
					}
					else
						tractarElementVisio( e, x, y, z);
				}
			}
			else	//Estem en mode distancies if (info_calculs.mode_distancies)
			{				
				continua=punt.buscar_z(mesh->triangulacio());
				z=punt.z();
				cout << "\nZ= " << z;
				cout << "\nContinua = " << continua; 
				
				if(continua || info_vis.punts_pla_poligon)
				{					
					if(info_vis.visio == INDEFINIT)
					{
						/*A partir dels valors de les coordenades (x,y,z), introduim el nou punt a la triangulació de Delaunay i ens guardem aquest nou punt al vector de vertexs*/
						//Punt p(x,y,0.);
						
						mesh->inserirPunt(Punt3d( x, y, z ));
						
						//Comprovem el valor de les coordenades (x, y) del nou punt per tal de comprovar si aquest es troba als extrems
						if(x < info_mon.vertex_xmin.x()) 
						{
							info_mon.vertex_xmin.modifica_coordenades( x, y, z );
							posicio = mesh->numero_de_vertexs();
						}
					}
					else
						tractarElementDistancies( e, x, y, z);
				}
			}
			
		}
		/*Actualitzem el dibuix, és a dir, el repintem*/
	
		updateGL();
		/*En el cas que anteriorment el terreny estiguès buit indiquem amb l'atribuit modificat que aquest ha variat i tot seguit emetem un signal amb el model de Delaunay creat fins al moment*/
		setModificat();
		emit terrenyModificat();
	}
	else
	{
		if ( e->button() == QMouseEvent::LeftButton ) /*En el cas que el mode actual sigui el de selecció, i el botó que s'hagi deixat a anar sigui l'esquerre, és a dir, el que ens permet moure el punt de lloc, indiquem que deixa d'estar apretat posant a fals l'atribut pressed*/
		{
			pressed = FALSE;
		}
	}
}

/*********************************************************************************/
/*   void EditorTerreny::tractarElementVisio()                                   */
/*Mètode que segons el valor de la variable visio realitzara una o altre operacio*/
/*********************************************************************************/

void EditorTerreny::tractarElementVisio(QMouseEvent *e, const double &x, const double &y, const double &z)
{
	Objecte * punter;
	bool interseccio = false;
	switch(info_vis.visio)
	{
		case PUNT_VISIO: 
			punter = new Punt( x, y, z );
			cout <<"\nPassem per tractar punt visio";
			tractarPuntVisio( punter );
			break;
		case SEGMENT_VISIO: 
			if(!info_vis.punt_segment)
			{
				info_vis.inici_segment.modifica_coordenades(x, y, z);
				tractarIniciSegment( );
			}
			else
			{
				info_vis.fi_segment.modifica_coordenades(x, y, z);
				punter = new Segment( info_vis.inici_segment, info_vis.fi_segment, true );
				tractarFiSegment( punter );
				if (!info_vis.punt_segment)
					objectes_visio->inserirObjecte( punter );
			}
			break; 
		case POLIGON_VISIO:
			/*Cal comprovar si el punt on ha marcat es correcte per poder definir un poligon, es a dir, que la recta que es forma no intersecta amb cap dels costats anteriors. Tan sols ho haurem de comprovar en el cas que estigui marcant l'ultim punt que formara el poligon*/
			if(info_vis.punts_poligon.size()>2)
			{
				Punt mon(x,y,info_vis.pla_poligon->calcula_Z(Punt2d(x,y), info_vis.punts_poligon[1]) );
				interseccio = comprovarPoligonMes2Vertexs( e, mon );
			}
			else
			{
				interseccio = comprovarPoligonMenys2Vertexs( x, y, z );
			}
			if (!interseccio)
			{
				if( e->button()==LeftButton )
				{
					info_vis.dibuixant_poligon = true;
					if(info_vis.punts_poligon.size()==3)
					{
						/*Un cop disposem dels 3 primers punts del poligon podem construir el pla que aquests defineixen i d'aquesta manera introduir de la resta de punts tan sols les coordenades (x,y), ja que la z serà calculada automàticament*/
						info_vis.pla_poligon = new Pla(info_vis.punts_poligon[0], info_vis.punts_poligon[1], info_vis.punts_poligon[2]);
						info_vis.punts_pla_poligon = true;
					}
				}
				else
				{
					punter = new Poligon( info_vis.punts_poligon, info_vis.punts_poligon.size() );
					objectes_visio->inserirObjecte( punter );
					info_vis.dibuixant_poligon = false;
					info_vis.punts_pla_poligon = false;
					info_vis.punts_poligon.clear();
				}
			}
			break;
		case POLIGONAL_VISIO:
			if(!info_vis.punt_segment)
			{
				info_vis.inici_segment.modifica_coordenades(x, y, z);
				tractarIniciSegment( );
				if (info_vis.punt_segment) 
				{
					info_vis.punts_poligon.push_back(info_vis.inici_segment);
					info_vis.dibuixant_poligon=true;
				}
			}
			else
			{
				info_vis.fi_segment.modifica_coordenades(x, y, z);
				punter = new Segment( info_vis.inici_segment, info_vis.fi_segment, true );
				tractarFiSegment( punter );
				if (!info_vis.punt_segment) 
				{
					info_vis.punts_poligon.push_back(info_vis.fi_segment);
					if( e->button()==RightButton )
					{
						punter = new Poligonal( info_vis.punts_poligon );
						objectes_visio->inserirObjecte( punter );
						info_vis.punt_segment = false;
						info_vis.dibuixant_poligon=false;
						info_vis.punts_poligon.clear();
					}
					else
					{
						info_vis.punt_segment = true;
						info_vis.inici_segment = info_vis.fi_segment;
					}
				}
			}
			break; 
	}
}


/*********************************************************************************/
/*   void EditorTerreny::tractarElementDistancies()                                   */
/*Mètode que segons el valor de la variable visio realitzara una o altre operacio*/
/*********************************************************************************/

void EditorTerreny::tractarElementDistancies(QMouseEvent *e, const double &x, const double &y, const double &z)
{
	Objecte * punter;
	switch(info_vis.visio)
	{
		case PUNT_VISIO: 
			punter = new Punt( x, y, z );
			cout <<"\nPassem per tractar punt distancies";
			tractarPuntDistancies( punter );
			break;
		case SEGMENT_VISIO: 
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Segment incorrecte" ));
			errors->message( "Aqui no es pot afegir un segment de distancies."); 
			break; 
		case POLIGON_VISIO:
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Polígon incorrecte" ));
			errors->message( "Aqui no es pot afegir un poligon de distancies."); 
			break;
		case POLIGONAL_VISIO:
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Poligonal incorrecte" ));
			errors->message( "Aqui no es pot afegir una poligonal de distancies."); 
			break; 
	}
}



/***************************************************************/
/*   void EditorTerreny::tractarPuntVisio(Objecte * punter)    */
/*Mètode que fa el tractament de la insercio d'un punt de visio*/
/***************************************************************/

void EditorTerreny::tractarPuntVisio(Objecte * punter)
{
	if(((Punt *)punter)->puntCorrecte(mesh->triangulacio()))
	{
		objectes_visio->inserirObjecte( punter );
	}
	else
	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Punt de visio incorrecte" ));
		errors->message( "El punt introduit es troba fora o per sota de la triangulacio! Cal que es trobi per sobre d'aquesta."); 
	}
}


/***************************************************************/
/*   void EditorTerreny::tractarPuntDistancies(Objecte * punter)    */
/*Mètode que fa el tractament de la insercio d'un punt de distàncies*/
/***************************************************************/

void EditorTerreny::tractarPuntDistancies(Objecte * punter)
{
	//En teoria ja inserim un punt correcte, ja que la z ja l'hem agafat anteriorment del terreny.
	bool continua=true;
	if (continua) 
	{
		if (!info_calculs.punt_distancies)
		{
			objectes_visio->inserirObjecte(punter);
			float x,y,z;
			x=((Punt *)punter)->x();
			y=((Punt *)punter)->y();
			z=((Punt *)punter)->z();
			cout << "\nX = " << ((Punt *)punter)->x() << " Y = " << ((Punt *)punter)->y() << " Z = " << ((Punt *)punter)->z();
			//printf( "\nPrintf X = %12.15f, Y = %12.15f, Z = %12.15f\n", x,y,z );    
			cout << "\nPunt de Distancies inserit!!!";
			cout << "\n";
		}
		else
		{
			objectes_distancies->inserirObjecteDistancies(punter);
			float x,y,z;
			x=((Punt *)punter)->x();
			y=((Punt *)punter)->y();
			z=((Punt *)punter)->z();
			cout << "\nX = " << ((Punt *)punter)->x() << " Y = " << ((Punt *)punter)->y() << " Z = " << ((Punt *)punter)->z();
			//printf( "\nPrintf X = %12.15f, Y = %12.15f, Z = %12.15f\n", x,y,z );    
			cout << "\nPunt de Inserir Objecte Distancies inserit!!!";
			cout << "\n";
		}
	}
	else
 	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Punt de distancies incorrecte" ));
		errors->message( "Aqui no es pot afegir un punt de distancies."); 
 	}
}

/***************************************************************/
/*   void EditorTerreny::calculsDistancies()    */
/*Mètode que realitza el calcul de distancies entre dos llavors diferents*/
/***************************************************************/

void EditorTerreny::calculsDistancies()
{
vector<pCara> cares;
Pla * pla;
bool correcte;
int i,j;
list<Event*> E;
double borrades;

mesh->calculGirsDesplegament();

cout << "\n********** PROPAGAR TERRENY ********** " << endl;
	
cares=mesh->getCares();
	
for(i=0;i<cares.size();i++)
{
	for (j=0;j<3;j++)
	{
		pCara cara_veina=cares[i]->neighbor(j);
		int index=cara_veina->index(cares[i]);
	
		if (mesh->es_cara_infinita(cara_veina))
		{ }
		else
		{	
			//pla=new Pla(Punt(cares[i]->vertex(0)->point()),Punt(cares[i]->vertex(1)->point()),Punt(cares[i]->vertex(2)->point()));	
			pla=new Pla(0,0,1,0); // pla=ax+by+cz+d=0			
		
			//Agafar la transformació que toqui i fer la imatge del punt
			Punt punt1=Punt(cares[i]->info().T[j](cara_veina->vertex(index)->point()));
			correcte=pla->puntPertanyPla(punt1);
	
			if (correcte)
			{	
				//cout <<"\nPunt pertany a pla!" << endl;
			}
			else
			{ 
				double resultat=pla->avaluarPunt(punt1);
				cout <<"\nPunt no pertany a pla!" << endl;
				cout <<"Resultat = " << resultat << " z = " << punt1.z() << endl;
				/*cout << "escriu una tecla" ;
				cin >> tecla;*/
			}

			delete pla;
		}
		
	}
}

inicialitzaVertexs(); //Inicialitza les distàncies del vèrtex a -1 
cout << "Hem inicialitzat les distancies dels vertexs a -1\n" << endl;

E = inicialitzacio();
cout << "Hem realitzat el mètode de inicialitzacio\n" << endl;
pintarInicialitzacio(E);

propagacio(E);
cout << "Hem realitzat el mètode de propagacio d'events\n" << endl;

/*char tec;
cout << "Prem una tecla per tornar a repintar les finestres ... " << endl;
cin >> tec;
*/
info_mostrar.repintar_finestres=TRUE;
repaint();
info_mostrar.repintar_finestres=FALSE;
//cout << "\nHe acabat de repintar les finestres de color groc" << endl;
		
/*cout << "\nComprovem que no hi hagi cap vèrtex amb distància = -1 " << endl;
comprovaVertexs();
cout << "No hi ha cap vèrtex amb distància = -1 " << endl;*/

esborrarFinestresRepetides(borrades);
//cout << "He esborrat finestres repetides = " << borrades << endl;

cout << "\n********** FI PROPAGAR TERRENY ********** " << endl;

} 


/***************************************************************/
/*   void EditorTerreny::calcularUnfoldings()   */
/*Mètode que inicialitza les distàncies dels vèrtexs a -1 */
/***************************************************************/

void EditorTerreny::inicialitzaVertexs()
{
//Inicialitza les distàncies dels vèrtexs de la triangulació a -1 (així quan trobem un vèrtex =-1 sabrem que no hi hem anat mai). 
mesh->inicialitzarVertexs();
}


void EditorTerreny::comprovaVertexs()
{
mesh->comprovarVertexs();
}


void EditorTerreny::esborrarFinestresRepetides(double &borrades)
{
list<Finestra>::iterator fj0,fvj0,fj1,fvj1,fj2,fvj2,tmp1,tmp2;
Finestra f1,f2;

borrades=0;

for (IteradorCares fit=(mesh->triangulacio())->finite_faces_begin(); fit!=(mesh->triangulacio())->finite_faces_end();fit++)
{
	if(!mesh->es_cara_infinita(fit))
	{
		fj0=fit->info().LlistaFinestres[0].begin();
		fvj0=fit->info().LlistaFinestres[0].end();
		fj1=fit->info().LlistaFinestres[1].begin();
		fvj1=fit->info().LlistaFinestres[1].end();
		fj2=fit->info().LlistaFinestres[2].begin();
		fvj2=fit->info().LlistaFinestres[2].end();
	
		while (fj0!=fvj0)
		{
			f1=*fj0;
			tmp1=fj0;
			fj0++;
			if (fj0!=fvj0)
			{
				f2=*fj0;
				tmp2=fj0;
				if (f1==f2)
				{
					fit->info().LlistaFinestres[0].remove(*tmp1);
					borrades=borrades+1;
				}
				fj0++;
			}
		}
		
		while (fj1!=fvj1)
		{
			f1=*fj1;
			tmp1=fj1;
			fj1++;
			if (fj1!=fvj1)
			{
				f2=*fj1;
				tmp2=fj1;
				if (f1==f2)
				{
					fit->info().LlistaFinestres[1].remove(*tmp1);
					borrades=borrades+1;
				}
				fj1++;
			}	
		}
		
		while (fj2!=fvj2)
		{
			f1=*fj2;
			tmp1=fj2;
			fj2++;
			if (fj2!=fvj2)
			{
				f2=*fj2;
				tmp2=fj2;
				if (f1==f2)
				{
					fit->info().LlistaFinestres[2].remove(*tmp1);
					borrades=borrades+1;
				}
				fj2++;
			}
		}
	}
}

}


void EditorTerreny::mirarIndexs(double &index1, double &index2, double &index3, double &index_1)
{
index1=0; index2=0; index3=0; index_1=0;

for (IteradorVertexs vit = mesh->triangulacio()->finite_vertices_begin(); vit != mesh->triangulacio()->finite_vertices_end(); vit++)
{
	if(vit->info().index==1)
	{
		index1++;
	}
	else if (vit->info().index==2)
	{
		index2++;
	}
	else if (vit->info().index==3)
	{
		index3++;
	}
	else if (vit->info().index==-1)
	{
		index_1++;
	}
}	

}

/***************************************************************/
/*   list<Event> EditorTerreny::inicialitzacio()   */
/*Mètode que ens inicialitza els paràmetres per cada punt  */
/***************************************************************/

list<Event*> EditorTerreny::inicialitzacio()
{
//Necessitem la llista de llavors, que estan inserides dins el VectorVisio de la classe Visio
Objecte * obj;
int k,l;
Punt3d punt3d, puntvertex1, puntvertex2;
int vertex;
Localitzacio tipus;
pCara tri;
double b0,b1,d0,d1,sigma;
pCara cara;
int j, index;
double pes;
pCara carapseudo;
pVertex vertexpseudo;
double sigmapseudo;
int indexpseudo;
list<Event*> E;
IteradorCares fit;
vector<pCara> cares;
double borrades;

for(k=0; k<objectes_visio->numeroObjectes(); k++)
{
obj = objectes_visio->obtenirObjecte(k+1); //Obtenim cada objecte dels que hem inserit (punts = llavors)
		
		switch(obj->tipusObjecte())
		{
			case PUNT_VISIO: 
			punt3d=((Punt *)obj)->getPunt();
			cout << "Punt = " << punt3d.x() << " , " << punt3d.y() << " , " << punt3d.z() << endl;
			
			tri=mesh->triangulacio()->locate(punt3d,tipus,vertex); //Trobem el triangle on està el punt
			cout << "Triangle que conté el source = " << tri->vertex(0)->point() << " , " << tri->vertex(1)->point() << " , " << tri->vertex(2)->point() << endl;
			
			info_calculs.caresSource.push_back(tri);
			
			for(l=0; l<3; l++) // Creem una finestra per cada costat amb les distancies que toquin. Seran Punts3D. 
			{
				puntvertex1=tri->vertex((l+1)%3)->point();
				puntvertex2=tri->vertex((l+2)%3)->point();
															
				d0=sqrt((punt3d.x()-puntvertex1.x())*(punt3d.x()-puntvertex1.x())+(punt3d.y()-puntvertex1.y())*(punt3d.y()-puntvertex1.y())+(punt3d.z()-puntvertex1.z())*(punt3d.z()-puntvertex1.z()));
				d1=sqrt((punt3d.x()-puntvertex2.x())*(punt3d.x()-puntvertex2.x())+(punt3d.y()-puntvertex2.y())*(punt3d.y()-puntvertex2.y())+(punt3d.z()-puntvertex2.z())*(punt3d.z()-puntvertex2.z()));
				b0=0;
				b1=sqrt((puntvertex1.x()-puntvertex2.x())*(puntvertex1.x()-puntvertex2.x())+(puntvertex1.y()-puntvertex2.y())*(puntvertex1.y()-puntvertex2.y())+(puntvertex1.z()-puntvertex2.z())*(puntvertex1.z()-puntvertex2.z()));
				sigma=0;
				cara=tri; //La que hem trobat abans
				index=k+1; //Index de la posicio del vector on hi ha el punt
				j=l; //Serà el vertex oposat
				
				Finestra * f= new Finestra(b0,b1,d0,d1,sigma,cara,j,index);
				//Els inserim a la Llista Finestres allà on toqui segons la j 
				cara->info().LlistaFinestres[j].push_front(*f); //Insereix finestra f a l'inici de Llista Finestres
				cout << "Creem la finestra amb parametres d0 = " << d0 << " d1 = " << d1 << " b0 = " << b0 << " b1 = " << b1 << " sigma = " << sigma << " index = " << index << " j = " << j << endl;
				cout << "Puntvertex 1 = " << puntvertex1.x() << " , " << puntvertex1.y() << " , " << puntvertex1.z() << endl;
				cout << "Puntvertex 2 = " << puntvertex2.x() << " , " << puntvertex2.y() << " , " << puntvertex2.z() << endl;
				//esborrarFinestresRepetides(borrades);
											
				//Inserim la finestra a la llista d'events (serà del tipus EVENT_FINESTRA(0))
				pes=sigma + min(d0,d1);
				f->setpes(pes);
				E.push_front(f); //Insereix events tipus Finestra a la llista d'Events
				cout << "Inserim a la llista d'Events la finestra amb j = " << j << " i amb pes = " << f->getpes() << endl;
					
				//d0 es la distancia del vertex l+1 al pseudosource
				if((cara->vertex((l+1)%3)->info().distancia==-1) || (cara->vertex((l+1)%3)->info().distancia>d0+sigma))
				{
					cara->vertex((l+1)%3)->info().distancia=d0+sigma;
					cara->vertex((l+1)%3)->info().index=index;
					//Mirem si són vèrtexs hiperbòlics (>2PI) 
					if (mesh->vertexHiperbolic(cara,cara->vertex((l)%3)))
					{
					carapseudo=cara;
					vertexpseudo=cara->vertex((l)%3);
					sigmapseudo=Punt(cara->vertex((l)%3)->point()).distancia(punt3d); 
					indexpseudo=k+1; //Index de la posicio del vector on hi ha el punt
					PseudoSource * ps= new PseudoSource(sigmapseudo, indexpseudo, vertexpseudo, carapseudo);
					
					//Inserim el pseudosource a la llista d'events (serà del tipus EVENT_PSEUDOSOURCE(1))
					ps->setpes(sigmapseudo);
					E.push_front(ps); //Insereix events tipus PseudoSource a la llista d'Events
					cout << "Inserim a la llista d'Events el pseudosource amb index = " << indexpseudo << " i amb pes : " << ps->getpes() << endl;
					Punt(vertexpseudo->point()).escriure();
					}
				}				
			}
						
			break;
			
			case SEGMENT_VISIO: 
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Error" ));
			errors->message( "Operacio temporalment no disponible per segments."); 
			break; 
			
			case POLIGON_VISIO:
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Error" ));
			errors->message( "Operacio temporalment no disponible per poligons."); 
			break;
			
			case POLIGONAL_VISIO:
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Error" ));
			errors->message( "Operacio temporalment no disponible per poligonals."); 
			break; 
		}
		
}
//Escriure llista E sense ordenar
Event * e3=E.front();
Event * e4=E.back();
cout << "Pes Primer Element Llista = " << e3->getpes() << " Pes Ultim Element Llista " << e4->getpes() << endl;
E.sort(Event::ordena); //Ordenem la llista d'events segons el pes.
//Escriure llista E ordenats per pes
Event * e5=E.front();
Event * e6=E.back();
cout << "Pes Primer Element Llista = " << e5->getpes() << " Pes Ultim Element Llista " << e6->getpes() << endl;

return E;
}
//El mètode d'iniciació ens retorna una llista d'events E (tant Finestres com PseudoSources) ordenats de menys a més pes


void EditorTerreny::pintarInicialitzacio(list<Event*> E)
{
list<Event*>::iterator pi,pf;
Event *ev;
pi=E.begin();
pf=E.end();

info_mostrar.propagacio=FALSE;

while(pi!=pf) 
{
	ev=*pi;
	info_calculs.eventactual=ev;
	info_mostrar.veure_dibuix=TRUE;	
	repaint();
	info_mostrar.veure_dibuix=FALSE;
	pi++;	
}

info_mostrar.propagacio=TRUE;

}



/*****************************************************************/
/*   EditorTerreny::propagacio(list<Event*> E)   */
/*Mètode que agafa el primer event de la llista i el propaga */
/*****************************************************************/
void EditorTerreny::propagacio(list<Event*> E)
{
list<Event*> PossiblesEvents;
Event* e;
Event* e2;
char tecla;

while(!E.empty()) 
{
	e=E.front(); //Ens retorna el primer element de la llista
	cout << "EVENT: "; 
	e->escriure();
	//PossiblesEvents.clear();
	if (e->tipusEvent()==0) //Si ens trobem amb events de tipus Finestra
	{			
		cout << "Trobem una finestra amb pes " << e->getpes() << endl;
		PossiblesEvents=e->propagar(mesh->triangulacio()); //Ens retorna una llista de possibles events, que haurem de detectar si són reals o no i si s'hauran d'inserir a la llista d'events E o no.
		cout << "possiblesEvents.size() " << PossiblesEvents.size() << endl;
	}
	else if (e->tipusEvent()==1) //Si ens trobem amb events de tipus PseudoSource
	{
		cout << "Trobem un PseudoSource amb pes " << e->getpes() << endl;
		PossiblesEvents=e->propagar(mesh->triangulacio()); //Retorna una llista de possibles events, que haurem de detectar si són reals o no i si s'hauran d'inserir a la llista d'events E o no.
	}
	
	cout << "possiblesEvents.size() " << PossiblesEvents.size() << endl;
	E.pop_front(); //Eliminem de la llista l'event que ja hem propagat, que serà el primer (el del pes més petit)	
			
	deteccioEventsReals(E,PossiblesEvents); //Detectem quins events de la llista PossiblesEvents són reals i els inserim a E
	cout << "Esborrem un event de tipus " << e->tipusEvent() << " i amb pes " << e->getpes() << endl;
	cout << "Events x processar = " << E.size() << endl;
	
	info_calculs.eventactual=e;
	info_mostrar.veure_dibuix=TRUE;	
	repaint();
	info_mostrar.veure_dibuix=FALSE;
	
	delete e; // Pensar que més tard s'haurà de fer servir
	//E.sort(Event::ordena); //Ordenem la llista d'events segons el pes.	
}

}


void EditorTerreny::propagacioNOMESPSEUDOSOURCE(list<Event*> E)
{
list<Event*> PossiblesEvents;
Event* e;

while(!E.empty()) 
{
e=E.front(); //Ens retorna el primer element de la llista
cout << "EVENT: "; 
e->escriure();
//PossiblesEvents.clear();
	if (e->tipusEvent()==0) //Si ens trobem amb events de tipus Finestra
	{
	}
	else if (e->tipusEvent()==1) //Si ens trobem amb events de tipus PseudoSource
	{
	cout << "Trobem un PseudoSource amb pes " << e->getpes() << endl;
	PossiblesEvents=e->propagar(mesh->triangulacio());
	cout << "possiblesEvents.size() " << PossiblesEvents.size() << endl;	
	}
	
E.pop_front(); //Eliminem de la llista l'event que ja hem propagat, que serà el primer (el del pes més petit)	
cout << "Esborrem un event de tipus " << e->tipusEvent() << " i amb pes " << e->getpes() << endl;
delete e; // Pensar que més tard s'haurà de fer servir	
}

}

void EditorTerreny::propagacioNOMES1FINESTRA(list<Event*> E)
{
list<Event*> PossiblesEvents;
Event* e;

while (!E.empty())
{
e=E.front(); //Ens retorna el primer element de la llista
cout << "EVENT: "; 
e->escriure();
//PossiblesEvents.clear();
if (e->tipusEvent()==0) //Si ens trobem amb events de tipus Finestra
{
	cout << "Trobem una finestra amb pes " << e->getpes() << endl;
	PossiblesEvents=e->propagar(mesh->triangulacio());
}
else if (e->tipusEvent()==1) //Si ens trobem amb events de tipus PseudoSource
{
}

E.pop_front(); //Eliminem de la llista l'event que ja hem propagat, que serà el primer (el del pes més petit)	
cout << "Esborrem un event de tipus " << e->tipusEvent() << " i amb pes " << e->getpes() << endl;

info_calculs.eventactual=e;
info_mostrar.veure_dibuix=TRUE;	
repaint();
info_mostrar.veure_dibuix=FALSE;

}

delete e; // Pensar que més tard s'haurà de fer servir	
}


/***************************************************************/
/*   list<Event*> EditorTerreny::deteccioEventsReals(list<Event*> E, list<Event*> PossiblesEvents)             */
/*Mètode que mira quins events de la llista de possibles events els podem inserir a la llista bona d'events */
/***************************************************************/

void EditorTerreny::deteccioEventsReals(list<Event*> &E, list<Event*> PossiblesEvents)
{
list<Finestra>::iterator f,fv,fa,fl,fvl;
list<Finestra>::iterator tmp,tmpveina;
Event * ev;
Finestra * fn;
pCara cara,cara_veina;
int j,jv;
double x,d,e,errors;
bool veina;
int e_reals=0;
double borrades;

while(!PossiblesEvents.empty())
{
ev=PossiblesEvents.front(); //Ens retorna el primer element de la llista
cout << "POSSIBLE EVENT: "; 
ev->escriure();
errors=0;
	
	if (ev->tipusEvent()==1) //L'event és un pseudosource
	{
		//cout << "Trobem un PseudoSource amb pes " << ev->getpes() << " i l'inserim a la llista Bona " << endl;
		e_reals++;
		E.push_back(ev); //Inserim l'event pseudosource a la llista Bona i llavors des d'allà el propagarem
	}
	else if (ev->tipusEvent()==0) //L'event és una finestra
	{
		//cout << "Trobem una Finestra amb pes " << ev->getpes() << endl;
		cout << " f, cc " << endl;
		cara=ev->getcara();
		j=ev->getj();
		
		//Per totes les finestres de l'edge (per la pCara Original) mirar si la finestra possible interseca amb la finestra que tenim guardada en el costat (Llista Finestres)
		f=cara->info().LlistaFinestres[j].begin();
		fl=cara->info().LlistaFinestres[j].end();
		
		//Calcular e abans de començar el bucle de les finestres de la cara_veina i passar-lo als mètodes veins que el necessiten
		cout << " ac, cv " << endl;
		cara_veina=cara->neighbor(j); 
		jv=cara_veina->index(cara);
		Punt p1(cara->vertex((j+1)%3)->point());
		Punt p2(cara->vertex((j+2)%3)->point());
		e=p1.distancia(p2.getPunt());	
				
		fv=cara_veina->info().LlistaFinestres[jv].begin();
		fvl=cara_veina->info().LlistaFinestres[jv].end();
		
		//acabat=!((f!=fl && f->getb0()<ev->getb1()) || (fv!=fvl && e-fv->getb1()<ev->getb1()));
		cout << "Mida llista f = " << cara->info().LlistaFinestres[j].size() << endl;
		cout << "Mida llista fv = " << cara_veina->info().LlistaFinestres[jv].size() << endl;
		int midaf=cara->info().LlistaFinestres[j].size();
		int midafv=cara_veina->info().LlistaFinestres[jv].size();
			
		bool sortir=false;
		while (!sortir)
		{
			if ((f==fl && fv==fvl) || (midaf==0 && midafv==0))//Sortim del bucle
			{
				//cout << "entro a cas 1" << endl;
				sortir=true;
				//cout << "Cas 1 Mida llista f = " << midaf << endl;
				//cout << "Cas 1 Mida llista fv = " << midafv << endl;
			}
			
			else if ((f!=fl && fv!=fvl) && (midaf!=0 && midafv!=0))//Puc fer f->getb0() i fv->getb1()
			{
				//cout << "Entro a cas 2" << endl;
				if((f!=fl) && ((fv==fvl) || (f->getb0()<e-fv->getb1())))
				{ 
					veina=false;
					fa=f; 
				}
				else
				{ 
					veina=true;
					fa=fv;
				}
				cout << "Finestra FA = "; fa->escriure();	
				cout << "v " << veina;
				
				bool iguals=false;
				Finestra fff=*fa;
				if((*(Finestra *)ev)==fff)				
				{
					iguals=true;
				}
				
				if (iguals) //si són iguals ja esborrem la finestra directament
				{
					//En comptes d'eliminar la finestra, modificar elsparàmetres de l'event (b0=b1). D'aquesta manera ja no entrarà al no_elimina i per tant ja no la ficarem a la cua d'events
					ev->setb0(ev->getb1());
					if (veina)
					{
						fv++;
						//cout << "ELIMINA IGUALS VEINA" << endl;
					}
					else
					{
						f++;
						//cout << "ELIMINA IGUALS NO VEINA" << endl;
					}	
					/*if(veina)
					{
						tmpveina=fv;
						fv++;
						//cara_veina->info().LlistaFinestres[jv].remove(*fv);
						cara_veina->info().LlistaFinestres[jv].remove(*tmpveina);
						cout << "ELIMINA IGUALS VEINA" << endl;
					}
					else
					{
						tmp=f;
						f++;
						//cara->info().LlistaFinestres[j].remove(*f);
						cara->info().LlistaFinestres[j].remove(*tmp);
						cout << "ELIMINA IGUALS NO VEINA" << endl;
					}*/		
				}
				
				else
				{
					if(ev->interseca(&(*fa),x,d,errors,e,veina))
					{
						cout << "x = " << x << ", d = " << d << ", e = " << e << endl;
						cout << "Trobo una finestra que interseca " << endl;			
						fn = new Finestra(*fa);
						if (ev->no_elimina(fn,x,d,errors,veina,e))
						{
						//podria ser que amb actualitzar(E,fn,fa); passessim, però no ho sé.
							if(veina) 
							{
								actualitzar(E,fn,&(*fv));
								//cout << "NO ELIMINA ACTUALITZAR VEINA" << endl;
								fv++;
							}
							else 
							{
								actualitzar(E,fn,&(*f));
								//cout << "NO ELIMINA ACTUALITZAR NO VEINA" << endl;
								f++;
							}
						}
						else 
						{
						// hem d'eliminar la finestra que esta guardada a cara o cara veina. Fem una llista de finestres per la cara veina i una per la cara normal i hi anirem inserint les finestres que hem d'eliminar, que ho farem al final del bucle.
							if(veina)
							{
								tmpveina=fv;
								fv++;
								//cara_veina->info().LlistaFinestres[jv].remove(*fv);
								cara_veina->info().LlistaFinestres[jv].remove(*tmpveina);
								//cout << "ELIMINA VEINA" << endl;
							}
							else
							{
								tmp=f;
								f++;
								//cara->info().LlistaFinestres[j].remove(*f);
								cara->info().LlistaFinestres[j].remove(*tmp);
								//cout << "ELIMINA NO VEINA" << endl;
							}			
						}
					delete fn;
					}
					else
					{					
						//cout << "NO INTERSECAAAAAAAAAAAA" << endl;
						if (veina) { fv++; }
						else { f++; }
					}
				}
				
				if (veina) 
				{
					//fv++;
					midafv--;
					fvl=cara_veina->info().LlistaFinestres[jv].end();
				}
				else
				{ 
					//f++;
					midaf--;
					fl=cara->info().LlistaFinestres[j].end();
				}
				//cout << "Cas 2 Mida llista f = " << midaf << endl;
				//cout << "Cas 2 Mida llista fv = " << midafv << endl;
			}			
			
			else if ((f==fl && fv!=fvl) && (midaf==0 && midafv!=0)) // No puc fer f.getb0(). Segur que serà veina
			{
				//cout << "entro a cas 3" << endl;
				veina=true;
				fa=fv;
				
				cout << "Finestra FA = "; fa->escriure();	
				cout << "v " << veina;
				
				bool iguals=false;
				Finestra fff=*fa;
				if((*(Finestra *)ev)==fff)				
				{
					iguals=true;
				}
				
				if (iguals)
				{
					ev->setb0(ev->getb1());
					//cout << "ELIMINA IGUALS VEINA" << endl;
					/*tmpveina=fv;
					cara_veina->info().LlistaFinestres[jv].remove(*tmpveina);
					cout << "ELIMINA IGUALS VEINA" << endl;
					fv=cara_veina->info().LlistaFinestres[jv].begin();*/
					//En comptes d'eliminar la finestra, modificar els paràmetres de l'event. b0=b1, d'aquesta manera ja no m'entrarà al no_elimina, i per tant, ja no la ficarem a la cua d'events.
				}
				else
				{
					if(ev->interseca(&(*fa),x,d,errors,e,veina))
					{
						cout << "x = " << x << ", d = " << d << ", e = " << e << endl;
						cout << "Trobo una finestra que interseca " << endl;			
						fn = new Finestra(*fa);
						if (ev->no_elimina(fn,x,d,errors,veina,e))
						{
						//podria ser que amb actualitzar(E,fn,fa); passessim, però no ho sé.
							actualitzar(E,fn,&(*fv));
							//cout << "NO ELIMINA ACTUALITZAR VEINA" << endl;
							fv++;
						}
						else 
						{
							tmpveina=fv;
							cara_veina->info().LlistaFinestres[jv].remove(*tmpveina);
							//cout << "ELIMINA VEINA" << endl;
							fv=cara_veina->info().LlistaFinestres[jv].begin();
						}				
					delete fn;	
					}
					else
					{					
						//cout << "NO INTERSECAAAAAAAAAAAA" << endl;
						fv++;
					}
				}
				//fv++;
				midafv--;
				fvl=cara_veina->info().LlistaFinestres[jv].end();
				cout << "Cas 3 Mida llista f = " << midaf << endl;
				cout << "Cas 3 Mida llista fv = " << midafv << endl;
			}
			
			else if ((f!=fl && fv==fvl) || (midaf!=0 && midafv==0))//No puc fer fv.getb0(). Segur que és no veina
			{
				//cout << "entro a cas 4" << endl;
				veina=false;
				fa=f; 
				
				cout << "Finestra FA = "; fa->escriure();	
				cout << "v " << veina;
				
				bool iguals=false;
				Finestra fff=*fa;
				if((*(Finestra *)ev)==fff)				
				{
					iguals=true;
				}
				
				if (iguals)
				{
					ev->setb0(ev->getb1());
					//cout << "ELIMINA IGUALS NO VEINA" << endl;
					/*tmp=f;
					//cara->info().LlistaFinestres[j].remove(*f);
					cara->info().LlistaFinestres[j].remove(*tmp);
					cout << "ELIMINA IGUALS NO VEINA" << endl;
					f=cara->info().LlistaFinestres[j].begin();*/
				}
				else
				{
					if(ev->interseca(&(*fa),x,d,errors,e,veina))
					{
						cout << "x = " << x << ", d = " << d << ", e = " << e << endl;
						cout << "Trobo una finestra que interseca " << endl;			
						fn = new Finestra(*fa);
						if (ev->no_elimina(fn,x,d,errors,veina,e))
						{
						//podria ser que amb actualitzar(E,fn,fa); passessim, però no ho sé.
							actualitzar(E,fn,&(*f));
							//cout << "NO ELIMINA ACTUALITZAR VEINA" << endl;
							f++;
						}
						else 
						{
						// hem d'eliminar la finestra que esta guardada a cara o cara veina. Fem una llista de finestres per la cara veina i una per la cara normal i hi anirem inserint les finestres que hem d'eliminar, que ho farem al final del bucle.
							tmp=f;
							//cara->info().LlistaFinestres[j].remove(*f);
							cara->info().LlistaFinestres[j].remove(*tmp);
							//cout << "ELIMINA NO VEINA" << endl;
							f=cara->info().LlistaFinestres[j].begin();
						}
					delete fn;
					}
					else
					{
						//cout << "NO INTERSECAAAAAAAAAAAA" << endl;
						f++;
					}
				}
			
				//f++;
				midaf--;
				fl=cara->info().LlistaFinestres[j].end();
				//cout << "Cas 4 Mida llista f = " << midaf << endl;
				//cout << "Cas 4 Mida llista fv = " << midafv << endl;
			}
		}
		
		//Si l'event és igual a una finestra que ja hi ha guardada
		if (ev->no_eliminada())
		{
			//cout << "Finestra no eliminada " << endl;
			e_reals++;
			E.push_back(ev);
			cara->info().LlistaFinestres[j].push_front(*(Finestra *)ev);
			//cout << "INSERIM A CARA AMB J = " << j << endl;
			//esborrarFinestresRepetides(borrades);
			//cara->info().LlistaFinestres[j].insert(f,*(Finestra*)ev);
		}
	}
	
	cout << "Errors que hem trobat en el no_elimina = " << errors << endl;
	
	/*if (errors>0)
	{
		char prova;
		cout << "Hi ha hagut algun error en el no_elimina. Prem tecla per continuar ... " << endl;
		cin >> prova;
	}*/	
	
cout << "Esborrem event de tipus " << ev->tipusEvent() << " i pes " << ev->getpes() << " de la Llista Possibles Events " << endl;
//delete ev; // Pensar que més tard s'haurà de fer servir	
PossiblesEvents.pop_front(); //Eliminem de la llista l'event que ja hem detectat, que serà el primer (el del pes més petit)	
}

cout << "" << e_reals << " events reals dels possibles " << endl;
E.sort(Event::ordena); //Ens ordena la llista Bona de menys a més pes
}


/***************************************************************/
/*   void EditorTerreny::actualitzar(list<Event*> &Bona, Finestra * f, Finestra * fn)   */
/*Mètode que actualitza la llista de finestres de la pCara i la llista bona d'events */
/***************************************************************/

void EditorTerreny::actualitzar(list<Event*> &Bona, Finestra * fn, Finestra * f)
{
bool trobat=false;
list<Event*>::iterator pi,pf;
Event *ev2;
pi=Bona.begin();
pf=Bona.end();

Event * e = new Finestra(*f); //Constructor de còpia
Event * ev = new Finestra(*fn); //Constructor de còpia

//Busquem a la llista d'events si n'hi ha un que sigui igual a e		
while((pi!=pf) && (!trobat))
{
	ev2=*pi;
	if (*e==*ev2)
	{
		trobat=true;
	}	
	else
	{
		pi++;
	}
}

if(trobat)
{
	cout << "eliminem 1 event" << endl;
	Bona.remove(ev2);
	Bona.push_front(ev);	
}
	
Bona.sort(Event::ordena); //Ens ordena la llista Bona de menys a més pes
f->modifica_finestra(fn->getb0(),fn->getb1(),fn->getd0(),fn->getd1(),fn->getsigma(),fn->getcara(),fn->getj(),fn->getindex());
//delete e;
//delete ev2;
}


//Ens retorna un vector de punts que comença per p (el groc) i acaba per el source més proper
vector<Punt> EditorTerreny::camiMesCurt(Punt p, pCara cara, pCara &cara_source)
{
vector<Punt> vp;
Finestra fminimafinal,fnova;
double xfinal,xnova;
Punt pactual,puntnou;
pCara cara_actual;
Punt punt_distancia_minima;
int mida_sources,kk;
Objecte * obj;
Objecte * obj2;

cara_actual=cara;
vp.push_back(p); //Inserim el punt p (el groc)
cout << "Insereixo el punt groc al vector de punts " << endl;

obj = new Punt(p);
objectes_cami_minim->inserirObjecteCamiMinim(obj);
//punt3d=((Punt *)obj)->getPunt();

//Hem acabat quan hem arribat a la cara del source. Haurem de tenir una llista amb cada pseudosource quina pCara conté.
//fer-ho a través de l'index, posant caresSource[index]

pactual=vp.back();
mida_sources=info_calculs.caresSource.size();
int k=1;
bool trobat=false;
while ((k<=mida_sources) && (!trobat))
{
	cara_source=info_calculs.caresSource[k-1];
	if (cara_actual==cara_source)
	{
		trobat=true;
	}
	else
	{
		k=k+1;
	}
}	

if (cara_actual==cara_source)
{
	cout << "Els 2 punts estan a la mateixa cara" << endl;
}
else
{
	trobarDistanciaMinima2(pactual,cara_actual,fminimafinal,xfinal,vp);
	vp.push_back(fminimafinal.trobarPunt3d(xfinal));
	obj2 = new Punt(fminimafinal.trobarPunt3d(xfinal));
	objectes_cami_minim->inserirObjecteCamiMinim(obj2);
	punt_distancia_minima=fminimafinal.trobarPunt3d(xfinal);
	cout << "Insereixo el punt de Trobar Distancia Mínima al vector de punts. El punt és = " << endl;
	punt_distancia_minima.escriure();
	cara_actual=fminimafinal.getcara();
	cara_source=info_calculs.caresSource[fminimafinal.getindex()-1];
	
	//int i=0;	
	//while (i<3)
	while(cara_source!=cara_actual)
	{
		//Anirem cridant al saltar veina, actualitzant la pcara que toqui.
		saltarVeina(fminimafinal,xfinal,fnova,xnova,puntnou,vp);
		vp.push_back(puntnou);
		obj2 = new Punt(puntnou);
		objectes_cami_minim->inserirObjecteCamiMinim(obj2);
		cout << "Insereixo un punt nou al vector de punts" << endl;
		cara_actual=fnova.getcara();
		fminimafinal=fnova;
		xfinal=xnova;
		//i++;
	}
	
}
//info_calculs.caresSource.pop_back();
return vp;
}



void EditorTerreny::trobarDistanciaMinima2(Punt p, pCara cara, Finestra &fminimafinal, double &xfinal, vector<Punt> &vp)
{
int j0,j1,j2,jv0,jv1,jv2,j;
pCara cara_veina0,cara_veina1,cara_veina2;
Finestra f;
double distanciaminima0,distanciaminimafinal,distanciaminimaveina,distanciaminimanoveina;
distanciaminimafinal=100;
distanciaminimaveina=100;
distanciaminimanoveina=100;
list<Finestra>::iterator fj0,fvj0,fj1,fvj1,fj2,fvj2;
double x,xb0,xb1,e,efinal;
bool veina=false;
Punt2d s;
Punt punt_groc_transformat,interseccio,punt_groc_transformat0,punt_groc_transformat1,punt_groc_transformat2;
bool interseca;
double midas1;
double distanciaminimab0,distanciaminimab1;
Objecte * obj;


cout << "--------- Finestres cara ----------- " << endl;
if (!mesh->es_cara_infinita(cara))
{
	fj0=cara->info().LlistaFinestres[0].begin();
	fvj0=cara->info().LlistaFinestres[0].end();
	fj1=cara->info().LlistaFinestres[1].begin();
	fvj1=cara->info().LlistaFinestres[1].end();
	fj2=cara->info().LlistaFinestres[2].begin();
	fvj2=cara->info().LlistaFinestres[2].end();
	
	cara_veina0=cara->neighbor(0);
	cara_veina1=cara->neighbor(1);
	cara_veina2=cara->neighbor(2);
	
	if(!mesh->es_cara_infinita(cara_veina0))
        {
		Punt p1(cara->vertex(1)->point());
		Vector3D u(p1,Punt(cara->vertex(2)->point()));
		Vector3D v(p1,Punt(cara->vertex(0)->point()));
		Vector3D n=u.producte_vectorial(v);
		v=n.producte_vectorial(u);
		
		Vector3D vaux(p1,Punt(cara_veina0->vertex(cara_veina0->index(cara))->point()));
		Vector3D naux=u.producte_vectorial(vaux);
		
		double angle=n.angle(naux,true); //false
		
		u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
		Transformation3 N(u.x(),v.x(),n.x(),p1.x(),u.y(),v.y(),n.y(),p1.y(),u.z(),v.z(),n.z(),p1.z(),1);
		Transformation3 Na=N.inverse();
		//punt_groc_transformat0=cara->info().T[0](cara_veina0->vertex(jv0)->point());
		punt_groc_transformat0=Na(p.getPunt());
	}	
	
	if(!mesh->es_cara_infinita(cara_veina1))
        {
		Punt p1(cara->vertex(2)->point());
		Vector3D u(p1,Punt(cara->vertex(0)->point()));
		Vector3D v(p1,Punt(cara->vertex(1)->point()));
		Vector3D n=u.producte_vectorial(v);
		v=n.producte_vectorial(u);
		
		Vector3D vaux(p1,Punt(cara_veina1->vertex(cara_veina1->index(cara))->point()));
		Vector3D naux=u.producte_vectorial(vaux);
		
		double angle=n.angle(naux,true); //false
		
		u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
		Transformation3 N(u.x(),v.x(),n.x(),p1.x(),u.y(),v.y(),n.y(),p1.y(),u.z(),v.z(),n.z(),p1.z(),1);
		Transformation3 Na=N.inverse();
		//punt_groc_transformat1=cara->info().T[0](cara_veina0->vertex(jv0)->point());
		punt_groc_transformat1=Na(p.getPunt());
	}
	
	if(!mesh->es_cara_infinita(cara_veina2))
        {
		Punt p1(cara->vertex(0)->point());
		Vector3D u(p1,Punt(cara->vertex(1)->point()));
		Vector3D v(p1,Punt(cara->vertex(2)->point()));
		Vector3D n=u.producte_vectorial(v);
		v=n.producte_vectorial(u);
		
		Vector3D vaux(p1,Punt(cara_veina2->vertex(cara_veina2->index(cara))->point()));
		Vector3D naux=u.producte_vectorial(vaux);
		
		double angle=n.angle(naux,true); //false
		
		u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
		Transformation3 N(u.x(),v.x(),n.x(),p1.x(),u.y(),v.y(),n.y(),p1.y(),u.z(),v.z(),n.z(),p1.z(),1);
		Transformation3 Na=N.inverse();
		//punt_groc_transformat2=cara->info().T[0](cara_veina0->vertex(jv0)->point());
		punt_groc_transformat2=Na(p.getPunt());
	}		
			
	while (fj0!=fvj0)
	{
		f=*fj0;
		f.escriure();
		s=f.trobarSource();
		Segment s1(Punt(f.getb0(),0.0,0.0),Punt(f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat0);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=s1.coord_p_on_e(paux)*midas1+f.getb0();
			distanciaminima0=punt_groc_transformat0.distancia(paux)+f.distancia(x);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=0;
			}	
		}
		else
		{
			xb0=f.getb0();
			distanciaminimab0=punt_groc_transformat0.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0);
			xb1=f.getb1();
			distanciaminimab1=punt_groc_transformat0.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1);
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=0;
			}	
		}
		fj0++;
	}
	
	while (fj1!=fvj1)
	{
		f=*fj1;
		f.escriure();
		s=f.trobarSource();
		Segment s1(Punt(f.getb0(),0.0,0.0),Punt(f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat1);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=s1.coord_p_on_e(paux)*midas1+f.getb0();
			distanciaminima0=punt_groc_transformat1.distancia(paux)+f.distancia(x);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=1;
			}	
		}
		else
		{
			xb0=f.getb0();
			distanciaminimab0=punt_groc_transformat1.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0);
			xb1=f.getb1();
			distanciaminimab1=punt_groc_transformat1.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1);
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=1;
			}	
		}
		fj1++;
	}
	
	while (fj2!=fvj2)
	{
		f=*fj2;
		f.escriure();
		s=f.trobarSource();
		Segment s1(Punt(f.getb0(),0.0,0.0),Punt(f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat2);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=s1.coord_p_on_e(paux)*midas1+f.getb0();
			distanciaminima0=punt_groc_transformat2.distancia(paux)+f.distancia(x);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=2;
			}	
		}
		else
		{
			xb0=f.getb0();
			distanciaminimab0=punt_groc_transformat2.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0);
			xb1=f.getb1();
			distanciaminimab1=punt_groc_transformat2.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1);
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				j=2;
			}	
		}
		fj2++;
	}
}

cout << "--------- Finestres cara veina ------------ " << endl;
if (!mesh->es_cara_infinita(cara_veina0))
{
	jv0=cara_veina0->index(cara);
	punt_groc_transformat0=cara->info().T[0](p.getPunt());
	fj0=cara_veina0->info().LlistaFinestres[jv0].begin();
	fvj0=cara_veina0->info().LlistaFinestres[jv0].end();
	
	Punt p1(cara->vertex(1)->point());
	Punt p2(cara->vertex(2)->point());
	e=p1.distancia(p2.getPunt());	
							
	while (fj0!=fvj0)
	{
		f=*fj0;
		f.escriure();
		//s=f.trobarSource();
		s=f.trobarSource(e,true);
		Segment s1(Punt(e-f.getb0(),0.0,0.0),Punt(e-f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat0);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=e-f.getb0()-s1.coord_p_on_e(paux)*midas1;
			//distanciaminima0=p.distancia(paux)+f.distancia(x);
			distanciaminima0=punt_groc_transformat0.distancia(paux)+f.distancia(x,e,true);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=true;
				efinal=e;
				j=jv0;
			}	
		}
		else
		{
			xb0=e-f.getb0();
			//distanciaminimab0=p.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0);
			distanciaminimab0=punt_groc_transformat0.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0,e,true);	
			xb1=e-f.getb1();
			//distanciaminimab1=p.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1);
			distanciaminimab1=punt_groc_transformat0.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1,e,true);
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=true;
				efinal=e;
				j=jv0;
			}	
		}
		fj0++;
	}		
}

if (!mesh->es_cara_infinita(cara_veina1))
{
	jv1=cara_veina1->index(cara);
	punt_groc_transformat1=cara->info().T[1](p.getPunt());
	fj1=cara_veina1->info().LlistaFinestres[jv1].begin();
	fvj1=cara_veina1->info().LlistaFinestres[jv1].end();
	
	Punt p1(cara->vertex(2)->point());
	Punt p2(cara->vertex(0)->point());
	e=p1.distancia(p2.getPunt());	

	while (fj1!=fvj1)
	{
		f=*fj1;
		f.escriure();
		s=f.trobarSource(e,true);
		Segment s1(Punt(e-f.getb0(),0.0,0.0),Punt(e-f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat1);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=e-f.getb0()-s1.coord_p_on_e(paux)*midas1;
			//distanciaminima0=p.distancia(paux)+f.distancia(x);
			distanciaminima0=punt_groc_transformat1.distancia(paux)+f.distancia(x,e,true);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=true;
				efinal=e;
				j=jv1;
			}	
		}
		else
		{
			xb0=e-f.getb0();
			distanciaminimab0=punt_groc_transformat1.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0,e,true);	
			xb1=e-f.getb1();
			distanciaminimab1=punt_groc_transformat1.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1,e,true);
			
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				efinal=e;
				j=jv1;
			}	
		}
		fj1++;
	}
}

if (!mesh->es_cara_infinita(cara_veina2))
{				
	jv2=cara_veina2->index(cara);
	punt_groc_transformat2=cara->info().T[2](p.getPunt());
	fj2=cara_veina2->info().LlistaFinestres[jv2].begin();
	fvj2=cara_veina2->info().LlistaFinestres[jv2].end();
	
	Punt p1(cara->vertex(0)->point());
	Punt p2(cara->vertex(1)->point());
	e=p1.distancia(p2.getPunt());		
	
	while (fj2!=fvj2)
	{
		f=*fj2;
		f.escriure();
		s=f.trobarSource(e,true);
		Segment s1(Punt(e-f.getb0(),0.0,0.0),Punt(e-f.getb1(),0.0,0.0));
		midas1=s1.midaSegment();
		Segment r(s,punt_groc_transformat2);
		interseca=s1.interseccioLinia(r,interseccio);
	
		if (interseca)
		{
			Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
			x=e-f.getb0()-s1.coord_p_on_e(paux)*midas1;
			distanciaminima0=punt_groc_transformat2.distancia(paux)+f.distancia(x,e,true);
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				efinal=e;
				j=jv2;
			}	
		}
		else
		{
			xb0=e-f.getb0();
			distanciaminimab0=punt_groc_transformat2.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0,e,true);	
			xb1=e-f.getb1();
			distanciaminimab1=punt_groc_transformat2.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1,e,true);
			if (distanciaminimab0<=distanciaminimab1)
			{
				distanciaminima0=distanciaminimab0;
				x=xb0;
			}
			else
			{
				distanciaminima0=distanciaminimab1;
				x=xb1;
			}
			cout << "distanciaminima0= " << distanciaminima0 << endl;
			if ((distanciaminima0>0) && (distanciaminima0<distanciaminimafinal))
			{
				distanciaminimafinal=distanciaminima0;
				fminimafinal=f;
				xfinal=x;
				veina=false;
				efinal=e;
				j=jv2;
			}	
		}
		fj2++;
	}
}

//cout << "Distancia minima veina = " << distanciaminimaveina << endl;
//cout << "Distancia minima no veina = " << distanciaminimanoveina << endl;
cout << "Distancia mínima final = " << distanciaminimafinal << endl;
cout << "Coordenada x final = " << xfinal << endl;
cout << "Finestra minima final = " << endl;
fminimafinal.escriure();

if (veina) 
{
	cout << "La finestra mínima s'aconsegueix des d'una cara VEINA" << endl;
	xfinal=efinal-xfinal;
	
}
else
{
	cout << "La finestra mínima s'aconsegueix des d'una cara NO VEINA" << endl;
}

cout << "xfinal = " << xfinal << endl;
cout << "efinal = " << efinal << endl;


//Si la xfinal ens retorna un vèrtex, llavors cridarem al mètode tractarVertex successivament fins que no ens retorni cap vertex
bool continuar=false;
int jfinal;
pCara cara_final;
jfinal=fminimafinal.getj();
cara_final=fminimafinal.getcara();
Punt puntnou;

/*if ((xfinal<=0+1e-3) || (xfinal>=efinal-1e-3))
{
	continuar=true;
}*/

while (continuar)
{
	if (xfinal<=0+1e-3)
	{
		vp.push_back(cara->vertex((j+1)%3)->point());
		Punt pp(cara->vertex((j+1)%3)->point());
		obj= new Punt(pp);
		objectes_cami_minim->inserirObjecteCamiMinim(obj);
		cout << "TROBAR DISTANCIA MINIMA2 : xfinal=0" << endl;
		cout << "TROBAR DISTANCIA MINIMA2 : Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j+1)%3)->point().x() << " , " << cara->vertex((j+1)%3)->point().y() << " , " << cara->vertex((j+1)%3)->point().z() << ")" << endl;
		//tractarVertex(jfinal,cara_final,fminimafinal,xfinal,puntnou);
		tractarVertex(((j+1)%3),cara,fminimafinal,xfinal,puntnou);
		cara_final=fminimafinal.getcara();
		jfinal=fminimafinal.getj();
		j=jfinal;
		cara=cara_final;
	}
	else if (xfinal>=efinal-1e-3) //Mirar si ha d'éser vertex j ó ((j+1)%3) ó ((j+2)%3)
	{
		vp.push_back(cara->vertex((j+2)%3)->point());
		Punt pp(cara->vertex((j+2)%3)->point());
		obj=new Punt(pp);
		objectes_cami_minim->inserirObjecteCamiMinim(obj);
		cout << "TROBAR DISTANCIA MINIMA2 : xfinal=efinal" << endl;
		cout << "TROBAR DISTANCIA MINIMA2 : Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j+2)%3)->point().x() << " , " << cara->vertex((j+2)%3)->point().y() << " , " << cara->vertex((j+2)%3)->point().z() << ")" << endl;
		//tractarVertex(((j+2)%3),cara_final,fminimafinal,xfinal,puntnou);
		tractarVertex(((j+2)%3),cara,fminimafinal,xfinal,puntnou);
		cara_final=fminimafinal.getcara();
		jfinal=fminimafinal.getj();
		j=jfinal;
		cara=cara_final;
	}
	else
	{
		continuar=false;
	}
}

}


//f és fminimafinal i x és xfinal
void EditorTerreny::saltarVeina(Finestra f, double x, Finestra &fnova, double &xnova, Punt &puntnou, vector<Punt> &vp)
{
Punt2d s;
Punt punt_transformat, interseccio;
pCara cara,cara_veina;
int j,jv,jvv;
double midas1,midas2,mida,e,xnovanova;
double distanciaminimafinal,distanciaminima0,distanciaminimaveina,distanciaminimanoveina;
list<Finestra>::iterator fj,fvj;
Finestra fff;
bool veina,interseca,interseca2,continuar;
interseca=false;
interseca2=false;
continuar=true;
distanciaminimafinal=100; distanciaminimaveina=100; distanciaminimanoveina=100;
Objecte * obj;

//Trobar el pseudoSource de la finestra
s=f.trobarSource();

//Trobar la transformada de la cara_veina
cara=f.getcara();
j=f.getj();
cara_veina=cara->neighbor(j);
jv=cara_veina->index(cara);
	
Punt p(cara->vertex((j+1)%3)->point());
Vector3D u(p,Punt(cara->vertex((j+2)%3)->point()));
Vector3D v(p,Punt(cara->vertex(j)->point()));
Vector3D n=u.producte_vectorial(v);
v=n.producte_vectorial(u);

Vector3D vaux(p,Punt(cara_veina->vertex(cara_veina->index(cara))->point()));
Vector3D naux=u.producte_vectorial(vaux);

double angle=n.angle(naux,true); //false

u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
Transformation3 N(u.x(),v.x(),n.x(),p.x(),u.y(),v.y(),n.y(),p.y(),u.z(),v.z(),n.z(),p.z(),1);
Transformation3 Na=N.inverse();
punt_transformat=Na(cara->vertex(j)->point());
//punt_transformat=cara->info().T[j](cara_veina->vertex(jv)->point());
	
cout << "Punt transformat = (" << punt_transformat.x() << "," << punt_transformat.y() << "," << punt_transformat.z() << ")" << endl;

Punt p1(cara->vertex((j+1)%3)->point());
Punt p2(cara->vertex((j+2)%3)->point());
e=p1.distancia(p2.getPunt());	
cout << "e = " << e << endl;

//Fer segment del (0,0) al vèrtex que hem trobat transformant i mirar si interseca amb la recta que va del Pseudosource al (x,0)
Segment s1(punt_transformat,Punt(0.0,0.0,0.0));
midas1=s1.midaSegment();
Segment r(s,Punt(x,0.0,0.0));
interseca=s1.interseccioLinia(r,interseccio);
	
if (interseca)
{
	cout << "Interseca al primer costat" << endl;
	Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
	xnova=s1.coord_p_on_e(paux)*midas1;
	cout << "xnova = " << xnova << endl;
	mida=midas1;
	cout << "mida = " << mida << endl;
}
else //si no interseca 
{
	cout << "No interseca, la busquem a l'altre costat" << endl;
	Segment s2(Punt(e,0.0,0.0),punt_transformat);
	midas2=s2.midaSegment();
	interseca2=s2.interseccioLinia(r,interseccio);
	Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
	xnova=s2.coord_p_on_e(paux)*midas2;
	cout << "xnova = " << xnova << endl;
	mida=midas2;
	cout << "mida = " << mida << endl;
	//xnova=min(xnova,mida);
}

continuar=true;

while (continuar)
{
	if (xnova<=0+1e-3)
	{
		if (interseca)
		{
			tractarVertex(j,cara,fnova,xnova,puntnou);
			cout << "xnova=0 Interseca" << endl;
			cout << "Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j)%3)->point().x() << " , " << cara->vertex((j)%3)->point().y() << " , " << cara->vertex((j)%3)->point().z() << ")" << endl;
			vp.push_back(cara->vertex(j)->point());
			Punt pp(cara->vertex(j)->point());
			obj = new Punt(pp);
			objectes_cami_minim->inserirObjecteCamiMinim(obj);
			xnovanova=xnova;
		}
		else if (interseca2)
		{
			tractarVertex(((j+2)%3),cara,fnova,xnova,puntnou);
			cout << "xnova=0 Interseca2" << endl;
			cout << "Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j+2)%3)->point().x() << " , " << cara->vertex((j+2)%3)->point().y() << " , " << cara->vertex((j+2)%3)->point().z() << ")" << endl;
			vp.push_back(cara->vertex((j+2)%3)->point());
			Punt pp(cara->vertex((j+2)%3)->point());
			obj = new Punt(pp);
			objectes_cami_minim->inserirObjecteCamiMinim(obj);
			xnovanova=xnova;
		}
		else
		{
			char a;
			cout << "xnova=0 i no interseca ni interseca2 ..." << endl;
			//cin >>a;
		}
		
		if ((xnovanova<=0+1e-3) || (xnovanova>=mida-1e-3))
		{
			continuar=true;
			j=fnova.getj();
			cara=fnova.getcara();
			
			s=fnova.trobarSource();
			cara_veina=cara->neighbor(j);
			jv=cara_veina->index(cara);
				
			Punt p(cara->vertex((j+1)%3)->point());
			Vector3D u(p,Punt(cara->vertex((j+2)%3)->point()));
			Vector3D v(p,Punt(cara->vertex(j)->point()));
			Vector3D n=u.producte_vectorial(v);
			v=n.producte_vectorial(u);
			
			Vector3D vaux(p,Punt(cara_veina->vertex(cara_veina->index(cara))->point()));
			Vector3D naux=u.producte_vectorial(vaux);
			
			double angle=n.angle(naux,true); //false
			
			u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
			Transformation3 N(u.x(),v.x(),n.x(),p.x(),u.y(),v.y(),n.y(),p.y(),u.z(),v.z(),n.z(),p.z(),1);
			Transformation3 Na=N.inverse();
			punt_transformat=Na(cara->vertex(j)->point());
			//punt_transformat=cara->info().T[j](cara_veina->vertex(jv)->point());
				
			cout << "Punt transformat = (" << punt_transformat.x() << "," << punt_transformat.y() << "," << punt_transformat.z() << ")" << endl;
			
			Punt p1(cara->vertex((j+1)%3)->point());
			Punt p2(cara->vertex((j+2)%3)->point());
			e=p1.distancia(p2.getPunt());	
			cout << "e = " << e << endl;
			
			//Fer segment del (0,0) al vèrtex que hem trobat transformant i mirar si interseca amb la recta que va del Pseudosource al (x,0)
			Segment s1(punt_transformat,Punt(0.0,0.0,0.0));
			midas1=s1.midaSegment();
			Segment r(s,Punt(x,0.0,0.0));
			interseca=s1.interseccioLinia(r,interseccio);
			
			if (interseca)
			{
				cout << "Interseca al primer costat" << endl;
				Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
				xnova=s1.coord_p_on_e(paux)*midas1;
				cout << "xnova = " << xnova << endl;
				mida=midas1;
				cout << "mida = " << mida << endl;
			}
			else //si no interseca 
			{
				cout << "No interseca, la busquem a l'altre costat" << endl;
				Segment s2(Punt(e,0.0,0.0),punt_transformat);
				midas2=s2.midaSegment();
				interseca2=s2.interseccioLinia(r,interseccio);
				Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
				xnova=s2.coord_p_on_e(paux)*midas2;
				cout << "xnova = " << xnova << endl;
				mida=midas2;
				cout << "mida = " << mida << endl;
			}
		}
		else
		{
			continuar=false;
		}
	}
	else if  (xnova>=mida-1e-3)
	{
		if (interseca)
		{
			tractarVertex(((j+1)%3),cara,fnova,xnova,puntnou);
			cout << "xnova=mida Interseca" << endl;
			cout << "Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j+1)%3)->point().x() << " , " << cara->vertex((j+1)%3)->point().y() << " , " << cara->vertex((j+1)%3)->point().z() << ")" << endl;
			vp.push_back(cara->vertex((j+1)%3)->point());
			Punt pp(cara->vertex((j+1)%3)->point());
			obj = new Punt(pp);
			objectes_cami_minim->inserirObjecteCamiMinim(obj);
			xnovanova=xnova;
		}
		else if (interseca2)
		{
			tractarVertex(j,cara,fnova,xnova,puntnou);
			cout << "xnova=mida Interseca2" << endl;
			cout << "Insereixo un nou punt de tractar vertex al vector de punts. El punt és (" << cara->vertex((j)%3)->point().x() << " , " << cara->vertex((j)%3)->point().y() << " , " << cara->vertex((j)%3)->point().z() << ")" << endl;
			vp.push_back(cara->vertex(j)->point());
			Punt pp(cara->vertex(j)->point());
			obj = new Punt(pp);
			objectes_cami_minim->inserirObjecteCamiMinim(obj);
			xnovanova=xnova;
		}
		else
		{
			char a;
			cout << "xnova>mida i no interseca ni interseca2 ..." << endl;
			//cin >>a;
		}
		
		if ((xnovanova<=0+1e-3) || (xnovanova>=mida-1e-3))
		{
			continuar=true;
			j=fnova.getj();
			cara=fnova.getcara();
						
			s=fnova.trobarSource();
			cara_veina=cara->neighbor(j);
			jv=cara_veina->index(cara);
				
			Punt p(cara->vertex((j+1)%3)->point());
			Vector3D u(p,Punt(cara->vertex((j+2)%3)->point()));
			Vector3D v(p,Punt(cara->vertex(j)->point()));
			Vector3D n=u.producte_vectorial(v);
			v=n.producte_vectorial(u);
			
			Vector3D vaux(p,Punt(cara_veina->vertex(cara_veina->index(cara))->point()));
			Vector3D naux=u.producte_vectorial(vaux);
			
			double angle=n.angle(naux,true); //false
			
			u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
			Transformation3 N(u.x(),v.x(),n.x(),p.x(),u.y(),v.y(),n.y(),p.y(),u.z(),v.z(),n.z(),p.z(),1);
			Transformation3 Na=N.inverse();
			punt_transformat=Na(cara->vertex(j)->point());
			//punt_transformat=cara->info().T[j](cara_veina->vertex(jv)->point());
				
			cout << "Punt transformat = (" << punt_transformat.x() << "," << punt_transformat.y() << "," << punt_transformat.z() << ")" << endl;
			
			Punt p1(cara->vertex((j+1)%3)->point());
			Punt p2(cara->vertex((j+2)%3)->point());
			e=p1.distancia(p2.getPunt());	
			cout << "e = " << e << endl;
			
			//Fer segment del (0,0) al vèrtex que hem trobat transformant i mirar si interseca amb la recta que va del Pseudosource al (x,0)
			Segment s1(punt_transformat,Punt(0.0,0.0,0.0));
			midas1=s1.midaSegment();
			Segment r(s,Punt(x,0.0,0.0));
			interseca=s1.interseccioLinia(r,interseccio);
			
			if (interseca)
			{
				cout << "Interseca al primer costat" << endl;
				Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
				xnova=s1.coord_p_on_e(paux)*midas1;
				cout << "xnova = " << xnova << endl;
				mida=midas1;
				cout << "mida = " << mida << endl;
			}
			else //si no interseca 
			{
				cout << "No interseca, la busquem a l'altre costat" << endl;
				Segment s2(Punt(e,0.0,0.0),punt_transformat);
				midas2=s2.midaSegment();
				interseca2=s2.interseccioLinia(r,interseccio);
				Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
				xnova=s2.coord_p_on_e(paux)*midas2;
				cout << "xnova = " << xnova << endl;
				mida=midas2;
				cout << "mida = " << mida << endl;
			}
		}
		else
		{
			continuar=false;
		}
	}
	else
	{
		//Mirar a quina finestra està xnova (Segurament el trobarem a alguna finestra de la cara_veina (de la cara_veina anterior) Quan trobem x, per buscar-lo de la finestra de la cara_veina, fer (e-x)
		continuar=false;
		if (interseca)
		{
			fj=cara->info().LlistaFinestres[(j+2)%3].begin();
			fvj=cara->info().LlistaFinestres[(j+2)%3].end();
			cara_veina=cara->neighbor((j+2)%3);
			jv=cara_veina->index(cara);
			//mida=midas1;
		}
		else if (interseca2)
		{
			fj=cara->info().LlistaFinestres[(j+1)%3].begin();
			fvj=cara->info().LlistaFinestres[(j+1)%3].end();
			cara_veina=cara->neighbor((j+1)%3);
			jv=cara_veina->index(cara);
			//mida=midas2;
		}
		else
		{
			char parar;
			cout << "midas1 = " << midas1 << ", midas2 = " << midas2 << endl;
			cout << "xnova = " << xnova << endl;
			cout << "SALTAR VEINA: No interseca ni amb un costat ni amb l'altre ..." << endl;
			//cin >> parar;
		}
					
		//finestres cara
		cout << "Saltar veina : miro finestres cara" << endl;
		while (fj!=fvj)
		{
			fff=*fj;
			distanciaminima0=fff.distancia(xnova);
			if ((xnova>=fff.getb0()-1e-5) && (xnova<=fff.getb1()+1e-5) && (distanciaminima0<distanciaminimafinal))
			{
				fnova=fff;
				veina=false;
				distanciaminimafinal=distanciaminima0;
				cout << "Trobat = true " << endl;
			}
			if (distanciaminima0<distanciaminimanoveina)
			{
				distanciaminimanoveina=distanciaminima0;
			}
			fj++;
		}
				
		//Finestres cara_veina
		cout << "Saltar veina : miro finestres cara_veina" << endl;
		if (!mesh->es_cara_infinita(cara_veina))
		{
			fj=cara_veina->info().LlistaFinestres[jv].begin();
			fvj=cara_veina->info().LlistaFinestres[jv].end();
			
			if (cara_veina->info().LlistaFinestres[jv].size()==0)
			{ 
				//char dd;
				cout << "No hi ha cap finestra a la cara_veina .." << endl;
				//cin >> dd;
			}
		
			while(fj!=fvj)
			{
				fff=*fj;
				distanciaminima0=fff.distancia(xnova,mida,true);
				if ((mida-xnova>=fff.getb0()-1e-5) && (mida-xnova<=fff.getb1()+1e-5) && (distanciaminima0<distanciaminimafinal))
				{
					fnova=fff;
					veina=true;
					distanciaminimafinal=distanciaminima0;
					cout << "Trobat veina = true " << endl;
				}
				if (distanciaminima0<distanciaminimaveina)
				{	
					distanciaminimaveina=distanciaminima0;
				}
				fj++;
			}
		}
		else 
		{
			//char ei;
			cout << "SALTAR VEINA: CARA_VEINA INFINITA!!! ... " << endl;
			//cin >> ei;
		}
				
		//Trobar el puntnou, el qual posteriorment inserirem al vector de punts
		if (veina) 
		{ 
			puntnou=fnova.trobarPunt3d(mida-xnova);
			xnova=mida-xnova;
			cout << "Punt nou trobat a la cara VEINA" << endl;
		}
		else 
		{ 
			puntnou=fnova.trobarPunt3d(xnova);
			cout << "Punt nou trobat a la cara NO_VEINA" << endl;
		}
	
	
		cout << "SALTAR VEINA : xnova = " << xnova << endl;
		
		cout << "Distancia minima no veina = " << distanciaminimanoveina << endl;
		cout << "Distancia minima veina = " << distanciaminimaveina << endl;	
		
	cout << "FNOVA = " << endl;
	fnova.escriure();
	cout << "PuntNou = " << endl; 
	puntnou.escriure(); 
	}

}

}


//Que retorni i ens actualitzi la fnova,la xnova i el puntnou.
void EditorTerreny::tractarVertex(int j, pCara cara, Finestra &fnova, double &xnova, Punt &puntnou)
{
pVertex vertex;
Punt3d punt;
list<Finestra>::iterator fj,fvj,fj0,fvj0;
Finestra f;
double e,distanciaminima0,distanciaminimab0,distanciaminimab1,distanciaminimafinal,midas1,x,xb0,xb1,xfinal,efinal;
int index_cara,index_cara_veina;
Punt2d s;
bool interseca,veina;
Punt interseccio,punt_transformat,punt_transformat_vei;
pCara cara_veina;

distanciaminimafinal=100;
punt=cara->vertex(j)->point();
vertex=cara->vertex(j);

//per cadascuana de les de davant per davant, buscar la de distància mínima.
CirculadorCares fit=(mesh->triangulacio()->incident_faces(vertex,cara));
fit++;
do{
	if(!mesh->es_cara_infinita(cara)) //Només fem quan no és una cara infinita
	{		
		index_cara=fit->index(vertex); //Així obtenim l'index del vertex en cada cara adjacent fit
		
		Punt p1(fit->vertex((index_cara+1)%3)->point());
		Punt p2(fit->vertex((index_cara+2)%3)->point());
		e=p1.distancia(p2.getPunt());	
		
		cara_veina=fit->neighbor(index_cara);
	
		fj=fit->info().LlistaFinestres[index_cara].begin();
		fvj=fit->info().LlistaFinestres[index_cara].end();
			
		if (!mesh->es_cara_infinita(cara_veina))
		{
			Punt p(fit->vertex((index_cara+1)%3)->point());
			Vector3D u(p,Punt(fit->vertex((index_cara+2)%3)->point()));
			Vector3D v(p,Punt(fit->vertex(index_cara)->point()));
			Vector3D n=u.producte_vectorial(v);
			v=n.producte_vectorial(u);
			
			Vector3D vaux(p,Punt(cara_veina->vertex(cara_veina->index(fit))->point()));
			Vector3D naux=u.producte_vectorial(vaux);
			
			double angle=n.angle(naux,true); //false
			
			u.transforma_unitari(); v.transforma_unitari(); n.transforma_unitari();//naux.transforma_unitari();
			Transformation3 N(u.x(),v.x(),n.x(),p.x(),u.y(),v.y(),n.y(),p.y(),u.z(),v.z(),n.z(),p.z(),1);
			Transformation3 Na=N.inverse();
			punt_transformat=Na(punt);
			
			while (fj!=fvj)
			{
				f=*fj;
				s=f.trobarSource();
				Segment s1(Punt(f.getb0(),0.0,0.0),Punt(f.getb1(),0.0,0.0));
				midas1=s1.midaSegment();
				Segment r(s,punt_transformat);
				interseca=s1.interseccioLinia(r,interseccio);
			
				if (interseca)
				{
					Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
					x=s1.coord_p_on_e(paux)*midas1+f.getb0();
					distanciaminima0=punt_transformat.distancia(paux)+f.distancia(x);
					if (distanciaminima0<distanciaminimafinal)
					{
						distanciaminimafinal=distanciaminima0;
						fnova=f;
						xfinal=x;
						veina=false;
					}	
				}
				else
				{
					xb0=f.getb0();
					distanciaminimab0=punt_transformat.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0);
					xb1=f.getb1();
					distanciaminimab1=punt_transformat.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1);
					if (distanciaminimab0<=distanciaminimab1)
					{
						distanciaminima0=distanciaminimab0;
						x=xb0;
					}
					else
					{
						distanciaminima0=distanciaminimab1;
						x=xb1;
					}
								
					if (distanciaminima0<distanciaminimafinal)
					{
						distanciaminimafinal=distanciaminima0;
						fnova=f;
						xfinal=x;
						veina=false;
					}	
				}
				fj++;
			}
			
			index_cara_veina=cara_veina->index(fit);
			punt_transformat_vei=fit->info().T[index_cara_veina](punt);
			fj0=cara_veina->info().LlistaFinestres[index_cara_veina].begin();
			fvj0=cara_veina->info().LlistaFinestres[index_cara_veina].end();
				
			while (fj0!=fvj0)
			{
				f=*fj0;
				f.escriure();
				s=f.trobarSource(e,true);
				Segment s1(Punt(e-f.getb0(),0.0,0.0),Punt(e-f.getb1(),0.0,0.0));
				midas1=s1.midaSegment();
				Segment r(s,punt_transformat_vei);
				interseca=s1.interseccioLinia(r,interseccio);
			
				if (interseca)
				{
					Punt3d paux(interseccio.x(),interseccio.y(),interseccio.z());
					x=e-f.getb0()-s1.coord_p_on_e(paux)*midas1;
					distanciaminima0=punt_transformat_vei.distancia(paux)+f.distancia(x,e,true);
					//cout << "distanciaminima0= " << distanciaminima0 << endl;
					if (distanciaminima0<distanciaminimafinal)
					{
						distanciaminimafinal=distanciaminima0;
						fnova=f;
						xfinal=x;
						veina=true;
						efinal=e;
					}	
				}
				else
				{
					xb0=e-f.getb0();
					distanciaminimab0=punt_transformat_vei.distancia(Punt3d(xb0,0.0,0.0))+f.distancia(xb0,e,true);	
					xb1=e-f.getb1();
					distanciaminimab1=punt_transformat_vei.distancia(Punt3d(xb1,0.0,0.0))+f.distancia(xb1,e,true);
					if (distanciaminimab0<=distanciaminimab1)
					{
						distanciaminima0=distanciaminimab0;
						x=xb0;
					}
					else
					{
						distanciaminima0=distanciaminimab1;
						x=xb1;
					}
					
					if (distanciaminima0<distanciaminimafinal)
					{
						distanciaminimafinal=distanciaminima0;
						fnova=f;
						xfinal=x;
						veina=true;
					}	
				}
				fj0++;
			}
		
		}			
	
			
	}
fit++;	
} while (fit!=cara);

if (veina) 
{
	cout << "TRACTAR VERTEX: La finestra mínima s'aconsegueix des d'una cara VEINA" << endl;
	xnova=efinal-xfinal;
	
}
else
{
	cout << "TRACTAR VERTEX: La finestra mínima s'aconsegueix des d'una cara NO VEINA" << endl;
	xnova=xfinal;
}

//xnova=xfinal;
puntnou=fnova.trobarPunt3d(xnova);

cout << "TRACTAR VERTEX: efinal = " << efinal << endl;
cout << "TRACTAR VERTEX: xnova = " << xnova << endl;
cout << "TRACTAR VERTEX: FNOVA = " << endl;
fnova.escriure();
cout << "TRACTAR VERTEX: PuntNou = " << endl; 
puntnou.escriure(); 

}


void EditorTerreny::trobaCami()
{
info_mostrar.pintar_cami=TRUE;
repaint();
}


/***************************************************************/
/*   void EditorTerreny::calculsCaminsMinims()    */
/*Mètode que realitza el calcul dels camí mínim entre un pseudosource i un punt*/
/***************************************************************/
void EditorTerreny::calculsCaminsMinims()
{
Objecte * obj;
Objecte * obj2;
int k,kk,mida_sources;
Punt3d punt3d, punt3dinicial;
Punt puntgroc;
int vertex;
Localitzacio tipus;
pCara cara,cara_source,cara_actual;
vector<Punt> vp;
Punt punt,punt2,punt3;
bool trobat;

cout << "\nEntro dins calculsCaminsMinims" << endl;
//triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10 ); 


for(k=0; k<objectes_distancies->numeroObjectesDistancies(); k++)
{
	obj = objectes_distancies->obtenirObjecteDistancies(k+1); //Obtenim els objectes que hem inserit al vector Distancies
		
	switch(obj->tipusObjecte())
	{
		case PUNT_VISIO: 
		punt3d=((Punt *)obj)->getPunt();
		cout << "Punt = " << punt3d.x() << " , " << punt3d.y() << " , " << punt3d.z() << endl;
						
		cara=mesh->triangulacio()->locate(punt3d,tipus,vertex); //Trobem el triangle on està el punt
		cout << "pCara que conté el punt = " << cara->vertex(0)->point() << " , " << cara->vertex(1)->point() << " , " << cara->vertex(2)->point() << endl;
		
		glColor3f(1.0,0.0,0.0);
		glLineWidth(2);

		//objectes_cami_minim->clear();		
		//Per cada punt groc calcularem i dibuixarem el camí mínim al source més proper
		vp=camiMesCurt(punt3d,cara,cara_source);		
			
		mida_sources=info_calculs.caresSource.size();
		kk=1;
		trobat=false;
		while ((kk<=mida_sources) && (!trobat))
		{
			cara_actual=info_calculs.caresSource[kk-1];
			if (cara_actual==cara_source)
			{
				obj2 = objectes_visio->obtenirObjecte(kk);
				//objectes_cami_minim->inserirObjecteCamiMinim(obj2);
				trobat=true;
			}
			else
			{
				kk=kk+1;
			}
		}
		
		if(obj2->tipusObjecte()==1)
		{
			punt3dinicial=((Punt *)obj2)->getPunt();
			cout << "Punt source inicial= " << punt3dinicial.x() << " , " << punt3dinicial.y() << " , " << punt3dinicial.z() << endl;
			Objecte * o = new Punt(punt3dinicial);
			objectes_cami_minim->inserirObjecteCamiMinim(o);
		}

		if(vp.size()==1) //Si només ens torna un punt, vol dir que tots dos estan a la mateixa cara, i només els hem d'ajuntar
		{
			punt=vp.back();
			cout << "dibuixem la linia entre el punt groc i el source" << endl;
			glEnable( GL_DEPTH_TEST );
			glDepthFunc(GL_LESS);
			glBegin(GL_LINES);
				glVertex3f(punt.x(),punt.y(),0.0);
				glVertex3f(punt3dinicial.x(),punt3dinicial.y(),0.0);
			glEnd();
			vp.pop_back();	
		}
		
		else //Ens torna més d'un punt 
		{
			//Ajuntem l'ultim punt del vector amb el punt inicial del source
			punt=vp.back();
			punt.escriure();
			vp.pop_back();
			glEnable( GL_DEPTH_TEST );
			glDepthFunc(GL_LESS);
			glBegin(GL_LINES);
				glVertex3f(punt.x(),punt.y(),0.0);
				glVertex3f(punt3dinicial.x(),punt3dinicial.y(),0.0);
			glEnd();
			cout << "dibuixem la línia entre el source i l'ultim punt del metode" << endl;
						
			while(vp.size()>1)
			{
				punt2=vp.back();
				punt2.escriure();
				vp.pop_back();
				glEnable(GL_DEPTH_TEST);
				glDepthFunc(GL_LESS);
				glBegin(GL_LINES);
					glVertex3f(punt.x(),punt.y(),0.0);
					glVertex3f(punt2.x(),punt2.y(),0.0);
				glEnd();
				punt=punt2;
				cout << "dibuixem les línies entre punts intermitjos" << endl;	
			}	
				
			//Ara ja només ens falta ajuntar l'últim punt (el groc)
			punt2=vp.back();
			punt2.escriure();
			vp.pop_back();
			glEnable(GL_DEPTH_TEST);
			glDepthFunc(GL_LESS);
			glBegin(GL_LINES);
				glVertex3f(punt.x(),punt.y(),0.0);
				glVertex3f(punt2.x(),punt2.y(),0.0);
			glEnd();
			cout << "dibuixem la línia entre el punt p i el segon punt" << endl;
		}
		cout << "He dibuixat el cami minim\n" << endl;
									
		break;
		
		case SEGMENT_VISIO: 
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Operacio temporalment no disponible per segments."); 
		break; 
		
		case POLIGON_VISIO:
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Operacio temporalment no disponible per poligons."); 
		break;
		
		case POLIGONAL_VISIO:
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Error" ));
		errors->message( "Operacio temporalment no disponible per poligonals."); 
		break; 
	}
}

}


/*****************************************************************************************************/
/*   void EditorTerreny::comprovarPoligonMes2Vertexs()                                               */
/*Mètode que comprova que el poligon que s'esta definint sigui correcte quan el numero de punts és >2*/
/*****************************************************************************************************/

bool EditorTerreny::comprovarPoligonMes2Vertexs(QMouseEvent *e, Punt mon)
{
	/*Si un punt esta a l'esquerra i l'altre a la dreta llavors existeix interseccio*/
	bool interseccio=false;
	unsigned int i=0;
	
	if(info_vis.punts_poligon.size()<2)
		return false;
		
	while(i<info_vis.punts_poligon.size()-2 && !interseccio)
	{
		if( e->button()==LeftButton ) /*En el cas que sigui l'últim punt del poligon*/
		{
			Segment s1(info_vis.punts_poligon[info_vis.punts_poligon.size()-i-2], info_vis.punts_poligon[info_vis.punts_poligon.size()-i-1]);
			Segment s2(mon, info_vis.punts_poligon[0]);
			interseccio = s1.interseccioSegments(s2);
		}
		if(!interseccio)
		{
			Segment s1(info_vis.punts_poligon[i], info_vis.punts_poligon[i+1]);
			Segment s2(mon, info_vis.punts_poligon[info_vis.punts_poligon.size()-1]);
			interseccio = s1.interseccioSegments(s2);
		}
		i++;
	}
	if(!interseccio) /*No hi ha interseccio i, per tant, puc afegir el nou punt mon a la llista de punts que formaran el poligon irregular*/
	{
		mon.modifica_z( info_vis.pla_poligon->calcula_Z(mon, info_vis.punts_poligon[1]) );
		info_vis.punts_poligon.push_back(mon);
		
		Poligon copia=Poligon(info_vis.punts_poligon, info_vis.punts_poligon.size());
		
		if(mesh->poligon_visio_intersecciona_terreny(copia))
		{
			info_vis.punts_poligon.pop_back();
			interseccio=true;
			errors = new QErrorMessage( this );
			errors->setCaption( tr( "Vertex incorrecte" ));
			errors->message( "El poligon definit intersecciona amb el terreny!"); 
		}
	}
	else
	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Vertex incorrecte" ));
		errors->message( "El punt introduit fa que el poligon irregular sigui incorrecte!\nIntrodueix un nou punt."); 
	}
	
	return interseccio;
}

/******************************************************************************************************/
/*   void EditorTerreny::comprovarPoligonMenys2Vertexs()                                              */
/*Mètode que comprova que el poligon que s'esta definint sigui correcte quan el numero de punts és <=2*/
/******************************************************************************************************/

bool EditorTerreny::comprovarPoligonMenys2Vertexs(const double &x, const double &y, const double &z)
{
	bool interseccio=false;
	Objecte * punter;
	Punt mon(x, y, z);
	switch(info_vis.punts_poligon.size())
	{
		case 0: /*Comprovació pel primer punt del poligon*/
			if(mon.puntCorrecte(mesh->triangulacio()))
			{
				info_vis.punts_poligon.push_back(Punt( x, y, z ));
			}
			else
			{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Punt de visio incorrecte" ));
				errors->message( "El punt introduit es troba fora o per sota de la triangulacio! Cal que es trobi per sobre d'aquesta."); 
				interseccio=true;
			}
			break;
		case 1: /*Comprovació pel segment que forma la primera aresta del poligon*/
			punter = new Segment( info_vis.punts_poligon[0], mon);
			if(((Segment *)punter)->segmentCorrecte(mesh->triangulacio()))
			{
				info_vis.punts_poligon.push_back(Punt( x, y, z ));
			}
			else
			{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Vertex poligon de visio incorrecte" ));
				errors->message( "El punt introduit com a vertex fa que l'aresta interseccioni amb la triangulacio! Cal que es trobi totalment per sobre d'aquesta."); 
				interseccio=true;
			}
			delete punter;
			break;
		case 2:
			punter = new Segment( info_vis.punts_poligon[1], mon);
			if(((Segment *)punter)->segmentCorrecte(mesh->triangulacio()))
			{
				info_vis.pla_poligon = new Pla(info_vis.punts_poligon[0], info_vis.punts_poligon[1], mon);
				if(info_vis.pla_poligon->es_vertical(info_vis.punts_poligon[1]))
				{
					errors = new QErrorMessage( this );
					errors->setCaption( tr( "Poligon de visio incorrecte" ));
					errors->message( "Els punts introduits fins al moment fan que el poligon es trobi en un pla vertical, i per tant, incorrecte! Introdueix un nou vertex.");
					interseccio=true;
				}
				else
				{
					info_vis.punts_poligon.push_back(Punt( x, y, z ));
				}
			}
			else
			{
				errors = new QErrorMessage( this );
				errors->setCaption( tr( "Vertex poligon de visio incorrecte" ));
				errors->message( "El punt introduit com a vertex fa que l'aresta interseccioni amb la triangulacio! Cal que es trobi totalment per sobre d'aquesta."); 
				interseccio=true;
			}
			delete punter;
			break;
	}
	return interseccio;
}

/*****************************************************************************/
/*   void EditorTerreny::tractarIniciSegment()                               */
/*Mètode que fa el tractament de la insercio de l'inici d'un segment de visio*/
/*****************************************************************************/

void EditorTerreny::tractarIniciSegment()
{
	if(info_vis.inici_segment.puntCorrecte(mesh->triangulacio()))
	{
		info_vis.punt_segment=true;
	}
	else
	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Inici de segment de visio incorrecte" ));
		errors->message( "El punt introduit com a inici de segment es troba fora o per sota de la triangulacio! Cal que es trobi per sobre d'aquesta."); 
	}
}

/*****************************************************************************/
/*   void EditorTerreny::tractarIniciSegment()                               */
/*Mètode que fa el tractament de la insercio del final d'un segment de visio */
/*****************************************************************************/

void EditorTerreny::tractarFiSegment( Objecte * punter )
{
	if(((Segment *)punter)->segmentCorrecte(mesh->triangulacio()))
	{
		info_vis.punt_segment=false;
	}
	else
	{
		errors = new QErrorMessage( this );
		errors->setCaption( tr( "Fi de segment de visio incorrecte" ));
		errors->message( "El punt introduit com a fi de segment fa que el segment interseccioni amb la triangulacio! Cal que es trobi totalment per sobre d'aquesta."); 
	}
}

/****************************************************************************/
/*   void EditorTerreny::mousePressEvent( QMouseEvent *e )                  */
/*Mètode que s'executa en el moment en que un dels botons del ratolí es prem*/
/****************************************************************************/

void EditorTerreny::mousePressEvent( QMouseEvent *e )
{
    GLuint selectBuf[BUFSIZE];
    GLint hits;
    GLint viewport[4];
    int xrata, yrata;     

    if (accio == GL_SELECT)  /*En el cas en que ens trobem en mode selecció*/
    {   
      if ( e->button() == QMouseEvent::RightButton ) /*En el cas en que el botó que s'hagi pres sigui el dret es desplega el menú que permet a l'usuari modificar, visualitzar o eliminar el punt seleccionat*/
      {
        if (info_mostrar.seleccionat != 0) auxmenu->popup( QCursor::pos() ); 
      }
      else if ( e->button() == QMouseEvent::LeftButton ) /*En el cas en que el botó premut sigui l'esquerra farem les operacions següents*/
      {
        makeCurrent();
        glGetIntegerv ( GL_VIEWPORT, viewport );  
        pressed = TRUE; 

	/*Ens guardem les coordenades en les quals es troba el ratolí en el moment en que s'ha pres el botó*/
        xrata = e->x();
        yrata = e->y();
       
        glSelectBuffer( BUFSIZE, selectBuf ); /*glSelectBuffer( GLsizei size, GLuint *buffer ); -> Instrucció que permet especificar quina és l'array que s'utilitzarà on el primer argument indica el número d'elements de l'array,amb la qual cosa es determina el número màxim d'objectes que poden estar superposats en una mateixa zona de la finestra*/
	
        glRenderMode( GL_SELECT ); /*Indiquem que el mode actual és de selecció*/

        glInitNames();
        glPushName(0);

        glMatrixMode( GL_PROJECTION );
        glPushMatrix();
        glLoadIdentity();
        gluPickMatrix( (GLdouble)xrata, (GLdouble)(viewport[3] - yrata), 10.0, 10.0, viewport );        
        glOrtho(info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -1.0, 1.0 );                
        dibuixaPunts(GL_SELECT);
        glFlush(); 
        glPopMatrix();
        hits = glRenderMode( GL_RENDER );   /*Mètode que retorna el número d'objectes que es troben sota del ratolí*/
        if( hits > 0 ) processHits (hits, selectBuf);
        else info_mostrar.seleccionat = 0;
        updateGL();
      }
    }
}

/*********************************************************************/
/*   void EditorTerreny::mouseMoveEvent( QMouseEvent *e)             */
/*Mètode que es executat quan el ratolí es mou sobre l'editor OpenGL */
/*********************************************************************/

void EditorTerreny::mouseMoveEvent( QMouseEvent *e)
{
    GLint viewport[4];
    GLdouble modelview[16];
    GLdouble projection[16];

    Punt3d aux;
    double x, y, z;
    int xrata, yrata;
    
    /*A partir de les coordenades del ratolí i les mides actuals de la finestra, es calculen les coordenades del món*/
    x = y = 0.0;
    xrata = e->x();
    yrata = e->y();

    makeCurrent();

    glGetDoublev( GL_MODELVIEW_MATRIX, modelview );
    glGetDoublev( GL_PROJECTION_MATRIX, projection );
    glGetIntegerv( GL_VIEWPORT, viewport );  
  
    gluUnProject( (GLdouble)xrata, (GLdouble)(viewport[3] - yrata), -1.0, modelview, projection, viewport, &x, &y, &z );
    
   
    if(info_mostrar.veure_parts_visibles)
    {
    	if(info_vis.mostrar.tipus_visualitzacio== COMBINAT)
		emit rataStatus(x,y);
	else
	{
		vector<float> uc_buff(3,0);
		glReadPixels(xrata,height()-yrata,1,1, GL_RGB, GL_FLOAT, &uc_buff[0]);
		bool counting =false;
		int vis=0;
		if(info_vis.mostrar.tipus_visualitzacio != COUNTING)
		{
			if(uc_buff[0]==1  ) vis=1;
			else if(info_vis.mostrar.tipus_visualitzacio == INTERSECCIO && uc_buff[1]==1) vis=1;
		}
		else
		{
			counting=true;
			for(unsigned int i=0;i<uc_buff.size();i++)
				vis=vis+round(uc_buff[i]*10.);
		}
			
		emit mostrarInfVisibilitatAmbRata(vis,counting,x,y);
	}
    }
    else  
   	  emit rataStatus( x, y );
    if (accio == GL_SELECT) /*En el cas que ens trobem en mode selecció*/
    {      
      if ( pressed ) /*Si el botó esquerra està pres*/
      {
        if(info_mostrar.seleccionat != 0) /*Si ens trobem en selecció. L'atribut selecció indica el punt que es troba seleccionat, indicant la seva posició segons l'ordre de creació*/
        {
          if (x < info_mon.xmin) x = info_mon.xmin;
          if (x > info_mon.xmax) x =info_mon. xmax;
          if (y < info_mon.ymin) y = info_mon.ymin;
          if (y > info_mon.ymax) y = info_mon.ymax;
//           model->elimina( vertexs[info_mostrar.seleccionat - 1] );
//           model->insereix( Punt3d( x, y, vertexs[info_mostrar.seleccionat - 1]->point().z() ) );
	  
	  mesh->eliminarPunt(info_mostrar.seleccionat); /*A partir del vector de vèrtexs que s'han anat creant accedim al que ocupa la posició: seleccio-1, corresponent al vèrtex que es troba actualment seleccionat. Aquest vèrtex és esborrat de l'estructura de Delaunay per ser tot seguit introduit de nou amb les noves coordenades*/
	  
          mesh->inserirPuntMogut( Punt3d( x, y, mesh->obtenirPunt(info_mostrar.seleccionat)->point().z() ), info_mostrar.seleccionat ); /*A continuació es torna a inserir el nou punt amb les coordenades calculades anteriorment però amb la mateixa alçada*/
	  
	  /*Comprovem el valor de les coordenades (x, y) del nou punt per tal de comprovar si aquest es troba als extrems*/
	  if(posicio == info_mostrar.seleccionat)
	  {
	  	info_mon.vertex_xmin = mesh->vertex_xmin( posicio);
		
		/*Calcul del tall de segment que talla i es troba contingut dins del triangle escollit*/
	  }
	  else
	  {
	  	if(x < info_mon.vertex_xmin.x()) 
		{
			info_mon.vertex_xmin.modifica_coordenades( x, y, z );
			posicio = mesh->numero_de_vertexs();
		}
	  }     
          updateGL();
          setModificat();
          emit terrenyModificat();  
        }
      }
    }
    else if ( info_vis.punt_segment || info_vis.dibuixant_poligon )
    {
    	info_vis.punt_actual.modifica_coordenades(x, y, 0.0);
    	repaint();
    }

}

/**************************************************/
/*   void EditorTerreny::eliminaElement()       */
/*Mètode que permet eliminar el vèrtex seleccionat*/
/**************************************************/

void EditorTerreny::eliminaElement()
{   
	if ((accio == GL_SELECT) && (info_mostrar.seleccionat != 0))/*En el cas que ens trobem en mode selecció, i haguem info_mostrar.seleccionat un dels punts representats*/
	{
		mesh->eliminarPunt(info_mostrar.seleccionat); /*Eliminem de la triangulació de Delaunay el vèrtex escollit*/
		/*Tot seguit s'elimina de la llista de vèrtexs el triat fent que aquesta llista quedi reordenada*/
		
		mesh->eliminarVertexs(info_mostrar.seleccionat);
		
		info_mostrar.seleccionat = 0; /*L'atribut de selecció passa a valer 0 ja que el punt ja ha desaparegut*/
		updateGL(); /*Repintem la plantilla d'OpenGL*/
		setModificat();
		emit terrenyModificat();
	}
}

/****************************************************************************************/
/*   void EditorTerreny::clear()                                                        */
/*Mètode que elimina tot el que pugui haver-hi en el terreny inicialitzant-ho tot a buit*/
/****************************************************************************************/

void EditorTerreny::clear()
{
   mesh->clear();
   objectes_visio->clear();
   objectes_distancies->clear();
   objectes_cami_minim->clear();
   info_mostrar.seleccionat = 0;
   info_vis.visio = INDEFINIT;
   info_vis.punt_segment=false;
   info_vis.punts_pla_poligon = false;
   info_vis.dibuixant_poligon = false;
   info_vis.tip = INDEFINIT;
   
   info_mostrar.dibuixa = FALSE; 
   updateGL();
   emit terrenyModificat();
}

/*********************************************************************/
/*   void EditorTerreny::processHits (GLint hits, GLuint buffer[])   */
/*Mètode que permet determinar quin dels vèrtexs ha estat seleccionat*/
/*********************************************************************/

void EditorTerreny::processHits (GLint hits, GLuint buffer[])
{
    GLint i;
    GLuint j;
    GLuint names, *ptr;
    
    ptr = (GLuint *) buffer; 
    for (i = 0; i < hits; i++) /*Per cadascun dels objectes seleccionats*/
    {   
        /* Per cada hit */
        names = *ptr;
        ptr++;
        ptr++;
        ptr++;
        for (j = 0; j < names; j++) 
	{   
	     /* Per cada nom */
            info_mostrar.seleccionat = *ptr;
	    ptr++;	
        }
    }
    /*Un cop finalitzada la cerca acabarem obtinguent el vèrtex que ha estat seleccionat, per poder posteriorment accedir-hi mitjançant: vertexs[seleccio-1]*/
}

/*************************************************************************************/
/*   void EditorTerreny::setAccio( GLenum act )                                      */
/*Mètode que permet modificar la variable accio segons el valor passat per paràmetres*/
/*************************************************************************************/

void EditorTerreny::setAccio( GLenum act )
{
     if (act == GL_RENDER) 
     {
        accio = GL_RENDER;
        info_mostrar.seleccionat = 0;
        updateGL();
     }
     else accio = GL_SELECT;
}

/**************************************************************************/
/*   void EditorTerreny::setVisio( int vis )                              */
/*Mètode que permet indicar el tipus d'element de visio es vol representar*/
/**************************************************************************/

void EditorTerreny::setVisio( int vis )
{
     accio = GL_RENDER;
     info_mostrar.seleccionat = 0;
     updateGL();
     
     info_vis.visio = vis;
     cout<<"Visio: "<<info_vis.visio<<endl;
     info_mostrar.veure_elements_visio = true;      
}


/**************************************************************************/
/*   void EditorTerreny::ConfiguracioVisio()                               */
/*Mètode que permet indicar que estem en el mode Visibilitat*/
/**************************************************************************/

void EditorTerreny::configuracioVisio()
{
   //cout << "\nEstem en mode visibilitat";
   info_calculs.mode_visibilitat=TRUE;
   info_calculs.mode_distancies=FALSE;
   //info_mostrar.veure_elements_visio=TRUE;        
}

/**************************************************************************/
/*   void EditorTerreny::ConfiguracioDistancies()                         */
/*Mètode que permet indicar que estem en el mode Distancies*/
/**************************************************************************/

void EditorTerreny::configuracioDistancies()
{
    //cout << "\nEstem en mode distancies";
    info_calculs.mode_distancies=TRUE;
    info_calculs.mode_visibilitat=FALSE;
    //info_mostrar.veure_elements_visio=FALSE;           
}

/**************************************************************************/
/*   void EditorTerreny::ConfiguracioPuntDistancies()                         */
/*Mètode que permet indicar que volem inserir un punt en el mode Distancies*/
/**************************************************************************/

void EditorTerreny::configuracioPuntDistancies()
{
    info_calculs.punt_distancies=TRUE;
}

/*******************************************************/
/*   bool EditorTerreny::getModeVisibilitat()                   */
/*Mètode que retorna el valor de la variable mode_visibilitat */
/*******************************************************/

bool EditorTerreny::getModeVisibilitat()
{
    return info_calculs.mode_visibilitat;
}

/*******************************************************/
/*   bool EditorTerreny::getModeDistancies()()                   */
/*Mètode que retorna el valor de la variable mode_distancies*/
/*******************************************************/

bool EditorTerreny::getModeDistancies()
{
    return info_calculs.mode_distancies;
}

/***********************************************************/
/*   void EditorTerreny::modificaZ()                       */
/*Mètode que permet modificar l'alçada del punt seleccionat*/
/***********************************************************/

void EditorTerreny::modificaZ()
{      
      double z = 0.0;
      ZDialeg *cz = new ZDialeg( this );
      cz->limits( 0.0, info_mon.zmax );
      cz->per_defecte( mesh->obtenirPunt(info_mostrar.seleccionat)->point().z() );
      if( cz->exec() == QDialog::Accepted )
      {
         z = cz->alcada();         
         mesh->modificarPunt(info_mostrar.seleccionat, z);
         setModificat();
         emit terrenyModificat();
      }
      delete cz;
}

/******************************************************/
/*   void EditorTerreny::borrarElement()              */
/*Mètode que simplement crida al mètode eliminaElement*/
/******************************************************/

void EditorTerreny::borraElement()
{
    eliminaElement();
}

/********************************************************/
/*   void EditorTerreny::mostraInfo()                   */
/*Mètode que mostra les coordenades del punt seleccionat*/
/********************************************************/

void EditorTerreny::mostraInfo()
{
   pVertex punt;
  
   if( info_mostrar.seleccionat != 0 )
   {
      InfoDialeg *idg = new InfoDialeg( this );
      
      punt=mesh->obtenirPunt(info_mostrar.seleccionat);

      idg->modificaText(punt->point().x(),
                        punt->point().y(),
                        punt->point().z() );
      
      idg->exec();
   }
}

/*******************************************************/
/*   bool EditorTerreny::modificat()                   */
/*Mètode que retorna el valor de la variable modificat */
/*******************************************************/

bool EditorTerreny::modificat()
{
    return info_mon._modificat;
}

/**************************************************/
/*   void EditorTerreny::setModificat()           */
/*Mètode que assigna True a la variable info_mon._modificat*/
/**************************************************/

void EditorTerreny::setModificat()
{
    info_mon._modificat = TRUE;
}

/*************************************************************************/
/*   void EditorTerreny::dibuixaPunts( GLenum mode, const QColor &color )*/
/*Mètode que dibuixa els punts de la triangulació del color escollit     */
/*************************************************************************/

void EditorTerreny::dibuixaPunts( GLenum mode, const QColor &color )
{
    glPointSize( 3.0 );
    GLuint pos = 0;    
    for( IteradorVertexs pit = (mesh->triangulacio())->finite_vertices_begin(); pit != (mesh->triangulacio())->finite_vertices_end(); ++pit )
    {

      if ( mode == GL_SELECT ) glLoadName( pos + 1 );  
      //qglColor( Qt::white ); 
      glBegin( GL_POINTS );
         qglColor( color );
         glVertex2d( pit->point().x(), pit->point().y() );              
      glEnd();
      pos++;   
    }
    
}

/*********************************************************************************/
/*   void EditorTerreny::dibuixaTriangulacio2D( const QColor &clinies )          */
/*Mètode que dibuixa la triangulació, és a dir, les arestes d'unió dels triangles*/
/*********************************************************************************/

void EditorTerreny::dibuixaTriangulacio2D( const QColor &clinies )
{
	glLineWidth(0.5);
	Punt p0, p1, p2;
	IteradorCares fit = (mesh->triangulacio())->finite_faces_begin();
	for( ; fit != (mesh->triangulacio())->finite_faces_end(); ++fit)
	{
		pVertex vh0 = (*fit).vertex(0);
		pVertex vh1 = (*fit).vertex(1);
		pVertex vh2 = (*fit).vertex(2);  
		
		glBegin( GL_LINE_LOOP );
			glColor3ub( clinies.red(), clinies.green(), clinies.blue() );
			/*Generem les tres arestes que crearan el triangle a través dels punts guaradats anteriorment.*/
			glVertex2d( vh0->point().x(), vh0->point().y() );
			glVertex2d( vh1->point().x(), vh1->point().y() );
			glVertex2d( vh2->point().x(), vh2->point().y() );
		glEnd();
	}
}

/****************************************************************************************************/
/*   void EditorTerreny::mostraCorbes( bool b )                                                     */
/*Mètode que activa les variables que fan que es mostrin les corbes de nivell sobre el terreny en 2D*/
/****************************************************************************************************/

void EditorTerreny::mostraCorbes( bool b )
{
  info_mostrar.veure_corbes = b;
  updateGL();
}

/***********************************************************************/
/*   void EditorTerreny::mostraTriangulacio( bool b )                  */
/*Mètode que activa les variables que fan que es mostri la triangulació*/
/***********************************************************************/

void EditorTerreny::mostraTriangulacio( bool b )
{
  info_mostrar.veure_triangulacio = b;
  updateGL();
}

/*****************************************************************************/
/*   void EditorTerreny::veureTextura(bool b)                                */
/*Mètode que activa les variables que fan que es mostri la textura equivalent*/
/*****************************************************************************/

void EditorTerreny::veureTextura(bool b)
{
	info_mostrar.veure_parts_visibles = b;
	updateGL();
}
/*********************************************************************************/
/*   void EditorTerreny::mostraVertexs( bool b )                                 */
/*Mètode que activa les variables que permeten veure els punts de la triangulació*/
/*********************************************************************************/

void EditorTerreny::mostraVertexs( bool b )
{
  info_mostrar.veure_vertexs = b;
  updateGL();
}

/******************************************************/
/*   void EditorTerreny::mostraElementsVisio()        */
/*Mètode que permet fer visibles els elements de visio*/
/******************************************************/

void EditorTerreny::mostraElementsVisio( bool b)
{
  info_mostrar.veure_elements_visio = b;
  if(!b) info_vis.visio=INDEFINIT;
  updateGL(); /*Repintem la plantilla d'OpenGL*/	
}

/*****************************************************************************/
/*   void EditorTerreny::dibuixaCorbes()                                     */
/*Mètode que dibuixa les corbes de nivell en 2D, a partir de una LlistaCorbes*/
/*****************************************************************************/

void EditorTerreny::dibuixaCorbes()
{
  LlistaCorbes lc;
  mesh->corbes2D(lc);

  LlistaPunts::iterator pit, inici_p, fi_p;
  LlistaCorbes::iterator pic, inici_c, fi_c;
  //Punt3d punt_inicial;
  //makeCurrent();
   
  inici_c = lc.begin();
  fi_c = lc.end();

  QColor color_c;
/*
qDebug("--------------------------");
  for( pit = inici; pit != fi; ++pit ) 
     qDebug( "PUNT: (%f, %f, %f).", pit->x(), pit->y(), pit->z() );

qDebug("--------------------------");
*/
  for( pic = inici_c; pic != fi_c; ++pic )
  {
    //qDebug("MIDA = %d",pic->punts.size());
    inici_p = pic->punts.begin();
    fi_p = pic->punts.end();
//qDebug("Hola");

    color_c = pic->color;

    glBegin( GL_LINES );
    qglColor(color_c );
    //pit = inici; 
    for( pit = inici_p; pit != fi_p; ++pit )
    { 
      glVertex2d( pit->x(), pit->y() ); 
    }
    glEnd();
  }
}

/***********************************************************************/
/*Mètodes que permeten seleccionar el tipus de visualització escollida */
/***********************************************************************/

void EditorTerreny::activaUnio()
{
	info_mostrar.comprovar_visibilitat=FALSE;
	mesh->settipusInterseccio(INDEFINIT);
	info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
	info_vis.mostrar.tipus_visualitzacio = UNIO;
	emit desactiva2();
	emit desactiva3();
}

void EditorTerreny::activaInterseccio()
{
	info_mostrar.comprovar_visibilitat=FALSE;
	mesh->settipusInterseccio(INDEFINIT);
	info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
	info_vis.mostrar.tipus_visualitzacio = INTERSECCIO;
	emit desactiva2();
	emit desactiva3();
}

void EditorTerreny::activaCounting()
{
	info_mostrar.comprovar_visibilitat=FALSE;
	mesh->settipusInterseccio(INDEFINIT);
	info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
	info_vis.mostrar.tipus_visualitzacio = COUNTING;
	emit desactiva2();
	emit desactiva3();
}

void EditorTerreny::activaSegment()
{
	if(info_vis.mostrar.tipus_anterior != SEGMENT_CANVIAT)
		info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
	info_vis.mostrar.tipus_visualitzacio = SEGMENT;
	emit desactiva1();
	emit desactiva2();
}

void EditorTerreny::activaCombinat()
{
	info_mostrar.comprovar_visibilitat=FALSE;
	mesh->settipusInterseccio(INDEFINIT);
	info_vis.mostrar.tipus_anterior = info_vis.mostrar.tipus_visualitzacio;
	info_vis.mostrar.tipus_visualitzacio = COMBINAT;
	emit desactiva1();
	emit desactiva3();
}

void EditorTerreny::segmentEscollit(int segment)
{
	info_vis.mostrar.segment_escollit = segment;
	info_vis.mostrar.tipus_anterior=SEGMENT_CANVIAT;
}

void EditorTerreny::primerSegmentVisio(int segment)
{
	info_vis.mostrar.primer_segment_visio = segment;
	info_vis.mostrar.tipus_anterior=SEGMENT_CANVIAT;
}

void EditorTerreny::segonSegmentVisio(int segment)
{
	info_vis.mostrar.segon_segment_visio = segment;
	info_vis.mostrar.tipus_anterior=SEGMENT_CANVIAT;
}

void EditorTerreny::tercerSegmentVisio(int segment)
{
	info_vis.mostrar.tercer_segment_visio = segment;
	info_vis.mostrar.tipus_anterior=SEGMENT_CANVIAT;
}

void EditorTerreny::dibuixemQ()
{

vector<pCara> cares = mesh->getCares();
triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10 );   

Objecte *obj;
Punt3d punt3d,aux;
Punt2d t;
int j;
double b0,b1,d0,d1;
Finestra * f;
Finestra f2;
Finestra f4;
list<Event*> PossiblesEvents2;
pCara cara_veina;

obj = objectes_visio->obtenirObjecte(1); //Obtenim el primer objecte que hem inserit (punts = llavors)
punt3d=((Punt *)obj)->getPunt();

cout << "\nDIBUIXEM Q" << endl;
cout << "\n\nParametres finestra ************ " << endl;
cout << "b0 = " ;
cin >> b0;
cout << ", b1 = " ;
cin >> b1;
cout << ", j = " ;
cin >> j;

double e;
e=sqrt((cares[0]->vertex((j+1)%3)->point().x()-cares[0]->vertex((j+2)%3)->point().x())*(cares[0]->vertex((j+1)%3)->point().x()-cares[0]->vertex((j+2)%3)->point().x())+(cares[0]->vertex((j+1)%3)->point().y()-cares[0]->vertex((j+2)%3)->point().y())*(cares[0]->vertex((j+1)%3)->point().y()-cares[0]->vertex((j+2)%3)->point().y())+(cares[0]->vertex((j+1)%3)->point().z()-cares[0]->vertex((j+2)%3)->point().z())*(cares[0]->vertex((j+1)%3)->point().z()-cares[0]->vertex((j+2)%3)->point().z()));
cout << "e = " << e << endl;

Segment s(cares[0]->vertex((j+1)%3)->point(),cares[0]->vertex((j+2)%3)->point());
Punt3d pb0,pb1;
pb0=s.e_t(b0/e);
pb1=s.e_t(b1/e);
cout << "Punt pb0 = " << pb0.x() << " , " << pb0.y() << " ; Punt pb1 = " << pb1.x() << " , " << pb1.y() << endl;
Punt p(punt3d);
d0=p.distancia(pb0);
d1=p.distancia(pb1);
cout << "d0 = " << d0 << " , d1 = " << d1 << endl;
			
f = new Finestra();
f->setb0(b0);
f->setb1(b1);
f->setd0(d0);
f->setd1(d1);	
f->setj(j);
f->setcara(cares[0]);
cout << "\nFinestra creada!!" << endl;

cares[0]->info().LlistaFinestres[j].push_front(*f);

PossiblesEvents2=mesh->propagarFinestra(*f);
cout << "Hem propagat la finestra f " << endl;
cout << "Mida PossiblesEvents2 = " << PossiblesEvents2.size() << endl;

Punt2d source=f->trobarSource();
cout << "Punt Source = " << source.x() << " , " << source.y() << endl;

cara_veina=cares[0]->neighbor(j);
if (!mesh->es_cara_infinita(cara_veina))
{			
	aux=cares[0]->info().T[j](cara_veina->vertex(cara_veina->index(cares[0]))->point());
	cout << "Aux 1 = " << aux.x() << " , " << aux.y() << endl;
	t=Punt2d(aux.x(),aux.y());
}

double xminima,yminima,xmaxima,ymaxima;
xminima=min(min(0.0,e),min(source.x(),t.x()));
xmaxima=max(max(0.0,e),max(source.x(),t.x()));
yminima=min(min(0.0,0.0),min(source.y(),t.y()));
ymaxima=max(min(0.0,0.0),max(source.y(),t.y()));

triemMon( xminima-1, xmaxima+1, yminima-1, ymaxima+1, -10, 10 );   

cout << "Punts Cares en el dibuixarQ " << endl;
cout << "Punts Cara [0] (Vermell) = " << cares[0]->vertex(0)->point().x() << " , " << cares[0]->vertex(0)->point().y() << " , " << cares[0]->vertex(0)->point().z() << " ; " << cares[0]->vertex(1)->point().x() << " , " << cares[0]->vertex(1)->point().y() << " , " << cares[0]->vertex(1)->point().z() << " ; " << cares[0]->vertex(2)->point().x() << " , " << cares[0]->vertex(2)->point().y() << " , " << cares[0]->vertex(2)->point().z() << endl;

cout << "Punts Cara [1] (Blau) = " << cares[1]->vertex(0)->point().x() << " , " << cares[1]->vertex(0)->point().y() << " , " << cares[1]->vertex(0)->point().z() << " ; " << cares[1]->vertex(1)->point().x() << " , " << cares[1]->vertex(1)->point().y() << " , " << cares[1]->vertex(1)->point().z() << " ; " << cares[1]->vertex(2)->point().x() << " , " << cares[1]->vertex(2)->point().y() << " , " << cares[1]->vertex(2)->point().z() << endl;
		
//Dibuixar el punt inserit
glColor3f(0,1,0);
glBegin( GL_POINTS );
	glVertex2d(punt3d.x(), punt3d.y());              
glEnd();
cout << "He dibuixat el pseudosource inicial de color verd" << endl;

//Dibuixar el source de la finestra
glColor3f(1,0,0);
glBegin(GL_POINTS);
	glVertex3f(source.x(), source.y(),0.0);
glEnd();
cout << "He dibuixat el source de la finestra de color vermell" << endl;

//Dibuixar el nou costat
cout << "Nou costat = [0.0, 0.0] a [" << e << ", 0.0] " << endl;
glColor3f(0,0,1);
glBegin(GL_LINES);
	glVertex3f(0.0,0.0,0.0);
	glVertex3f(e,0.0,0.0);
glEnd();
cout << "He dibuixat el nou costat de color blau" << endl;

//Dibuixar la finestra inicial
glColor3f(0,1,0);
cout << "Finestra Inicial = [" << f->getb0() << ", 0.0] a [" << f->getb1() << ", 0.0] " << endl; 
glBegin(GL_LINES);
	glVertex3f(f->getb0(),0.0,0.0);
	glVertex3f(f->getb1(),0.0,0.0);
glEnd();
cout << "He dibuixat la finestra inicial de color verd" << endl;

//Dibuixar la pCara veina
glColor3f(0.5,0.5,0.3);
cout << "pCara Veina = [0.0, 0.0] , [" << e << ", 0.0] a [" << t.x() << ", " << t.y() << "]" << endl; 
glBegin(GL_LINES);
	glVertex3f(0.0,0.0,0.0);
	glVertex3f(t.x(),t.y(),0.0);
glEnd();
glBegin(GL_LINES);
	glVertex3f(t.x(),t.y(),0.0);
	glVertex3f(e,0.0,0.0);
glEnd();
cout << "He dibuixat la pcara veina de color gris" << endl;

//Dibuixar les noves finestres
f2=cara_veina->info().LlistaFinestres[cara_veina->index(cares[0]->vertex((j+2)%3))].front();
f4=cara_veina->info().LlistaFinestres[cara_veina->index(cares[0]->vertex((j+1)%3))].front();
cout << "Recuperar paràmetres finestra f2 (" << f2.getb0() << " , " << f2.getb1() << " , " << f2.getd0() << " , " << f2.getd1() << endl;
cout << "Recuperar paràmetres finestra f4 (" << f4.getb0() << " , " << f4.getb1() << " , " << f4.getd0() << " , " << f4.getd1() << endl;

int jnova=f2.getj();
cout << "jnova = " << jnova << endl;
Punt3d pb0n,pb1n;
if (jnova==0)
{
	cout << "Finestra nova sobre el costat (0.0, 0.0), (" << aux.x() << " , " << aux.y() <<  ")" << endl;
	Segment sn(Punt(0.0,0.0,0.0),Punt(aux.x(),aux.y(),0.0));
	pb0n=sn.e_t(f2.getb0()/sn.midaSegment());
	pb1n=sn.e_t(f2.getb1()/sn.midaSegment());
	cout << "Punt pb0n = " << pb0n.x() << " , " << pb0n.y() << " ; Punt pb1n = " << pb1n.x() << " , " << pb1n.y() << endl;
}
else if (jnova==1)
{	
	cout << "Finestra nova sobre el costat (" << aux.x() << " , " << aux.y() << "), (" << e << " , 0.0)" << endl;
	Segment sn(Punt(aux.x(),aux.y(),0.0),Punt(e,0.0,0.0));
	pb0n=sn.e_t(f2.getb0()/sn.midaSegment());
	pb1n=sn.e_t(f2.getb1()/sn.midaSegment());
	cout << "Punt pb0n = " << pb0n.x() << " , " << pb0n.y() << " ; Punt pb1n = " << pb1n.x() << " , " << pb1n.y() << endl;
}
else if (jnova==2)
{
	cout << "Finestra nova sobre el costat (0,0),(e,0) Aquest cas és impossible!! " << endl;
}

cout << "Finestra Nova = [" << pb0n.x() << ", " << pb0n.y() << "] a [" << pb1n.x() << ", " << pb1n.y() << "]" << endl; 
glColor3f(1,0.0,0.0);
glBegin(GL_LINES);
	glVertex3f(pb0n.x(),pb0n.y(),0.0);
	glVertex3f(pb1n.x(),pb1n.y(),0.0);
glEnd();
cout << "He dibuixat la nova finestra de color vermell" << endl;

int jnova2=f4.getj();
cout << "jnova2 = " << jnova2 << endl;
Punt3d pb0n2,pb1n2;
if (jnova2==0)
{
	cout << "Finestra nova sobre el costat (0.0, 0.0), (" << aux.x() << " , " << aux.y() <<  ")" << endl;
	Segment sn2(Punt(0.0,0.0,0.0),Punt(aux.x(),aux.y(),0.0));
	pb0n2=sn2.e_t(f4.getb0()/sn2.midaSegment());
	pb1n2=sn2.e_t(f4.getb1()/sn2.midaSegment());
	cout << "Punt pb0n2 = " << pb0n2.x() << " , " << pb0n2.y() << " ; Punt pb1n2 = " << pb1n2.x() << " , " << pb1n2.y() << endl;
}
else if (jnova2==1)
{	
	cout << "Finestra nova sobre el costat (" << aux.x() << " , " << aux.y() << "), (" << e << " , 0.0)" << endl;
	Segment sn2(Punt(aux.x(),aux.y(),0.0),Punt(e,0.0,0.0));
	pb0n2=sn2.e_t(f4.getb0()/sn2.midaSegment());
	pb1n2=sn2.e_t(f4.getb1()/sn2.midaSegment());
	cout << "Punt pb0n2 = " << pb0n2.x() << " , " << pb0n2.y() << " ; Punt pb1n2 = " << pb1n2.x() << " , " << pb1n2.y() << endl;
}
else if (jnova2==2)
{
	cout << "Finestra nova sobre el costat (0,0),(e,0) Aquest cas és impossible!! " << endl;
}

cout << "Finestra Nova = [" << pb0n2.x() << ", " << pb0n2.y() << "] a [" << pb1n2.x() << ", " << pb1n2.y() << "]" << endl; 
glColor3f(1,0.0,0.0);
glBegin(GL_LINES);
	glVertex3f(pb0n2.x(),pb0n2.y(),0.0);
	glVertex3f(pb1n2.x(),pb1n2.y(),0.0);
glEnd();
cout << "He dibuixat la nova finestra de color vermell" << endl;


//tornem a deixar el món tal i com estava per poder dibuixar el que es dibuixava abans.
//triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, info_mon.zmin, info_mon.zmax );   
triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10 );   
}


void EditorTerreny::repintarFinestres()
{
list<Finestra>::iterator fj0,fvj0,fj1,fvj1,fj2,fvj2;
Finestra f;
pCara cara_veina;

triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10 ); 

for (IteradorCares fit=(mesh->triangulacio())->finite_faces_begin(); fit!=(mesh->triangulacio())->finite_faces_end();fit++)
{
	if(!mesh->es_cara_infinita(fit))
	{
		fj0=fit->info().LlistaFinestres[0].begin();
		fvj0=fit->info().LlistaFinestres[0].end();
		fj1=fit->info().LlistaFinestres[1].begin();
		fvj1=fit->info().LlistaFinestres[1].end();
		fj2=fit->info().LlistaFinestres[2].begin();
		fvj2=fit->info().LlistaFinestres[2].end();
	
		while (fj0!=fvj0)
		{
			f=*fj0;
			//f.escriure();
			f.setPinta(true);
			f.pintarEvent();
			fj0++;
		}
		
		while (fj1!=fvj1)
		{
			f=*fj1;
			//f.escriure();
			f.setPinta(true);
			f.pintarEvent();
			fj1++;
		}
		
		while (fj2!=fvj2)
		{
			f=*fj2;
			//f.escriure();
			f.setPinta(true);
			f.pintarEvent();
			fj2++;
		}
	
		for (int i=0;i<3;i++)
		{
			pCara cara_veina=fit->neighbor(i);
            		
			if (!mesh->es_cara_infinita(cara_veina))
			{
				fj0=cara_veina->info().LlistaFinestres[i].begin();
				fvj0=cara_veina->info().LlistaFinestres[i].end();
										
				while (fj0!=fvj0)
				{
					f=*fj0;
					//f.escriure();
					f.setPinta(true);
					f.pintarEvent();
					fj0++;
				}
			}
		}
	}
}

}


void EditorTerreny::dibuixemFinestres()
{
	if (info_mostrar.propagacio==TRUE)
	{
		triemMon( info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10 ); 
		assignarColorDV(info_calculs.eventactual->getindex());
		info_calculs.eventactual->pintarFinestres(mesh->triangulacio());
	}
	else //propagacio=FALSE
	{
		triemMon(info_mon.xmin, info_mon.xmax, info_mon.ymin, info_mon.ymax, -10, 10);
		assignarColorDV(info_calculs.eventactual->getindex());
		//info_calculs.eventactual->pintarFinestres(mesh->triangulacio());   
		info_calculs.eventactual->pintarEvent();
	}
}

void EditorTerreny::triemMon(double xmin, double xmax, double ymin, double ymax, double zmin, double zmax)
{
	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();
	glOrtho(xmin, xmax, ymin, ymax, zmin,zmax );   
	glMatrixMode(GL_MODELVIEW);
    	glLoadIdentity();
}


void EditorTerreny::dibuixarDiagramesVoronoi()
{
	cout << "Entro a dibuixarDiagramesVoronoi" << endl;
	info_mostrar.veure_diagrama_voronoi=TRUE;
	repaint();
	info_mostrar.veure_diagrama_voronoi=FALSE;
	char ei;
	cout << "Hem dibuixat els diagrames de Voronoi. Prem tecla x continuar ..." << endl;
	cin >> ei;
}


void EditorTerreny::mostrarDiagramaVoronoi()
{
	if(mesh->infoVertexNoCorrecte())
		cout<<"NO CORRECTE"<<endl;
		
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_GEQUAL);
	glColor4f(0,0,0,1);
	glBegin(GL_QUADS);
		glVertex3d(info_mon.xmin,info_mon.ymin,-10);
		glVertex3d(info_mon.xmin,info_mon.ymax,-10);
		glVertex3d(info_mon.xmax,info_mon.ymax,-10);
		glVertex3d(info_mon.xmax,info_mon.ymin,-10);
	glEnd();
	glEnable(GL_DEPTH_TEST);
	glDepthFunc(GL_LEQUAL);
	
	activarFragment();	
	cgGLEnableProfile(info_vis.gh.fragmentProfile);
	
	glBegin( GL_TRIANGLES );
	
	vector<pCara> v_cares=mesh->getCares();
	for(int j=0; j<v_cares.size(); j++)
		propagacioCaraAproximada(v_cares[j]);
	
	glEnd();
	
	cgGLDisableProfile(info_vis.gh.fragmentProfile);
	glDisable(GL_DEPTH_TEST);
}

void EditorTerreny::propagacioCaraAproximada(pCara cara)
{
	int is=cara->vertex(0)->info().index;
	bool canvia=false;
	for( int i=0;i<3;i++)
	{
		assignarColorDV(cara->vertex(i)->info().index);
		if(is!=cara->vertex(i)->info().index) canvia=true;
		
		double d0=cara->vertex(i)->info().distancia;
		glTexCoord1f(d0);
		glVertex2f(cara->vertex(i)->point().x(), cara->vertex(i)->point().y()); 
		
		double d1=Segment(Punt(cara->vertex(i)->point()),Punt(cara->vertex((i+1)%3)->point())).midaSegment();
		glTexCoord1f(d0+d1);
		glVertex2f(cara->vertex((i+1)%3)->point().x(), cara->vertex((i+1)%3)->point().y()); 
		
		double d2=Segment(Punt(cara->vertex(i)->point()),Punt(cara->vertex((i+2)%3)->point())).midaSegment();
		glTexCoord1f(d0+d2);
		glVertex2f(cara->vertex((i+2)%3)->point().x(),cara->vertex((i+2)%3)->point().y()); 
		
	}
	
 //	d=v_cares[j]->vertex(0)->info().distancia/dmax; glVertex3f(v_cares[j]->vertex(0)->point().x(), v_cares[j]->vertex(0)->point().y(),d);
}



void EditorTerreny::activarFragment()
{
    	ChooseProfiles();
   
   //LoadCgPrograms;
	info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "vectordistancia.cg", info_vis.gh.fragmentProfile, NULL, NULL);
//     info_vis.gh.fragmentProgram = cgCreateProgramFromFile(info_vis.gh.context, CG_SOURCE,  "vectordistanciaDF.cg", info_vis.gh.fragmentProfile, NULL, NULL);
	
	if (!cgIsProgramCompiled(info_vis.gh.fragmentProgram))
	cgCompileProgram(info_vis.gh.fragmentProgram);
	cgGLEnableProfile(info_vis.gh.fragmentProfile);
	cgGLLoadProgram(info_vis.gh.fragmentProgram);
	cgGLBindProgram(info_vis.gh.fragmentProgram);
	glDisable(GL_DITHER);
	glDisable(GL_BLEND);
}


void EditorTerreny::assignarColorDV(int j)
{
	QColor c;
	double max=(double)RAND_MAX;
	srand(5*j+26);
	glColor3f((double)rand()/max,(double)rand()/max,(double)rand()/max);
// 	c.setRgb(255*(double)rand()/max,255*(double)rand()/max,255*(double)rand()/max);
// 	qglColor(c);
	//if(j!=1 && j!=2) cout<<" j no contemplada !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! "<<j<<endl;
}


/***************************************************************/
/*   void EditorTerreny::jocProves()    */
/*Mètode que realitza el testeig d'algunes funcions de l'aplicacio */
/***************************************************************/

void EditorTerreny::jocProves()
{
vector<pCara> cares;
Pla * pla;
bool correcte;
int i,j;
char tecla;
list<Event*> E;

mesh->calculGirsDesplegament();

int opcio, opcio2;
bool continuar;
char continua;
bool acabar;

cout << "\n********** Escull quina opció vols realitzar ********** " << endl;
cout << "1 --> Testejar funcions " << endl;
cout << "******************************************************* " << endl;
cout << "\nEscriu una opció : " ;
cin >> opcio;

acabar=false;
continuar = true;
while (continuar)
{
	if (opcio==1)
	{
		cout << "\n\n*************** TESTEIG DE FUNCIONS ******************* " << endl;
		cout << "1 --> Trobar Source " << endl;
		cout << "2 --> Inicialització " << endl;
		cout << "3 --> Propagar PseudoSource " << endl;
		cout << "4 --> Propagar Finestra " << endl;
		cout << "5 --> Interseca " << endl;
		cout << "6 --> No_Elimina " << endl;
		cout << "7 --> Actualitzar " << endl;
		cout << "******************************************************* " << endl;
		cout << "\nEscriu una opció : " ;
		cin >> opcio2;
		
		if (opcio2==1) //Trobar Source
		{
			double b0,b1,d0,d1,e,veins;
			Finestra * f;
			Punt2d s;
			cout << "\n\nParametres finestra ************ " << endl;
			cout << "b0 = " ;
			cin >> b0;
			cout << ", b1 = " ;
			cin >> b1;
			cout << ", d0 = " ;
			cin >> d0;
			cout << ", d1 = " ;
			cin >> d1;
			cout << ", veins = (1=true,0=false) " ;
			cin >> veins;
			cout << ", e = " ;
			cin >> e;
			
			f = new Finestra();
			f->setb0(b0);
			f->setb1(b1);
			f->setd0(d0);
			f->setd1(d1);	
			
			if (veins==0)
			{
				s=f->trobarSource();
			}
			else
			{
				s=f->trobarSource(e,true);
			}
			cout << "\nSource s = ( " << s.x() << " , " << s.y() << " ) " << endl;	
			
			Punt2d p1(b0,0.0);
			Punt2d p2(b1,0.0);
			
			double distancia1,distancia2;
			distancia1= sqrt((p1.x()-s.x())*(p1.x()-s.x())+(p1.y()-s.y())*(p1.y()-s.y()));
			distancia2= sqrt((p2.x()-s.x())*(p2.x()-s.x())+(p2.y()-s.y())*(p2.y()-s.y()));
			cout << "Distancia del punt (b0,0) al source = " << distancia1 << endl;
			cout << "Distancia del punt (b1,0) al source = " << distancia2 << endl;
			
			cout << "Fi testeig del Trobar Source " << endl;
		}	
			
		if (opcio2==2) //Inicialització
		{
			inicialitzaVertexs(); //Inicialitza les distàncies del vèrtex a -1 
			cout << "Hem inicialitzat les distancies dels vertexs a -1\n" << endl;
		
			E = inicialitzacio();
			cout << "Hem realitzat el mètode de inicialitzacio\n" << endl;
		
		} 
		
		if (opcio2==3) //Propagar PseudoSource
		{
			inicialitzaVertexs(); //Inicialitza les distàncies del vèrtex a -1 
			cout << "Hem inicialitzat les distancies dels vertexs a -1\n" << endl;

			E = inicialitzacio();
			cout << "Hem realitzat el mètode de inicialitzacio\n" << endl;

			propagacioNOMESPSEUDOSOURCE(E);
			cout << "Hem realitzat el mètode de propagacio d'events\n" << endl;
		
		}
		
		if (opcio2==4) //Propagar Finestra
		{				
			bool prova=true;
			char tecla;
			double b0,b1,d0,d1;
			int j;
			Finestra * f;
			list<Event*> PossiblesEvents2;
			Objecte *obj;
			Punt3d punt3d;
			
			cares=mesh->getCares();
			
			cout << "Punts Cares abans de crear la finestra " << endl;
			cout << "Punts Cara [0] = " << cares[0]->vertex(0)->point().x() << " , " << cares[0]->vertex(0)->point().y() << " , " << cares[0]->vertex(0)->point().z() << " ; " << cares[0]->vertex(1)->point().x() << " , " << cares[0]->vertex(1)->point().y() << " , " << cares[0]->vertex(1)->point().z() << " ; " << cares[0]->vertex(2)->point().x() << " , " << cares[0]->vertex(2)->point().y() << " , " << cares[0]->vertex(2)->point().z() << endl;
			
			cout << "Punts Cara [1] = " << cares[1]->vertex(0)->point().x() << " , " << cares[1]->vertex(0)->point().y() << " , " << cares[1]->vertex(0)->point().z() << " ; " << cares[1]->vertex(1)->point().x() << " , " << cares[1]->vertex(1)->point().y() << " , " << cares[1]->vertex(1)->point().z() << " ; " << cares[1]->vertex(2)->point().x() << " , " << cares[1]->vertex(2)->point().y() << " , " << cares[1]->vertex(2)->point().z() << endl;
					
									
			while (prova)
			{
				info_mostrar.veure_dibuix=TRUE;	
				repaint();
				info_mostrar.veure_dibuix=FALSE;
				
				cout << "prem tecla per veure el resultat per la pantalla" << endl;
				cin >> tecla;
				
				if(tecla=='n')
				{
					prova=false;
				}
			}
		
		}
		
		if (opcio2==5) //Interseca
		{
			double b0,b1,d0,d1,sigma;
			double b0_,b1_,d0_,d1_,sigma_;
			double e,veins;
			double x,d,errors;
			Finestra * f1;
			Finestra * f2;
			Punt2d s,r;
			cout << "\n\nParametres finestra 1 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0;
			cout << ", b1 = " ;
			cin >> b1;
			cout << ", d0 = " ;
			cin >> d0;
			cout << ", d1 = " ;
			cin >> d1;
			cout << ", sigma = " ;
			cin >> sigma;
						
			f1 = new Finestra();
			f1->setb0(b0);
			f1->setb1(b1);
			f1->setd0(d0);
			f1->setd1(d1);	
			f1->setsigma(sigma);
			cout << "\nFinestra 1 creada!!" << endl;
			
			cout << "\n\nParametres finestra 2 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0_;
			cout << ", b1 = " ;
			cin >> b1_;
			cout << ", d0 = " ;
			cin >> d0_;
			cout << ", d1 = " ;
			cin >> d1_;
			cout << ", sigma = " ;
			cin >> sigma_;
			
			f2 = new Finestra();
			f2->setb0(b0_);
			f2->setb1(b1_);
			f2->setd0(d0_);
			f2->setd1(d1_);	
			f2->setsigma(sigma_);
			cout << "\nFinestra 2 creada!!" << endl;
			
			cout << ", veins = (1=true,0=false) " ;
			cin >> veins;
			cout << ", e = " ;
			cin >> e;
			
			bool interseccio;
			interseccio=f1->interseca(f2,x,d,errors,e,veins);
			
			cout << "VALORS QUE RETORNA : x = " << x << " , d = " << d << endl;
			double distanciaf1,distanciaf2;
			distanciaf1=f1->distancia(x);
			if (veins==0)
			{
				distanciaf2=f2->distancia(x);
			}
			else if (veins==1)
			{
				distanciaf2=f2->distancia(x,e,veins);			
			}
			
			cout << "Distancia de la finestra 1 a x = " << distanciaf1 << endl;
			cout << "Distancia de la finestra 2 a x = " << distanciaf2 << endl;
			
			if (interseccio)
			{
				cout << "LES FINESTRES INTERSEQUEN!!" << endl;
			}
			else
			{
				cout << "LES FINESTRES NO INTERSEQUEN!!" << endl;
			}
		}
		
		if (opcio2==6) //No_Elimina
		{
		//L'Interseca ja funciona
			double b0,b1,d0,d1,sigma;
			double b0_,b1_,d0_,d1_,sigma_;
			double e,veins;
			double x,d,errors;
			Finestra * f1;
			Finestra * f2;
			Punt2d s,r;
			cout << "\n\nParametres finestra 1 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0;
			cout << ", b1 = " ;
			cin >> b1;
			cout << ", d0 = " ;
			cin >> d0;
			cout << ", d1 = " ;
			cin >> d1;
			cout << ", sigma = " ;
			cin >> sigma;
						
			f1 = new Finestra();
			f1->setb0(b0);
			f1->setb1(b1);
			f1->setd0(d0);
			f1->setd1(d1);
			f1->setsigma(sigma);	
			
			cout << "\nFinestra 1 creada!!" << endl;
			
			cout << "\n\nParametres finestra 2 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0_;
			cout << ", b1 = " ;
			cin >> b1_;
			cout << ", d0 = " ;
			cin >> d0_;
			cout << ", d1 = " ;
			cin >> d1_;
			cout << ", sigma = " ;
			cin >> sigma_;
			
			f2 = new Finestra();
			f2->setb0(b0_);
			f2->setb1(b1_);
			f2->setd0(d0_);
			f2->setd1(d1_);	
			f2->setsigma(sigma_);
			
			cout << "\nFinestra 2 creada!!" << endl;
			
			cout << ", veins = (1=true,0=false) " ;
			cin >> veins;
			cout << ", e = " ;
			cin >> e;
			
			bool interseccio;
			bool no_eliminen;
			interseccio=f1->interseca(f2,x,d,errors,e,veins);
			cout << "VALORS QUE RETORNA : x = " << x << " , d = " << d << endl;
			if (interseccio)
			{
				cout << "LES FINESTRES INTERSEQUEN!!" << endl;
				no_eliminen=f1->no_elimina(f2,x,d,errors,veins,e);
				cout << "Valor no_elimina = (1-true,0-false) " << no_eliminen << endl;
			}
			else
			{
				cout << "LES FINESTRES NO INTERSEQUEN!!. NO CRIDEM AL NO_ELIMINA" << endl;
			}
		}
		
		if (opcio2==7) //Actualitzar
		{
			double b0,b1,d0,d1,sigma,j,index;
			double b0_,b1_,d0_,d1_,sigma_,j_,index_;
			list<Event*> E2;
			Finestra * f1;
			Finestra * f2;
			vector<pCara> cares = mesh->getCares();
				
			cout << "\n\nParametres finestra 1 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0;
			cout << ", b1 = " ;
			cin >> b1;
			cout << ", d0 = " ;
			cin >> d0;
			cout << ", d1 = " ;
			cin >> d1;
			cout << ", sigma = " ;
			cin >> sigma;
			cout << ", j = " ;
			cin >> j;
			cout << ", index = " ;
			cin >> index;
						
			f1 = new Finestra();
			f1->setb0(b0);
			f1->setb1(b1);
			f1->setd0(d0);
			f1->setd1(d1);
			f1->setsigma(sigma);	
			f1->setj(j);
			f1->setindex(index);
			f1->setcara(cares[0]);
			
			cout << "\nFinestra 1 creada!!" << endl;
			
			cout << "\n\nParametres finestra 2 ************ " << endl;
			cout << "b0 = " ;
			cin >> b0_;
			cout << ", b1 = " ;
			cin >> b1_;
			cout << ", d0 = " ;
			cin >> d0_;
			cout << ", d1 = " ;
			cin >> d1_;
			cout << ", sigma = " ;
			cin >> sigma_;
			cout << ", j = " ;
			cin >> j_;
			cout << ", index = " ;
			cin >> index_;
						
			f2 = new Finestra();
			f2->setb0(b0_);
			f2->setb1(b1_);
			f2->setd0(d0_);
			f2->setd1(d1_);
			f2->setsigma(sigma_);	
			f2->setj(j_);
			f2->setindex(index_);
			f2->setcara(cares[1]);
			
			cout << "\nFinestra 2 creada!!" << endl;
								
			E2.push_front(f1);
			E2.push_front(f2);
			cout << "Mida llista = " << E2.size() << endl;
			
			Event * ev = new Finestra(*f1); //Constructor de còpia de la finestra f1
			//Event* ev=E2.front();
			cout << "Event ev = " ;
			ev->escriure();
			
			bool trobat=false;
			list<Event*>::iterator pi,pf;
			Event *ev2;
			pi=E2.begin();
			pf=E2.end();
						
			while((pi!=pf) && (!trobat))
			{
				ev2=*pi;
				if (*ev==*ev2)
				{
					trobat=true;
				}	
				else
				{
					pi++;
				}
			}
			
			if(trobat)
			{
				E2.remove(ev2);
				cout << "eliminem 1 event" << endl;
			}
			cout << "Mida llista = " << E2.size() << endl;
						
		}
	}
	
	if (!acabar)
	{
		cout << "Vols realitzar alguna altra execucio? [s,S,N,n] " << endl;
		cin >> continua;
		if (continua=='S' || continua=='s')
		{
			continuar=true;
		}
		else if (continua=='N' || continua=='n')
		{
			continuar=false;
		}
	}

}//FI CONTINUAR

} 






